#define TOUTS (TCK|TMS|TDI)

#define JTAG_CMD_USER2    (0x3FC3<<22)
#define JTAG_CMD_CFGOUT   (0x3FC4<<22)
#define JTAG_CMD_CFGIN    (0x3FC5<<22)
#define JTAG_CMD_UIDCODE  (0x3FC8<<22)
#define JTAG_CMD_IDCODE   (0x3FC9<<22)
#define JTAG_CMD_JPROGB   (0x3FCB<<22)
#define JTAG_CMD_JSTART   (0x3FCC<<22)
#define JTAG_CMD_JSHUTD   (0x3FCD<<22)

#define JTAG_TMS_CMD    0x380000DF
#define JTAG_TMS_CMDR   0x180000DF
#define JTAG_TMS_SET32X 0x00000000
#define JTAG_TMS_SET32  0x80000000
#define JTAG_TMS_GET32X 0x00000000
#define JTAG_TMS_GET32  0x80000000
#define JTAG_TMS_RST    0x000000FF
#define JTAG_TMS_STARTUP 0x0000F000

inline void udelay (int usec) {
  long msec = usec/1000;
  long time = millis();
  while (millis()-time < msec);
}

inline void jtag_tdi (uint32_t tdi, uint8_t nbits) {
  uint32_t val, shf;
  uint32_t base = REG_PORT_OUT0 & ~TOUTS;
  for (int m=0; m<nbits; m++) {
    shf = (1<<m);
    val = (tdi&shf)? TDI:0;
    REG_PORT_OUT0 = base|val;		// register with clk low
    REG_PORT_OUT0 = base|val|TCK;	// set clock high
  }
}

inline void jtag_tdi_bitrev (uint32_t tdi, uint8_t nbits) {
  uint32_t val, shf;
  uint32_t base = REG_PORT_OUT0 & ~TOUTS;
  for (int m=nbits-1; m>=0; m--) {
    shf = (1<<m);
    val = (tdi&shf)? TDI:0;
    REG_PORT_OUT0 = base|val;		// register with clk low
    REG_PORT_OUT0 = base|val|TCK;	// set clock high
  }
}

inline void jtag_tms_tdi (uint32_t tms, uint32_t tdi, uint8_t nbits) {
  uint32_t val, shf;
  uint32_t base = REG_PORT_OUT0 & ~TOUTS;
  for (int m=0; m<nbits; m++) {
    shf = (1<<m);
    val = (tdi&shf)? TDI:0;
    if (tms&shf) val |= TMS;
    REG_PORT_OUT0 = base|val;		// register with clk low
    REG_PORT_OUT0 = base|val|TCK;	// set clock high
  }
}

inline uint32_t jtag_get (uint8_t nbits) {
  uint32_t shf, tdo=0;
  uint32_t base = REG_PORT_OUT0 & ~TOUTS;
  for (int m=0; m<nbits; m++) {
    shf = (1<<m);
    REG_PORT_OUT0 = base|0;		// register with clk low
    REG_PORT_OUT0 = base|TCK;		// set clock high
    if (REG_PORT_IN0&TDO) tdo |= shf;
  }
  return tdo;
}

inline uint32_t jtag_tdi_get (uint32_t tdi, uint8_t nbits) {
  uint32_t val, shf, tdo=0;
  uint32_t base = REG_PORT_OUT0 & ~TOUTS;
  for (int m=0; m<nbits; m++) {
    shf = (1<<m);
    val = (tdi&shf)? TDI:0;
    REG_PORT_OUT0 = base|val;		// register with clk low
    REG_PORT_OUT0 = base|val|TCK;	// set clock high
    if (REG_PORT_IN0&TDO) tdo |= shf;
  }
  return tdo;
}

inline uint32_t jtag_tms_tdi_get (uint32_t tms, uint32_t tdi, uint8_t nbits) {
  uint32_t val, shf, tdo=0;
  uint32_t base = REG_PORT_OUT0 & ~TOUTS;
  for (int m=0; m<nbits; m++) {
    shf = (1<<m);
    val = (tdi&shf)? TDI:0;
    if (tms&shf) val |= TMS;
    REG_PORT_OUT0 = base|val;		// register with clk low
    REG_PORT_OUT0 = base|val|TCK;	// set clock high
    if (REG_PORT_IN0&TDO) tdo |= shf;
  }
  return tdo;
}

int jtag_pci_wr (int offset, int data) {
  uint32_t jtmp; offset |= 0x4000;
  jtag_tms_tdi(JTAG_TMS_CMD,JTAG_CMD_USER2,32);
  jtmp = 0x00060002 | ((offset&0xFF00)<<12) | ((offset&0xFF)<<4);
  jtag_tms_tdi(JTAG_TMS_SET32X,jtmp,32);
  jtmp = 0x00080008 | ((data&0xFF00)<<12) | ((data&0x00FF)<<4);
  jtag_tms_tdi(JTAG_TMS_SET32X,jtmp,32);
  jtmp = 0x00080008 | ((data&0xFF000000)>>4) | ((data&0x00FF0000)>>12);
  jtag_tms_tdi(JTAG_TMS_SET32X,jtmp,32);
  jtag_tms_tdi(JTAG_TMS_SET32,0x00060002,32);
  jtag_tms_tdi(JTAG_TMS_RST,0,8);
  return data;
}

int jtag_pci_rd (int offset) {
  uint32_t jtmp; int stat,data;  offset |= 0x4000;
  jtag_tms_tdi(JTAG_TMS_CMD,JTAG_CMD_USER2,32);
  jtmp = 0x00060002 | ((offset&0xFF00)<<12) | ((offset&0xFF)<<4);
  jtag_tms_tdi(JTAG_TMS_SET32X,jtmp,32);
  stat = jtag_tms_tdi_get(JTAG_TMS_GET32X,0x00090009,32);
  data = ((stat>>8)&0xFF) | ((stat>>16)&0xFF00);
  stat = jtag_tms_tdi_get(JTAG_TMS_GET32X,0x00090009,32);
  data |= ((stat<<8)&0xFF0000) | (stat&0xFF000000);
  jtag_tms_tdi(JTAG_TMS_SET32,0x00060002,32);
  jtag_tms_tdi(JTAG_TMS_RST,0,8);
  return data;
}

void setupJtag() {  
  /* setup pins */
  REG_PORT_DIRSET0 = TCK;
  REG_PORT_DIRSET0 = TMS;
  REG_PORT_DIRSET0 = TDI;
  REG_PORT_DIRCLR0 = TDO;
  /* ensure we're in the correct state (5 1's will always put us in TLR */
  jtag_tms_tdi (0x1f, 0, 5);
}

