/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/*
  Xilinx RocketIO 3GIO Controller

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  1/15/2003


*/
`ifdef RIO_GEN3
`define RIO_DBW 128
`else
`define RIO_DBW 64
`endif

module riocontroller (
  bsclk,bscs,bsrd,bswr,bspage,bsaddr,bsdata,bsdatao, // Boundary Scan Interface
  sclk,srst, scs,saddr, swr,swrbus, srd,srdbus,
  pxclkx, pxrxp,pxrxn, pxtxp,pxtxn, pxctl,
  rclk, clk1,rst1,rbus1,tbus1, clk2,rst2,rbus2,tbus2,
  test);			

  parameter PORTS=8;
  parameter IOTYPE1=0;
  parameter IOTYPE2=0;
  parameter IBW=64;

  localparam MPORTS=16;
  localparam DBW=`RIO_DBW;
  localparam DBWP=4+1+1+DBW;	// hold+rdy+valid+bus

  // BSCAN ports
  input bsclk,bscs,bsrd,bswr;
  input [7:0] bspage,bsaddr,bsdata;
  output [7:0] bsdatao;

  // system bus
  input sclk,srst,scs,swr,srd;
  input [31:0] saddr,swrbus;
  output [31:0] srdbus;
        
  // MGT ports
  input [3:0] pxclkx;
  input [PORTS-1:0] pxrxp,pxrxn;
  output [PORTS-1:0] pxtxp,pxtxn;
  inout [MPORTS-1:0] pxctl;

  // IO port busses
  input rclk;
  output [1:0] clk1,clk2;
  input  rst1,rst2;
  output [DBWP-1:0] rbus1,rbus2;
  input  [DBWP-1:0] tbus1,tbus2;

  // test port
  output [7:0] test;

  // global signals 
  wire F=0, T=1;

  wire pxclk  = pxclkx[0];
  wire pxclkp = pxclkx[2];
  wire pxclkn = pxclkx[3];

  // boundary scan monitor
  reg         drp_en,drp_en0,drp_enx;
  reg         drp_we,drp_we0,drp_wex;
  reg  [12:0] drp_addr;
  reg  [15:0] drp_di,drp_dor;
  wire        drp_rdy,drp_rdyxi;
  wire [15:0] drp_do,drp_doxi;
  wire [15:0] dmon_do,dmon_doxi;
  wire [7:0]  drp_rdyx;
  wire [PORTS*16-1:0] drp_dox;
  wire [PORTS*17-1:0] dmon_dox;
  wire        drp_clk=sclk;

  wire bsrw  = bscs && (bswr || bsrd);
  reg drpen,drpwe,drpad,drpx;
  reg [13:0] drpaddr;
  always @(posedge bsclk) begin
    drpx <= bspage[6];
    if (bsrw) drpaddr <= {bspage,bsaddr[7:1]};
    if (bsrw) drp_di <= {bsdata,drp_di[15:8]};
    if (bsrw) drpad  <= bsaddr[0];
    if (bsrw) drpwe  <= (bswr && bsaddr[0]);
    if (!bscs) drpen <= 0; else drpen <= (bswr && bsaddr[0]) || (bsrd && !bsaddr[0]);
  end
  assign bsdatao = drpad? drp_dor[15:8] : drp_dor[7:0];
  reg drpgo_,drpgo;
  always @(posedge drp_clk) begin
    drpgo   <= drpen;
    drpgo_  <= drpgo;
    drp_en  <= drpgo && !drpgo_;
    drp_we  <= drpgo && !drpgo_ && drpwe;
    drp_addr<= drpaddr;
    drp_en0 <= drp_en && !drp_addr[12];
    drp_enx <= drp_en &&  drp_addr[12];
    drp_we0 <= drp_en && drp_we && !drp_addr[12];
    drp_wex <= drp_en && drp_we &&  drp_addr[12];
    if (drpx) drp_dor <= dmon_doxi;
    else if (drp_en) drp_dor <= 16'hDEAD;
    else if (drp_rdy) drp_dor <= drp_do;
    else if (drp_rdyxi) drp_dor <= drp_doxi;
  end
  muxMxN #(PORTS,1) drpmr (drp_rdyxi, drp_rdyx, drp_addr[11:9]);
  muxMxN #(PORTS,16) drpmd (drp_doxi, drp_dox, drp_addr[11:9]);
  muxMxN #(PORTS,17) dmonmd (dmon_doxi, dmon_dox, drp_addr[11:9]);

  reg rst1x; always @(posedge sclk) rst1x <= rst1 || srst;
  reg rst2x; always @(posedge sclk) rst2x <= rst2 || srst;
  wire [3:0] rdy1,rdy2;
  wire [7:0] test1,test2;

  reg  [7:0]  icet_route;
  reg  [31:0] icetsys1,icetsys2;
  reg  [31:0] dbgmemaddr;
  wire [31:0] dbgmemdat;
  wire [31:0] txaux1,txaux2,rxaux1,rxaux2;
  wire [31:0] dbgsys = 32'h12345678;

  // system register address resolution - with pipeline
  wire [15:0] sb; swrbank #(0,16) swb (sclk,scs,swr,saddr,sb);
  
  // system registers - written to by pic_cport() host control software
  always @(posedge sclk) begin
    if (srst) icetsys1   <= 0; else if (sb[0]) icetsys1   <= swrbus;
    if (srst) icetsys2   <= 0; else if (sb[1]) icetsys2   <= swrbus;
    if (srst) dbgmemaddr <= 0; else if (sb[2]) dbgmemaddr <= swrbus;
    if (srst) icet_route <= 0; else if (sb[3]) icet_route <= swrbus[7:0];
  end

`ifdef RIO_GEN3

  reg [3:0] rx1cnt,rx2cnt; reg [127:0] rxdbg1,rxdbg2;

  // this next mux concatenates all inputs from right-to-left in second argument
  muxMxNp #(16,32) (srdbus,{rxdbg2,rxdbg1, rxaux2,rxaux1,dbgsys,24'hDEAD03,icet_route, txaux2,txaux1,icetsys2,icetsys1}, saddr[5:2],sclk);

  wire [DBW:0] rbus11,rbus21;
  icepm_lcl #(4,DBW) bnk1 (sclk,icetsys1, clk1[0],clk1[1], rdy1, tbus1[DBW],tbus1[DBW-1:0], rbus11[DBW],rbus11[DBW-1:0],
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[3:0],drp_dox[16*4-1:16*0],dmon_dox[17*4-1:17*0],
		pxclk, pxtxp[3:0],pxtxn[3:0], pxrxp[3:0],pxrxn[3:0], test1,txaux1,rxaux1);
`ifndef ICE_PM
  icepm_lcl #(4,DBW) bnk2 (sclk,icetsys2, clk2[0],clk2[1], rdy2, tbus2[DBW],tbus2[DBW-1:0], rbus21[DBW],rbus21[DBW-1:0],
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[7:4],drp_dox[16*8-1:16*4],dmon_dox[17*8-1:17*4],
		pxclk, pxtxp[7:4],pxtxn[7:4], pxrxp[7:4],pxrxn[7:4], test2,txaux2,rxaux2);
`endif
  reg [DBWP-1:0] rbus1,rbus2;
  always @(posedge clk1[1]) rbus1 <= {rxaux1[3:0],rdy1[0],rbus11[DBW:0]};
  always @(posedge clk2[1]) rbus2 <= {rxaux2[3:0],rdy2[0],rbus21[DBW:0]};
  always @(posedge clk1[1]) begin rx1cnt <= rx1cnt+1; if (rx1cnt==0) rxdbg1 <= rbus1; end
  always @(posedge clk2[1]) begin rx2cnt <= rx2cnt+1; if (rx2cnt==0) rxdbg2 <= rbus2; end

`else

  wire [31:0] ch0_tpkt,ch0_npkt,ch0_nerr;
  wire [31:0] ch1_tpkt,ch1_npkt,ch1_nerr;
  wire [31:0] ch2_tpkt,ch2_npkt,ch2_nerr;
  wire [31:0] ch3_tpkt,ch3_npkt,ch3_nerr;
  wire [31:0] ch4_tpkt,ch4_npkt,ch4_nerr;
  wire [31:0] ch5_tpkt,ch5_npkt,ch5_nerr;
  wire [31:0] ch6_tpkt,ch6_npkt,ch6_nerr;
  wire [31:0] ch7_tpkt,ch7_npkt,ch7_nerr;
  wire [63:0] ch0_lcorr,ch1_lcorr,ch2_lcorr,ch3_lcorr;
  wire [63:0] ch7_lcorr,ch6_lcorr,ch5_lcorr,ch4_lcorr;
  assign txaux1 = tbus1[DBWP-1:DBWP-4];
  assign txaux2 = tbus2[DBWP-1:DBWP-4];

  // this mux concatenates all inputs from right-to-left in second argument
  muxMxNp #(32,32) (srdbus,{ch7_tpkt,ch6_tpkt,ch5_tpkt,ch4_tpkt,ch3_tpkt,ch2_tpkt,ch1_tpkt,ch0_tpkt,
			    ch7_nerr,ch6_nerr,ch5_nerr,ch4_nerr,ch3_nerr,ch2_nerr,ch1_nerr,ch0_nerr,
			    ch7_npkt,ch6_npkt,ch5_npkt,ch4_npkt,ch3_npkt,ch2_npkt,ch1_npkt,ch0_npkt,
	ch1_lcorr[63:0],dbgsys,24'hDEAD00,icet_route, dbgmemdat,dbgmemaddr,icetsys2,icetsys1}, saddr[6:2],sclk);

  assign clk1 = {rclk,rclk};
  assign clk2 = {rclk,rclk};

  wire [DBW:0] rbus11,rbus12,rbus21,rbus22;
  icet_lcl #(1) bnk1 (clk1,rst1x,rdy1, tbus1[DBW],tbus1[DBW-1:0], rbus11[DBW],rbus11[DBW-1:0], txaux1,rxaux1,
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[3:0],drp_dox[16*4-1:16*0],dmon_dox[17*4-1:17*0],
		pxclk, pxtxp[3:0],pxtxn[3:0], pxrxp[3:0],pxrxn[3:0], test1,icetsys1, 
                {ch3_npkt,ch2_npkt,ch1_npkt,ch0_npkt},{ch3_nerr,ch2_nerr,ch1_nerr,ch0_nerr},{ch3_tpkt,ch2_tpkt,ch1_tpkt,ch0_tpkt});
`ifndef ICE_PM
  icet_lcl #(1) bnk2 (clk2,rst2x,rdy2, tbus2[DBW],tbus2[DBW-1:0], rbus21[DBW],rbus21[DBW-1:0], txaux2,rxaux2,
		drp_clk,F,drp_enx,drp_addr,drp_wex,drp_di,drp_rdyx[7:4],drp_dox[16*8-1:16*4],dmon_dox[17*8-1:17*4],
		pxclk, pxtxp[7:4],pxtxn[7:4], pxrxp[7:4],pxrxn[7:4], test2,icetsys2,
                {ch7_npkt,ch6_npkt,ch5_npkt,ch4_npkt},{ch7_nerr,ch6_nerr,ch5_nerr,ch4_nerr},{ch7_tpkt,ch6_tpkt,ch5_tpkt,ch4_tpkt});
`endif
  reg [DBWP-1:0] rbus1,rbus2;
  always @(posedge clk1[1]) rbus1 <= {rxaux1[3:0],rdy1[0],rbus11[DBW:0]};
  always @(posedge clk2[1]) rbus2 <= {rxaux2[3:0],rdy2[0],rbus21[DBW:0]};

`endif

//  assign test = {rdy2,rdy1,clk2,clk1,tbus2[32],tbus1[32],rbus2[32],rbus1[32]};
//  assign test = {test1,tbus1[DBW],rbus1[DBW],clk1,srst};
  assign test = test1;

endmodule

`ifdef RIO_GEN3

module icepm_lcl (sclk,sys, txclk,rxclk, rdy, txval,txdata, rxval,rxdata,
    drpclk,drprst,drpen,drpaddr,drpwe,drpdi,drprdy,drpdo,dmonitorout,
    gthrefclk, gthtxp,gthtxn, gthrxp,gthrxn, testout,status,data
  );
   
  parameter NC = 4;
  parameter BW = 32*NC;
  parameter INV = 0;
  parameter LOOPBACK = 0;

  localparam CW = 32;
  localparam LCW = NC*CW;

  input sclk;
  input [31:0] sys;

  output txclk,rxclk;
  output [NC-1:0] rdy;

  input txval;
  input [BW-1:0] txdata;

  output rxval;
  output [BW-1:0] rxdata;

  input drpclk;
  input drprst;
  input drpen;
  input [8:0] drpaddr;
  input drpwe;
  input [15:0] drpdi;
  output [NC-1:0] drprdy;
  output [(NC*16)-1:0] drpdo;
  output [(NC*17)-1:0] dmonitorout;

  input gthrefclk;
  output [NC-1:0] gthtxp,gthtxn;
  input  [NC-1:0] gthrxp,gthrxn;

  output [7:0] testout;
  output [31:0] status;
  output [31:0] data;

  wire [NC-1:0] txclks,rxclks;
  (* dont_touch = "yes" *) wire tclk; BUFG_GT txbufg (.I(txclks[0]),.O(tclk)); assign txclk = tclk;
  (* dont_touch = "yes" *) wire rclk; BUFG_GT rxbufg (.I(rxclks[0]),.O(rclk)); assign rxclk = rclk;

  wire cxrst; ff #(1,2) resyncc (cxrst,sys[0],sclk);
  wire txrst; ff #(1,2) resynct (txrst,sys[1],txclk);
  wire rxrst; ff #(1,2) resyncr (rxrst,sys[2],rxclk);
  wire pxrst; ff #(1,2) resyncp (pxrst,sys[3],rxclk);
  wire txrstb; ff #(1,2) resynctb (txrstb,sys[4],txclk);
  wire rxrstb; ff #(1,2) resyncrb (rxrstb,sys[5],rxclk);
  wire rxrstd; ff #(1,2) resyncrd (rxrstd,sys[6],rxclk);
  wire txrsts; ff #(1,2) resyncs  (txrsts,sys[7],txclk);

  // prep resets and clocks
  reg [7:0] txcnt; always @(posedge txclk or posedge cxrst) if (cxrst) txcnt <= 0; else txcnt <= txcnt+1;
  reg [7:0] rxcnt; always @(posedge rxclk or posedge cxrst) if (cxrst) rxcnt <= 0; else rxcnt <= rxcnt+1;
  reg txact; always @(posedge txclk or posedge cxrst) if (cxrst) txact <= 0; else txact <= txact || txcnt[7];
  reg rxact; always @(posedge rxclk or posedge cxrst) if (cxrst) rxact <= 0; else rxact <= rxact || rxcnt[7];
  wire gtwiz_buffbypass_tx_done_out,gtwiz_buffbypass_tx_error_out;
  wire gtwiz_buffbypass_rx_done_out,gtwiz_buffbypass_rx_error_out;
  wire refclk,gtwiz_reset_rx_cdr_stable_out,gtwiz_reset_tx_done_out,gtwiz_reset_rx_done_out;
  wire [NC-1:0] gtpowergood_int,rxpmaresetdone_out,txpmaresetdone_out,txprgdivresetdone_out;
  wire gtwiz_userclk_tx_active = txact;
  wire gtwiz_userclk_rx_active = rxact;
  wire gtwiz_buffbypass_tx_reset_in = txrstb;
  wire gtwiz_buffbypass_rx_reset_in = rxrstb;

  // training sequence 
  //  7
  //  0 txstr1  0xEE11EE11  
  //  1      2             rxscren
  //  2 txscren
  //
  reg txstr1,txstr2;
  reg txscren,rxscren;
  reg [2:0] tp; always @(posedge txclk) tp <= tp+1;
  wire [3:0] tp0={tp,txstr1}, tp1={tp,1'h1}; 
  wire [31:0] tpf = {~tp1,~tp1,tp1,tp1,~tp0,~tp0,tp0,tp0};
  wire [31:0] txscramvec,rxscramvec;
  scrambler32dp scr (txclk,!txscren,txscramvec, rxclk,!rxscren,rxscramvec);

  // prep output streams
  reg tval;
  wire [BW-1:0] txdatb = txval? txdata : {NC{tpf}};
  reg [BW-1:0] txdatc;
  reg [7:0] tcnt;
//  scrambler32 tscr (txclk,!txscren,txscramvec);
  always @(posedge txclk) begin
    txstr1 <= (tp==7) && txrsts && !txscren;
    txstr2 <= txstr1;
    if (txrst) txscren <= 0; else txscren <= txstr2 || txscren;
    txdatc <= {NC{txscramvec}} ^ txdatb;	// polarity in PMs is always inverted
    tval   <= txval && (txdata[7:0] == 8'hA5);
    if (txrst) tcnt <= 0; else if (tval) tcnt <= tcnt+1;
  end

  // align input streams
  reg rxgood,rval;
  wire [BW-1:0] rxdatc;
  reg [BW-1:0] rxdata;
  reg [7:0] rxseq,rcnt;
  reg [31:0] data;
  reg [NC-1:0] rxslide,rxaligned;
//  scrambler32 rscr (rxclk,!rxscren,rxscramvec);
  always @(posedge rxclk) begin
    rxdata <= {NC{~rxscramvec}} ^ rxdatc;
    rxseq <= rxseq+1;
    rxgood <= rxaligned == {NC{1'b1}};
    rval <= rxdata[7:0] == 8'hA5;
    if (!rxgood) rcnt <= 0; else if (rval) rcnt <= rcnt+1;
    if (rxseq[2:0]==0) data <= rxscramvec;
    if (rxrst) rxscren <= 0; else if (rxgood) rxscren <= rxscren || (rxdata[31:0]==32'hEE11EE11);
  end
  genvar i;
  generate
  for (i=0; i<NC; i=i+1) begin
    reg [2:0] cnt; reg rok;
    wire [31:0] rdati = rxdata[31+i*32:i*32];
    always @(posedge rxclk) begin
      rok = (rdati==32'hEE11FF00);
      if (rxrstb|(!rxseq[7])) cnt <= 0; else if (rok) cnt <= cnt+1; // need 4 since last rxslide or 32 clocks
      if (rxrstb) rxaligned[i] <= 0; else rxaligned[i] <= rxaligned[i] || (cnt[2] && pxrst);
      rxslide[i] <= pxrst && !rxaligned[i] && (rxseq[7:1]==1);
    end
  end
  endgenerate

  assign rxval = rxgood;
  assign rdy = {NC{rxgood}};

  // call the coregen for all 4 channels
  gt6250 gtinst (
    .gthrxn_in                               (gthrxn)
   ,.gthrxp_in                               (gthrxp)
   ,.gthtxn_out                              (gthtxn)
   ,.gthtxp_out                              (gthtxp)
   ,.gtwiz_userclk_tx_active_in              (gtwiz_userclk_tx_active)
   ,.gtwiz_userclk_rx_active_in              (gtwiz_userclk_rx_active)
   ,.gtwiz_buffbypass_tx_reset_in            (gtwiz_buffbypass_tx_reset_in)
   ,.gtwiz_buffbypass_tx_start_user_in       (1'b0)
   ,.gtwiz_buffbypass_tx_done_out            (gtwiz_buffbypass_tx_done_out)
   ,.gtwiz_buffbypass_tx_error_out           (gtwiz_buffbypass_tx_error_out)
   ,.gtwiz_buffbypass_rx_reset_in            (gtwiz_buffbypass_rx_reset_in)
   ,.gtwiz_buffbypass_rx_start_user_in       (1'b0)
   ,.gtwiz_buffbypass_rx_done_out            (gtwiz_buffbypass_rx_done_out)
   ,.gtwiz_buffbypass_rx_error_out           (gtwiz_buffbypass_rx_error_out)
   ,.gtwiz_reset_clk_freerun_in              (sclk)
   ,.gtwiz_reset_all_in                      (cxrst)
   ,.gtwiz_reset_tx_pll_and_datapath_in      (txrst)
   ,.gtwiz_reset_tx_datapath_in              (1'b0)
   ,.gtwiz_reset_rx_pll_and_datapath_in      (rxrst)
   ,.gtwiz_reset_rx_datapath_in              (rxrstd)
   ,.gtwiz_reset_rx_cdr_stable_out           (gtwiz_reset_rx_cdr_stable_out)
   ,.gtwiz_reset_tx_done_out                 (gtwiz_reset_tx_done_out)
   ,.gtwiz_reset_rx_done_out                 (gtwiz_reset_rx_done_out)
   ,.gtwiz_userdata_tx_in                    (txdatc)
   ,.gtwiz_userdata_rx_out                   (rxdatc)
   ,.drpaddr_in                              ({NC{drpaddr}})
   ,.drpclk_in                               ({NC{drpclk}})
   ,.drpdi_in                                ({NC{drpdi}})
   ,.drpen_in                                ({NC{drpen}})
   ,.drpwe_in                                ({NC{drpwe}})
   ,.drpdo_out                               (drpdo)
   ,.drprdy_out                              (drprdy)
   ,.gtrefclk00_in                           (gthrefclk)
   ,.qpll0outrefclk_out                      (refclk)
   ,.rxslide_in                              (rxslide)
   ,.rxusrclk_in                             ({NC{rxclk}})
   ,.rxusrclk2_in                            ({NC{rxclk}})
   ,.txusrclk_in                             ({NC{txclk}})
   ,.txusrclk2_in                            ({NC{txclk}})
   ,.gtpowergood_out                         (gtpowergood_int)
   ,.rxoutclk_out                            (rxclks)
   ,.rxpmaresetdone_out                      (rxpmaresetdone_out)
   ,.txoutclk_out                            (txclks)
   ,.txpmaresetdone_out                      (txpmaresetdone_out)
   ,.txprgdivresetdone_out                   (txprgdivresetdone_out)
  );

  wire [3:0] txstat = {gtwiz_buffbypass_tx_done_out,gtwiz_buffbypass_tx_reset_in,gtwiz_reset_tx_done_out,gtwiz_userclk_tx_active};
  wire [3:0] rxstat = {gtwiz_buffbypass_rx_done_out,gtwiz_buffbypass_rx_reset_in,gtwiz_reset_rx_done_out,gtwiz_userclk_rx_active};
  wire [3:0] cxstat = {rxscren,txscren};
  reg ttt; always @(posedge txclk) ttt <= !ttt;
  reg rrr; always @(posedge rxclk) rrr <= !rrr;
  assign testout = {rxstat, rrr,rxrst,ttt,txrst};
  assign status = {rxaligned,cxstat, rxstat,txstat, rcnt,tcnt};

endmodule

`endif
