/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/*
  ICE-DTDM Main Controller

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  1/15/2003

*/
`define IO_CPP
`define PE_CPP
`define CARD_CPP
`define HSV HSV_CPP
`include "vdefs.h"
`include "mdefs.h"

module top (
  x1, x2p, x2n,                                            //  3 
  vipc, vinc, vip, vin, vopc, vonc, vop, von,              // 36
  mckp, mckn, mpd, mcs, mfn, mba, ma, mdqs, mdq,           // 58
  t1a, t1c, t1rd, t1wr, t5rd, t5wr,   
  t1d, t1val, t1sa, t1clk, t5clk, t1pck, t5pck, t1p, t5p,  // 47
  t2a, t2c, t2rd, t2wr, t6rd, t6wr,  
  t2d, t2val, t2sa, t2clk, t6clk, t2pck, t6pck, t2p, t6p,  // 47
  t3a, t3c, t3rd, t3wr, t7rd, t7wr,  
  t3d, t3val, t3sa, t3clk, t7clk, t3pck, t7pck, t3p, t7p,  // 47
  t4a, t4c, t4rd, t4wr, t8rd, t8wr,  
  t4d, t4val, t4sa, t4clk, t8clk, t4pck, t8pck, t4p, t8p   // 47
  );

  // parameters
  parameter IBW=32;
  parameter MBW=64;
  parameter PBW=32;
  parameter BCLK=2;	// DSOCM clock 1=full 2=half

  // clocks  
  input  x1, x2p, x2n;
  // hyper-transport
  input  vipc, vinc;
  input  [7:0] vip, vin;
  output vopc, vonc;
  output [7:0] vop, von;
  // memory interface
  inout mckp;
  output mckn, mpd, mcs, mfn;
  output [1:0] mba;
  output [14:0] ma;
  inout  [3:0] mdqs;
  inout  [31:0] mdq;
  // tuner bank 1
  output [4:0] t1a;
  inout  [7:0] t1c;
  output t1rd, t1wr, t5rd, t5wr;
  output [15:0] t1d;
  output t1val, t1sa, t1clk, t5clk;
  input  t1pck, t5pck;
  input  [3:0] t1p, t5p;
  // tuner bank 2
  output [4:0] t2a;
  inout  [7:0] t2c;
  output t2rd, t2wr, t6rd, t6wr;
  output [15:0] t2d;
  output t2val, t2sa, t2clk, t6clk;
  input  t2pck, t6pck;
  input  [3:0] t2p, t6p;
  // tuner bank 3
  output [4:0] t3a;
  inout  [7:0] t3c;
  output t3rd, t3wr, t7rd, t7wr;
  output [15:0] t3d;
  output t3val, t3sa, t3clk, t7clk;
  input  t3pck, t7pck;
  input  [3:0] t3p, t7p;
  // tuner bank 4
  output [4:0] t4a;
  inout  [7:0] t4c;
  output t4rd, t4wr, t8rd, t8wr;
  output [15:0] t4d;
  output t4val, t4sa, t4clk, t8clk;
  input  t4pck, t8pck;
  input  [3:0] t4p, t8p;

// globals
wire L=0, H=1;

// global clocks
wire gclkt,gclkr,gclkw,gclks,gclkx,gclkvi,gclkvo,gclkf;
wire [3:0] gclkvis,gclkvos;

// JTAG control bus
wire clk,cs,rd,wr;
wire [7:0] page,addr,data,datao,dataod,dataot,datao1,datao2,datao3,datao4;

// tuner input data
wire xvala,xvalb,xvalc;
wire [15:0] xdata,xdatb,xdatc;
wire tpulse1,tpulse2,tpulse3,tpulse4;
wire [7:0] ovsra,ovsrb;
wire ifmta,ifmtb;

// tuner link port resynchronizers 
wire t1pr,t2pr,t3pr,t4pr,t5pr,t6pr,t7pr,t8pr;
wire t1pv,t2pv,t3pv,t4pv,t5pv,t6pv,t7pv,t8pv;
wire [3:0] t1pd,t2pd,t3pd,t4pd,t5pd,t6pd,t7pd,t8pd;

// tuner output
wire y1req,y2req,y3req,y4req,y5req,y6req,y7req,y8req;
wire [1:0] y1sel,y2sel,y3sel,y4sel,y5sel,y6sel,y7sel,y8sel;
wire y1gnt,y2gnt,y3gnt,y4gnt,y5gnt,y6gnt,y7gnt,y8gnt;
wire [15:0] y1dat,y2dat,y3dat,y4dat,y5dat,y6dat,y7dat,y8dat;

// hypertransport lines
wire vclk,hirsta,hirstb,nclk;
wire [3:0] hihold,hohold;

// PPC system control bus
wire srw, srds,swrs, srdx,swrx, srdf,swrf;
wire [31:0] sdbus, srdbus, swrbus, saddr, sads,sadf;
wire [31:0] swrbusp, srdbusd, srdbusm, srdbush,srdbusn, srdbusca, srdbuscb;
wire [7:0] sack;
wire [7:0] csel = saddr[31:24];

// PPC local bus
wire busyp,okp;
wire [31:0] addrp;
wire [1:0] sizep;
wire [31:0] reqp;

// BRAM control bus
wire bclk = (BCLK==2)? gclks : gclkx;
wire brd,bwr;
wire [15:0] baddr;
wire [31:0] brdbus, bwrbus;

// system dma bus
wire ioclk = gclkx;
wire irdym, ienam, ienap, ienah, ienaca, ienacb, ienaha,ienahb, ienata,ienatb;
wire ordym, oenam, oenap, oenah, oenaca, oenacb, oenaha,oenahb, oenata,oenatb;
wire [IBW-1:0] ibusm, ibusp,ibush, ibusca,ibuscb, ibusha,ibushb, ibusta,ibustb;
wire [IBW-1:0] obusm, obusp,obush, obusca,obuscb, obusha,obushb, obusta,obustb;
wire [7:0] iselca,iselcb,oselca,oselcb,iselta,iseltb,oselta,oseltb,iselha,iselhb,oselha,oselhb;
wire [15:0] istat,ostat;
wire [23:0] qrdym,qselm;
wire [3:0] hpchn;
wire hprst;

// NDRAM interface
wire [35:0] acmd;
wire mclk,ago,apd,afull;

// system resets / clock speeds
wire rdmac,rxt,booted,rscs,rprc,rmem,rtun,rio,rmod;
wire hypm = hpchn[0]; // hyper calibration mode
wire [7:0] speed;
wire [2:0] trsp;

// tuner control mux
wire tcclk,tcrd,tcwr;
wire [7:0] tcpage,tcaddr,tcdata;

// control address page resolution
wire pagez = (page==0);
wire paged = (pagez && !addr[7]);
wire pagep = (pagez && addr[7]);
wire paget = (cs && !pagez);

// test output signals
wire[7:0] testm,testp,testd,testca,testcb,testta,testtb,testsc;
wire[7:0] testhi,testho,testtb1,testtb2,testtb3,testtb4,testx;

// JTAG interface for read/write of control words
bscanif bsif (clk,cs,rd,wr,page,addr,data,datao);

// mux the JTAG control output data from tuner-banks, memory, and processor
mux6x8pe tm (datao, datao1,datao2,datao3,datao4,dataod,dataot, page,paged,pagep,clk,H);

// mux the system read busses
muxTxN #(32) srd (sdbus, srdbusd,  srw&csel[`DMAC]);
muxTxN #(32) srm (sdbus, srdbusm,  srw&csel[`HP]);
muxTxN #(32) sr3 (sdbus, srdbush , srw&csel[`HA]);
muxTxN #(32) sr4 (sdbus, srdbusn , srw&csel[`HB]);
muxTxN #(32) sr1 (sdbus, srdbusca, srw&csel[`CA]);
muxTxN #(32) sr2 (sdbus, srdbuscb, srw&csel[`CB]);
muxTxN #(32) swp (sdbus, swrbusp,  !srw);
assign swrbus = sdbus;
assign srdbus = sdbus;

// system clocks for hyper-transport, memory, tuners and PPC
wire crst = (rd && page==`C0F);
wire srst = csel[`DMAC] && saddr[14] && swrs;
sysclocks #(`CLOCKMODE) sc (
  x1,x2p,x2n,vipc,vinc,vopc,vonc,mclk,clk,
  crst,addr, srst,swrbus,srdbusm,srdbusn, rdmac,
  booted,rscs,rprc,rmem,rtun,rio,rmod, speed, trsp,
  gclkc,gclkt,gclks,gclkx,gclkr,gclkw,gclkvi,gclkvis,gclkvo,gclkvos,gclkf, testsc);

// PowerPC logic 
processor #(IBW,1,BCLK,1) ppc (
  clk,rd,wr,page,addr,data,dataod, paged,			// JTAG IF
  bclk,baddr, brd,brdbus, bwr,bwrbus, H,			// BRAM IF
  gclkx,rprc, srw,saddr, srdx,srdbus, swrx,swrbusp, sack,	// System Control
  gclks,srds,swrs,sads, gclkf,srdf,swrf,sadf, 
  ioclk, istat[`PROC],ienap,ibusp, ostat[`PROC],oenap,obusp,	// Data Bus
  addrp,sizep,busyp,okp,reqp, testp);				// Memory access

// input/output bus muxes and DMA sequencer
dmacontroller #(IBW) dmac (
  bclk,baddr, brd,brdbus, bwr,bwrbus,			// BRAM IF
  gclkx,csel[`DMAC],saddr, srdx,srdbusd,swrx,swrbus,	// System IF
  gclkx,ago,apd,acmd,afull,qrdym,qselm,			// Memory control
  irdym,ienam,ibusm,ordym,oenam,obusm,                  // Memory bus
  ioclk,rio,rdmac,rxt, istat,ostat,			// Block status
  ienap,ibusp,oenap,obusp, addrp,sizep,busyp,okp,	// PPC local bus
  ienah,ibush,oenah,obush, hpchn,hprst,			// Host/TunerC bus
  ienaha,ibusha,iselha,oenaha,obusha,oselha, 		// Hyper IF
  ienahb,ibushb,iselhb,oenahb,obushb,oselhb,		// Hyper IF
  ienaca,ibusca,iselca,oenaca,obusca,oselca,		// Core bus
  ienacb,ibuscb,iselcb,oenacb,obuscb,oselcb,		// Core bus
  ienata,ibusta,iselta,oenata,obusta,oselta,		// Tuner IF
  ienatb,ibustb,iseltb,oenatb,obustb,oseltb,		// Tuner IF
  testd);

// network DRAM controller 
ndram #(MBW,IBW) mb (
  gclkx,gclkr,gclkw, rmem, ago,apd,acmd,afull,qrdym,qselm,
  ioclk, irdym,ienam,ibusm, ordym,oenam,obusm,
  mckp,mckn,mpd,mcs,mfn,mba,ma,mdqs,mdq,
  mclk,trsp,gclks, testm);

// hyper-transport input port
hypin #(IBW,`CLOCKMODE,1) hi (gclkvi,gclkvis, vip,vin, hihold,hohold,
  ioclk,rio, ostat[`HA],oenaha,oselha,obusha, ostat[`HB],oenahb,oselhb,obushb,
  hypm,srdbusd,srdbush,hirsta,hirstb,testhi);

// Core 1 engine
`ENGINE1 #(1,IBW) core1 (gclkf,rmod, csel[`CA],saddr, swrf,swrbus, srdf,srdbusca, sack[`CA],
  ioclk, istat[`CA],ienaca,iselca,ibusca, ostat[`CA],oenaca,oselca,obusca, testca);

// Core 2 engine
`ENGINE2 #(2,IBW) core2 (gclkf,rmod, csel[`CB],saddr, swrf,swrbus, srdf,srdbuscb, sack[`CB],
  ioclk, istat[`CB],ienacb,iselcb,ibuscb, ostat[`CB],oenacb,oselcb,obuscb, testcb);

// tuner control bus
tunercontrol tctl (
  clk,paget,rd,wr,page,addr,data,
  gclkx,rprc,csel[`TCTL],saddr, srdx,swrx,swrbus, 
  tcclk,tcrd,tcwr,tcpage,tcaddr,tcdata,
  gclkt,tpulse1,tpulse2,tpulse3,tpulse4, ovsra,ovsrb,ifmta,ifmtb);

// tuner input busses
wire tirsta = srdbusd[`HB2TA]? hirstb : hirsta;
wire tirstb = srdbusd[`HA2TB]? hirsta : hirstb;
tunerin #(`OVS) tia (ioclk,tirsta,istat[`TA],ienata,iselta,ibusta, gclkt,xvala,xdata, tpulse3,ifmta,srdbusd[`R2TA],ovsra);
tunerin #(`OVS) tib (ioclk,tirstb,istat[`TB],ienatb,iseltb,ibustb, gclkt,xvalb,xdatb, tpulse3,ifmtb,srdbusd[`R2TB],ovsrb);
tunerin #(0)    tic (ioclk,rxt,   istat[`TC],ienah, 8'd0,  ibush,  gclkt,xvalc,xdatc, tpulse4,L,    srdbusd[`R2TC],8'd0);

// tuner link port resynchronizers
tlport tp1 (t1pck,t1p, gclkt,t1pv,t1pd,t1pr);
tlport tp2 (t2pck,t2p, gclkt,t2pv,t2pd,t2pr);
tlport tp3 (t3pck,t3p, gclkt,t3pv,t3pd,t3pr);
tlport tp4 (t4pck,t4p, gclkt,t4pv,t4pd,t4pr);
tlport tp5 (t5pck,t5p, gclkt,t5pv,t5pd,t5pr);
tlport tp6 (t6pck,t6p, gclkt,t6pv,t6pd,t6pr);
tlport tp7 (t7pck,t7p, gclkt,t7pv,t7pd,t7pr);
tlport tp8 (t8pck,t8p, gclkt,t8pv,t8pd,t8pr);

// data input, output, and control of tuner bank 1		 
tunerbank tb1 (tcclk,tcrd,tcwr,tcaddr,tcdata,datao1, tcpage[0],tcpage[4],
  gclkt,tpulse1,tpulse2, xvala,xdata, xvalc,xdatc, 
  y1req,y1sel,y1gnt,y1dat, y5req,y5sel,y5gnt,y5dat, 
  t1a,t1c,t1rd,t1wr,t5rd,t5wr,t1d,t1val,t1sa,t1clk,t5clk,
  t1pv,t5pv,t1pd,t5pd,t1pr,t5pr, testtb1);

// data input, output, and control of tuner bank 2
tunerbank tb2 (tcclk,tcrd,tcwr,tcaddr,tcdata,datao2, tcpage[1],tcpage[5],
  gclkt,tpulse1,tpulse2, xvalb,xdatb, xvalc,xdatc, 
  y2req,y2sel,y2gnt,y2dat, y6req,y6sel,y6gnt,y6dat, 
  t2a,t2c,t2rd,t2wr,t6rd,t6wr,t2d,t2val,t2sa,t2clk,t6clk,
  t2pv,t6pv,t2pd,t6pd,t2pr,t6pr, testtb2);

// data input, output, and control of tuner bank 3
tunerbank tb3 (tcclk,tcrd,tcwr,tcaddr,tcdata,datao3, tcpage[2],tcpage[6],
  gclkt,tpulse1,tpulse2, xvala,xdata, xvalc,xdatc, 
  y3req,y3sel,y3gnt,y3dat, y7req,y7sel,y7gnt,y7dat, 
  t3a,t3c,t3rd,t3wr,t7rd,t7wr,t3d,t3val,t3sa,t3clk,t7clk,
  t3pv,t7pv,t3pd,t7pd,t3pr,t7pr, testtb3);

// data input, output, and control of tuner bank 4
tunerbank tb4 (tcclk,tcrd,tcwr,tcaddr,tcdata,datao4, tcpage[3],tcpage[7],
  gclkt,tpulse1,tpulse2, xvalb,xdatb, xvalc,xdatc, 
  y4req,y4sel,y4gnt,y4dat, y8req,y8sel,y8gnt,y8dat,  
  t4a,t4c,t4rd,t4wr,t8rd,t8wr,t4d,t4val,t4sa,t4clk,t8clk,
  t4pv,t8pv,t4pd,t8pd,t4pr,t8pr, testtb4);

// packet mux of output of A side tuners
packetizer tpa (ioclk,rio|rtun,ostat[`TA],oenata,oselta,obusta, gclkt, 
  y1req,y1sel,y1gnt,y1dat, y5req,y5sel,y5gnt,y5dat, 
  y3req,y3sel,y3gnt,y3dat, y7req,y7sel,y7gnt,y7dat,
  testta);

// packet mux of output of B side tuners
packetizer tpb (ioclk,rio|rtun,ostat[`TB],oenatb,oseltb,obustb, gclkt,	
  y2req,y2sel,y2gnt,y2dat,  y6req,y6sel,y6gnt,y6dat,
  y4req,y4sel,y4gnt,y4dat,  y8req,y8sel,y8gnt,y8dat,
  testtb);

// hyper-transport output port
hypout #(IBW,`CLOCKMODE) ho (gclkvo,gclkvos, vop,von, hihold,hohold,
  ioclk,rio, istat[`HA],ienaha,iselha,ibusha, istat[`HB],ienahb,iselhb,ibushb,
  istat[`HA1],istat[`HB1],srdbusd,speed,testho);

// Test Port Output
wire tsync;
wire oppms=0;
wire [7:0] testf = {4'h5,testtb4[7],testtb3[7],testtb2[7],testtb1[7]};
testport #(0,0) tport (
  clk,crst, rd,wr,pagep,addr,data,dataot, testx,tsync,oppms,L,
  testd,testf,testhi,testho, testca,testcb,testta,testtb);

endmodule

