/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/*
  ICE DTDM Module System Clock Generators, etc.

	7p	6s	5p	4s	3p	2s	1p	0s
		clkn	clks	clkx	clkvi	clkvo		clkt	
	clkw	clkr			clkm		clkf 			
	7s	6p	5s	4p	3s	2p	1s	0p

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  1/15/2003

  NOTE: Cannot use FEEDBACK=2X in Virtex2-Pro so we use gclks to drive gclk1x and gclk2x

  NOTE: defparam numbers in quotes get treated as HEX in ISE 7.1

*/
`ifndef SYS_CLKF
`define SYS_CLKF 133
`endif
`ifndef SYS_CLKM
`define SYS_CLKM 200
`endif
`ifndef SYS_CLKX
`define SYS_CLKX 200
`endif
`ifndef SYS_CLKT
`define SYS_CLKT 110
`endif


module sysclocks (x1,x2p,x2n,vipc,vinc,vopc,vonc,mclk,jclk,
	crst,addr, srst,swrbus,srdbus1,srdbus2, rdmac,
	booted,rscs,rprc,rmem,rtun,rio,rmod, speed, trsp,
	gclkc,gclkt,gclks,gclkx,gclkr,gclkw,gclkvi,gclkvis,gclkvo,gclkvos,gclkf, test);

  parameter MODE=4;
  localparam CLKF=`SYS_CLKF;
  localparam CLKM=`SYS_CLKM;
  localparam CLKT=`SYS_CLKT;

  input x1,x2p,x2n,vipc,vinc,jclk;
  input crst,srst,rdmac;
  input [7:0] addr;
  input [31:0] swrbus;
  output [31:0] srdbus1,srdbus2;
  output reg booted,rscs,rprc,rmem,rtun,rio,rmod;
  output reg [7:0] speed;
  input [2:0] trsp;
  output gclkc,gclkt,gclks,gclkx,gclkr,gclkw,gclkvi,gclkvo,gclkf,vopc,vonc,mclk;
  output [3:0] gclkvis,gclkvos;
  output [7:0] test;

  wire vclk,nclk,lclkr,lclkm,lclkw,lclkf,lclks,lclkx,lclkvi,lclkvo,lclkvh,lclkn,lclkt,hlock,lock, gclkvs,gclkvq,gclksc,shifted;
  wire H=1, L=0; 
  reg tcsel;
  wire tvclk = (MODE>4)? lclkt : vclk;

  // System resets
  always @(posedge gclkc)  rscs <= (crst && addr[0]);
  always @(posedge gclkc)  if (crst && addr[1]) tcsel <= addr[7];
  always @(posedge gclkt)  rtun <= (crst && addr[5]);
  always @(posedge gclkf)  rmod <= rprc;
  always @(posedge gclkx) begin
    booted <= crst || booted;
    rprc   <= !booted || rscs || (crst && addr[2] && !addr[7])
              || (rprc && !(crst && addr[2] && addr[7]));
    rio    <= (crst && addr[6]) || rdmac;
  end

  // Clock recovery
  reg [7:0] tune; reg trspe; wire trspa,trspu;
  timing_recovery trp (mclk,rmem, tune,trsp, gclks,trspe,trspa,trspu, srdbus1);
  wire [2:0] trsv; reg trsve; wire trsva,trsvu;
  timing_recovery trv (nclk,rmem, 8'hC9,trsv, gclks,trsve,trsva,trsvu, srdbus2);

  // Clock adjusts
  reg rscv,ascv,dscv,rscp,ascp,dscp,rmems;
  always @(posedge gclks) begin
    rscv  <= (srst && !swrbus[9] && swrbus[0]);
    ascv  <= (srst && !swrbus[9] && swrbus[1]) || (trsve && trsva);
    dscv  <= (srst && !swrbus[9] && swrbus[8]) || (trsve && trsva && trsvu);
    trsve <= (srst && !swrbus[9] && swrbus[5]) || (trsve && !rscv);
    rscp  <= (srst && !swrbus[9] && swrbus[2]);
    ascp  <= (srst && !swrbus[9] && swrbus[3]) || (trspe && trspa);
    dscp  <= (srst && !swrbus[9] && swrbus[8]) || (trspe && trspa && trspu);
    trspe <= (srst && !swrbus[9] && swrbus[6]) || (trspe && !rscp);
    rmems <= (srst && !swrbus[9] && swrbus[4]);
    if (srst&&swrbus[9]) speed <= swrbus[7:0];
    if (srst && swrbus[6]) tune <= swrbus[23:16];
  end

  always @(posedge gclkx) rmem <= rprc || rmems;

  // Global Clocks
  assign gclkc = jclk;

  IBUFGDS _clksc (gclksc,x2p,x2n);	// 100MHz Diff Crystal
  BUFGMUX _clkt  (gclkt,x1,tvclk,tcsel); // 101MHz | 105MHz for GrayChips

  BUFG    _clks  (gclks,lclks);		// 100MHz for DDR and PPC 
  BUFG    _clkx  (gclkx,lclkx);		// 200MHz for DDR and PPC 
  BUFG    _clkr  (gclkr,lclkr);		// 200MHz for DDR phase=~90 for read + 2.5ns out pad delay
  BUFG    _clkw  (gclkw,lclkw);		// 200MHz for DDR phase=270 for write
  BUFG    _clkf  (gclkf,lclkf);		// 133MHz for FFT/Demod 

  BUFG    _clkm  (mclk,lclkm);		// 200MHz DDR read strobe + 270 for timing recovery
//  BUFG    _clkn  (nclk,lclkn);		// 105|266MHz for HyperTransport in + 270 for timing recovery
  assign nclk = gclkvi;

  // the 200MHz System clock for the PPC, NDRAM, and DMA engine 
  DCM x_dcm (.DSSEN(L),.PSCLK(L),.PSEN(L),.PSINCDEC(L),.RST(rscs),.LOCKED(lock),
  		.CLKIN(gclksc),.CLKFB(gclks),.CLK0(lclks),.CLK2X(lclkx),.CLKFX(lclkt));
  defparam x_dcm.DUTY_CYCLE_CORRECTION = "TRUE";
  defparam x_dcm.CLK_FEEDBACK = "1X";
  defparam x_dcm.CLKIN_PERIOD = 10;
  defparam x_dcm.STARTUP_WAIT = "FALSE";
  defparam x_dcm.DFS_FREQUENCY_MODE = "LOW";
  defparam x_dcm.CLKFX_MULTIPLY = (CLKT==120)? 6 : (CLKT==112)? 9 : 11;
  defparam x_dcm.CLKFX_DIVIDE   = (CLKT==120)? 5 : (CLKT==112)? 8 : 10;

  // the 200MHz clock for the NDRAM read strobe
  DCM p_dcm (.DSSEN(L),.PSCLK(gclks),.PSEN(ascp),.PSINCDEC(dscp),.RST(rscp),
  		.CLKIN(gclkx),.CLKFB(gclkr),.CLK0(lclkr),.CLK270(lclkm),.PSDONE(shifted));
  defparam p_dcm.DLL_FREQUENCY_MODE = "LOW";  
  defparam p_dcm.DUTY_CYCLE_CORRECTION = "TRUE";  
  defparam p_dcm.CLK_FEEDBACK = "1X";
  defparam p_dcm.CLKIN_PERIOD = 5;  
  defparam p_dcm.CLKOUT_PHASE_SHIFT = "VARIABLE";
  defparam p_dcm.PHASE_SHIFT  = -120;
  defparam p_dcm.STARTUP_WAIT = "FALSE";  
  
  // the 200MHz clock for the NDRAM write strobe (close to the read strobe) - and the function clock
  DCM q_dcm (.DSSEN(L),.PSCLK(L),.PSEN(L),.PSINCDEC(L),.RST(rscs||!lock),
  		.CLKIN(gclkx),.CLKFB(gclkw),.CLK0(lclkw),.CLKFX(lclkf));
  defparam q_dcm.DLL_FREQUENCY_MODE = "HIGH";  
  defparam q_dcm.DUTY_CYCLE_CORRECTION = "TRUE";  
  defparam q_dcm.CLK_FEEDBACK = "1X";
  defparam q_dcm.CLKIN_PERIOD = 5;  
  defparam q_dcm.CLKOUT_PHASE_SHIFT = "FIXED";
  defparam q_dcm.PHASE_SHIFT  = 192;
  defparam q_dcm.STARTUP_WAIT = "FALSE";  
  defparam q_dcm.DFS_FREQUENCY_MODE = "LOW";  
  defparam q_dcm.CLKFX_MULTIPLY = (CLKF==166)? 5 : (CLKF==150)? 3 : (CLKF==125)? 5 : (CLKF==112)?  9 : (CLKF==110)? 11 : (CLKF==66)? 2 : 4;
  defparam q_dcm.CLKFX_DIVIDE   = (CLKF==166)? 6 : (CLKF==150)? 4 : (CLKF==125)? 8 : (CLKF==112)? 16 : (CLKF==110)? 20 : (CLKF==66)? 6 : 6;

  // the 360MHz HyperChannel read strobe (phase shift 50+-30)
  DCM v_dcm (.DSSEN(L),.PSCLK(gclks),.PSEN(ascv),.PSINCDEC(dscv),.RST(rscv),.LOCKED(hlock),
  		.CLKIN(vclk),.CLKFB(gclkvs),.CLK0(lclkvi),.CLK180(lclkvo),.CLK270(lclkn),.CLKDV(lclkvh));
  defparam v_dcm.DUTY_CYCLE_CORRECTION = "TRUE";  
  defparam v_dcm.CLK_FEEDBACK = "1X";
  defparam v_dcm.CLKOUT_PHASE_SHIFT = "VARIABLE";
  defparam v_dcm.PHASE_SHIFT  = 0;
  defparam v_dcm.STARTUP_WAIT = "FALSE";  
  defparam v_dcm.CLKDV_DIVIDE = 2;

`ifdef IO_HH
  defparam v_dcm.DLL_FREQUENCY_MODE = "HIGH";  
  defparam v_dcm.CLKIN_PERIOD = 2.78;
  IBUFDS_LVDS_25 _bufvi (vclk, vipc,vinc);
  BUFG    _clkvi (gclkvs,lclkvi);	// 360MHz for HyperTransport DDR in/out
  BUFG    _clkvo (gclkvq,lclkvh);	// 180MHz for HyperTransport QDR internal

  wire orst = rscv || !hlock;
  wire vrst,ss; 
  doqdrc _bufvo (vopc,vonc, gclkvq,orst,gclkvs,vrst,ss,L);
  assign gclkvi = gclkvq;
  assign gclkvo = gclkvq;
  assign gclkvis = {L,ss,vrst,gclkvs};
  assign gclkvos = {L,ss,vrst,gclkvs};

`elsif IO_SS
  defparam v_dcm.DLL_FREQUENCY_MODE = "LOW";  
  defparam v_dcm.CLKIN_PERIOD = 8.0;  
  BUFG    _clkvi (gclkvi,lclkvi);	// 105MHz for HyperTransport DDR in
  BUFG    _clkvo (gclkvo,lclkvo);	// 105MHz for HyperTransport SDR out
  assign gclkvs = gclkvi;
`endif

//  assign test = {gclkt,gclkf, gclks,gclkx, gclkr,gclkw, gclkvi,gclkvo};
  assign test = 0;

endmodule
