/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/*
  ICE-K8M Main Controller

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  6/29/2012

*/
`define IO_CPP
`define PE_CPP
`define CARD_CPP
`define HSV HSV_CPP
`include "vdefs.h"
`include "mdefs.h"

module top (

  cclkp,cclkn,					// 2
  dclkp,dclkn,					// 2

  pclkp,pclkn,					// 2 MGT
  qclkp,qclkn,					// 2 MGT
  rclkp,rclkn,					// 2 MGT
  sclkp,sclkn,					// 2 MGT

  pyrxp,pyrxn, pytxp,pytxn,

  mckp,mckn,mcke,mwe,mcas,mras,ma,mba,mdm,mdq,mdqsp,mdqsn,
  mcs,mrst,modt,mact,

  led_grn,led_red,
  fram_scl,fram_sda,
  tsense,
  cfg,						// 4
  tp						// 8

);

  // parameters
`ifdef DMA_PKT_4X
  parameter IBW=256;		// I/O bus width
`else
  parameter IBW=128;		// I/O bus width
`endif
  parameter PXLW=`PX_LW;	// PCI Express Lane Width
  parameter PYLW=`PY_LW;	// Alternate 3GIO ports
  parameter MBL=8;		// Memory chip byte lanes
`ifdef RIO_GEN3
  parameter RBW=128;
`else
  parameter RBW=64;
`endif

  // programmable clocks
  input cclkp,cclkn; 
  input dclkp,dclkn; 

  // GTE clocks  
  input pclkp,pclkn;
  input qclkp,qclkn;
  input rclkp,rclkn;
  input sclkp,sclkn;

  // other MGT interfaces
  input [PYLW-1:0] pyrxp, pyrxn;
  output [PYLW-1:0] pytxp, pytxn;

  // memory interface
  inout  mckp,mckn;
  output mcke,mwe,mcas,mras;
  output [13:0] ma;
  output [2:0] mba;
  inout  [MBL-1:0] mdm;
  inout  [8*MBL-1:0] mdq;
  inout  [MBL-1:0] mdqsp,mdqsn;
  output mcs,mrst,modt,mact;

  // external interfaces
  output led_red,led_grn;
  inout fram_scl,fram_sda;
  input [3:0] cfg;
  input tsense;

  // test port
  inout [7:0] tp;

// global signals
wire L=0, H=1;

// global clocks
wire gclkt,gclkr,gclkw,gclks,gclkx,gclky,gclkv;
wire [1:0] lclktp,lclktq,lclktr,lclkts,gclkf;
wire [39:0] gclkms;

// activity detect
reg led1,led2;
assign led_grn=!led1;
assign led_red=!led2;

// PPC system control bus
wire srw, srds,swrs, srdx,swrx, srdf,swrf;
(* dont_touch = "yes" *) wire [31:0] saddr,sads,sadf,swrbus,srdbus;
wire [31:0] srdbusp, srdbusr, srdbusd, srdbusc, srdbusj;
wire [31:0] srdbusha,srdbushb, srdbusca,srdbuscb, srdbusta,srdbustb;
wire [7:0] sack;
wire [7:0] csel = saddr[31:24];
wire [7:0] csels = sads[31:24];
wire [7:0] cself = sadf[31:24];

// PPC local bus
wire busyp,okp;
wire [31:0] addrp;
wire [1:0] sizep;
wire [31:0] reqp;

// BRAM control bus - also JVM clock
wire bclk = gclky;
wire brd,bwr;
wire [15:0] baddr;
wire [31:0] brdbus, bwrbus;

// system dma bus
wire ioclk = gclkx;
wire ienam, ienar,ienad, ienap,ienah, ienaca,ienacb, ienaha,ienahb, ienata,ienatb;
wire oenam, oenar,oenad, oenap,oenah, oenaca,oenacb, oenaha,oenahb, oenata,oenatb;
wire [IBW-1:0] ibusm, ibusr,ibusd, ibusp,ibush, ibusca,ibuscb, ibusha,ibushb, ibusta,ibustb;
wire [IBW-1:0] obusm, obusr,obusd, obusp,obush, obusca,obuscb, obusha,obushb, obusta,obustb;
wire [7:0]  iselca,iselcb,oselca,oselcb,iselta,iseltb,iselha,iselhb,oselha,oselhb;
wire [11:0] oselta,oseltb;
wire [31:0] iadrd;
wire [15:0] istat,ostat;
wire [23:0] qrdym,qselm;
wire [5:0] hpchn;
wire hprst;
wire hypm = hpchn[0]; // hyper calibration mode
wire [7:0] speed;

// hypertransport lines
wire hirsta,hirstb;
wire [3:0] hihold,hohold;
wire [3:0] hihold_,hohold_;

// DRAM interface
wire [35:0] acmd;
wire mclk,ago,apd,afull,irdym,ordym;
wire [2:0] trsp;

// system resets
wire rpci,rprc,rioc,rdmac,rxt;
wire rscs,rhyp,rmem,rio,rsts,rmod,boot,oppms,oppus,oppts;

// RIO  signals
wire rrst1,rrst2;
wire [1:0] rclk1,rclk2;
wire [RBW+6-1:0] ribus1,ribus2, robus1,robus2;

// test output signals
wire [7:0] testsc,testx,testp,testd,testca,testcb,testj,temp;
wire [7:0] testhi,testho,testta,testtb,testpci,testrio,testr;

// control address page resolution
wire pagez = (page==0);
wire pagep = (pagez && !addr[7]);
wire paget = (pagez &&  addr[7]);
wire pages = (page==3);
wire pager = page[7];

// extra address selectors
wire csel0 = (csels==0);		// no select
wire seljt = (csel0 && sads[23]);	// JTAG
wire selau = (csel0 && sads[22]);	// Auxiliary
wire selrr = (csel0 && sads[21]);	// RocketIO
wire selsc = (csel0 && sads[20]);	// Sysclock
wire selic = (csel0 && sads[19]);	// ICAP

// mux the system bus read ports
muxTxN #(32) srd (srdbus, srdbusd,  csel[`DMAC]);
muxTxN #(32) srp (srdbus, srdbusp,  csel[`HP]);
muxTxN #(32) sr1 (srdbus, srdbusha, csel[`HA]);
muxTxN #(32) sr2 (srdbus, srdbushb, csel[`HB]);
muxTxN #(32) sra (srdbus, srdbusca, csel[`CA]);
muxTxN #(32) srb (srdbus, srdbuscb, csel[`CB]);
muxTxN #(32) sr7 (srdbus, srdbusta, csel[`TA]);
muxTxN #(32) sr8 (srdbus, srdbustb, csel[`TB]);
muxTxN #(32) srr (srdbus, srdbusr,  selrr);
muxTxN #(32) srj (srdbus, srdbusj,  seljt);
muxTxN #(32) srm (srdbus, srdbusc,  selsc);

// configuration signals
wire spi_clk,spi_fcs,cfg_clk;
wire [3:0] spi_dat;
wire [3:0] spi_i; assign spi_dat[1] = spi_i[1];
wire [3:0] spi_o = {spi_dat[3],spi_dat[2],H,spi_dat[0]};
wire [3:0] spi_dts = 2;
STARTUPE3 se3 (.USRCCLKO(spi_clk), .FCSBO(spi_fcs), .FCSBTS(L), .DO(spi_o), .DI(spi_i), .DTS(spi_dts), .USRCCLKTS(L), .CFGMCLK(cfg_clk) );
//wire icapwr = selic && swrs;
//ICAPE3 ice3 (.CLK(gclks),.CSIB(!icapwr),.I(swrbus),.RDWRB(sads[0]));

// JTAG control bus
wire clk,cs,rd,wr,rdp,wrs; 
wire [1:0] rds; assign {rdp,rd}=rds;
wire [7:0] page,addr,data,datat,datap,datar,datao;
bscanif bsif (clk,cs,rds,wrs,page,addr,data,datao, spi_clk,spi_fcs,spi_dat);
assign wr=wrs;

// generate system clocks/resets for hyper-transport, memory, cores and PPC
wire crst = (rd && page==`C0F);
wire swrc = csels[`DMAC] && saddr[14] && swrs;
wire rset = csels[`DMAC] && saddr[13] && swrs;
wire pset = csels[`DMAC] && saddr[12] && swrs;

sysclocks sc (
  cclkp,cclkn,dclkp,dclkn,pclkp,pclkn,qclkp,qclkn,rclkp,rclkn,sclkp,sclkn, cclk,mclk,trsp,
  crst,addr, swrc,swrbus,srdbusc, rprc,rdmac,pset,rset,
  rscs,rhyp,rmem,rio,rsts,rmod,boot,
  gclks,gclkx,gclky,gclkms,gclkf,gclkv,
  lclktp,lclktq,lclktr,lclkts,pclk, testsc);

div_to_pps (gclks,oppms,oppus);

assign datao = pages? temp : pager? datar : paget? datat : datap;

// handle activity detect
reg activity;
reg [9:0] acnt1,acnt2;
always @(posedge gclks) begin
  activity <= rd || wr || oenaha || oenahb || ienaha || ienahb;
  if (oppms) acnt1 <= acnt1+1;
  if (activity) acnt2 <= 100; else if (oppms&&led2) acnt2 <= acnt2-1;
  led1 <= acnt1[9];
  led2 <= acnt2>0;
end

// handle JVM logic 
processor #(IBW,0) ppc (
  clk,rd,wr,page,addr,data,datap, pagep,			// JTAG IF
  bclk,baddr, brd,brdbus, bwr,bwrbus, H,			// BRAM IF
  gclkx,rprc, srw,saddr, srdx,srdbus, swrx,swrbus, sack,	// System Control Bus
  gclks,srds,swrs,sads, gclkf,srdf,swrf,sadf, 			// Alternate SC rates
  ioclk, istat[`PROC],ienap,ibusp, ostat[`PROC],oenap,obusp,	// Data Bus
  addrp,sizep,busyp,okp,reqp, testp);

// input/output bus muxes and DMA sequencer
dmacontroller #(IBW) dmac (
  bclk,baddr, brd,brdbus, bwr,bwrbus,			// BRAM IF
  gclkx,csel[`DMAC],saddr, srdx,srdbusd,swrx,swrbus,	// System IF
  ioclk,ago,apd,acmd,afull,qrdym,qselm,			// Memory control
  irdym,ienam,ibusm,ordym,oenam,obusm,			// Memory bus
  ioclk,rio,rdmac,rxt, istat,ostat,			// Block status
  ienap,ibusp,oenap,obusp, addrp,sizep,busyp,okp,	// PPC local bus
  ienah,ibush,oenah,obush, hpchn,hprst,			// Host Port
  ienaha,ibusha,iselha,oenaha,obusha,oselha,		// I/O Module 1 
  ienahb,ibushb,iselhb,oenahb,obushb,oselhb,		// I/O Module 2 
  ienaca,ibusca,iselca,oenaca,obusca,oselca,		// Core 1
  ienacb,ibuscb,iselcb,oenacb,obuscb,oselcb,		// Core 2
  ienata,ibusta,iselta,oenata,obusta,oselta,		// Processor Module 1
  ienatb,ibustb,iseltb,oenatb,obustb,oseltb,		// Processor Module 2
  testd);

// SDRAM controller 
sddrXram #(IBW,8,MBL) mb (
  gclkms, rmem, ago,apd,acmd,afull,qrdym,qselm,
  ioclk, irdym,ienam,ibusm, ordym,oenam,obusm,
  mckp,mckn,mcke,mwe,mcas,mras,ma,mba,mdm,mdq,mdqsp,mdqsn,mcs,mrst,modt,mact,
  mclk,trsp, testr);

// Rocket IO interface
wire rrio = rprc;
riocontroller #(PYLW,3,3) riop (
  clk, pager,rdp,wr,page,addr,data,datar,  // JTAG debug port
  gclks,rrio, selrr,sads, swrs,swrbus, srds,srdbusr,
  lclktr, pyrxp,pyrxn,pytxp,pytxn, pyctl,
  gclkv, rclk1,rrst1,ribus1,robus1, rclk2,rrst2,ribus2,robus2,
  testrio);

assign rrst1 = L;
assign rrst2 = L;
`ifdef RIO_GEN3
assign hohold = hohold_;
`else
assign hohold = ribus1[RBW+5:RBW+2];
assign robus1[RBW+5:RBW+2] = hihold;
`endif

// hyper-transport input port 
hypin #(IBW,8) hi (rclk1[1],rrst1,ribus1[RBW],ribus1[RBW-1:0], hihold,hohold_,
  ioclk,rio, ostat[`HA],oenaha,oselha,obusha, ostat[`HB],oenahb,oselhb,obushb, 
  hypm,srdbusd,srdbush,hirsta,hirstb,testhi);

// Core 1 engine
`ENGINE1 #(1,IBW) core1 (gclkf,rmod, csel[`CA],saddr, swrf,swrbus, srdf,srdbusca, sack[`CA],
  ioclk, istat[`CA],ienaca,iselca,ibusca, ostat[`CA],oenaca,oselca,obusca, testca);

// Core 2 engine
`ENGINE2 #(2,IBW) core2 (gclkf,rmod, csel[`CB],saddr, swrf,swrbus, srdf,srdbuscb, sack[`CB],
  ioclk, istat[`CB],ienacb,iselcb,ibuscb, ostat[`CB],oenacb,oselcb,obuscb, testcb);

// Core 3 / MultiCore 1 engine
`ENGINE3 #(3,IBW) mcore1 (gclkf,rmod, csel[`TA],saddr, swrf,swrbus, srdf,srdbusta, sack[`TA],
  ioclk, istat[`TA],ienata,iselta,ibusta, ostat[`TA],oenata,oselta,obusta, testta);

// Core 4 / MultiCore 2 engine
`ENGINE4 #(4,IBW) mcore2 (gclkf,rmod, csel[`TB],saddr, swrf,swrbus, srdf,srdbustb, sack[`TB],
  ioclk, istat[`TB],ienatb,iseltb,ibustb, ostat[`TB],oenatb,oseltb,obustb, testtb);

// hyper-transport output port
hypout #(IBW,8) ho (rclk1[0],rrst1,ribus1[RBW+1],robus1[RBW],robus1[RBW-1:0], hihold,hohold,
  ioclk,rio, istat[`HA],ienaha,iselha,ibusha, istat[`HB],ienahb,iselhb,ibushb, 
  istat[`HA1],istat[`HB1],srdbusd,speed,testho);

// Test Port Output
wire tsync;
testport #(0,0) tport (
  clk,L, rd,wr,paget,addr,data,datat, tp,tsync,oppms,cfg,
  testd,testr,testhi,testho, testca,testrio,testta,testtb);

// temp Sensor
max6577 ts (gclks,oppms,tsense,temp);

endmodule
