/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/** 
  Conversion routines for different bit lengths

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  12/15/2004

*/

/*
 12b/16b pack and unpack routines

  12b  256by = 64x4by = 84x3by + 4by = 21 loops of ABC or 63 lines then extra read of 1 line
  16b  336by =          84x4by + 0by

  A 76543210  
  B 76543210
  C 76543210

    543_210_
    321_076_ 
    107_654_
    765_432_

   go vena   cnt
   go1 wbus
   go2 q     sel
   go3 wbus

*/

module convert_12to16 (clk,rst, vrdy,vena,vbus, wrdy,wena,wbus);
  parameter BW=32;
  input clk,rst;
  input vrdy,wrdy;
  output vena,wena;
  input [BW-1:0] vbus;
  output [BW-1:0] wbus;

  reg [6:0] count;
  reg [1:0] sel,seld;
  reg go,go1,go2,go3,cap;
  wire cdone = (count==83);
  wire crst = rst || (go && cdone);
  assign vena = go && (count[1:0]!=3 || cdone);
  assign wena = go3;
  always @(posedge clk) begin
    go <= vrdy && wrdy; 
    go1 <= go; go2 <= go1; go3 <= go2;
    cap <= go && (count[1:0]!=3);
    if (crst) count <= 0; else if (go) count <= count+1;
    sel <= count[1:0];
    seld <= sel;
  end

genvar i;
generate
for (i=0; i<BW; i=i+32) begin
  reg [63:0] q;
  always @(posedge clk) if (cap) q <= {vbus[31+i:i],q[63:32]};
  wire [31:0] q0 = {q[55:44],4'h0,q[43:32],4'h0};	// c=0
  wire [31:0] q1 = {q[47:36],4'h1,q[35:24],4'h1};	// c=1
  wire [31:0] q2 = {q[39:28],4'h2,q[27:16],4'h2};	// c=2
  wire [31:0] q3 = {q[63:52],4'h3,q[51:40],4'h3};	// c=3
  mux4xNp #(32) mx (wbus[31+i:i], q0,q1,q2,q3, seld,clk);
end
endgenerate

endmodule

module convert_12to16i (clk,rst,swap, vrdy,vena,vbus, wrdy,wena,wbus);
  parameter BW=64;
  input clk,rst,swap;
  input vrdy,wrdy;
  output vena,wena;
  input [BW-1:0] vbus;
  output [BW-1:0] wbus;

  reg [6:0] count;
  reg [1:0] sel,seld;
  reg go,go1,go2,go3,cap;
  wire cdone = (count==41);
  wire crst = rst || (go && cdone);
  assign vena = go && (count[1:0]!=3);
  assign wena = go3;
  always @(posedge clk) begin
    go <= vrdy && wrdy; 
    go1 <= go; go2 <= go1; go3 <= go2;
    cap <= go && (count[1:0]!=3);
    if (crst) count <= 0; else if (go) count <= count+1;
    sel <= count[1:0];
    seld <= sel;
  end

  reg [2*BW-1:0] q;
  wire [BW-1:0] vbusx = {vbus[63:48],vbus[31:16],vbus[47:32],vbus[15:0]};
  always @(posedge clk) if (cap) q <= {vbus,q[2*BW-1:BW]};
  wire [BW-1:0] q0 = {q[11+100:100],4'h0,  q[11+88:88],  4'h0,  q[11+76:76],4'h0,  q[11+64:64],4'h0};	// c=0
  wire [BW-1:0] q1 = {q[11+84:84],  4'h0,  q[11+72:72],  4'h0,  q[11+60:60],4'h0,  q[11+48:48],4'h0};	// c=1
  wire [BW-1:0] q2 = {q[11+68:68],  4'h0,  q[11+56:56],  4'h0,  q[11+44:44],4'h0,  q[11+32:32],4'h0};	// c=2
  wire [BW-1:0] q3 = {q[11+116:116],4'h0,  q[11+104:104],4'h0,  q[11+92:92],4'h0,  q[11+80:80],4'h0};	// c=3
  wire [BW-1:0] wbusx; mux4xNp #(BW) mx (wbusx, q0,q1,q2,q3, seld,clk);
  assign wbus = swap? {wbusx[63:48],wbusx[31:16],wbusx[47:32],wbusx[15:0]} : wbusx;	// fix QQIIQQII ordering

endmodule
