// $Id: sim_main.cpp 712 2006-06-29 16:11:21Z wsnyder $
// DESCRIPTION: Verilator Example: Top level main for invoking model
//
// Copyright 2003-2006 by Wilson Snyder. This program is free software; you can
// redistribute it and/or modify it under the terms of either the GNU
// General Public License or the Perl Artistic License.

#include <verilated.h>		// Defines common routines
#include <verilated_vcd_c.h>		// Defines common routines
#include "Vcore.h"		// From Verilating "core.v"

Vcore *core;			// Instantiation of module	
int flg=0;
int ttime;
VerilatedVcdC *tfp;

int timeclk (int sc, int ic) {
  core->sclk=sc; core->ioclk=ic; core->eval();
  if (tfp!=NULL) tfp->dump(ttime++);
}

int timestep () {
  timeclk(1,1);
  timeclk(1,0);
  timeclk(0,1);
  timeclk(0,0);
}


extern "C" int VCore_open () {
  int i;
  char *tfn = getenv("VERILATOR_TFN");
  tfp=NULL;
  ttime=0;
  core = new Vcore;		// Create instance of module
  Verilated::debug(0);
  Verilated::traceEverOn(true);
  core->srst = 1;
  core->scs = 1;		// leave chip select always on
  core->swr = 0;
  core->srd = 0;
  for (i=0; i<32; i++) {	// some clocks with reset
    if (i==16) core->srst = 0;
    timestep();
  }
  if (tfn!=NULL) {
    tfp = new VerilatedVcdC;
    core->trace (tfp,99);
    printf("Trace File Name=%s\n",tfn);
    tfp->open(tfn);
  } 
}

extern "C" int VCore_syswr (int addr, int value) {
  int i;
  core->saddr = addr;
  core->swrbus = value;
  for (i=0; i<8; i++) {	// some clocks 
    core->swr = (i==2)?1:0;
    timestep();
  }
}

extern "C" int VCore_syswrblk (int addr, int *value, int size) {
  int i; for (i=0; i<size; i++) VCore_syswr(addr,value[i]);
}

extern "C" int VCore_syswrn (int addr, int value, int n) {
  int i;
  core->saddr = addr;
  core->swrbus = value;
  for (i=0; i<n; i++) {	// some clocks 
    core->swr = (i==0)?1:0;
    timestep();
  }
}

extern "C" int VCore_sysrd (int addr) {
  int i,value;
  core->saddr = addr;
  for (i=0; i<8; i++) {	// some clocks 
    core->srd = (i==4)?1:0;
    timestep();
    if (i==4) value = core->srdbus; 
  }
  return value;
}

extern "C" int VCore_process (int *in, int nin, int *out, int nout, int nbw) {
  int i, j, k=nbw/32, iin=0, iout=0, ic=0;
  int extra = (nin<=0)? 1024 : 64; // to flush pipelines
  nin /= 4; nout /= 4;		// 32 bit data interface
//  printf("In to nin=%d nout=%d extra=%d\n",nin,nout,extra);
  for (i=0; i<extra; i++, ic=1-ic) {	// some clocks 
    core->iena = (core->istat!=0 && iin<nin)?1:0;
    core->oena = (core->ostat!=0 && iout<nout)?1:0;
    timeclk(ic,1);
    timeclk(ic,0);
    if (core->iena) for (j=0; j<k; j++) core->ibus[j] = in[iin++];
    if (core->oena) for (j=0; j<k; j++) out[iout++] = core->obus[j];
    if (flg) printf("Proc i=%08x test=%08x\n",i,core->test);
    if (iin<nin || core->iena || core->oena) i=0;	// keep going
  }
//  printf("Out to iin=%d iout=%d i=%d\n",iin,iout,i);
  return iout*4;
}

extern "C" int VCore_close() {
  if (tfp!=NULL) tfp->close();
  core->final();
}

