/**********************************************
 ICE Proprietary Software - do NOT disseminate
 **********************************************/
/*
  Core Engine Simulator Interface

  Jeff Schoen
  Innovative Computer Engineering, Inc.
  1/15/2003

*/
`define IO_CPP
`define PE_CPP
`undef ENGINE1
`include "mdefs.h"

module core (sclk,srst, scs,saddr, swr,swrbus, srd,srdbus, sack,
	     ioclk, istat,iena,isel,ibus, ostat,oena,osel,obus, test);

  parameter PORT=1;	// engine index
  parameter IBW=`SIM_IBW;  // 32b/64b in processor modules, 128b in PCIe sim
  parameter SBW=32;	// System bus width - always 32

  input sclk;		// system clock
  input srst;		// system reset
  input scs;		// system select
  input swr;		// register write
  input srd;		// register read
  input [31:0] saddr;	// register address
  input [31:0] swrbus;	// register data
  output [31:0] srdbus;	// register data
  output sack;

  input ioclk;		// I/O data bus clock
  output istat;		// istat - ready to accept 64 byte burst
  input iena;		// input enable  (one cycle early) (continuous 64 byte burst)
  input [7:0] isel;
  input [127:0] ibus;	// input data bus
  output ostat;		// ostat - ready to provide 64 byte burst
  input oena;		// output enable (one cycle early) (continuous 64 byte burst)
  output [7:0] osel;
  output [127:0] obus;	// output data bus

  output [7:0] test;	// 8 test signals for scope probes

  wire L=0, H=1;

  // buffer the input as verilator has trouble with direct operations on this bus interface to C
  reg jena; reg [7:0] jsel; reg [IBW-1:0] jbus; 
  always @(posedge ioclk) jena <= iena;
  always @(posedge ioclk) jsel <= isel;
  always @(posedge ioclk) jbus <= ibus;
  wire [1:0] sclks = {ioclk,sclk};

`ifdef CORE_DUO
  reg mduo,opha,msel,mena,oval,oenax,ovalx; reg [3:0] ofmt;
  wire [7:0] osel1,osel2; wire [IBW-1:0] obus1,obus2,obusx; 
  wire istat1,istat2,ostat1,ostat2,ostatx; wire [7:0] test1,test2;
  always @(posedge sclk) begin
    if (scs&&swr&&saddr==0) begin 
      ofmt <= swrbus[15:12];
      mduo <= swrbus[3];
      mena <= swrbus[0];
    end
  end
  always @(posedge ioclk) begin
    if (!mena) opha <= !mduo; else if (mduo&oenax) opha <= !opha;
    msel <= mduo && opha;
    oenax <= (mduo? ostat1 && ostat2 : ostat1) && ostatx;
    ovalx <= oenax;
    oval <= oena;
  end
  `ENGINE #(1,IBW) inst1 (sclk,srst, scs,saddr, swr,swrbus, srd,srdbus, sack,
                        ioclk, istat1,jena&&(mduo?H:H),jsel,jbus, ostat1,oenax&&(mduo?opha:H),osel1,obus1, test1);
  `ENGINE #(2,IBW) inst2 (sclk,srst, scs,saddr, swr,swrbus, srd,srdbus, sack,
                        ioclk, istat2,jena&&(mduo?H:L),jsel,jbus, ostat2,oenax&&(mduo?opha:L),osel2,obus2, test2);
  wire [1:0] mfmt=(ofmt==8)?2:(ofmt==9)?1:(ofmt==0)?1:0;	// CI,CB,SI,SB - always CI
  mix2xNci #(IBW) mx (obusx, obus1,obus2, msel,mduo);		// every other 32b from alternate sides
  // buffer the output as verilator has trouble with direct operations on this bus interface to C
  fifoNxM #(IBW,IBW) mf (ioclk,!mena, ostatx,ovalx,obusx, ioclk,!mena, ostat,oena,obus);
  assign istat = mduo? istat1 && istat2 : istat1;
  assign osel  = osel1;
`else
  `ENGINE #(1,IBW) inst (sclks,srst, scs,saddr, swr,swrbus, srd,srdbus, sack,
                        ioclk, istat,jena,jsel,jbus, ostat,oena,osel,obus, test);
`endif

reg [31:0] jjj;
reg [31:0] kkk;
always @(posedge ioclk) begin
//  if (!mena) jjj <= 0; else if (oval) jjj <= jjj+1;
//  kkk <= kkk+1;
//  if (oval && jjj<10) $write("Out jjj=%d kkk=%d mduo=%d msel=%d obus=%x\n",jjj,kkk,mduo,msel,obus);
//  if (oval && jjj<20) $write("Out jjj=%d kkk=%d oval=%d mena=%d mduo=%d opha=%d obus=%x\n",jjj,kkk,oval,mena,mduo,opha,obus);
end

`ifdef PE_IOM
  defparam inst.IOTYPE = `IOTYPE;
`endif

endmodule

