
`timescale 1ps/1ps

`define HYP_RIO_ONOF 1
//`define LOOPBACK 1
//`define PIC8DBG 1
//`define RATE_8
`define RATE_C 1

`ifdef RATE_B
`define RIO_QPLL_DIV 88
`define RIO_LINE_RATE 6.875
`define RIO_USR_CLKF 208.3333333
`elsif RATE_C
`define RIO_QPLL_DIV 84
`define RIO_LINE_RATE 6.5625
`define RIO_USR_CLKF 198.8636363
`else
`define RIO_QPLL_DIV 80
`define RIO_LINE_RATE 6.250
`define RIO_USR_CLKF 189.3939394
`endif

`ifdef PIC8DBG
  `include "../iom/parts.v"
`endif

module icet_lcl(clk,reset,rdy, txval,txdata, rxval,rxdata, txaux,rxaux,
                drpclk,drprst,drpen,drpaddr,drpwe,drpdi,drprdy,drpdo,dmonitorout,
                gthrefclk, gthtxp,gthtxn, gthrxp,gthrxn, testout, dbgsys,pkterr_cerr,pkterr_terr,pkterr_tpkt
`ifdef PIC8DBG
  ,dbgstat, icedbg_clk,icedbg_memaddr,icedbg_memdata
`endif
  );
   
  parameter CH = 1;
  parameter UCW = 32;
  parameter PCW = 64;
  parameter INV = 0;
  parameter LOOPBACK = 0;

  localparam PCH = 4; /* Physical number of channels is always 4 */
  localparam PLC = (CH==4)? 1 : (CH==2)? 2 : 4; /* Number of physical channels per logical channel */
  localparam LCW = PLC*PCW;
  localparam LHW = PLC*6;
  localparam SYNC_SEQ_LEN = 33;

  input clk;
  input [4:0] reset;
  output [CH-1:0] rdy;

  input [CH-1:0] txval;
  input [(CH*UCW)-1:0] txdata;

  output [CH-1:0] rxval;
  output [(CH*UCW)-1:0] rxdata;

  input [31:0] txaux;
  output [31:0] rxaux;

  input drpclk;
  input drprst;
  input drpen;
  input [8:0] drpaddr;
  input drpwe;
  input [15:0] drpdi;
  output [PCH-1:0] drprdy;
  output [(PCH*16)-1:0] drpdo;
  output [(PCH*17)-1:0] dmonitorout;

  input gthrefclk;
  output [PCH-1:0] gthtxp,gthtxn;
  input  [PCH-1:0] gthrxp,gthrxn;

  output [7:0] testout;

  input [31:0] dbgsys;
  output [4*32-1:0] pkterr_cerr;
  output [4*32-1:0] pkterr_terr;
  output [4*32-1:0] pkterr_tpkt;

`ifdef PIC8DBG
  output [31:0] dbgstat;
  input icedbg_clk; 
  input [17:0] icedbg_memaddr;
  output [31:0] icedbg_memdata;
`endif

  genvar i;
  wire tx_clk;
  (* dont_touch = "yes" *) wire [PCH-1:0] txch_clk;
  wire [PCH-1:0] txch_active;
  wire tx_rst,tx_reset,tx_reset_done;
  wire [PCH-1:0] txch_reset_done,txch_reset_done_;
  wire [CH-1:0] txrdy, tx_rdy, txrdys;
  wire [CH*LCW-1:0] txpkt_data;
  wire [CH-1:0] txpkt_rdy,txpkt_en;
  wire [CH*LCW-1:0] tx_data_xcvr,tx_data_xcvr_icet,tx_data_xcvr_lpbk;
  wire [(6*PCH)-1:0] tx_header_xcvr,tx_header_xcvr_icet,tx_header_xcvr_lpbk;

  wire [CH-1:0] rx_clk;
  wire [PCH-1:0] rxch_uclk;
  (* dont_touch = "yes" *) wire [PCH-1:0] rxch_clk;
  wire [CH-1:0] rx_rst,rx_reset,rx_reset_done;
  wire [CH-1:0] rx_locked;
  wire [CH-1:0] rx_type;
  wire [CH-1:0] rx_type_err;
  wire [CH-1:0] rx_active;
  wire [CH*LCW-1:0] rx_data_xcvr;
  wire [CH*LCW-1:0] rxsync_data,rxsync_datal;
  wire [CH-1:0] rxsync_val;
  wire [CH-1:0] rx_rdy,rxstat;
  wire [(CH*UCW)-1:0] rxdata_i;
  reg  [CH-1:0] rxena,rxval_i;
  wire [CH-1:0] rx_resync;
  wire [PCH-1:0] rxch_reset_done;
  wire [PCH-1:0] rxch_active;
  wire [PCH-1:0] rx_gearboxslip;
  wire [PCH-1:0] rxlpmen_in = {PCH{1'b1}};
  wire [(PCH*6)-1:0] rx_header_xcvr;
  wire [PCH-1:0] rx_resync_;

  //Debug Breakout...
  wire [2:0] dbg_rst    = dbgsys[2:0];
  wire txeq_en          = dbgsys[3];
  wire capture_rst      = dbgsys[4];
  wire invert_en        = !dbgsys[5];
  wire pkterr_rst       = dbgsys[6];
  wire capture_sel      = dbgsys[7];
  wire [3:0] trig_sel   = dbgsys[11:8];
  wire [1:0] dbg_osel   = dbgsys[13:12];
  wire [1:0] dbg_sel    = dbgsys[15:14];
  wire [3:0] txeinj     = dbgsys[19:16];
  wire loopback_en      = dbgsys[20];
  wire [2:0] loopback_mode = dbgsys[23:21];

  wire rst = reset[0] | dbg_rst[0];
  wire [PCH-1:0] txrst_clkdata = {PCH{reset[1]}};
  wire [PCH-1:0] txrst_data    = {PCH{reset[2]}} | {PCH{dbg_rst[1]}};
  wire [PCH-1:0] rxrst_clkdata = {PCH{reset[3]}};
  wire [PCH-1:0] rxrst_data    = {PCH{reset[4]}} | {PCH{dbg_rst[2]}} | rx_resync_;

  wire [PCH-1:0] txrst_any = {PCH{rst}} | txrst_clkdata | txrst_data;
  wire [PCH-1:0] rxrst_any = {PCH{rst}} | rxrst_clkdata | rxrst_data;
  wire rst_any = |reset;

//*************************************************************************************************************//
//       TX Side
//*************************************************************************************************************//
  wire [3:0] sync_state;
  wire [5:0] sync_cnt;
  wire [63:0] tx_data_dbg [4:0];
  wire [CH-1:0] tx_data_dbg_val;
  reg [CH-1:0] txpkt_end;
  always@(posedge tx_clk) txpkt_end <= txpkt_en;

  bit_sync tx_reset_bit_synchronizer_inst (tx_clk,(|txrst_any),tx_rst);
  assign tx_reset = tx_rst | ~tx_reset_done;

  wire [3:0] txeinj_c,txeinj_r;
  reg [3:0] txeinj_d;
  bit_sync txeinj_bit_synchronizer_inst0 (tx_clk,txeinj[0],txeinj_c[0]);
  bit_sync txeinj_bit_synchronizer_inst1 (tx_clk,txeinj[1],txeinj_c[1]);
  bit_sync txeinj_bit_synchronizer_inst2 (tx_clk,txeinj[2],txeinj_c[2]);
  bit_sync txeinj_bit_synchronizer_inst3 (tx_clk,txeinj[3],txeinj_c[3]);

  always@(posedge tx_clk) begin
    txeinj_d <= txeinj_c;
  end
  assign txeinj_r[0] = txeinj_c[0] & !txeinj_d[0];
  assign txeinj_r[1] = txeinj_c[1] & !txeinj_d[1];
  assign txeinj_r[2] = txeinj_c[2] & !txeinj_d[2];
  assign txeinj_r[3] = txeinj_c[3] & !txeinj_d[3];

generate
for(i=0;i<CH;i=i+1) begin: TX_CH_LOOP

  // Synchonize rdy signal to user clk domain
  bit_sync rdy_bit_synchronizer_inst (clk,(~rst_any & tx_rdy[i] & rx_locked[i]), txrdys[i]);	// trying to keep FIFO full
  assign rdy[i] = txrdys[i] && txrdy[i];
//  bit_sync rdy_bit_synchronizer_inst (clk,(~rst_any & tx_rdy[i] & rx_locked[i]),rdy[i]);	// original

  icet_pkthandle #(UCW,LCW,1) txpkt_inst (clk,tx_reset, txrdy[i],txval[i],txdata[(i+1)*UCW-1:i*UCW], tx_clk,(tx_reset|~tx_rdy[i]), txpkt_rdy[i],txpkt_en[i],txpkt_data[(i+1)*LCW-1:i*LCW]);

  icet_txsynclayer #(LCW,PCW,SYNC_SEQ_LEN,i) txsync_inst (tx_clk,tx_reset,tx_rdy[i],tx0_idle, txpkt_rdy[i],txpkt_en[i],txpkt_data[(i+1)*LCW-1:i*LCW],
                                           tx_data_xcvr_icet[(i+1)*LCW-1:i*LCW],tx_header_xcvr_icet[(i+1)*LHW-1:i*LHW],txeinj_r[(i+1)*PLC-1:i*PLC], txaux);
end
endgenerate

  assign tx_data_xcvr = tx_data_xcvr_icet;
  assign tx_header_xcvr = tx_header_xcvr_icet;

//*************************************************************************************************************//
//       RX Side
//*************************************************************************************************************//

generate
if(LOOPBACK==1) begin
  assign rxsync_datal[(1*64)-1:0*64] = rxsync_data[(2*64)-1:1*64];
  assign rxsync_datal[(2*64)-1:1*64] = rxsync_data[(1*64)-1:0*64];
  assign rxsync_datal[(3*64)-1:2*64] = rxsync_data[(4*64)-1:3*64];
  assign rxsync_datal[(4*64)-1:3*64] = rxsync_data[(3*64)-1:2*64];
end
else if(LOOPBACK==2) begin
  assign rxsync_datal[(1*64)-1:0*64] = rxsync_data[(4*64)-1:3*64];
  assign rxsync_datal[(2*64)-1:1*64] = rxsync_data[(3*64)-1:2*64];
  assign rxsync_datal[(3*64)-1:2*64] = rxsync_data[(2*64)-1:1*64];
  assign rxsync_datal[(4*64)-1:3*64] = rxsync_data[(1*64)-1:0*64];
end
else begin
  assign rxsync_datal[(1*64)-1:0*64] = rxsync_data[(1*64)-1:0*64];
  assign rxsync_datal[(2*64)-1:1*64] = rxsync_data[(2*64)-1:1*64];
  assign rxsync_datal[(3*64)-1:2*64] = rxsync_data[(3*64)-1:2*64];
  assign rxsync_datal[(4*64)-1:3*64] = rxsync_data[(4*64)-1:3*64];
end
endgenerate

  //assign testout = rxsync_dbg[7:0];
  //assign testout = {4'hB,2'b0,tx_clk,rx_clk};

  wire [63:0] rxsync_dbg;
  wire rxsync_dbgtrig;
  wire [63:0] rxsync_dbgdata [7:0];
  wire [63:0] rxsync_dbgdatax [3:0];
  wire [31:0] rxfec_dbg [3:0];
  wire [31:0] rxfec_dbgx [3:0];

//  assign testout = {rxstat[0],txpkt_rdy[0],rxena[0],txpkt_en[0]};
  assign testout = {rxstat[0],rxena[0],txrdy[0],txpkt_rdy[0],txval[0],txpkt_en[0]};

generate
for(i=0;i<CH;i=i+1) begin: RX_CH_LOOP
 bit_sync rx_reset_bit_synchronizer_inst2 (rx_clk[i],rxrst_any[i],rx_rst[i]);
 assign rx_reset[i] = rx_rst[i] | ~rx_reset_done[i];

 if(i==0) begin
  icet_rxsynclayer #(LCW,PCW,SYNC_SEQ_LEN,i) rxsync_inst0 (rx_clk[i],rx_reset[i],rx_active[i],rx_resync[i],rx_gearboxslip[(i+1)*PLC-1:i*PLC],rx_locked[i],rx_type[i],rx_type_err[i],
       (invert_en==1)? ~rx_header_xcvr[(i+1)*LHW-1:i*LHW]:rx_header_xcvr[(i+1)*LHW-1:i*LHW], (invert_en==1)? ~rx_data_xcvr[(i+1)*LCW-1:i*LCW]:rx_data_xcvr[(i+1)*LCW-1:i*LCW],
       rxsync_val[i],rxsync_data[(i+1)*LCW-1:i*LCW], rxaux,
       pkterr_rst,pkterr_cerr[(i+1)*32*PLC-1:i*32*PLC],pkterr_terr[(i+1)*32*PLC-1:i*32*PLC],pkterr_tpkt[(i+1)*32*PLC-1:i*32*PLC],
       idbg_rst,rxsync_dbgtrig,{rxsync_dbgdata[7],rxsync_dbgdata[6],rxsync_dbgdata[5],rxsync_dbgdata[4],rxsync_dbgdata[3],rxsync_dbgdata[2],rxsync_dbgdata[1],rxsync_dbgdata[0]});
 end
 else begin
  wire [31:0] rxaux_;
  icet_rxsynclayer #(LCW,PCW,SYNC_SEQ_LEN,i) rxsync_inst (rx_clk[i],rx_reset[i],rx_active[i],rx_resync[i],rx_gearboxslip[(i+1)*PLC-1:i*PLC],rx_locked[i],rx_type[i],rx_type_err[i],
       (invert_en==1)? ~rx_header_xcvr[(i+1)*LHW-1:i*LHW]:rx_header_xcvr[(i+1)*LHW-1:i*LHW], (invert_en==1)? ~rx_data_xcvr[(i+1)*LCW-1:i*LCW]:rx_data_xcvr[(i+1)*LCW-1:i*LCW],
       rxsync_val[i],rxsync_data[(i+1)*LCW-1:i*LCW], rxaux_,
       pkterr_rst,pkterr_cerr[(i+1)*32*PLC-1:i*32*PLC],pkterr_terr[(i+1)*32*PLC-1:i*32*PLC],pkterr_tpkt[(i+1)*32*PLC-1:i*32*PLC],,,);
 end
  icet_pkthandle #(LCW,UCW,0) rxpkt_inst (rx_clk[i],rx_reset[i], rx_rdy[i],rxsync_val[i],rxsync_datal[(i+1)*LCW-1:i*LCW], clk,rst, rxstat[i],rxena[i],rxdata_i[(i+1)*UCW-1:i*UCW]);
end
endgenerate

//*************************************************************************************************************//
//       Debug
//*************************************************************************************************************//
`ifdef PIC8DBG
  wire idbg_clk;
  wire [1:0] idbg_trig;
  wire [2*512-1:0] idbg_dat;
  wire [512-1:0] idbg_dat0;
  wire [512-1:0] idbg_dat1;
  wire idbg_rst;
  wire idbg_csel,idbg_tsel,idbg_dsel;
  wire idbg_oval;
  wire [UCW-1:0] idbg_odat;
  reg [31:0] idbg_tcnt;
  wire [31:0] icedbg_status,icedbg_status_;
  wire [31:0] icedbg_memdata_;

  assign idbg_clk = rx_clk[0];
  assign idbg_rst = capture_rst;

/*
  assign idbg_trig = {txval[1],rxval[1]};

  assign idbg_dat0[(0+1)*64-1:0*64] = {rst,31'd0,24'hAAAAAA,3'd0,tx_reset,2'd0,txval};
  assign idbg_dat0[(1+1)*64-1:1*64] = txdata[63:0];  
  assign idbg_dat0[(2+1)*64-1:2*64] = txdata[127:64];
  assign idbg_dat0[(3+1)*64-1:3*64] = 64'h1023456789ABCDEF; 
  assign idbg_dat0[(4+1)*64-1:4*64] = {rst,31'd0,2'd0,rxsync_val,12'hBBB,2'd0,rxstat,2'd0,rxena,rx_reset,2'd0,rxval}; 
  assign idbg_dat0[(5+1)*64-1:5*64] = rxdata[63:0];
  assign idbg_dat0[(6+1)*64-1:6*64] = rxdata[127:64];
  assign idbg_dat0[(7+1)*64-1:7*64] = 64'hFFFFFFFFDEADBEEF;

  assign idbg_dat1 = 0;
*/
  assign idbg_trig = {1'b1,rxsync_dbgtrig};

  assign idbg_dat0 = {rxsync_dbgdata[7],rxsync_dbgdata[6],rxsync_dbgdata[5],rxsync_dbgdata[4],
                      rxsync_dbgdata[3],rxsync_dbgdata[2],rxsync_dbgdata[1],rxsync_dbgdata[0]};

  assign idbg_dat1 = {64'h0000000000000000,64'h1111111111111111,64'h2222222222222222,64'h3333333344444444,
                      64'h5555555566666666,64'h777788889999AAAA,64'hBBBBCCCCDDDDEEEE,12'h123,rxch_reset_done,12'h456,txch_reset_done_,idbg_tcnt};

  assign idbg_csel = capture_sel;
  assign idbg_tsel = trig_sel[0];
  assign idbg_dsel = dbg_sel[0];
  assign idbg_dat = {idbg_dat1,idbg_dat0};

  always@(posedge idbg_clk) idbg_tcnt <= (capture_rst)? 0 : idbg_tcnt + 1;

  ice_debug #(2,2,512,UCW,"MEM") xdbg_inst (idbg_clk,idbg_rst,idbg_csel,idbg_tsel,idbg_dsel,idbg_trig,idbg_dat, 
                                            clk,rst,idbg_oval,idbg_odat, icedbg_clk,icedbg_memaddr,icedbg_memdata_, icedbg_status);

  assign icedbg_memdata = (icedbg_memaddr[17])? 32'h01234567 : icedbg_memdata_;
  generate
  for (i=0;i<32;i=i+1) begin: dbg_sync_loop
    bit_sync icedbg_stat_bit_synchronizer_inst (clk,icedbg_status[i],icedbg_status_[i]);
  end
  endgenerate

  assign dbgstat = (icedbg_memaddr[18])? idbg_trig : (icedbg_memaddr[17])? 32'h89ABCDEF : icedbg_status_;

`else
  wire idbg_oval = rxval_i;
  wire [UCW-1:0] idbg_odat = {4{16'h01CE}};
`endif
//*************************************************************************************************************//
//     End Debug
//*************************************************************************************************************//

  //Configure the ICE Transport as a streaming interface to the upper level
  always@(posedge clk) begin
    rxena <= rxstat;
    rxval_i <= rxena;
  end

  assign rxval  = (dbg_osel==2)? txval  : (dbg_osel==1)? (capture_rst)? {CH{txval}} : idbg_oval : rxval_i;
  assign rxdata = (dbg_osel==2)? txdata : (dbg_osel==1)? (capture_rst)? {CH{64'h01234567DEADBEEF}} : idbg_odat : rxdata_i;

//*************************************************************************************************************//
//       Transceiver Interface
//*************************************************************************************************************//
  wire [PCH-1:0]     txclk_reset;
  wire [PCH-1:0] txprgdivresetdone_int;
  wire [PCH-1:0] txpmaresetdone_int;
  assign txclk_reset = ~(txprgdivresetdone_int & txpmaresetdone_int);

  wire [PCH-1:0] rxusrclk,rxusrclk2; 
  wire [PCH-1:0] rxclk_reset;
  wire [PCH-1:0] rxprgdivresetdone_int;
  wire [PCH-1:0] rxpmaresetdone_int;
  assign rxclk_reset = ~(rxprgdivresetdone_int & rxpmaresetdone_int);

  wire [2:0] txrst_state;
  wire reset_plllock_tx;

  assign tx_clk = txch_clk[0];

  wire [PCH-1:0] txrst_clkdata_;
  wire [PCH-1:0] txrst_data_;
  wire [PCH-1:0] rxrst_clkdata_;
  wire [PCH-1:0] rxrst_data_;

  bit_sync txrstcd_bit_synchronizer_inst0 (txch_clk[0],txrst_clkdata[0],txrst_clkdata_[0]);
  bit_sync txrstcd_bit_synchronizer_inst1 (txch_clk[1],txrst_clkdata[1],txrst_clkdata_[1]);
  bit_sync txrstcd_bit_synchronizer_inst2 (txch_clk[2],txrst_clkdata[2],txrst_clkdata_[2]);
  bit_sync txrstcd_bit_synchronizer_inst3 (txch_clk[3],txrst_clkdata[3],txrst_clkdata_[3]);

  bit_sync txrstd_bit_synchronizer_inst0 (txch_clk[0],txrst_data[0],txrst_data_[0]);
  bit_sync txrstd_bit_synchronizer_inst1 (txch_clk[1],txrst_data[1],txrst_data_[1]);
  bit_sync txrstd_bit_synchronizer_inst2 (txch_clk[2],txrst_data[2],txrst_data_[2]);
  bit_sync txrstd_bit_synchronizer_inst3 (txch_clk[3],txrst_data[3],txrst_data_[3]);

  bit_sync txrst_bit_synchronizer_inst0 (tx_clk,txch_reset_done[0],txch_reset_done_[0]);
  bit_sync txrst_bit_synchronizer_inst1 (tx_clk,txch_reset_done[1],txch_reset_done_[1]);
  bit_sync txrst_bit_synchronizer_inst2 (tx_clk,txch_reset_done[2],txch_reset_done_[2]);
  bit_sync txrst_bit_synchronizer_inst3 (tx_clk,txch_reset_done[3],txch_reset_done_[3]);
  assign tx_reset_done = &txch_reset_done_;

  bit_sync rxrstcd_bit_synchronizer_inst0 (rxch_clk[0],rxrst_clkdata[0],rxrst_clkdata_[0]);
  bit_sync rxrstcd_bit_synchronizer_inst1 (rxch_clk[1],rxrst_clkdata[1],rxrst_clkdata_[1]);
  bit_sync rxrstcd_bit_synchronizer_inst2 (rxch_clk[2],rxrst_clkdata[2],rxrst_clkdata_[2]);
  bit_sync rxrstcd_bit_synchronizer_inst3 (rxch_clk[3],rxrst_clkdata[3],rxrst_clkdata_[3]);

  bit_sync rxrstd_bit_synchronizer_inst0 (rxch_clk[0],rxrst_data[0],rxrst_data_[0]);
  bit_sync rxrstd_bit_synchronizer_inst1 (rxch_clk[1],rxrst_data[1],rxrst_data_[1]);
  bit_sync rxrstd_bit_synchronizer_inst2 (rxch_clk[2],rxrst_data[2],rxrst_data_[2]);
  bit_sync rxrstd_bit_synchronizer_inst3 (rxch_clk[3],rxrst_data[3],rxrst_data_[3]);

generate
  if(CH==4) begin
    assign rxusrclk  = rxch_uclk;
    assign rxusrclk2 = rxch_clk;
    assign rx_clk = rxch_clk;
    assign rx_reset_done = rxch_reset_done;
    assign rx_active = rxch_active;
    assign rx_resync_ = rx_resync;
  end
  else if(CH==2) begin
    assign rxusrclk  = {rxch_uclk[2],rxch_uclk[2],rxch_uclk[0],rxch_uclk[0]};
    assign rxusrclk2 = {rxch_clk[2],rxch_clk[2],rxch_clk[0],rxch_clk[0]};
    assign rx_clk = {rxch_clk[2],rxch_clk[0]};
    bit_sync rxrst_bit_synchronizer_inst0 (rx_clk[0],(rxch_reset_done[0] & rxch_reset_done[1]),rx_reset_done[0]);
    bit_sync rxrst_bit_synchronizer_inst1 (rx_clk[1],(rxch_reset_done[2] & rxch_reset_done[3]),rx_reset_done[1]);
    bit_sync rxact_bit_synchronizer_inst0 (rx_clk[0],(rxch_active[0] & rxch_active[1]),rx_active[0]);
    bit_sync rxact_bit_synchronizer_inst1 (rx_clk[1],(rxch_active[2] & rxch_active[3]),rx_active[1]);
    assign rx_resync_ = {{2{rx_resync[1]}},{2{rx_resync[0]}}};
  end
  else begin
    assign rxusrclk  = {4{rxch_uclk[0]}};
    assign rxusrclk2 = {4{rxch_clk[0]}};
    assign rx_clk = rxch_clk[0];
    bit_sync rxrst_bit_synchronizer_inst0 (rx_clk[0],(rxch_reset_done[3] & rxch_reset_done[2] & rxch_reset_done[1] & rxch_reset_done[0]),rx_reset_done[0]);
    bit_sync rxact_bit_synchronizer_inst0 (rx_clk[0],(rxch_active[3] & rxch_active[2] & rxch_active[1] & rxch_active[0]),rx_active[0]);
    assign rx_resync_ = {4{rx_resync[0]}};
  end
endgenerate

ice_gth_t_wrap icet_gth_inst (
  .gthrxn_in(gthrxn),
  .gthrxp_in(gthrxp),
  .gthtxn_out(gthtxn),
  .gthtxp_out(gthtxp),
  .gtwiz_userclk_tx_reset_in(txclk_reset),
  .gtwiz_userclk_tx_srcclk_out(),
  .gtwiz_userclk_tx_usrclk_out(),
  .gtwiz_userclk_tx_usrclk2_out(txch_clk),
  .gtwiz_userclk_tx_active_out(),
  .gtwiz_userclk_rx_reset_in(rxclk_reset),
  .gtwiz_userclk_rx_srcclk_out(),
  .gtwiz_userclk_rx_usrclk_out(rxch_uclk),
  .gtwiz_userclk_rx_usrclk2_out(rxch_clk),
  .gtwiz_userclk_rx_active_out(rxch_active),
  .gtwiz_reset_clk_freerun_in({1{clk}}),
  .gtwiz_reset_all_in({4{rst}}),
  .gtwiz_reset_tx_pll_and_datapath_in(txrst_clkdata_),
  .gtwiz_reset_tx_datapath_in(txrst_data_),
  .gtwiz_reset_rx_pll_and_datapath_in(rxrst_clkdata_),
  .gtwiz_reset_rx_datapath_in(rxrst_data_),
  .gtwiz_reset_rx_cdr_stable_out(),
  .gtwiz_reset_tx_done_out(txch_reset_done),
  .gtwiz_reset_rx_done_out(rxch_reset_done),
  .gtwiz_userdata_tx_in(tx_data_xcvr),
  .gtwiz_userdata_rx_out(rx_data_xcvr),
  .gtrefclk00_in(gthrefclk),
  .qpll0outclk_out(),
  .qpll0outrefclk_out(),
  .qpll1outclk_out(),
  .qpll1outrefclk_out(),
  .rxgearboxslip_in(rx_gearboxslip),
  .txheader_in(tx_header_xcvr),
  .txsequence_in(0),
  .rxdatavalid_out(),
  .rxheader_out(rx_header_xcvr),
  .rxheadervalid_out(),
  .rxpmaresetdone_out(rxpmaresetdone_int),
  .rxprgdivresetdone_out(rxprgdivresetdone_int),
  .rxstartofseq_out(),
  .txpmaresetdone_out(txpmaresetdone_int),
  .txprgdivresetdone_out(txprgdivresetdone_int),
  .drpaddr_in({4{drpaddr}}),
  .drpclk_in ({4{drpclk}}),
  .drpdi_in  ({4{drpdi}}),
  .drpen_in  ({4{drpen}}),
  .drprst_in ({4{drprst}}),
  .drpwe_in  ({4{drpwe}}),
  .drpdo_out (drpdo),
  .drprdy_out(drprdy),
  .rxlpmen_in(rxlpmen_in),
  .txeq_en(txeq_en),
  .rxusrclk2_in(rxusrclk2),
  .rxusrclk_in(rxusrclk)
);

endmodule 

module icet_syncgen32 (clk,rst,en, dout);
  parameter ND = 1;
  
  input clk,rst,en;
  output [ND-1:0] dout;

  wire [31:0] sync_code = 32'h1B3D2B88;
  reg [31:0] sync_sreg;

  initial sync_sreg = sync_code;
  always@(posedge clk) begin
    if(rst | ~en) sync_sreg <= sync_code;
    else sync_sreg <= {sync_sreg[30:0],sync_sreg[31]};
  end

  assign dout = {ND{sync_sreg[31]}};

endmodule
 
module icet_syncgen (clk,rst,en, dout);
  parameter SL = 33;
  parameter SINIT = 33'h18A5C240D;
  parameter ND = 1;
  
  input clk,rst,en;
  output [ND-1:0] dout;

  wire [SL-1:0] sync_code = SINIT;
  reg [SL-1:0] sync_sreg;

  initial sync_sreg = sync_code;
  always@(posedge clk) begin
    if(rst | ~en) sync_sreg <= sync_code;
    else sync_sreg <= {sync_sreg[SL-2:0],sync_sreg[SL-1]};
  end

  assign dout = {ND{sync_sreg[SL-1]}};

endmodule
 
module icet_rxsynclayer(clk,rst,en,resync,slip,locked,ptype,type_err, theader,tdat, oval,odat, rxaux, pkterr_rst,pkterr_cerr,pkterr_terr,pkterr_tpkt, rxdbgrst,rxdbgtrig,rxdbgdata);
  parameter LCW = 128; 
  parameter PCW = 64; /* Physical Channel Bus Width */
  parameter SEQ_LEN = 33;
  parameter CHNUM = 0;
  parameter N = LCW/PCW; /* Number of Physical Transceiver Links in Logical Link*/

  input clk,rst,en;
  output resync;
  output locked,ptype,type_err;
  output [N-1:0] slip; 
  input [6*N-1:0] theader;
  input [(N*PCW)-1:0] tdat;
  output oval;
  output [(N*PCW)-1:0] odat;
  output [31:0] rxaux;
  input pkterr_rst;
  output [N*32-1:0] pkterr_cerr;
  output [N*32-1:0] pkterr_terr;
  output [N*32-1:0] pkterr_tpkt;
  input rxdbgrst;
  output rxdbgtrig;
  output [8*64-1:0] rxdbgdata;

  wire lockedd;
  wire [5:0] header [N-1:0];
  wire [N-1:0] ch_resync;
  wire [N-1:0] ch_lock;
  wire [N-1:0] ch_phase,ch_phase_;
  wire [N-1:0] ch_align;
  wire [N-1:0] ch_pheadr,ch_pheadrd;
  wire [N-1:0] ch_scramsync;
  //reg  [N-1:0] ch_fecclr;
  wire  [N-1:0] ch_fecen;
  reg  [N-1:0] ch_fecen_;
  wire [31:0]  ch_fec [N-1:0];
  wire [PCW-1:0] ch_fec_din;
  //wire [(N*PCW)-1:0] sdat;
  wire [PCW-1:0] scram_seq [N-1:0];
  wire [PCW-1:0] sdat [N-1:0];
  wire [PCW-1:0] sadat [N-1:0];
  wire [PCW-1:0] sdatd [N-1:0];
  wire [N-1:0] irdy,iena;
  wire [N-1:0] ostat;
  reg  oena;
  reg  val;
  wire [5:0] ch_corr_coe [N-1:0];
  wire [3:0] ch_dstate [N-1:0];
  wire [5:0] ch_dcnt0 [N-1:0]; 
  wire [5:0] ch_dcnt1 [N-1:0]; 
  wire [31:0] ch_dsreg [N-1:0];
  wire [31:0] ch_dcreg [N-1:0];
  wire [31:0] ch_dxor [N-1:0];
  wire [31:0] ch_tpkt_cnt [N-1:0];
  wire [31:0] ch_terr_cnt [N-1:0];
  wire [31:0] ch_cerr_cnt [N-1:0];
  wire [PCW-1:0] edat [N-1:0];
  wire [PCW-1:0] cdat [N-1:0];
  wire [N-1:0] cval;
  wire [8*64-1:0] fecdbg;
  wire fecdtrig;

  reg locked_i;
  always@(posedge clk) locked_i <= (rst)? 0 : ((&ch_lock) & ~(|ch_phase)) | (locked_i & (ch_lock));
  assign locked = locked_i; 

  assign ptype = (ch_phase==0)? 0 : 1;
  assign type_err = (ch_phase != 0) & (ch_phase != {N{1'b1}});
  assign oval = val;

  always@(posedge clk) begin
    if(rst | (ostat != {N{1'b1}})) oena <= 1'b0;
    else oena <= 1'b1;

    val <= oena;
  end

  assign resync = (!rst) & (|ch_resync);

  genvar i;
  generate
  for(i=0; i<N; i=i+1) begin: rxsyncgen
    assign header[i] = theader[(i+1)*6-1:i*6];

    icet_sync #(SEQ_LEN) sync_inst (clk,rst,en, ch_resync[i],header[i][1], slip[i], ch_lock[i], ch_phase[i],ch_align[i], ch_pheadr[i],ch_scramsync[i], 
                                    ch_corr_coe[i], ch_dstate[i],ch_dcnt0[i],ch_dcnt1[i],ch_dsreg[i],ch_dcreg[i],ch_dxor[i]);

    icet_ascrambler deascramble_inst (clk,rst,ch_scramsync[i],1'b1,1'b0, tdat[(i+1)*PCW-1:i*PCW], sdat[i]);

    assign ch_fecen[i] = (locked & ch_align[i]) | ch_fecen_[i];
    always@(posedge clk) ch_fecen_[i] <= (~locked)? 0 : ch_fecen[i];

    delayreg #(1,51) ch_phasepipe (clk,rst, ch_phase[i],ch_phase_[i]); /* pipeline match delay through FEC Decode */

    wire [31:0] rxaux_;
    if(i==0) fec_decode fec_decode_inst0 (clk,(rst|~locked), ch_fecen[i],sdat[i], cval[i],cdat[i],rxaux, pkterr_rst,ch_terr_cnt[i],ch_cerr_cnt[i],ch_tpkt_cnt[i],rxdbgrst,fecdbg,fecdtrig);
    else fec_decode fec_decode_inst (clk,(rst|~locked), ch_fecen[i],sdat[i], cval[i],cdat[i],rxaux_, pkterr_rst,ch_terr_cnt[i],ch_cerr_cnt[i],ch_tpkt_cnt[i],,,);

    assign pkterr_tpkt[(i+1)*32-1:i*32] = ch_tpkt_cnt[i];
    assign pkterr_terr[(i+1)*32-1:i*32] = ch_terr_cnt[i];
    assign pkterr_cerr[(i+1)*32-1:i*32] = ch_cerr_cnt[i];

    assign iena[i] = cval[i] & ch_phase_[i];
//    fifoNxM #(PCW,PCW,0,6) tfifo (clk,rst, irdy[i],iena[i],cdat[i], clk,rst, ostat[i],oena,odat[(i+1)*PCW-1:i*PCW]);
    que16xNaf #(PCW,6) tfifo (clk,rst, iena[i],oena, cdat[i],odat[(i+1)*PCW-1:i*PCW], irdy[i],ostat[i]);
  end
  endgenerate


`ifdef PIC8DBG
  wire [511:0] rxdbgdata_;

  assign rxdbgtrig  = fecdtrig;
  assign rxdbgdata_ = fecdbg;
  //assign rxdbgtrig = (ch_cerr_cnt[0] != ch_terr_cnt[0]); //oval; 
/*
  assign rxdbgdata[(0+1)*64-1:0*64] = {en,2'd0,rst,ch_dstate[1],4'd0,ch_dstate[0],2'd0,ch_corr_coe[1],2'd0,ch_corr_coe[0],6'd0,ch_resync,6'd0,ch_phase,6'd0,iena,6'd0,ostat};
  assign rxdbgdata[(1+1)*64-1:1*64] = 64'h0123456789ABCDEF; 
  assign rxdbgdata[(2+1)*64-1:2*64] = sdat[0]; //64'h2222222233333333;
  assign rxdbgdata[(3+1)*64-1:3*64] = sdat[1]; //64'h4444444455555555;
  assign rxdbgdata[(4+1)*64-1:4*64] = {32'hDEADBEEF,31'd0,oval};
  assign rxdbgdata[(5+1)*64-1:5*64] = odat[PCW-1:0];
  assign rxdbgdata[(6+1)*64-1:6*64] = odat[2*PCW-1:PCW]; //64'h1023ABCD789A4567;
  assign rxdbgdata[(7+1)*64-1:7*64] = {en,3'd0,3'd0,rst,2'd0,slip,2'd0,ch_lock}; //64'hFFFFFFFFFFFFFFFF; 
*/
/*
  assign rxdbgdata_[(0+1)*64-1:0*64] = {en,2'd0,rst,ch_dstate[0],4'd0,ch_dstate[0],2'd0,ch_corr_coe[0],2'd0,ch_corr_coe[0],7'd0,ch_resync,7'd0,ch_phase,7'd0,iena,7'd0,ostat};
  assign rxdbgdata_[(1+1)*64-1:1*64] = 64'h0123456789ABCDEF; 
  assign rxdbgdata_[(2+1)*64-1:2*64] = sdat[0]; //64'h2222222233333333;
  assign rxdbgdata_[(3+1)*64-1:3*64] = {ch_cerr_cnt[0],ch_terr_cnt[0]}; //64'h4444444455555555;
  assign rxdbgdata_[(4+1)*64-1:4*64] = {32'hDEADBEEF,31'd0,oval};
  assign rxdbgdata_[(5+1)*64-1:5*64] = odat[PCW-1:0];
  assign rxdbgdata_[(6+1)*64-1:6*64] = 64'h1023ABCD789A4567;
  assign rxdbgdata_[(7+1)*64-1:7*64] = {en,3'd0,3'd0,rst,3'd0,slip,3'd0,ch_lock}; //64'hFFFFFFFFFFFFFFFF; 
*/
  delayreg #(512,64) dbg_delay (clk,rst,rxdbgdata_,rxdbgdata);

`else
  assign rxdbgtrig = 0;
  assign rxdbgdata = 0;
`endif

endmodule


module icet_pkthandle (iclk,irst, istat,ival,ibus, oclk,orst, ostat,oena,obus);
  parameter ICW = 64; /* Input Channel Width */
  parameter OCW = 64; /* Output Channel Width */
  parameter DIR = 1;  /* 1=output 0=input */

  localparam NR = (DIR>0)?8:1;
//  localparam BW = 4;
//  localparam IFLG = `LOOSE;
//  localparam OFLG = (DIR>0)? `DBLBUF : `LOOSE;

//  localparam NR = (DIR>0)? 0 : 1;
  localparam BW = 4;
  localparam IFLG = `LOOSE;
  localparam OFLG = (DIR>0)? 0 : `LOOSE;

  input iclk,irst,ival;
  input [ICW-1:0] ibus;
  input oclk,orst,oena;
  output istat,ostat;
  output [OCW-1:0] obus;

  fifoNxM #(ICW,OCW,NR,BW,IFLG,OFLG) tfifo (iclk,irst, istat,ival,ibus, oclk,orst, ostat,oena,obus);	// 2k x4 split implements double buffer
//  fifoNxM #(ICW,OCW,0,4) tfifo (iclk,irst, istat,ival,ibus, oclk,orst, ostat,oena,obus);	// 2k x4 split implements double buffer

endmodule

module icet_txsynclayer(clk,rst,rdy,idle, txdrdy,txden,txdin, txdout,txhout, txeinj,txaux);
  parameter LCW = 128;  /* Logical Channel Width */
  parameter PCW = 64;   /* Physical Channel Width */
  parameter SYNC_LEN = 33;
  parameter CHNUM = 0;

  localparam CWR = LCW/PCW; /* Channel Width Ratio */
  localparam SYNC_PKT = 1; //2048;
  localparam SYNC_SEQ = (SYNC_LEN==32)? 32'h1B3D2B88 : 33'h18A5C240D;

  input clk,rst;
  output rdy;
  input idle;
  input txdrdy;
  output txden;
  input [LCW-1:0] txdin;
  output [LCW-1:0] txdout;
  output [(6*CWR)-1:0] txhout;
  input [CWR-1:0] txeinj;
  input [31:0] txaux;

  genvar i;
  reg [3:0] state,nextstate;
  reg idle_i;
  reg rdy_i;
  reg txden_i;
  reg txdval,txdvalx;
  wire [31:0] ch_fec [CWR-1:0];
  wire [31:0] txhead [CWR-1:0];
  wire [PCW-1:0] txheadfec [CWR-1:0];
  wire [PCW-1:0] fec_txd [CWR-1:0];
  wire [PCW-1:0] txd [CWR-1:0];
  wire [PCW-1:0] txdd [CWR-1:0];
  reg  [PCW-1:0] txddd [CWR-1:0];
  wire [PCW-1:0] txdh [CWR-1:0];
  wire [PCW-1:0] txdhd [CWR-1:0];
  wire [PCW-1:0] txds [CWR-1:0];
  wire [PCW-1:0] scram_seq [CWR-1:0];
  wire sync_seq;
  reg [5:0] sync_cnt;
  reg [15:0] pkt_cnt;
  wire txhen,fec_clr,fec_clrp;
  reg [3:0] txhend;

  always@(posedge clk) begin
    if(rst || (sync_cnt==SYNC_LEN-1)) sync_cnt <= 0;
    else sync_cnt <= sync_cnt + 1;

    if(rst) pkt_cnt <= 0;
    else if(sync_cnt==SYNC_LEN-1) pkt_cnt <= pkt_cnt + 1;

    if(rst) txdval <= 0;
    else txdval <= txden;

    if(rst) txdvalx <= 0;
    else txdvalx <= txdval;
  end

  icet_syncgen #(SYNC_LEN,SYNC_SEQ) syncgen_inst (clk,rst,~rst,sync_seq);

  assign fec_clr = (sync_cnt==0)? 1 : 0;
  delayreg #(1,SYNC_LEN+2) clrdlypipe (clk,rst, fec_clr,fec_clrp);

  assign txden = (sync_cnt==31)? 0 : (sync_cnt==32)? txdrdy : txdval;
  assign txhen = (sync_cnt==32)? 1 : 0;
  always@(posedge clk) txhend <= (rst)? 0 : {txhend[2:0],txhen};

  generate
  for(i=0; i<CWR; i=i+1) begin: logical_chan_loop
`ifdef HYP_RIO_ONOF
    reg [31:0] txauxd; 
    always @(posedge clk) if (sync_cnt==31) txauxd <= txaux; 
    assign txhead[i] = txauxd;
`else
    assign txhead[i] = 32'hAFECBFEC;
`endif

    assign txd[i] = (txhen)? {txhead[i],32'h0} : (txdval)? txdin[((i+1)*PCW)-1:i*PCW] : 0;
    always@(posedge clk) txddd[i] <= txd[i];

    delayreg #(PCW,4) dlypipe (clk,rst, txd[i],txdd[i]);

    assign fec_txd[i] = {txddd[i][31:0],txd[i][63:32]};

    fec_gen fec_inst (clk,rst,fec_clr,fec_txd[i],ch_fec[i]);
    assign txheadfec[i] = {txhead[i],ch_fec[i]};

    assign txdh[i] = (txhend[3])? txheadfec[i] : txdd[i];

    delayreg #(PCW,SYNC_LEN-2) dlypipe2 (clk,rst, txdh[i],txdhd[i]);

    icet_ascrambler scramble (clk,rst,fec_clrp,1'b1,1'b0, txdhd[i],txds[i]);

    assign txdout[((i+1)*PCW)-1:i*PCW] = txds[i]^(txeinj[i]<<2);
  end
  endgenerate

  assign rdy = ~rst; //rdy_i;
  assign txhout = {CWR{(txdval | (txhen & txdvalx))? {4'b0,sync_seq,1'b1} : {4'b0,~sync_seq,1'b0}}};

  reg [63:0] txsync_dbg [4:0];
  reg txsync_dbgt;
  always@(posedge clk) begin
    if(rst) begin
      txsync_dbg[0] <= 0;
      txsync_dbg[1] <= 0;
      txsync_dbg[2] <= 0;
      txsync_dbg[3] <= 0;
      txsync_dbg[4] <= 0;
      txsync_dbgt   <= 0;
    end
    else begin
      txsync_dbg[0] <= txdh[1];
      txsync_dbg[1] <= txdh[0];
      txsync_dbg[2] <= fec_txd[0];
      txsync_dbg[3] <= {ch_fec[0],pkt_cnt,10'd0,sync_cnt};
      txsync_dbg[4] <= {4'hA,4'd0,        state,          3'd0,txdrdy,
                        4'hB,3'd0,txden,  3'd0,txhend[1], 3'd0,txdvalx,
                        4'hC,4'd0,        4'd0,           4'd0,
                        4'hD,3'd0,txdval, 3'd0,fec_clr,    3'd0,sync_seq};
      txsync_dbgt   <= txdrdy | txsync_dbgt;
    end
  end

endmodule



module delayreg (clk,rst, din,dout);
  parameter D = 64;
  parameter P = 32;

  input clk,rst;
  input [D-1:0] din;
  output [D-1:0] dout;

  genvar i;
  reg [D-1:0] pipe [P-1:0];

  generate
  for (i=0; i<P; i=i+1) begin: PIPE_LOOP
    always@(posedge clk) begin
      if(rst) begin
        pipe[i] <= 0;
      end
      else begin
        if(i==0) pipe[i] <= din;
        else pipe[i] <= pipe[i-1];
      end
    end
  end
  endgenerate
  
  assign dout = pipe[P-1];

endmodule

module icet_xscrambler(clk,rst,sync,en,by, din,dout, scram_seq);
  parameter ND = 64;
  parameter S  = 1; //1 to scramble, 0 to descramble

  input clk,rst,sync,en,by;
  input [ND-1:0] din;
  output [ND-1:0] dout;
  output [ND-1:0] scram_seq;

  integer i;
  reg [ND-1:0] scramdat;
  reg [ND-1:0] scramdat_reg;
  reg [ND-1:0] scram_seq_i;
  reg [57:0] scramble_state;
  reg [57:0] lfsr;
  reg fdbk_bit;

  always@(din,lfsr) begin
    scramble_state = lfsr;
    for(i=0; i<ND; i=i+1) begin
      fdbk_bit = scramble_state[38] ^ scramble_state[57];
      if(S==1)
        scramble_state = {scramble_state[56:0],fdbk_bit};
      else
        scramble_state = {scramble_state[56:0],scramble_state[57]};
      scramdat[i] = din[i] ^ fdbk_bit;
      scram_seq_i[i] = scramble_state[38] ^ scramble_state[57];
    end
  end

  assign scram_seq = scram_seq_i;

  always@(posedge clk) begin
    if(rst) begin
      lfsr <= 58'h155_5555_5555_5555;
      scramdat_reg <= 0;
    end
    else if(sync) begin
      lfsr <= 58'h155_5555_5555_5555;
      scramdat_reg <= scramdat;
    end
    else if(en) begin
      lfsr <= scramble_state;
      scramdat_reg <= scramdat;
    end
  end

  assign dout = (by)? din : scramdat_reg;
      
endmodule

module icet_ascrambler(clk,rst,sync,en,by, din,dout);

  input clk,rst,sync,en,by;
  input [63:0] din;
  output [63:0] dout;

  reg [57:0] lfsr_q,lfsr_c;
  reg [63:0] data_c;

  always @(*) begin
    lfsr_c[0] = lfsr_q[13] ^ lfsr_q[32] ^ lfsr_q[51] ^ lfsr_q[52];
    lfsr_c[1] = lfsr_q[14] ^ lfsr_q[33] ^ lfsr_q[52] ^ lfsr_q[53];
    lfsr_c[2] = lfsr_q[15] ^ lfsr_q[34] ^ lfsr_q[53] ^ lfsr_q[54];
    lfsr_c[3] = lfsr_q[16] ^ lfsr_q[35] ^ lfsr_q[54] ^ lfsr_q[55];
    lfsr_c[4] = lfsr_q[17] ^ lfsr_q[36] ^ lfsr_q[55] ^ lfsr_q[56];
    lfsr_c[5] = lfsr_q[18] ^ lfsr_q[37] ^ lfsr_q[56] ^ lfsr_q[57];
    lfsr_c[6] = lfsr_q[0] ^ lfsr_q[19] ^ lfsr_q[38] ^ lfsr_q[57];
    lfsr_c[7] = lfsr_q[1] ^ lfsr_q[20] ^ lfsr_q[39];
    lfsr_c[8] = lfsr_q[2] ^ lfsr_q[21] ^ lfsr_q[40];
    lfsr_c[9] = lfsr_q[3] ^ lfsr_q[22] ^ lfsr_q[41];
    lfsr_c[10] = lfsr_q[4] ^ lfsr_q[23] ^ lfsr_q[42];
    lfsr_c[11] = lfsr_q[5] ^ lfsr_q[24] ^ lfsr_q[43];
    lfsr_c[12] = lfsr_q[6] ^ lfsr_q[25] ^ lfsr_q[44];
    lfsr_c[13] = lfsr_q[7] ^ lfsr_q[26] ^ lfsr_q[45];
    lfsr_c[14] = lfsr_q[8] ^ lfsr_q[27] ^ lfsr_q[46];
    lfsr_c[15] = lfsr_q[9] ^ lfsr_q[28] ^ lfsr_q[47];
    lfsr_c[16] = lfsr_q[10] ^ lfsr_q[29] ^ lfsr_q[48];
    lfsr_c[17] = lfsr_q[11] ^ lfsr_q[30] ^ lfsr_q[49];
    lfsr_c[18] = lfsr_q[12] ^ lfsr_q[31] ^ lfsr_q[50];
    lfsr_c[19] = lfsr_q[13] ^ lfsr_q[32] ^ lfsr_q[51];
    lfsr_c[20] = lfsr_q[14] ^ lfsr_q[33] ^ lfsr_q[52];
    lfsr_c[21] = lfsr_q[15] ^ lfsr_q[34] ^ lfsr_q[53];
    lfsr_c[22] = lfsr_q[16] ^ lfsr_q[35] ^ lfsr_q[54];
    lfsr_c[23] = lfsr_q[17] ^ lfsr_q[36] ^ lfsr_q[55];
    lfsr_c[24] = lfsr_q[18] ^ lfsr_q[37] ^ lfsr_q[56];
    lfsr_c[25] = lfsr_q[19] ^ lfsr_q[38] ^ lfsr_q[57];
    lfsr_c[26] = lfsr_q[20] ^ lfsr_q[39];
    lfsr_c[27] = lfsr_q[21] ^ lfsr_q[40];
    lfsr_c[28] = lfsr_q[22] ^ lfsr_q[41];
    lfsr_c[29] = lfsr_q[23] ^ lfsr_q[42];
    lfsr_c[30] = lfsr_q[24] ^ lfsr_q[43];
    lfsr_c[31] = lfsr_q[25] ^ lfsr_q[44];
    lfsr_c[32] = lfsr_q[26] ^ lfsr_q[45];
    lfsr_c[33] = lfsr_q[27] ^ lfsr_q[46];
    lfsr_c[34] = lfsr_q[28] ^ lfsr_q[47];
    lfsr_c[35] = lfsr_q[29] ^ lfsr_q[48];
    lfsr_c[36] = lfsr_q[30] ^ lfsr_q[49];
    lfsr_c[37] = lfsr_q[31] ^ lfsr_q[50];
    lfsr_c[38] = lfsr_q[32] ^ lfsr_q[51];
    lfsr_c[39] = lfsr_q[13] ^ lfsr_q[32] ^ lfsr_q[33] ^ lfsr_q[51];
    lfsr_c[40] = lfsr_q[14] ^ lfsr_q[33] ^ lfsr_q[34] ^ lfsr_q[52];
    lfsr_c[41] = lfsr_q[15] ^ lfsr_q[34] ^ lfsr_q[35] ^ lfsr_q[53];
    lfsr_c[42] = lfsr_q[16] ^ lfsr_q[35] ^ lfsr_q[36] ^ lfsr_q[54];
    lfsr_c[43] = lfsr_q[17] ^ lfsr_q[36] ^ lfsr_q[37] ^ lfsr_q[55];
    lfsr_c[44] = lfsr_q[18] ^ lfsr_q[37] ^ lfsr_q[38] ^ lfsr_q[56];
    lfsr_c[45] = lfsr_q[0] ^ lfsr_q[19] ^ lfsr_q[38] ^ lfsr_q[39] ^ lfsr_q[57];
    lfsr_c[46] = lfsr_q[1] ^ lfsr_q[20] ^ lfsr_q[39] ^ lfsr_q[40];
    lfsr_c[47] = lfsr_q[2] ^ lfsr_q[21] ^ lfsr_q[40] ^ lfsr_q[41];
    lfsr_c[48] = lfsr_q[3] ^ lfsr_q[22] ^ lfsr_q[41] ^ lfsr_q[42];
    lfsr_c[49] = lfsr_q[4] ^ lfsr_q[23] ^ lfsr_q[42] ^ lfsr_q[43];
    lfsr_c[50] = lfsr_q[5] ^ lfsr_q[24] ^ lfsr_q[43] ^ lfsr_q[44];
    lfsr_c[51] = lfsr_q[6] ^ lfsr_q[25] ^ lfsr_q[44] ^ lfsr_q[45];
    lfsr_c[52] = lfsr_q[7] ^ lfsr_q[26] ^ lfsr_q[45] ^ lfsr_q[46];
    lfsr_c[53] = lfsr_q[8] ^ lfsr_q[27] ^ lfsr_q[46] ^ lfsr_q[47];
    lfsr_c[54] = lfsr_q[9] ^ lfsr_q[28] ^ lfsr_q[47] ^ lfsr_q[48];
    lfsr_c[55] = lfsr_q[10] ^ lfsr_q[29] ^ lfsr_q[48] ^ lfsr_q[49];
    lfsr_c[56] = lfsr_q[11] ^ lfsr_q[30] ^ lfsr_q[49] ^ lfsr_q[50];
    lfsr_c[57] = lfsr_q[12] ^ lfsr_q[31] ^ lfsr_q[50] ^ lfsr_q[51];

    data_c[0] = din[0] ^ lfsr_q[57];
    data_c[1] = din[1] ^ lfsr_q[56];
    data_c[2] = din[2] ^ lfsr_q[55];
    data_c[3] = din[3] ^ lfsr_q[54];
    data_c[4] = din[4] ^ lfsr_q[53];
    data_c[5] = din[5] ^ lfsr_q[52];
    data_c[6] = din[6] ^ lfsr_q[51];
    data_c[7] = din[7] ^ lfsr_q[50];
    data_c[8] = din[8] ^ lfsr_q[49];
    data_c[9] = din[9] ^ lfsr_q[48];
    data_c[10] = din[10] ^ lfsr_q[47];
    data_c[11] = din[11] ^ lfsr_q[46];
    data_c[12] = din[12] ^ lfsr_q[45];
    data_c[13] = din[13] ^ lfsr_q[44];
    data_c[14] = din[14] ^ lfsr_q[43];
    data_c[15] = din[15] ^ lfsr_q[42];
    data_c[16] = din[16] ^ lfsr_q[41];
    data_c[17] = din[17] ^ lfsr_q[40];
    data_c[18] = din[18] ^ lfsr_q[39];
    data_c[19] = din[19] ^ lfsr_q[38] ^ lfsr_q[57];
    data_c[20] = din[20] ^ lfsr_q[37] ^ lfsr_q[56];
    data_c[21] = din[21] ^ lfsr_q[36] ^ lfsr_q[55];
    data_c[22] = din[22] ^ lfsr_q[35] ^ lfsr_q[54];
    data_c[23] = din[23] ^ lfsr_q[34] ^ lfsr_q[53];
    data_c[24] = din[24] ^ lfsr_q[33] ^ lfsr_q[52];
    data_c[25] = din[25] ^ lfsr_q[32] ^ lfsr_q[51];
    data_c[26] = din[26] ^ lfsr_q[31] ^ lfsr_q[50];
    data_c[27] = din[27] ^ lfsr_q[30] ^ lfsr_q[49];
    data_c[28] = din[28] ^ lfsr_q[29] ^ lfsr_q[48];
    data_c[29] = din[29] ^ lfsr_q[28] ^ lfsr_q[47];
    data_c[30] = din[30] ^ lfsr_q[27] ^ lfsr_q[46];
    data_c[31] = din[31] ^ lfsr_q[26] ^ lfsr_q[45];
    data_c[32] = din[32] ^ lfsr_q[25] ^ lfsr_q[44];
    data_c[33] = din[33] ^ lfsr_q[24] ^ lfsr_q[43];
    data_c[34] = din[34] ^ lfsr_q[23] ^ lfsr_q[42];
    data_c[35] = din[35] ^ lfsr_q[22] ^ lfsr_q[41];
    data_c[36] = din[36] ^ lfsr_q[21] ^ lfsr_q[40];
    data_c[37] = din[37] ^ lfsr_q[20] ^ lfsr_q[39];
    data_c[38] = din[38] ^ lfsr_q[19] ^ lfsr_q[38] ^ lfsr_q[57];
    data_c[39] = din[39] ^ lfsr_q[18] ^ lfsr_q[37] ^ lfsr_q[56];
    data_c[40] = din[40] ^ lfsr_q[17] ^ lfsr_q[36] ^ lfsr_q[55];
    data_c[41] = din[41] ^ lfsr_q[16] ^ lfsr_q[35] ^ lfsr_q[54];
    data_c[42] = din[42] ^ lfsr_q[15] ^ lfsr_q[34] ^ lfsr_q[53];
    data_c[43] = din[43] ^ lfsr_q[14] ^ lfsr_q[33] ^ lfsr_q[52];
    data_c[44] = din[44] ^ lfsr_q[13] ^ lfsr_q[32] ^ lfsr_q[51];
    data_c[45] = din[45] ^ lfsr_q[12] ^ lfsr_q[31] ^ lfsr_q[50];
    data_c[46] = din[46] ^ lfsr_q[11] ^ lfsr_q[30] ^ lfsr_q[49];
    data_c[47] = din[47] ^ lfsr_q[10] ^ lfsr_q[29] ^ lfsr_q[48];
    data_c[48] = din[48] ^ lfsr_q[9] ^ lfsr_q[28] ^ lfsr_q[47];
    data_c[49] = din[49] ^ lfsr_q[8] ^ lfsr_q[27] ^ lfsr_q[46];
    data_c[50] = din[50] ^ lfsr_q[7] ^ lfsr_q[26] ^ lfsr_q[45];
    data_c[51] = din[51] ^ lfsr_q[6] ^ lfsr_q[25] ^ lfsr_q[44];
    data_c[52] = din[52] ^ lfsr_q[5] ^ lfsr_q[24] ^ lfsr_q[43];
    data_c[53] = din[53] ^ lfsr_q[4] ^ lfsr_q[23] ^ lfsr_q[42];
    data_c[54] = din[54] ^ lfsr_q[3] ^ lfsr_q[22] ^ lfsr_q[41];
    data_c[55] = din[55] ^ lfsr_q[2] ^ lfsr_q[21] ^ lfsr_q[40];
    data_c[56] = din[56] ^ lfsr_q[1] ^ lfsr_q[20] ^ lfsr_q[39];
    data_c[57] = din[57] ^ lfsr_q[0] ^ lfsr_q[19] ^ lfsr_q[38] ^ lfsr_q[57];
    data_c[58] = din[58] ^ lfsr_q[18] ^ lfsr_q[37] ^ lfsr_q[56] ^ lfsr_q[57];
    data_c[59] = din[59] ^ lfsr_q[17] ^ lfsr_q[36] ^ lfsr_q[55] ^ lfsr_q[56];
    data_c[60] = din[60] ^ lfsr_q[16] ^ lfsr_q[35] ^ lfsr_q[54] ^ lfsr_q[55];
    data_c[61] = din[61] ^ lfsr_q[15] ^ lfsr_q[34] ^ lfsr_q[53] ^ lfsr_q[54];
    data_c[62] = din[62] ^ lfsr_q[14] ^ lfsr_q[33] ^ lfsr_q[52] ^ lfsr_q[53];
    data_c[63] = din[63] ^ lfsr_q[13] ^ lfsr_q[32] ^ lfsr_q[51] ^ lfsr_q[52];
  end // always

  reg [63:0] dout_i;
  always @(posedge clk) begin
    if(rst) begin
      lfsr_q <= 58'h155_5555_5555_5555;
      dout_i <= 64'd0;
    end
    else begin
      lfsr_q <= sync ? 58'h155_5555_5555_5555 : en ? lfsr_c : lfsr_q;
      dout_i <= en ? data_c : dout_i;
    end
  end

  assign dout = dout_i;

endmodule
        
module icet_scrambler(clk,rst,en,by, din,dout);
  parameter ND = 64;
  parameter S  = 1; //1 to scramble, 0 to descramble

  input clk,rst,en,by;
  input [ND-1:0] din;
  output [ND-1:0] dout;

  integer i;
  reg [ND-1:0] scramdat;
  reg [ND-1:0] scramdat_reg;
  reg [57:0] scramble_state;
  reg [57:0] lfsr;
  reg fdbk_bit;

  always@(din,lfsr) begin
    scramble_state = lfsr;
    for(i=0; i<ND; i=i+1) begin
      fdbk_bit = din[i] ^ scramble_state[38] ^ scramble_state[57];
      if(S==1)
        scramble_state = {scramble_state[56:0],fdbk_bit};
      else
        scramble_state = {scramble_state[56:0],din[i]};
      scramdat[i] = fdbk_bit;
    end
  end

  always@(posedge clk) begin
    if(rst) begin
      lfsr <= 58'h155_5555_5555_5555;
      scramdat_reg <= 0;
    end
    else if(en) begin
      lfsr <= scramble_state;
      scramdat_reg <= scramdat;
    end
  end

  assign dout = (by)? din : scramdat_reg;
      
endmodule

module icet_synccorr32 (clk,rst, din, corr_coe,corr_phase, dcnt0,dcnt1,dsreg,dcreg,dxor,dir);
  parameter ND = 32;
  parameter SEQ = 32'h1B3D2B88;

  input clk,rst;
  input din;
  output [5:0] corr_coe;
  output corr_phase;
//Debugs...
  output [5:0] dcnt0;
  output [5:0] dcnt1;
  output [ND-1:0] dsreg;
  output [ND-1:0] dcreg;
  output [ND-1:0] dxor;
  input dir;

  wire [31:0] CORR_SEQ = SEQ;

  integer i;
  reg [31:0] sreg;
  reg [31:0] sreg_xor,sreg_xord,sreg_xnor;
  integer count,count0,count1;
  reg phase;

  always@(posedge clk) begin
    if(rst) begin
      sreg <= 32'h0;
      count <= 0;
      phase <= 0;
    end
    else begin
      sreg <= (dir)? {din,sreg[31:1]} : {sreg[30:0],din};
      sreg_xor <= CORR_SEQ ^ sreg;
      sreg_xnor <= ~sreg_xor;
      sreg_xord <= sreg_xor;

      if(count0[5] || &count0[4:1]) begin
        count <= count0;
        phase <= 0;
      end
      else if(count1[5] || &count1[4:1]) begin
        count <= count1;
        phase <= 1;
      end
      else count <= 0;
    end
  end

  always@(posedge clk) begin
    if(rst) begin
      count0 = 0;
      count1 = 0;
    end
    else begin
      count0 = 0;
      count1 = 0;
      for(i=0; i<ND; i=i+1) begin: corr_coe_loop
        //if(sreg_xord[i] == 1'b1) count0 = count0+1;
        //if(sreg_xnor[i] == 1'b1) count1 = count1+1;
        count0 = count0 + sreg_xord[i];
        count1 = count1 + sreg_xnor[i];
      end
    end
  end

  assign corr_coe = count;
  assign corr_phase = phase;

  assign dcnt0 = count0;
  assign dcnt1 = count1;
  assign dsreg = sreg;
  assign dcreg = CORR_SEQ;
  assign dxor  = sreg_xor;

endmodule

module icet_synccorr (clk,rst, din, corr_coe,corr_phase,palign, dcnt0,dcnt1,dsreg,dcreg,dxor,dir);
  parameter ND = 33;
  parameter SEQ = 33'h18A5C240D;

  input clk,rst;
  input din;
  output [5:0] corr_coe;
  output corr_phase;
  output palign;
//Debugs...
  output [5:0] dcnt0;
  output [5:0] dcnt1;
  output [ND-1:0] dsreg;
  output [ND-1:0] dcreg;
  output [ND-1:0] dxor;
  input dir;

  wire [ND-1:0] CORR_SEQ = SEQ;

  integer i;
  reg [ND-1:0] sreg;
  reg [ND-1:0] sreg_xor,sreg_xord,sreg_xnor;
  integer count,count0,count1;
  reg phase,ph_align;

  assign palign = ph_align;
  always@(posedge clk) begin
    if(rst) begin
      sreg  <= 0;
      count <= 0;
      phase <= 0;
      ph_align <= 0;
    end
    else begin
      sreg <= (dir)? {din,sreg[ND-1:1]} : {sreg[ND-2:0],din};
      sreg_xor <= CORR_SEQ ^ sreg;
      sreg_xnor <= ~sreg_xor;
      sreg_xord <= sreg_xor;

      if(count0[5] || &count0[4:1]) begin
        ph_align <= 1;
        count <= count0;
        phase <= 0;
      end
      else if(count1[5] || &count1[4:1]) begin
        ph_align <= 1;
        count <= count1;
        phase <= 1;
      end 
      else begin
        ph_align <= 0;
        count <= 0;
      end
    end
  end

  always@(posedge clk) begin
    if(rst) begin
      count0 = 0;
      count1 = 0;
    end
    else begin
      count0 = 0;
      count1 = 0;
      for(i=0; i<ND; i=i+1) begin: corr_coe_loop
        //if(sreg_xord[i] == 1'b1) count0 = count0+1;
        //if(sreg_xnor[i] == 1'b1) count1 = count1+1;
        count0 = count0 + sreg_xord[i];
        count1 = count1 + sreg_xnor[i];
      end
    end
  end

  assign corr_coe = count;
  assign corr_phase = phase;

  assign dcnt0 = count0;
  assign dcnt1 = count1;
  assign dsreg = sreg;
  assign dcreg = CORR_SEQ;
  assign dxor  = sreg_xor;

endmodule

module icet_sync (clk,rst,en,resync, din, slip, locked, phase,calign, header,scramsync, corr_coe, dstate,dcnt0,dcnt1,dsreg,dcreg,dxor);
  parameter N = 32;  // Number of samples to correlate over
  parameter T = 30;  //Correlation threshold
  parameter M = 3;   // Number of successive synchronized frames before lock declared
  parameter P = 2;   // Number of missed syncs before lock lost

  localparam SYNC_SEQ = (N==32)? 32'h1B3D2B88 : 33'h18A5C240D;

  input clk;
  input rst;
  input en;
  output resync;
  input din;

  output slip;
  output locked;
  output phase;
  output calign;
  output header;
  output scramsync;
  output [5:0] corr_coe;

  output [3:0] dstate;
  output [5:0] dcnt0,dcnt1;
  output [31:0] dsreg,dcreg,dxor;

  localparam WBS = 8; 
  localparam CCYC = 32;

  reg [3:0] sync_state;
  reg slip_i;
  reg [5:0] cntnc;
  reg [5:0] corr_cnt;
  reg [4:0] slip_wait;
  reg [5:0] sync_lock_cnt;
  reg [4:0] sync_accum_cnt;
  reg [4:0] err_accum_cnt;
  reg sync_locked;
  reg [7:0] slip_cnt;
  reg rst_rxchain;

  //wire [5:0] corr_coe;
  wire corr_phase;
  wire corr_align;
  //wire [5:0] dcnt0,dcnt1;
  //wire [31:0] dsreg,dcreg,dxor;

  assign resync = rst_rxchain;
  assign slip = slip_i;
  assign locked = sync_locked;
  assign phase = corr_phase;
  assign calign = corr_align;
  assign header = (corr_cnt==N-2);
  assign scramsync = (corr_cnt==N-2);
  assign dstate = sync_state;

  icet_synccorr #(N,SYNC_SEQ) corr_inst (clk,(rst|slip_i), din, corr_coe,corr_phase,corr_align, dcnt0,dcnt1,dsreg,dcreg,dxor,1'b0);

  //Sync State Machine...
  always@(posedge clk) begin
    if(rst) begin
      sync_locked <= 1'b0;
      sync_state <= 4'd0;
      slip_cnt <= 8'd0;
      rst_rxchain <= 1'b0;
      cntnc <= 6'd0;
      corr_cnt <= 6'd0;
      slip_i <= 1'b0;
      slip_wait <= 5'd0;
      sync_lock_cnt <= 6'd0;
      sync_accum_cnt <= 5'd0;
      err_accum_cnt <= 5'd0;
    end
    else begin
      case(sync_state) 
        4'd0: begin
          //Monitor Correlator output over N samples
          if(cntnc < N) begin
            if(corr_coe >= T) begin 
              //Found Correlation Sync, reset sync counter
              corr_cnt <= 0;
              cntnc <= 0;
              sync_state <= 4'd3;
            end
            else begin
              cntnc <= cntnc + 1;
            end
          end
          else if(slip_cnt == 8'd67) begin
            //Have slipped througha all positions, try resetting RX Chain...
            slip_cnt <= 8'd0;
            rst_rxchain <= 1'b1;
            sync_state <= 4'd5;
          end
          else begin
          //Need to engage bitslip
            slip_i <= 1'b1;
            cntnc <= 0;
            sync_state <= 4'd1;
          end
        end
        4'd1: begin
          //No header, slip gearbox
          slip_i <= 0;
          sync_state <= 4'd2;
        end
        4'd2: begin
          if(slip_wait < WBS)
            slip_wait <= slip_wait + 1'd1;
          else begin
            slip_wait <= 5'd0;
            sync_state <= 4'd0;
            slip_cnt <= slip_cnt + 1;
          end
        end
        4'd3: begin
          if(corr_cnt == N-1) begin
            corr_cnt <= 0;
            if(corr_coe >= T) begin
              if(sync_lock_cnt < M)
                sync_lock_cnt <= sync_lock_cnt + 6'd1;
              else begin
                sync_lock_cnt <= 6'd0;
                sync_state <= 4'd4;
                sync_locked <= 1'b1;
              end 
            end
            else begin
              sync_state <= 4'd0;
            end
          end
          else begin
            corr_cnt <= corr_cnt + 1;
          end
        end
        4'd4: begin
          if(corr_cnt == N-1) begin
            corr_cnt <= 0;
            if(corr_coe < T) begin
              if(err_accum_cnt > P) begin
                //Sync is lost, resync
                sync_locked <= 1'b0;
                err_accum_cnt <= 5'd0;
                sync_state <= 4'd0;
              end
              else begin
                err_accum_cnt <= err_accum_cnt + 1;
              end
            end
            else begin
              //Still In Sync, clear error count (looking for sucessive lost syncs)
              err_accum_cnt <= 0; 
            end
          end
          else begin
            corr_cnt <= corr_cnt + 1;
          end
        end
        4'd5: begin
          rst_rxchain <= 1'b0;
          if(corr_cnt == 6'd63) begin
             sync_state <= 4'd0;
             corr_cnt <= 6'd0;
          end
          else begin
             corr_cnt <= corr_cnt + 1;
          end
        end
        default:
          sync_state <= 4'd0;
      endcase
    end
  end

endmodule

`ifdef BOGUS
module bit_sync (clk_in,i_in,o_out);
  parameter INITIALIZE = 5'b00000;

  input  clk_in;
  input  i_in;
  output o_out;

  reg tmpreg;
  always @(posedge clk_in) begin
    tmpreg  <= i_in;
  end

  assign o_out = tmpreg;

endmodule
`endif

module bit_sync (clk_in,i_in,o_out);
  parameter INITIALIZE = 5'b00000;

  input  wire clk_in;
  input  wire i_in;
  output wire o_out;

  // Use 5 flip-flops as a single synchronizer, and tag each declaration with the appropriate synthesis attribute to
  // enable clustering. Their GSR default values are provided by the INITIALIZE parameter.

  (* ASYNC_REG = "TRUE" *) reg i_in_meta  = INITIALIZE[0];
  (* ASYNC_REG = "TRUE" *) reg i_in_sync1 = INITIALIZE[1];
  (* ASYNC_REG = "TRUE" *) reg i_in_sync2 = INITIALIZE[2];
  (* ASYNC_REG = "TRUE" *) reg i_in_sync3 = INITIALIZE[3];
                           reg i_in_out   = INITIALIZE[4];

  always @(posedge clk_in) begin
    i_in_meta  <= i_in;
    i_in_sync1 <= i_in_meta;
    i_in_sync2 <= i_in_sync1;
    i_in_sync3 <= i_in_sync2;
    i_in_out   <= i_in_sync3;
  end

  assign o_out = i_in_out;

endmodule


module ice_gth_t_wrap (
  input  wire [3:0] gthrxn_in,
  input  wire [3:0] gthrxp_in,
  output wire [3:0] gthtxn_out,
  output wire [3:0] gthtxp_out,
  input  wire [3:0] gtwiz_userclk_tx_reset_in,
  output wire [3:0] gtwiz_userclk_tx_srcclk_out,
  output wire [3:0] gtwiz_userclk_tx_usrclk_out,
  output wire [3:0] gtwiz_userclk_tx_usrclk2_out,
  output wire [3:0] gtwiz_userclk_tx_active_out,
  input  wire [3:0] gtwiz_userclk_rx_reset_in,
  output wire [3:0] gtwiz_userclk_rx_srcclk_out,
  output wire [3:0] gtwiz_userclk_rx_usrclk_out,
  output wire [3:0] gtwiz_userclk_rx_usrclk2_out,
  output wire [3:0] gtwiz_userclk_rx_active_out,
  input  wire [0:0] gtwiz_reset_clk_freerun_in,
  input  wire [3:0] gtwiz_reset_all_in,
  input  wire [3:0] gtwiz_reset_tx_pll_and_datapath_in,
  input  wire [3:0] gtwiz_reset_tx_datapath_in,
  input  wire [3:0] gtwiz_reset_rx_pll_and_datapath_in,
  input  wire [3:0] gtwiz_reset_rx_datapath_in,
  output wire [0:0] gtwiz_reset_rx_cdr_stable_out,
  output wire [3:0] gtwiz_reset_tx_done_out,
  output wire [3:0] gtwiz_reset_rx_done_out,
  input  wire [4*64-1:0] gtwiz_userdata_tx_in,
  output wire [4*64-1:0] gtwiz_userdata_rx_out,
  input  wire [0:0] gtrefclk00_in,
  output wire [0:0] qpll0outclk_out,
  output wire [0:0] qpll0outrefclk_out,
  output wire [0:0] qpll1outclk_out,
  output wire [0:0] qpll1outrefclk_out,
  input  wire [3:0] rxgearboxslip_in,
  input  wire [4*6-1:0] txheader_in,
  input  wire [4*7-1:0] txsequence_in,
  output wire [4*2-1:0] rxdatavalid_out,
  output wire [4*6-1:0] rxheader_out,
  output wire [4*2-1:0] rxheadervalid_out,
  output wire [3:0] rxpmaresetdone_out,
  output wire [3:0] rxprgdivresetdone_out,
  output wire [4*2-1:0] rxstartofseq_out,
  output wire [3:0] txpmaresetdone_out,
  output wire [3:0] txprgdivresetdone_out,
  input  wire [(4*9)-1:0] drpaddr_in,
  input  wire [(4*1)-1:0] drpclk_in,
  input  wire [(4*16)-1:0] drpdi_in,
  input  wire [(4*1)-1:0] drpen_in,
  input  wire [(4*1)-1:0] drprst_in,
  input  wire [(4*1)-1:0] drpwe_in,
  output wire [(4*16)-1:0] drpdo_out,
  output wire [(4*1)-1:0] drprdy_out,
  input  wire [3:0] rxlpmen_in,
  input  wire [3:0] txeq_en,
  input  wire [3:0] rxusrclk2_in,
  input  wire [3:0] rxusrclk_in
);


  // ===================================================================================================================
  // PARAMETERS AND FUNCTIONS
  // ===================================================================================================================

  // Declare and initialize local parameters and functions used for HDL generation
  localparam [191:0] P_CHANNEL_ENABLE = 192'b000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001;
  `include "icet_functions.v"
  localparam integer P_TX_MASTER_CH_PACKED_IDX = f_calc_pk_mc_idx(0);
  localparam integer P_RX_MASTER_CH_PACKED_IDX = f_calc_pk_mc_idx(0);


  // ===================================================================================================================
  // HELPER BLOCKS
  // ===================================================================================================================

  // Any helper blocks which the user chose to exclude from the core will appear below. In addition, some signal
  // assignments related to optionally-enabled ports may appear below.

  // Required declarations to connect COMMON reset and lock signals to reset controller helper block within the core
  wire [0:0] gtwiz_reset_qpll0lock_int;
  wire [3:0] gtwiz_reset_qpll0reset_int;

  // ===================================================================================================================
  // TRANSCEIVER COMMON BLOCK
  // ===================================================================================================================

  wire [0:0] gtrefclk00_int;
  wire [0:0] qpll0clk_int;
  wire [0:0] qpll0refclk_int;

  // When QPLL0 is used, the following assignments support the use of the transceiver COMMON block in the example design
  assign gtrefclk00_int = gtrefclk00_in;
  genvar gi_ch_to_cm0;
  generate for (gi_ch_to_cm0 = 0; gi_ch_to_cm0 < 1; gi_ch_to_cm0 = gi_ch_to_cm0 + 1) begin : gen_ch_to_cm0
    assign qpll0clk_int    [gi_ch_to_cm0] = qpll0outclk_out    [f_idx_cm(gi_ch_to_cm0)];
    assign qpll0refclk_int [gi_ch_to_cm0] = qpll0outrefclk_out [f_idx_cm(gi_ch_to_cm0)];
  end
  endgenerate

  wire [0:0] qpll0reset_int;
  wire [0:0] qpll0lock_int;
  assign qpll0reset_int            = |gtwiz_reset_qpll0reset_int;
  assign gtwiz_reset_qpll0lock_int = qpll0lock_int;

  wire [0:0] gtrefclk01_int;
  wire [0:0] qpll1clk_int;
  wire [0:0] qpll1refclk_int;
  wire [0:0] qpll1reset_int;
  wire [0:0] qpll1lock_int;

  // When QPLL1 is not used, the following assignments tie off QPLL1-related inputs as appropriate
  assign gtrefclk01_int  = {1{1'b0}};
  assign qpll1clk_int    = {1{1'b0}};
  assign qpll1refclk_int = {1{1'b0}};
  assign qpll1reset_int  = {1{1'b1}};

  localparam [47:0] P_COMMON_ENABLE = f_pop_cm_en(0);

  // The following HDL generate loop iterates across each possible transceiver quad, instantiating only the enabled
  // transceiver COMMON blocks, each within a configuration-specific parameterization wrapper
  genvar cm;
  generate for (cm = 0; cm < 48; cm = cm + 1) begin : gen_common_container
    if (P_COMMON_ENABLE[cm] == 1'b1) begin : gen_enabled_common

      ice_gth_t_gthe3_common_wrapper gthe3_common_wrapper_inst (
        .GTHE3_COMMON_BGBYPASSB         (1'b1),
        .GTHE3_COMMON_BGMONITORENB      (1'b1),
        .GTHE3_COMMON_BGPDB             (1'b1),
        .GTHE3_COMMON_BGRCALOVRD        (5'b11111),
        .GTHE3_COMMON_BGRCALOVRDENB     (1'b1),
        .GTHE3_COMMON_DRPADDR           (9'b000000000),
        .GTHE3_COMMON_DRPCLK            (1'b0),
        .GTHE3_COMMON_DRPDI             (16'b0000000000000000),
        .GTHE3_COMMON_DRPEN             (1'b0),
        .GTHE3_COMMON_DRPWE             (1'b0),
        .GTHE3_COMMON_GTGREFCLK0        (1'b0),
        .GTHE3_COMMON_GTGREFCLK1        (1'b0),
        .GTHE3_COMMON_GTNORTHREFCLK00   (1'b0),
        .GTHE3_COMMON_GTNORTHREFCLK01   (1'b0),
        .GTHE3_COMMON_GTNORTHREFCLK10   (1'b0),
        .GTHE3_COMMON_GTNORTHREFCLK11   (1'b0),
        .GTHE3_COMMON_GTREFCLK00        (gtrefclk00_int [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_GTREFCLK01        (gtrefclk01_int [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_GTREFCLK10        (1'b0),
        .GTHE3_COMMON_GTREFCLK11        (1'b0),
        .GTHE3_COMMON_GTSOUTHREFCLK00   (1'b0),
        .GTHE3_COMMON_GTSOUTHREFCLK01   (1'b0),
        .GTHE3_COMMON_GTSOUTHREFCLK10   (1'b0),
        .GTHE3_COMMON_GTSOUTHREFCLK11   (1'b0),
        .GTHE3_COMMON_PMARSVD0          (8'b00000000),
        .GTHE3_COMMON_PMARSVD1          (8'b00000000),
        .GTHE3_COMMON_QPLL0CLKRSVD0     (1'b0),
        .GTHE3_COMMON_QPLL0CLKRSVD1     (1'b0),
        .GTHE3_COMMON_QPLL0LOCKDETCLK   (1'b0),
        .GTHE3_COMMON_QPLL0LOCKEN       (1'b1),
        .GTHE3_COMMON_QPLL0PD           (1'b0),
        .GTHE3_COMMON_QPLL0REFCLKSEL    (3'b001),
        .GTHE3_COMMON_QPLL0RESET        (qpll0reset_int [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_QPLL1CLKRSVD0     (1'b0),
        .GTHE3_COMMON_QPLL1CLKRSVD1     (1'b0),
        .GTHE3_COMMON_QPLL1LOCKDETCLK   (1'b0),
        .GTHE3_COMMON_QPLL1LOCKEN       (1'b0),
        .GTHE3_COMMON_QPLL1PD           (1'b1),
        .GTHE3_COMMON_QPLL1REFCLKSEL    (3'b001),
        .GTHE3_COMMON_QPLL1RESET        (qpll1reset_int [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_QPLLRSVD1         (8'b00000000),
        .GTHE3_COMMON_QPLLRSVD2         (5'b00000),
        .GTHE3_COMMON_QPLLRSVD3         (5'b00000),
        .GTHE3_COMMON_QPLLRSVD4         (8'b00000000),
        .GTHE3_COMMON_RCALENB           (1'b1),
        .GTHE3_COMMON_DRPDO             (),
        .GTHE3_COMMON_DRPRDY            (),
        .GTHE3_COMMON_PMARSVDOUT0       (),
        .GTHE3_COMMON_PMARSVDOUT1       (),
        .GTHE3_COMMON_QPLL0FBCLKLOST    (),
        .GTHE3_COMMON_QPLL0LOCK         (qpll0lock_int      [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_QPLL0OUTCLK       (qpll0outclk_out    [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_QPLL0OUTREFCLK    (qpll0outrefclk_out [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_QPLL0REFCLKLOST   (),
        .GTHE3_COMMON_QPLL1FBCLKLOST    (),
        .GTHE3_COMMON_QPLL1LOCK         (qpll1lock_int      [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_QPLL1OUTCLK       (qpll1outclk_out    [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_QPLL1OUTREFCLK    (qpll1outrefclk_out [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
        .GTHE3_COMMON_QPLL1REFCLKLOST   (),
        .GTHE3_COMMON_QPLLDMONITOR0     (),
        .GTHE3_COMMON_QPLLDMONITOR1     (),
        .GTHE3_COMMON_REFCLKOUTMONITOR0 (),
        .GTHE3_COMMON_REFCLKOUTMONITOR1 (),
        .GTHE3_COMMON_RXRECCLK0_SEL     (),
        .GTHE3_COMMON_RXRECCLK1_SEL     ()
      );

    end
  end
  endgenerate


  // ===================================================================================================================
  // CORE INSTANCE
  // ===================================================================================================================
genvar i;
generate
for (i=0; i<4; i=i+1) begin: CHAN_GEN_LOOP
  // Instantiate the core, mapping its enabled ports to example design ports and helper blocks as appropriate
  ice_gth_t ice_gth_t_inst (
   .gthrxn_in                               (gthrxn_in[i]),
   .gthrxp_in                               (gthrxp_in[i]),
   .gthtxn_out                              (gthtxn_out[i]),
   .gthtxp_out                              (gthtxp_out[i]),
   .gtwiz_userclk_tx_reset_in               (gtwiz_userclk_tx_reset_in[i]),
   .gtwiz_userclk_tx_srcclk_out             (gtwiz_userclk_tx_srcclk_out[i]),
   .gtwiz_userclk_tx_usrclk_out             (gtwiz_userclk_tx_usrclk_out[i]),
   .gtwiz_userclk_tx_usrclk2_out            (gtwiz_userclk_tx_usrclk2_out[i]),
   .gtwiz_userclk_tx_active_out             (gtwiz_userclk_tx_active_out[i]),
   .gtwiz_userclk_rx_reset_in               (gtwiz_userclk_rx_reset_in[i]),
   .gtwiz_userclk_rx_srcclk_out             (gtwiz_userclk_rx_srcclk_out[i]),
   .gtwiz_userclk_rx_usrclk_out             (gtwiz_userclk_rx_usrclk_out[i]),
   .gtwiz_userclk_rx_usrclk2_out            (gtwiz_userclk_rx_usrclk2_out[i]),
   .gtwiz_userclk_rx_active_out             (gtwiz_userclk_rx_active_out[i]),
   .gtwiz_reset_clk_freerun_in              (gtwiz_reset_clk_freerun_in),
   .gtwiz_reset_all_in                      (gtwiz_reset_all_in[i]),
   .gtwiz_reset_tx_pll_and_datapath_in      (gtwiz_reset_tx_pll_and_datapath_in[i]),
   .gtwiz_reset_tx_datapath_in              (gtwiz_reset_tx_datapath_in[i]),
   .gtwiz_reset_rx_pll_and_datapath_in      (gtwiz_reset_rx_pll_and_datapath_in[i]),
   .gtwiz_reset_rx_datapath_in              (gtwiz_reset_rx_datapath_in[i]),
   .gtwiz_reset_qpll0lock_in                (gtwiz_reset_qpll0lock_int),
   .gtwiz_reset_rx_cdr_stable_out           (gtwiz_reset_rx_cdr_stable_out[i]),
   .gtwiz_reset_tx_done_out                 (gtwiz_reset_tx_done_out[i]),
   .gtwiz_reset_rx_done_out                 (gtwiz_reset_rx_done_out[i]),
   .gtwiz_reset_qpll0reset_out              (gtwiz_reset_qpll0reset_int[i]),
   .gtwiz_userdata_tx_in                    (gtwiz_userdata_tx_in[(i+1)*64-1:i*64]),
   .gtwiz_userdata_rx_out                   (gtwiz_userdata_rx_out[(i+1)*64-1:i*64]),
   .qpll0clk_in                             (qpll0clk_int),
   .qpll0refclk_in                          (qpll0refclk_int),
   .qpll1clk_in                             (qpll1clk_int),
   .qpll1refclk_in                          (qpll1refclk_int),
   .rxgearboxslip_in                        (rxgearboxslip_in[i]),
   .txheader_in                             (txheader_in[(i+1)*6-1:i*6]),
   .txsequence_in                           (txsequence_in[(i+1)*7-1:i*7]),
   .rxdatavalid_out                         (rxdatavalid_out[(i+1)*2-1:i*2]),
   .rxheader_out                            (rxheader_out[(i+1)*6-1:i*6]),
   .rxheadervalid_out                       (rxheadervalid_out[(i+1)*2-1:i*2]),
   .rxpmaresetdone_out                      (rxpmaresetdone_out[i]),
   .rxprgdivresetdone_out                   (rxprgdivresetdone_out[i]),
   .rxstartofseq_out                        (rxstartofseq_out[i]),
   .txpmaresetdone_out                      (txpmaresetdone_out[i]),
   .txprgdivresetdone_out                   (txprgdivresetdone_out[i]),
   .drpaddr_in                              (drpaddr_in[(i+1)*9-1:i*9]),
   .drpclk_in                               (drpclk_in[i]),
   .drpdi_in                                (drpdi_in[(i+1)*16-1:i*16]),
   .drpen_in                                (drpen_in[i]),
   .drprst_in                               (drprst_in[i]),
   .drpwe_in                                (drpwe_in[i]),
   .drpdo_out                               (drpdo_out[(i+1)*16-1:i*16]),
   .drprdy_out                              (drprdy_out[i]),
   .rxlpmen_in                              (rxlpmen_in[i]),
   .txeq_en                                 (txeq_en[i]),
   .rxusrclk2_in                            (rxusrclk2_in[i]),
   .rxusrclk_in                             (rxusrclk_in[i])
);
end
endgenerate

endmodule

module ice_gth_t_gthe3_common_wrapper (
  input                                   [0:0] GTHE3_COMMON_BGBYPASSB,
  input                                   [0:0] GTHE3_COMMON_BGMONITORENB,
  input                                   [0:0] GTHE3_COMMON_BGPDB,
  input                                   [4:0] GTHE3_COMMON_BGRCALOVRD,
  input                                   [0:0] GTHE3_COMMON_BGRCALOVRDENB,
  input                                   [8:0] GTHE3_COMMON_DRPADDR,
  input                                   [0:0] GTHE3_COMMON_DRPCLK,
  input                                  [15:0] GTHE3_COMMON_DRPDI,
  input                                   [0:0] GTHE3_COMMON_DRPEN,
  input                                   [0:0] GTHE3_COMMON_DRPWE,
  input                                   [0:0] GTHE3_COMMON_GTGREFCLK0,
  input                                   [0:0] GTHE3_COMMON_GTGREFCLK1,
  input                                   [0:0] GTHE3_COMMON_GTNORTHREFCLK00,
  input                                   [0:0] GTHE3_COMMON_GTNORTHREFCLK01,
  input                                   [0:0] GTHE3_COMMON_GTNORTHREFCLK10,
  input                                   [0:0] GTHE3_COMMON_GTNORTHREFCLK11,
  input                                   [0:0] GTHE3_COMMON_GTREFCLK00,
  input                                   [0:0] GTHE3_COMMON_GTREFCLK01,
  input                                   [0:0] GTHE3_COMMON_GTREFCLK10,
  input                                   [0:0] GTHE3_COMMON_GTREFCLK11,
  input                                   [0:0] GTHE3_COMMON_GTSOUTHREFCLK00,
  input                                   [0:0] GTHE3_COMMON_GTSOUTHREFCLK01,
  input                                   [0:0] GTHE3_COMMON_GTSOUTHREFCLK10,
  input                                   [0:0] GTHE3_COMMON_GTSOUTHREFCLK11,
  input                                   [7:0] GTHE3_COMMON_PMARSVD0,
  input                                   [7:0] GTHE3_COMMON_PMARSVD1,
  input                                   [0:0] GTHE3_COMMON_QPLL0CLKRSVD0,
  input                                   [0:0] GTHE3_COMMON_QPLL0CLKRSVD1,
  input                                   [0:0] GTHE3_COMMON_QPLL0LOCKDETCLK,
  input                                   [0:0] GTHE3_COMMON_QPLL0LOCKEN,
  input                                   [0:0] GTHE3_COMMON_QPLL0PD,
  input                                   [2:0] GTHE3_COMMON_QPLL0REFCLKSEL,
  input                                   [0:0] GTHE3_COMMON_QPLL0RESET,
  input                                   [0:0] GTHE3_COMMON_QPLL1CLKRSVD0,
  input                                   [0:0] GTHE3_COMMON_QPLL1CLKRSVD1,
  input                                   [0:0] GTHE3_COMMON_QPLL1LOCKDETCLK,
  input                                   [0:0] GTHE3_COMMON_QPLL1LOCKEN,
  input                                   [0:0] GTHE3_COMMON_QPLL1PD,
  input                                   [2:0] GTHE3_COMMON_QPLL1REFCLKSEL,
  input                                   [0:0] GTHE3_COMMON_QPLL1RESET,
  input                                   [7:0] GTHE3_COMMON_QPLLRSVD1,
  input                                   [4:0] GTHE3_COMMON_QPLLRSVD2,
  input                                   [4:0] GTHE3_COMMON_QPLLRSVD3,
  input                                   [7:0] GTHE3_COMMON_QPLLRSVD4,
  input                                   [0:0] GTHE3_COMMON_RCALENB,

  output                                 [15:0] GTHE3_COMMON_DRPDO,
  output                                  [0:0] GTHE3_COMMON_DRPRDY,
  output                                  [7:0] GTHE3_COMMON_PMARSVDOUT0,
  output                                  [7:0] GTHE3_COMMON_PMARSVDOUT1,
  output                                  [0:0] GTHE3_COMMON_QPLL0FBCLKLOST,
  output                                  [0:0] GTHE3_COMMON_QPLL0LOCK,
  output                                  [0:0] GTHE3_COMMON_QPLL0OUTCLK,
  output                                  [0:0] GTHE3_COMMON_QPLL0OUTREFCLK,
  output                                  [0:0] GTHE3_COMMON_QPLL0REFCLKLOST,
  output                                  [0:0] GTHE3_COMMON_QPLL1FBCLKLOST,
  output                                  [0:0] GTHE3_COMMON_QPLL1LOCK,
  output                                  [0:0] GTHE3_COMMON_QPLL1OUTCLK,
  output                                  [0:0] GTHE3_COMMON_QPLL1OUTREFCLK,
  output                                  [0:0] GTHE3_COMMON_QPLL1REFCLKLOST,
  output                                  [7:0] GTHE3_COMMON_QPLLDMONITOR0,
  output                                  [7:0] GTHE3_COMMON_QPLLDMONITOR1,
  output                                  [0:0] GTHE3_COMMON_REFCLKOUTMONITOR0,
  output                                  [0:0] GTHE3_COMMON_REFCLKOUTMONITOR1,
  output                                  [1:0] GTHE3_COMMON_RXRECCLK0_SEL,
  output                                  [1:0] GTHE3_COMMON_RXRECCLK1_SEL
);



gtwizard_ultrascale_v1_6_2_gthe3_common #(
  .GTHE3_COMMON_BGBYPASSB_TIE_EN                (1'b1),
  .GTHE3_COMMON_BGBYPASSB_VAL                   (1'b1),
  .GTHE3_COMMON_BGMONITORENB_TIE_EN             (1'b1),
  .GTHE3_COMMON_BGMONITORENB_VAL                (1'b1),
  .GTHE3_COMMON_BGPDB_TIE_EN                    (1'b1),
  .GTHE3_COMMON_BGPDB_VAL                       (1'b1),
  .GTHE3_COMMON_BGRCALOVRDENB_TIE_EN            (1'b1),
  .GTHE3_COMMON_BGRCALOVRDENB_VAL               (1'b1),
  .GTHE3_COMMON_BGRCALOVRD_TIE_EN               (1'b1),
  .GTHE3_COMMON_BGRCALOVRD_VAL                  (5'b11111),
  .GTHE3_COMMON_BIAS_CFG0                       (16'b0000000000000000),
  .GTHE3_COMMON_BIAS_CFG1                       (16'b0000000000000000),
  .GTHE3_COMMON_BIAS_CFG2                       (16'b0000000000000000),
  .GTHE3_COMMON_BIAS_CFG3                       (16'b0000000001000000),
  .GTHE3_COMMON_BIAS_CFG4                       (16'b0000000000000000),
  .GTHE3_COMMON_BIAS_CFG_RSVD                   (10'b0000000000),
  .GTHE3_COMMON_COMMON_CFG0                     (16'b0000000000000000),
  .GTHE3_COMMON_COMMON_CFG1                     (16'b0000000000000000),
  .GTHE3_COMMON_DRPADDR_TIE_EN                  (1'b0),
  .GTHE3_COMMON_DRPADDR_VAL                     (9'b000000000),
  .GTHE3_COMMON_DRPCLK_TIE_EN                   (1'b0),
  .GTHE3_COMMON_DRPCLK_VAL                      (1'b0),
  .GTHE3_COMMON_DRPDI_TIE_EN                    (1'b0),
  .GTHE3_COMMON_DRPDI_VAL                       (16'b0000000000000000),
  .GTHE3_COMMON_DRPEN_TIE_EN                    (1'b0),
  .GTHE3_COMMON_DRPEN_VAL                       (1'b0),
  .GTHE3_COMMON_DRPWE_TIE_EN                    (1'b0),
  .GTHE3_COMMON_DRPWE_VAL                       (1'b0),
  .GTHE3_COMMON_GTGREFCLK0_TIE_EN               (1'b0),
  .GTHE3_COMMON_GTGREFCLK0_VAL                  (1'b0),
  .GTHE3_COMMON_GTGREFCLK1_TIE_EN               (1'b0),
  .GTHE3_COMMON_GTGREFCLK1_VAL                  (1'b0),
  .GTHE3_COMMON_GTNORTHREFCLK00_TIE_EN          (1'b0),
  .GTHE3_COMMON_GTNORTHREFCLK00_VAL             (1'b0),
  .GTHE3_COMMON_GTNORTHREFCLK01_TIE_EN          (1'b0),
  .GTHE3_COMMON_GTNORTHREFCLK01_VAL             (1'b0),
  .GTHE3_COMMON_GTNORTHREFCLK10_TIE_EN          (1'b0),
  .GTHE3_COMMON_GTNORTHREFCLK10_VAL             (1'b0),
  .GTHE3_COMMON_GTNORTHREFCLK11_TIE_EN          (1'b0),
  .GTHE3_COMMON_GTNORTHREFCLK11_VAL             (1'b0),
  .GTHE3_COMMON_GTREFCLK00_TIE_EN               (1'b0),
  .GTHE3_COMMON_GTREFCLK00_VAL                  (1'b0),
  .GTHE3_COMMON_GTREFCLK01_TIE_EN               (1'b0),
  .GTHE3_COMMON_GTREFCLK01_VAL                  (1'b0),
  .GTHE3_COMMON_GTREFCLK10_TIE_EN               (1'b0),
  .GTHE3_COMMON_GTREFCLK10_VAL                  (1'b0),
  .GTHE3_COMMON_GTREFCLK11_TIE_EN               (1'b0),
  .GTHE3_COMMON_GTREFCLK11_VAL                  (1'b0),
  .GTHE3_COMMON_GTSOUTHREFCLK00_TIE_EN          (1'b0),
  .GTHE3_COMMON_GTSOUTHREFCLK00_VAL             (1'b0),
  .GTHE3_COMMON_GTSOUTHREFCLK01_TIE_EN          (1'b0),
  .GTHE3_COMMON_GTSOUTHREFCLK01_VAL             (1'b0),
  .GTHE3_COMMON_GTSOUTHREFCLK10_TIE_EN          (1'b0),
  .GTHE3_COMMON_GTSOUTHREFCLK10_VAL             (1'b0),
  .GTHE3_COMMON_GTSOUTHREFCLK11_TIE_EN          (1'b0),
  .GTHE3_COMMON_GTSOUTHREFCLK11_VAL             (1'b0),
  .GTHE3_COMMON_PMARSVD0_TIE_EN                 (1'b1),
  .GTHE3_COMMON_PMARSVD0_VAL                    (8'b00000000),
  .GTHE3_COMMON_PMARSVD1_TIE_EN                 (1'b1),
  .GTHE3_COMMON_PMARSVD1_VAL                    (8'b00000000),
  .GTHE3_COMMON_POR_CFG                         (16'b0000000000000100),
  .GTHE3_COMMON_QPLL0CLKRSVD0_TIE_EN            (1'b0),
  .GTHE3_COMMON_QPLL0CLKRSVD0_VAL               (1'b0),
  .GTHE3_COMMON_QPLL0CLKRSVD1_TIE_EN            (1'b0),
  .GTHE3_COMMON_QPLL0CLKRSVD1_VAL               (1'b0),
  .GTHE3_COMMON_QPLL0LOCKDETCLK_TIE_EN          (1'b0),
  .GTHE3_COMMON_QPLL0LOCKDETCLK_VAL             (1'b0),
  .GTHE3_COMMON_QPLL0LOCKEN_TIE_EN              (1'b0),
  .GTHE3_COMMON_QPLL0LOCKEN_VAL                 (1'b1),
  .GTHE3_COMMON_QPLL0PD_TIE_EN                  (1'b0),
  .GTHE3_COMMON_QPLL0PD_VAL                     (1'b0),
  .GTHE3_COMMON_QPLL0REFCLKSEL_TIE_EN           (1'b0),
  .GTHE3_COMMON_QPLL0REFCLKSEL_VAL              (3'b001),
  .GTHE3_COMMON_QPLL0RESET_TIE_EN               (1'b0),
  .GTHE3_COMMON_QPLL0RESET_VAL                  (1'b0),
  .GTHE3_COMMON_QPLL0_CFG0                      (16'b0011001000011100),
  .GTHE3_COMMON_QPLL0_CFG1                      (16'b0001000000011000),
  .GTHE3_COMMON_QPLL0_CFG1_G3                   (16'b0001000000011000),
  .GTHE3_COMMON_QPLL0_CFG2                      (16'b0000000001001000),
  .GTHE3_COMMON_QPLL0_CFG2_G3                   (16'b0000000001001000),
  .GTHE3_COMMON_QPLL0_CFG3                      (16'b0000000100100000),
  .GTHE3_COMMON_QPLL0_CFG4                      (16'b0000000000001001),
  .GTHE3_COMMON_QPLL0_CP                        (10'b0111111111),
  .GTHE3_COMMON_QPLL0_CP_G3                     (10'b1111111111),
  .GTHE3_COMMON_QPLL0_FBDIV                     (`RIO_QPLL_DIV),
  .GTHE3_COMMON_QPLL0_FBDIV_G3                  (`RIO_QPLL_DIV),
  .GTHE3_COMMON_QPLL0_INIT_CFG0                 (16'b0000001010110010),
  .GTHE3_COMMON_QPLL0_INIT_CFG1                 (8'b00000000),
  .GTHE3_COMMON_QPLL0_LOCK_CFG                  (16'b0010000111101000),
  .GTHE3_COMMON_QPLL0_LOCK_CFG_G3               (16'b0010000111101000),
  .GTHE3_COMMON_QPLL0_LPF                       (10'b1111111100),
  .GTHE3_COMMON_QPLL0_LPF_G3                    (10'b0000010101),
  .GTHE3_COMMON_QPLL0_REFCLK_DIV                (1),
  .GTHE3_COMMON_QPLL0_SDM_CFG0                  (16'b0000000000000000),
  .GTHE3_COMMON_QPLL0_SDM_CFG1                  (16'b0000000000000000),
  .GTHE3_COMMON_QPLL0_SDM_CFG2                  (16'b0000000000000000),
  .GTHE3_COMMON_QPLL1CLKRSVD0_TIE_EN            (1'b0),
  .GTHE3_COMMON_QPLL1CLKRSVD0_VAL               (1'b0),
  .GTHE3_COMMON_QPLL1CLKRSVD1_TIE_EN            (1'b0),
  .GTHE3_COMMON_QPLL1CLKRSVD1_VAL               (1'b0),
  .GTHE3_COMMON_QPLL1LOCKDETCLK_TIE_EN          (1'b0),
  .GTHE3_COMMON_QPLL1LOCKDETCLK_VAL             (1'b0),
  .GTHE3_COMMON_QPLL1LOCKEN_TIE_EN              (1'b0),
  .GTHE3_COMMON_QPLL1LOCKEN_VAL                 (1'b0),
  .GTHE3_COMMON_QPLL1PD_TIE_EN                  (1'b0),
  .GTHE3_COMMON_QPLL1PD_VAL                     (1'b1),
  .GTHE3_COMMON_QPLL1REFCLKSEL_TIE_EN           (1'b0),
  .GTHE3_COMMON_QPLL1REFCLKSEL_VAL              (3'b001),
  .GTHE3_COMMON_QPLL1RESET_TIE_EN               (1'b0),
  .GTHE3_COMMON_QPLL1RESET_VAL                  (1'b1),
  .GTHE3_COMMON_QPLL1_CFG0                      (16'b0011001000011100),
  .GTHE3_COMMON_QPLL1_CFG1                      (16'b0001000000011000),
  .GTHE3_COMMON_QPLL1_CFG1_G3                   (16'b0001000000011000),
  .GTHE3_COMMON_QPLL1_CFG2                      (16'b0000000001000000),
  .GTHE3_COMMON_QPLL1_CFG2_G3                   (16'b0000000001000000),
  .GTHE3_COMMON_QPLL1_CFG3                      (16'b0000000100100000),
  .GTHE3_COMMON_QPLL1_CFG4                      (16'b0000000000001001),
  .GTHE3_COMMON_QPLL1_CP                        (10'b0001111111),
  .GTHE3_COMMON_QPLL1_CP_G3                     (10'b1111111111),
  .GTHE3_COMMON_QPLL1_FBDIV                     (66),
  .GTHE3_COMMON_QPLL1_FBDIV_G3                  (80),
  .GTHE3_COMMON_QPLL1_INIT_CFG0                 (16'b0000001010110010),
  .GTHE3_COMMON_QPLL1_INIT_CFG1                 (8'b00000000),
  .GTHE3_COMMON_QPLL1_LOCK_CFG                  (16'b0010000111101000),
  .GTHE3_COMMON_QPLL1_LOCK_CFG_G3               (16'b0010000111101000),
  .GTHE3_COMMON_QPLL1_LPF                       (10'b1111111100),
  .GTHE3_COMMON_QPLL1_LPF_G3                    (10'b0000010101),
  .GTHE3_COMMON_QPLL1_REFCLK_DIV                (1),
  .GTHE3_COMMON_QPLL1_SDM_CFG0                  (16'b0000000000000000),
  .GTHE3_COMMON_QPLL1_SDM_CFG1                  (16'b0000000000000000),
  .GTHE3_COMMON_QPLL1_SDM_CFG2                  (16'b0000000000000000),
  .GTHE3_COMMON_QPLLRSVD1_TIE_EN                (1'b0),
  .GTHE3_COMMON_QPLLRSVD1_VAL                   (8'b00000000),
  .GTHE3_COMMON_QPLLRSVD2_TIE_EN                (1'b0),
  .GTHE3_COMMON_QPLLRSVD2_VAL                   (5'b00000),
  .GTHE3_COMMON_QPLLRSVD3_TIE_EN                (1'b0),
  .GTHE3_COMMON_QPLLRSVD3_VAL                   (5'b00000),
  .GTHE3_COMMON_QPLLRSVD4_TIE_EN                (1'b0),
  .GTHE3_COMMON_QPLLRSVD4_VAL                   (8'b00000000),
  .GTHE3_COMMON_RCALENB_TIE_EN                  (1'b1),
  .GTHE3_COMMON_RCALENB_VAL                     (1'b1),
  .GTHE3_COMMON_RSVD_ATTR0                      (16'b0000000000000000),
  .GTHE3_COMMON_RSVD_ATTR1                      (16'b0000000000000000),
  .GTHE3_COMMON_RSVD_ATTR2                      (16'b0000000000000000),
  .GTHE3_COMMON_RSVD_ATTR3                      (16'b0000000000000000),
  .GTHE3_COMMON_RXRECCLKOUT0_SEL                (2'b00),
  .GTHE3_COMMON_RXRECCLKOUT1_SEL                (2'b00),
  .GTHE3_COMMON_SARC_EN                         (1'b1),
  .GTHE3_COMMON_SARC_SEL                        (1'b0),
  .GTHE3_COMMON_SDM0DATA1_0                     (16'b0000000000000000),
  .GTHE3_COMMON_SDM0DATA1_1                     (9'b000000000),
  .GTHE3_COMMON_SDM0INITSEED0_0                 (16'b0000000000000000),
  .GTHE3_COMMON_SDM0INITSEED0_1                 (9'b000000000),
  .GTHE3_COMMON_SDM0_DATA_PIN_SEL               (1'b0),
  .GTHE3_COMMON_SDM0_WIDTH_PIN_SEL              (1'b0),
  .GTHE3_COMMON_SDM1DATA1_0                     (16'b0000000000000000),
  .GTHE3_COMMON_SDM1DATA1_1                     (9'b000000000),
  .GTHE3_COMMON_SDM1INITSEED0_0                 (16'b0000000000000000),
  .GTHE3_COMMON_SDM1INITSEED0_1                 (9'b000000000),
  .GTHE3_COMMON_SDM1_DATA_PIN_SEL               (1'b0),
  .GTHE3_COMMON_SDM1_WIDTH_PIN_SEL              (1'b0),
  .GTHE3_COMMON_SIM_RESET_SPEEDUP               ("TRUE"),
  .GTHE3_COMMON_SIM_VERSION                     (2)
) common_inst (

  // inputs
  .GTHE3_COMMON_BGBYPASSB                       (GTHE3_COMMON_BGBYPASSB),
  .GTHE3_COMMON_BGMONITORENB                    (GTHE3_COMMON_BGMONITORENB),
  .GTHE3_COMMON_BGPDB                           (GTHE3_COMMON_BGPDB),
  .GTHE3_COMMON_BGRCALOVRD                      (GTHE3_COMMON_BGRCALOVRD),
  .GTHE3_COMMON_BGRCALOVRDENB                   (GTHE3_COMMON_BGRCALOVRDENB),
  .GTHE3_COMMON_DRPADDR                         (GTHE3_COMMON_DRPADDR),
  .GTHE3_COMMON_DRPCLK                          (GTHE3_COMMON_DRPCLK),
  .GTHE3_COMMON_DRPDI                           (GTHE3_COMMON_DRPDI),
  .GTHE3_COMMON_DRPEN                           (GTHE3_COMMON_DRPEN),
  .GTHE3_COMMON_DRPWE                           (GTHE3_COMMON_DRPWE),
  .GTHE3_COMMON_GTGREFCLK0                      (GTHE3_COMMON_GTGREFCLK0),
  .GTHE3_COMMON_GTGREFCLK1                      (GTHE3_COMMON_GTGREFCLK1),
  .GTHE3_COMMON_GTNORTHREFCLK00                 (GTHE3_COMMON_GTNORTHREFCLK00),
  .GTHE3_COMMON_GTNORTHREFCLK01                 (GTHE3_COMMON_GTNORTHREFCLK01),
  .GTHE3_COMMON_GTNORTHREFCLK10                 (GTHE3_COMMON_GTNORTHREFCLK10),
  .GTHE3_COMMON_GTNORTHREFCLK11                 (GTHE3_COMMON_GTNORTHREFCLK11),
  .GTHE3_COMMON_GTREFCLK00                      (GTHE3_COMMON_GTREFCLK00),
  .GTHE3_COMMON_GTREFCLK01                      (GTHE3_COMMON_GTREFCLK01),
  .GTHE3_COMMON_GTREFCLK10                      (GTHE3_COMMON_GTREFCLK10),
  .GTHE3_COMMON_GTREFCLK11                      (GTHE3_COMMON_GTREFCLK11),
  .GTHE3_COMMON_GTSOUTHREFCLK00                 (GTHE3_COMMON_GTSOUTHREFCLK00),
  .GTHE3_COMMON_GTSOUTHREFCLK01                 (GTHE3_COMMON_GTSOUTHREFCLK01),
  .GTHE3_COMMON_GTSOUTHREFCLK10                 (GTHE3_COMMON_GTSOUTHREFCLK10),
  .GTHE3_COMMON_GTSOUTHREFCLK11                 (GTHE3_COMMON_GTSOUTHREFCLK11),
  .GTHE3_COMMON_PMARSVD0                        (GTHE3_COMMON_PMARSVD0),
  .GTHE3_COMMON_PMARSVD1                        (GTHE3_COMMON_PMARSVD1),
  .GTHE3_COMMON_QPLL0CLKRSVD0                   (GTHE3_COMMON_QPLL0CLKRSVD0),
  .GTHE3_COMMON_QPLL0CLKRSVD1                   (GTHE3_COMMON_QPLL0CLKRSVD1),
  .GTHE3_COMMON_QPLL0LOCKDETCLK                 (GTHE3_COMMON_QPLL0LOCKDETCLK),
  .GTHE3_COMMON_QPLL0LOCKEN                     (GTHE3_COMMON_QPLL0LOCKEN),
  .GTHE3_COMMON_QPLL0PD                         (GTHE3_COMMON_QPLL0PD),
  .GTHE3_COMMON_QPLL0REFCLKSEL                  (GTHE3_COMMON_QPLL0REFCLKSEL),
  .GTHE3_COMMON_QPLL0RESET                      (GTHE3_COMMON_QPLL0RESET),
  .GTHE3_COMMON_QPLL1CLKRSVD0                   (GTHE3_COMMON_QPLL1CLKRSVD0),
  .GTHE3_COMMON_QPLL1CLKRSVD1                   (GTHE3_COMMON_QPLL1CLKRSVD1),
  .GTHE3_COMMON_QPLL1LOCKDETCLK                 (GTHE3_COMMON_QPLL1LOCKDETCLK),
  .GTHE3_COMMON_QPLL1LOCKEN                     (GTHE3_COMMON_QPLL1LOCKEN),
  .GTHE3_COMMON_QPLL1PD                         (GTHE3_COMMON_QPLL1PD),
  .GTHE3_COMMON_QPLL1REFCLKSEL                  (GTHE3_COMMON_QPLL1REFCLKSEL),
  .GTHE3_COMMON_QPLL1RESET                      (GTHE3_COMMON_QPLL1RESET),
  .GTHE3_COMMON_QPLLRSVD1                       (GTHE3_COMMON_QPLLRSVD1),
  .GTHE3_COMMON_QPLLRSVD2                       (GTHE3_COMMON_QPLLRSVD2),
  .GTHE3_COMMON_QPLLRSVD3                       (GTHE3_COMMON_QPLLRSVD3),
  .GTHE3_COMMON_QPLLRSVD4                       (GTHE3_COMMON_QPLLRSVD4),
  .GTHE3_COMMON_RCALENB                         (GTHE3_COMMON_RCALENB),

  // outputs
  .GTHE3_COMMON_DRPDO                           (GTHE3_COMMON_DRPDO),
  .GTHE3_COMMON_DRPRDY                          (GTHE3_COMMON_DRPRDY),
  .GTHE3_COMMON_PMARSVDOUT0                     (GTHE3_COMMON_PMARSVDOUT0),
  .GTHE3_COMMON_PMARSVDOUT1                     (GTHE3_COMMON_PMARSVDOUT1),
  .GTHE3_COMMON_QPLL0FBCLKLOST                  (GTHE3_COMMON_QPLL0FBCLKLOST),
  .GTHE3_COMMON_QPLL0LOCK                       (GTHE3_COMMON_QPLL0LOCK),
  .GTHE3_COMMON_QPLL0OUTCLK                     (GTHE3_COMMON_QPLL0OUTCLK),
  .GTHE3_COMMON_QPLL0OUTREFCLK                  (GTHE3_COMMON_QPLL0OUTREFCLK),
  .GTHE3_COMMON_QPLL0REFCLKLOST                 (GTHE3_COMMON_QPLL0REFCLKLOST),
  .GTHE3_COMMON_QPLL1FBCLKLOST                  (GTHE3_COMMON_QPLL1FBCLKLOST),
  .GTHE3_COMMON_QPLL1LOCK                       (GTHE3_COMMON_QPLL1LOCK),
  .GTHE3_COMMON_QPLL1OUTCLK                     (GTHE3_COMMON_QPLL1OUTCLK),
  .GTHE3_COMMON_QPLL1OUTREFCLK                  (GTHE3_COMMON_QPLL1OUTREFCLK),
  .GTHE3_COMMON_QPLL1REFCLKLOST                 (GTHE3_COMMON_QPLL1REFCLKLOST),
  .GTHE3_COMMON_QPLLDMONITOR0                   (GTHE3_COMMON_QPLLDMONITOR0),
  .GTHE3_COMMON_QPLLDMONITOR1                   (GTHE3_COMMON_QPLLDMONITOR1),
  .GTHE3_COMMON_REFCLKOUTMONITOR0               (GTHE3_COMMON_REFCLKOUTMONITOR0),
  .GTHE3_COMMON_REFCLKOUTMONITOR1               (GTHE3_COMMON_REFCLKOUTMONITOR1),
  .GTHE3_COMMON_RXRECCLK0_SEL                   (GTHE3_COMMON_RXRECCLK0_SEL),
  .GTHE3_COMMON_RXRECCLK1_SEL                   (GTHE3_COMMON_RXRECCLK1_SEL)
);

endmodule

module ice_gth_t (
  gtwiz_userclk_tx_reset_in,
  gtwiz_userclk_tx_srcclk_out,
  gtwiz_userclk_tx_usrclk_out,
  gtwiz_userclk_tx_usrclk2_out,
  gtwiz_userclk_tx_active_out,
  gtwiz_userclk_rx_reset_in,
  gtwiz_userclk_rx_srcclk_out,
  gtwiz_userclk_rx_usrclk_out,
  gtwiz_userclk_rx_usrclk2_out,
  gtwiz_userclk_rx_active_out,
  gtwiz_reset_clk_freerun_in,
  gtwiz_reset_all_in,
  gtwiz_reset_tx_pll_and_datapath_in,
  gtwiz_reset_tx_datapath_in,
  gtwiz_reset_rx_pll_and_datapath_in,
  gtwiz_reset_rx_datapath_in,
  gtwiz_reset_qpll0lock_in,
  gtwiz_reset_rx_cdr_stable_out,
  gtwiz_reset_tx_done_out,
  gtwiz_reset_rx_done_out,
  gtwiz_reset_qpll0reset_out,
  gtwiz_userdata_tx_in,
  gtwiz_userdata_rx_out,
  gthrxn_in,
  gthrxp_in,
  qpll0clk_in,
  qpll0refclk_in,
  qpll1clk_in,
  qpll1refclk_in,
  rxgearboxslip_in,
  txheader_in,
  txsequence_in,
  gthtxn_out,
  gthtxp_out,
  rxdatavalid_out,
  rxheader_out,
  rxheadervalid_out,
  rxpmaresetdone_out,
  rxprgdivresetdone_out,
  rxstartofseq_out,
  txpmaresetdone_out,
  txprgdivresetdone_out,
  drpaddr_in,
  drpclk_in,
  drpdi_in,
  drpen_in,
  drprst_in,
  drpwe_in,
  drpdo_out,
  drprdy_out,
  rxlpmen_in,
  txeq_en,
  rxusrclk2_in,
  rxusrclk_in
);

input wire [0 : 0] gtwiz_userclk_tx_reset_in;
output wire [0 : 0] gtwiz_userclk_tx_srcclk_out;
output wire [0 : 0] gtwiz_userclk_tx_usrclk_out;
output wire [0 : 0] gtwiz_userclk_tx_usrclk2_out;
output wire [0 : 0] gtwiz_userclk_tx_active_out;
input wire [0 : 0] gtwiz_userclk_rx_reset_in;
output wire [0 : 0] gtwiz_userclk_rx_srcclk_out;
output wire [0 : 0] gtwiz_userclk_rx_usrclk_out;
output wire [0 : 0] gtwiz_userclk_rx_usrclk2_out;
output wire [0 : 0] gtwiz_userclk_rx_active_out;
input wire [0 : 0] gtwiz_reset_clk_freerun_in;
input wire [0 : 0] gtwiz_reset_all_in;
input wire [0 : 0] gtwiz_reset_tx_pll_and_datapath_in;
input wire [0 : 0] gtwiz_reset_tx_datapath_in;
input wire [0 : 0] gtwiz_reset_rx_pll_and_datapath_in;
input wire [0 : 0] gtwiz_reset_rx_datapath_in;
input wire [0 : 0] gtwiz_reset_qpll0lock_in;
output wire [0 : 0] gtwiz_reset_rx_cdr_stable_out;
output wire [0 : 0] gtwiz_reset_tx_done_out;
output wire [0 : 0] gtwiz_reset_rx_done_out;
output wire [0 : 0] gtwiz_reset_qpll0reset_out;
input wire [63 : 0] gtwiz_userdata_tx_in;
output wire [63 : 0] gtwiz_userdata_rx_out;
input wire [0 : 0] gthrxn_in;
input wire [0 : 0] gthrxp_in;
input wire [0 : 0] qpll0clk_in;
input wire [0 : 0] qpll0refclk_in;
input wire [0 : 0] qpll1clk_in;
input wire [0 : 0] qpll1refclk_in;
input wire [0 : 0] rxgearboxslip_in;
input wire [5 : 0] txheader_in;
input wire [6 : 0] txsequence_in;
output wire [0 : 0] gthtxn_out;
output wire [0 : 0] gthtxp_out;
output wire [1 : 0] rxdatavalid_out;
output wire [5 : 0] rxheader_out;
output wire [1 : 0] rxheadervalid_out;
output wire [0 : 0] rxpmaresetdone_out;
output wire [0 : 0] rxprgdivresetdone_out;
output wire [1 : 0] rxstartofseq_out;
output wire [0 : 0] txpmaresetdone_out;
output wire [0 : 0] txprgdivresetdone_out;
input  wire [(4*9)-1:0] drpaddr_in;
input  wire [(4*1)-1:0] drpclk_in;
input  wire [(4*16)-1:0] drpdi_in;
input  wire [(4*1)-1:0] drpen_in;
input  wire [(4*1)-1:0] drprst_in;
input  wire [(4*1)-1:0] drpwe_in;
output wire [(4*16)-1:0] drpdo_out;
output wire [(4*1)-1:0] drprdy_out;
input  wire [0 : 0] rxlpmen_in;
input  wire [0 : 0] txeq_en;
input  wire [0 : 0] rxusrclk2_in;
input  wire [0 : 0] rxusrclk_in;


  ice_gth_t_gtwizard_top #(
    .C_CHANNEL_ENABLE(192'B000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001),
    .C_COMMON_SCALING_FACTOR(1),
    .C_CPLL_VCO_FREQUENCY(2578.125),
    .C_FORCE_COMMONS(0),
    .C_FREERUN_FREQUENCY(50),
    .C_GT_TYPE(0),
    .C_GT_REV(17),
    .C_INCLUDE_CPLL_CAL(2),
    .C_LOCATE_COMMON(1),
    .C_LOCATE_RESET_CONTROLLER(0),
    .C_LOCATE_USER_DATA_WIDTH_SIZING(0),
    .C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER(0),
    .C_LOCATE_IN_SYSTEM_IBERT_CORE(2),
    .C_LOCATE_RX_USER_CLOCKING(0),
    .C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER(0),
    .C_LOCATE_TX_USER_CLOCKING(0),
    .C_RESET_CONTROLLER_INSTANCE_CTRL(0),
    .C_RX_BUFFBYPASS_MODE(0),
    .C_RX_BUFFER_BYPASS_INSTANCE_CTRL(0),
    .C_RX_BUFFER_MODE(1),
    .C_RX_CB_DISP(8'B00000000),
    .C_RX_CB_K(8'B00000000),
    .C_RX_CB_MAX_LEVEL(1),
    .C_RX_CB_LEN_SEQ(1),
    .C_RX_CB_NUM_SEQ(0),
    .C_RX_CB_VAL(80'B00000000000000000000000000000000000000000000000000000000000000000000000000000000),
    .C_RX_CC_DISP(8'B00000000),
    .C_RX_CC_ENABLE(0),
    .C_RESET_SEQUENCE_INTERVAL(0),
    .C_RX_CC_K(8'B00000000),
    .C_RX_CC_LEN_SEQ(1),
    .C_RX_CC_NUM_SEQ(0),
    .C_RX_CC_PERIODICITY(5000),
    .C_RX_CC_VAL(80'B00000000000000000000000000000000000000000000000000000000000000000000000000000000),
    .C_RX_COMMA_M_ENABLE(0),
    .C_RX_COMMA_M_VAL(10'B1010000011),
    .C_RX_COMMA_P_ENABLE(0),
    .C_RX_COMMA_P_VAL(10'B0101111100),
    .C_RX_DATA_DECODING(4),
    .C_RX_ENABLE(1),
    .C_RX_INT_DATA_WIDTH(32),
    .C_RX_LINE_RATE(`RIO_LINE_RATE),
    .C_RX_OUTCLK_FREQUENCY(`RIO_USR_CLKF),
    .C_RX_MASTER_CHANNEL_IDX(0),
    .C_RX_OUTCLK_BUFG_GT_DIV(1),
    .C_RX_OUTCLK_SOURCE(4),
    .C_RX_PLL_TYPE(0),
    .C_RX_RECCLK_OUTPUT(192'H000000000000000000000000000000000000000000000000),
    .C_RX_REFCLK_FREQUENCY(156.25),
    .C_RX_SLIDE_MODE(0),
    .C_RX_USER_CLOCKING_CONTENTS(0),
    .C_RX_USER_CLOCKING_INSTANCE_CTRL(0),
    .C_RX_USER_CLOCKING_RATIO_FSRC_FUSRCLK(1),
    .C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2(2),
    .C_RX_USER_CLOCKING_SOURCE(0),
    .C_RX_USER_DATA_WIDTH(64),
    .C_RX_USRCLK_FREQUENCY(`RIO_USR_CLKF),
    .C_RX_USRCLK2_FREQUENCY(`RIO_USR_CLKF/2),
    .C_SECONDARY_QPLL_ENABLE(0),
    .C_SECONDARY_QPLL_REFCLK_FREQUENCY(257.8125),
    .C_TOTAL_NUM_CHANNELS(1),
    .C_TOTAL_NUM_COMMONS(0),
    .C_TOTAL_NUM_COMMONS_EXAMPLE(1),
    .C_TXPROGDIV_FREQ_ENABLE(0),
    .C_TXPROGDIV_FREQ_SOURCE(0),
    .C_TXPROGDIV_FREQ_VAL(`RIO_USR_CLKF),
    .C_TX_BUFFBYPASS_MODE(0),
    .C_TX_BUFFER_BYPASS_INSTANCE_CTRL(0),
    .C_TX_BUFFER_MODE(1),
    .C_TX_DATA_ENCODING(4),
    .C_TX_ENABLE(1),
    .C_TX_INT_DATA_WIDTH(32),
    .C_TX_LINE_RATE(`RIO_LINE_RATE),
    .C_TX_OUTCLK_FREQUENCY(`RIO_USR_CLKF),
    .C_TX_MASTER_CHANNEL_IDX(0),
    .C_TX_OUTCLK_BUFG_GT_DIV(1),
    .C_TX_OUTCLK_SOURCE(4),
    .C_TX_PLL_TYPE(0),
    .C_TX_REFCLK_FREQUENCY(156.25),
    .C_TX_USER_CLOCKING_CONTENTS(0),
    .C_TX_USER_CLOCKING_INSTANCE_CTRL(0),
    .C_TX_USER_CLOCKING_RATIO_FSRC_FUSRCLK(1),
    .C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2(2),
    .C_TX_USER_CLOCKING_SOURCE(0),
    .C_TX_USER_DATA_WIDTH(64),
    .C_TX_USRCLK_FREQUENCY(`RIO_USR_CLKF),
    .C_TX_USRCLK2_FREQUENCY(`RIO_USR_CLKF/2)
  ) inst (
    .gtwiz_userclk_tx_reset_in(gtwiz_userclk_tx_reset_in),
    .gtwiz_userclk_tx_active_in(1'B0),
    .gtwiz_userclk_tx_srcclk_out(gtwiz_userclk_tx_srcclk_out),
    .gtwiz_userclk_tx_usrclk_out(gtwiz_userclk_tx_usrclk_out),
    .gtwiz_userclk_tx_usrclk2_out(gtwiz_userclk_tx_usrclk2_out),
    .gtwiz_userclk_tx_active_out(gtwiz_userclk_tx_active_out),
    .gtwiz_userclk_rx_reset_in(gtwiz_userclk_rx_reset_in),
    .gtwiz_userclk_rx_active_in(1'B0),
    .gtwiz_userclk_rx_srcclk_out(gtwiz_userclk_rx_srcclk_out),
    .gtwiz_userclk_rx_usrclk_out(gtwiz_userclk_rx_usrclk_out),
    .gtwiz_userclk_rx_usrclk2_out(gtwiz_userclk_rx_usrclk2_out),
    .gtwiz_userclk_rx_active_out(gtwiz_userclk_rx_active_out),
    .gtwiz_buffbypass_tx_reset_in(1'B0),
    .gtwiz_buffbypass_tx_start_user_in(1'B0),
    .gtwiz_buffbypass_tx_done_out(),
    .gtwiz_buffbypass_tx_error_out(),
    .gtwiz_buffbypass_rx_reset_in(1'B0),
    .gtwiz_buffbypass_rx_start_user_in(1'B0),
    .gtwiz_buffbypass_rx_done_out(),
    .gtwiz_buffbypass_rx_error_out(),
    .gtwiz_reset_clk_freerun_in(gtwiz_reset_clk_freerun_in),
    .gtwiz_reset_all_in(gtwiz_reset_all_in),
    .gtwiz_reset_tx_pll_and_datapath_in(gtwiz_reset_tx_pll_and_datapath_in),
    .gtwiz_reset_tx_datapath_in(gtwiz_reset_tx_datapath_in),
    .gtwiz_reset_rx_pll_and_datapath_in(gtwiz_reset_rx_pll_and_datapath_in),
    .gtwiz_reset_rx_datapath_in(gtwiz_reset_rx_datapath_in),
    .gtwiz_reset_tx_done_in(1'B0),
    .gtwiz_reset_rx_done_in(1'B0),
    .gtwiz_reset_qpll0lock_in(gtwiz_reset_qpll0lock_in),
    .gtwiz_reset_qpll1lock_in(1'B0),
    .gtwiz_reset_rx_cdr_stable_out(gtwiz_reset_rx_cdr_stable_out),
    .gtwiz_reset_tx_done_out(gtwiz_reset_tx_done_out),
    .gtwiz_reset_rx_done_out(gtwiz_reset_rx_done_out),
    .gtwiz_reset_qpll0reset_out(gtwiz_reset_qpll0reset_out),
    .gtwiz_reset_qpll1reset_out(),
    .gtwiz_gthe3_cpll_cal_txoutclk_period_in(18'B0),
    .gtwiz_gthe3_cpll_cal_cnt_tol_in(18'B0),
    .gtwiz_gthe3_cpll_cal_bufg_ce_in(1'B0),
    .gtwiz_userdata_tx_in(gtwiz_userdata_tx_in),
    .gtwiz_userdata_rx_out(gtwiz_userdata_rx_out),
    .bgbypassb_in(1'H1),
    .bgmonitorenb_in(1'H1),
    .bgpdb_in(1'H1),
    .bgrcalovrd_in(5'H1F),
    .bgrcalovrdenb_in(1'H1),
    .drpaddr_common_in(9'H000),
    .drpclk_common_in(1'H0),
    .drpdi_common_in(16'H0000),
    .drpen_common_in(1'H0),
    .drpwe_common_in(1'H0),
    .gtgrefclk0_in(1'H0),
    .gtgrefclk1_in(1'H0),
    .gtnorthrefclk00_in(1'H0),
    .gtnorthrefclk01_in(1'H0),
    .gtnorthrefclk10_in(1'H0),
    .gtnorthrefclk11_in(1'H0),
    .gtrefclk00_in(1'H0),
    .gtrefclk01_in(1'H0),
    .gtrefclk10_in(1'H0),
    .gtrefclk11_in(1'H0),
    .gtsouthrefclk00_in(1'H0),
    .gtsouthrefclk01_in(1'H0),
    .gtsouthrefclk10_in(1'H0),
    .gtsouthrefclk11_in(1'H0),
    .pcierateqpll0_in(1'B0),
    .pcierateqpll1_in(1'B0),
    .pmarsvd0_in(8'H00),
    .pmarsvd1_in(8'H00),
    .qpll0clkrsvd0_in(1'H0),
    .qpll0clkrsvd1_in(1'H0),
    .qpll0fbdiv_in(1'B0),
    .qpll0lockdetclk_in(1'H0),
    .qpll0locken_in(1'H1),
    .qpll0pd_in(1'H0),
    .qpll0refclksel_in(3'H1),
    .qpll0reset_in(1'H0),
    .qpll1clkrsvd0_in(1'H0),
    .qpll1clkrsvd1_in(1'H0),
    .qpll1fbdiv_in(1'B0),
    .qpll1lockdetclk_in(1'H0),
    .qpll1locken_in(1'H0),
    .qpll1pd_in(1'H1),
    .qpll1refclksel_in(3'H1),
    .qpll1reset_in(1'H1),
    .qpllrsvd1_in(8'H00),
    .qpllrsvd2_in(5'H00),
    .qpllrsvd3_in(5'H00),
    .qpllrsvd4_in(8'H00),
    .rcalenb_in(1'H1),
    .sdm0data_in(1'B0),
    .sdm0reset_in(1'B0),
    .sdm0toggle_in(1'B0),
    .sdm0width_in(1'B0),
    .sdm1data_in(1'B0),
    .sdm1reset_in(1'B0),
    .sdm1toggle_in(1'B0),
    .sdm1width_in(1'B0),
    .tcongpi_in(1'B0),
    .tconpowerup_in(1'B0),
    .tconreset_in(1'B0),
    .tconrsvdin1_in(1'B0),
    .ubcfgstreamen_in(1'B0),
    .ubdo_in(1'B0),
    .ubdrdy_in(1'B0),
    .ubenable_in(1'B0),
    .ubgpi_in(1'B0),
    .ubintr_in(1'B0),
    .ubiolmbrst_in(1'B0),
    .ubmbrst_in(1'B0),
    .ubmdmcapture_in(1'B0),
    .ubmdmdbgrst_in(1'B0),
    .ubmdmdbgupdate_in(1'B0),
    .ubmdmregen_in(1'B0),
    .ubmdmshift_in(1'B0),
    .ubmdmsysrst_in(1'B0),
    .ubmdmtck_in(1'B0),
    .ubmdmtdi_in(1'B0),
    .drpdo_common_out(),
    .drprdy_common_out(),
    .pmarsvdout0_out(),
    .pmarsvdout1_out(),
    .qpll0fbclklost_out(),
    .qpll0lock_out(),
    .qpll0outclk_out(),
    .qpll0outrefclk_out(),
    .qpll0refclklost_out(),
    .qpll1fbclklost_out(),
    .qpll1lock_out(),
    .qpll1outclk_out(),
    .qpll1outrefclk_out(),
    .qpll1refclklost_out(),
    .qplldmonitor0_out(),
    .qplldmonitor1_out(),
    .refclkoutmonitor0_out(),
    .refclkoutmonitor1_out(),
    .rxrecclk0_sel_out(),
    .rxrecclk1_sel_out(),
    .rxrecclk0sel_out(),
    .rxrecclk1sel_out(),
    .sdm0finalout_out(),
    .sdm0testdata_out(),
    .sdm1finalout_out(),
    .sdm1testdata_out(),
    .tcongpo_out(),
    .tconrsvdout0_out(),
    .ubdaddr_out(),
    .ubden_out(),
    .ubdi_out(),
    .ubdwe_out(),
    .ubmdmtdo_out(),
    .ubrsvdout_out(),
    .ubtxuart_out(),
    .cdrstepdir_in(1'B0),
    .cdrstepsq_in(1'B0),
    .cdrstepsx_in(1'B0),
    .cfgreset_in(1'H0),
    .clkrsvd0_in(1'H0),
    .clkrsvd1_in(1'H0),
    .cpllfreqlock_in(1'B0),
    .cplllockdetclk_in(1'H0),
    .cplllocken_in(1'H0),
    .cpllpd_in(1'H1),
    .cpllrefclksel_in(3'H1),
    .cpllreset_in(1'H1),
    .dmonfiforeset_in(1'H0),
    .dmonitorclk_in(1'H0),
    .drpaddr_in(drpaddr_in),
    .drpclk_in(drpclk_in),
    .drpdi_in(drpdi_in),
    .drpen_in(drpen_in),
    .drprst_in(drprst_in),
    .drpwe_in(drpwe_in),
    .elpcaldvorwren_in(1'B0),
    .elpcalpaorwren_in(1'B0),
    .evoddphicaldone_in(1'H0),
    .evoddphicalstart_in(1'H0),
    .evoddphidrden_in(1'H0),
    .evoddphidwren_in(1'H0),
    .evoddphixrden_in(1'H0),
    .evoddphixwren_in(1'H0),
    .eyescanmode_in(1'H0),
    .eyescanreset_in(1'H0),
    .eyescantrigger_in(1'H0),
    .freqos_in(1'B0),
    .gtgrefclk_in(1'H0),
    .gthrxn_in(gthrxn_in),
    .gthrxp_in(gthrxp_in),
    .gtnorthrefclk0_in(1'H0),
    .gtnorthrefclk1_in(1'H0),
    .gtrefclk0_in(1'H0),
    .gtrefclk1_in(1'H0),
    .gtresetsel_in(1'H0),
    .gtrsvd_in(16'H0000),
    .gtrxreset_in(1'H0),
    .gtrxresetsel_in(1'B0),
    .gtsouthrefclk0_in(1'H0),
    .gtsouthrefclk1_in(1'H0),
    .gttxreset_in(1'H0),
    .gttxresetsel_in(1'B0),
    .incpctrl_in(1'B0),
    .gtyrxn_in(1'B0),
    .gtyrxp_in(1'B0),
    .loopback_in(3'H0),
    .looprsvd_in(1'B0),
    .lpbkrxtxseren_in(1'H0),
    .lpbktxrxseren_in(1'H0),
    .pcieeqrxeqadaptdone_in(1'H0),
    .pcierstidle_in(1'H0),
    .pciersttxsyncstart_in(1'H0),
    .pcieuserratedone_in(1'H0),
    .pcsrsvdin_in(16'H0000),
    .pcsrsvdin2_in(5'H00),
    .pmarsvdin_in(5'H00),
    .qpll0clk_in(qpll0clk_in),
    .qpll0freqlock_in(1'B0),
    .qpll0refclk_in(qpll0refclk_in),
    .qpll1clk_in(qpll1clk_in),
    .qpll1freqlock_in(1'B0),
    .qpll1refclk_in(qpll1refclk_in),
    .resetovrd_in(1'H0),
    .rstclkentx_in(1'H0),
    .rx8b10ben_in(1'H0),
    .rxafecfoken_in(1'B0),
    .rxbufreset_in(1'H0),
    .rxcdrfreqreset_in(1'H0),
    .rxcdrhold_in(1'H0),
    .rxcdrovrden_in(1'H0),
    .rxcdrreset_in(1'H0),
    .rxcdrresetrsv_in(1'H0),
    .rxchbonden_in(1'H0),
    .rxchbondi_in(5'H00),
    .rxchbondlevel_in(3'H0),
    .rxchbondmaster_in(1'H0),
    .rxchbondslave_in(1'H0),
    .rxckcalreset_in(1'B0),
    .rxckcalstart_in(1'B0),
    .rxcommadeten_in(1'H0),
    .rxdfeagcctrl_in(2'H1),
    .rxdccforcestart_in(1'B0),
    .rxdfeagchold_in(1'H0),
    .rxdfeagcovrden_in(1'H0),
    .rxdfecfokfcnum_in(1'B0),
    .rxdfecfokfen_in(1'B0),
    .rxdfecfokfpulse_in(1'B0),
    .rxdfecfokhold_in(1'B0),
    .rxdfecfokovren_in(1'B0),
    .rxdfekhhold_in(1'B0),
    .rxdfekhovrden_in(1'B0),
    .rxdfelfhold_in(1'H0),
    .rxdfelfovrden_in(1'H0),
    .rxdfelpmreset_in(1'H0),
    .rxdfetap10hold_in(1'H0),
    .rxdfetap10ovrden_in(1'H0),
    .rxdfetap11hold_in(1'H0),
    .rxdfetap11ovrden_in(1'H0),
    .rxdfetap12hold_in(1'H0),
    .rxdfetap12ovrden_in(1'H0),
    .rxdfetap13hold_in(1'H0),
    .rxdfetap13ovrden_in(1'H0),
    .rxdfetap14hold_in(1'H0),
    .rxdfetap14ovrden_in(1'H0),
    .rxdfetap15hold_in(1'H0),
    .rxdfetap15ovrden_in(1'H0),
    .rxdfetap2hold_in(1'H0),
    .rxdfetap2ovrden_in(1'H0),
    .rxdfetap3hold_in(1'H0),
    .rxdfetap3ovrden_in(1'H0),
    .rxdfetap4hold_in(1'H0),
    .rxdfetap4ovrden_in(1'H0),
    .rxdfetap5hold_in(1'H0),
    .rxdfetap5ovrden_in(1'H0),
    .rxdfetap6hold_in(1'H0),
    .rxdfetap6ovrden_in(1'H0),
    .rxdfetap7hold_in(1'H0),
    .rxdfetap7ovrden_in(1'H0),
    .rxdfetap8hold_in(1'H0),
    .rxdfetap8ovrden_in(1'H0),
    .rxdfetap9hold_in(1'H0),
    .rxdfetap9ovrden_in(1'H0),
    .rxdfeuthold_in(1'H0),
    .rxdfeutovrden_in(1'H0),
    .rxdfevphold_in(1'H0),
    .rxdfevpovrden_in(1'H0),
    .rxdfevsen_in(1'H0),
    .rxdfexyden_in(1'H1),
    .rxdlybypass_in(1'H1),
    .rxdlyen_in(1'H0),
    .rxdlyovrden_in(1'H0),
    .rxdlysreset_in(1'H0),
    .rxelecidlemode_in(2'H3),
    .rxeqtraining_in(1'B0),
    .rxgearboxslip_in(rxgearboxslip_in),
    .rxlatclk_in(1'H0),
    .rxlpmen_in(rxlpmen_in), //1'H1),
    .rxlpmgchold_in(1'H0),
    .rxlpmgcovrden_in(1'H0),
    .rxlpmhfhold_in(1'H0),
    .rxlpmhfovrden_in(1'H0),
    .rxlpmlfhold_in(1'H0),
    .rxlpmlfklovrden_in(1'H0),
    .rxlpmoshold_in(1'H0),
    .rxlpmosovrden_in(1'H0),
    .rxmcommaalignen_in(1'H0),
    .rxmonitorsel_in(2'H0),
    .rxoobreset_in(1'H0),
    .rxoscalreset_in(1'H0),
    .rxoshold_in(1'H0),
    .rxosintcfg_in(4'HD),
    .rxosinten_in(1'H1),
    .rxosinthold_in(1'H0),
    .rxosintovrden_in(1'H0),
    .rxosintstrobe_in(1'H0),
    .rxosinttestovrden_in(1'H0),
    .rxosovrden_in(1'H0),
    .rxoutclksel_in(3'H5),
    .rxpcommaalignen_in(1'H0),
    .rxpcsreset_in(1'H0),
    .rxpd_in(2'H0),
    .rxphalign_in(1'H0),
    .rxphalignen_in(1'H0),
    .rxphdlypd_in(1'H1),
    .rxphdlyreset_in(1'H0),
    .rxphovrden_in(1'H0),
    .rxpllclksel_in(2'H3),
    .rxpmareset_in(1'H0),
    .rxpolarity_in(1'H0),
    .rxprbscntreset_in(1'H0),
    .rxprbssel_in(4'H0),
    .rxprogdivreset_in(1'H0),
    .rxqpien_in(1'H0),
    .rxrate_in(3'H0),
    .rxratemode_in(1'H0),
    .rxslide_in(1'H0),
    .rxslipoutclk_in(1'H0),
    .rxslippma_in(1'H0),
    .rxsyncallin_in(1'H0),
    .rxsyncin_in(1'H0),
    .rxsyncmode_in(1'H0),
    .rxsysclksel_in(2'H2),
    .rxtermination_in(1'B0),
    .rxuserrdy_in(1'H1),
    .rxusrclk_in(rxusrclk_in), // JJB::: 1'H0),
    .rxusrclk2_in(rxusrclk2_in), // JJB::: 1'H0),
    .sigvalidclk_in(1'H0),
    .tstin_in(20'H00000),
    .tx8b10bbypass_in(8'H00),
    .tx8b10ben_in(1'H0),
    .txbufdiffctrl_in(3'H0),
    .txcominit_in(1'H0),
    .txcomsas_in(1'H0),
    .txcomwake_in(1'H0),
    .txctrl0_in(16'H0000),
    .txctrl1_in(16'H0000),
    .txctrl2_in(8'H00),
    .txdata_in(128'B0),
    .txdataextendrsvd_in(8'H00),
    .txdccforcestart_in(1'B0),
    .txdccreset_in(1'B0),
    .txdeemph_in(1'H0),
    .txdetectrx_in(1'H0),
    .txdiffctrl_in((txeq_en)? 4'HC : 4'HF),
    .txdiffpd_in(1'H0),
    .txdlybypass_in(1'H1),
    .txdlyen_in(1'H0),
    .txdlyhold_in(1'H0),
    .txdlyovrden_in(1'H0),
    .txdlysreset_in(1'H0),
    .txdlyupdown_in(1'H0),
    .txelecidle_in(1'H0),
    .txelforcestart_in(1'B0),
    .txheader_in(txheader_in),
    .txinhibit_in(1'H0),
    .txlatclk_in(1'H0),
    .txlfpstreset_in(1'B0),
    .txlfpsu2lpexit_in(1'B0),
    .txlfpsu3wake_in(1'B0),
    .txmaincursor_in(7'H40),
    .txmargin_in(3'H0),
    .txmuxdcdexhold_in(1'B0),
    .txmuxdcdorwren_in(1'B0),
    .txoneszeros_in(1'B0),
    .txoutclksel_in(3'H5),
    .txpcsreset_in(1'H0),
    .txpd_in(2'H0),
    .txpdelecidlemode_in(1'H0),
    .txphalign_in(1'H0),
    .txphalignen_in(1'H0),
    .txphdlypd_in(1'H1),
    .txphdlyreset_in(1'H0),
    .txphdlytstclk_in(1'H0),
    .txphinit_in(1'H0),
    .txphovrden_in(1'H0),
    .txpippmen_in(1'H0),
    .txpippmovrden_in(1'H0),
    .txpippmpd_in(1'H0),
    .txpippmsel_in(1'H0),
    .txpippmstepsize_in(5'H00),
    .txpisopd_in(1'H0),
    .txpllclksel_in(2'H3),
    .txpmareset_in(1'H0),
    .txpolarity_in(1'H0),
    .txpostcursor_in(5'H00),
    .txpostcursorinv_in(1'H0),
    .txprbsforceerr_in(1'H0),
    .txprbssel_in(4'H0),
    .txprecursor_in(5'H00),
    .txprecursorinv_in(1'H0),
    .txprogdivreset_in(1'H0),
    .txqpibiasen_in(1'H0),
    .txqpistrongpdown_in(1'H0),
    .txqpiweakpup_in(1'H0),
    .txrate_in(3'H0),
    .txratemode_in(1'H0),
    .txsequence_in(txsequence_in),
    .txswing_in(1'H0),
    .txsyncallin_in(1'H0),
    .txsyncin_in(1'H0),
    .txsyncmode_in(1'H0),
    .txsysclksel_in(2'H2),
    .txuserrdy_in(1'H1),
    .txusrclk_in(1'H0),
    .txusrclk2_in(1'H0),
    .bufgtce_out(),
    .bufgtcemask_out(),
    .bufgtdiv_out(),
    .bufgtreset_out(),
    .bufgtrstmask_out(),
    .cpllfbclklost_out(),
    .cplllock_out(),
    .cpllrefclklost_out(),
    .dmonitorout_out(),
    .dmonitoroutclk_out(),
    .drpdo_out(drpdo_out),
    .drprdy_out(drprdy_out),
    .eyescandataerror_out(),
    .gthtxn_out(gthtxn_out),
    .gthtxp_out(gthtxp_out),
    .gtpowergood_out(),
    .gtrefclkmonitor_out(),
    .gtytxn_out(),
    .gtytxp_out(),
    .pcierategen3_out(),
    .pcierateidle_out(),
    .pcierateqpllpd_out(),
    .pcierateqpllreset_out(),
    .pciesynctxsyncdone_out(),
    .pcieusergen3rdy_out(),
    .pcieuserphystatusrst_out(),
    .pcieuserratestart_out(),
    .pcsrsvdout_out(),
    .phystatus_out(),
    .pinrsrvdas_out(),
    .powerpresent_out(),
    .resetexception_out(),
    .rxbufstatus_out(),
    .rxbyteisaligned_out(),
    .rxbyterealign_out(),
    .rxcdrlock_out(),
    .rxcdrphdone_out(),
    .rxchanbondseq_out(),
    .rxchanisaligned_out(),
    .rxchanrealign_out(),
    .rxchbondo_out(),
    .rxckcaldone_out(),
    .rxclkcorcnt_out(),
    .rxcominitdet_out(),
    .rxcommadet_out(),
    .rxcomsasdet_out(),
    .rxcomwakedet_out(),
    .rxctrl0_out(),
    .rxctrl1_out(),
    .rxctrl2_out(),
    .rxctrl3_out(),
    .rxdata_out(),
    .rxdataextendrsvd_out(),
    .rxdatavalid_out(rxdatavalid_out),
    .rxdlysresetdone_out(),
    .rxelecidle_out(),
    .rxheader_out(rxheader_out),
    .rxheadervalid_out(rxheadervalid_out),
    .rxlfpstresetdet_out(),
    .rxlfpsu2lpexitdet_out(),
    .rxlfpsu3wakedet_out(),
    .rxmonitorout_out(),
    .rxosintdone_out(),
    .rxosintstarted_out(),
    .rxosintstrobedone_out(),
    .rxosintstrobestarted_out(),
    .rxoutclk_out(),
    .rxoutclkfabric_out(),
    .rxoutclkpcs_out(),
    .rxphaligndone_out(),
    .rxphalignerr_out(),
    .rxpmaresetdone_out(rxpmaresetdone_out),
    .rxprbserr_out(),
    .rxprbslocked_out(),
    .rxprgdivresetdone_out(rxprgdivresetdone_out),
    .rxqpisenn_out(),
    .rxqpisenp_out(),
    .rxratedone_out(),
    .rxrecclkout_out(),
    .rxresetdone_out(),
    .rxsliderdy_out(),
    .rxslipdone_out(),
    .rxslipoutclkrdy_out(),
    .rxslippmardy_out(),
    .rxstartofseq_out(rxstartofseq_out),
    .rxstatus_out(),
    .rxsyncdone_out(),
    .rxsyncout_out(),
    .rxvalid_out(),
    .txbufstatus_out(),
    .txcomfinish_out(),
    .txdccdone_out(),
    .txdlysresetdone_out(),
    .txoutclk_out(),
    .txoutclkfabric_out(),
    .txoutclkpcs_out(),
    .txphaligndone_out(),
    .txphinitdone_out(),
    .txpmaresetdone_out(txpmaresetdone_out),
    .txprgdivresetdone_out(txprgdivresetdone_out),
    .txqpisenn_out(),
    .txqpisenp_out(),
    .txratedone_out(),
    .txresetdone_out(),
    .txsyncdone_out(),
    .txsyncout_out()
  );
endmodule

`define ice_gth_t_MAX_NUM_CHANNELS 192
`define ice_gth_t_MAX_NUM_COMMONS 48
`define ice_gth_t_N_CM C_TOTAL_NUM_COMMONS
`define ice_gth_t_N_CH C_TOTAL_NUM_CHANNELS
`define ice_gth_t_SF_CM C_COMMON_SCALING_FACTOR
`define ice_gth_t_FORCE_COMMONS__DO_NOT_FORCE 0
`define ice_gth_t_FORCE_COMMONS__FORCE 1
`define ice_gth_t_GT_TYPE__GTHE3 0
`define ice_gth_t_GT_TYPE__GTYE3 1
`define ice_gth_t_GT_TYPE__GTHE4 2
`define ice_gth_t_GT_TYPE__GTYE4 3
`define ice_gth_t_INCLUDE_CPLL_CAL__EXCLUDE 0
`define ice_gth_t_INCLUDE_CPLL_CAL__INCLUDE 1
`define ice_gth_t_INCLUDE_CPLL_CAL__DEPENDENT 2
`define ice_gth_t_LOCATE_COMMON__CORE 0
`define ice_gth_t_LOCATE_COMMON__EXAMPLE_DESIGN 1
`define ice_gth_t_LOCATE_RESET_CONTROLLER__CORE 0
`define ice_gth_t_LOCATE_RESET_CONTROLLER__EXAMPLE_DESIGN 1
`define ice_gth_t_LOCATE_USER_DATA_WIDTH_SIZING__CORE 0
`define ice_gth_t_LOCATE_USER_DATA_WIDTH_SIZING__EXAMPLE_DESIGN 1
`define ice_gth_t_LOCATE_RX_BUFFER_BYPASS_CONTROLLER__CORE 0
`define ice_gth_t_LOCATE_RX_BUFFER_BYPASS_CONTROLLER__EXAMPLE_DESIGN 1
`define ice_gth_t_LOCATE_IN_SYSTEM_IBERT_CORE__EXAMPLE_DESIGN 1
`define ice_gth_t_LOCATE_IN_SYSTEM_IBERT_CORE__NONE 2
`define ice_gth_t_LOCATE_RX_USER_CLOCKING__CORE 0
`define ice_gth_t_LOCATE_RX_USER_CLOCKING__EXAMPLE_DESIGN 1
`define ice_gth_t_LOCATE_TX_BUFFER_BYPASS_CONTROLLER__CORE 0
`define ice_gth_t_LOCATE_TX_BUFFER_BYPASS_CONTROLLER__EXAMPLE_DESIGN 1
`define ice_gth_t_LOCATE_TX_USER_CLOCKING__CORE 0
`define ice_gth_t_LOCATE_TX_USER_CLOCKING__EXAMPLE_DESIGN 1
`define ice_gth_t_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_RESET_CONTROLLER_INSTANCE_CTRL__PER_CHANNEL 0
`define ice_gth_t_RX_BUFFBYPASS_MODE__AUTO 0
`define ice_gth_t_RX_BUFFBYPASS_MODE__MANUAL 1
`define ice_gth_t_RX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_RX_BUFFER_BYPASS_INSTANCE_CTRL__PER_CHANNEL 1
`define ice_gth_t_RX_BUFFER_MODE__BYPASS 0
`define ice_gth_t_RX_BUFFER_MODE__USE 1
`define ice_gth_t_RX_CC_ENABLE__DISABLED 0
`define ice_gth_t_RX_CC_ENABLE__ENABLED 1
`define ice_gth_t_RX_COMMA_M_ENABLE__DISABLED 0
`define ice_gth_t_RX_COMMA_M_ENABLE__ENABLED 1
`define ice_gth_t_RX_COMMA_P_ENABLE__DISABLED 0
`define ice_gth_t_RX_COMMA_P_ENABLE__ENABLED 1
`define ice_gth_t_RX_DATA_DECODING__RAW 0
`define ice_gth_t_RX_DATA_DECODING__8B10B 1
`define ice_gth_t_RX_DATA_DECODING__64B66B 2
`define ice_gth_t_RX_DATA_DECODING__64B66B_CAUI 3
`define ice_gth_t_RX_DATA_DECODING__64B66B_ASYNC 4
`define ice_gth_t_RX_DATA_DECODING__64B66B_ASYNC_CAUI 5
`define ice_gth_t_RX_DATA_DECODING__64B67B 6
`define ice_gth_t_RX_DATA_DECODING__64B67B_CAUI 7
`define ice_gth_t_RX_DATA_DECODING__128B130B 10
`define ice_gth_t_RX_ENABLE__DISABLED 0
`define ice_gth_t_RX_ENABLE__ENABLED 1
`define ice_gth_t_RX_OUTCLK_SOURCE__RXOUTCLKPCS 0
`define ice_gth_t_RX_OUTCLK_SOURCE__RXOUTCLKPMA 1
`define ice_gth_t_RX_OUTCLK_SOURCE__RXPLLREFCLK_DIV1 2
`define ice_gth_t_RX_OUTCLK_SOURCE__RXPLLREFCLK_DIV2 3
`define ice_gth_t_RX_OUTCLK_SOURCE__RXPROGDIVCLK 4
`define ice_gth_t_RX_PLL_TYPE__QPLL0 0
`define ice_gth_t_RX_PLL_TYPE__QPLL1 1
`define ice_gth_t_RX_PLL_TYPE__CPLL 2
`define ice_gth_t_RX_SLIDE_MODE__OFF 0
`define ice_gth_t_RX_SLIDE_MODE__PCS 1
`define ice_gth_t_RX_SLIDE_MODE__PMA 2
`define ice_gth_t_RX_SLIDE_MODE__AUTO 3
`define ice_gth_t_RX_USER_CLOCKING_CONTENTS__BUFG_GT 0
`define ice_gth_t_RX_USER_CLOCKING_CONTENTS__BUFG 1
`define ice_gth_t_RX_USER_CLOCKING_CONTENTS__MMCM 2
`define ice_gth_t_RX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_RX_USER_CLOCKING_INSTANCE_CTRL__PER_CHANNEL 1
`define ice_gth_t_RX_USER_CLOCKING_SOURCE__RXOUTCLK 0
`define ice_gth_t_RX_USER_CLOCKING_SOURCE__IBUFDS 1
`define ice_gth_t_RX_USER_CLOCKING_SOURCE__TXOUTCLK 2
`define ice_gth_t_SECONDARY_QPLL_ENABLE__DISABLED 0
`define ice_gth_t_SECONDARY_QPLL_ENABLE__ENABLED 1
`define ice_gth_t_TXPROGDIV_FREQ_ENABLE__DISABLED 0
`define ice_gth_t_TXPROGDIV_FREQ_ENABLE__ENABLED 1
`define ice_gth_t_TXPROGDIV_FREQ_SOURCE__QPLL0 0
`define ice_gth_t_TXPROGDIV_FREQ_SOURCE__QPLL1 1
`define ice_gth_t_TXPROGDIV_FREQ_SOURCE__CPLL 2
`define ice_gth_t_TX_BUFFBYPASS_MODE__AUTO 0
`define ice_gth_t_TX_BUFFBYPASS_MODE__MANUAL 1
`define ice_gth_t_TX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_TX_BUFFER_BYPASS_INSTANCE_CTRL__PER_CHANNEL 1
`define ice_gth_t_TX_BUFFER_MODE__BYPASS 0
`define ice_gth_t_TX_BUFFER_MODE__USE 1
`define ice_gth_t_TX_DATA_ENCODING__RAW 0
`define ice_gth_t_TX_DATA_ENCODING__8B10B 1
`define ice_gth_t_TX_DATA_ENCODING__64B66B 2
`define ice_gth_t_TX_DATA_ENCODING__64B66B_CAUI 3
`define ice_gth_t_TX_DATA_ENCODING__64B66B_ASYNC 4
`define ice_gth_t_TX_DATA_ENCODING__64B66B_ASYNC_CAUI 5
`define ice_gth_t_TX_DATA_ENCODING__64B67B 6
`define ice_gth_t_TX_DATA_ENCODING__64B67B_CAUI 7
`define ice_gth_t_TX_DATA_ENCODING__128B130B 10
`define ice_gth_t_TX_ENABLE__DISABLED 0
`define ice_gth_t_TX_ENABLE__ENABLED 1
`define ice_gth_t_TX_OUTCLK_SOURCE__TXOUTCLKPCS 0
`define ice_gth_t_TX_OUTCLK_SOURCE__TXOUTCLKPMA 1
`define ice_gth_t_TX_OUTCLK_SOURCE__TXPLLREFCLK_DIV1 2
`define ice_gth_t_TX_OUTCLK_SOURCE__TXPLLREFCLK_DIV2 3
`define ice_gth_t_TX_OUTCLK_SOURCE__TXPROGDIVCLK 4
`define ice_gth_t_TX_PLL_TYPE__QPLL0 0
`define ice_gth_t_TX_PLL_TYPE__QPLL1 1
`define ice_gth_t_TX_PLL_TYPE__CPLL 2
`define ice_gth_t_TX_USER_CLOCKING_CONTENTS__BUFG_GT 0
`define ice_gth_t_TX_USER_CLOCKING_CONTENTS__BUFG 1
`define ice_gth_t_TX_USER_CLOCKING_CONTENTS__MMCM 2
`define ice_gth_t_TX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_TX_USER_CLOCKING_INSTANCE_CTRL__PER_CHANNEL 1
`define ice_gth_t_TX_USER_CLOCKING_SOURCE__TXOUTCLK 0
`define ice_gth_t_TX_USER_CLOCKING_SOURCE__IBUFDS 1

module ice_gth_t_gtwizard_top #(

  parameter [191:0] C_CHANNEL_ENABLE                          = 192'b1,
  parameter integer C_COMMON_SCALING_FACTOR                   = 1,
  parameter real    C_CPLL_VCO_FREQUENCY                      = 5156.25,
  parameter integer C_FORCE_COMMONS                           = `ice_gth_t_FORCE_COMMONS__DO_NOT_FORCE,
  parameter real    C_FREERUN_FREQUENCY                       = 200,
  parameter integer C_GT_TYPE                                 = `ice_gth_t_GT_TYPE__GTHE3,
  parameter integer C_GT_REV                                  = 17,
  parameter integer C_INCLUDE_CPLL_CAL                        = `ice_gth_t_INCLUDE_CPLL_CAL__DEPENDENT,
  parameter integer C_LOCATE_COMMON                           = `ice_gth_t_LOCATE_COMMON__CORE,
  parameter integer C_LOCATE_RESET_CONTROLLER                 = `ice_gth_t_LOCATE_RESET_CONTROLLER__CORE,
  parameter integer C_LOCATE_USER_DATA_WIDTH_SIZING           = `ice_gth_t_LOCATE_USER_DATA_WIDTH_SIZING__CORE,
  parameter integer C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER      = `ice_gth_t_LOCATE_RX_BUFFER_BYPASS_CONTROLLER__CORE,
  parameter integer C_LOCATE_IN_SYSTEM_IBERT_CORE             = `ice_gth_t_LOCATE_IN_SYSTEM_IBERT_CORE__NONE,
  parameter integer C_LOCATE_RX_USER_CLOCKING                 = `ice_gth_t_LOCATE_RX_USER_CLOCKING__EXAMPLE_DESIGN,
  parameter integer C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER      = `ice_gth_t_LOCATE_TX_BUFFER_BYPASS_CONTROLLER__CORE,
  parameter integer C_LOCATE_TX_USER_CLOCKING                 = `ice_gth_t_LOCATE_TX_USER_CLOCKING__EXAMPLE_DESIGN,
  parameter integer C_RESET_CONTROLLER_INSTANCE_CTRL          = `ice_gth_t_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_RX_BUFFBYPASS_MODE                      = `ice_gth_t_RX_BUFFBYPASS_MODE__AUTO,
  parameter integer C_RX_BUFFER_BYPASS_INSTANCE_CTRL          = `ice_gth_t_RX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_RX_BUFFER_MODE                          = `ice_gth_t_RX_BUFFER_MODE__USE,
  parameter   [7:0] C_RX_CB_DISP                              = 8'b0,
  parameter   [7:0] C_RX_CB_K                                 = 8'b0,
  parameter integer C_RX_CB_MAX_LEVEL                         = 1,
  parameter integer C_RX_CB_LEN_SEQ                           = 1,
  parameter integer C_RX_CB_NUM_SEQ                           = 0,
  parameter  [79:0] C_RX_CB_VAL                               = 80'b0,
  parameter   [7:0] C_RX_CC_DISP                              = 8'b0,
  parameter integer C_RX_CC_ENABLE                            = `ice_gth_t_RX_CC_ENABLE__DISABLED,
  parameter integer C_RESET_SEQUENCE_INTERVAL                 = 0,
  parameter   [7:0] C_RX_CC_K                                 = 8'b0,
  parameter integer C_RX_CC_LEN_SEQ                           = 1,
  parameter integer C_RX_CC_NUM_SEQ                           = 0,
  parameter integer C_RX_CC_PERIODICITY                       = 5000,
  parameter  [79:0] C_RX_CC_VAL                               = 80'b0,
  parameter integer C_RX_COMMA_M_ENABLE                       = `ice_gth_t_RX_COMMA_M_ENABLE__DISABLED,
  parameter   [9:0] C_RX_COMMA_M_VAL                          = 10'b1010000011,
  parameter integer C_RX_COMMA_P_ENABLE                       = `ice_gth_t_RX_COMMA_P_ENABLE__DISABLED,
  parameter   [9:0] C_RX_COMMA_P_VAL                          = 10'b0101111100,
  parameter integer C_RX_DATA_DECODING                        = `ice_gth_t_RX_DATA_DECODING__RAW,
  parameter integer C_RX_ENABLE                               = `ice_gth_t_RX_ENABLE__ENABLED,
  parameter integer C_RX_INT_DATA_WIDTH                       = 32,
  parameter real    C_RX_LINE_RATE                            = 10.3125,
  parameter integer C_RX_MASTER_CHANNEL_IDX                   = 0,
  parameter integer C_RX_OUTCLK_BUFG_GT_DIV                   = 1,
  parameter real    C_RX_OUTCLK_FREQUENCY                     = 322.265625,
  parameter integer C_RX_OUTCLK_SOURCE                        = `ice_gth_t_RX_OUTCLK_SOURCE__RXOUTCLKPMA,
  parameter integer C_RX_PLL_TYPE                             = `ice_gth_t_RX_PLL_TYPE__QPLL0,
  parameter [191:0] C_RX_RECCLK_OUTPUT                        = 192'b0,
  parameter real    C_RX_REFCLK_FREQUENCY                     = 257.8125,
  parameter integer C_RX_SLIDE_MODE                           = `ice_gth_t_RX_SLIDE_MODE__OFF,
  parameter integer C_RX_USER_CLOCKING_CONTENTS               = `ice_gth_t_RX_USER_CLOCKING_CONTENTS__BUFG_GT,
  parameter integer C_RX_USER_CLOCKING_INSTANCE_CTRL          = `ice_gth_t_RX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_RX_USER_CLOCKING_RATIO_FSRC_FUSRCLK     = 1,
  parameter integer C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 = 1,
  parameter integer C_RX_USER_CLOCKING_SOURCE                 = `ice_gth_t_RX_USER_CLOCKING_SOURCE__RXOUTCLK,
  parameter integer C_RX_USER_DATA_WIDTH                      = 32,
  parameter real    C_RX_USRCLK_FREQUENCY                     = 322.265625,
  parameter real    C_RX_USRCLK2_FREQUENCY                    = 322.265625,
  parameter integer C_SECONDARY_QPLL_ENABLE                   = `ice_gth_t_SECONDARY_QPLL_ENABLE__DISABLED,
  parameter real    C_SECONDARY_QPLL_REFCLK_FREQUENCY         = 257.8125,
  parameter integer C_TOTAL_NUM_CHANNELS                      = 1,
  parameter integer C_TOTAL_NUM_COMMONS                       = 1,
  parameter integer C_TOTAL_NUM_COMMONS_EXAMPLE               = 0,
  parameter integer C_TXPROGDIV_FREQ_ENABLE                   = `ice_gth_t_TXPROGDIV_FREQ_ENABLE__DISABLED,
  parameter integer C_TXPROGDIV_FREQ_SOURCE                   = `ice_gth_t_TXPROGDIV_FREQ_SOURCE__QPLL0,
  parameter real    C_TXPROGDIV_FREQ_VAL                      = 322.265625,
  parameter integer C_TX_BUFFBYPASS_MODE                      = `ice_gth_t_TX_BUFFBYPASS_MODE__AUTO,
  parameter integer C_TX_BUFFER_BYPASS_INSTANCE_CTRL          = `ice_gth_t_TX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_TX_BUFFER_MODE                          = `ice_gth_t_TX_BUFFER_MODE__USE,
  parameter integer C_TX_DATA_ENCODING                        = `ice_gth_t_TX_DATA_ENCODING__RAW,
  parameter integer C_TX_ENABLE                               = `ice_gth_t_TX_ENABLE__ENABLED,
  parameter integer C_TX_INT_DATA_WIDTH                       = 32,
  parameter real    C_TX_LINE_RATE                            = 10.3125,
  parameter integer C_TX_MASTER_CHANNEL_IDX                   = 0,
  parameter integer C_TX_OUTCLK_BUFG_GT_DIV                   = 1,
  parameter real    C_TX_OUTCLK_FREQUENCY                     = 322.265625,
  parameter integer C_TX_OUTCLK_SOURCE                        = `ice_gth_t_TX_OUTCLK_SOURCE__TXOUTCLKPMA,
  parameter integer C_TX_PLL_TYPE                             = `ice_gth_t_TX_PLL_TYPE__QPLL0,
  parameter real    C_TX_REFCLK_FREQUENCY                     = 257.8125,
  parameter integer C_TX_USER_CLOCKING_CONTENTS               = `ice_gth_t_TX_USER_CLOCKING_CONTENTS__BUFG_GT,
  parameter integer C_TX_USER_CLOCKING_INSTANCE_CTRL          = `ice_gth_t_TX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_TX_USER_CLOCKING_RATIO_FSRC_FUSRCLK     = 1,
  parameter integer C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 = 1,
  parameter integer C_TX_USER_CLOCKING_SOURCE                 = `ice_gth_t_TX_USER_CLOCKING_SOURCE__TXOUTCLK,
  parameter integer C_TX_USER_DATA_WIDTH                      = 32,
  parameter real    C_TX_USRCLK_FREQUENCY                     = 322.265625,
  parameter real    C_TX_USRCLK2_FREQUENCY                    = 322.265625

)(

  // Transmitter user clocking network helper block ports
  input  wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_tx_reset_in,
  input  wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_tx_active_in,
  output wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_tx_srcclk_out,
  output wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_tx_usrclk_out,
  output wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_tx_usrclk2_out,
  output wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_tx_active_out,

  // Receiver user clocking network helper block ports
  input  wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_rx_reset_in,
  input  wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_rx_active_in,
  output wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_rx_srcclk_out,
  output wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_rx_usrclk_out,
  output wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_rx_usrclk2_out,
  output wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_userclk_rx_active_out,

  // Transmitter buffer bypass controller helper block ports
  input  wire [(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_buffbypass_tx_reset_in,
  input  wire [(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_buffbypass_tx_start_user_in,
  output wire [(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_buffbypass_tx_done_out,
  output wire [(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_buffbypass_tx_error_out,

  // Receiver buffer bypass controller helper block ports
  input  wire [(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_buffbypass_rx_reset_in,
  input  wire [(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_buffbypass_rx_start_user_in,
  output wire [(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_buffbypass_rx_done_out,
  output wire [(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_buffbypass_rx_error_out,

  // Reset controller helper block ports
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_clk_freerun_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_all_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_tx_pll_and_datapath_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_tx_datapath_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_rx_pll_and_datapath_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_rx_datapath_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_tx_done_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_rx_done_in,
  input  wire [                                  (`ice_gth_t_SF_CM-1):0] gtwiz_reset_qpll0lock_in,
  input  wire [                                  (`ice_gth_t_SF_CM-1):0] gtwiz_reset_qpll1lock_in,
  output wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_rx_cdr_stable_out,
  output wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_tx_done_out,
  output wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_N_CH-1)):0] gtwiz_reset_rx_done_out,
  output wire [                                  (`ice_gth_t_SF_CM-1):0] gtwiz_reset_qpll0reset_out,
  output wire [                                  (`ice_gth_t_SF_CM-1):0] gtwiz_reset_qpll1reset_out,

  // CPLL calibration block ports
  input  wire [(`ice_gth_t_N_CH* 18)-1:0] gtwiz_gthe3_cpll_cal_txoutclk_period_in,
  input  wire [(`ice_gth_t_N_CH* 18)-1:0] gtwiz_gthe3_cpll_cal_cnt_tol_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gtwiz_gthe3_cpll_cal_bufg_ce_in,

  // Transmitter user data width sizing helper block ports
  input  wire [(`ice_gth_t_N_CH*C_TX_USER_DATA_WIDTH)-1:0] gtwiz_userdata_tx_in,

  // Receiver user data width sizing helper block ports
  output wire [(`ice_gth_t_N_CH*C_RX_USER_DATA_WIDTH)-1:0] gtwiz_userdata_rx_out,

  // Transceiver common block ports
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] bgbypassb_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] bgmonitorenb_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] bgpdb_in,
  input  wire [(`ice_gth_t_SF_CM*  5)-1:0] bgrcalovrd_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] bgrcalovrdenb_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_SF_CM*  9))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM* 10))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM* 16))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM* 16))-1:0] drpaddr_common_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] drpclk_common_in,
  input  wire [(`ice_gth_t_SF_CM* 16)-1:0] drpdi_common_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] drpen_common_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] drpwe_common_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtgrefclk0_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtgrefclk1_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtnorthrefclk00_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtnorthrefclk01_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtnorthrefclk10_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtnorthrefclk11_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtrefclk00_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtrefclk01_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtrefclk10_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtrefclk11_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtsouthrefclk00_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtsouthrefclk01_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtsouthrefclk10_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] gtsouthrefclk11_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  3))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  3))-1:0] pcierateqpll0_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  3))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  3))-1:0] pcierateqpll1_in,
  input  wire [(`ice_gth_t_SF_CM*  8)-1:0] pmarsvd0_in,
  input  wire [(`ice_gth_t_SF_CM*  8)-1:0] pmarsvd1_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0clkrsvd0_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] qpll0clkrsvd1_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  8))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  8))-1:0] qpll0fbdiv_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0lockdetclk_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0locken_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0pd_in,
  input  wire [(`ice_gth_t_SF_CM*  3)-1:0] qpll0refclksel_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0reset_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1clkrsvd0_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] qpll1clkrsvd1_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  8))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  8))-1:0] qpll1fbdiv_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1lockdetclk_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1locken_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1pd_in,
  input  wire [(`ice_gth_t_SF_CM*  3)-1:0] qpll1refclksel_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1reset_in,
  input  wire [(`ice_gth_t_SF_CM*  8)-1:0] qpllrsvd1_in,
  input  wire [(`ice_gth_t_SF_CM*  5)-1:0] qpllrsvd2_in,
  input  wire [(`ice_gth_t_SF_CM*  5)-1:0] qpllrsvd3_in,
  input  wire [(`ice_gth_t_SF_CM*  8)-1:0] qpllrsvd4_in,
  input  wire [(`ice_gth_t_SF_CM*  1)-1:0] rcalenb_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM* 25))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM* 25))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM* 25))-1:0] sdm0data_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] sdm0reset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] sdm0toggle_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  2))-1:0] sdm0width_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM* 25))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM* 25))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM* 25))-1:0] sdm1data_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] sdm1reset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] sdm1toggle_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  2))-1:0] sdm1width_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM* 10))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] tcongpi_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] tconpowerup_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] tconreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] tconrsvdin1_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubcfgstreamen_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM* 16))-1:0] ubdo_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubdrdy_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubenable_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  2))-1:0] ubgpi_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  2))-1:0] ubintr_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubiolmbrst_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubmbrst_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubmdmcapture_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubmdmdbgrst_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubmdmdbgupdate_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  4))-1:0] ubmdmregen_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubmdmshift_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubmdmsysrst_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubmdmtck_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubmdmtdi_in,

  output wire [(`ice_gth_t_SF_CM* 16)-1:0] drpdo_common_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] drprdy_common_out,
  output wire [(`ice_gth_t_SF_CM*  8)-1:0] pmarsvdout0_out,
  output wire [(`ice_gth_t_SF_CM*  8)-1:0] pmarsvdout1_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0fbclklost_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0lock_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0outclk_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0outrefclk_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll0refclklost_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1fbclklost_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1lock_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1outclk_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1outrefclk_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] qpll1refclklost_out,
  output wire [(`ice_gth_t_SF_CM*  8)-1:0] qplldmonitor0_out,
  output wire [(`ice_gth_t_SF_CM*  8)-1:0] qplldmonitor1_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] refclkoutmonitor0_out,
  output wire [(`ice_gth_t_SF_CM*  1)-1:0] refclkoutmonitor1_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxrecclk0_sel_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxrecclk1_sel_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  2))-1:0] rxrecclk0sel_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  2))-1:0] rxrecclk1sel_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM*  4))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  4))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  4))-1:0] sdm0finalout_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM* 15))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM* 15))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM* 15))-1:0] sdm0testdata_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM*  4))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  4))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  4))-1:0] sdm1finalout_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_SF_CM* 15))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM* 15))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM* 15))-1:0] sdm1testdata_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM* 10))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] tcongpo_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_SF_CM*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] tconrsvdout0_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM* 16))-1:0] ubdaddr_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubden_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM* 16))-1:0] ubdi_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubdwe_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubmdmtdo_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubrsvdout_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_SF_CM*  1))-1:0] ubtxuart_out,

  // Transceiver channel block ports
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] cdrstepdir_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] cdrstepsq_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] cdrstepsx_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] cfgreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] clkrsvd0_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] clkrsvd1_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] cpllfreqlock_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] cplllockdetclk_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] cplllocken_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] cpllpd_in,
  input  wire [(`ice_gth_t_N_CH*  3)-1:0] cpllrefclksel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] cpllreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] dmonfiforeset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] dmonitorclk_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  9))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH* 10))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH* 10))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH* 10))-1:0] drpaddr_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] drpclk_in,
  input  wire [(`ice_gth_t_N_CH* 16)-1:0] drpdi_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] drpen_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] drprst_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] drpwe_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] elpcaldvorwren_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] elpcalpaorwren_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] evoddphicaldone_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] evoddphicalstart_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] evoddphidrden_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] evoddphidwren_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] evoddphixrden_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] evoddphixwren_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] eyescanmode_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] eyescanreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] eyescantrigger_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] freqos_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gtgrefclk_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] gthrxn_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] gthrxp_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gtnorthrefclk0_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gtnorthrefclk1_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gtrefclk0_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gtrefclk1_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] gtresetsel_in,
  input  wire [(`ice_gth_t_N_CH* 16)-1:0] gtrsvd_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gtrxreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] gtrxresetsel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gtsouthrefclk0_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gtsouthrefclk1_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] gttxreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] gttxresetsel_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] incpctrl_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] gtyrxn_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] gtyrxp_in,
  input  wire [(`ice_gth_t_N_CH*  3)-1:0] loopback_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH* 16))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] looprsvd_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] lpbkrxtxseren_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] lpbktxrxseren_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] pcieeqrxeqadaptdone_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] pcierstidle_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] pciersttxsyncstart_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] pcieuserratedone_in,
  input  wire [(`ice_gth_t_N_CH* 16)-1:0] pcsrsvdin_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  5))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  5))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] pcsrsvdin2_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  5))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  5))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] pmarsvdin_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] qpll0clk_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] qpll0freqlock_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] qpll0refclk_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] qpll1clk_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] qpll1freqlock_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] qpll1refclk_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] resetovrd_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rstclkentx_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rx8b10ben_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxafecfoken_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxbufreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxcdrfreqreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxcdrhold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxcdrovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxcdrreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxcdrresetrsv_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxchbonden_in,
  input  wire [(`ice_gth_t_N_CH*  5)-1:0] rxchbondi_in,
  input  wire [(`ice_gth_t_N_CH*  3)-1:0] rxchbondlevel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxchbondmaster_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxchbondslave_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxckcalreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  7))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  7))-1:0] rxckcalstart_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxcommadeten_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxdfeagcctrl_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxdccforcestart_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfeagchold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfeagcovrden_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  4))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  4))-1:0] rxdfecfokfcnum_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxdfecfokfen_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxdfecfokfpulse_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxdfecfokhold_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxdfecfokovren_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxdfekhhold_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxdfekhovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfelfhold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfelfovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfelpmreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap10hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap10ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap11hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap11ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap12hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap12ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap13hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap13ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap14hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap14ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap15hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap15ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap2hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap2ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap3hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap3ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap4hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap4ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap5hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap5ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap6hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap6ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap7hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap7ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap8hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap8ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap9hold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfetap9ovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfeuthold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfeutovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfevphold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfevpovrden_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxdfevsen_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdfexyden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdlybypass_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdlyen_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdlyovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxdlysreset_in,
  input  wire [(`ice_gth_t_N_CH*  2)-1:0] rxelecidlemode_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxeqtraining_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxgearboxslip_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlatclk_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlpmen_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlpmgchold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlpmgcovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlpmhfhold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlpmhfovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlpmlfhold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlpmlfklovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlpmoshold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxlpmosovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxmcommaalignen_in,
  input  wire [(`ice_gth_t_N_CH*  2)-1:0] rxmonitorsel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxoobreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxoscalreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxoshold_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  4))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  4))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxosintcfg_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxosinten_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxosinthold_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxosintovrden_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxosintstrobe_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxosinttestovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxosovrden_in,
  input  wire [(`ice_gth_t_N_CH*  3)-1:0] rxoutclksel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxpcommaalignen_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxpcsreset_in,
  input  wire [(`ice_gth_t_N_CH*  2)-1:0] rxpd_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxphalign_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxphalignen_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxphdlypd_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxphdlyreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxphovrden_in,
  input  wire [(`ice_gth_t_N_CH*  2)-1:0] rxpllclksel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxpmareset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxpolarity_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxprbscntreset_in,
  input  wire [(`ice_gth_t_N_CH*  4)-1:0] rxprbssel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxprogdivreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxqpien_in,
  input  wire [(`ice_gth_t_N_CH*  3)-1:0] rxrate_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxratemode_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxslide_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxslipoutclk_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxslippma_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxsyncallin_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxsyncin_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxsyncmode_in,
  input  wire [(`ice_gth_t_N_CH*  2)-1:0] rxsysclksel_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxtermination_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxuserrdy_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxusrclk_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] rxusrclk2_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] sigvalidclk_in,
  input  wire [(`ice_gth_t_N_CH* 20)-1:0] tstin_in,
  input  wire [(`ice_gth_t_N_CH*  8)-1:0] tx8b10bbypass_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] tx8b10ben_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  3))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  3))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txbufdiffctrl_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txcominit_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txcomsas_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txcomwake_in,
  input  wire [(`ice_gth_t_N_CH* 16)-1:0] txctrl0_in,
  input  wire [(`ice_gth_t_N_CH* 16)-1:0] txctrl1_in,
  input  wire [(`ice_gth_t_N_CH*  8)-1:0] txctrl2_in,
  input  wire [(`ice_gth_t_N_CH*128)-1:0] txdata_in,
  input  wire [(`ice_gth_t_N_CH*  8)-1:0] txdataextendrsvd_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] txdccforcestart_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] txdccreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  2))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  2))-1:0] txdeemph_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txdetectrx_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  4))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  5))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  5))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  5))-1:0] txdiffctrl_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txdiffpd_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txdlybypass_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txdlyen_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txdlyhold_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txdlyovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txdlysreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txdlyupdown_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txelecidle_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txelforcestart_in,
  input  wire [(`ice_gth_t_N_CH*  6)-1:0] txheader_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txinhibit_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txlatclk_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] txlfpstreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] txlfpsu2lpexit_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] txlfpsu3wake_in,
  input  wire [(`ice_gth_t_N_CH*  7)-1:0] txmaincursor_in,
  input  wire [(`ice_gth_t_N_CH*  3)-1:0] txmargin_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] txmuxdcdexhold_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] txmuxdcdorwren_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] txoneszeros_in,
  input  wire [(`ice_gth_t_N_CH*  3)-1:0] txoutclksel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txpcsreset_in,
  input  wire [(`ice_gth_t_N_CH*  2)-1:0] txpd_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txpdelecidlemode_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txphalign_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txphalignen_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txphdlypd_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txphdlyreset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txphdlytstclk_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txphinit_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txphovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txpippmen_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txpippmovrden_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txpippmpd_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txpippmsel_in,
  input  wire [(`ice_gth_t_N_CH*  5)-1:0] txpippmstepsize_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txpisopd_in,
  input  wire [(`ice_gth_t_N_CH*  2)-1:0] txpllclksel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txpmareset_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txpolarity_in,
  input  wire [(`ice_gth_t_N_CH*  5)-1:0] txpostcursor_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txpostcursorinv_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txprbsforceerr_in,
  input  wire [(`ice_gth_t_N_CH*  4)-1:0] txprbssel_in,
  input  wire [(`ice_gth_t_N_CH*  5)-1:0] txprecursor_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txprecursorinv_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txprogdivreset_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txqpibiasen_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txqpistrongpdown_in,
  input  wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txqpiweakpup_in,
  input  wire [(`ice_gth_t_N_CH*  3)-1:0] txrate_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txratemode_in,
  input  wire [(`ice_gth_t_N_CH*  7)-1:0] txsequence_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txswing_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txsyncallin_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txsyncin_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txsyncmode_in,
  input  wire [(`ice_gth_t_N_CH*  2)-1:0] txsysclksel_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txuserrdy_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txusrclk_in,
  input  wire [(`ice_gth_t_N_CH*  1)-1:0] txusrclk2_in,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  3))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  3))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] bufgtce_out,
  output wire [(`ice_gth_t_N_CH*  3)-1:0] bufgtcemask_out,
  output wire [(`ice_gth_t_N_CH*  9)-1:0] bufgtdiv_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  3))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  3))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] bufgtreset_out,
  output wire [(`ice_gth_t_N_CH*  3)-1:0] bufgtrstmask_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] cpllfbclklost_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] cplllock_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] cpllrefclklost_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH* 17))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH* 17))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH* 16))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH* 16))-1:0] dmonitorout_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] dmonitoroutclk_out,
  output wire [(`ice_gth_t_N_CH* 16)-1:0] drpdo_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] drprdy_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] eyescandataerror_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] gthtxn_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] gthtxp_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] gtpowergood_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] gtrefclkmonitor_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] gtytxn_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] gtytxp_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] pcierategen3_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] pcierateidle_out,
  output wire [(`ice_gth_t_N_CH*  2)-1:0] pcierateqpllpd_out,
  output wire [(`ice_gth_t_N_CH*  2)-1:0] pcierateqpllreset_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] pciesynctxsyncdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] pcieusergen3rdy_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] pcieuserphystatusrst_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] pcieuserratestart_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH* 12))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH* 16))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH* 16))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH* 16))-1:0] pcsrsvdout_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] phystatus_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  8))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  8))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH* 16))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH* 16))-1:0] pinrsrvdas_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] powerpresent_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] resetexception_out,
  output wire [(`ice_gth_t_N_CH*  3)-1:0] rxbufstatus_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxbyteisaligned_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxbyterealign_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxcdrlock_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxcdrphdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxchanbondseq_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxchanisaligned_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxchanrealign_out,
  output wire [(`ice_gth_t_N_CH*  5)-1:0] rxchbondo_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxckcaldone_out,
  output wire [(`ice_gth_t_N_CH*  2)-1:0] rxclkcorcnt_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxcominitdet_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxcommadet_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxcomsasdet_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxcomwakedet_out,
  output wire [(`ice_gth_t_N_CH* 16)-1:0] rxctrl0_out,
  output wire [(`ice_gth_t_N_CH* 16)-1:0] rxctrl1_out,
  output wire [(`ice_gth_t_N_CH*  8)-1:0] rxctrl2_out,
  output wire [(`ice_gth_t_N_CH*  8)-1:0] rxctrl3_out,
  output wire [(`ice_gth_t_N_CH*128)-1:0] rxdata_out,
  output wire [(`ice_gth_t_N_CH*  8)-1:0] rxdataextendrsvd_out,
  output wire [(`ice_gth_t_N_CH*  2)-1:0] rxdatavalid_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxdlysresetdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxelecidle_out,
  output wire [(`ice_gth_t_N_CH*  6)-1:0] rxheader_out,
  output wire [(`ice_gth_t_N_CH*  2)-1:0] rxheadervalid_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxlfpstresetdet_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxlfpsu2lpexitdet_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] rxlfpsu3wakedet_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  7))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  7))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  8))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  8))-1:0] rxmonitorout_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxosintdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxosintstarted_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxosintstrobedone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxosintstrobestarted_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxoutclk_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxoutclkfabric_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxoutclkpcs_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxphaligndone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxphalignerr_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxpmaresetdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxprbserr_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxprbslocked_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxprgdivresetdone_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxqpisenn_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] rxqpisenp_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxratedone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxrecclkout_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxresetdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxsliderdy_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxslipdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxslipoutclkrdy_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxslippmardy_out,
  output wire [(`ice_gth_t_N_CH*  2)-1:0] rxstartofseq_out,
  output wire [(`ice_gth_t_N_CH*  3)-1:0] rxstatus_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxsyncdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxsyncout_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] rxvalid_out,
  output wire [(`ice_gth_t_N_CH*  2)-1:0] txbufstatus_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txcomfinish_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(`ice_gth_t_N_CH*  1))-1:0] txdccdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txdlysresetdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txoutclk_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txoutclkfabric_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txoutclkpcs_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txphaligndone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txphinitdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txpmaresetdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txprgdivresetdone_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txqpisenn_out,
  output wire [((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE3)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE3)*(1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTHE4)*(`ice_gth_t_N_CH*  1))+
               ((C_GT_TYPE==`ice_gth_t_GT_TYPE__GTYE4)*(1))-1:0] txqpisenp_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txratedone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txresetdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txsyncdone_out,
  output wire [(`ice_gth_t_N_CH*  1)-1:0] txsyncout_out

);


  // ================================================================================================================
  // CONDITIONAL INSTANTIATION OF TRANSCEIVERS WIZARD SUBMODULE BASED ON TRANSCEIVER TYPE
  // ================================================================================================================

  generate if (C_GT_TYPE == `ice_gth_t_GT_TYPE__GTHE3) begin : gen_gtwizard_gthe3_top

    // Generate GTHE3-type Transceivers Wizard submodule
    ice_gth_t_gtwizard_gthe3 #(
      .C_CHANNEL_ENABLE                          (C_CHANNEL_ENABLE                         ),
      .C_COMMON_SCALING_FACTOR                   (C_COMMON_SCALING_FACTOR                  ),
      .C_CPLL_VCO_FREQUENCY                      (C_CPLL_VCO_FREQUENCY                     ),
      .C_FREERUN_FREQUENCY                       (C_FREERUN_FREQUENCY                      ),
      .C_GT_REV                                  (C_GT_REV                                 ),
      .C_INCLUDE_CPLL_CAL                        (C_INCLUDE_CPLL_CAL                       ),
      .C_LOCATE_RESET_CONTROLLER                 (C_LOCATE_RESET_CONTROLLER                ),
      .C_LOCATE_USER_DATA_WIDTH_SIZING           (C_LOCATE_USER_DATA_WIDTH_SIZING          ),
      .C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER      (C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER     ),
      .C_LOCATE_RX_USER_CLOCKING                 (C_LOCATE_RX_USER_CLOCKING                ),
      .C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER      (C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER     ),
      .C_LOCATE_TX_USER_CLOCKING                 (C_LOCATE_TX_USER_CLOCKING                ),
      .C_RESET_CONTROLLER_INSTANCE_CTRL          (C_RESET_CONTROLLER_INSTANCE_CTRL         ),
      .C_RX_BUFFBYPASS_MODE                      (C_RX_BUFFBYPASS_MODE                     ),
      .C_RX_BUFFER_BYPASS_INSTANCE_CTRL          (C_RX_BUFFER_BYPASS_INSTANCE_CTRL         ),
      .C_RX_BUFFER_MODE                          (C_RX_BUFFER_MODE                         ),
      .C_RX_DATA_DECODING                        (C_RX_DATA_DECODING                       ),
      .C_RX_ENABLE                               (C_RX_ENABLE                              ),
      .C_RX_INT_DATA_WIDTH                       (C_RX_INT_DATA_WIDTH                      ),
      .C_RX_LINE_RATE                            (C_RX_LINE_RATE                           ),
      .C_RX_MASTER_CHANNEL_IDX                   (C_RX_MASTER_CHANNEL_IDX                  ),
      .C_RX_OUTCLK_BUFG_GT_DIV                   (C_RX_OUTCLK_BUFG_GT_DIV                  ),
      .C_RX_PLL_TYPE                             (C_RX_PLL_TYPE                            ),
      .C_RX_USER_CLOCKING_CONTENTS               (C_RX_USER_CLOCKING_CONTENTS              ),
      .C_RX_USER_CLOCKING_INSTANCE_CTRL          (C_RX_USER_CLOCKING_INSTANCE_CTRL         ),
      .C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 (C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2),
      .C_RX_USER_CLOCKING_SOURCE                 (C_RX_USER_CLOCKING_SOURCE                ),
      .C_RX_USER_DATA_WIDTH                      (C_RX_USER_DATA_WIDTH                     ),
      .C_TOTAL_NUM_CHANNELS                      (C_TOTAL_NUM_CHANNELS                     ),
      .C_TOTAL_NUM_COMMONS                       (C_TOTAL_NUM_COMMONS                      ),
      .C_TXPROGDIV_FREQ_ENABLE                   (C_TXPROGDIV_FREQ_ENABLE                  ),
      .C_TXPROGDIV_FREQ_SOURCE                   (C_TXPROGDIV_FREQ_SOURCE                  ),
      .C_TX_BUFFBYPASS_MODE                      (C_TX_BUFFBYPASS_MODE                     ),
      .C_TX_BUFFER_BYPASS_INSTANCE_CTRL          (C_TX_BUFFER_BYPASS_INSTANCE_CTRL         ),
      .C_TX_BUFFER_MODE                          (C_TX_BUFFER_MODE                         ),
      .C_TX_DATA_ENCODING                        (C_TX_DATA_ENCODING                       ),
      .C_TX_ENABLE                               (C_TX_ENABLE                              ),
      .C_TX_INT_DATA_WIDTH                       (C_TX_INT_DATA_WIDTH                      ),
      .C_TX_MASTER_CHANNEL_IDX                   (C_TX_MASTER_CHANNEL_IDX                  ),
      .C_TX_OUTCLK_BUFG_GT_DIV                   (C_TX_OUTCLK_BUFG_GT_DIV                  ),
      .C_TX_PLL_TYPE                             (C_TX_PLL_TYPE                            ),
      .C_TX_USER_CLOCKING_CONTENTS               (C_TX_USER_CLOCKING_CONTENTS              ),
      .C_TX_USER_CLOCKING_INSTANCE_CTRL          (C_TX_USER_CLOCKING_INSTANCE_CTRL         ),
      .C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 (C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2),
      .C_TX_USER_CLOCKING_SOURCE                 (C_TX_USER_CLOCKING_SOURCE                ),
      .C_TX_USER_DATA_WIDTH                      (C_TX_USER_DATA_WIDTH                     )
    ) ice_gth_t_gtwizard_gthe3_inst (
      .gtwiz_userclk_tx_reset_in               (gtwiz_userclk_tx_reset_in              ),
      .gtwiz_userclk_tx_active_in              (gtwiz_userclk_tx_active_in             ),
      .gtwiz_userclk_tx_srcclk_out             (gtwiz_userclk_tx_srcclk_out            ),
      .gtwiz_userclk_tx_usrclk_out             (gtwiz_userclk_tx_usrclk_out            ),
      .gtwiz_userclk_tx_usrclk2_out            (gtwiz_userclk_tx_usrclk2_out           ),
      .gtwiz_userclk_tx_active_out             (gtwiz_userclk_tx_active_out            ),
      .gtwiz_userclk_rx_reset_in               (gtwiz_userclk_rx_reset_in              ),
      .gtwiz_userclk_rx_active_in              (gtwiz_userclk_rx_active_in             ),
      .gtwiz_userclk_rx_srcclk_out             (gtwiz_userclk_rx_srcclk_out            ),
      .gtwiz_userclk_rx_usrclk_out             (gtwiz_userclk_rx_usrclk_out            ),
      .gtwiz_userclk_rx_usrclk2_out            (gtwiz_userclk_rx_usrclk2_out           ),
      .gtwiz_userclk_rx_active_out             (gtwiz_userclk_rx_active_out            ),
      .gtwiz_buffbypass_tx_reset_in            (gtwiz_buffbypass_tx_reset_in           ),
      .gtwiz_buffbypass_tx_start_user_in       (gtwiz_buffbypass_tx_start_user_in      ),
      .gtwiz_buffbypass_tx_done_out            (gtwiz_buffbypass_tx_done_out           ),
      .gtwiz_buffbypass_tx_error_out           (gtwiz_buffbypass_tx_error_out          ),
      .gtwiz_buffbypass_rx_reset_in            (gtwiz_buffbypass_rx_reset_in           ),
      .gtwiz_buffbypass_rx_start_user_in       (gtwiz_buffbypass_rx_start_user_in      ),
      .gtwiz_buffbypass_rx_done_out            (gtwiz_buffbypass_rx_done_out           ),
      .gtwiz_buffbypass_rx_error_out           (gtwiz_buffbypass_rx_error_out          ),
      .gtwiz_reset_clk_freerun_in              (gtwiz_reset_clk_freerun_in             ),
      .gtwiz_reset_all_in                      (gtwiz_reset_all_in                     ),
      .gtwiz_reset_tx_pll_and_datapath_in      (gtwiz_reset_tx_pll_and_datapath_in     ),
      .gtwiz_reset_tx_datapath_in              (gtwiz_reset_tx_datapath_in             ),
      .gtwiz_reset_rx_pll_and_datapath_in      (gtwiz_reset_rx_pll_and_datapath_in     ),
      .gtwiz_reset_rx_datapath_in              (gtwiz_reset_rx_datapath_in             ),
      .gtwiz_reset_tx_done_in                  (gtwiz_reset_tx_done_in                 ),
      .gtwiz_reset_rx_done_in                  (gtwiz_reset_rx_done_in                 ),
      .gtwiz_reset_qpll0lock_in                (gtwiz_reset_qpll0lock_in               ),
      .gtwiz_reset_qpll1lock_in                (gtwiz_reset_qpll1lock_in               ),
      .gtwiz_reset_rx_cdr_stable_out           (gtwiz_reset_rx_cdr_stable_out          ),
      .gtwiz_reset_tx_done_out                 (gtwiz_reset_tx_done_out                ),
      .gtwiz_reset_rx_done_out                 (gtwiz_reset_rx_done_out                ),
      .gtwiz_reset_qpll0reset_out              (gtwiz_reset_qpll0reset_out             ),
      .gtwiz_reset_qpll1reset_out              (gtwiz_reset_qpll1reset_out             ),
      .gtwiz_gthe3_cpll_cal_txoutclk_period_in (gtwiz_gthe3_cpll_cal_txoutclk_period_in),
      .gtwiz_gthe3_cpll_cal_cnt_tol_in         (gtwiz_gthe3_cpll_cal_cnt_tol_in        ),
      .gtwiz_gthe3_cpll_cal_bufg_ce_in         (gtwiz_gthe3_cpll_cal_bufg_ce_in        ),
      .gtwiz_userdata_tx_in                    (gtwiz_userdata_tx_in                   ),
      .gtwiz_userdata_rx_out                   (gtwiz_userdata_rx_out                  ),
      .bgbypassb_in                            (bgbypassb_in                           ),
      .bgmonitorenb_in                         (bgmonitorenb_in                        ),
      .bgpdb_in                                (bgpdb_in                               ),
      .bgrcalovrd_in                           (bgrcalovrd_in                          ),
      .bgrcalovrdenb_in                        (bgrcalovrdenb_in                       ),
      .drpaddr_common_in                       (drpaddr_common_in                      ),
      .drpclk_common_in                        (drpclk_common_in                       ),
      .drpdi_common_in                         (drpdi_common_in                        ),
      .drpen_common_in                         (drpen_common_in                        ),
      .drpwe_common_in                         (drpwe_common_in                        ),
      .gtgrefclk0_in                           (gtgrefclk0_in                          ),
      .gtgrefclk1_in                           (gtgrefclk1_in                          ),
      .gtnorthrefclk00_in                      (gtnorthrefclk00_in                     ),
      .gtnorthrefclk01_in                      (gtnorthrefclk01_in                     ),
      .gtnorthrefclk10_in                      (gtnorthrefclk10_in                     ),
      .gtnorthrefclk11_in                      (gtnorthrefclk11_in                     ),
      .gtrefclk00_in                           (gtrefclk00_in                          ),
      .gtrefclk01_in                           (gtrefclk01_in                          ),
      .gtrefclk10_in                           (gtrefclk10_in                          ),
      .gtrefclk11_in                           (gtrefclk11_in                          ),
      .gtsouthrefclk00_in                      (gtsouthrefclk00_in                     ),
      .gtsouthrefclk01_in                      (gtsouthrefclk01_in                     ),
      .gtsouthrefclk10_in                      (gtsouthrefclk10_in                     ),
      .gtsouthrefclk11_in                      (gtsouthrefclk11_in                     ),
      .pmarsvd0_in                             (pmarsvd0_in                            ),
      .pmarsvd1_in                             (pmarsvd1_in                            ),
      .qpll0clkrsvd0_in                        (qpll0clkrsvd0_in                       ),
      .qpll0clkrsvd1_in                        (qpll0clkrsvd1_in                       ),
      .qpll0lockdetclk_in                      (qpll0lockdetclk_in                     ),
      .qpll0locken_in                          (qpll0locken_in                         ),
      .qpll0pd_in                              (qpll0pd_in                             ),
      .qpll0refclksel_in                       (qpll0refclksel_in                      ),
      .qpll0reset_in                           (qpll0reset_in                          ),
      .qpll1clkrsvd0_in                        (qpll1clkrsvd0_in                       ),
      .qpll1clkrsvd1_in                        (qpll1clkrsvd1_in                       ),
      .qpll1lockdetclk_in                      (qpll1lockdetclk_in                     ),
      .qpll1locken_in                          (qpll1locken_in                         ),
      .qpll1pd_in                              (qpll1pd_in                             ),
      .qpll1refclksel_in                       (qpll1refclksel_in                      ),
      .qpll1reset_in                           (qpll1reset_in                          ),
      .qpllrsvd1_in                            (qpllrsvd1_in                           ),
      .qpllrsvd2_in                            (qpllrsvd2_in                           ),
      .qpllrsvd3_in                            (qpllrsvd3_in                           ),
      .qpllrsvd4_in                            (qpllrsvd4_in                           ),
      .rcalenb_in                              (rcalenb_in                             ),
      .drpdo_common_out                        (drpdo_common_out                       ),
      .drprdy_common_out                       (drprdy_common_out                      ),
      .pmarsvdout0_out                         (pmarsvdout0_out                        ),
      .pmarsvdout1_out                         (pmarsvdout1_out                        ),
      .qpll0fbclklost_out                      (qpll0fbclklost_out                     ),
      .qpll0lock_out                           (qpll0lock_out                          ),
      .qpll0outclk_out                         (qpll0outclk_out                        ),
      .qpll0outrefclk_out                      (qpll0outrefclk_out                     ),
      .qpll0refclklost_out                     (qpll0refclklost_out                    ),
      .qpll1fbclklost_out                      (qpll1fbclklost_out                     ),
      .qpll1lock_out                           (qpll1lock_out                          ),
      .qpll1outclk_out                         (qpll1outclk_out                        ),
      .qpll1outrefclk_out                      (qpll1outrefclk_out                     ),
      .qpll1refclklost_out                     (qpll1refclklost_out                    ),
      .qplldmonitor0_out                       (qplldmonitor0_out                      ),
      .qplldmonitor1_out                       (qplldmonitor1_out                      ),
      .refclkoutmonitor0_out                   (refclkoutmonitor0_out                  ),
      .refclkoutmonitor1_out                   (refclkoutmonitor1_out                  ),
      .rxrecclk0_sel_out                       (rxrecclk0_sel_out                      ),
      .rxrecclk1_sel_out                       (rxrecclk1_sel_out                      ),
      .cfgreset_in                             (cfgreset_in                            ),
      .clkrsvd0_in                             (clkrsvd0_in                            ),
      .clkrsvd1_in                             (clkrsvd1_in                            ),
      .cplllockdetclk_in                       (cplllockdetclk_in                      ),
      .cplllocken_in                           (cplllocken_in                          ),
      .cpllpd_in                               (cpllpd_in                              ),
      .cpllrefclksel_in                        (cpllrefclksel_in                       ),
      .cpllreset_in                            (cpllreset_in                           ),
      .dmonfiforeset_in                        (dmonfiforeset_in                       ),
      .dmonitorclk_in                          (dmonitorclk_in                         ),
      .drpaddr_in                              (drpaddr_in                             ),
      .drpclk_in                               (drpclk_in                              ),
      .drpdi_in                                (drpdi_in                               ),
      .drpen_in                                (drpen_in                               ),
      .drpwe_in                                (drpwe_in                               ),
      .evoddphicaldone_in                      (evoddphicaldone_in                     ),
      .evoddphicalstart_in                     (evoddphicalstart_in                    ),
      .evoddphidrden_in                        (evoddphidrden_in                       ),
      .evoddphidwren_in                        (evoddphidwren_in                       ),
      .evoddphixrden_in                        (evoddphixrden_in                       ),
      .evoddphixwren_in                        (evoddphixwren_in                       ),
      .eyescanmode_in                          (eyescanmode_in                         ),
      .eyescanreset_in                         (eyescanreset_in                        ),
      .eyescantrigger_in                       (eyescantrigger_in                      ),
      .gtgrefclk_in                            (gtgrefclk_in                           ),
      .gthrxn_in                               (gthrxn_in                              ),
      .gthrxp_in                               (gthrxp_in                              ),
      .gtnorthrefclk0_in                       (gtnorthrefclk0_in                      ),
      .gtnorthrefclk1_in                       (gtnorthrefclk1_in                      ),
      .gtrefclk0_in                            (gtrefclk0_in                           ),
      .gtrefclk1_in                            (gtrefclk1_in                           ),
      .gtresetsel_in                           (gtresetsel_in                          ),
      .gtrsvd_in                               (gtrsvd_in                              ),
      .gtrxreset_in                            (gtrxreset_in                           ),
      .gtsouthrefclk0_in                       (gtsouthrefclk0_in                      ),
      .gtsouthrefclk1_in                       (gtsouthrefclk1_in                      ),
      .gttxreset_in                            (gttxreset_in                           ),
      .loopback_in                             (loopback_in                            ),
      .lpbkrxtxseren_in                        (lpbkrxtxseren_in                       ),
      .lpbktxrxseren_in                        (lpbktxrxseren_in                       ),
      .pcieeqrxeqadaptdone_in                  (pcieeqrxeqadaptdone_in                 ),
      .pcierstidle_in                          (pcierstidle_in                         ),
      .pciersttxsyncstart_in                   (pciersttxsyncstart_in                  ),
      .pcieuserratedone_in                     (pcieuserratedone_in                    ),
      .pcsrsvdin_in                            (pcsrsvdin_in                           ),
      .pcsrsvdin2_in                           (pcsrsvdin2_in                          ),
      .pmarsvdin_in                            (pmarsvdin_in                           ),
      .qpll0clk_in                             (qpll0clk_in                            ),
      .qpll0refclk_in                          (qpll0refclk_in                         ),
      .qpll1clk_in                             (qpll1clk_in                            ),
      .qpll1refclk_in                          (qpll1refclk_in                         ),
      .resetovrd_in                            (resetovrd_in                           ),
      .rstclkentx_in                           (rstclkentx_in                          ),
      .rx8b10ben_in                            (rx8b10ben_in                           ),
      .rxbufreset_in                           (rxbufreset_in                          ),
      .rxcdrfreqreset_in                       (rxcdrfreqreset_in                      ),
      .rxcdrhold_in                            (rxcdrhold_in                           ),
      .rxcdrovrden_in                          (rxcdrovrden_in                         ),
      .rxcdrreset_in                           (rxcdrreset_in                          ),
      .rxcdrresetrsv_in                        (rxcdrresetrsv_in                       ),
      .rxchbonden_in                           (rxchbonden_in                          ),
      .rxchbondi_in                            (rxchbondi_in                           ),
      .rxchbondlevel_in                        (rxchbondlevel_in                       ),
      .rxchbondmaster_in                       (rxchbondmaster_in                      ),
      .rxchbondslave_in                        (rxchbondslave_in                       ),
      .rxcommadeten_in                         (rxcommadeten_in                        ),
      .rxdfeagcctrl_in                         (rxdfeagcctrl_in                        ),
      .rxdfeagchold_in                         (rxdfeagchold_in                        ),
      .rxdfeagcovrden_in                       (rxdfeagcovrden_in                      ),
      .rxdfelfhold_in                          (rxdfelfhold_in                         ),
      .rxdfelfovrden_in                        (rxdfelfovrden_in                       ),
      .rxdfelpmreset_in                        (rxdfelpmreset_in                       ),
      .rxdfetap10hold_in                       (rxdfetap10hold_in                      ),
      .rxdfetap10ovrden_in                     (rxdfetap10ovrden_in                    ),
      .rxdfetap11hold_in                       (rxdfetap11hold_in                      ),
      .rxdfetap11ovrden_in                     (rxdfetap11ovrden_in                    ),
      .rxdfetap12hold_in                       (rxdfetap12hold_in                      ),
      .rxdfetap12ovrden_in                     (rxdfetap12ovrden_in                    ),
      .rxdfetap13hold_in                       (rxdfetap13hold_in                      ),
      .rxdfetap13ovrden_in                     (rxdfetap13ovrden_in                    ),
      .rxdfetap14hold_in                       (rxdfetap14hold_in                      ),
      .rxdfetap14ovrden_in                     (rxdfetap14ovrden_in                    ),
      .rxdfetap15hold_in                       (rxdfetap15hold_in                      ),
      .rxdfetap15ovrden_in                     (rxdfetap15ovrden_in                    ),
      .rxdfetap2hold_in                        (rxdfetap2hold_in                       ),
      .rxdfetap2ovrden_in                      (rxdfetap2ovrden_in                     ),
      .rxdfetap3hold_in                        (rxdfetap3hold_in                       ),
      .rxdfetap3ovrden_in                      (rxdfetap3ovrden_in                     ),
      .rxdfetap4hold_in                        (rxdfetap4hold_in                       ),
      .rxdfetap4ovrden_in                      (rxdfetap4ovrden_in                     ),
      .rxdfetap5hold_in                        (rxdfetap5hold_in                       ),
      .rxdfetap5ovrden_in                      (rxdfetap5ovrden_in                     ),
      .rxdfetap6hold_in                        (rxdfetap6hold_in                       ),
      .rxdfetap6ovrden_in                      (rxdfetap6ovrden_in                     ),
      .rxdfetap7hold_in                        (rxdfetap7hold_in                       ),
      .rxdfetap7ovrden_in                      (rxdfetap7ovrden_in                     ),
      .rxdfetap8hold_in                        (rxdfetap8hold_in                       ),
      .rxdfetap8ovrden_in                      (rxdfetap8ovrden_in                     ),
      .rxdfetap9hold_in                        (rxdfetap9hold_in                       ),
      .rxdfetap9ovrden_in                      (rxdfetap9ovrden_in                     ),
      .rxdfeuthold_in                          (rxdfeuthold_in                         ),
      .rxdfeutovrden_in                        (rxdfeutovrden_in                       ),
      .rxdfevphold_in                          (rxdfevphold_in                         ),
      .rxdfevpovrden_in                        (rxdfevpovrden_in                       ),
      .rxdfevsen_in                            (rxdfevsen_in                           ),
      .rxdfexyden_in                           (rxdfexyden_in                          ),
      .rxdlybypass_in                          (rxdlybypass_in                         ),
      .rxdlyen_in                              (rxdlyen_in                             ),
      .rxdlyovrden_in                          (rxdlyovrden_in                         ),
      .rxdlysreset_in                          (rxdlysreset_in                         ),
      .rxelecidlemode_in                       (rxelecidlemode_in                      ),
      .rxgearboxslip_in                        (rxgearboxslip_in                       ),
      .rxlatclk_in                             (rxlatclk_in                            ),
      .rxlpmen_in                              (rxlpmen_in                             ),
      .rxlpmgchold_in                          (rxlpmgchold_in                         ),
      .rxlpmgcovrden_in                        (rxlpmgcovrden_in                       ),
      .rxlpmhfhold_in                          (rxlpmhfhold_in                         ),
      .rxlpmhfovrden_in                        (rxlpmhfovrden_in                       ),
      .rxlpmlfhold_in                          (rxlpmlfhold_in                         ),
      .rxlpmlfklovrden_in                      (rxlpmlfklovrden_in                     ),
      .rxlpmoshold_in                          (rxlpmoshold_in                         ),
      .rxlpmosovrden_in                        (rxlpmosovrden_in                       ),
      .rxmcommaalignen_in                      (rxmcommaalignen_in                     ),
      .rxmonitorsel_in                         (rxmonitorsel_in                        ),
      .rxoobreset_in                           (rxoobreset_in                          ),
      .rxoscalreset_in                         (rxoscalreset_in                        ),
      .rxoshold_in                             (rxoshold_in                            ),
      .rxosintcfg_in                           (rxosintcfg_in                          ),
      .rxosinten_in                            (rxosinten_in                           ),
      .rxosinthold_in                          (rxosinthold_in                         ),
      .rxosintovrden_in                        (rxosintovrden_in                       ),
      .rxosintstrobe_in                        (rxosintstrobe_in                       ),
      .rxosinttestovrden_in                    (rxosinttestovrden_in                   ),
      .rxosovrden_in                           (rxosovrden_in                          ),
      .rxoutclksel_in                          (rxoutclksel_in                         ),
      .rxpcommaalignen_in                      (rxpcommaalignen_in                     ),
      .rxpcsreset_in                           (rxpcsreset_in                          ),
      .rxpd_in                                 (rxpd_in                                ),
      .rxphalign_in                            (rxphalign_in                           ),
      .rxphalignen_in                          (rxphalignen_in                         ),
      .rxphdlypd_in                            (rxphdlypd_in                           ),
      .rxphdlyreset_in                         (rxphdlyreset_in                        ),
      .rxphovrden_in                           (rxphovrden_in                          ),
      .rxpllclksel_in                          (rxpllclksel_in                         ),
      .rxpmareset_in                           (rxpmareset_in                          ),
      .rxpolarity_in                           (rxpolarity_in                          ),
      .rxprbscntreset_in                       (rxprbscntreset_in                      ),
      .rxprbssel_in                            (rxprbssel_in                           ),
      .rxprogdivreset_in                       (rxprogdivreset_in                      ),
      .rxqpien_in                              (rxqpien_in                             ),
      .rxrate_in                               (rxrate_in                              ),
      .rxratemode_in                           (rxratemode_in                          ),
      .rxslide_in                              (rxslide_in                             ),
      .rxslipoutclk_in                         (rxslipoutclk_in                        ),
      .rxslippma_in                            (rxslippma_in                           ),
      .rxsyncallin_in                          (rxsyncallin_in                         ),
      .rxsyncin_in                             (rxsyncin_in                            ),
      .rxsyncmode_in                           (rxsyncmode_in                          ),
      .rxsysclksel_in                          (rxsysclksel_in                         ),
      .rxuserrdy_in                            (rxuserrdy_in                           ),
      .rxusrclk_in                             (rxusrclk_in                            ),
      .rxusrclk2_in                            (rxusrclk2_in                           ),
      .sigvalidclk_in                          (sigvalidclk_in                         ),
      .tstin_in                                (tstin_in                               ),
      .tx8b10bbypass_in                        (tx8b10bbypass_in                       ),
      .tx8b10ben_in                            (tx8b10ben_in                           ),
      .txbufdiffctrl_in                        (txbufdiffctrl_in                       ),
      .txcominit_in                            (txcominit_in                           ),
      .txcomsas_in                             (txcomsas_in                            ),
      .txcomwake_in                            (txcomwake_in                           ),
      .txctrl0_in                              (txctrl0_in                             ),
      .txctrl1_in                              (txctrl1_in                             ),
      .txctrl2_in                              (txctrl2_in                             ),
      .txdata_in                               (txdata_in                              ),
      .txdataextendrsvd_in                     (txdataextendrsvd_in                    ),
      .txdeemph_in                             (txdeemph_in                            ),
      .txdetectrx_in                           (txdetectrx_in                          ),
      .txdiffctrl_in                           (txdiffctrl_in                          ),
      .txdiffpd_in                             (txdiffpd_in                            ),
      .txdlybypass_in                          (txdlybypass_in                         ),
      .txdlyen_in                              (txdlyen_in                             ),
      .txdlyhold_in                            (txdlyhold_in                           ),
      .txdlyovrden_in                          (txdlyovrden_in                         ),
      .txdlysreset_in                          (txdlysreset_in                         ),
      .txdlyupdown_in                          (txdlyupdown_in                         ),
      .txelecidle_in                           (txelecidle_in                          ),
      .txheader_in                             (txheader_in                            ),
      .txinhibit_in                            (txinhibit_in                           ),
      .txlatclk_in                             (txlatclk_in                            ),
      .txmaincursor_in                         (txmaincursor_in                        ),
      .txmargin_in                             (txmargin_in                            ),
      .txoutclksel_in                          (txoutclksel_in                         ),
      .txpcsreset_in                           (txpcsreset_in                          ),
      .txpd_in                                 (txpd_in                                ),
      .txpdelecidlemode_in                     (txpdelecidlemode_in                    ),
      .txphalign_in                            (txphalign_in                           ),
      .txphalignen_in                          (txphalignen_in                         ),
      .txphdlypd_in                            (txphdlypd_in                           ),
      .txphdlyreset_in                         (txphdlyreset_in                        ),
      .txphdlytstclk_in                        (txphdlytstclk_in                       ),
      .txphinit_in                             (txphinit_in                            ),
      .txphovrden_in                           (txphovrden_in                          ),
      .txpippmen_in                            (txpippmen_in                           ),
      .txpippmovrden_in                        (txpippmovrden_in                       ),
      .txpippmpd_in                            (txpippmpd_in                           ),
      .txpippmsel_in                           (txpippmsel_in                          ),
      .txpippmstepsize_in                      (txpippmstepsize_in                     ),
      .txpisopd_in                             (txpisopd_in                            ),
      .txpllclksel_in                          (txpllclksel_in                         ),
      .txpmareset_in                           (txpmareset_in                          ),
      .txpolarity_in                           (txpolarity_in                          ),
      .txpostcursor_in                         (txpostcursor_in                        ),
      .txpostcursorinv_in                      (txpostcursorinv_in                     ),
      .txprbsforceerr_in                       (txprbsforceerr_in                      ),
      .txprbssel_in                            (txprbssel_in                           ),
      .txprecursor_in                          (txprecursor_in                         ),
      .txprecursorinv_in                       (txprecursorinv_in                      ),
      .txprogdivreset_in                       (txprogdivreset_in                      ),
      .txqpibiasen_in                          (txqpibiasen_in                         ),
      .txqpistrongpdown_in                     (txqpistrongpdown_in                    ),
      .txqpiweakpup_in                         (txqpiweakpup_in                        ),
      .txrate_in                               (txrate_in                              ),
      .txratemode_in                           (txratemode_in                          ),
      .txsequence_in                           (txsequence_in                          ),
      .txswing_in                              (txswing_in                             ),
      .txsyncallin_in                          (txsyncallin_in                         ),
      .txsyncin_in                             (txsyncin_in                            ),
      .txsyncmode_in                           (txsyncmode_in                          ),
      .txsysclksel_in                          (txsysclksel_in                         ),
      .txuserrdy_in                            (txuserrdy_in                           ),
      .txusrclk_in                             (txusrclk_in                            ),
      .txusrclk2_in                            (txusrclk2_in                           ),
      .bufgtce_out                             (bufgtce_out                            ),
      .bufgtcemask_out                         (bufgtcemask_out                        ),
      .bufgtdiv_out                            (bufgtdiv_out                           ),
      .bufgtreset_out                          (bufgtreset_out                         ),
      .bufgtrstmask_out                        (bufgtrstmask_out                       ),
      .cpllfbclklost_out                       (cpllfbclklost_out                      ),
      .cplllock_out                            (cplllock_out                           ),
      .cpllrefclklost_out                      (cpllrefclklost_out                     ),
      .dmonitorout_out                         (dmonitorout_out                        ),
      .drpdo_out                               (drpdo_out                              ),
      .drprdy_out                              (drprdy_out                             ),
      .eyescandataerror_out                    (eyescandataerror_out                   ),
      .gthtxn_out                              (gthtxn_out                             ),
      .gthtxp_out                              (gthtxp_out                             ),
      .gtpowergood_out                         (gtpowergood_out                        ),
      .gtrefclkmonitor_out                     (gtrefclkmonitor_out                    ),
      .pcierategen3_out                        (pcierategen3_out                       ),
      .pcierateidle_out                        (pcierateidle_out                       ),
      .pcierateqpllpd_out                      (pcierateqpllpd_out                     ),
      .pcierateqpllreset_out                   (pcierateqpllreset_out                  ),
      .pciesynctxsyncdone_out                  (pciesynctxsyncdone_out                 ),
      .pcieusergen3rdy_out                     (pcieusergen3rdy_out                    ),
      .pcieuserphystatusrst_out                (pcieuserphystatusrst_out               ),
      .pcieuserratestart_out                   (pcieuserratestart_out                  ),
      .pcsrsvdout_out                          (pcsrsvdout_out                         ),
      .phystatus_out                           (phystatus_out                          ),
      .pinrsrvdas_out                          (pinrsrvdas_out                         ),
      .resetexception_out                      (resetexception_out                     ),
      .rxbufstatus_out                         (rxbufstatus_out                        ),
      .rxbyteisaligned_out                     (rxbyteisaligned_out                    ),
      .rxbyterealign_out                       (rxbyterealign_out                      ),
      .rxcdrlock_out                           (rxcdrlock_out                          ),
      .rxcdrphdone_out                         (rxcdrphdone_out                        ),
      .rxchanbondseq_out                       (rxchanbondseq_out                      ),
      .rxchanisaligned_out                     (rxchanisaligned_out                    ),
      .rxchanrealign_out                       (rxchanrealign_out                      ),
      .rxchbondo_out                           (rxchbondo_out                          ),
      .rxclkcorcnt_out                         (rxclkcorcnt_out                        ),
      .rxcominitdet_out                        (rxcominitdet_out                       ),
      .rxcommadet_out                          (rxcommadet_out                         ),
      .rxcomsasdet_out                         (rxcomsasdet_out                        ),
      .rxcomwakedet_out                        (rxcomwakedet_out                       ),
      .rxctrl0_out                             (rxctrl0_out                            ),
      .rxctrl1_out                             (rxctrl1_out                            ),
      .rxctrl2_out                             (rxctrl2_out                            ),
      .rxctrl3_out                             (rxctrl3_out                            ),
      .rxdata_out                              (rxdata_out                             ),
      .rxdataextendrsvd_out                    (rxdataextendrsvd_out                   ),
      .rxdatavalid_out                         (rxdatavalid_out                        ),
      .rxdlysresetdone_out                     (rxdlysresetdone_out                    ),
      .rxelecidle_out                          (rxelecidle_out                         ),
      .rxheader_out                            (rxheader_out                           ),
      .rxheadervalid_out                       (rxheadervalid_out                      ),
      .rxmonitorout_out                        (rxmonitorout_out                       ),
      .rxosintdone_out                         (rxosintdone_out                        ),
      .rxosintstarted_out                      (rxosintstarted_out                     ),
      .rxosintstrobedone_out                   (rxosintstrobedone_out                  ),
      .rxosintstrobestarted_out                (rxosintstrobestarted_out               ),
      .rxoutclk_out                            (rxoutclk_out                           ),
      .rxoutclkfabric_out                      (rxoutclkfabric_out                     ),
      .rxoutclkpcs_out                         (rxoutclkpcs_out                        ),
      .rxphaligndone_out                       (rxphaligndone_out                      ),
      .rxphalignerr_out                        (rxphalignerr_out                       ),
      .rxpmaresetdone_out                      (rxpmaresetdone_out                     ),
      .rxprbserr_out                           (rxprbserr_out                          ),
      .rxprbslocked_out                        (rxprbslocked_out                       ),
      .rxprgdivresetdone_out                   (rxprgdivresetdone_out                  ),
      .rxqpisenn_out                           (rxqpisenn_out                          ),
      .rxqpisenp_out                           (rxqpisenp_out                          ),
      .rxratedone_out                          (rxratedone_out                         ),
      .rxrecclkout_out                         (rxrecclkout_out                        ),
      .rxresetdone_out                         (rxresetdone_out                        ),
      .rxsliderdy_out                          (rxsliderdy_out                         ),
      .rxslipdone_out                          (rxslipdone_out                         ),
      .rxslipoutclkrdy_out                     (rxslipoutclkrdy_out                    ),
      .rxslippmardy_out                        (rxslippmardy_out                       ),
      .rxstartofseq_out                        (rxstartofseq_out                       ),
      .rxstatus_out                            (rxstatus_out                           ),
      .rxsyncdone_out                          (rxsyncdone_out                         ),
      .rxsyncout_out                           (rxsyncout_out                          ),
      .rxvalid_out                             (rxvalid_out                            ),
      .txbufstatus_out                         (txbufstatus_out                        ),
      .txcomfinish_out                         (txcomfinish_out                        ),
      .txdlysresetdone_out                     (txdlysresetdone_out                    ),
      .txoutclk_out                            (txoutclk_out                           ),
      .txoutclkfabric_out                      (txoutclkfabric_out                     ),
      .txoutclkpcs_out                         (txoutclkpcs_out                        ),
      .txphaligndone_out                       (txphaligndone_out                      ),
      .txphinitdone_out                        (txphinitdone_out                       ),
      .txpmaresetdone_out                      (txpmaresetdone_out                     ),
      .txprgdivresetdone_out                   (txprgdivresetdone_out                  ),
      .txqpisenn_out                           (txqpisenn_out                          ),
      .txqpisenp_out                           (txqpisenp_out                          ),
      .txratedone_out                          (txratedone_out                         ),
      .txresetdone_out                         (txresetdone_out                        ),
      .txsyncdone_out                          (txsyncdone_out                         ),
      .txsyncout_out                           (txsyncout_out                          )
    );

    // Drive unused outputs to constant values
    assign rxrecclk0sel_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign rxrecclk1sel_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign sdm0finalout_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign sdm0testdata_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign sdm1finalout_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign sdm1testdata_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign tcongpo_out           = {{`ice_gth_t_SF_CM}{1'b0}};
    assign tconrsvdout0_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubdaddr_out           = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubden_out             = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubdi_out              = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubdwe_out             = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubmdmtdo_out          = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubrsvdout_out         = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubtxuart_out          = {{`ice_gth_t_SF_CM}{1'b0}};
    assign dmonitoroutclk_out    = {{`ice_gth_t_N_CH}{1'b0}};
    assign gtytxn_out            = {{`ice_gth_t_N_CH}{1'b0}};
    assign gtytxp_out            = {{`ice_gth_t_N_CH}{1'b0}};
    assign powerpresent_out      = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxckcaldone_out       = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxlfpstresetdet_out   = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxlfpsu2lpexitdet_out = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxlfpsu3wakedet_out   = {{`ice_gth_t_N_CH}{1'b0}};
    assign txdccdone_out         = {{`ice_gth_t_N_CH}{1'b0}};

  end
  else if (C_GT_TYPE == `ice_gth_t_GT_TYPE__GTYE3) begin : gen_gtwizard_gtye3_top

    // Generate GTYE3-type Transceivers Wizard submodule
    ice_gth_t_gtwizard_gtye3 #(
      .C_CHANNEL_ENABLE                          (C_CHANNEL_ENABLE                         ),
      .C_COMMON_SCALING_FACTOR                   (C_COMMON_SCALING_FACTOR                  ),
      .C_FREERUN_FREQUENCY                       (C_FREERUN_FREQUENCY                      ),
      .C_GT_REV                                  (C_GT_REV                                 ),
      .C_LOCATE_RESET_CONTROLLER                 (C_LOCATE_RESET_CONTROLLER                ),
      .C_LOCATE_USER_DATA_WIDTH_SIZING           (C_LOCATE_USER_DATA_WIDTH_SIZING          ),
      .C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER      (C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER     ),
      .C_LOCATE_RX_USER_CLOCKING                 (C_LOCATE_RX_USER_CLOCKING                ),
      .C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER      (C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER     ),
      .C_LOCATE_TX_USER_CLOCKING                 (C_LOCATE_TX_USER_CLOCKING                ),
      .C_RESET_CONTROLLER_INSTANCE_CTRL          (C_RESET_CONTROLLER_INSTANCE_CTRL         ),
      .C_RX_BUFFBYPASS_MODE                      (C_RX_BUFFBYPASS_MODE                     ),
      .C_RX_BUFFER_BYPASS_INSTANCE_CTRL          (C_RX_BUFFER_BYPASS_INSTANCE_CTRL         ),
      .C_RX_BUFFER_MODE                          (C_RX_BUFFER_MODE                         ),
      .C_RX_DATA_DECODING                        (C_RX_DATA_DECODING                       ),
      .C_RX_ENABLE                               (C_RX_ENABLE                              ),
      .C_RX_INT_DATA_WIDTH                       (C_RX_INT_DATA_WIDTH                      ),
      .C_RX_LINE_RATE                            (C_RX_LINE_RATE                           ),
      .C_RX_MASTER_CHANNEL_IDX                   (C_RX_MASTER_CHANNEL_IDX                  ),
      .C_RX_OUTCLK_BUFG_GT_DIV                   (C_RX_OUTCLK_BUFG_GT_DIV                  ),
      .C_RX_PLL_TYPE                             (C_RX_PLL_TYPE                            ),
      .C_RX_USER_CLOCKING_CONTENTS               (C_RX_USER_CLOCKING_CONTENTS              ),
      .C_RX_USER_CLOCKING_INSTANCE_CTRL          (C_RX_USER_CLOCKING_INSTANCE_CTRL         ),
      .C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 (C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2),
      .C_RX_USER_CLOCKING_SOURCE                 (C_RX_USER_CLOCKING_SOURCE                ),
      .C_RX_USER_DATA_WIDTH                      (C_RX_USER_DATA_WIDTH                     ),
      .C_TOTAL_NUM_CHANNELS                      (C_TOTAL_NUM_CHANNELS                     ),
      .C_TOTAL_NUM_COMMONS                       (C_TOTAL_NUM_COMMONS                      ),
      .C_TX_BUFFBYPASS_MODE                      (C_TX_BUFFBYPASS_MODE                     ),
      .C_TX_BUFFER_BYPASS_INSTANCE_CTRL          (C_TX_BUFFER_BYPASS_INSTANCE_CTRL         ),
      .C_TX_BUFFER_MODE                          (C_TX_BUFFER_MODE                         ),
      .C_TX_DATA_ENCODING                        (C_TX_DATA_ENCODING                       ),
      .C_TX_ENABLE                               (C_TX_ENABLE                              ),
      .C_TX_INT_DATA_WIDTH                       (C_TX_INT_DATA_WIDTH                      ),
      .C_TX_MASTER_CHANNEL_IDX                   (C_TX_MASTER_CHANNEL_IDX                  ),
      .C_TX_OUTCLK_BUFG_GT_DIV                   (C_TX_OUTCLK_BUFG_GT_DIV                  ),
      .C_TX_PLL_TYPE                             (C_TX_PLL_TYPE                            ),
      .C_TX_USER_CLOCKING_CONTENTS               (C_TX_USER_CLOCKING_CONTENTS              ),
      .C_TX_USER_CLOCKING_INSTANCE_CTRL          (C_TX_USER_CLOCKING_INSTANCE_CTRL         ),
      .C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 (C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2),
      .C_TX_USER_CLOCKING_SOURCE                 (C_TX_USER_CLOCKING_SOURCE                ),
      .C_TX_USER_DATA_WIDTH                      (C_TX_USER_DATA_WIDTH                     )
    ) ice_gth_t_gtwizard_gtye3_inst (
      .gtwiz_userclk_tx_reset_in          (gtwiz_userclk_tx_reset_in         ),
      .gtwiz_userclk_tx_active_in         (gtwiz_userclk_tx_active_in        ),
      .gtwiz_userclk_tx_srcclk_out        (gtwiz_userclk_tx_srcclk_out       ),
      .gtwiz_userclk_tx_usrclk_out        (gtwiz_userclk_tx_usrclk_out       ),
      .gtwiz_userclk_tx_usrclk2_out       (gtwiz_userclk_tx_usrclk2_out      ),
      .gtwiz_userclk_tx_active_out        (gtwiz_userclk_tx_active_out       ),
      .gtwiz_userclk_rx_reset_in          (gtwiz_userclk_rx_reset_in         ),
      .gtwiz_userclk_rx_active_in         (gtwiz_userclk_rx_active_in        ),
      .gtwiz_userclk_rx_srcclk_out        (gtwiz_userclk_rx_srcclk_out       ),
      .gtwiz_userclk_rx_usrclk_out        (gtwiz_userclk_rx_usrclk_out       ),
      .gtwiz_userclk_rx_usrclk2_out       (gtwiz_userclk_rx_usrclk2_out      ),
      .gtwiz_userclk_rx_active_out        (gtwiz_userclk_rx_active_out       ),
      .gtwiz_buffbypass_tx_reset_in       (gtwiz_buffbypass_tx_reset_in      ),
      .gtwiz_buffbypass_tx_start_user_in  (gtwiz_buffbypass_tx_start_user_in ),
      .gtwiz_buffbypass_tx_done_out       (gtwiz_buffbypass_tx_done_out      ),
      .gtwiz_buffbypass_tx_error_out      (gtwiz_buffbypass_tx_error_out     ),
      .gtwiz_buffbypass_rx_reset_in       (gtwiz_buffbypass_rx_reset_in      ),
      .gtwiz_buffbypass_rx_start_user_in  (gtwiz_buffbypass_rx_start_user_in ),
      .gtwiz_buffbypass_rx_done_out       (gtwiz_buffbypass_rx_done_out      ),
      .gtwiz_buffbypass_rx_error_out      (gtwiz_buffbypass_rx_error_out     ),
      .gtwiz_reset_clk_freerun_in         (gtwiz_reset_clk_freerun_in        ),
      .gtwiz_reset_all_in                 (gtwiz_reset_all_in                ),
      .gtwiz_reset_tx_pll_and_datapath_in (gtwiz_reset_tx_pll_and_datapath_in),
      .gtwiz_reset_tx_datapath_in         (gtwiz_reset_tx_datapath_in        ),
      .gtwiz_reset_rx_pll_and_datapath_in (gtwiz_reset_rx_pll_and_datapath_in),
      .gtwiz_reset_rx_datapath_in         (gtwiz_reset_rx_datapath_in        ),
      .gtwiz_reset_tx_done_in             (gtwiz_reset_tx_done_in            ),
      .gtwiz_reset_rx_done_in             (gtwiz_reset_rx_done_in            ),
      .gtwiz_reset_qpll0lock_in           (gtwiz_reset_qpll0lock_in          ),
      .gtwiz_reset_qpll1lock_in           (gtwiz_reset_qpll1lock_in          ),
      .gtwiz_reset_rx_cdr_stable_out      (gtwiz_reset_rx_cdr_stable_out     ),
      .gtwiz_reset_tx_done_out            (gtwiz_reset_tx_done_out           ),
      .gtwiz_reset_rx_done_out            (gtwiz_reset_rx_done_out           ),
      .gtwiz_reset_qpll0reset_out         (gtwiz_reset_qpll0reset_out        ),
      .gtwiz_reset_qpll1reset_out         (gtwiz_reset_qpll1reset_out        ),
      .gtwiz_userdata_tx_in               (gtwiz_userdata_tx_in              ),
      .gtwiz_userdata_rx_out              (gtwiz_userdata_rx_out             ),
      .bgbypassb_in                       (bgbypassb_in                      ),
      .bgmonitorenb_in                    (bgmonitorenb_in                   ),
      .bgpdb_in                           (bgpdb_in                          ),
      .bgrcalovrd_in                      (bgrcalovrd_in                     ),
      .bgrcalovrdenb_in                   (bgrcalovrdenb_in                  ),
      .drpaddr_common_in                  (drpaddr_common_in                 ),
      .drpclk_common_in                   (drpclk_common_in                  ),
      .drpdi_common_in                    (drpdi_common_in                   ),
      .drpen_common_in                    (drpen_common_in                   ),
      .drpwe_common_in                    (drpwe_common_in                   ),
      .gtgrefclk0_in                      (gtgrefclk0_in                     ),
      .gtgrefclk1_in                      (gtgrefclk1_in                     ),
      .gtnorthrefclk00_in                 (gtnorthrefclk00_in                ),
      .gtnorthrefclk01_in                 (gtnorthrefclk01_in                ),
      .gtnorthrefclk10_in                 (gtnorthrefclk10_in                ),
      .gtnorthrefclk11_in                 (gtnorthrefclk11_in                ),
      .gtrefclk00_in                      (gtrefclk00_in                     ),
      .gtrefclk01_in                      (gtrefclk01_in                     ),
      .gtrefclk10_in                      (gtrefclk10_in                     ),
      .gtrefclk11_in                      (gtrefclk11_in                     ),
      .gtsouthrefclk00_in                 (gtsouthrefclk00_in                ),
      .gtsouthrefclk01_in                 (gtsouthrefclk01_in                ),
      .gtsouthrefclk10_in                 (gtsouthrefclk10_in                ),
      .gtsouthrefclk11_in                 (gtsouthrefclk11_in                ),
      .pmarsvd0_in                        (pmarsvd0_in                       ),
      .pmarsvd1_in                        (pmarsvd1_in                       ),
      .qpll0clkrsvd0_in                   (qpll0clkrsvd0_in                  ),
      .qpll0lockdetclk_in                 (qpll0lockdetclk_in                ),
      .qpll0locken_in                     (qpll0locken_in                    ),
      .qpll0pd_in                         (qpll0pd_in                        ),
      .qpll0refclksel_in                  (qpll0refclksel_in                 ),
      .qpll0reset_in                      (qpll0reset_in                     ),
      .qpll1clkrsvd0_in                   (qpll1clkrsvd0_in                  ),
      .qpll1lockdetclk_in                 (qpll1lockdetclk_in                ),
      .qpll1locken_in                     (qpll1locken_in                    ),
      .qpll1pd_in                         (qpll1pd_in                        ),
      .qpll1refclksel_in                  (qpll1refclksel_in                 ),
      .qpll1reset_in                      (qpll1reset_in                     ),
      .qpllrsvd1_in                       (qpllrsvd1_in                      ),
      .qpllrsvd2_in                       (qpllrsvd2_in                      ),
      .qpllrsvd3_in                       (qpllrsvd3_in                      ),
      .qpllrsvd4_in                       (qpllrsvd4_in                      ),
      .rcalenb_in                         (rcalenb_in                        ),
      .sdm0data_in                        (sdm0data_in                       ),
      .sdm0reset_in                       (sdm0reset_in                      ),
      .sdm0width_in                       (sdm0width_in                      ),
      .sdm1data_in                        (sdm1data_in                       ),
      .sdm1reset_in                       (sdm1reset_in                      ),
      .sdm1width_in                       (sdm1width_in                      ),
      .drpdo_common_out                   (drpdo_common_out                  ),
      .drprdy_common_out                  (drprdy_common_out                 ),
      .pmarsvdout0_out                    (pmarsvdout0_out                   ),
      .pmarsvdout1_out                    (pmarsvdout1_out                   ),
      .qpll0fbclklost_out                 (qpll0fbclklost_out                ),
      .qpll0lock_out                      (qpll0lock_out                     ),
      .qpll0outclk_out                    (qpll0outclk_out                   ),
      .qpll0outrefclk_out                 (qpll0outrefclk_out                ),
      .qpll0refclklost_out                (qpll0refclklost_out               ),
      .qpll1fbclklost_out                 (qpll1fbclklost_out                ),
      .qpll1lock_out                      (qpll1lock_out                     ),
      .qpll1outclk_out                    (qpll1outclk_out                   ),
      .qpll1outrefclk_out                 (qpll1outrefclk_out                ),
      .qpll1refclklost_out                (qpll1refclklost_out               ),
      .qplldmonitor0_out                  (qplldmonitor0_out                 ),
      .qplldmonitor1_out                  (qplldmonitor1_out                 ),
      .refclkoutmonitor0_out              (refclkoutmonitor0_out             ),
      .refclkoutmonitor1_out              (refclkoutmonitor1_out             ),
      .rxrecclk0_sel_out                  (rxrecclk0_sel_out                 ),
      .rxrecclk1_sel_out                  (rxrecclk1_sel_out                 ),
      .sdm0finalout_out                   (sdm0finalout_out                  ),
      .sdm0testdata_out                   (sdm0testdata_out                  ),
      .sdm1finalout_out                   (sdm1finalout_out                  ),
      .sdm1testdata_out                   (sdm1testdata_out                  ),
      .cdrstepdir_in                      (cdrstepdir_in                     ),
      .cdrstepsq_in                       (cdrstepsq_in                      ),
      .cdrstepsx_in                       (cdrstepsx_in                      ),
      .cfgreset_in                        (cfgreset_in                       ),
      .clkrsvd0_in                        (clkrsvd0_in                       ),
      .clkrsvd1_in                        (clkrsvd1_in                       ),
      .cplllockdetclk_in                  (cplllockdetclk_in                 ),
      .cplllocken_in                      (cplllocken_in                     ),
      .cpllpd_in                          (cpllpd_in                         ),
      .cpllrefclksel_in                   (cpllrefclksel_in                  ),
      .cpllreset_in                       (cpllreset_in                      ),
      .dmonfiforeset_in                   (dmonfiforeset_in                  ),
      .dmonitorclk_in                     (dmonitorclk_in                    ),
      .drpaddr_in                         (drpaddr_in                        ),
      .drpclk_in                          (drpclk_in                         ),
      .drpdi_in                           (drpdi_in                          ),
      .drpen_in                           (drpen_in                          ),
      .drpwe_in                           (drpwe_in                          ),
      .elpcaldvorwren_in                  (elpcaldvorwren_in                 ),
      .elpcalpaorwren_in                  (elpcalpaorwren_in                 ),
      .evoddphicaldone_in                 (evoddphicaldone_in                ),
      .evoddphicalstart_in                (evoddphicalstart_in               ),
      .evoddphidrden_in                   (evoddphidrden_in                  ),
      .evoddphidwren_in                   (evoddphidwren_in                  ),
      .evoddphixrden_in                   (evoddphixrden_in                  ),
      .evoddphixwren_in                   (evoddphixwren_in                  ),
      .eyescanmode_in                     (eyescanmode_in                    ),
      .eyescanreset_in                    (eyescanreset_in                   ),
      .eyescantrigger_in                  (eyescantrigger_in                 ),
      .gtgrefclk_in                       (gtgrefclk_in                      ),
      .gtnorthrefclk0_in                  (gtnorthrefclk0_in                 ),
      .gtnorthrefclk1_in                  (gtnorthrefclk1_in                 ),
      .gtrefclk0_in                       (gtrefclk0_in                      ),
      .gtrefclk1_in                       (gtrefclk1_in                      ),
      .gtresetsel_in                      (gtresetsel_in                     ),
      .gtrsvd_in                          (gtrsvd_in                         ),
      .gtrxreset_in                       (gtrxreset_in                      ),
      .gtsouthrefclk0_in                  (gtsouthrefclk0_in                 ),
      .gtsouthrefclk1_in                  (gtsouthrefclk1_in                 ),
      .gttxreset_in                       (gttxreset_in                      ),
      .gtyrxn_in                          (gtyrxn_in                         ),
      .gtyrxp_in                          (gtyrxp_in                         ),
      .loopback_in                        (loopback_in                       ),
      .looprsvd_in                        (looprsvd_in                       ),
      .lpbkrxtxseren_in                   (lpbkrxtxseren_in                  ),
      .lpbktxrxseren_in                   (lpbktxrxseren_in                  ),
      .pcieeqrxeqadaptdone_in             (pcieeqrxeqadaptdone_in            ),
      .pcierstidle_in                     (pcierstidle_in                    ),
      .pciersttxsyncstart_in              (pciersttxsyncstart_in             ),
      .pcieuserratedone_in                (pcieuserratedone_in               ),
      .pcsrsvdin_in                       (pcsrsvdin_in                      ),
      .pcsrsvdin2_in                      (pcsrsvdin2_in                     ),
      .pmarsvdin_in                       (pmarsvdin_in                      ),
      .qpll0clk_in                        (qpll0clk_in                       ),
      .qpll0refclk_in                     (qpll0refclk_in                    ),
      .qpll1clk_in                        (qpll1clk_in                       ),
      .qpll1refclk_in                     (qpll1refclk_in                    ),
      .resetovrd_in                       (resetovrd_in                      ),
      .rstclkentx_in                      (rstclkentx_in                     ),
      .rx8b10ben_in                       (rx8b10ben_in                      ),
      .rxbufreset_in                      (rxbufreset_in                     ),
      .rxcdrfreqreset_in                  (rxcdrfreqreset_in                 ),
      .rxcdrhold_in                       (rxcdrhold_in                      ),
      .rxcdrovrden_in                     (rxcdrovrden_in                    ),
      .rxcdrreset_in                      (rxcdrreset_in                     ),
      .rxcdrresetrsv_in                   (rxcdrresetrsv_in                  ),
      .rxchbonden_in                      (rxchbonden_in                     ),
      .rxchbondi_in                       (rxchbondi_in                      ),
      .rxchbondlevel_in                   (rxchbondlevel_in                  ),
      .rxchbondmaster_in                  (rxchbondmaster_in                 ),
      .rxchbondslave_in                   (rxchbondslave_in                  ),
      .rxckcalreset_in                    (rxckcalreset_in                   ),
      .rxcommadeten_in                    (rxcommadeten_in                   ),
      .rxdccforcestart_in                 (rxdccforcestart_in                ),
      .rxdfeagchold_in                    (rxdfeagchold_in                   ),
      .rxdfeagcovrden_in                  (rxdfeagcovrden_in                 ),
      .rxdfelfhold_in                     (rxdfelfhold_in                    ),
      .rxdfelfovrden_in                   (rxdfelfovrden_in                  ),
      .rxdfelpmreset_in                   (rxdfelpmreset_in                  ),
      .rxdfetap10hold_in                  (rxdfetap10hold_in                 ),
      .rxdfetap10ovrden_in                (rxdfetap10ovrden_in               ),
      .rxdfetap11hold_in                  (rxdfetap11hold_in                 ),
      .rxdfetap11ovrden_in                (rxdfetap11ovrden_in               ),
      .rxdfetap12hold_in                  (rxdfetap12hold_in                 ),
      .rxdfetap12ovrden_in                (rxdfetap12ovrden_in               ),
      .rxdfetap13hold_in                  (rxdfetap13hold_in                 ),
      .rxdfetap13ovrden_in                (rxdfetap13ovrden_in               ),
      .rxdfetap14hold_in                  (rxdfetap14hold_in                 ),
      .rxdfetap14ovrden_in                (rxdfetap14ovrden_in               ),
      .rxdfetap15hold_in                  (rxdfetap15hold_in                 ),
      .rxdfetap15ovrden_in                (rxdfetap15ovrden_in               ),
      .rxdfetap2hold_in                   (rxdfetap2hold_in                  ),
      .rxdfetap2ovrden_in                 (rxdfetap2ovrden_in                ),
      .rxdfetap3hold_in                   (rxdfetap3hold_in                  ),
      .rxdfetap3ovrden_in                 (rxdfetap3ovrden_in                ),
      .rxdfetap4hold_in                   (rxdfetap4hold_in                  ),
      .rxdfetap4ovrden_in                 (rxdfetap4ovrden_in                ),
      .rxdfetap5hold_in                   (rxdfetap5hold_in                  ),
      .rxdfetap5ovrden_in                 (rxdfetap5ovrden_in                ),
      .rxdfetap6hold_in                   (rxdfetap6hold_in                  ),
      .rxdfetap6ovrden_in                 (rxdfetap6ovrden_in                ),
      .rxdfetap7hold_in                   (rxdfetap7hold_in                  ),
      .rxdfetap7ovrden_in                 (rxdfetap7ovrden_in                ),
      .rxdfetap8hold_in                   (rxdfetap8hold_in                  ),
      .rxdfetap8ovrden_in                 (rxdfetap8ovrden_in                ),
      .rxdfetap9hold_in                   (rxdfetap9hold_in                  ),
      .rxdfetap9ovrden_in                 (rxdfetap9ovrden_in                ),
      .rxdfeuthold_in                     (rxdfeuthold_in                    ),
      .rxdfeutovrden_in                   (rxdfeutovrden_in                  ),
      .rxdfevphold_in                     (rxdfevphold_in                    ),
      .rxdfevpovrden_in                   (rxdfevpovrden_in                  ),
      .rxdfevsen_in                       (rxdfevsen_in                      ),
      .rxdfexyden_in                      (rxdfexyden_in                     ),
      .rxdlybypass_in                     (rxdlybypass_in                    ),
      .rxdlyen_in                         (rxdlyen_in                        ),
      .rxdlyovrden_in                     (rxdlyovrden_in                    ),
      .rxdlysreset_in                     (rxdlysreset_in                    ),
      .rxelecidlemode_in                  (rxelecidlemode_in                 ),
      .rxgearboxslip_in                   (rxgearboxslip_in                  ),
      .rxlatclk_in                        (rxlatclk_in                       ),
      .rxlpmen_in                         (rxlpmen_in                        ),
      .rxlpmgchold_in                     (rxlpmgchold_in                    ),
      .rxlpmgcovrden_in                   (rxlpmgcovrden_in                  ),
      .rxlpmhfhold_in                     (rxlpmhfhold_in                    ),
      .rxlpmhfovrden_in                   (rxlpmhfovrden_in                  ),
      .rxlpmlfhold_in                     (rxlpmlfhold_in                    ),
      .rxlpmlfklovrden_in                 (rxlpmlfklovrden_in                ),
      .rxlpmoshold_in                     (rxlpmoshold_in                    ),
      .rxlpmosovrden_in                   (rxlpmosovrden_in                  ),
      .rxmcommaalignen_in                 (rxmcommaalignen_in                ),
      .rxmonitorsel_in                    (rxmonitorsel_in                   ),
      .rxoobreset_in                      (rxoobreset_in                     ),
      .rxoscalreset_in                    (rxoscalreset_in                   ),
      .rxoshold_in                        (rxoshold_in                       ),
      .rxosintcfg_in                      (rxosintcfg_in                     ),
      .rxosinten_in                       (rxosinten_in                      ),
      .rxosinthold_in                     (rxosinthold_in                    ),
      .rxosintovrden_in                   (rxosintovrden_in                  ),
      .rxosintstrobe_in                   (rxosintstrobe_in                  ),
      .rxosinttestovrden_in               (rxosinttestovrden_in              ),
      .rxosovrden_in                      (rxosovrden_in                     ),
      .rxoutclksel_in                     (rxoutclksel_in                    ),
      .rxpcommaalignen_in                 (rxpcommaalignen_in                ),
      .rxpcsreset_in                      (rxpcsreset_in                     ),
      .rxpd_in                            (rxpd_in                           ),
      .rxphalign_in                       (rxphalign_in                      ),
      .rxphalignen_in                     (rxphalignen_in                    ),
      .rxphdlypd_in                       (rxphdlypd_in                      ),
      .rxphdlyreset_in                    (rxphdlyreset_in                   ),
      .rxphovrden_in                      (rxphovrden_in                     ),
      .rxpllclksel_in                     (rxpllclksel_in                    ),
      .rxpmareset_in                      (rxpmareset_in                     ),
      .rxpolarity_in                      (rxpolarity_in                     ),
      .rxprbscntreset_in                  (rxprbscntreset_in                 ),
      .rxprbssel_in                       (rxprbssel_in                      ),
      .rxprogdivreset_in                  (rxprogdivreset_in                 ),
      .rxrate_in                          (rxrate_in                         ),
      .rxratemode_in                      (rxratemode_in                     ),
      .rxslide_in                         (rxslide_in                        ),
      .rxslipoutclk_in                    (rxslipoutclk_in                   ),
      .rxslippma_in                       (rxslippma_in                      ),
      .rxsyncallin_in                     (rxsyncallin_in                    ),
      .rxsyncin_in                        (rxsyncin_in                       ),
      .rxsyncmode_in                      (rxsyncmode_in                     ),
      .rxsysclksel_in                     (rxsysclksel_in                    ),
      .rxuserrdy_in                       (rxuserrdy_in                      ),
      .rxusrclk_in                        (rxusrclk_in                       ),
      .rxusrclk2_in                       (rxusrclk2_in                      ),
      .sigvalidclk_in                     (sigvalidclk_in                    ),
      .tstin_in                           (tstin_in                          ),
      .tx8b10bbypass_in                   (tx8b10bbypass_in                  ),
      .tx8b10ben_in                       (tx8b10ben_in                      ),
      .txbufdiffctrl_in                   (txbufdiffctrl_in                  ),
      .txcominit_in                       (txcominit_in                      ),
      .txcomsas_in                        (txcomsas_in                       ),
      .txcomwake_in                       (txcomwake_in                      ),
      .txctrl0_in                         (txctrl0_in                        ),
      .txctrl1_in                         (txctrl1_in                        ),
      .txctrl2_in                         (txctrl2_in                        ),
      .txdata_in                          (txdata_in                         ),
      .txdataextendrsvd_in                (txdataextendrsvd_in               ),
      .txdccforcestart_in                 (txdccforcestart_in                ),
      .txdccreset_in                      (txdccreset_in                     ),
      .txdeemph_in                        (txdeemph_in                       ),
      .txdetectrx_in                      (txdetectrx_in                     ),
      .txdiffctrl_in                      (txdiffctrl_in                     ),
      .txdiffpd_in                        (txdiffpd_in                       ),
      .txdlybypass_in                     (txdlybypass_in                    ),
      .txdlyen_in                         (txdlyen_in                        ),
      .txdlyhold_in                       (txdlyhold_in                      ),
      .txdlyovrden_in                     (txdlyovrden_in                    ),
      .txdlysreset_in                     (txdlysreset_in                    ),
      .txdlyupdown_in                     (txdlyupdown_in                    ),
      .txelecidle_in                      (txelecidle_in                     ),
      .txelforcestart_in                  (txelforcestart_in                 ),
      .txheader_in                        (txheader_in                       ),
      .txinhibit_in                       (txinhibit_in                      ),
      .txlatclk_in                        (txlatclk_in                       ),
      .txmaincursor_in                    (txmaincursor_in                   ),
      .txmargin_in                        (txmargin_in                       ),
      .txoutclksel_in                     (txoutclksel_in                    ),
      .txpcsreset_in                      (txpcsreset_in                     ),
      .txpd_in                            (txpd_in                           ),
      .txpdelecidlemode_in                (txpdelecidlemode_in               ),
      .txphalign_in                       (txphalign_in                      ),
      .txphalignen_in                     (txphalignen_in                    ),
      .txphdlypd_in                       (txphdlypd_in                      ),
      .txphdlyreset_in                    (txphdlyreset_in                   ),
      .txphdlytstclk_in                   (txphdlytstclk_in                  ),
      .txphinit_in                        (txphinit_in                       ),
      .txphovrden_in                      (txphovrden_in                     ),
      .txpippmen_in                       (txpippmen_in                      ),
      .txpippmovrden_in                   (txpippmovrden_in                  ),
      .txpippmpd_in                       (txpippmpd_in                      ),
      .txpippmsel_in                      (txpippmsel_in                     ),
      .txpippmstepsize_in                 (txpippmstepsize_in                ),
      .txpisopd_in                        (txpisopd_in                       ),
      .txpllclksel_in                     (txpllclksel_in                    ),
      .txpmareset_in                      (txpmareset_in                     ),
      .txpolarity_in                      (txpolarity_in                     ),
      .txpostcursor_in                    (txpostcursor_in                   ),
      .txprbsforceerr_in                  (txprbsforceerr_in                 ),
      .txprbssel_in                       (txprbssel_in                      ),
      .txprecursor_in                     (txprecursor_in                    ),
      .txprogdivreset_in                  (txprogdivreset_in                 ),
      .txrate_in                          (txrate_in                         ),
      .txratemode_in                      (txratemode_in                     ),
      .txsequence_in                      (txsequence_in                     ),
      .txswing_in                         (txswing_in                        ),
      .txsyncallin_in                     (txsyncallin_in                    ),
      .txsyncin_in                        (txsyncin_in                       ),
      .txsyncmode_in                      (txsyncmode_in                     ),
      .txsysclksel_in                     (txsysclksel_in                    ),
      .txuserrdy_in                       (txuserrdy_in                      ),
      .txusrclk_in                        (txusrclk_in                       ),
      .txusrclk2_in                       (txusrclk2_in                      ),
      .bufgtce_out                        (bufgtce_out                       ),
      .bufgtcemask_out                    (bufgtcemask_out                   ),
      .bufgtdiv_out                       (bufgtdiv_out                      ),
      .bufgtreset_out                     (bufgtreset_out                    ),
      .bufgtrstmask_out                   (bufgtrstmask_out                  ),
      .cpllfbclklost_out                  (cpllfbclklost_out                 ),
      .cplllock_out                       (cplllock_out                      ),
      .cpllrefclklost_out                 (cpllrefclklost_out                ),
      .dmonitorout_out                    (dmonitorout_out                   ),
      .drpdo_out                          (drpdo_out                         ),
      .drprdy_out                         (drprdy_out                        ),
      .eyescandataerror_out               (eyescandataerror_out              ),
      .gtpowergood_out                    (gtpowergood_out                   ),
      .gtrefclkmonitor_out                (gtrefclkmonitor_out               ),
      .gtytxn_out                         (gtytxn_out                        ),
      .gtytxp_out                         (gtytxp_out                        ),
      .pcierategen3_out                   (pcierategen3_out                  ),
      .pcierateidle_out                   (pcierateidle_out                  ),
      .pcierateqpllpd_out                 (pcierateqpllpd_out                ),
      .pcierateqpllreset_out              (pcierateqpllreset_out             ),
      .pciesynctxsyncdone_out             (pciesynctxsyncdone_out            ),
      .pcieusergen3rdy_out                (pcieusergen3rdy_out               ),
      .pcieuserphystatusrst_out           (pcieuserphystatusrst_out          ),
      .pcieuserratestart_out              (pcieuserratestart_out             ),
      .pcsrsvdout_out                     (pcsrsvdout_out                    ),
      .phystatus_out                      (phystatus_out                     ),
      .pinrsrvdas_out                     (pinrsrvdas_out                    ),
      .resetexception_out                 (resetexception_out                ),
      .rxbufstatus_out                    (rxbufstatus_out                   ),
      .rxbyteisaligned_out                (rxbyteisaligned_out               ),
      .rxbyterealign_out                  (rxbyterealign_out                 ),
      .rxcdrlock_out                      (rxcdrlock_out                     ),
      .rxcdrphdone_out                    (rxcdrphdone_out                   ),
      .rxchanbondseq_out                  (rxchanbondseq_out                 ),
      .rxchanisaligned_out                (rxchanisaligned_out               ),
      .rxchanrealign_out                  (rxchanrealign_out                 ),
      .rxchbondo_out                      (rxchbondo_out                     ),
      .rxckcaldone_out                    (rxckcaldone_out                   ),
      .rxclkcorcnt_out                    (rxclkcorcnt_out                   ),
      .rxcominitdet_out                   (rxcominitdet_out                  ),
      .rxcommadet_out                     (rxcommadet_out                    ),
      .rxcomsasdet_out                    (rxcomsasdet_out                   ),
      .rxcomwakedet_out                   (rxcomwakedet_out                  ),
      .rxctrl0_out                        (rxctrl0_out                       ),
      .rxctrl1_out                        (rxctrl1_out                       ),
      .rxctrl2_out                        (rxctrl2_out                       ),
      .rxctrl3_out                        (rxctrl3_out                       ),
      .rxdata_out                         (rxdata_out                        ),
      .rxdataextendrsvd_out               (rxdataextendrsvd_out              ),
      .rxdatavalid_out                    (rxdatavalid_out                   ),
      .rxdlysresetdone_out                (rxdlysresetdone_out               ),
      .rxelecidle_out                     (rxelecidle_out                    ),
      .rxheader_out                       (rxheader_out                      ),
      .rxheadervalid_out                  (rxheadervalid_out                 ),
      .rxmonitorout_out                   (rxmonitorout_out                  ),
      .rxosintdone_out                    (rxosintdone_out                   ),
      .rxosintstarted_out                 (rxosintstarted_out                ),
      .rxosintstrobedone_out              (rxosintstrobedone_out             ),
      .rxosintstrobestarted_out           (rxosintstrobestarted_out          ),
      .rxoutclk_out                       (rxoutclk_out                      ),
      .rxoutclkfabric_out                 (rxoutclkfabric_out                ),
      .rxoutclkpcs_out                    (rxoutclkpcs_out                   ),
      .rxphaligndone_out                  (rxphaligndone_out                 ),
      .rxphalignerr_out                   (rxphalignerr_out                  ),
      .rxpmaresetdone_out                 (rxpmaresetdone_out                ),
      .rxprbserr_out                      (rxprbserr_out                     ),
      .rxprbslocked_out                   (rxprbslocked_out                  ),
      .rxprgdivresetdone_out              (rxprgdivresetdone_out             ),
      .rxratedone_out                     (rxratedone_out                    ),
      .rxrecclkout_out                    (rxrecclkout_out                   ),
      .rxresetdone_out                    (rxresetdone_out                   ),
      .rxsliderdy_out                     (rxsliderdy_out                    ),
      .rxslipdone_out                     (rxslipdone_out                    ),
      .rxslipoutclkrdy_out                (rxslipoutclkrdy_out               ),
      .rxslippmardy_out                   (rxslippmardy_out                  ),
      .rxstartofseq_out                   (rxstartofseq_out                  ),
      .rxstatus_out                       (rxstatus_out                      ),
      .rxsyncdone_out                     (rxsyncdone_out                    ),
      .rxsyncout_out                      (rxsyncout_out                     ),
      .rxvalid_out                        (rxvalid_out                       ),
      .txbufstatus_out                    (txbufstatus_out                   ),
      .txcomfinish_out                    (txcomfinish_out                   ),
      .txdccdone_out                      (txdccdone_out                     ),
      .txdlysresetdone_out                (txdlysresetdone_out               ),
      .txoutclk_out                       (txoutclk_out                      ),
      .txoutclkfabric_out                 (txoutclkfabric_out                ),
      .txoutclkpcs_out                    (txoutclkpcs_out                   ),
      .txphaligndone_out                  (txphaligndone_out                 ),
      .txphinitdone_out                   (txphinitdone_out                  ),
      .txpmaresetdone_out                 (txpmaresetdone_out                ),
      .txprgdivresetdone_out              (txprgdivresetdone_out             ),
      .txratedone_out                     (txratedone_out                    ),
      .txresetdone_out                    (txresetdone_out                   ),
      .txsyncdone_out                     (txsyncdone_out                    ),
      .txsyncout_out                      (txsyncout_out                     )
    );

    // Drive unused outputs to constant values
    assign rxrecclk0sel_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign rxrecclk1sel_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign tcongpo_out           = {{`ice_gth_t_SF_CM}{1'b0}};
    assign tconrsvdout0_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubdaddr_out           = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubden_out             = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubdi_out              = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubdwe_out             = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubmdmtdo_out          = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubrsvdout_out         = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubtxuart_out          = {{`ice_gth_t_SF_CM}{1'b0}};
    assign dmonitoroutclk_out    = {{`ice_gth_t_N_CH}{1'b0}};
    assign gthtxn_out            = {{`ice_gth_t_N_CH}{1'b0}};
    assign gthtxp_out            = {{`ice_gth_t_N_CH}{1'b0}};
    assign powerpresent_out      = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxlfpstresetdet_out   = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxlfpsu2lpexitdet_out = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxlfpsu3wakedet_out   = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxqpisenn_out         = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxqpisenp_out         = {{`ice_gth_t_N_CH}{1'b0}};
    assign txqpisenn_out         = {{`ice_gth_t_N_CH}{1'b0}};
    assign txqpisenp_out         = {{`ice_gth_t_N_CH}{1'b0}};

  end
  else if (C_GT_TYPE == `ice_gth_t_GT_TYPE__GTHE4) begin : gen_gtwizard_gthe4_top

    // Generate GTHE4-type Transceivers Wizard submodule
    ice_gth_t_gtwizard_gthe4 #(
      .C_CHANNEL_ENABLE                          (C_CHANNEL_ENABLE                         ),
      .C_COMMON_SCALING_FACTOR                   (C_COMMON_SCALING_FACTOR                  ),
      .C_FREERUN_FREQUENCY                       (C_FREERUN_FREQUENCY                      ),
      .C_LOCATE_RESET_CONTROLLER                 (C_LOCATE_RESET_CONTROLLER                ),
      .C_LOCATE_USER_DATA_WIDTH_SIZING           (C_LOCATE_USER_DATA_WIDTH_SIZING          ),
      .C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER      (C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER     ),
      .C_LOCATE_RX_USER_CLOCKING                 (C_LOCATE_RX_USER_CLOCKING                ),
      .C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER      (C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER     ),
      .C_LOCATE_TX_USER_CLOCKING                 (C_LOCATE_TX_USER_CLOCKING                ),
      .C_RESET_CONTROLLER_INSTANCE_CTRL          (C_RESET_CONTROLLER_INSTANCE_CTRL         ),
      .C_RX_BUFFBYPASS_MODE                      (C_RX_BUFFBYPASS_MODE                     ),
      .C_RX_BUFFER_BYPASS_INSTANCE_CTRL          (C_RX_BUFFER_BYPASS_INSTANCE_CTRL         ),
      .C_RX_BUFFER_MODE                          (C_RX_BUFFER_MODE                         ),
      .C_RX_DATA_DECODING                        (C_RX_DATA_DECODING                       ),
      .C_RX_ENABLE                               (C_RX_ENABLE                              ),
      .C_RX_INT_DATA_WIDTH                       (C_RX_INT_DATA_WIDTH                      ),
      .C_RX_LINE_RATE                            (C_RX_LINE_RATE                           ),
      .C_RX_MASTER_CHANNEL_IDX                   (C_RX_MASTER_CHANNEL_IDX                  ),
      .C_RX_OUTCLK_BUFG_GT_DIV                   (C_RX_OUTCLK_BUFG_GT_DIV                  ),
      .C_RX_PLL_TYPE                             (C_RX_PLL_TYPE                            ),
      .C_RX_USER_CLOCKING_CONTENTS               (C_RX_USER_CLOCKING_CONTENTS              ),
      .C_RX_USER_CLOCKING_INSTANCE_CTRL          (C_RX_USER_CLOCKING_INSTANCE_CTRL         ),
      .C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 (C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2),
      .C_RX_USER_CLOCKING_SOURCE                 (C_RX_USER_CLOCKING_SOURCE                ),
      .C_RX_USER_DATA_WIDTH                      (C_RX_USER_DATA_WIDTH                     ),
      .C_TOTAL_NUM_CHANNELS                      (C_TOTAL_NUM_CHANNELS                     ),
      .C_TOTAL_NUM_COMMONS                       (C_TOTAL_NUM_COMMONS                      ),
      .C_TX_BUFFBYPASS_MODE                      (C_TX_BUFFBYPASS_MODE                     ),
      .C_TX_BUFFER_BYPASS_INSTANCE_CTRL          (C_TX_BUFFER_BYPASS_INSTANCE_CTRL         ),
      .C_TX_BUFFER_MODE                          (C_TX_BUFFER_MODE                         ),
      .C_TX_DATA_ENCODING                        (C_TX_DATA_ENCODING                       ),
      .C_TX_ENABLE                               (C_TX_ENABLE                              ),
      .C_TX_INT_DATA_WIDTH                       (C_TX_INT_DATA_WIDTH                      ),
      .C_TX_MASTER_CHANNEL_IDX                   (C_TX_MASTER_CHANNEL_IDX                  ),
      .C_TX_OUTCLK_BUFG_GT_DIV                   (C_TX_OUTCLK_BUFG_GT_DIV                  ),
      .C_TX_PLL_TYPE                             (C_TX_PLL_TYPE                            ),
      .C_TX_USER_CLOCKING_CONTENTS               (C_TX_USER_CLOCKING_CONTENTS              ),
      .C_TX_USER_CLOCKING_INSTANCE_CTRL          (C_TX_USER_CLOCKING_INSTANCE_CTRL         ),
      .C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 (C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2),
      .C_TX_USER_CLOCKING_SOURCE                 (C_TX_USER_CLOCKING_SOURCE                ),
      .C_TX_USER_DATA_WIDTH                      (C_TX_USER_DATA_WIDTH                     )
    ) ice_gth_t_gtwizard_gthe4_inst (
      .gtwiz_userclk_tx_reset_in          (gtwiz_userclk_tx_reset_in         ),
      .gtwiz_userclk_tx_active_in         (gtwiz_userclk_tx_active_in        ),
      .gtwiz_userclk_tx_srcclk_out        (gtwiz_userclk_tx_srcclk_out       ),
      .gtwiz_userclk_tx_usrclk_out        (gtwiz_userclk_tx_usrclk_out       ),
      .gtwiz_userclk_tx_usrclk2_out       (gtwiz_userclk_tx_usrclk2_out      ),
      .gtwiz_userclk_tx_active_out        (gtwiz_userclk_tx_active_out       ),
      .gtwiz_userclk_rx_reset_in          (gtwiz_userclk_rx_reset_in         ),
      .gtwiz_userclk_rx_active_in         (gtwiz_userclk_rx_active_in        ),
      .gtwiz_userclk_rx_srcclk_out        (gtwiz_userclk_rx_srcclk_out       ),
      .gtwiz_userclk_rx_usrclk_out        (gtwiz_userclk_rx_usrclk_out       ),
      .gtwiz_userclk_rx_usrclk2_out       (gtwiz_userclk_rx_usrclk2_out      ),
      .gtwiz_userclk_rx_active_out        (gtwiz_userclk_rx_active_out       ),
      .gtwiz_buffbypass_tx_reset_in       (gtwiz_buffbypass_tx_reset_in      ),
      .gtwiz_buffbypass_tx_start_user_in  (gtwiz_buffbypass_tx_start_user_in ),
      .gtwiz_buffbypass_tx_done_out       (gtwiz_buffbypass_tx_done_out      ),
      .gtwiz_buffbypass_tx_error_out      (gtwiz_buffbypass_tx_error_out     ),
      .gtwiz_buffbypass_rx_reset_in       (gtwiz_buffbypass_rx_reset_in      ),
      .gtwiz_buffbypass_rx_start_user_in  (gtwiz_buffbypass_rx_start_user_in ),
      .gtwiz_buffbypass_rx_done_out       (gtwiz_buffbypass_rx_done_out      ),
      .gtwiz_buffbypass_rx_error_out      (gtwiz_buffbypass_rx_error_out     ),
      .gtwiz_reset_clk_freerun_in         (gtwiz_reset_clk_freerun_in        ),
      .gtwiz_reset_all_in                 (gtwiz_reset_all_in                ),
      .gtwiz_reset_tx_pll_and_datapath_in (gtwiz_reset_tx_pll_and_datapath_in),
      .gtwiz_reset_tx_datapath_in         (gtwiz_reset_tx_datapath_in        ),
      .gtwiz_reset_rx_pll_and_datapath_in (gtwiz_reset_rx_pll_and_datapath_in),
      .gtwiz_reset_rx_datapath_in         (gtwiz_reset_rx_datapath_in        ),
      .gtwiz_reset_tx_done_in             (gtwiz_reset_tx_done_in            ),
      .gtwiz_reset_rx_done_in             (gtwiz_reset_rx_done_in            ),
      .gtwiz_reset_qpll0lock_in           (gtwiz_reset_qpll0lock_in          ),
      .gtwiz_reset_qpll1lock_in           (gtwiz_reset_qpll1lock_in          ),
      .gtwiz_reset_rx_cdr_stable_out      (gtwiz_reset_rx_cdr_stable_out     ),
      .gtwiz_reset_tx_done_out            (gtwiz_reset_tx_done_out           ),
      .gtwiz_reset_rx_done_out            (gtwiz_reset_rx_done_out           ),
      .gtwiz_reset_qpll0reset_out         (gtwiz_reset_qpll0reset_out        ),
      .gtwiz_reset_qpll1reset_out         (gtwiz_reset_qpll1reset_out        ),
      .gtwiz_userdata_tx_in               (gtwiz_userdata_tx_in              ),
      .gtwiz_userdata_rx_out              (gtwiz_userdata_rx_out             ),
      .bgbypassb_in                       (bgbypassb_in                      ),
      .bgmonitorenb_in                    (bgmonitorenb_in                   ),
      .bgpdb_in                           (bgpdb_in                          ),
      .bgrcalovrd_in                      (bgrcalovrd_in                     ),
      .bgrcalovrdenb_in                   (bgrcalovrdenb_in                  ),
      .drpaddr_common_in                  (drpaddr_common_in                 ),
      .drpclk_common_in                   (drpclk_common_in                  ),
      .drpdi_common_in                    (drpdi_common_in                   ),
      .drpen_common_in                    (drpen_common_in                   ),
      .drpwe_common_in                    (drpwe_common_in                   ),
      .gtgrefclk0_in                      (gtgrefclk0_in                     ),
      .gtgrefclk1_in                      (gtgrefclk1_in                     ),
      .gtnorthrefclk00_in                 (gtnorthrefclk00_in                ),
      .gtnorthrefclk01_in                 (gtnorthrefclk01_in                ),
      .gtnorthrefclk10_in                 (gtnorthrefclk10_in                ),
      .gtnorthrefclk11_in                 (gtnorthrefclk11_in                ),
      .gtrefclk00_in                      (gtrefclk00_in                     ),
      .gtrefclk01_in                      (gtrefclk01_in                     ),
      .gtrefclk10_in                      (gtrefclk10_in                     ),
      .gtrefclk11_in                      (gtrefclk11_in                     ),
      .gtsouthrefclk00_in                 (gtsouthrefclk00_in                ),
      .gtsouthrefclk01_in                 (gtsouthrefclk01_in                ),
      .gtsouthrefclk10_in                 (gtsouthrefclk10_in                ),
      .gtsouthrefclk11_in                 (gtsouthrefclk11_in                ),
      .pcierateqpll0_in                   (pcierateqpll0_in                  ),
      .pcierateqpll1_in                   (pcierateqpll1_in                  ),
      .pmarsvd0_in                        (pmarsvd0_in                       ),
      .pmarsvd1_in                        (pmarsvd1_in                       ),
      .qpll0clkrsvd0_in                   (qpll0clkrsvd0_in                  ),
      .qpll0clkrsvd1_in                   (qpll0clkrsvd1_in                  ),
      .qpll0fbdiv_in                      (qpll0fbdiv_in                     ),
      .qpll0lockdetclk_in                 (qpll0lockdetclk_in                ),
      .qpll0locken_in                     (qpll0locken_in                    ),
      .qpll0pd_in                         (qpll0pd_in                        ),
      .qpll0refclksel_in                  (qpll0refclksel_in                 ),
      .qpll0reset_in                      (qpll0reset_in                     ),
      .qpll1clkrsvd0_in                   (qpll1clkrsvd0_in                  ),
      .qpll1clkrsvd1_in                   (qpll1clkrsvd1_in                  ),
      .qpll1fbdiv_in                      (qpll1fbdiv_in                     ),
      .qpll1lockdetclk_in                 (qpll1lockdetclk_in                ),
      .qpll1locken_in                     (qpll1locken_in                    ),
      .qpll1pd_in                         (qpll1pd_in                        ),
      .qpll1refclksel_in                  (qpll1refclksel_in                 ),
      .qpll1reset_in                      (qpll1reset_in                     ),
      .qpllrsvd1_in                       (qpllrsvd1_in                      ),
      .qpllrsvd2_in                       (qpllrsvd2_in                      ),
      .qpllrsvd3_in                       (qpllrsvd3_in                      ),
      .qpllrsvd4_in                       (qpllrsvd4_in                      ),
      .rcalenb_in                         (rcalenb_in                        ),
      .sdm0data_in                        (sdm0data_in                       ),
      .sdm0reset_in                       (sdm0reset_in                      ),
      .sdm0toggle_in                      (sdm0toggle_in                     ),
      .sdm0width_in                       (sdm0width_in                      ),
      .sdm1data_in                        (sdm1data_in                       ),
      .sdm1reset_in                       (sdm1reset_in                      ),
      .sdm1toggle_in                      (sdm1toggle_in                     ),
      .sdm1width_in                       (sdm1width_in                      ),
      .tcongpi_in                         (tcongpi_in                        ),
      .tconpowerup_in                     (tconpowerup_in                    ),
      .tconreset_in                       (tconreset_in                      ),
      .tconrsvdin1_in                     (tconrsvdin1_in                    ),
      .drpdo_common_out                   (drpdo_common_out                  ),
      .drprdy_common_out                  (drprdy_common_out                 ),
      .pmarsvdout0_out                    (pmarsvdout0_out                   ),
      .pmarsvdout1_out                    (pmarsvdout1_out                   ),
      .qpll0fbclklost_out                 (qpll0fbclklost_out                ),
      .qpll0lock_out                      (qpll0lock_out                     ),
      .qpll0outclk_out                    (qpll0outclk_out                   ),
      .qpll0outrefclk_out                 (qpll0outrefclk_out                ),
      .qpll0refclklost_out                (qpll0refclklost_out               ),
      .qpll1fbclklost_out                 (qpll1fbclklost_out                ),
      .qpll1lock_out                      (qpll1lock_out                     ),
      .qpll1outclk_out                    (qpll1outclk_out                   ),
      .qpll1outrefclk_out                 (qpll1outrefclk_out                ),
      .qpll1refclklost_out                (qpll1refclklost_out               ),
      .qplldmonitor0_out                  (qplldmonitor0_out                 ),
      .qplldmonitor1_out                  (qplldmonitor1_out                 ),
      .refclkoutmonitor0_out              (refclkoutmonitor0_out             ),
      .refclkoutmonitor1_out              (refclkoutmonitor1_out             ),
      .rxrecclk0sel_out                   (rxrecclk0sel_out                  ),
      .rxrecclk1sel_out                   (rxrecclk1sel_out                  ),
      .sdm0finalout_out                   (sdm0finalout_out                  ),
      .sdm0testdata_out                   (sdm0testdata_out                  ),
      .sdm1finalout_out                   (sdm1finalout_out                  ),
      .sdm1testdata_out                   (sdm1testdata_out                  ),
      .tcongpo_out                        (tcongpo_out                       ),
      .tconrsvdout0_out                   (tconrsvdout0_out                  ),
      .cdrstepdir_in                      (cdrstepdir_in                     ),
      .cdrstepsq_in                       (cdrstepsq_in                      ),
      .cdrstepsx_in                       (cdrstepsx_in                      ),
      .cfgreset_in                        (cfgreset_in                       ),
      .clkrsvd0_in                        (clkrsvd0_in                       ),
      .clkrsvd1_in                        (clkrsvd1_in                       ),
      .cpllfreqlock_in                    (cpllfreqlock_in                   ),
      .cplllockdetclk_in                  (cplllockdetclk_in                 ),
      .cplllocken_in                      (cplllocken_in                     ),
      .cpllpd_in                          (cpllpd_in                         ),
      .cpllrefclksel_in                   (cpllrefclksel_in                  ),
      .cpllreset_in                       (cpllreset_in                      ),
      .dmonfiforeset_in                   (dmonfiforeset_in                  ),
      .dmonitorclk_in                     (dmonitorclk_in                    ),
      .drpaddr_in                         (drpaddr_in                        ),
      .drpclk_in                          (drpclk_in                         ),
      .drpdi_in                           (drpdi_in                          ),
      .drpen_in                           (drpen_in                          ),
      .drprst_in                          (drprst_in                         ),
      .drpwe_in                           (drpwe_in                          ),
      .eyescanreset_in                    (eyescanreset_in                   ),
      .eyescantrigger_in                  (eyescantrigger_in                 ),
      .freqos_in                          (freqos_in                         ),
      .gtgrefclk_in                       (gtgrefclk_in                      ),
      .gthrxn_in                          (gthrxn_in                         ),
      .gthrxp_in                          (gthrxp_in                         ),
      .gtnorthrefclk0_in                  (gtnorthrefclk0_in                 ),
      .gtnorthrefclk1_in                  (gtnorthrefclk1_in                 ),
      .gtrefclk0_in                       (gtrefclk0_in                      ),
      .gtrefclk1_in                       (gtrefclk1_in                      ),
      .gtrsvd_in                          (gtrsvd_in                         ),
      .gtrxreset_in                       (gtrxreset_in                      ),
      .gtrxresetsel_in                    (gtrxresetsel_in                   ),
      .gtsouthrefclk0_in                  (gtsouthrefclk0_in                 ),
      .gtsouthrefclk1_in                  (gtsouthrefclk1_in                 ),
      .gttxreset_in                       (gttxreset_in                      ),
      .gttxresetsel_in                    (gttxresetsel_in                   ),
      .incpctrl_in                        (incpctrl_in                       ),
      .loopback_in                        (loopback_in                       ),
      .pcieeqrxeqadaptdone_in             (pcieeqrxeqadaptdone_in            ),
      .pcierstidle_in                     (pcierstidle_in                    ),
      .pciersttxsyncstart_in              (pciersttxsyncstart_in             ),
      .pcieuserratedone_in                (pcieuserratedone_in               ),
      .pcsrsvdin_in                       (pcsrsvdin_in                      ),
      .qpll0clk_in                        (qpll0clk_in                       ),
      .qpll0freqlock_in                   (qpll0freqlock_in                  ),
      .qpll0refclk_in                     (qpll0refclk_in                    ),
      .qpll1clk_in                        (qpll1clk_in                       ),
      .qpll1freqlock_in                   (qpll1freqlock_in                  ),
      .qpll1refclk_in                     (qpll1refclk_in                    ),
      .resetovrd_in                       (resetovrd_in                      ),
      .rx8b10ben_in                       (rx8b10ben_in                      ),
      .rxafecfoken_in                     (rxafecfoken_in                    ),
      .rxbufreset_in                      (rxbufreset_in                     ),
      .rxcdrfreqreset_in                  (rxcdrfreqreset_in                 ),
      .rxcdrhold_in                       (rxcdrhold_in                      ),
      .rxcdrovrden_in                     (rxcdrovrden_in                    ),
      .rxcdrreset_in                      (rxcdrreset_in                     ),
      .rxchbonden_in                      (rxchbonden_in                     ),
      .rxchbondi_in                       (rxchbondi_in                      ),
      .rxchbondlevel_in                   (rxchbondlevel_in                  ),
      .rxchbondmaster_in                  (rxchbondmaster_in                 ),
      .rxchbondslave_in                   (rxchbondslave_in                  ),
      .rxckcalreset_in                    (rxckcalreset_in                   ),
      .rxckcalstart_in                    (rxckcalstart_in                   ),
      .rxcommadeten_in                    (rxcommadeten_in                   ),
      .rxdfeagcctrl_in                    (rxdfeagcctrl_in                   ),
      .rxdfeagchold_in                    (rxdfeagchold_in                   ),
      .rxdfeagcovrden_in                  (rxdfeagcovrden_in                 ),
      .rxdfecfokfcnum_in                  (rxdfecfokfcnum_in                 ),
      .rxdfecfokfen_in                    (rxdfecfokfen_in                   ),
      .rxdfecfokfpulse_in                 (rxdfecfokfpulse_in                ),
      .rxdfecfokhold_in                   (rxdfecfokhold_in                  ),
      .rxdfecfokovren_in                  (rxdfecfokovren_in                 ),
      .rxdfekhhold_in                     (rxdfekhhold_in                    ),
      .rxdfekhovrden_in                   (rxdfekhovrden_in                  ),
      .rxdfelfhold_in                     (rxdfelfhold_in                    ),
      .rxdfelfovrden_in                   (rxdfelfovrden_in                  ),
      .rxdfelpmreset_in                   (rxdfelpmreset_in                  ),
      .rxdfetap10hold_in                  (rxdfetap10hold_in                 ),
      .rxdfetap10ovrden_in                (rxdfetap10ovrden_in               ),
      .rxdfetap11hold_in                  (rxdfetap11hold_in                 ),
      .rxdfetap11ovrden_in                (rxdfetap11ovrden_in               ),
      .rxdfetap12hold_in                  (rxdfetap12hold_in                 ),
      .rxdfetap12ovrden_in                (rxdfetap12ovrden_in               ),
      .rxdfetap13hold_in                  (rxdfetap13hold_in                 ),
      .rxdfetap13ovrden_in                (rxdfetap13ovrden_in               ),
      .rxdfetap14hold_in                  (rxdfetap14hold_in                 ),
      .rxdfetap14ovrden_in                (rxdfetap14ovrden_in               ),
      .rxdfetap15hold_in                  (rxdfetap15hold_in                 ),
      .rxdfetap15ovrden_in                (rxdfetap15ovrden_in               ),
      .rxdfetap2hold_in                   (rxdfetap2hold_in                  ),
      .rxdfetap2ovrden_in                 (rxdfetap2ovrden_in                ),
      .rxdfetap3hold_in                   (rxdfetap3hold_in                  ),
      .rxdfetap3ovrden_in                 (rxdfetap3ovrden_in                ),
      .rxdfetap4hold_in                   (rxdfetap4hold_in                  ),
      .rxdfetap4ovrden_in                 (rxdfetap4ovrden_in                ),
      .rxdfetap5hold_in                   (rxdfetap5hold_in                  ),
      .rxdfetap5ovrden_in                 (rxdfetap5ovrden_in                ),
      .rxdfetap6hold_in                   (rxdfetap6hold_in                  ),
      .rxdfetap6ovrden_in                 (rxdfetap6ovrden_in                ),
      .rxdfetap7hold_in                   (rxdfetap7hold_in                  ),
      .rxdfetap7ovrden_in                 (rxdfetap7ovrden_in                ),
      .rxdfetap8hold_in                   (rxdfetap8hold_in                  ),
      .rxdfetap8ovrden_in                 (rxdfetap8ovrden_in                ),
      .rxdfetap9hold_in                   (rxdfetap9hold_in                  ),
      .rxdfetap9ovrden_in                 (rxdfetap9ovrden_in                ),
      .rxdfeuthold_in                     (rxdfeuthold_in                    ),
      .rxdfeutovrden_in                   (rxdfeutovrden_in                  ),
      .rxdfevphold_in                     (rxdfevphold_in                    ),
      .rxdfevpovrden_in                   (rxdfevpovrden_in                  ),
      .rxdfexyden_in                      (rxdfexyden_in                     ),
      .rxdlybypass_in                     (rxdlybypass_in                    ),
      .rxdlyen_in                         (rxdlyen_in                        ),
      .rxdlyovrden_in                     (rxdlyovrden_in                    ),
      .rxdlysreset_in                     (rxdlysreset_in                    ),
      .rxelecidlemode_in                  (rxelecidlemode_in                 ),
      .rxeqtraining_in                    (rxeqtraining_in                   ),
      .rxgearboxslip_in                   (rxgearboxslip_in                  ),
      .rxlatclk_in                        (rxlatclk_in                       ),
      .rxlpmen_in                         (rxlpmen_in                        ),
      .rxlpmgchold_in                     (rxlpmgchold_in                    ),
      .rxlpmgcovrden_in                   (rxlpmgcovrden_in                  ),
      .rxlpmhfhold_in                     (rxlpmhfhold_in                    ),
      .rxlpmhfovrden_in                   (rxlpmhfovrden_in                  ),
      .rxlpmlfhold_in                     (rxlpmlfhold_in                    ),
      .rxlpmlfklovrden_in                 (rxlpmlfklovrden_in                ),
      .rxlpmoshold_in                     (rxlpmoshold_in                    ),
      .rxlpmosovrden_in                   (rxlpmosovrden_in                  ),
      .rxmcommaalignen_in                 (rxmcommaalignen_in                ),
      .rxmonitorsel_in                    (rxmonitorsel_in                   ),
      .rxoobreset_in                      (rxoobreset_in                     ),
      .rxoscalreset_in                    (rxoscalreset_in                   ),
      .rxoshold_in                        (rxoshold_in                       ),
      .rxosovrden_in                      (rxosovrden_in                     ),
      .rxoutclksel_in                     (rxoutclksel_in                    ),
      .rxpcommaalignen_in                 (rxpcommaalignen_in                ),
      .rxpcsreset_in                      (rxpcsreset_in                     ),
      .rxpd_in                            (rxpd_in                           ),
      .rxphalign_in                       (rxphalign_in                      ),
      .rxphalignen_in                     (rxphalignen_in                    ),
      .rxphdlypd_in                       (rxphdlypd_in                      ),
      .rxphdlyreset_in                    (rxphdlyreset_in                   ),
      .rxphovrden_in                      (rxphovrden_in                     ),
      .rxpllclksel_in                     (rxpllclksel_in                    ),
      .rxpmareset_in                      (rxpmareset_in                     ),
      .rxpolarity_in                      (rxpolarity_in                     ),
      .rxprbscntreset_in                  (rxprbscntreset_in                 ),
      .rxprbssel_in                       (rxprbssel_in                      ),
      .rxprogdivreset_in                  (rxprogdivreset_in                 ),
      .rxqpien_in                         (rxqpien_in                        ),
      .rxrate_in                          (rxrate_in                         ),
      .rxratemode_in                      (rxratemode_in                     ),
      .rxslide_in                         (rxslide_in                        ),
      .rxslipoutclk_in                    (rxslipoutclk_in                   ),
      .rxslippma_in                       (rxslippma_in                      ),
      .rxsyncallin_in                     (rxsyncallin_in                    ),
      .rxsyncin_in                        (rxsyncin_in                       ),
      .rxsyncmode_in                      (rxsyncmode_in                     ),
      .rxsysclksel_in                     (rxsysclksel_in                    ),
      .rxtermination_in                   (rxtermination_in                  ),
      .rxuserrdy_in                       (rxuserrdy_in                      ),
      .rxusrclk_in                        (rxusrclk_in                       ),
      .rxusrclk2_in                       (rxusrclk2_in                      ),
      .sigvalidclk_in                     (sigvalidclk_in                    ),
      .tstin_in                           (tstin_in                          ),
      .tx8b10bbypass_in                   (tx8b10bbypass_in                  ),
      .tx8b10ben_in                       (tx8b10ben_in                      ),
      .txcominit_in                       (txcominit_in                      ),
      .txcomsas_in                        (txcomsas_in                       ),
      .txcomwake_in                       (txcomwake_in                      ),
      .txctrl0_in                         (txctrl0_in                        ),
      .txctrl1_in                         (txctrl1_in                        ),
      .txctrl2_in                         (txctrl2_in                        ),
      .txdata_in                          (txdata_in                         ),
      .txdataextendrsvd_in                (txdataextendrsvd_in               ),
      .txdccforcestart_in                 (txdccforcestart_in                ),
      .txdccreset_in                      (txdccreset_in                     ),
      .txdeemph_in                        (txdeemph_in                       ),
      .txdetectrx_in                      (txdetectrx_in                     ),
      .txdiffctrl_in                      (txdiffctrl_in                     ),
      .txdlybypass_in                     (txdlybypass_in                    ),
      .txdlyen_in                         (txdlyen_in                        ),
      .txdlyhold_in                       (txdlyhold_in                      ),
      .txdlyovrden_in                     (txdlyovrden_in                    ),
      .txdlysreset_in                     (txdlysreset_in                    ),
      .txdlyupdown_in                     (txdlyupdown_in                    ),
      .txelecidle_in                      (txelecidle_in                     ),
      .txheader_in                        (txheader_in                       ),
      .txinhibit_in                       (txinhibit_in                      ),
      .txlatclk_in                        (txlatclk_in                       ),
      .txlfpstreset_in                    (txlfpstreset_in                   ),
      .txlfpsu2lpexit_in                  (txlfpsu2lpexit_in                 ),
      .txlfpsu3wake_in                    (txlfpsu3wake_in                   ),
      .txmaincursor_in                    (txmaincursor_in                   ),
      .txmargin_in                        (txmargin_in                       ),
      .txmuxdcdexhold_in                  (txmuxdcdexhold_in                 ),
      .txmuxdcdorwren_in                  (txmuxdcdorwren_in                 ),
      .txoneszeros_in                     (txoneszeros_in                    ),
      .txoutclksel_in                     (txoutclksel_in                    ),
      .txpcsreset_in                      (txpcsreset_in                     ),
      .txpd_in                            (txpd_in                           ),
      .txpdelecidlemode_in                (txpdelecidlemode_in               ),
      .txphalign_in                       (txphalign_in                      ),
      .txphalignen_in                     (txphalignen_in                    ),
      .txphdlypd_in                       (txphdlypd_in                      ),
      .txphdlyreset_in                    (txphdlyreset_in                   ),
      .txphdlytstclk_in                   (txphdlytstclk_in                  ),
      .txphinit_in                        (txphinit_in                       ),
      .txphovrden_in                      (txphovrden_in                     ),
      .txpippmen_in                       (txpippmen_in                      ),
      .txpippmovrden_in                   (txpippmovrden_in                  ),
      .txpippmpd_in                       (txpippmpd_in                      ),
      .txpippmsel_in                      (txpippmsel_in                     ),
      .txpippmstepsize_in                 (txpippmstepsize_in                ),
      .txpisopd_in                        (txpisopd_in                       ),
      .txpllclksel_in                     (txpllclksel_in                    ),
      .txpmareset_in                      (txpmareset_in                     ),
      .txpolarity_in                      (txpolarity_in                     ),
      .txpostcursor_in                    (txpostcursor_in                   ),
      .txprbsforceerr_in                  (txprbsforceerr_in                 ),
      .txprbssel_in                       (txprbssel_in                      ),
      .txprecursor_in                     (txprecursor_in                    ),
      .txprogdivreset_in                  (txprogdivreset_in                 ),
      .txqpibiasen_in                     (txqpibiasen_in                    ),
      .txqpiweakpup_in                    (txqpiweakpup_in                   ),
      .txrate_in                          (txrate_in                         ),
      .txratemode_in                      (txratemode_in                     ),
      .txsequence_in                      (txsequence_in                     ),
      .txswing_in                         (txswing_in                        ),
      .txsyncallin_in                     (txsyncallin_in                    ),
      .txsyncin_in                        (txsyncin_in                       ),
      .txsyncmode_in                      (txsyncmode_in                     ),
      .txsysclksel_in                     (txsysclksel_in                    ),
      .txuserrdy_in                       (txuserrdy_in                      ),
      .txusrclk_in                        (txusrclk_in                       ),
      .txusrclk2_in                       (txusrclk2_in                      ),
      .bufgtce_out                        (bufgtce_out                       ),
      .bufgtcemask_out                    (bufgtcemask_out                   ),
      .bufgtdiv_out                       (bufgtdiv_out                      ),
      .bufgtreset_out                     (bufgtreset_out                    ),
      .bufgtrstmask_out                   (bufgtrstmask_out                  ),
      .cpllfbclklost_out                  (cpllfbclklost_out                 ),
      .cplllock_out                       (cplllock_out                      ),
      .cpllrefclklost_out                 (cpllrefclklost_out                ),
      .dmonitorout_out                    (dmonitorout_out                   ),
      .dmonitoroutclk_out                 (dmonitoroutclk_out                ),
      .drpdo_out                          (drpdo_out                         ),
      .drprdy_out                         (drprdy_out                        ),
      .eyescandataerror_out               (eyescandataerror_out              ),
      .gthtxn_out                         (gthtxn_out                        ),
      .gthtxp_out                         (gthtxp_out                        ),
      .gtpowergood_out                    (gtpowergood_out                   ),
      .gtrefclkmonitor_out                (gtrefclkmonitor_out               ),
      .pcierategen3_out                   (pcierategen3_out                  ),
      .pcierateidle_out                   (pcierateidle_out                  ),
      .pcierateqpllpd_out                 (pcierateqpllpd_out                ),
      .pcierateqpllreset_out              (pcierateqpllreset_out             ),
      .pciesynctxsyncdone_out             (pciesynctxsyncdone_out            ),
      .pcieusergen3rdy_out                (pcieusergen3rdy_out               ),
      .pcieuserphystatusrst_out           (pcieuserphystatusrst_out          ),
      .pcieuserratestart_out              (pcieuserratestart_out             ),
      .pcsrsvdout_out                     (pcsrsvdout_out                    ),
      .phystatus_out                      (phystatus_out                     ),
      .pinrsrvdas_out                     (pinrsrvdas_out                    ),
      .powerpresent_out                   (powerpresent_out                  ),
      .resetexception_out                 (resetexception_out                ),
      .rxbufstatus_out                    (rxbufstatus_out                   ),
      .rxbyteisaligned_out                (rxbyteisaligned_out               ),
      .rxbyterealign_out                  (rxbyterealign_out                 ),
      .rxcdrlock_out                      (rxcdrlock_out                     ),
      .rxcdrphdone_out                    (rxcdrphdone_out                   ),
      .rxchanbondseq_out                  (rxchanbondseq_out                 ),
      .rxchanisaligned_out                (rxchanisaligned_out               ),
      .rxchanrealign_out                  (rxchanrealign_out                 ),
      .rxchbondo_out                      (rxchbondo_out                     ),
      .rxckcaldone_out                    (rxckcaldone_out                   ),
      .rxclkcorcnt_out                    (rxclkcorcnt_out                   ),
      .rxcominitdet_out                   (rxcominitdet_out                  ),
      .rxcommadet_out                     (rxcommadet_out                    ),
      .rxcomsasdet_out                    (rxcomsasdet_out                   ),
      .rxcomwakedet_out                   (rxcomwakedet_out                  ),
      .rxctrl0_out                        (rxctrl0_out                       ),
      .rxctrl1_out                        (rxctrl1_out                       ),
      .rxctrl2_out                        (rxctrl2_out                       ),
      .rxctrl3_out                        (rxctrl3_out                       ),
      .rxdata_out                         (rxdata_out                        ),
      .rxdataextendrsvd_out               (rxdataextendrsvd_out              ),
      .rxdatavalid_out                    (rxdatavalid_out                   ),
      .rxdlysresetdone_out                (rxdlysresetdone_out               ),
      .rxelecidle_out                     (rxelecidle_out                    ),
      .rxheader_out                       (rxheader_out                      ),
      .rxheadervalid_out                  (rxheadervalid_out                 ),
      .rxlfpstresetdet_out                (rxlfpstresetdet_out               ),
      .rxlfpsu2lpexitdet_out              (rxlfpsu2lpexitdet_out             ),
      .rxlfpsu3wakedet_out                (rxlfpsu3wakedet_out               ),
      .rxmonitorout_out                   (rxmonitorout_out                  ),
      .rxosintdone_out                    (rxosintdone_out                   ),
      .rxosintstarted_out                 (rxosintstarted_out                ),
      .rxosintstrobedone_out              (rxosintstrobedone_out             ),
      .rxosintstrobestarted_out           (rxosintstrobestarted_out          ),
      .rxoutclk_out                       (rxoutclk_out                      ),
      .rxoutclkfabric_out                 (rxoutclkfabric_out                ),
      .rxoutclkpcs_out                    (rxoutclkpcs_out                   ),
      .rxphaligndone_out                  (rxphaligndone_out                 ),
      .rxphalignerr_out                   (rxphalignerr_out                  ),
      .rxpmaresetdone_out                 (rxpmaresetdone_out                ),
      .rxprbserr_out                      (rxprbserr_out                     ),
      .rxprbslocked_out                   (rxprbslocked_out                  ),
      .rxprgdivresetdone_out              (rxprgdivresetdone_out             ),
      .rxqpisenn_out                      (rxqpisenn_out                     ),
      .rxqpisenp_out                      (rxqpisenp_out                     ),
      .rxratedone_out                     (rxratedone_out                    ),
      .rxrecclkout_out                    (rxrecclkout_out                   ),
      .rxresetdone_out                    (rxresetdone_out                   ),
      .rxsliderdy_out                     (rxsliderdy_out                    ),
      .rxslipdone_out                     (rxslipdone_out                    ),
      .rxslipoutclkrdy_out                (rxslipoutclkrdy_out               ),
      .rxslippmardy_out                   (rxslippmardy_out                  ),
      .rxstartofseq_out                   (rxstartofseq_out                  ),
      .rxstatus_out                       (rxstatus_out                      ),
      .rxsyncdone_out                     (rxsyncdone_out                    ),
      .rxsyncout_out                      (rxsyncout_out                     ),
      .rxvalid_out                        (rxvalid_out                       ),
      .txbufstatus_out                    (txbufstatus_out                   ),
      .txcomfinish_out                    (txcomfinish_out                   ),
      .txdccdone_out                      (txdccdone_out                     ),
      .txdlysresetdone_out                (txdlysresetdone_out               ),
      .txoutclk_out                       (txoutclk_out                      ),
      .txoutclkfabric_out                 (txoutclkfabric_out                ),
      .txoutclkpcs_out                    (txoutclkpcs_out                   ),
      .txphaligndone_out                  (txphaligndone_out                 ),
      .txphinitdone_out                   (txphinitdone_out                  ),
      .txpmaresetdone_out                 (txpmaresetdone_out                ),
      .txprgdivresetdone_out              (txprgdivresetdone_out             ),
      .txqpisenn_out                      (txqpisenn_out                     ),
      .txqpisenp_out                      (txqpisenp_out                     ),
      .txratedone_out                     (txratedone_out                    ),
      .txresetdone_out                    (txresetdone_out                   ),
      .txsyncdone_out                     (txsyncdone_out                    ),
      .txsyncout_out                      (txsyncout_out                     )
    );

    // Drive unused outputs to constant values
    assign rxrecclk0_sel_out  = {{`ice_gth_t_SF_CM}{1'b0}};
    assign rxrecclk1_sel_out  = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubdaddr_out        = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubden_out          = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubdi_out           = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubdwe_out          = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubmdmtdo_out       = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubrsvdout_out      = {{`ice_gth_t_SF_CM}{1'b0}};
    assign ubtxuart_out       = {{`ice_gth_t_SF_CM}{1'b0}};
    assign gtytxn_out         = {{`ice_gth_t_N_CH}{1'b0}};
    assign gtytxp_out         = {{`ice_gth_t_N_CH}{1'b0}};

  end
  else if (C_GT_TYPE == `ice_gth_t_GT_TYPE__GTYE4) begin : gen_gtwizard_gtye4_top

    // Generate GTYE4-type Transceivers Wizard submodule
    ice_gth_t_gtwizard_gtye4 #(
      .C_CHANNEL_ENABLE                          (C_CHANNEL_ENABLE                         ),
      .C_COMMON_SCALING_FACTOR                   (C_COMMON_SCALING_FACTOR                  ),
      .C_FREERUN_FREQUENCY                       (C_FREERUN_FREQUENCY                      ),
      .C_LOCATE_RESET_CONTROLLER                 (C_LOCATE_RESET_CONTROLLER                ),
      .C_LOCATE_USER_DATA_WIDTH_SIZING           (C_LOCATE_USER_DATA_WIDTH_SIZING          ),
      .C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER      (C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER     ),
      .C_LOCATE_RX_USER_CLOCKING                 (C_LOCATE_RX_USER_CLOCKING                ),
      .C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER      (C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER     ),
      .C_LOCATE_TX_USER_CLOCKING                 (C_LOCATE_TX_USER_CLOCKING                ),
      .C_RESET_CONTROLLER_INSTANCE_CTRL          (C_RESET_CONTROLLER_INSTANCE_CTRL         ),
      .C_RX_BUFFBYPASS_MODE                      (C_RX_BUFFBYPASS_MODE                     ),
      .C_RX_BUFFER_BYPASS_INSTANCE_CTRL          (C_RX_BUFFER_BYPASS_INSTANCE_CTRL         ),
      .C_RX_BUFFER_MODE                          (C_RX_BUFFER_MODE                         ),
      .C_RX_DATA_DECODING                        (C_RX_DATA_DECODING                       ),
      .C_RX_ENABLE                               (C_RX_ENABLE                              ),
      .C_RX_INT_DATA_WIDTH                       (C_RX_INT_DATA_WIDTH                      ),
      .C_RX_LINE_RATE                            (C_RX_LINE_RATE                           ),
      .C_RX_MASTER_CHANNEL_IDX                   (C_RX_MASTER_CHANNEL_IDX                  ),
      .C_RX_OUTCLK_BUFG_GT_DIV                   (C_RX_OUTCLK_BUFG_GT_DIV                  ),
      .C_RX_PLL_TYPE                             (C_RX_PLL_TYPE                            ),
      .C_RX_USER_CLOCKING_CONTENTS               (C_RX_USER_CLOCKING_CONTENTS              ),
      .C_RX_USER_CLOCKING_INSTANCE_CTRL          (C_RX_USER_CLOCKING_INSTANCE_CTRL         ),
      .C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 (C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2),
      .C_RX_USER_CLOCKING_SOURCE                 (C_RX_USER_CLOCKING_SOURCE                ),
      .C_RX_USER_DATA_WIDTH                      (C_RX_USER_DATA_WIDTH                     ),
      .C_TOTAL_NUM_CHANNELS                      (C_TOTAL_NUM_CHANNELS                     ),
      .C_TOTAL_NUM_COMMONS                       (C_TOTAL_NUM_COMMONS                      ),
      .C_TX_BUFFBYPASS_MODE                      (C_TX_BUFFBYPASS_MODE                     ),
      .C_TX_BUFFER_BYPASS_INSTANCE_CTRL          (C_TX_BUFFER_BYPASS_INSTANCE_CTRL         ),
      .C_TX_BUFFER_MODE                          (C_TX_BUFFER_MODE                         ),
      .C_TX_DATA_ENCODING                        (C_TX_DATA_ENCODING                       ),
      .C_TX_ENABLE                               (C_TX_ENABLE                              ),
      .C_TX_INT_DATA_WIDTH                       (C_TX_INT_DATA_WIDTH                      ),
      .C_TX_MASTER_CHANNEL_IDX                   (C_TX_MASTER_CHANNEL_IDX                  ),
      .C_TX_OUTCLK_BUFG_GT_DIV                   (C_TX_OUTCLK_BUFG_GT_DIV                  ),
      .C_TX_PLL_TYPE                             (C_TX_PLL_TYPE                            ),
      .C_TX_USER_CLOCKING_CONTENTS               (C_TX_USER_CLOCKING_CONTENTS              ),
      .C_TX_USER_CLOCKING_INSTANCE_CTRL          (C_TX_USER_CLOCKING_INSTANCE_CTRL         ),
      .C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 (C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2),
      .C_TX_USER_CLOCKING_SOURCE                 (C_TX_USER_CLOCKING_SOURCE                ),
      .C_TX_USER_DATA_WIDTH                      (C_TX_USER_DATA_WIDTH                     )
    ) ice_gth_t_gtwizard_gtye4_inst (
      .gtwiz_userclk_tx_reset_in          (gtwiz_userclk_tx_reset_in         ),
      .gtwiz_userclk_tx_active_in         (gtwiz_userclk_tx_active_in        ),
      .gtwiz_userclk_tx_srcclk_out        (gtwiz_userclk_tx_srcclk_out       ),
      .gtwiz_userclk_tx_usrclk_out        (gtwiz_userclk_tx_usrclk_out       ),
      .gtwiz_userclk_tx_usrclk2_out       (gtwiz_userclk_tx_usrclk2_out      ),
      .gtwiz_userclk_tx_active_out        (gtwiz_userclk_tx_active_out       ),
      .gtwiz_userclk_rx_reset_in          (gtwiz_userclk_rx_reset_in         ),
      .gtwiz_userclk_rx_active_in         (gtwiz_userclk_rx_active_in        ),
      .gtwiz_userclk_rx_srcclk_out        (gtwiz_userclk_rx_srcclk_out       ),
      .gtwiz_userclk_rx_usrclk_out        (gtwiz_userclk_rx_usrclk_out       ),
      .gtwiz_userclk_rx_usrclk2_out       (gtwiz_userclk_rx_usrclk2_out      ),
      .gtwiz_userclk_rx_active_out        (gtwiz_userclk_rx_active_out       ),
      .gtwiz_buffbypass_tx_reset_in       (gtwiz_buffbypass_tx_reset_in      ),
      .gtwiz_buffbypass_tx_start_user_in  (gtwiz_buffbypass_tx_start_user_in ),
      .gtwiz_buffbypass_tx_done_out       (gtwiz_buffbypass_tx_done_out      ),
      .gtwiz_buffbypass_tx_error_out      (gtwiz_buffbypass_tx_error_out     ),
      .gtwiz_buffbypass_rx_reset_in       (gtwiz_buffbypass_rx_reset_in      ),
      .gtwiz_buffbypass_rx_start_user_in  (gtwiz_buffbypass_rx_start_user_in ),
      .gtwiz_buffbypass_rx_done_out       (gtwiz_buffbypass_rx_done_out      ),
      .gtwiz_buffbypass_rx_error_out      (gtwiz_buffbypass_rx_error_out     ),
      .gtwiz_reset_clk_freerun_in         (gtwiz_reset_clk_freerun_in        ),
      .gtwiz_reset_all_in                 (gtwiz_reset_all_in                ),
      .gtwiz_reset_tx_pll_and_datapath_in (gtwiz_reset_tx_pll_and_datapath_in),
      .gtwiz_reset_tx_datapath_in         (gtwiz_reset_tx_datapath_in        ),
      .gtwiz_reset_rx_pll_and_datapath_in (gtwiz_reset_rx_pll_and_datapath_in),
      .gtwiz_reset_rx_datapath_in         (gtwiz_reset_rx_datapath_in        ),
      .gtwiz_reset_tx_done_in             (gtwiz_reset_tx_done_in            ),
      .gtwiz_reset_rx_done_in             (gtwiz_reset_rx_done_in            ),
      .gtwiz_reset_qpll0lock_in           (gtwiz_reset_qpll0lock_in          ),
      .gtwiz_reset_qpll1lock_in           (gtwiz_reset_qpll1lock_in          ),
      .gtwiz_reset_rx_cdr_stable_out      (gtwiz_reset_rx_cdr_stable_out     ),
      .gtwiz_reset_tx_done_out            (gtwiz_reset_tx_done_out           ),
      .gtwiz_reset_rx_done_out            (gtwiz_reset_rx_done_out           ),
      .gtwiz_reset_qpll0reset_out         (gtwiz_reset_qpll0reset_out        ),
      .gtwiz_reset_qpll1reset_out         (gtwiz_reset_qpll1reset_out        ),
      .gtwiz_userdata_tx_in               (gtwiz_userdata_tx_in              ),
      .gtwiz_userdata_rx_out              (gtwiz_userdata_rx_out             ),
      .bgbypassb_in                       (bgbypassb_in                      ),
      .bgmonitorenb_in                    (bgmonitorenb_in                   ),
      .bgpdb_in                           (bgpdb_in                          ),
      .bgrcalovrd_in                      (bgrcalovrd_in                     ),
      .bgrcalovrdenb_in                   (bgrcalovrdenb_in                  ),
      .drpaddr_common_in                  (drpaddr_common_in                 ),
      .drpclk_common_in                   (drpclk_common_in                  ),
      .drpdi_common_in                    (drpdi_common_in                   ),
      .drpen_common_in                    (drpen_common_in                   ),
      .drpwe_common_in                    (drpwe_common_in                   ),
      .gtgrefclk0_in                      (gtgrefclk0_in                     ),
      .gtgrefclk1_in                      (gtgrefclk1_in                     ),
      .gtnorthrefclk00_in                 (gtnorthrefclk00_in                ),
      .gtnorthrefclk01_in                 (gtnorthrefclk01_in                ),
      .gtnorthrefclk10_in                 (gtnorthrefclk10_in                ),
      .gtnorthrefclk11_in                 (gtnorthrefclk11_in                ),
      .gtrefclk00_in                      (gtrefclk00_in                     ),
      .gtrefclk01_in                      (gtrefclk01_in                     ),
      .gtrefclk10_in                      (gtrefclk10_in                     ),
      .gtrefclk11_in                      (gtrefclk11_in                     ),
      .gtsouthrefclk00_in                 (gtsouthrefclk00_in                ),
      .gtsouthrefclk01_in                 (gtsouthrefclk01_in                ),
      .gtsouthrefclk10_in                 (gtsouthrefclk10_in                ),
      .gtsouthrefclk11_in                 (gtsouthrefclk11_in                ),
      .pcierateqpll0_in                   (pcierateqpll0_in                  ),
      .pcierateqpll1_in                   (pcierateqpll1_in                  ),
      .pmarsvd0_in                        (pmarsvd0_in                       ),
      .pmarsvd1_in                        (pmarsvd1_in                       ),
      .qpll0clkrsvd0_in                   (qpll0clkrsvd0_in                  ),
      .qpll0clkrsvd1_in                   (qpll0clkrsvd1_in                  ),
      .qpll0fbdiv_in                      (qpll0fbdiv_in                     ),
      .qpll0lockdetclk_in                 (qpll0lockdetclk_in                ),
      .qpll0locken_in                     (qpll0locken_in                    ),
      .qpll0pd_in                         (qpll0pd_in                        ),
      .qpll0refclksel_in                  (qpll0refclksel_in                 ),
      .qpll0reset_in                      (qpll0reset_in                     ),
      .qpll1clkrsvd0_in                   (qpll1clkrsvd0_in                  ),
      .qpll1clkrsvd1_in                   (qpll1clkrsvd1_in                  ),
      .qpll1fbdiv_in                      (qpll1fbdiv_in                     ),
      .qpll1lockdetclk_in                 (qpll1lockdetclk_in                ),
      .qpll1locken_in                     (qpll1locken_in                    ),
      .qpll1pd_in                         (qpll1pd_in                        ),
      .qpll1refclksel_in                  (qpll1refclksel_in                 ),
      .qpll1reset_in                      (qpll1reset_in                     ),
      .qpllrsvd1_in                       (qpllrsvd1_in                      ),
      .qpllrsvd2_in                       (qpllrsvd2_in                      ),
      .qpllrsvd3_in                       (qpllrsvd3_in                      ),
      .qpllrsvd4_in                       (qpllrsvd4_in                      ),
      .rcalenb_in                         (rcalenb_in                        ),
      .sdm0data_in                        (sdm0data_in                       ),
      .sdm0reset_in                       (sdm0reset_in                      ),
      .sdm0toggle_in                      (sdm0toggle_in                     ),
      .sdm0width_in                       (sdm0width_in                      ),
      .sdm1data_in                        (sdm1data_in                       ),
      .sdm1reset_in                       (sdm1reset_in                      ),
      .sdm1toggle_in                      (sdm1toggle_in                     ),
      .sdm1width_in                       (sdm1width_in                      ),
      .ubcfgstreamen_in                   (ubcfgstreamen_in                  ),
      .ubdo_in                            (ubdo_in                           ),
      .ubdrdy_in                          (ubdrdy_in                         ),
      .ubenable_in                        (ubenable_in                       ),
      .ubgpi_in                           (ubgpi_in                          ),
      .ubintr_in                          (ubintr_in                         ),
      .ubiolmbrst_in                      (ubiolmbrst_in                     ),
      .ubmbrst_in                         (ubmbrst_in                        ),
      .ubmdmcapture_in                    (ubmdmcapture_in                   ),
      .ubmdmdbgrst_in                     (ubmdmdbgrst_in                    ),
      .ubmdmdbgupdate_in                  (ubmdmdbgupdate_in                 ),
      .ubmdmregen_in                      (ubmdmregen_in                     ),
      .ubmdmshift_in                      (ubmdmshift_in                     ),
      .ubmdmsysrst_in                     (ubmdmsysrst_in                    ),
      .ubmdmtck_in                        (ubmdmtck_in                       ),
      .ubmdmtdi_in                        (ubmdmtdi_in                       ),
      .drpdo_common_out                   (drpdo_common_out                  ),
      .drprdy_common_out                  (drprdy_common_out                 ),
      .pmarsvdout0_out                    (pmarsvdout0_out                   ),
      .pmarsvdout1_out                    (pmarsvdout1_out                   ),
      .qpll0fbclklost_out                 (qpll0fbclklost_out                ),
      .qpll0lock_out                      (qpll0lock_out                     ),
      .qpll0outclk_out                    (qpll0outclk_out                   ),
      .qpll0outrefclk_out                 (qpll0outrefclk_out                ),
      .qpll0refclklost_out                (qpll0refclklost_out               ),
      .qpll1fbclklost_out                 (qpll1fbclklost_out                ),
      .qpll1lock_out                      (qpll1lock_out                     ),
      .qpll1outclk_out                    (qpll1outclk_out                   ),
      .qpll1outrefclk_out                 (qpll1outrefclk_out                ),
      .qpll1refclklost_out                (qpll1refclklost_out               ),
      .qplldmonitor0_out                  (qplldmonitor0_out                 ),
      .qplldmonitor1_out                  (qplldmonitor1_out                 ),
      .refclkoutmonitor0_out              (refclkoutmonitor0_out             ),
      .refclkoutmonitor1_out              (refclkoutmonitor1_out             ),
      .rxrecclk0sel_out                   (rxrecclk0sel_out                  ),
      .rxrecclk1sel_out                   (rxrecclk1sel_out                  ),
      .sdm0finalout_out                   (sdm0finalout_out                  ),
      .sdm0testdata_out                   (sdm0testdata_out                  ),
      .sdm1finalout_out                   (sdm1finalout_out                  ),
      .sdm1testdata_out                   (sdm1testdata_out                  ),
      .ubdaddr_out                        (ubdaddr_out                       ),
      .ubden_out                          (ubden_out                         ),
      .ubdi_out                           (ubdi_out                          ),
      .ubdwe_out                          (ubdwe_out                         ),
      .ubmdmtdo_out                       (ubmdmtdo_out                      ),
      .ubrsvdout_out                      (ubrsvdout_out                     ),
      .ubtxuart_out                       (ubtxuart_out                      ),
      .cdrstepdir_in                      (cdrstepdir_in                     ),
      .cdrstepsq_in                       (cdrstepsq_in                      ),
      .cdrstepsx_in                       (cdrstepsx_in                      ),
      .cfgreset_in                        (cfgreset_in                       ),
      .clkrsvd0_in                        (clkrsvd0_in                       ),
      .clkrsvd1_in                        (clkrsvd1_in                       ),
      .cpllfreqlock_in                    (cpllfreqlock_in                   ),
      .cplllockdetclk_in                  (cplllockdetclk_in                 ),
      .cplllocken_in                      (cplllocken_in                     ),
      .cpllpd_in                          (cpllpd_in                         ),
      .cpllrefclksel_in                   (cpllrefclksel_in                  ),
      .cpllreset_in                       (cpllreset_in                      ),
      .dmonfiforeset_in                   (dmonfiforeset_in                  ),
      .dmonitorclk_in                     (dmonitorclk_in                    ),
      .drpaddr_in                         (drpaddr_in                        ),
      .drpclk_in                          (drpclk_in                         ),
      .drpdi_in                           (drpdi_in                          ),
      .drpen_in                           (drpen_in                          ),
      .drprst_in                          (drprst_in                         ),
      .drpwe_in                           (drpwe_in                          ),
      .eyescanreset_in                    (eyescanreset_in                   ),
      .eyescantrigger_in                  (eyescantrigger_in                 ),
      .freqos_in                          (freqos_in                         ),
      .gtgrefclk_in                       (gtgrefclk_in                      ),
      .gtnorthrefclk0_in                  (gtnorthrefclk0_in                 ),
      .gtnorthrefclk1_in                  (gtnorthrefclk1_in                 ),
      .gtrefclk0_in                       (gtrefclk0_in                      ),
      .gtrefclk1_in                       (gtrefclk1_in                      ),
      .gtrsvd_in                          (gtrsvd_in                         ),
      .gtrxreset_in                       (gtrxreset_in                      ),
      .gtrxresetsel_in                    (gtrxresetsel_in                   ),
      .gtsouthrefclk0_in                  (gtsouthrefclk0_in                 ),
      .gtsouthrefclk1_in                  (gtsouthrefclk1_in                 ),
      .gttxreset_in                       (gttxreset_in                      ),
      .gttxresetsel_in                    (gttxresetsel_in                   ),
      .incpctrl_in                        (incpctrl_in                       ),
      .gtyrxn_in                          (gtyrxn_in                         ),
      .gtyrxp_in                          (gtyrxp_in                         ),
      .loopback_in                        (loopback_in                       ),
      .pcieeqrxeqadaptdone_in             (pcieeqrxeqadaptdone_in            ),
      .pcierstidle_in                     (pcierstidle_in                    ),
      .pciersttxsyncstart_in              (pciersttxsyncstart_in             ),
      .pcieuserratedone_in                (pcieuserratedone_in               ),
      .pcsrsvdin_in                       (pcsrsvdin_in                      ),
      .qpll0clk_in                        (qpll0clk_in                       ),
      .qpll0freqlock_in                   (qpll0freqlock_in                  ),
      .qpll0refclk_in                     (qpll0refclk_in                    ),
      .qpll1clk_in                        (qpll1clk_in                       ),
      .qpll1freqlock_in                   (qpll1freqlock_in                  ),
      .qpll1refclk_in                     (qpll1refclk_in                    ),
      .resetovrd_in                       (resetovrd_in                      ),
      .rx8b10ben_in                       (rx8b10ben_in                      ),
      .rxafecfoken_in                     (rxafecfoken_in                    ),
      .rxbufreset_in                      (rxbufreset_in                     ),
      .rxcdrfreqreset_in                  (rxcdrfreqreset_in                 ),
      .rxcdrhold_in                       (rxcdrhold_in                      ),
      .rxcdrovrden_in                     (rxcdrovrden_in                    ),
      .rxcdrreset_in                      (rxcdrreset_in                     ),
      .rxchbonden_in                      (rxchbonden_in                     ),
      .rxchbondi_in                       (rxchbondi_in                      ),
      .rxchbondlevel_in                   (rxchbondlevel_in                  ),
      .rxchbondmaster_in                  (rxchbondmaster_in                 ),
      .rxchbondslave_in                   (rxchbondslave_in                  ),
      .rxckcalreset_in                    (rxckcalreset_in                   ),
      .rxckcalstart_in                    (rxckcalstart_in                   ),
      .rxcommadeten_in                    (rxcommadeten_in                   ),
      .rxdfeagchold_in                    (rxdfeagchold_in                   ),
      .rxdfeagcovrden_in                  (rxdfeagcovrden_in                 ),
      .rxdfecfokfcnum_in                  (rxdfecfokfcnum_in                 ),
      .rxdfecfokfen_in                    (rxdfecfokfen_in                   ),
      .rxdfecfokfpulse_in                 (rxdfecfokfpulse_in                ),
      .rxdfecfokhold_in                   (rxdfecfokhold_in                  ),
      .rxdfecfokovren_in                  (rxdfecfokovren_in                 ),
      .rxdfekhhold_in                     (rxdfekhhold_in                    ),
      .rxdfekhovrden_in                   (rxdfekhovrden_in                  ),
      .rxdfelfhold_in                     (rxdfelfhold_in                    ),
      .rxdfelfovrden_in                   (rxdfelfovrden_in                  ),
      .rxdfelpmreset_in                   (rxdfelpmreset_in                  ),
      .rxdfetap10hold_in                  (rxdfetap10hold_in                 ),
      .rxdfetap10ovrden_in                (rxdfetap10ovrden_in               ),
      .rxdfetap11hold_in                  (rxdfetap11hold_in                 ),
      .rxdfetap11ovrden_in                (rxdfetap11ovrden_in               ),
      .rxdfetap12hold_in                  (rxdfetap12hold_in                 ),
      .rxdfetap12ovrden_in                (rxdfetap12ovrden_in               ),
      .rxdfetap13hold_in                  (rxdfetap13hold_in                 ),
      .rxdfetap13ovrden_in                (rxdfetap13ovrden_in               ),
      .rxdfetap14hold_in                  (rxdfetap14hold_in                 ),
      .rxdfetap14ovrden_in                (rxdfetap14ovrden_in               ),
      .rxdfetap15hold_in                  (rxdfetap15hold_in                 ),
      .rxdfetap15ovrden_in                (rxdfetap15ovrden_in               ),
      .rxdfetap2hold_in                   (rxdfetap2hold_in                  ),
      .rxdfetap2ovrden_in                 (rxdfetap2ovrden_in                ),
      .rxdfetap3hold_in                   (rxdfetap3hold_in                  ),
      .rxdfetap3ovrden_in                 (rxdfetap3ovrden_in                ),
      .rxdfetap4hold_in                   (rxdfetap4hold_in                  ),
      .rxdfetap4ovrden_in                 (rxdfetap4ovrden_in                ),
      .rxdfetap5hold_in                   (rxdfetap5hold_in                  ),
      .rxdfetap5ovrden_in                 (rxdfetap5ovrden_in                ),
      .rxdfetap6hold_in                   (rxdfetap6hold_in                  ),
      .rxdfetap6ovrden_in                 (rxdfetap6ovrden_in                ),
      .rxdfetap7hold_in                   (rxdfetap7hold_in                  ),
      .rxdfetap7ovrden_in                 (rxdfetap7ovrden_in                ),
      .rxdfetap8hold_in                   (rxdfetap8hold_in                  ),
      .rxdfetap8ovrden_in                 (rxdfetap8ovrden_in                ),
      .rxdfetap9hold_in                   (rxdfetap9hold_in                  ),
      .rxdfetap9ovrden_in                 (rxdfetap9ovrden_in                ),
      .rxdfeuthold_in                     (rxdfeuthold_in                    ),
      .rxdfeutovrden_in                   (rxdfeutovrden_in                  ),
      .rxdfevphold_in                     (rxdfevphold_in                    ),
      .rxdfevpovrden_in                   (rxdfevpovrden_in                  ),
      .rxdfexyden_in                      (rxdfexyden_in                     ),
      .rxdlybypass_in                     (rxdlybypass_in                    ),
      .rxdlyen_in                         (rxdlyen_in                        ),
      .rxdlyovrden_in                     (rxdlyovrden_in                    ),
      .rxdlysreset_in                     (rxdlysreset_in                    ),
      .rxelecidlemode_in                  (rxelecidlemode_in                 ),
      .rxeqtraining_in                    (rxeqtraining_in                   ),
      .rxgearboxslip_in                   (rxgearboxslip_in                  ),
      .rxlatclk_in                        (rxlatclk_in                       ),
      .rxlpmen_in                         (rxlpmen_in                        ),
      .rxlpmgchold_in                     (rxlpmgchold_in                    ),
      .rxlpmgcovrden_in                   (rxlpmgcovrden_in                  ),
      .rxlpmhfhold_in                     (rxlpmhfhold_in                    ),
      .rxlpmhfovrden_in                   (rxlpmhfovrden_in                  ),
      .rxlpmlfhold_in                     (rxlpmlfhold_in                    ),
      .rxlpmlfklovrden_in                 (rxlpmlfklovrden_in                ),
      .rxlpmoshold_in                     (rxlpmoshold_in                    ),
      .rxlpmosovrden_in                   (rxlpmosovrden_in                  ),
      .rxmcommaalignen_in                 (rxmcommaalignen_in                ),
      .rxmonitorsel_in                    (rxmonitorsel_in                   ),
      .rxoobreset_in                      (rxoobreset_in                     ),
      .rxoscalreset_in                    (rxoscalreset_in                   ),
      .rxoshold_in                        (rxoshold_in                       ),
      .rxosovrden_in                      (rxosovrden_in                     ),
      .rxoutclksel_in                     (rxoutclksel_in                    ),
      .rxpcommaalignen_in                 (rxpcommaalignen_in                ),
      .rxpcsreset_in                      (rxpcsreset_in                     ),
      .rxpd_in                            (rxpd_in                           ),
      .rxphalign_in                       (rxphalign_in                      ),
      .rxphalignen_in                     (rxphalignen_in                    ),
      .rxphdlypd_in                       (rxphdlypd_in                      ),
      .rxphdlyreset_in                    (rxphdlyreset_in                   ),
      .rxpllclksel_in                     (rxpllclksel_in                    ),
      .rxpmareset_in                      (rxpmareset_in                     ),
      .rxpolarity_in                      (rxpolarity_in                     ),
      .rxprbscntreset_in                  (rxprbscntreset_in                 ),
      .rxprbssel_in                       (rxprbssel_in                      ),
      .rxprogdivreset_in                  (rxprogdivreset_in                 ),
      .rxrate_in                          (rxrate_in                         ),
      .rxratemode_in                      (rxratemode_in                     ),
      .rxslide_in                         (rxslide_in                        ),
      .rxslipoutclk_in                    (rxslipoutclk_in                   ),
      .rxslippma_in                       (rxslippma_in                      ),
      .rxsyncallin_in                     (rxsyncallin_in                    ),
      .rxsyncin_in                        (rxsyncin_in                       ),
      .rxsyncmode_in                      (rxsyncmode_in                     ),
      .rxsysclksel_in                     (rxsysclksel_in                    ),
      .rxtermination_in                   (rxtermination_in                  ),
      .rxuserrdy_in                       (rxuserrdy_in                      ),
      .rxusrclk_in                        (rxusrclk_in                       ),
      .rxusrclk2_in                       (rxusrclk2_in                      ),
      .sigvalidclk_in                     (sigvalidclk_in                    ),
      .tstin_in                           (tstin_in                          ),
      .tx8b10bbypass_in                   (tx8b10bbypass_in                  ),
      .tx8b10ben_in                       (tx8b10ben_in                      ),
      .txcominit_in                       (txcominit_in                      ),
      .txcomsas_in                        (txcomsas_in                       ),
      .txcomwake_in                       (txcomwake_in                      ),
      .txctrl0_in                         (txctrl0_in                        ),
      .txctrl1_in                         (txctrl1_in                        ),
      .txctrl2_in                         (txctrl2_in                        ),
      .txdata_in                          (txdata_in                         ),
      .txdataextendrsvd_in                (txdataextendrsvd_in               ),
      .txdccforcestart_in                 (txdccforcestart_in                ),
      .txdccreset_in                      (txdccreset_in                     ),
      .txdeemph_in                        (txdeemph_in                       ),
      .txdetectrx_in                      (txdetectrx_in                     ),
      .txdiffctrl_in                      (txdiffctrl_in                     ),
      .txdlybypass_in                     (txdlybypass_in                    ),
      .txdlyen_in                         (txdlyen_in                        ),
      .txdlyhold_in                       (txdlyhold_in                      ),
      .txdlyovrden_in                     (txdlyovrden_in                    ),
      .txdlysreset_in                     (txdlysreset_in                    ),
      .txdlyupdown_in                     (txdlyupdown_in                    ),
      .txelecidle_in                      (txelecidle_in                     ),
      .txheader_in                        (txheader_in                       ),
      .txinhibit_in                       (txinhibit_in                      ),
      .txlatclk_in                        (txlatclk_in                       ),
      .txlfpstreset_in                    (txlfpstreset_in                   ),
      .txlfpsu2lpexit_in                  (txlfpsu2lpexit_in                 ),
      .txlfpsu3wake_in                    (txlfpsu3wake_in                   ),
      .txmaincursor_in                    (txmaincursor_in                   ),
      .txmargin_in                        (txmargin_in                       ),
      .txmuxdcdexhold_in                  (txmuxdcdexhold_in                 ),
      .txmuxdcdorwren_in                  (txmuxdcdorwren_in                 ),
      .txoneszeros_in                     (txoneszeros_in                    ),
      .txoutclksel_in                     (txoutclksel_in                    ),
      .txpcsreset_in                      (txpcsreset_in                     ),
      .txpd_in                            (txpd_in                           ),
      .txpdelecidlemode_in                (txpdelecidlemode_in               ),
      .txphalign_in                       (txphalign_in                      ),
      .txphalignen_in                     (txphalignen_in                    ),
      .txphdlypd_in                       (txphdlypd_in                      ),
      .txphdlyreset_in                    (txphdlyreset_in                   ),
      .txphdlytstclk_in                   (txphdlytstclk_in                  ),
      .txphinit_in                        (txphinit_in                       ),
      .txphovrden_in                      (txphovrden_in                     ),
      .txpippmen_in                       (txpippmen_in                      ),
      .txpippmovrden_in                   (txpippmovrden_in                  ),
      .txpippmpd_in                       (txpippmpd_in                      ),
      .txpippmsel_in                      (txpippmsel_in                     ),
      .txpippmstepsize_in                 (txpippmstepsize_in                ),
      .txpisopd_in                        (txpisopd_in                       ),
      .txpllclksel_in                     (txpllclksel_in                    ),
      .txpmareset_in                      (txpmareset_in                     ),
      .txpolarity_in                      (txpolarity_in                     ),
      .txpostcursor_in                    (txpostcursor_in                   ),
      .txprbsforceerr_in                  (txprbsforceerr_in                 ),
      .txprbssel_in                       (txprbssel_in                      ),
      .txprecursor_in                     (txprecursor_in                    ),
      .txprogdivreset_in                  (txprogdivreset_in                 ),
      .txrate_in                          (txrate_in                         ),
      .txratemode_in                      (txratemode_in                     ),
      .txsequence_in                      (txsequence_in                     ),
      .txswing_in                         (txswing_in                        ),
      .txsyncallin_in                     (txsyncallin_in                    ),
      .txsyncin_in                        (txsyncin_in                       ),
      .txsyncmode_in                      (txsyncmode_in                     ),
      .txsysclksel_in                     (txsysclksel_in                    ),
      .txuserrdy_in                       (txuserrdy_in                      ),
      .txusrclk_in                        (txusrclk_in                       ),
      .txusrclk2_in                       (txusrclk2_in                      ),
      .bufgtce_out                        (bufgtce_out                       ),
      .bufgtcemask_out                    (bufgtcemask_out                   ),
      .bufgtdiv_out                       (bufgtdiv_out                      ),
      .bufgtreset_out                     (bufgtreset_out                    ),
      .bufgtrstmask_out                   (bufgtrstmask_out                  ),
      .cpllfbclklost_out                  (cpllfbclklost_out                 ),
      .cplllock_out                       (cplllock_out                      ),
      .cpllrefclklost_out                 (cpllrefclklost_out                ),
      .dmonitorout_out                    (dmonitorout_out                   ),
      .dmonitoroutclk_out                 (dmonitoroutclk_out                ),
      .drpdo_out                          (drpdo_out                         ),
      .drprdy_out                         (drprdy_out                        ),
      .eyescandataerror_out               (eyescandataerror_out              ),
      .gtpowergood_out                    (gtpowergood_out                   ),
      .gtrefclkmonitor_out                (gtrefclkmonitor_out               ),
      .gtytxn_out                         (gtytxn_out                        ),
      .gtytxp_out                         (gtytxp_out                        ),
      .pcierategen3_out                   (pcierategen3_out                  ),
      .pcierateidle_out                   (pcierateidle_out                  ),
      .pcierateqpllpd_out                 (pcierateqpllpd_out                ),
      .pcierateqpllreset_out              (pcierateqpllreset_out             ),
      .pciesynctxsyncdone_out             (pciesynctxsyncdone_out            ),
      .pcieusergen3rdy_out                (pcieusergen3rdy_out               ),
      .pcieuserphystatusrst_out           (pcieuserphystatusrst_out          ),
      .pcieuserratestart_out              (pcieuserratestart_out             ),
      .pcsrsvdout_out                     (pcsrsvdout_out                    ),
      .phystatus_out                      (phystatus_out                     ),
      .pinrsrvdas_out                     (pinrsrvdas_out                    ),
      .powerpresent_out                   (powerpresent_out                  ),
      .resetexception_out                 (resetexception_out                ),
      .rxbufstatus_out                    (rxbufstatus_out                   ),
      .rxbyteisaligned_out                (rxbyteisaligned_out               ),
      .rxbyterealign_out                  (rxbyterealign_out                 ),
      .rxcdrlock_out                      (rxcdrlock_out                     ),
      .rxcdrphdone_out                    (rxcdrphdone_out                   ),
      .rxchanbondseq_out                  (rxchanbondseq_out                 ),
      .rxchanisaligned_out                (rxchanisaligned_out               ),
      .rxchanrealign_out                  (rxchanrealign_out                 ),
      .rxchbondo_out                      (rxchbondo_out                     ),
      .rxckcaldone_out                    (rxckcaldone_out                   ),
      .rxclkcorcnt_out                    (rxclkcorcnt_out                   ),
      .rxcominitdet_out                   (rxcominitdet_out                  ),
      .rxcommadet_out                     (rxcommadet_out                    ),
      .rxcomsasdet_out                    (rxcomsasdet_out                   ),
      .rxcomwakedet_out                   (rxcomwakedet_out                  ),
      .rxctrl0_out                        (rxctrl0_out                       ),
      .rxctrl1_out                        (rxctrl1_out                       ),
      .rxctrl2_out                        (rxctrl2_out                       ),
      .rxctrl3_out                        (rxctrl3_out                       ),
      .rxdata_out                         (rxdata_out                        ),
      .rxdataextendrsvd_out               (rxdataextendrsvd_out              ),
      .rxdatavalid_out                    (rxdatavalid_out                   ),
      .rxdlysresetdone_out                (rxdlysresetdone_out               ),
      .rxelecidle_out                     (rxelecidle_out                    ),
      .rxheader_out                       (rxheader_out                      ),
      .rxheadervalid_out                  (rxheadervalid_out                 ),
      .rxlfpstresetdet_out                (rxlfpstresetdet_out               ),
      .rxlfpsu2lpexitdet_out              (rxlfpsu2lpexitdet_out             ),
      .rxlfpsu3wakedet_out                (rxlfpsu3wakedet_out               ),
      .rxmonitorout_out                   (rxmonitorout_out                  ),
      .rxosintdone_out                    (rxosintdone_out                   ),
      .rxosintstarted_out                 (rxosintstarted_out                ),
      .rxosintstrobedone_out              (rxosintstrobedone_out             ),
      .rxosintstrobestarted_out           (rxosintstrobestarted_out          ),
      .rxoutclk_out                       (rxoutclk_out                      ),
      .rxoutclkfabric_out                 (rxoutclkfabric_out                ),
      .rxoutclkpcs_out                    (rxoutclkpcs_out                   ),
      .rxphaligndone_out                  (rxphaligndone_out                 ),
      .rxphalignerr_out                   (rxphalignerr_out                  ),
      .rxpmaresetdone_out                 (rxpmaresetdone_out                ),
      .rxprbserr_out                      (rxprbserr_out                     ),
      .rxprbslocked_out                   (rxprbslocked_out                  ),
      .rxprgdivresetdone_out              (rxprgdivresetdone_out             ),
      .rxratedone_out                     (rxratedone_out                    ),
      .rxrecclkout_out                    (rxrecclkout_out                   ),
      .rxresetdone_out                    (rxresetdone_out                   ),
      .rxsliderdy_out                     (rxsliderdy_out                    ),
      .rxslipdone_out                     (rxslipdone_out                    ),
      .rxslipoutclkrdy_out                (rxslipoutclkrdy_out               ),
      .rxslippmardy_out                   (rxslippmardy_out                  ),
      .rxstartofseq_out                   (rxstartofseq_out                  ),
      .rxstatus_out                       (rxstatus_out                      ),
      .rxsyncdone_out                     (rxsyncdone_out                    ),
      .rxsyncout_out                      (rxsyncout_out                     ),
      .rxvalid_out                        (rxvalid_out                       ),
      .txbufstatus_out                    (txbufstatus_out                   ),
      .txcomfinish_out                    (txcomfinish_out                   ),
      .txdccdone_out                      (txdccdone_out                     ),
      .txdlysresetdone_out                (txdlysresetdone_out               ),
      .txoutclk_out                       (txoutclk_out                      ),
      .txoutclkfabric_out                 (txoutclkfabric_out                ),
      .txoutclkpcs_out                    (txoutclkpcs_out                   ),
      .txphaligndone_out                  (txphaligndone_out                 ),
      .txphinitdone_out                   (txphinitdone_out                  ),
      .txpmaresetdone_out                 (txpmaresetdone_out                ),
      .txprgdivresetdone_out              (txprgdivresetdone_out             ),
      .txratedone_out                     (txratedone_out                    ),
      .txresetdone_out                    (txresetdone_out                   ),
      .txsyncdone_out                     (txsyncdone_out                    ),
      .txsyncout_out                      (txsyncout_out                     )
    );

    // Drive unused outputs to constant values
    assign rxrecclk0_sel_out = {{`ice_gth_t_SF_CM}{1'b0}};
    assign rxrecclk1_sel_out = {{`ice_gth_t_SF_CM}{1'b0}};
    assign tcongpo_out       = {{`ice_gth_t_SF_CM}{1'b0}};
    assign tconrsvdout0_out  = {{`ice_gth_t_SF_CM}{1'b0}};
    assign gthtxn_out        = {{`ice_gth_t_N_CH}{1'b0}};
    assign gthtxp_out        = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxqpisenn_out     = {{`ice_gth_t_N_CH}{1'b0}};
    assign rxqpisenp_out     = {{`ice_gth_t_N_CH}{1'b0}};
    assign txqpisenn_out     = {{`ice_gth_t_N_CH}{1'b0}};
    assign txqpisenp_out     = {{`ice_gth_t_N_CH}{1'b0}};

  end
  endgenerate


endmodule

`define ice_gth_t_gtwizard_gthe3_MAX_NUM_CHANNELS 192
`define ice_gth_t_gtwizard_gthe3_MAX_NUM_COMMONS 48
`define ice_gth_t_gtwizard_gthe3_N_CM C_TOTAL_NUM_COMMONS
`define ice_gth_t_gtwizard_gthe3_N_CH C_TOTAL_NUM_CHANNELS
`define ice_gth_t_gtwizard_gthe3_SF_CM C_COMMON_SCALING_FACTOR
`define ice_gth_t_gtwizard_gthe3_INCLUDE_CPLL_CAL__EXCLUDE 0
`define ice_gth_t_gtwizard_gthe3_INCLUDE_CPLL_CAL__INCLUDE 1
`define ice_gth_t_gtwizard_gthe3_INCLUDE_CPLL_CAL__DEPENDENT 2
`define ice_gth_t_gtwizard_gthe3_LOCATE_RESET_CONTROLLER__CORE 0
`define ice_gth_t_gtwizard_gthe3_LOCATE_RESET_CONTROLLER__EXAMPLE_DESIGN 1
`define ice_gth_t_gtwizard_gthe3_LOCATE_USER_DATA_WIDTH_SIZING__CORE 0
`define ice_gth_t_gtwizard_gthe3_LOCATE_USER_DATA_WIDTH_SIZING__EXAMPLE_DESIGN 1
`define ice_gth_t_gtwizard_gthe3_LOCATE_RX_BUFFER_BYPASS_CONTROLLER__CORE 0
`define ice_gth_t_gtwizard_gthe3_LOCATE_RX_BUFFER_BYPASS_CONTROLLER__EXAMPLE_DESIGN 1
`define ice_gth_t_gtwizard_gthe3_LOCATE_RX_USER_CLOCKING__CORE 0
`define ice_gth_t_gtwizard_gthe3_LOCATE_RX_USER_CLOCKING__EXAMPLE_DESIGN 1
`define ice_gth_t_gtwizard_gthe3_LOCATE_TX_BUFFER_BYPASS_CONTROLLER__CORE 0
`define ice_gth_t_gtwizard_gthe3_LOCATE_TX_BUFFER_BYPASS_CONTROLLER__EXAMPLE_DESIGN 1
`define ice_gth_t_gtwizard_gthe3_LOCATE_TX_USER_CLOCKING__CORE 0
`define ice_gth_t_gtwizard_gthe3_LOCATE_TX_USER_CLOCKING__EXAMPLE_DESIGN 1
`define ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__PER_CHANNEL 0
`define ice_gth_t_gtwizard_gthe3_RX_BUFFBYPASS_MODE__AUTO 0
`define ice_gth_t_gtwizard_gthe3_RX_BUFFBYPASS_MODE__MANUAL 1
`define ice_gth_t_gtwizard_gthe3_RX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_gtwizard_gthe3_RX_BUFFER_BYPASS_INSTANCE_CTRL__PER_CHANNEL 1
`define ice_gth_t_gtwizard_gthe3_RX_BUFFER_MODE__BYPASS 0
`define ice_gth_t_gtwizard_gthe3_RX_BUFFER_MODE__USE 1
`define ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__RAW 0
`define ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__8B10B 1
`define ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__64B66B 2
`define ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__64B66B_CAUI 3
`define ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__64B66B_ASYNC 4
`define ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__64B66B_ASYNC_CAUI 5
`define ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__64B67B 6
`define ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__64B67B_CAUI 7
`define ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__128B130B 10
`define ice_gth_t_gtwizard_gthe3_RX_ENABLE__DISABLED 0
`define ice_gth_t_gtwizard_gthe3_RX_ENABLE__ENABLED 1
`define ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0 0
`define ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1 1
`define ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL 2
`define ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_CONTENTS__BUFG_GT 0
`define ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_CONTENTS__BUFG 1
`define ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_CONTENTS__MMCM 2
`define ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_INSTANCE_CTRL__PER_CHANNEL 1
`define ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__RXOUTCLK 0
`define ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__IBUFDS 1
`define ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__TXOUTCLK 2
`define ice_gth_t_gtwizard_gthe3_TXPROGDIV_FREQ_ENABLE__DISABLED 0
`define ice_gth_t_gtwizard_gthe3_TXPROGDIV_FREQ_ENABLE__ENABLED 1
`define ice_gth_t_gtwizard_gthe3_TXPROGDIV_FREQ_SOURCE__QPLL0 0
`define ice_gth_t_gtwizard_gthe3_TXPROGDIV_FREQ_SOURCE__QPLL1 1
`define ice_gth_t_gtwizard_gthe3_TXPROGDIV_FREQ_SOURCE__CPLL 2
`define ice_gth_t_gtwizard_gthe3_TX_BUFFBYPASS_MODE__AUTO 0
`define ice_gth_t_gtwizard_gthe3_TX_BUFFBYPASS_MODE__MANUAL 1
`define ice_gth_t_gtwizard_gthe3_TX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_gtwizard_gthe3_TX_BUFFER_BYPASS_INSTANCE_CTRL__PER_CHANNEL 1
`define ice_gth_t_gtwizard_gthe3_TX_BUFFER_MODE__BYPASS 0
`define ice_gth_t_gtwizard_gthe3_TX_BUFFER_MODE__USE 1
`define ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__RAW 0
`define ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__8B10B 1
`define ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__64B66B 2
`define ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__64B66B_CAUI 3
`define ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__64B66B_ASYNC 4
`define ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__64B66B_ASYNC_CAUI 5
`define ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__64B67B 6
`define ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__64B67B_CAUI 7
`define ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__128B130B 10
`define ice_gth_t_gtwizard_gthe3_TX_ENABLE__DISABLED 0
`define ice_gth_t_gtwizard_gthe3_TX_ENABLE__ENABLED 1
`define ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0 0
`define ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1 1
`define ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL 2
`define ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_CONTENTS__BUFG_GT 0
`define ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_CONTENTS__BUFG 1
`define ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_CONTENTS__MMCM 2
`define ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE 0
`define ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_INSTANCE_CTRL__PER_CHANNEL 1
`define ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_SOURCE__TXOUTCLK 0
`define ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_SOURCE__IBUFDS 1

module ice_gth_t_gtwizard_gthe3 #(

  parameter [191:0] C_CHANNEL_ENABLE                          = 192'b1,
  parameter integer C_COMMON_SCALING_FACTOR                   = 1,
  parameter real    C_CPLL_VCO_FREQUENCY                      = 5156.25,
  parameter real    C_FREERUN_FREQUENCY                       = 200,
  parameter integer C_GT_REV                                  = 17,
  parameter integer C_INCLUDE_CPLL_CAL                        = `ice_gth_t_gtwizard_gthe3_INCLUDE_CPLL_CAL__DEPENDENT,
  parameter integer C_LOCATE_RESET_CONTROLLER                 = `ice_gth_t_gtwizard_gthe3_LOCATE_RESET_CONTROLLER__CORE,
  parameter integer C_LOCATE_USER_DATA_WIDTH_SIZING           = `ice_gth_t_gtwizard_gthe3_LOCATE_USER_DATA_WIDTH_SIZING__CORE,
  parameter integer C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER      = `ice_gth_t_gtwizard_gthe3_LOCATE_RX_BUFFER_BYPASS_CONTROLLER__CORE,
  parameter integer C_LOCATE_RX_USER_CLOCKING                 = `ice_gth_t_gtwizard_gthe3_LOCATE_RX_USER_CLOCKING__EXAMPLE_DESIGN,
  parameter integer C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER      = `ice_gth_t_gtwizard_gthe3_LOCATE_TX_BUFFER_BYPASS_CONTROLLER__CORE,
  parameter integer C_LOCATE_TX_USER_CLOCKING                 = `ice_gth_t_gtwizard_gthe3_LOCATE_TX_USER_CLOCKING__EXAMPLE_DESIGN,
  parameter integer C_RESET_CONTROLLER_INSTANCE_CTRL          = `ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_RX_BUFFBYPASS_MODE                      = `ice_gth_t_gtwizard_gthe3_RX_BUFFBYPASS_MODE__AUTO,
  parameter integer C_RX_BUFFER_BYPASS_INSTANCE_CTRL          = `ice_gth_t_gtwizard_gthe3_RX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_RX_BUFFER_MODE                          = `ice_gth_t_gtwizard_gthe3_RX_BUFFER_MODE__USE,
  parameter integer C_RX_DATA_DECODING                        = `ice_gth_t_gtwizard_gthe3_RX_DATA_DECODING__RAW,
  parameter integer C_RX_ENABLE                               = `ice_gth_t_gtwizard_gthe3_RX_ENABLE__ENABLED,
  parameter integer C_RX_INT_DATA_WIDTH                       = 32,
  parameter real    C_RX_LINE_RATE                            = 10.3125,
  parameter integer C_RX_MASTER_CHANNEL_IDX                   = 0,
  parameter integer C_RX_OUTCLK_BUFG_GT_DIV                   = 1,
  parameter integer C_RX_PLL_TYPE                             = `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0,
  parameter integer C_RX_USER_CLOCKING_CONTENTS               = `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_CONTENTS__BUFG_GT,
  parameter integer C_RX_USER_CLOCKING_INSTANCE_CTRL          = `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 = 1,
  parameter integer C_RX_USER_CLOCKING_SOURCE                 = `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__RXOUTCLK,
  parameter integer C_RX_USER_DATA_WIDTH                      = 32,
  parameter integer C_TOTAL_NUM_CHANNELS                      = 1,
  parameter integer C_TOTAL_NUM_COMMONS                       = 1,
  parameter integer C_TXPROGDIV_FREQ_ENABLE                   = `ice_gth_t_gtwizard_gthe3_TXPROGDIV_FREQ_ENABLE__DISABLED,
  parameter integer C_TXPROGDIV_FREQ_SOURCE                   = `ice_gth_t_gtwizard_gthe3_TXPROGDIV_FREQ_SOURCE__QPLL0,
  parameter integer C_TX_BUFFBYPASS_MODE                      = `ice_gth_t_gtwizard_gthe3_TX_BUFFBYPASS_MODE__AUTO,
  parameter integer C_TX_BUFFER_BYPASS_INSTANCE_CTRL          = `ice_gth_t_gtwizard_gthe3_TX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_TX_BUFFER_MODE                          = `ice_gth_t_gtwizard_gthe3_TX_BUFFER_MODE__USE,
  parameter integer C_TX_DATA_ENCODING                        = `ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__RAW,
  parameter integer C_TX_ENABLE                               = `ice_gth_t_gtwizard_gthe3_TX_ENABLE__ENABLED,
  parameter integer C_TX_INT_DATA_WIDTH                       = 32,
  parameter integer C_TX_MASTER_CHANNEL_IDX                   = 0,
  parameter integer C_TX_OUTCLK_BUFG_GT_DIV                   = 1,
  parameter integer C_TX_PLL_TYPE                             = `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0,
  parameter integer C_TX_USER_CLOCKING_CONTENTS               = `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_CONTENTS__BUFG_GT,
  parameter integer C_TX_USER_CLOCKING_INSTANCE_CTRL          = `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE,
  parameter integer C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2 = 1,
  parameter integer C_TX_USER_CLOCKING_SOURCE                 = `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_SOURCE__TXOUTCLK,
  parameter integer C_TX_USER_DATA_WIDTH                      = 32

)(

  // Transmitter user clocking network helper block ports
  input  wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_tx_reset_in,
  input  wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_tx_active_in,
  output wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_tx_srcclk_out,
  output wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_tx_usrclk_out,
  output wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_tx_usrclk2_out,
  output wire [(C_TX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_tx_active_out,

  // Receiver user clocking network helper block ports
  input  wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_rx_reset_in,
  input  wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_rx_active_in,
  output wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_rx_srcclk_out,
  output wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_rx_usrclk_out,
  output wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_rx_usrclk2_out,
  output wire [(C_RX_USER_CLOCKING_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_userclk_rx_active_out,

  // Transmitter buffer bypass controller helper block ports
  input  wire [(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_buffbypass_tx_reset_in,
  input  wire [(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_buffbypass_tx_start_user_in,
  output wire [(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_buffbypass_tx_done_out,
  output wire [(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_buffbypass_tx_error_out,

  // Receiver buffer bypass controller helper block ports
  input  wire [(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_buffbypass_rx_reset_in,
  input  wire [(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_buffbypass_rx_start_user_in,
  output wire [(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_buffbypass_rx_done_out,
  output wire [(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_buffbypass_rx_error_out,

  // Reset controller helper block ports
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_clk_freerun_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_all_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_tx_pll_and_datapath_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_tx_datapath_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_rx_pll_and_datapath_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_rx_datapath_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_tx_done_in,
  input  wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_rx_done_in,
  input  wire [                                  (`ice_gth_t_gtwizard_gthe3_SF_CM-1):0] gtwiz_reset_qpll0lock_in,
  input  wire [                                  (`ice_gth_t_gtwizard_gthe3_SF_CM-1):0] gtwiz_reset_qpll1lock_in,
  output wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_rx_cdr_stable_out,
  output wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_tx_done_out,
  output wire [(C_RESET_CONTROLLER_INSTANCE_CTRL*(`ice_gth_t_gtwizard_gthe3_N_CH-1)):0] gtwiz_reset_rx_done_out,
  output wire [                                  (`ice_gth_t_gtwizard_gthe3_SF_CM-1):0] gtwiz_reset_qpll0reset_out,
  output wire [                                  (`ice_gth_t_gtwizard_gthe3_SF_CM-1):0] gtwiz_reset_qpll1reset_out,

  // CPLL calibration block ports
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 18)-1:0] gtwiz_gthe3_cpll_cal_txoutclk_period_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 18)-1:0] gtwiz_gthe3_cpll_cal_cnt_tol_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtwiz_gthe3_cpll_cal_bufg_ce_in,

  // Transmitter user data width sizing helper block ports
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*C_TX_USER_DATA_WIDTH)-1:0] gtwiz_userdata_tx_in,

  // Receiver user data width sizing helper block ports
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*C_RX_USER_DATA_WIDTH)-1:0] gtwiz_userdata_rx_out,

  // Transceiver common block ports
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] bgbypassb_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] bgmonitorenb_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] bgpdb_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  5)-1:0] bgrcalovrd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] bgrcalovrdenb_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  9)-1:0] drpaddr_common_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] drpclk_common_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 16)-1:0] drpdi_common_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] drpen_common_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] drpwe_common_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtgrefclk0_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtgrefclk1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtnorthrefclk00_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtnorthrefclk01_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtnorthrefclk10_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtnorthrefclk11_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtrefclk00_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtrefclk01_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtrefclk10_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtrefclk11_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtsouthrefclk00_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtsouthrefclk01_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtsouthrefclk10_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] gtsouthrefclk11_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  8)-1:0] pmarsvd0_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  8)-1:0] pmarsvd1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0clkrsvd0_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0clkrsvd1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0lockdetclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0locken_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0pd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  3)-1:0] qpll0refclksel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0reset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1clkrsvd0_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1clkrsvd1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1lockdetclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1locken_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1pd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  3)-1:0] qpll1refclksel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1reset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  8)-1:0] qpllrsvd1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  5)-1:0] qpllrsvd2_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  5)-1:0] qpllrsvd3_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  8)-1:0] qpllrsvd4_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] rcalenb_in,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 16)-1:0] drpdo_common_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] drprdy_common_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  8)-1:0] pmarsvdout0_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  8)-1:0] pmarsvdout1_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0fbclklost_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0lock_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0outclk_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0outrefclk_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll0refclklost_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1fbclklost_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1lock_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1outclk_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1outrefclk_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] qpll1refclklost_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  8)-1:0] qplldmonitor0_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  8)-1:0] qplldmonitor1_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] refclkoutmonitor0_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  1)-1:0] refclkoutmonitor1_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  2)-1:0] rxrecclk0_sel_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*  2)-1:0] rxrecclk1_sel_out,

  // Transceiver channel block ports
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cfgreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] clkrsvd0_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] clkrsvd1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cplllockdetclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cplllocken_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllpd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] cpllrefclksel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] dmonfiforeset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] dmonitorclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  9)-1:0] drpaddr_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drpclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] drpdi_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drpen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drpwe_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphicaldone_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphicalstart_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphidrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphidwren_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphixrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphixwren_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] eyescanmode_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] eyescanreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] eyescantrigger_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtgrefclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gthrxn_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gthrxp_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtnorthrefclk0_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtnorthrefclk1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtrefclk0_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtrefclk1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtresetsel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] gtrsvd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtrxreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtsouthrefclk0_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtsouthrefclk1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gttxreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] loopback_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] lpbkrxtxseren_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] lpbktxrxseren_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieeqrxeqadaptdone_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcierstidle_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pciersttxsyncstart_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieuserratedone_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] pcsrsvdin_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] pcsrsvdin2_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] pmarsvdin_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] qpll0clk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] qpll0refclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] qpll1clk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] qpll1refclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] resetovrd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rstclkentx_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rx8b10ben_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxbufreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrfreqreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrhold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrresetrsv_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchbonden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] rxchbondi_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxchbondlevel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchbondmaster_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchbondslave_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcommadeten_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxdfeagcctrl_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfeagchold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfeagcovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfelfhold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfelfovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfelpmreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap10hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap10ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap11hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap11ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap12hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap12ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap13hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap13ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap14hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap14ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap15hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap15ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap2hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap2ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap3hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap3ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap4hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap4ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap5hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap5ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap6hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap6ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap7hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap7ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap8hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap8ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap9hold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap9ovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfeuthold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfeutovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfevphold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfevpovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfevsen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfexyden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlybypass_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlyen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlyovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlysreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxelecidlemode_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxgearboxslip_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlatclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmgchold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmgcovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmhfhold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmhfovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmlfhold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmlfklovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmoshold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmosovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxmcommaalignen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxmonitorsel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoobreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoscalreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoshold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  4)-1:0] rxosintcfg_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosinten_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosinthold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintstrobe_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosinttestovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxoutclksel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpcommaalignen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpcsreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxpd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphalign_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphalignen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphdlypd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphdlyreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxpllclksel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpmareset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpolarity_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprbscntreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  4)-1:0] rxprbssel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprogdivreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxqpien_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxrate_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxratemode_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslide_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslipoutclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslippma_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncallin_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncin_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncmode_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxsysclksel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxuserrdy_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxusrclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxusrclk2_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] sigvalidclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 20)-1:0] tstin_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] tx8b10bbypass_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] tx8b10ben_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txbufdiffctrl_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txcominit_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txcomsas_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txcomwake_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] txctrl0_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] txctrl1_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] txctrl2_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*128)-1:0] txdata_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] txdataextendrsvd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdeemph_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdetectrx_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  4)-1:0] txdiffctrl_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdiffpd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlybypass_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlyen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlyhold_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlyovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlysreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlyupdown_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txelecidle_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  6)-1:0] txheader_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txinhibit_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txlatclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  7)-1:0] txmaincursor_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txmargin_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txoutclksel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpcsreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] txpd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpdelecidlemode_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphalign_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphalignen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphdlypd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphdlyreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphdlytstclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphinit_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpippmen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpippmovrden_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpippmpd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpippmsel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] txpippmstepsize_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpisopd_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] txpllclksel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpmareset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpolarity_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] txpostcursor_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpostcursorinv_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txprbsforceerr_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  4)-1:0] txprbssel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] txprecursor_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txprecursorinv_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txprogdivreset_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpibiasen_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpistrongpdown_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpiweakpup_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txrate_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txratemode_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  7)-1:0] txsequence_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txswing_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncallin_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncin_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncmode_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] txsysclksel_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txuserrdy_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txusrclk_in,
  input  wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txusrclk2_in,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] bufgtce_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] bufgtcemask_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  9)-1:0] bufgtdiv_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] bufgtreset_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] bufgtrstmask_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllfbclklost_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cplllock_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllrefclklost_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 17)-1:0] dmonitorout_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] drpdo_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drprdy_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] eyescandataerror_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gthtxn_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gthtxp_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtpowergood_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtrefclkmonitor_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcierategen3_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcierateidle_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] pcierateqpllpd_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] pcierateqpllreset_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pciesynctxsyncdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieusergen3rdy_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieuserphystatusrst_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieuserratestart_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 12)-1:0] pcsrsvdout_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] phystatus_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] pinrsrvdas_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] resetexception_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxbufstatus_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxbyteisaligned_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxbyterealign_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrlock_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrphdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchanbondseq_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchanisaligned_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchanrealign_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] rxchbondo_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxclkcorcnt_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcominitdet_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcommadet_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcomsasdet_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcomwakedet_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] rxctrl0_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] rxctrl1_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] rxctrl2_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] rxctrl3_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*128)-1:0] rxdata_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] rxdataextendrsvd_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxdatavalid_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlysresetdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxelecidle_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  6)-1:0] rxheader_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxheadervalid_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  7)-1:0] rxmonitorout_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintstarted_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintstrobedone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintstrobestarted_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoutclk_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoutclkfabric_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoutclkpcs_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphaligndone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphalignerr_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpmaresetdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprbserr_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprbslocked_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprgdivresetdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxqpisenn_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxqpisenp_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxratedone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxrecclkout_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxresetdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsliderdy_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslipdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslipoutclkrdy_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslippmardy_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxstartofseq_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxstatus_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncout_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxvalid_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] txbufstatus_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txcomfinish_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlysresetdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txoutclk_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txoutclkfabric_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txoutclkpcs_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphaligndone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphinitdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpmaresetdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txprgdivresetdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpisenn_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpisenp_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txratedone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txresetdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncdone_out,
  output wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncout_out

);


  // ================================================================================================================
  // LOCAL PARAMETERS AND FUNCTIONS
  // ================================================================================================================

  // ----------------------------------------------------------------------------------------------------------------
  // Declare and initialize local parameters used for HDL generation
  // ----------------------------------------------------------------------------------------------------------------
  localparam [`ice_gth_t_gtwizard_gthe3_MAX_NUM_COMMONS-1:0] P_COMMON_ENABLE = f_pop_cm_en(0);
  localparam integer P_TX_MASTER_CH_PACKED_IDX = f_calc_pk_mc_idx(C_TX_MASTER_CHANNEL_IDX);
  localparam integer P_RX_MASTER_CH_PACKED_IDX = f_calc_pk_mc_idx(C_RX_MASTER_CHANNEL_IDX);
  localparam integer P_USE_CPLL_CAL                   = (C_GT_REV == 11 || C_GT_REV == 12 || C_GT_REV == 14 || C_GT_REV == 16) ? 1 : 0;
  localparam [15:0] P_CPLL_CAL_FREQ_COUNT_WINDOW      = 16'd16000;
  localparam [17:0] P_CPLL_CAL_TXOUTCLK_PERIOD        = (C_CPLL_VCO_FREQUENCY/20) * (P_CPLL_CAL_FREQ_COUNT_WINDOW/(4*C_FREERUN_FREQUENCY));
  localparam [15:0] P_CPLL_CAL_WAIT_DEASSERT_CPLLPD   = 16'd256;
  localparam [17:0] P_CPLL_CAL_TXOUTCLK_PERIOD_DIV100 = (C_CPLL_VCO_FREQUENCY/20) * (P_CPLL_CAL_FREQ_COUNT_WINDOW/(400*C_FREERUN_FREQUENCY));
  localparam [25:0] P_CDR_TIMEOUT_FREERUN_CYC         = (37000 * C_FREERUN_FREQUENCY) / C_RX_LINE_RATE;


  // ----------------------------------------------------------------------------------------------------------------
  // Functions used for HDL generation
  // ----------------------------------------------------------------------------------------------------------------

  // Function to populate a bit mapping of enabled transceiver common blocks to transceiver quads
  function [`ice_gth_t_gtwizard_gthe3_MAX_NUM_COMMONS-1:0] f_pop_cm_en (
    input integer in_null
  );
  begin : main_f_pop_cm_en
    integer i;
    reg [`ice_gth_t_gtwizard_gthe3_MAX_NUM_COMMONS-1:0] tmp;
    for (i = 0; i < `ice_gth_t_gtwizard_gthe3_MAX_NUM_CHANNELS; i = i + 4) begin
      if ((C_CHANNEL_ENABLE[i]   ==  1'b1) ||
          (C_CHANNEL_ENABLE[i+1] ==  1'b1) ||
          (C_CHANNEL_ENABLE[i+2] ==  1'b1) ||
          (C_CHANNEL_ENABLE[i+3] ==  1'b1))
        tmp[i/4] = 1'b1;
      else
        tmp[i/4] = 1'b0;
    end
    f_pop_cm_en = tmp;
  end
  endfunction

  // Function to calculate a pointer to a master channel's packed index
  function integer f_calc_pk_mc_idx (
    input integer idx_mc
  );
  begin : main_f_calc_pk_mc_idx
    integer i, j;
    integer tmp;
    j = 0;
    for (i = 0; i < `ice_gth_t_gtwizard_gthe3_MAX_NUM_CHANNELS; i = i + 1) begin
      if (C_CHANNEL_ENABLE[i] == 1'b1) begin
        if (i == idx_mc)
          tmp = j;
        else
          j = j + 1;
      end
    end
    f_calc_pk_mc_idx = tmp;
  end
  endfunction

  // Function to calculate the upper bound of a transceiver common-related signal within a packed vector, for a given
  // signal width and unpacked common index
  function integer f_ub_cm (
    input integer width,
    input integer index
  );
  begin : main_f_ub_cm
    integer i, j;
    j = 0;
    for (i = 0; i <= index; i = i + 4) begin
      if (C_CHANNEL_ENABLE[i]   == 1'b1 ||
          C_CHANNEL_ENABLE[i+1] == 1'b1 ||
          C_CHANNEL_ENABLE[i+2] == 1'b1 ||
          C_CHANNEL_ENABLE[i+3] == 1'b1)
        j = j + 1;
    end
    f_ub_cm = (width * j) - 1;
  end
  endfunction

  // Function to calculate the lower bound of a transceiver common-related signal within a packed vector, for a given
  // signal width and unpacked common index
  function integer f_lb_cm (
    input integer width,
    input integer index
  );
  begin : main_f_lb_cm
    integer i, j;
    j = 0;
    for (i = 0; i < index; i = i + 4) begin
      if (C_CHANNEL_ENABLE[i]   == 1'b1 ||
          C_CHANNEL_ENABLE[i+1] == 1'b1 ||
          C_CHANNEL_ENABLE[i+2] == 1'b1 ||
          C_CHANNEL_ENABLE[i+3] == 1'b1)
        j = j + 1;
    end
    f_lb_cm = (width * j);
  end
  endfunction

  // Function to calculate the number of transceiver channels enabled within a given transceiver quad
  function integer f_num_ch_in_quad (
    input integer chan_en_idx_lb
  );
  begin : main_f_num_ch_in_quad
    f_num_ch_in_quad =
      ((C_CHANNEL_ENABLE[chan_en_idx_lb+3] == 1'b1 ? 1 : 0) +
       (C_CHANNEL_ENABLE[chan_en_idx_lb+2] == 1'b1 ? 1 : 0) +
       (C_CHANNEL_ENABLE[chan_en_idx_lb+1] == 1'b1 ? 1 : 0) +
       (C_CHANNEL_ENABLE[chan_en_idx_lb]   == 1'b1 ? 1 : 0));
  end
  endfunction

  // Function to calculate the upper bound of a transceiver channel-related signal within a packed vector, for a given
  // signal width and unpacked channel index
  function integer f_ub_ch (
    input integer width,
    input integer index
  );
  begin : main_f_ub_ch
    integer i, j;
    j = 0;
    for (i = 0; i <= index; i = i + 1) begin
      if (C_CHANNEL_ENABLE[i] == 1'b1)
        j = j + 1;
    end
    f_ub_ch = (width * j) - 1;
  end
  endfunction

  // Function to calculate the lower bound of a transceiver channel-related signal within a packed vector, for a given
  // signal width and unpacked channel index
  function integer f_lb_ch (
    input integer width,
    input integer index
  );
  begin : main_f_lb_ch
    integer i, j;
    j = 0;
    for (i = 0; i < index; i = i + 1) begin
      if (C_CHANNEL_ENABLE[i] == 1'b1)
        j = j + 1;
    end
    f_lb_ch = (width * j);
  end
  endfunction

  // Function to calculate the packed vector index of a transceiver common, provided the packed vector index of the
  // associated transceiver channel
  function integer f_idx_cm (
    input integer index
  );
  begin : main_f_idx_cm
    integer i, j, k, flag, result;
    j    = 0;
    k    = 0;
    flag = 0;
    for (i = 0; (i < `ice_gth_t_gtwizard_gthe3_MAX_NUM_CHANNELS) && (flag == 0); i = i + 4) begin
      if (C_CHANNEL_ENABLE[i]   == 1'b1 ||
          C_CHANNEL_ENABLE[i+1] == 1'b1 ||
          C_CHANNEL_ENABLE[i+2] == 1'b1 ||
          C_CHANNEL_ENABLE[i+3] == 1'b1) begin
        k = k + 1;
        if (C_CHANNEL_ENABLE[i+3] == 1'b1)
          j = j + 1;
        if (C_CHANNEL_ENABLE[i+2] == 1'b1)
          j = j + 1;
        if (C_CHANNEL_ENABLE[i+1] == 1'b1)
          j = j + 1;
        if (C_CHANNEL_ENABLE[i]   == 1'b1)
          j = j + 1;
      end

      if (j >= (index + 1)) begin
        flag   = 1;
        result = k;
      end
    end
    f_idx_cm = result - 1;
  end
  endfunction

  // Function to calculate the packed vector index of the upper bound transceiver channel which is associated with the
  // provided transceiver common packed vector index
  function integer f_idx_ch_ub (
    input integer index
  );
  begin : main_f_idx_ch_ub
    integer i, j, k, flag, result;
    j    = 0;
    k    = 0;
    flag = 0;
    for (i = 0; (i < `ice_gth_t_gtwizard_gthe3_MAX_NUM_CHANNELS) && (flag == 0); i = i + 4) begin

      if (C_CHANNEL_ENABLE[i]   == 1'b1 ||
          C_CHANNEL_ENABLE[i+1] == 1'b1 ||
          C_CHANNEL_ENABLE[i+2] == 1'b1 ||
          C_CHANNEL_ENABLE[i+3] == 1'b1) begin
        k = k + 1;
        if (C_CHANNEL_ENABLE[i]   == 1'b1)
          j = j + 1;
        if (C_CHANNEL_ENABLE[i+1] == 1'b1)
          j = j + 1;
        if (C_CHANNEL_ENABLE[i+2] == 1'b1)
          j = j + 1;
        if (C_CHANNEL_ENABLE[i+3] == 1'b1)
          j = j + 1;
        if (k == index + 1) begin
          flag   = 1;
          result = j;
        end
      end

    end
    f_idx_ch_ub = result - 1;
  end
  endfunction

  // Function to calculate the packed vector index of the lower bound transceiver channel which is associated with the
  // provided transceiver common packed vector index
  function integer f_idx_ch_lb (
    input integer index
  );
  begin : main_f_idx_ch_lb
    integer i, j, k, flag, result;
    j    = 0;
    k    = 0;
    flag = 0;
    for (i = 0; (i < `ice_gth_t_gtwizard_gthe3_MAX_NUM_CHANNELS) && (flag == 0); i = i + 4) begin

      if (C_CHANNEL_ENABLE[i]   == 1'b1 ||
          C_CHANNEL_ENABLE[i+1] == 1'b1 ||
          C_CHANNEL_ENABLE[i+2] == 1'b1 ||
          C_CHANNEL_ENABLE[i+3] == 1'b1) begin
        k = k + 1;
        if (k == index + 1) begin
          flag   = 1;
          result = j + 1;
        end
        else begin
          if (C_CHANNEL_ENABLE[i]   == 1'b1)
            j = j + 1;
          if (C_CHANNEL_ENABLE[i+1] == 1'b1)
            j = j + 1;
          if (C_CHANNEL_ENABLE[i+2] == 1'b1)
            j = j + 1;
          if (C_CHANNEL_ENABLE[i+3] == 1'b1)
            j = j + 1;
        end
      end

    end
    f_idx_ch_lb = result - 1;
  end
  endfunction


  // Begin main body of generated code
  generate if (1) begin : gen_gtwizard_gthe3

    // ================================================================================================================
    // TRANSCEIVER BLOCKS
    // ================================================================================================================

    // ----------------------------------------------------------------------------------------------------------------
    // Transceiver common block declarations, assignments, and primitive wrapper conditional generation
    // ----------------------------------------------------------------------------------------------------------------

    // Declare the signal vectors used to map to transceiver common-related ports
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] bgbypassb_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] bgmonitorenb_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] bgpdb_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 5)-1:0] bgrcalovrd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] bgrcalovrdenb_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 9)-1:0] drpaddr_common_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] drpclk_common_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*16)-1:0] drpdi_common_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] drpen_common_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] drpwe_common_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtgrefclk0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtgrefclk1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtnorthrefclk00_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtnorthrefclk01_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtnorthrefclk10_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtnorthrefclk11_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtrefclk00_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtrefclk01_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtrefclk10_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtrefclk11_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtsouthrefclk00_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtsouthrefclk01_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtsouthrefclk10_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] gtsouthrefclk11_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 8)-1:0] pmarsvd0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 8)-1:0] pmarsvd1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0clkrsvd0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0clkrsvd1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0lockdetclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0locken_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0pd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 3)-1:0] qpll0refclksel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0reset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1clkrsvd0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1clkrsvd1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1lockdetclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1locken_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1pd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 3)-1:0] qpll1refclksel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1reset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 8)-1:0] qpllrsvd1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 5)-1:0] qpllrsvd2_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 5)-1:0] qpllrsvd3_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 8)-1:0] qpllrsvd4_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] rcalenb_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM*16)-1:0] drpdo_common_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] drprdy_common_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 8)-1:0] pmarsvdout0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 8)-1:0] pmarsvdout1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0fbclklost_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0lock_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0outclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0outrefclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll0refclklost_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1fbclklost_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1lock_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1outclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1outrefclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] qpll1refclklost_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 8)-1:0] qplldmonitor0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 8)-1:0] qplldmonitor1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] refclkoutmonitor0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 1)-1:0] refclkoutmonitor1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 2)-1:0] rxrecclk0_sel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_SF_CM* 2)-1:0] rxrecclk1_sel_int;

    // Assign transceiver common-related inputs to the corresponding internal signal vectors
    // Note: commented assignments are replaced by assignments associated with helper blocks, later in this module
    assign bgbypassb_int         = bgbypassb_in;
    assign bgmonitorenb_int      = bgmonitorenb_in;
    assign bgpdb_int             = bgpdb_in;
    assign bgrcalovrd_int        = bgrcalovrd_in;
    assign bgrcalovrdenb_int     = bgrcalovrdenb_in;
    assign drpaddr_common_int    = drpaddr_common_in;
    assign drpclk_common_int     = drpclk_common_in;
    assign drpdi_common_int      = drpdi_common_in;
    assign drpen_common_int      = drpen_common_in;
    assign drpwe_common_int      = drpwe_common_in;
    assign gtgrefclk0_int        = gtgrefclk0_in;
    assign gtgrefclk1_int        = gtgrefclk1_in;
    assign gtnorthrefclk00_int   = gtnorthrefclk00_in;
    assign gtnorthrefclk01_int   = gtnorthrefclk01_in;
    assign gtnorthrefclk10_int   = gtnorthrefclk10_in;
    assign gtnorthrefclk11_int   = gtnorthrefclk11_in;
    assign gtrefclk00_int        = gtrefclk00_in;
    assign gtrefclk01_int        = gtrefclk01_in;
    assign gtrefclk10_int        = gtrefclk10_in;
    assign gtrefclk11_int        = gtrefclk11_in;
    assign gtsouthrefclk00_int   = gtsouthrefclk00_in;
    assign gtsouthrefclk01_int   = gtsouthrefclk01_in;
    assign gtsouthrefclk10_int   = gtsouthrefclk10_in;
    assign gtsouthrefclk11_int   = gtsouthrefclk11_in;
    assign pmarsvd0_int          = pmarsvd0_in;
    assign pmarsvd1_int          = pmarsvd1_in;
    assign qpll0clkrsvd0_int     = qpll0clkrsvd0_in;
    assign qpll0clkrsvd1_int     = qpll0clkrsvd1_in;
    assign qpll0lockdetclk_int   = qpll0lockdetclk_in;
    assign qpll0locken_int       = qpll0locken_in;
    assign qpll0pd_int           = qpll0pd_in;
    assign qpll0refclksel_int    = qpll0refclksel_in;
  //assign qpll0reset_int        = qpll0reset_in;
    assign qpll1clkrsvd0_int     = qpll1clkrsvd0_in;
    assign qpll1clkrsvd1_int     = qpll1clkrsvd1_in;
    assign qpll1lockdetclk_int   = qpll1lockdetclk_in;
    assign qpll1locken_int       = qpll1locken_in;
    assign qpll1pd_int           = qpll1pd_in;
    assign qpll1refclksel_int    = qpll1refclksel_in;
  //assign qpll1reset_int        = qpll1reset_in;
    assign qpllrsvd1_int         = qpllrsvd1_in;
    assign qpllrsvd2_int         = qpllrsvd2_in;
    assign qpllrsvd3_int         = qpllrsvd3_in;
    assign qpllrsvd4_int         = qpllrsvd4_in;
    assign rcalenb_int           = rcalenb_in;
    assign drpdo_common_out      = drpdo_common_int;
    assign drprdy_common_out     = drprdy_common_int;
    assign pmarsvdout0_out       = pmarsvdout0_int;
    assign pmarsvdout1_out       = pmarsvdout1_int;
    assign qpll0fbclklost_out    = qpll0fbclklost_int;
    assign qpll0lock_out         = qpll0lock_int;
    assign qpll0outclk_out       = qpll0outclk_int;
    assign qpll0outrefclk_out    = qpll0outrefclk_int;
    assign qpll0refclklost_out   = qpll0refclklost_int;
    assign qpll1fbclklost_out    = qpll1fbclklost_int;
    assign qpll1lock_out         = qpll1lock_int;
    assign qpll1outclk_out       = qpll1outclk_int;
    assign qpll1outrefclk_out    = qpll1outrefclk_int;
    assign qpll1refclklost_out   = qpll1refclklost_int;
    assign qplldmonitor0_out     = qplldmonitor0_int;
    assign qplldmonitor1_out     = qplldmonitor1_int;
    assign refclkoutmonitor0_out = refclkoutmonitor0_int;
    assign refclkoutmonitor1_out = refclkoutmonitor1_int;
    assign rxrecclk0_sel_out     = rxrecclk0_sel_int;
    assign rxrecclk1_sel_out     = rxrecclk1_sel_int;

    // If transceiver common blocks are to be used within the core boundary, then for each transceiver quad which
    // contains one or more transceiver channel resources, generate an instance of the primitive wrapper containing
    // one parameterized common primitive
    if (`ice_gth_t_gtwizard_gthe3_N_CM > 0) begin : gen_common
      genvar cm;
      for (cm = 0; cm < `ice_gth_t_gtwizard_gthe3_MAX_NUM_COMMONS; cm = cm + 1) begin : gen_common_container
        if (P_COMMON_ENABLE[cm] == 1'b1) begin : gen_enabled_common

          ice_gth_t_gthe3_common_wrapper gthe3_common_wrapper_inst (
            .GTHE3_COMMON_BGBYPASSB         (bgbypassb_int         [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_BGMONITORENB      (bgmonitorenb_int      [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_BGPDB             (bgpdb_int             [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_BGRCALOVRD        (bgrcalovrd_int        [f_ub_cm( 5,(4*cm)+3) : f_lb_cm( 5,4*cm)]),
            .GTHE3_COMMON_BGRCALOVRDENB     (bgrcalovrdenb_int     [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_DRPADDR           (drpaddr_common_int    [f_ub_cm( 9,(4*cm)+3) : f_lb_cm( 9,4*cm)]),
            .GTHE3_COMMON_DRPCLK            (drpclk_common_int     [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_DRPDI             (drpdi_common_int      [f_ub_cm(16,(4*cm)+3) : f_lb_cm(16,4*cm)]),
            .GTHE3_COMMON_DRPEN             (drpen_common_int      [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_DRPWE             (drpwe_common_int      [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTGREFCLK0        (gtgrefclk0_int        [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTGREFCLK1        (gtgrefclk1_int        [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTNORTHREFCLK00   (gtnorthrefclk00_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTNORTHREFCLK01   (gtnorthrefclk01_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTNORTHREFCLK10   (gtnorthrefclk10_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTNORTHREFCLK11   (gtnorthrefclk11_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTREFCLK00        (gtrefclk00_int        [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTREFCLK01        (gtrefclk01_int        [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTREFCLK10        (gtrefclk10_int        [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTREFCLK11        (gtrefclk11_int        [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTSOUTHREFCLK00   (gtsouthrefclk00_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTSOUTHREFCLK01   (gtsouthrefclk01_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTSOUTHREFCLK10   (gtsouthrefclk10_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_GTSOUTHREFCLK11   (gtsouthrefclk11_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_PMARSVD0          (pmarsvd0_int          [f_ub_cm( 8,(4*cm)+3) : f_lb_cm( 8,4*cm)]),
            .GTHE3_COMMON_PMARSVD1          (pmarsvd1_int          [f_ub_cm( 8,(4*cm)+3) : f_lb_cm( 8,4*cm)]),
            .GTHE3_COMMON_QPLL0CLKRSVD0     (qpll0clkrsvd0_int     [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL0CLKRSVD1     (qpll0clkrsvd1_int     [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL0LOCKDETCLK   (qpll0lockdetclk_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL0LOCKEN       (qpll0locken_int       [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL0PD           (qpll0pd_int           [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL0REFCLKSEL    (qpll0refclksel_int    [f_ub_cm( 3,(4*cm)+3) : f_lb_cm( 3,4*cm)]),
            .GTHE3_COMMON_QPLL0RESET        (qpll0reset_int        [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1CLKRSVD0     (qpll1clkrsvd0_int     [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1CLKRSVD1     (qpll1clkrsvd1_int     [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1LOCKDETCLK   (qpll1lockdetclk_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1LOCKEN       (qpll1locken_int       [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1PD           (qpll1pd_int           [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1REFCLKSEL    (qpll1refclksel_int    [f_ub_cm( 3,(4*cm)+3) : f_lb_cm( 3,4*cm)]),
            .GTHE3_COMMON_QPLL1RESET        (qpll1reset_int        [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLLRSVD1         (qpllrsvd1_int         [f_ub_cm( 8,(4*cm)+3) : f_lb_cm( 8,4*cm)]),
            .GTHE3_COMMON_QPLLRSVD2         (qpllrsvd2_int         [f_ub_cm( 5,(4*cm)+3) : f_lb_cm( 5,4*cm)]),
            .GTHE3_COMMON_QPLLRSVD3         (qpllrsvd3_int         [f_ub_cm( 5,(4*cm)+3) : f_lb_cm( 5,4*cm)]),
            .GTHE3_COMMON_QPLLRSVD4         (qpllrsvd4_int         [f_ub_cm( 8,(4*cm)+3) : f_lb_cm( 8,4*cm)]),
            .GTHE3_COMMON_RCALENB           (rcalenb_int           [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_DRPDO             (drpdo_common_int      [f_ub_cm(16,(4*cm)+3) : f_lb_cm(16,4*cm)]),
            .GTHE3_COMMON_DRPRDY            (drprdy_common_int     [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_PMARSVDOUT0       (pmarsvdout0_int       [f_ub_cm( 8,(4*cm)+3) : f_lb_cm( 8,4*cm)]),
            .GTHE3_COMMON_PMARSVDOUT1       (pmarsvdout1_int       [f_ub_cm( 8,(4*cm)+3) : f_lb_cm( 8,4*cm)]),
            .GTHE3_COMMON_QPLL0FBCLKLOST    (qpll0fbclklost_int    [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL0LOCK         (qpll0lock_int         [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL0OUTCLK       (qpll0outclk_int       [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL0OUTREFCLK    (qpll0outrefclk_int    [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL0REFCLKLOST   (qpll0refclklost_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1FBCLKLOST    (qpll1fbclklost_int    [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1LOCK         (qpll1lock_int         [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1OUTCLK       (qpll1outclk_int       [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1OUTREFCLK    (qpll1outrefclk_int    [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLL1REFCLKLOST   (qpll1refclklost_int   [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_QPLLDMONITOR0     (qplldmonitor0_int     [f_ub_cm( 8,(4*cm)+3) : f_lb_cm( 8,4*cm)]),
            .GTHE3_COMMON_QPLLDMONITOR1     (qplldmonitor1_int     [f_ub_cm( 8,(4*cm)+3) : f_lb_cm( 8,4*cm)]),
            .GTHE3_COMMON_REFCLKOUTMONITOR0 (refclkoutmonitor0_int [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_REFCLKOUTMONITOR1 (refclkoutmonitor1_int [f_ub_cm( 1,(4*cm)+3) : f_lb_cm( 1,4*cm)]),
            .GTHE3_COMMON_RXRECCLK0_SEL     (rxrecclk0_sel_int     [f_ub_cm( 2,(4*cm)+3) : f_lb_cm( 2,4*cm)]),
            .GTHE3_COMMON_RXRECCLK1_SEL     (rxrecclk1_sel_int     [f_ub_cm( 2,(4*cm)+3) : f_lb_cm( 2,4*cm)])
          );

        end
      end

    end
    else begin : gen_no_common

      assign drpdo_common_int      = {`ice_gth_t_gtwizard_gthe3_SF_CM{16'b0}};
      assign drprdy_common_int     = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign pmarsvdout0_int       = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 8'b0}};
      assign pmarsvdout1_int       = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 8'b0}};
      assign qpll0fbclklost_int    = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign qpll0lock_int         = gtwiz_reset_qpll0lock_in;
      assign qpll0outclk_int       = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign qpll0outrefclk_int    = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign qpll0refclklost_int   = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign qpll1fbclklost_int    = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign qpll1lock_int         = gtwiz_reset_qpll1lock_in;
      assign qpll1outclk_int       = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign qpll1outrefclk_int    = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign qpll1refclklost_int   = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign qplldmonitor0_int     = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 8'b0}};
      assign qplldmonitor1_int     = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 8'b0}};
      assign refclkoutmonitor0_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign refclkoutmonitor1_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 1'b0}};
      assign rxrecclk0_sel_int     = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 2'b0}};
      assign rxrecclk1_sel_int     = {`ice_gth_t_gtwizard_gthe3_SF_CM{ 2'b0}};

    end


    // ----------------------------------------------------------------------------------------------------------------
    // Transceiver channel block declarations, assignments, and primitive wrapper conditional generation
    // ----------------------------------------------------------------------------------------------------------------

    // Declare the signal vectors used to map to transceiver channel-related ports
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cfgreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] clkrsvd0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] clkrsvd1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cplllockdetclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cplllocken_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllpd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllpd_ch_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] cpllrefclksel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllreset_ch_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] dmonfiforeset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] dmonitorclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  9)-1:0] drpaddr_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  9)-1:0] drpaddr_ch_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drpclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] drpdi_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] drpdi_ch_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drpen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drpen_ch_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drpwe_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drpwe_ch_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphicaldone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphicalstart_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphidrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphidwren_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphixrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] evoddphixwren_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] eyescanmode_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] eyescanreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] eyescantrigger_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtgrefclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gthrxn_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gthrxp_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtnorthrefclk0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtnorthrefclk1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtrefclk0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtrefclk1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtresetsel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] gtrsvd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtrxreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtsouthrefclk0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtsouthrefclk1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gttxreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] loopback_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] lpbkrxtxseren_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] lpbktxrxseren_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieeqrxeqadaptdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcierstidle_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pciersttxsyncstart_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieuserratedone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] pcsrsvdin_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] pcsrsvdin2_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] pmarsvdin_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] qpll0clk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] qpll0refclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] qpll1clk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] qpll1refclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] resetovrd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rstclkentx_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rx8b10ben_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxbufreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrfreqreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrhold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrresetrsv_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchbonden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] rxchbondi_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxchbondlevel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchbondmaster_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchbondslave_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcommadeten_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxdfeagcctrl_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfeagchold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfeagcovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfelfhold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfelfovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfelpmreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap10hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap10ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap11hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap11ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap12hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap12ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap13hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap13ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap14hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap14ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap15hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap15ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap2hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap2ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap3hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap3ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap4hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap4ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap5hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap5ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap6hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap6ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap7hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap7ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap8hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap8ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap9hold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfetap9ovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfeuthold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfeutovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfevphold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfevpovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfevsen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdfexyden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlybypass_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlyen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlyovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlysreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxelecidlemode_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxgearboxslip_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlatclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmgchold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmgcovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmhfhold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmhfovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmlfhold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmlfklovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmoshold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxlpmosovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxmcommaalignen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxmonitorsel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoobreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoscalreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoshold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  4)-1:0] rxosintcfg_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosinten_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosinthold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintstrobe_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosinttestovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxoutclksel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpcommaalignen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpcsreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxpd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphalign_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphalignen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphdlypd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphdlyreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxpllclksel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpmareset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpolarity_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprbscntreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  4)-1:0] rxprbssel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprogdivreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxqpien_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxrate_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxratemode_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslide_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslipoutclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslippma_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncallin_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncin_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncmode_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxsysclksel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxuserrdy_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxusrclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxusrclk2_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] sigvalidclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 20)-1:0] tstin_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] tx8b10bbypass_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] tx8b10ben_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txbufdiffctrl_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txcominit_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txcomsas_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txcomwake_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] txctrl0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] txctrl1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] txctrl2_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*128)-1:0] txdata_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] txdataextendrsvd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdeemph_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdetectrx_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  4)-1:0] txdiffctrl_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdiffpd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlybypass_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlyen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlyhold_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlyovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlysreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlyupdown_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txelecidle_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  6)-1:0] txheader_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txinhibit_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txlatclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  7)-1:0] txmaincursor_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txmargin_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txoutclksel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txoutclksel_ch_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpcsreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] txpd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpdelecidlemode_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphalign_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphalignen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphdlypd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphdlyreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphdlytstclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphinit_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpippmen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpippmovrden_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpippmpd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpippmsel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] txpippmstepsize_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpisopd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] txpllclksel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpmareset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpolarity_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] txpostcursor_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpostcursorinv_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txprbsforceerr_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  4)-1:0] txprbssel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] txprecursor_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txprecursorinv_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txprogdivreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txprogdivreset_ch_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpibiasen_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpistrongpdown_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpiweakpup_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txrate_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txratemode_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  7)-1:0] txsequence_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txswing_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncallin_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncin_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncmode_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] txsysclksel_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txuserrdy_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txusrclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txusrclk2_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] bufgtce_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] bufgtcemask_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  9)-1:0] bufgtdiv_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] bufgtreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] bufgtrstmask_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllfbclklost_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cplllock_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cplllock_ch_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpllrefclklost_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 17)-1:0] dmonitorout_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] drpdo_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] drprdy_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] eyescandataerror_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gthtxn_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gthtxp_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtpowergood_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] gtrefclkmonitor_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcierategen3_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcierateidle_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] pcierateqpllpd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] pcierateqpllreset_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pciesynctxsyncdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieusergen3rdy_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieuserphystatusrst_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] pcieuserratestart_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 12)-1:0] pcsrsvdout_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] phystatus_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] pinrsrvdas_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] resetexception_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxbufstatus_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxbyteisaligned_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxbyterealign_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrlock_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcdrphdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchanbondseq_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchanisaligned_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxchanrealign_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  5)-1:0] rxchbondo_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxclkcorcnt_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcominitdet_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcommadet_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcomsasdet_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxcomwakedet_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] rxctrl0_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] rxctrl1_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] rxctrl2_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] rxctrl3_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*128)-1:0] rxdata_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  8)-1:0] rxdataextendrsvd_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxdatavalid_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxdlysresetdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxelecidle_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  6)-1:0] rxheader_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxheadervalid_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  7)-1:0] rxmonitorout_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintstarted_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintstrobedone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxosintstrobestarted_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoutclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoutclkfabric_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxoutclkpcs_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphaligndone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxphalignerr_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxpmaresetdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprbserr_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprbslocked_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxprgdivresetdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxqpisenn_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxqpisenp_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxratedone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxrecclkout_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxresetdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsliderdy_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslipdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslipoutclkrdy_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxslippmardy_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] rxstartofseq_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] rxstatus_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxsyncout_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] rxvalid_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  2)-1:0] txbufstatus_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txcomfinish_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txdlysresetdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txoutclk_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txoutclkfabric_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txoutclkpcs_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphaligndone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txphinitdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txpmaresetdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txprgdivresetdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpisenn_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txqpisenp_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txratedone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txresetdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncdone_int;
    wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] txsyncout_int;

    // Assign transceiver channel-related inputs to the corresponding internal signal vectors
    // Note: commented assignments are replaced by assignments associated with helper blocks, later in this module
    assign cfgreset_int             = cfgreset_in;
    assign clkrsvd0_int             = clkrsvd0_in;
    assign clkrsvd1_int             = clkrsvd1_in;
    assign cplllockdetclk_int       = cplllockdetclk_in;
    assign cplllocken_int           = cplllocken_in;
  //assign cpllpd_int               = cpllpd_in;
    assign cpllrefclksel_int        = cpllrefclksel_in;
  //assign cpllreset_int            = cpllreset_in;
    assign dmonfiforeset_int        = dmonfiforeset_in;
    assign dmonitorclk_int          = dmonitorclk_in;
    assign drpaddr_int              = drpaddr_in;
    assign drpclk_int               = drpclk_in;
    assign drpdi_int                = drpdi_in;
    assign drpen_int                = drpen_in;
    assign drpwe_int                = drpwe_in;
    assign evoddphicaldone_int      = evoddphicaldone_in;
    assign evoddphicalstart_int     = evoddphicalstart_in;
    assign evoddphidrden_int        = evoddphidrden_in;
    assign evoddphidwren_int        = evoddphidwren_in;
    assign evoddphixrden_int        = evoddphixrden_in;
    assign evoddphixwren_int        = evoddphixwren_in;
    assign eyescanmode_int          = eyescanmode_in;
    assign eyescanreset_int         = eyescanreset_in;
    assign eyescantrigger_int       = eyescantrigger_in;
    assign gtgrefclk_int            = gtgrefclk_in;
    assign gthrxn_int               = gthrxn_in;
    assign gthrxp_int               = gthrxp_in;
    assign gtnorthrefclk0_int       = gtnorthrefclk0_in;
    assign gtnorthrefclk1_int       = gtnorthrefclk1_in;
    assign gtrefclk0_int            = gtrefclk0_in;
    assign gtrefclk1_int            = gtrefclk1_in;
    assign gtresetsel_int           = gtresetsel_in;
    assign gtrsvd_int               = gtrsvd_in;
  //assign gtrxreset_int            = gtrxreset_in;
    assign gtsouthrefclk0_int       = gtsouthrefclk0_in;
    assign gtsouthrefclk1_int       = gtsouthrefclk1_in;
  //assign gttxreset_int            = gttxreset_in;
    assign loopback_int             = loopback_in;
    assign lpbkrxtxseren_int        = lpbkrxtxseren_in;
    assign lpbktxrxseren_int        = lpbktxrxseren_in;
    assign pcieeqrxeqadaptdone_int  = pcieeqrxeqadaptdone_in;
    assign pcierstidle_int          = pcierstidle_in;
    assign pciersttxsyncstart_int   = pciersttxsyncstart_in;
    assign pcieuserratedone_int     = pcieuserratedone_in;
    assign pcsrsvdin_int            = pcsrsvdin_in;
    assign pcsrsvdin2_int           = pcsrsvdin2_in;
    assign pmarsvdin_int            = pmarsvdin_in;
  //assign qpll0clk_int             = qpll0clk_in;
  //assign qpll0refclk_int          = qpll0refclk_in;
  //assign qpll1clk_int             = qpll1clk_in;
  //assign qpll1refclk_int          = qpll1refclk_in;
    assign resetovrd_int            = resetovrd_in;
    assign rstclkentx_int           = rstclkentx_in;
    assign rx8b10ben_int            = rx8b10ben_in;
    assign rxbufreset_int           = rxbufreset_in;
    assign rxcdrfreqreset_int       = rxcdrfreqreset_in;
    assign rxcdrhold_int            = rxcdrhold_in;
    assign rxcdrovrden_int          = rxcdrovrden_in;
    assign rxcdrreset_int           = rxcdrreset_in;
    assign rxcdrresetrsv_int        = rxcdrresetrsv_in;
    assign rxchbonden_int           = rxchbonden_in;
    assign rxchbondi_int            = rxchbondi_in;
    assign rxchbondlevel_int        = rxchbondlevel_in;
    assign rxchbondmaster_int       = rxchbondmaster_in;
    assign rxchbondslave_int        = rxchbondslave_in;
    assign rxcommadeten_int         = rxcommadeten_in;
    assign rxdfeagcctrl_int         = rxdfeagcctrl_in;
    assign rxdfeagchold_int         = rxdfeagchold_in;
    assign rxdfeagcovrden_int       = rxdfeagcovrden_in;
    assign rxdfelfhold_int          = rxdfelfhold_in;
    assign rxdfelfovrden_int        = rxdfelfovrden_in;
    assign rxdfelpmreset_int        = rxdfelpmreset_in;
    assign rxdfetap10hold_int       = rxdfetap10hold_in;
    assign rxdfetap10ovrden_int     = rxdfetap10ovrden_in;
    assign rxdfetap11hold_int       = rxdfetap11hold_in;
    assign rxdfetap11ovrden_int     = rxdfetap11ovrden_in;
    assign rxdfetap12hold_int       = rxdfetap12hold_in;
    assign rxdfetap12ovrden_int     = rxdfetap12ovrden_in;
    assign rxdfetap13hold_int       = rxdfetap13hold_in;
    assign rxdfetap13ovrden_int     = rxdfetap13ovrden_in;
    assign rxdfetap14hold_int       = rxdfetap14hold_in;
    assign rxdfetap14ovrden_int     = rxdfetap14ovrden_in;
    assign rxdfetap15hold_int       = rxdfetap15hold_in;
    assign rxdfetap15ovrden_int     = rxdfetap15ovrden_in;
    assign rxdfetap2hold_int        = rxdfetap2hold_in;
    assign rxdfetap2ovrden_int      = rxdfetap2ovrden_in;
    assign rxdfetap3hold_int        = rxdfetap3hold_in;
    assign rxdfetap3ovrden_int      = rxdfetap3ovrden_in;
    assign rxdfetap4hold_int        = rxdfetap4hold_in;
    assign rxdfetap4ovrden_int      = rxdfetap4ovrden_in;
    assign rxdfetap5hold_int        = rxdfetap5hold_in;
    assign rxdfetap5ovrden_int      = rxdfetap5ovrden_in;
    assign rxdfetap6hold_int        = rxdfetap6hold_in;
    assign rxdfetap6ovrden_int      = rxdfetap6ovrden_in;
    assign rxdfetap7hold_int        = rxdfetap7hold_in;
    assign rxdfetap7ovrden_int      = rxdfetap7ovrden_in;
    assign rxdfetap8hold_int        = rxdfetap8hold_in;
    assign rxdfetap8ovrden_int      = rxdfetap8ovrden_in;
    assign rxdfetap9hold_int        = rxdfetap9hold_in;
    assign rxdfetap9ovrden_int      = rxdfetap9ovrden_in;
    assign rxdfeuthold_int          = rxdfeuthold_in;
    assign rxdfeutovrden_int        = rxdfeutovrden_in;
    assign rxdfevphold_int          = rxdfevphold_in;
    assign rxdfevpovrden_int        = rxdfevpovrden_in;
    assign rxdfevsen_int            = rxdfevsen_in;
    assign rxdfexyden_int           = rxdfexyden_in;
  //assign rxdlybypass_int          = rxdlybypass_in;
  //assign rxdlyen_int              = rxdlyen_in;
  //assign rxdlyovrden_int          = rxdlyovrden_in;
  //assign rxdlysreset_int          = rxdlysreset_in;
    assign rxelecidlemode_int       = rxelecidlemode_in;
    assign rxgearboxslip_int        = rxgearboxslip_in;
    assign rxlatclk_int             = rxlatclk_in;
    assign rxlpmen_int              = rxlpmen_in;
    assign rxlpmgchold_int          = rxlpmgchold_in;
    assign rxlpmgcovrden_int        = rxlpmgcovrden_in;
    assign rxlpmhfhold_int          = rxlpmhfhold_in;
    assign rxlpmhfovrden_int        = rxlpmhfovrden_in;
    assign rxlpmlfhold_int          = rxlpmlfhold_in;
    assign rxlpmlfklovrden_int      = rxlpmlfklovrden_in;
    assign rxlpmoshold_int          = rxlpmoshold_in;
    assign rxlpmosovrden_int        = rxlpmosovrden_in;
    assign rxmcommaalignen_int      = rxmcommaalignen_in;
    assign rxmonitorsel_int         = rxmonitorsel_in;
    assign rxoobreset_int           = rxoobreset_in;
    assign rxoscalreset_int         = rxoscalreset_in;
    assign rxoshold_int             = rxoshold_in;
    assign rxosintcfg_int           = rxosintcfg_in;
    assign rxosinten_int            = rxosinten_in;
    assign rxosinthold_int          = rxosinthold_in;
    assign rxosintovrden_int        = rxosintovrden_in;
    assign rxosintstrobe_int        = rxosintstrobe_in;
    assign rxosinttestovrden_int    = rxosinttestovrden_in;
    assign rxosovrden_int           = rxosovrden_in;
    assign rxoutclksel_int          = rxoutclksel_in;
    assign rxpcommaalignen_int      = rxpcommaalignen_in;
    assign rxpcsreset_int           = rxpcsreset_in;
    assign rxpd_int                 = rxpd_in;
  //assign rxphalign_int            = rxphalign_in;
  //assign rxphalignen_int          = rxphalignen_in;
  //assign rxphdlypd_int            = rxphdlypd_in;
  //assign rxphdlyreset_int         = rxphdlyreset_in;
  //assign rxphovrden_int           = rxphovrden_in;
    assign rxpllclksel_int          = rxpllclksel_in;
    assign rxpmareset_int           = rxpmareset_in;
    assign rxpolarity_int           = rxpolarity_in;
    assign rxprbscntreset_int       = rxprbscntreset_in;
    assign rxprbssel_int            = rxprbssel_in;
  //assign rxprogdivreset_int       = rxprogdivreset_in;
    assign rxqpien_int              = rxqpien_in;
    assign rxrate_int               = rxrate_in;
    assign rxratemode_int           = rxratemode_in;
    assign rxslide_int              = rxslide_in;
    assign rxslipoutclk_int         = rxslipoutclk_in;
    assign rxslippma_int            = rxslippma_in;
  //assign rxsyncallin_int          = rxsyncallin_in;
  //assign rxsyncin_int             = rxsyncin_in;
  //assign rxsyncmode_int           = rxsyncmode_in;
    assign rxsysclksel_int          = rxsysclksel_in;
  //assign rxuserrdy_int            = rxuserrdy_in;
    assign rxusrclk_int             = rxusrclk_in;  //JJB:::
    assign rxusrclk2_int            = rxusrclk2_in; //JJB:::
    assign sigvalidclk_int          = sigvalidclk_in;
    assign tstin_int                = tstin_in;
    assign tx8b10bbypass_int        = tx8b10bbypass_in;
    assign tx8b10ben_int            = tx8b10ben_in;
    assign txbufdiffctrl_int        = txbufdiffctrl_in;
    assign txcominit_int            = txcominit_in;
    assign txcomsas_int             = txcomsas_in;
    assign txcomwake_int            = txcomwake_in;
  //assign txctrl0_int              = txctrl0_in;
  //assign txctrl1_int              = txctrl1_in;
    assign txctrl2_int              = txctrl2_in;
  //assign txdata_int               = txdata_in;
    assign txdataextendrsvd_int     = txdataextendrsvd_in;
    assign txdeemph_int             = txdeemph_in;
    assign txdetectrx_int           = txdetectrx_in;
    assign txdiffctrl_int           = txdiffctrl_in;
    assign txdiffpd_int             = txdiffpd_in;
  //assign txdlybypass_int          = txdlybypass_in;
  //assign txdlyen_int              = txdlyen_in;
  //assign txdlyhold_int            = txdlyhold_in;
  //assign txdlyovrden_int          = txdlyovrden_in;
  //assign txdlysreset_int          = txdlysreset_in;
  //assign txdlyupdown_int          = txdlyupdown_in;
    assign txelecidle_int           = txelecidle_in;
    assign txheader_int             = txheader_in;
    assign txinhibit_int            = txinhibit_in;
    assign txlatclk_int             = txlatclk_in;
    assign txmaincursor_int         = txmaincursor_in;
    assign txmargin_int             = txmargin_in;
    assign txoutclksel_int          = txoutclksel_in;
    assign txpcsreset_int           = txpcsreset_in;
    assign txpd_int                 = txpd_in;
    assign txpdelecidlemode_int     = txpdelecidlemode_in;
  //assign txphalign_int            = txphalign_in;
  //assign txphalignen_int          = txphalignen_in;
  //assign txphdlypd_int            = txphdlypd_in;
  //assign txphdlyreset_int         = txphdlyreset_in;
  //assign txphdlytstclk_int        = txphdlytstclk_in;
  //assign txphinit_int             = txphinit_in;
  //assign txphovrden_int           = txphovrden_in;
    assign txpippmen_int            = txpippmen_in;
    assign txpippmovrden_int        = txpippmovrden_in;
    assign txpippmpd_int            = txpippmpd_in;
    assign txpippmsel_int           = txpippmsel_in;
    assign txpippmstepsize_int      = txpippmstepsize_in;
    assign txpisopd_int             = txpisopd_in;
    assign txpllclksel_int          = txpllclksel_in;
    assign txpmareset_int           = txpmareset_in;
    assign txpolarity_int           = txpolarity_in;
    assign txpostcursor_int         = txpostcursor_in;
    assign txpostcursorinv_int      = txpostcursorinv_in;
    assign txprbsforceerr_int       = txprbsforceerr_in;
    assign txprbssel_int            = txprbssel_in;
    assign txprecursor_int          = txprecursor_in;
    assign txprecursorinv_int       = txprecursorinv_in;
  //assign txprogdivreset_int       = txprogdivreset_in;
    assign txqpibiasen_int          = txqpibiasen_in;
    assign txqpistrongpdown_int     = txqpistrongpdown_in;
    assign txqpiweakpup_int         = txqpiweakpup_in;
    assign txrate_int               = txrate_in;
    assign txratemode_int           = txratemode_in;
    assign txsequence_int           = txsequence_in;
    assign txswing_int              = txswing_in;
  //assign txsyncallin_int          = txsyncallin_in;
  //assign txsyncin_int             = txsyncin_in;
  //assign txsyncmode_int           = txsyncmode_in;
    assign txsysclksel_int          = txsysclksel_in;
  //assign txuserrdy_int            = txuserrdy_in;
  //assign txusrclk_int             = txusrclk_in;
  //assign txusrclk2_int            = txusrclk2_in;
    assign bufgtce_out              = bufgtce_int;
    assign bufgtcemask_out          = bufgtcemask_int;
    assign bufgtdiv_out             = bufgtdiv_int;
    assign bufgtreset_out           = bufgtreset_int;
    assign bufgtrstmask_out         = bufgtrstmask_int;
    assign cpllfbclklost_out        = cpllfbclklost_int;
    assign cplllock_out             = cplllock_int;
    assign cpllrefclklost_out       = cpllrefclklost_int;
    assign dmonitorout_out          = dmonitorout_int;
  //assign drpdo_out                = drpdo_int;
  //assign drprdy_out               = drprdy_int;
    assign eyescandataerror_out     = eyescandataerror_int;
    assign gthtxn_out               = gthtxn_int;
    assign gthtxp_out               = gthtxp_int;
    assign gtpowergood_out          = gtpowergood_int;
    assign gtrefclkmonitor_out      = gtrefclkmonitor_int;
    assign pcierategen3_out         = pcierategen3_int;
    assign pcierateidle_out         = pcierateidle_int;
    assign pcierateqpllpd_out       = pcierateqpllpd_int;
    assign pcierateqpllreset_out    = pcierateqpllreset_int;
    assign pciesynctxsyncdone_out   = pciesynctxsyncdone_int;
    assign pcieusergen3rdy_out      = pcieusergen3rdy_int;
    assign pcieuserphystatusrst_out = pcieuserphystatusrst_int;
    assign pcieuserratestart_out    = pcieuserratestart_int;
    assign pcsrsvdout_out           = pcsrsvdout_int;
    assign phystatus_out            = phystatus_int;
    assign pinrsrvdas_out           = pinrsrvdas_int;
    assign resetexception_out       = resetexception_int;
    assign rxbufstatus_out          = rxbufstatus_int;
    assign rxbyteisaligned_out      = rxbyteisaligned_int;
    assign rxbyterealign_out        = rxbyterealign_int;
    assign rxcdrlock_out            = rxcdrlock_int;
    assign rxcdrphdone_out          = rxcdrphdone_int;
    assign rxchanbondseq_out        = rxchanbondseq_int;
    assign rxchanisaligned_out      = rxchanisaligned_int;
    assign rxchanrealign_out        = rxchanrealign_int;
    assign rxchbondo_out            = rxchbondo_int;
    assign rxclkcorcnt_out          = rxclkcorcnt_int;
    assign rxcominitdet_out         = rxcominitdet_int;
    assign rxcommadet_out           = rxcommadet_int;
    assign rxcomsasdet_out          = rxcomsasdet_int;
    assign rxcomwakedet_out         = rxcomwakedet_int;
    assign rxctrl0_out              = rxctrl0_int;
    assign rxctrl1_out              = rxctrl1_int;
    assign rxctrl2_out              = rxctrl2_int;
    assign rxctrl3_out              = rxctrl3_int;
    assign rxdata_out               = rxdata_int;
    assign rxdataextendrsvd_out     = rxdataextendrsvd_int;
    assign rxdatavalid_out          = rxdatavalid_int;
    assign rxdlysresetdone_out      = rxdlysresetdone_int;
    assign rxelecidle_out           = rxelecidle_int;
    assign rxheader_out             = rxheader_int;
    assign rxheadervalid_out        = rxheadervalid_int;
    assign rxmonitorout_out         = rxmonitorout_int;
    assign rxosintdone_out          = rxosintdone_int;
    assign rxosintstarted_out       = rxosintstarted_int;
    assign rxosintstrobedone_out    = rxosintstrobedone_int;
    assign rxosintstrobestarted_out = rxosintstrobestarted_int;
    assign rxoutclk_out             = rxoutclk_int;
    assign rxoutclkfabric_out       = rxoutclkfabric_int;
    assign rxoutclkpcs_out          = rxoutclkpcs_int;
    assign rxphaligndone_out        = rxphaligndone_int;
    assign rxphalignerr_out         = rxphalignerr_int;
    assign rxpmaresetdone_out       = rxpmaresetdone_int;
    assign rxprbserr_out            = rxprbserr_int;
    assign rxprbslocked_out         = rxprbslocked_int;
    assign rxprgdivresetdone_out    = rxprgdivresetdone_int;
    assign rxqpisenn_out            = rxqpisenn_int;
    assign rxqpisenp_out            = rxqpisenp_int;
    assign rxratedone_out           = rxratedone_int;
    assign rxrecclkout_out          = rxrecclkout_int;
    assign rxresetdone_out          = rxresetdone_int;
    assign rxsliderdy_out           = rxsliderdy_int;
    assign rxslipdone_out           = rxslipdone_int;
    assign rxslipoutclkrdy_out      = rxslipoutclkrdy_int;
    assign rxslippmardy_out         = rxslippmardy_int;
    assign rxstartofseq_out         = rxstartofseq_int;
    assign rxstatus_out             = rxstatus_int;
    assign rxsyncdone_out           = rxsyncdone_int;
    assign rxsyncout_out            = rxsyncout_int;
    assign rxvalid_out              = rxvalid_int;
    assign txbufstatus_out          = txbufstatus_int;
    assign txcomfinish_out          = txcomfinish_int;
    assign txdlysresetdone_out      = txdlysresetdone_int;
    assign txoutclk_out             = txoutclk_int;
    assign txoutclkfabric_out       = txoutclkfabric_int;
    assign txoutclkpcs_out          = txoutclkpcs_int;
    assign txphaligndone_out        = txphaligndone_int;
    assign txphinitdone_out         = txphinitdone_int;
    assign txpmaresetdone_out       = txpmaresetdone_int;
  //assign txprgdivresetdone_out    = txprgdivresetdone_int;
    assign txqpisenn_out            = txqpisenn_int;
    assign txqpisenp_out            = txqpisenp_int;
    assign txratedone_out           = txratedone_int;
    assign txresetdone_out          = txresetdone_int;
    assign txsyncdone_out           = txsyncdone_int;
    assign txsyncout_out            = txsyncout_int;

    // For each transceiver quad which contains one or more transceiver channel resources, generate an instance of
    // the primitive wrapper containing that number of parameterized channel primitives
    genvar ch;
    for (ch = 0; ch < `ice_gth_t_gtwizard_gthe3_MAX_NUM_COMMONS; ch = ch + 1) begin : gen_channel_container
      if (f_num_ch_in_quad(4*ch) > 0) begin : gen_enabled_channel

        ice_gth_t_gthe3_channel_wrapper #(
          .NUM_CHANNELS (f_num_ch_in_quad(4*ch))
        ) gthe3_channel_wrapper_inst (
          .GTHE3_CHANNEL_CFGRESET             (cfgreset_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_CLKRSVD0             (clkrsvd0_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_CLKRSVD1             (clkrsvd1_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_CPLLLOCKDETCLK       (cplllockdetclk_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_CPLLLOCKEN           (cplllocken_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_CPLLPD               (cpllpd_ch_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_CPLLREFCLKSEL        (cpllrefclksel_int        [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_CPLLRESET            (cpllreset_ch_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_DMONFIFORESET        (dmonfiforeset_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_DMONITORCLK          (dmonitorclk_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_DRPADDR              (drpaddr_ch_int           [f_ub_ch(  9,(4*ch)+3) : f_lb_ch(  9,4*ch)]),
          .GTHE3_CHANNEL_DRPCLK               (drpclk_int               [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_DRPDI                (drpdi_ch_int             [f_ub_ch( 16,(4*ch)+3) : f_lb_ch( 16,4*ch)]),
          .GTHE3_CHANNEL_DRPEN                (drpen_ch_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_DRPWE                (drpwe_ch_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EVODDPHICALDONE      (evoddphicaldone_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EVODDPHICALSTART     (evoddphicalstart_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EVODDPHIDRDEN        (evoddphidrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EVODDPHIDWREN        (evoddphidwren_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EVODDPHIXRDEN        (evoddphixrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EVODDPHIXWREN        (evoddphixwren_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EYESCANMODE          (eyescanmode_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EYESCANRESET         (eyescanreset_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EYESCANTRIGGER       (eyescantrigger_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTGREFCLK            (gtgrefclk_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTHRXN               (gthrxn_int               [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTHRXP               (gthrxp_int               [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTNORTHREFCLK0       (gtnorthrefclk0_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTNORTHREFCLK1       (gtnorthrefclk1_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTREFCLK0            (gtrefclk0_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTREFCLK1            (gtrefclk1_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTRESETSEL           (gtresetsel_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTRSVD               (gtrsvd_int               [f_ub_ch( 16,(4*ch)+3) : f_lb_ch( 16,4*ch)]),
          .GTHE3_CHANNEL_GTRXRESET            (gtrxreset_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTSOUTHREFCLK0       (gtsouthrefclk0_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTSOUTHREFCLK1       (gtsouthrefclk1_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTTXRESET            (gttxreset_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_LOOPBACK             (loopback_int             [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_LPBKRXTXSEREN        (lpbkrxtxseren_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_LPBKTXRXSEREN        (lpbktxrxseren_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE  (pcieeqrxeqadaptdone_int  [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIERSTIDLE          (pcierstidle_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIERSTTXSYNCSTART   (pciersttxsyncstart_int   [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIEUSERRATEDONE     (pcieuserratedone_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCSRSVDIN            (pcsrsvdin_int            [f_ub_ch( 16,(4*ch)+3) : f_lb_ch( 16,4*ch)]),
          .GTHE3_CHANNEL_PCSRSVDIN2           (pcsrsvdin2_int           [f_ub_ch(  5,(4*ch)+3) : f_lb_ch(  5,4*ch)]),
          .GTHE3_CHANNEL_PMARSVDIN            (pmarsvdin_int            [f_ub_ch(  5,(4*ch)+3) : f_lb_ch(  5,4*ch)]),
          .GTHE3_CHANNEL_QPLL0CLK             (qpll0clk_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_QPLL0REFCLK          (qpll0refclk_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_QPLL1CLK             (qpll1clk_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_QPLL1REFCLK          (qpll1refclk_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RESETOVRD            (resetovrd_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RSTCLKENTX           (rstclkentx_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RX8B10BEN            (rx8b10ben_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXBUFRESET           (rxbufreset_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCDRFREQRESET       (rxcdrfreqreset_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCDRHOLD            (rxcdrhold_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCDROVRDEN          (rxcdrovrden_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCDRRESET           (rxcdrreset_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCDRRESETRSV        (rxcdrresetrsv_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCHBONDEN           (rxchbonden_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCHBONDI            (rxchbondi_int            [f_ub_ch(  5,(4*ch)+3) : f_lb_ch(  5,4*ch)]),
          .GTHE3_CHANNEL_RXCHBONDLEVEL        (rxchbondlevel_int        [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_RXCHBONDMASTER       (rxchbondmaster_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCHBONDSLAVE        (rxchbondslave_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCOMMADETEN         (rxcommadeten_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFEAGCCTRL         (rxdfeagcctrl_int         [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXDFEAGCHOLD         (rxdfeagchold_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFEAGCOVRDEN       (rxdfeagcovrden_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFELFHOLD          (rxdfelfhold_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFELFOVRDEN        (rxdfelfovrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFELPMRESET        (rxdfelpmreset_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP10HOLD       (rxdfetap10hold_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP10OVRDEN     (rxdfetap10ovrden_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP11HOLD       (rxdfetap11hold_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP11OVRDEN     (rxdfetap11ovrden_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP12HOLD       (rxdfetap12hold_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP12OVRDEN     (rxdfetap12ovrden_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP13HOLD       (rxdfetap13hold_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP13OVRDEN     (rxdfetap13ovrden_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP14HOLD       (rxdfetap14hold_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP14OVRDEN     (rxdfetap14ovrden_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP15HOLD       (rxdfetap15hold_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP15OVRDEN     (rxdfetap15ovrden_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP2HOLD        (rxdfetap2hold_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP2OVRDEN      (rxdfetap2ovrden_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP3HOLD        (rxdfetap3hold_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP3OVRDEN      (rxdfetap3ovrden_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP4HOLD        (rxdfetap4hold_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP4OVRDEN      (rxdfetap4ovrden_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP5HOLD        (rxdfetap5hold_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP5OVRDEN      (rxdfetap5ovrden_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP6HOLD        (rxdfetap6hold_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP6OVRDEN      (rxdfetap6ovrden_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP7HOLD        (rxdfetap7hold_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP7OVRDEN      (rxdfetap7ovrden_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP8HOLD        (rxdfetap8hold_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP8OVRDEN      (rxdfetap8ovrden_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP9HOLD        (rxdfetap9hold_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFETAP9OVRDEN      (rxdfetap9ovrden_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFEUTHOLD          (rxdfeuthold_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFEUTOVRDEN        (rxdfeutovrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFEVPHOLD          (rxdfevphold_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFEVPOVRDEN        (rxdfevpovrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFEVSEN            (rxdfevsen_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDFEXYDEN           (rxdfexyden_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDLYBYPASS          (rxdlybypass_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDLYEN              (rxdlyen_int              [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDLYOVRDEN          (rxdlyovrden_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXDLYSRESET          (rxdlysreset_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXELECIDLEMODE       (rxelecidlemode_int       [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXGEARBOXSLIP        (rxgearboxslip_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLATCLK             (rxlatclk_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLPMEN              (rxlpmen_int              [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLPMGCHOLD          (rxlpmgchold_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLPMGCOVRDEN        (rxlpmgcovrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLPMHFHOLD          (rxlpmhfhold_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLPMHFOVRDEN        (rxlpmhfovrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLPMLFHOLD          (rxlpmlfhold_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLPMLFKLOVRDEN      (rxlpmlfklovrden_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLPMOSHOLD          (rxlpmoshold_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXLPMOSOVRDEN        (rxlpmosovrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXMCOMMAALIGNEN      (rxmcommaalignen_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXMONITORSEL         (rxmonitorsel_int         [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXOOBRESET           (rxoobreset_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSCALRESET         (rxoscalreset_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSHOLD             (rxoshold_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTCFG           (rxosintcfg_int           [f_ub_ch(  4,(4*ch)+3) : f_lb_ch(  4,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTEN            (rxosinten_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTHOLD          (rxosinthold_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTOVRDEN        (rxosintovrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTSTROBE        (rxosintstrobe_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTTESTOVRDEN    (rxosinttestovrden_int    [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSOVRDEN           (rxosovrden_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOUTCLKSEL          (rxoutclksel_int          [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_RXPCOMMAALIGNEN      (rxpcommaalignen_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPCSRESET           (rxpcsreset_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPD                 (rxpd_int                 [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXPHALIGN            (rxphalign_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPHALIGNEN          (rxphalignen_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPHDLYPD            (rxphdlypd_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPHDLYRESET         (rxphdlyreset_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPHOVRDEN           (rxphovrden_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPLLCLKSEL          (rxpllclksel_int          [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXPMARESET           (rxpmareset_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPOLARITY           (rxpolarity_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPRBSCNTRESET       (rxprbscntreset_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPRBSSEL            (rxprbssel_int            [f_ub_ch(  4,(4*ch)+3) : f_lb_ch(  4,4*ch)]),
          .GTHE3_CHANNEL_RXPROGDIVRESET       (rxprogdivreset_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXQPIEN              (rxqpien_int              [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXRATE               (rxrate_int               [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_RXRATEMODE           (rxratemode_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSLIDE              (rxslide_int              [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSLIPOUTCLK         (rxslipoutclk_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSLIPPMA            (rxslippma_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSYNCALLIN          (rxsyncallin_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSYNCIN             (rxsyncin_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSYNCMODE           (rxsyncmode_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSYSCLKSEL          (rxsysclksel_int          [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXUSERRDY            (rxuserrdy_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXUSRCLK             (rxusrclk_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXUSRCLK2            (rxusrclk2_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_SIGVALIDCLK          (sigvalidclk_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TSTIN                (tstin_int                [f_ub_ch( 20,(4*ch)+3) : f_lb_ch( 20,4*ch)]),
          .GTHE3_CHANNEL_TX8B10BBYPASS        (tx8b10bbypass_int        [f_ub_ch(  8,(4*ch)+3) : f_lb_ch(  8,4*ch)]),
          .GTHE3_CHANNEL_TX8B10BEN            (tx8b10ben_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXBUFDIFFCTRL        (txbufdiffctrl_int        [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_TXCOMINIT            (txcominit_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXCOMSAS             (txcomsas_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXCOMWAKE            (txcomwake_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXCTRL0              (txctrl0_int              [f_ub_ch( 16,(4*ch)+3) : f_lb_ch( 16,4*ch)]),
          .GTHE3_CHANNEL_TXCTRL1              (txctrl1_int              [f_ub_ch( 16,(4*ch)+3) : f_lb_ch( 16,4*ch)]),
          .GTHE3_CHANNEL_TXCTRL2              (txctrl2_int              [f_ub_ch(  8,(4*ch)+3) : f_lb_ch(  8,4*ch)]),
          .GTHE3_CHANNEL_TXDATA               (txdata_int               [f_ub_ch(128,(4*ch)+3) : f_lb_ch(128,4*ch)]),
          .GTHE3_CHANNEL_TXDATAEXTENDRSVD     (txdataextendrsvd_int     [f_ub_ch(  8,(4*ch)+3) : f_lb_ch(  8,4*ch)]),
          .GTHE3_CHANNEL_TXDEEMPH             (txdeemph_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXDETECTRX           (txdetectrx_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXDIFFCTRL           (txdiffctrl_int           [f_ub_ch(  4,(4*ch)+3) : f_lb_ch(  4,4*ch)]),
          .GTHE3_CHANNEL_TXDIFFPD             (txdiffpd_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXDLYBYPASS          (txdlybypass_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXDLYEN              (txdlyen_int              [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXDLYHOLD            (txdlyhold_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXDLYOVRDEN          (txdlyovrden_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXDLYSRESET          (txdlysreset_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXDLYUPDOWN          (txdlyupdown_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXELECIDLE           (txelecidle_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXHEADER             (txheader_int             [f_ub_ch(  6,(4*ch)+3) : f_lb_ch(  6,4*ch)]),
          .GTHE3_CHANNEL_TXINHIBIT            (txinhibit_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXLATCLK             (txlatclk_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXMAINCURSOR         (txmaincursor_int         [f_ub_ch(  7,(4*ch)+3) : f_lb_ch(  7,4*ch)]),
          .GTHE3_CHANNEL_TXMARGIN             (txmargin_int             [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_TXOUTCLKSEL          (txoutclksel_ch_int       [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_TXPCSRESET           (txpcsreset_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPD                 (txpd_int                 [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_TXPDELECIDLEMODE     (txpdelecidlemode_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPHALIGN            (txphalign_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPHALIGNEN          (txphalignen_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPHDLYPD            (txphdlypd_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPHDLYRESET         (txphdlyreset_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPHDLYTSTCLK        (txphdlytstclk_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPHINIT             (txphinit_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPHOVRDEN           (txphovrden_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPIPPMEN            (txpippmen_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPIPPMOVRDEN        (txpippmovrden_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPIPPMPD            (txpippmpd_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPIPPMSEL           (txpippmsel_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPIPPMSTEPSIZE      (txpippmstepsize_int      [f_ub_ch(  5,(4*ch)+3) : f_lb_ch(  5,4*ch)]),
          .GTHE3_CHANNEL_TXPISOPD             (txpisopd_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPLLCLKSEL          (txpllclksel_int          [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_TXPMARESET           (txpmareset_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPOLARITY           (txpolarity_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPOSTCURSOR         (txpostcursor_int         [f_ub_ch(  5,(4*ch)+3) : f_lb_ch(  5,4*ch)]),
          .GTHE3_CHANNEL_TXPOSTCURSORINV      (txpostcursorinv_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPRBSFORCEERR       (txprbsforceerr_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPRBSSEL            (txprbssel_int            [f_ub_ch(  4,(4*ch)+3) : f_lb_ch(  4,4*ch)]),
          .GTHE3_CHANNEL_TXPRECURSOR          (txprecursor_int          [f_ub_ch(  5,(4*ch)+3) : f_lb_ch(  5,4*ch)]),
          .GTHE3_CHANNEL_TXPRECURSORINV       (txprecursorinv_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPROGDIVRESET       (txprogdivreset_ch_int    [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXQPIBIASEN          (txqpibiasen_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXQPISTRONGPDOWN     (txqpistrongpdown_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXQPIWEAKPUP         (txqpiweakpup_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXRATE               (txrate_int               [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_TXRATEMODE           (txratemode_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXSEQUENCE           (txsequence_int           [f_ub_ch(  7,(4*ch)+3) : f_lb_ch(  7,4*ch)]),
          .GTHE3_CHANNEL_TXSWING              (txswing_int              [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXSYNCALLIN          (txsyncallin_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXSYNCIN             (txsyncin_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXSYNCMODE           (txsyncmode_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXSYSCLKSEL          (txsysclksel_int          [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_TXUSERRDY            (txuserrdy_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXUSRCLK             (txusrclk_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXUSRCLK2            (txusrclk2_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_BUFGTCE              (bufgtce_int              [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_BUFGTCEMASK          (bufgtcemask_int          [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_BUFGTDIV             (bufgtdiv_int             [f_ub_ch(  9,(4*ch)+3) : f_lb_ch(  9,4*ch)]),
          .GTHE3_CHANNEL_BUFGTRESET           (bufgtreset_int           [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_BUFGTRSTMASK         (bufgtrstmask_int         [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_CPLLFBCLKLOST        (cpllfbclklost_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_CPLLLOCK             (cplllock_ch_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_CPLLREFCLKLOST       (cpllrefclklost_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_DMONITOROUT          (dmonitorout_int          [f_ub_ch( 17,(4*ch)+3) : f_lb_ch( 17,4*ch)]),
          .GTHE3_CHANNEL_DRPDO                (drpdo_int                [f_ub_ch( 16,(4*ch)+3) : f_lb_ch( 16,4*ch)]),
          .GTHE3_CHANNEL_DRPRDY               (drprdy_int               [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_EYESCANDATAERROR     (eyescandataerror_int     [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTHTXN               (gthtxn_int               [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTHTXP               (gthtxp_int               [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTPOWERGOOD          (gtpowergood_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_GTREFCLKMONITOR      (gtrefclkmonitor_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIERATEGEN3         (pcierategen3_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIERATEIDLE         (pcierateidle_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIERATEQPLLPD       (pcierateqpllpd_int       [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_PCIERATEQPLLRESET    (pcierateqpllreset_int    [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_PCIESYNCTXSYNCDONE   (pciesynctxsyncdone_int   [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIEUSERGEN3RDY      (pcieusergen3rdy_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIEUSERPHYSTATUSRST (pcieuserphystatusrst_int [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCIEUSERRATESTART    (pcieuserratestart_int    [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PCSRSVDOUT           (pcsrsvdout_int           [f_ub_ch( 12,(4*ch)+3) : f_lb_ch( 12,4*ch)]),
          .GTHE3_CHANNEL_PHYSTATUS            (phystatus_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_PINRSRVDAS           (pinrsrvdas_int           [f_ub_ch(  8,(4*ch)+3) : f_lb_ch(  8,4*ch)]),
          .GTHE3_CHANNEL_RESETEXCEPTION       (resetexception_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXBUFSTATUS          (rxbufstatus_int          [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_RXBYTEISALIGNED      (rxbyteisaligned_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXBYTEREALIGN        (rxbyterealign_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCDRLOCK            (rxcdrlock_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCDRPHDONE          (rxcdrphdone_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCHANBONDSEQ        (rxchanbondseq_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCHANISALIGNED      (rxchanisaligned_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCHANREALIGN        (rxchanrealign_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCHBONDO            (rxchbondo_int            [f_ub_ch(  5,(4*ch)+3) : f_lb_ch(  5,4*ch)]),
          .GTHE3_CHANNEL_RXCLKCORCNT          (rxclkcorcnt_int          [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXCOMINITDET         (rxcominitdet_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCOMMADET           (rxcommadet_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCOMSASDET          (rxcomsasdet_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCOMWAKEDET         (rxcomwakedet_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXCTRL0              (rxctrl0_int              [f_ub_ch( 16,(4*ch)+3) : f_lb_ch( 16,4*ch)]),
          .GTHE3_CHANNEL_RXCTRL1              (rxctrl1_int              [f_ub_ch( 16,(4*ch)+3) : f_lb_ch( 16,4*ch)]),
          .GTHE3_CHANNEL_RXCTRL2              (rxctrl2_int              [f_ub_ch(  8,(4*ch)+3) : f_lb_ch(  8,4*ch)]),
          .GTHE3_CHANNEL_RXCTRL3              (rxctrl3_int              [f_ub_ch(  8,(4*ch)+3) : f_lb_ch(  8,4*ch)]),
          .GTHE3_CHANNEL_RXDATA               (rxdata_int               [f_ub_ch(128,(4*ch)+3) : f_lb_ch(128,4*ch)]),
          .GTHE3_CHANNEL_RXDATAEXTENDRSVD     (rxdataextendrsvd_int     [f_ub_ch(  8,(4*ch)+3) : f_lb_ch(  8,4*ch)]),
          .GTHE3_CHANNEL_RXDATAVALID          (rxdatavalid_int          [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXDLYSRESETDONE      (rxdlysresetdone_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXELECIDLE           (rxelecidle_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXHEADER             (rxheader_int             [f_ub_ch(  6,(4*ch)+3) : f_lb_ch(  6,4*ch)]),
          .GTHE3_CHANNEL_RXHEADERVALID        (rxheadervalid_int        [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXMONITOROUT         (rxmonitorout_int         [f_ub_ch(  7,(4*ch)+3) : f_lb_ch(  7,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTDONE          (rxosintdone_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTSTARTED       (rxosintstarted_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTSTROBEDONE    (rxosintstrobedone_int    [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOSINTSTROBESTARTED (rxosintstrobestarted_int [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOUTCLK             (rxoutclk_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOUTCLKFABRIC       (rxoutclkfabric_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXOUTCLKPCS          (rxoutclkpcs_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPHALIGNDONE        (rxphaligndone_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPHALIGNERR         (rxphalignerr_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPMARESETDONE       (rxpmaresetdone_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPRBSERR            (rxprbserr_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPRBSLOCKED         (rxprbslocked_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXPRGDIVRESETDONE    (rxprgdivresetdone_int    [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXQPISENN            (rxqpisenn_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXQPISENP            (rxqpisenp_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXRATEDONE           (rxratedone_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXRECCLKOUT          (rxrecclkout_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXRESETDONE          (rxresetdone_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSLIDERDY           (rxsliderdy_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSLIPDONE           (rxslipdone_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSLIPOUTCLKRDY      (rxslipoutclkrdy_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSLIPPMARDY         (rxslippmardy_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSTARTOFSEQ         (rxstartofseq_int         [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_RXSTATUS             (rxstatus_int             [f_ub_ch(  3,(4*ch)+3) : f_lb_ch(  3,4*ch)]),
          .GTHE3_CHANNEL_RXSYNCDONE           (rxsyncdone_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXSYNCOUT            (rxsyncout_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_RXVALID              (rxvalid_int              [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXBUFSTATUS          (txbufstatus_int          [f_ub_ch(  2,(4*ch)+3) : f_lb_ch(  2,4*ch)]),
          .GTHE3_CHANNEL_TXCOMFINISH          (txcomfinish_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXDLYSRESETDONE      (txdlysresetdone_int      [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXOUTCLK             (txoutclk_int             [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXOUTCLKFABRIC       (txoutclkfabric_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXOUTCLKPCS          (txoutclkpcs_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPHALIGNDONE        (txphaligndone_int        [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPHINITDONE         (txphinitdone_int         [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPMARESETDONE       (txpmaresetdone_int       [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXPRGDIVRESETDONE    (txprgdivresetdone_int    [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXQPISENN            (txqpisenn_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXQPISENP            (txqpisenp_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXRATEDONE           (txratedone_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXRESETDONE          (txresetdone_int          [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXSYNCDONE           (txsyncdone_int           [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)]),
          .GTHE3_CHANNEL_TXSYNCOUT            (txsyncout_int            [f_ub_ch(  1,(4*ch)+3) : f_lb_ch(  1,4*ch)])
        );

      end
    end


    // ----------------------------------------------------------------------------------------------------------------
    // Transceiver channel CPLL calibration block
    // ----------------------------------------------------------------------------------------------------------------

    // If the transceiver channel CPLL calibration block is required, instantiate it for each transceiver channel
    if ((C_INCLUDE_CPLL_CAL         == `ice_gth_t_gtwizard_gthe3_INCLUDE_CPLL_CAL__INCLUDE) ||
        (((C_INCLUDE_CPLL_CAL       == `ice_gth_t_gtwizard_gthe3_INCLUDE_CPLL_CAL__DEPENDENT) &&
         ((C_GT_REV                 == 11) ||
          (C_GT_REV                 == 12) ||
          (C_GT_REV                 == 14) ||
          (C_GT_REV                 == 16))) &&
         (((C_TX_ENABLE             == `ice_gth_t_gtwizard_gthe3_TX_ENABLE__ENABLED) &&
           (C_TX_PLL_TYPE           == `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL)) ||
          ((C_RX_ENABLE             == `ice_gth_t_gtwizard_gthe3_RX_ENABLE__ENABLED) &&
           (C_RX_PLL_TYPE           == `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL)) ||
          ((C_TXPROGDIV_FREQ_ENABLE == `ice_gth_t_gtwizard_gthe3_TXPROGDIV_FREQ_ENABLE__ENABLED) &&
           (C_TXPROGDIV_FREQ_SOURCE == `ice_gth_t_gtwizard_gthe3_TXPROGDIV_FREQ_SOURCE__CPLL))))) begin : gen_cpll_cal

      // Use local parameters and declare both debug and connectivity wires for use with the CPLL calibration block
      wire [15:0] p_cpll_cal_freq_count_window_int      = P_CPLL_CAL_FREQ_COUNT_WINDOW;
      wire [17:0] p_cpll_cal_txoutclk_period_int        = P_CPLL_CAL_TXOUTCLK_PERIOD;
      wire [15:0] p_cpll_cal_wait_deassert_cpllpd_int   = P_CPLL_CAL_WAIT_DEASSERT_CPLLPD;
      wire [17:0] p_cpll_cal_txoutclk_period_div100_int = P_CPLL_CAL_TXOUTCLK_PERIOD_DIV100;

      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] cpll_cal_fail_int;
      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] cpll_cal_done_int;
      wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] cpll_cal_debug_int;

      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] cpll_cal_reset_int;
      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] txprgdivresetdone_cpll_cal_int;
      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] cplllock_cpll_cal_int;
      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] drprdy_cpll_cal_int;
      wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] drpdo_cpll_cal_int;
      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] cpllreset_cpll_cal_int;
      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] cpllpd_cpll_cal_int;
      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] txprogdivreset_cpll_cal_int;
      wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  3)-1:0] txoutclksel_cpll_cal_int;
      wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  9)-1:0] drpaddr_cpll_cal_int;
      wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 16)-1:0] drpdi_cpll_cal_int;
      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] drpen_cpll_cal_int;
      wire [ `ice_gth_t_gtwizard_gthe3_N_CH     -1:0] drpwe_cpll_cal_int;

      // The TXOUTCLK_PERIOD_IN and CNT_TOL_IN ports are normally driven by an internally-calculated value. When INCLUDE_CPLL_CAL is 1,
      // they are driven as inputs for PLL-switching and rate change special cases, and the BUFG_GT CE input is provided by the user.
      wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 18)-1:0] cpll_cal_txoutclk_period_int;
      wire [(`ice_gth_t_gtwizard_gthe3_N_CH* 18)-1:0] cpll_cal_cnt_tol_int;
      wire [(`ice_gth_t_gtwizard_gthe3_N_CH*  1)-1:0] cpll_cal_bufg_ce_int;
      if (C_INCLUDE_CPLL_CAL == `ice_gth_t_gtwizard_gthe3_INCLUDE_CPLL_CAL__INCLUDE) begin : gen_txoutclk_pd_input
        assign cpll_cal_txoutclk_period_int = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_gthe3_cpll_cal_txoutclk_period_in}};
        assign cpll_cal_cnt_tol_int         = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_gthe3_cpll_cal_cnt_tol_in}};
        assign cpll_cal_bufg_ce_int         = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_gthe3_cpll_cal_bufg_ce_in}};
      end
      else begin : gen_txoutclk_pd_internal
        assign cpll_cal_txoutclk_period_int = {`ice_gth_t_gtwizard_gthe3_N_CH{p_cpll_cal_txoutclk_period_int}};
        assign cpll_cal_cnt_tol_int         = {`ice_gth_t_gtwizard_gthe3_N_CH{p_cpll_cal_txoutclk_period_div100_int}};
        assign cpll_cal_bufg_ce_int         = {`ice_gth_t_gtwizard_gthe3_N_CH{1'b1}};
      end

      // The CPLL calibration block for a given transceiver channel is reset when either the CPLL reset signal or the
      // CPLL power-down signal is asserted for that channel
      assign cpll_cal_reset_int = cpllreset_int | cpllpd_int;

      // Instantiate one CPLL calibration block for each transceiver channel
      genvar cal;
      for (cal = 0; cal < `ice_gth_t_gtwizard_gthe3_N_CH; cal = cal + 1) begin : gen_cpll_cal_inst
        gtwizard_ultrascale_v1_6_2_gthe3_cpll_cal gtwizard_ultrascale_v1_6_2_gthe3_cpll_cal_inst (
          .TXOUTCLK_PERIOD_IN         (cpll_cal_txoutclk_period_int[(18*cal)+17:18*cal]),
          .WAIT_DEASSERT_CPLLPD_IN    (p_cpll_cal_wait_deassert_cpllpd_int),
          .CNT_TOL_IN                 (cpll_cal_cnt_tol_int[(18*cal)+17:18*cal]),
          .FREQ_COUNT_WINDOW_IN       (p_cpll_cal_freq_count_window_int),
          .RESET_IN                   (cpll_cal_reset_int[cal]),
          .CLK_IN                     (drpclk_int[cal]),
          .USER_TXOUTCLK_BUFG_CE_IN   (cpll_cal_bufg_ce_int[cal]),
          .USER_TXOUTCLK_BUFG_CLR_IN  (gtwiz_userclk_tx_reset_in),
          .USER_TXPROGDIVRESET_IN     (txprogdivreset_int[cal]),
          .USER_TXPRGDIVRESETDONE_OUT (txprgdivresetdone_cpll_cal_int[cal]),
          .USER_TXOUTCLKSEL_IN        (txoutclksel_int[(3*cal)+2:3*cal]),
          .USER_CPLLLOCK_OUT          (cplllock_cpll_cal_int[cal]),
          .USER_CHANNEL_DRPADDR_IN    (drpaddr_int[(9*cal)+8:9*cal]),
          .USER_CHANNEL_DRPDI_IN      (drpdi_int[(16*cal)+15:16*cal]),
          .USER_CHANNEL_DRPEN_IN      (drpen_int[cal]),
          .USER_CHANNEL_DRPWE_IN      (drpwe_int[cal]),
          .USER_CHANNEL_DRPRDY_OUT    (drprdy_cpll_cal_int[cal]),
          .USER_CHANNEL_DRPDO_OUT     (drpdo_cpll_cal_int[(16*cal)+15:16*cal]),
          .CPLL_CAL_FAIL              (cpll_cal_fail_int[cal]),
          .CPLL_CAL_DONE              (cpll_cal_done_int[cal]),
          .DEBUG_OUT                  (cpll_cal_debug_int[(16*cal)+15:16*cal]),
          .GTHE3_TXOUTCLK_IN          (txoutclk_int[cal]),
          .GTHE3_CPLLLOCK_IN          (cplllock_ch_int[cal]),
          .GTHE3_CPLLRESET_OUT        (cpllreset_cpll_cal_int[cal]),
          .GTHE3_CPLLPD_OUT           (cpllpd_cpll_cal_int[cal]),
          .GTHE3_TXPROGDIVRESET_OUT   (txprogdivreset_cpll_cal_int[cal]),
          .GTHE3_TXOUTCLKSEL_OUT      (txoutclksel_cpll_cal_int[(3*cal)+2:3*cal]),
          .GTHE3_TXPRGDIVRESETDONE_IN (txprgdivresetdone_int[cal]),
          .GTHE3_CHANNEL_DRPADDR_OUT  (drpaddr_cpll_cal_int[(9*cal)+8:9*cal]),
          .GTHE3_CHANNEL_DRPDI_OUT    (drpdi_cpll_cal_int[(16*cal)+15:16*cal]),
          .GTHE3_CHANNEL_DRPEN_OUT    (drpen_cpll_cal_int[cal]),
          .GTHE3_CHANNEL_DRPWE_OUT    (drpwe_cpll_cal_int[cal]),
          .GTHE3_CHANNEL_DRPRDY_IN    (drprdy_int[cal]),
          .GTHE3_CHANNEL_DRPDO_IN     (drpdo_int[(16*cal)+15:16*cal])
        );
      end

      // Assign signals as appropriate to connect to the CPLL calibration block when it is instantiated
      assign txprgdivresetdone_out = txprgdivresetdone_cpll_cal_int;
      assign cplllock_int          = cplllock_cpll_cal_int;
      assign drprdy_out            = drprdy_cpll_cal_int;
      assign drpdo_out             = drpdo_cpll_cal_int;
      assign cpllreset_ch_int      = cpllreset_cpll_cal_int;
      assign cpllpd_ch_int         = cpllpd_cpll_cal_int;
      assign txprogdivreset_ch_int = txprogdivreset_cpll_cal_int;
      assign txoutclksel_ch_int    = txoutclksel_cpll_cal_int;
      assign drpaddr_ch_int        = drpaddr_cpll_cal_int;
      assign drpdi_ch_int          = drpdi_cpll_cal_int;
      assign drpen_ch_int          = drpen_cpll_cal_int;
      assign drpwe_ch_int          = drpwe_cpll_cal_int;
    end

    // Assign signals as appropriate to bypass the CPLL calibration block when it is not instantiated
    else begin : gen_no_cpll_cal
      assign txprgdivresetdone_out = txprgdivresetdone_int;
      assign cplllock_int          = cplllock_ch_int;
      assign drprdy_out            = drprdy_int;
      assign drpdo_out             = drpdo_int;
      assign cpllreset_ch_int      = cpllreset_int;
      assign cpllpd_ch_int         = cpllpd_int;
      assign txprogdivreset_ch_int = txprogdivreset_int;
      assign txoutclksel_ch_int    = txoutclksel_int;
      assign drpaddr_ch_int        = drpaddr_int;
      assign drpdi_ch_int          = drpdi_int;
      assign drpen_ch_int          = drpen_int;
      assign drpwe_ch_int          = drpwe_int;
    end


    // ----------------------------------------------------------------------------------------------------------------
    // Transceiver common block to channel block internal wiring
    // ----------------------------------------------------------------------------------------------------------------

    // If one or more transceiver common blocks are included within the bounds of the core, then connect the relevant
    // QPLL outputs of those blocks to the corresponding inputs of the channel blocks, on a per-quad basis. Otherwise,
    // drive the channel QPLL-related inputs with the corresponding core inputs.
    if (`ice_gth_t_gtwizard_gthe3_N_CM > 0) begin : gen_common_channel_wiring_internal
      genvar gi_ch_to_cm;
      for (gi_ch_to_cm = 0; gi_ch_to_cm < `ice_gth_t_gtwizard_gthe3_N_CH; gi_ch_to_cm = gi_ch_to_cm + 1) begin : gen_quad
        assign qpll0clk_int    [gi_ch_to_cm] = qpll0outclk_int    [f_idx_cm(gi_ch_to_cm)];
        assign qpll0refclk_int [gi_ch_to_cm] = qpll0outrefclk_int [f_idx_cm(gi_ch_to_cm)];
        assign qpll1clk_int    [gi_ch_to_cm] = qpll1outclk_int    [f_idx_cm(gi_ch_to_cm)];
        assign qpll1refclk_int [gi_ch_to_cm] = qpll1outrefclk_int [f_idx_cm(gi_ch_to_cm)];
      end
    end
    else begin : gen_common_channel_wiring_external
      assign qpll0clk_int    = qpll0clk_in;
      assign qpll0refclk_int = qpll0refclk_in;
      assign qpll1clk_int    = qpll1clk_in;
      assign qpll1refclk_int = qpll1refclk_in;
    end


    // ================================================================================================================
    // HELPER BLOCKS
    // ================================================================================================================

    // ----------------------------------------------------------------------------------------------------------------
    // Transmitter user clocking network helper block
    // ----------------------------------------------------------------------------------------------------------------

    // Include the helper block within the core
    if ((C_TX_ENABLE == `ice_gth_t_gtwizard_gthe3_TX_ENABLE__ENABLED) &&
        (C_LOCATE_TX_USER_CLOCKING == `ice_gth_t_gtwizard_gthe3_LOCATE_TX_USER_CLOCKING__CORE)) begin : gen_tx_user_clocking_internal

      // Generate a single module instance which is driven by a clock source associated with the master transmitter
      // channel, and which drives TXUSRCLK and TXUSRCLK2 for all channels
      if (C_TX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance

        // The source clock is TXOUTCLK from the master transmitter channel
        if (C_TX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_SOURCE__TXOUTCLK) begin : gen_txoutclk_source
          assign gtwiz_userclk_tx_srcclk_out = txoutclk_int[P_TX_MASTER_CH_PACKED_IDX];
        end

        // The source clock is the fabric-accessible output of the IBUFDS_GTE3 associated with the master transmitter
        // channel
        else if (C_TX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_SOURCE__IBUFDS) begin : gen_ibufds_source
          if (C_TX_PLL_TYPE == `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0) begin: gen_ibufds_source_qpll0
            assign gtwiz_userclk_tx_srcclk_out = gtrefclk00_int[f_idx_cm(P_TX_MASTER_CH_PACKED_IDX)];
          end
          else if (C_TX_PLL_TYPE == `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1) begin: gen_ibufds_source_qpll1
            assign gtwiz_userclk_tx_srcclk_out = gtrefclk01_int[f_idx_cm(P_TX_MASTER_CH_PACKED_IDX)];
          end
          else begin: gen_ibufds_source_cpll
            assign gtwiz_userclk_tx_srcclk_out = gtrefclk0_int[P_TX_MASTER_CH_PACKED_IDX];
          end
        end

        // Instantiate a single instance of the transmitter user clocking network helper block
        gtwizard_ultrascale_v1_6_2_gtwiz_userclk_tx #(
          .P_CONTENTS                     (C_TX_USER_CLOCKING_CONTENTS),
          .P_FREQ_RATIO_SOURCE_TO_USRCLK  (C_TX_OUTCLK_BUFG_GT_DIV),
          .P_FREQ_RATIO_USRCLK_TO_USRCLK2 (C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2)
        ) gtwiz_userclk_tx_inst_xxx (
          .gtwiz_userclk_tx_srcclk_in   (gtwiz_userclk_tx_srcclk_out),
          .gtwiz_userclk_tx_reset_in    (gtwiz_userclk_tx_reset_in),
          .gtwiz_userclk_tx_usrclk_out  (gtwiz_userclk_tx_usrclk_out),
          .gtwiz_userclk_tx_usrclk2_out (gtwiz_userclk_tx_usrclk2_out),
          .gtwiz_userclk_tx_active_out  (gtwiz_userclk_tx_active_out)
        );

        // Drive TXUSRCLK and TXUSRCLK2 for all channels with the respective helper block outputs
        assign txusrclk_int  = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_userclk_tx_usrclk_out}};
        assign txusrclk2_int = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_userclk_tx_usrclk2_out}};

      end

      // For each channel, generate one module instance which is driven by a clock source associated with that channel,
      // and which drives TXUSRCLK and TXUSRCLK2 for that same channel
      else if (C_TX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_INSTANCE_CTRL__PER_CHANNEL)
      begin : gen_per_channel_instance

        // Instantiate one instance of the transmitter user clocking network helper block for each channel
        genvar gi_hb_txclk;
        for (gi_hb_txclk = 0; gi_hb_txclk < `ice_gth_t_gtwizard_gthe3_N_CH; gi_hb_txclk = gi_hb_txclk + 1) begin : gen_gtwiz_userclk_tx

          // The source clock for a given instance is TXOUTCLK from the associated channel
          if (C_TX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_SOURCE__TXOUTCLK) begin : gen_txoutclk_source
            assign gtwiz_userclk_tx_srcclk_out[gi_hb_txclk] = txoutclk_int[gi_hb_txclk];
          end

          // The source clock for a given instance is the fabric-accessible output of the IBUFDS_GTE3 associated with that
          // channel
          else if (C_TX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_SOURCE__IBUFDS) begin : gen_ibufds_source
            if (C_TX_PLL_TYPE == `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0) begin: gen_ibufds_source_qpll0
              assign gtwiz_userclk_tx_srcclk_out[gi_hb_txclk] = gtrefclk00_int[f_idx_cm(gi_hb_txclk)];
            end
            else if (C_TX_PLL_TYPE == `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1) begin: gen_ibufds_source_qpll1
              assign gtwiz_userclk_tx_srcclk_out[gi_hb_txclk] = gtrefclk01_int[f_idx_cm(gi_hb_txclk)];
            end
            else begin: gen_ibufds_source_cpll
              assign gtwiz_userclk_tx_srcclk_out[gi_hb_txclk] = gtrefclk0_int[gi_hb_txclk];
            end
          end

          gtwizard_ultrascale_v1_6_2_gtwiz_userclk_tx #(
            .P_CONTENTS                     (C_TX_USER_CLOCKING_CONTENTS),
            .P_FREQ_RATIO_SOURCE_TO_USRCLK  (C_TX_OUTCLK_BUFG_GT_DIV),
            .P_FREQ_RATIO_USRCLK_TO_USRCLK2 (C_TX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2)
          ) gtwiz_userclk_tx_inst_jjb (
            .gtwiz_userclk_tx_srcclk_in   (gtwiz_userclk_tx_srcclk_out  [gi_hb_txclk]),
            .gtwiz_userclk_tx_reset_in    (gtwiz_userclk_tx_reset_in    [gi_hb_txclk]),
            .gtwiz_userclk_tx_usrclk_out  (gtwiz_userclk_tx_usrclk_out  [gi_hb_txclk]),
            .gtwiz_userclk_tx_usrclk2_out (gtwiz_userclk_tx_usrclk2_out [gi_hb_txclk]),
            .gtwiz_userclk_tx_active_out  (gtwiz_userclk_tx_active_out  [gi_hb_txclk])
          );
        end

        // Drive TXUSRCLK and TXUSRCLK2 for each channel with the respective outputs of the associated helper block
        assign txusrclk_int  = gtwiz_userclk_tx_usrclk_out;
        assign txusrclk2_int = gtwiz_userclk_tx_usrclk2_out;

      end

    end

    // Do not include the helper block within the core
    else begin : gen_tx_user_clocking_external

      if (C_TX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance
        assign gtwiz_userclk_tx_srcclk_out  = 1'b0;
        assign gtwiz_userclk_tx_usrclk_out  = 1'b0;
        assign gtwiz_userclk_tx_usrclk2_out = 1'b0;
      end
      else begin : gen_per_channel_instance
        assign gtwiz_userclk_tx_srcclk_out  = {(C_TX_USER_CLOCKING_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
        assign gtwiz_userclk_tx_usrclk_out  = {(C_TX_USER_CLOCKING_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
        assign gtwiz_userclk_tx_usrclk2_out = {(C_TX_USER_CLOCKING_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
      end

      assign gtwiz_userclk_tx_active_out = gtwiz_userclk_tx_active_in;
      assign txusrclk_int                = txusrclk_in;
      assign txusrclk2_int               = txusrclk2_in;
    end


    // ----------------------------------------------------------------------------------------------------------------
    // Receiver user clocking network helper block
    // ----------------------------------------------------------------------------------------------------------------

    // Include the helper block within the core
    if ((C_RX_ENABLE == `ice_gth_t_gtwizard_gthe3_RX_ENABLE__ENABLED) &&
        (C_LOCATE_RX_USER_CLOCKING == `ice_gth_t_gtwizard_gthe3_LOCATE_RX_USER_CLOCKING__CORE)) begin : gen_rx_user_clocking_internal

      // Generate a single module instance which is driven by a clock source associated with the master receiver
      // channel, and which drives RXUSRCLK and RXUSRCLK2 for all channels
      if (C_RX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance

        // The source clock is RXOUTCLK from the master receiver channel
        if (C_RX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__RXOUTCLK) begin : gen_rxoutclk_source
          assign gtwiz_userclk_rx_srcclk_out = rxoutclk_int[P_RX_MASTER_CH_PACKED_IDX];
        end

        // The source clock is the fabric-accessible output of the IBUFDS_GTE3 associated with the master receiver
        // channel
        else if (C_RX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__IBUFDS) begin : gen_ibufds_source
          if (C_RX_PLL_TYPE == `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0) begin: gen_ibufds_source_qpll0
            assign gtwiz_userclk_rx_srcclk_out = gtrefclk00_int[f_idx_cm(P_RX_MASTER_CH_PACKED_IDX)];
          end
          else if (C_RX_PLL_TYPE == `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1) begin: gen_ibufds_source_qpll1
            assign gtwiz_userclk_rx_srcclk_out = gtrefclk01_int[f_idx_cm(P_RX_MASTER_CH_PACKED_IDX)];
          end
          else begin: gen_ibufds_source_cpll
            assign gtwiz_userclk_rx_srcclk_out = gtrefclk0_int[P_RX_MASTER_CH_PACKED_IDX];
          end
        end

        // The source clock is TXOUTCLK from the master transmitter channel
        else if (C_RX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__TXOUTCLK) begin : gen_txoutclk_source
          assign gtwiz_userclk_rx_srcclk_out = txoutclk_int[P_TX_MASTER_CH_PACKED_IDX];
        end

        // Instantiate a single instance of the receiver user clocking network helper block
        gtwizard_ultrascale_v1_6_2_gtwiz_userclk_rx #(
          .P_CONTENTS                     (C_RX_USER_CLOCKING_CONTENTS),
          .P_FREQ_RATIO_SOURCE_TO_USRCLK  (C_RX_OUTCLK_BUFG_GT_DIV),
          .P_FREQ_RATIO_USRCLK_TO_USRCLK2 (C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2)
        ) gtwiz_userclk_rx_inst (
          .gtwiz_userclk_rx_srcclk_in   (gtwiz_userclk_rx_srcclk_out),
          .gtwiz_userclk_rx_reset_in    (gtwiz_userclk_rx_reset_in),
          .gtwiz_userclk_rx_usrclk_out  (gtwiz_userclk_rx_usrclk_out),
          .gtwiz_userclk_rx_usrclk2_out (gtwiz_userclk_rx_usrclk2_out),
          .gtwiz_userclk_rx_active_out  (gtwiz_userclk_rx_active_out)
        );

        // Drive RXUSRCLK and RXUSRCLK2 for all channels with the respective helper block outputs
        //assign rxusrclk_int  = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_userclk_rx_usrclk_out}};  //JJB:::
        //assign rxusrclk2_int = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_userclk_rx_usrclk2_out}}; //JJB:::

      end

      // For each channel, generate one module instance which is driven by a clock source associated with that channel,
      // and which drives RXUSRCLK and RXUSRCLK2 for that same channel
      else if (C_RX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_INSTANCE_CTRL__PER_CHANNEL)
      begin : gen_per_channel_instance

        // Instantiate one instance of the receiver user clocking network helper block for each channel
        genvar gi_hb_rxclk;
        for (gi_hb_rxclk = 0; gi_hb_rxclk < `ice_gth_t_gtwizard_gthe3_N_CH; gi_hb_rxclk = gi_hb_rxclk + 1) begin : gen_gtwiz_userclk_rx

          // The source clock for a given instance is RXOUTCLK from the associated channel
          if (C_RX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__RXOUTCLK) begin : gen_rxoutclk_source
            assign gtwiz_userclk_rx_srcclk_out[gi_hb_rxclk] = rxoutclk_int[gi_hb_rxclk];
          end

          // The source clock for a given instance is the fabric-accessible output of the IBUFDS_GTE3 associated with that
          // channel
          else if (C_RX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__IBUFDS) begin : gen_ibufds_source
            if (C_RX_PLL_TYPE == `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0) begin: gen_ibufds_source_qpll0
              assign gtwiz_userclk_rx_srcclk_out[gi_hb_rxclk] = gtrefclk00_int[f_idx_cm(gi_hb_rxclk)];
            end
            else if (C_RX_PLL_TYPE == `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1) begin: gen_ibufds_source_qpll1
              assign gtwiz_userclk_rx_srcclk_out[gi_hb_rxclk] = gtrefclk01_int[f_idx_cm(gi_hb_rxclk)];
            end
            else begin: gen_ibufds_source_cpll
              assign gtwiz_userclk_rx_srcclk_out[gi_hb_rxclk] = gtrefclk0_int[gi_hb_rxclk];
            end
          end

          // The source clock for a given instance is TXOUTCLK from the associated channel
          else if (C_RX_USER_CLOCKING_SOURCE == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_SOURCE__TXOUTCLK) begin : gen_txoutclk_source
            assign gtwiz_userclk_rx_srcclk_out[gi_hb_rxclk] = txoutclk_int[gi_hb_rxclk];
          end

          gtwizard_ultrascale_v1_6_2_gtwiz_userclk_rx #(
            .P_CONTENTS                     (C_RX_USER_CLOCKING_CONTENTS),
            .P_FREQ_RATIO_SOURCE_TO_USRCLK  (C_RX_OUTCLK_BUFG_GT_DIV),
            .P_FREQ_RATIO_USRCLK_TO_USRCLK2 (C_RX_USER_CLOCKING_RATIO_FUSRCLK_FUSRCLK2)
          ) gtwiz_userclk_rx_inst (
            .gtwiz_userclk_rx_srcclk_in   (gtwiz_userclk_rx_srcclk_out  [gi_hb_rxclk]),
            .gtwiz_userclk_rx_reset_in    (gtwiz_userclk_rx_reset_in    [gi_hb_rxclk]),
            .gtwiz_userclk_rx_usrclk_out  (gtwiz_userclk_rx_usrclk_out  [gi_hb_rxclk]),
            .gtwiz_userclk_rx_usrclk2_out (gtwiz_userclk_rx_usrclk2_out [gi_hb_rxclk]),
            .gtwiz_userclk_rx_active_out  (gtwiz_userclk_rx_active_out  [gi_hb_rxclk])
          );
        end

        // Drive RXUSRCLK and RXUSRCLK2 for each channel with the respective outputs of the associated helper block
        //assign rxusrclk_int  = gtwiz_userclk_rx_usrclk_out;  //JJB:::
        //assign rxusrclk2_int = gtwiz_userclk_rx_usrclk2_out; //JJB:::

      end

    end

    // Do not include the helper block within the core
    else begin : gen_rx_user_clocking_external

      if (C_RX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance
        assign gtwiz_userclk_rx_srcclk_out  = 1'b0;
        assign gtwiz_userclk_rx_usrclk_out  = 1'b0;
        assign gtwiz_userclk_rx_usrclk2_out = 1'b0;
      end
      else begin : gen_per_channel_instance
        assign gtwiz_userclk_rx_srcclk_out  = {(C_RX_USER_CLOCKING_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
        assign gtwiz_userclk_rx_usrclk_out  = {(C_RX_USER_CLOCKING_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
        assign gtwiz_userclk_rx_usrclk2_out = {(C_RX_USER_CLOCKING_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
      end

      assign gtwiz_userclk_rx_active_out = gtwiz_userclk_rx_active_in;
      //assign rxusrclk_int                = rxusrclk_in;  //JJB:::
      //assign rxusrclk2_int               = rxusrclk2_in; //JJB:::
    end


    // ----------------------------------------------------------------------------------------------------------------
    // Transmitter buffer bypass controller helper block
    // ----------------------------------------------------------------------------------------------------------------

    // Include the helper block within the core
    if ((C_TX_ENABLE == `ice_gth_t_gtwizard_gthe3_TX_ENABLE__ENABLED) &&
        (C_TX_BUFFER_MODE == `ice_gth_t_gtwizard_gthe3_TX_BUFFER_MODE__BYPASS) &&
        (C_LOCATE_TX_BUFFER_BYPASS_CONTROLLER == `ice_gth_t_gtwizard_gthe3_LOCATE_TX_BUFFER_BYPASS_CONTROLLER__CORE))
        begin : gen_tx_buffer_bypass_internal

      // Generate a single module instance which uses the designated transmitter master channel as the transmit buffer
      // bypass master channel, and all other channels as transmit buffer bypass slave channels
      if (C_TX_BUFFER_BYPASS_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_TX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance

        // Depending on the number of reset controller helper blocks, either use the single reset done indicator or the
        // logical combination of per-channel reset done indicators as the reset done indicator for use in this block
        wire gtwiz_buffbypass_tx_resetdone_int;

        if (C_RESET_CONTROLLER_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE)
        begin : gen_resetdone_single_instance
          assign gtwiz_buffbypass_tx_resetdone_int = gtwiz_reset_tx_done_out;
        end
        else begin : gen_resetdone_per_channel_instance
          assign gtwiz_buffbypass_tx_resetdone_int = &gtwiz_reset_tx_done_out;
        end

        gtwizard_ultrascale_v1_6_2_gtwiz_buffbypass_tx #(
          .P_BUFFER_BYPASS_MODE       (C_TX_BUFFBYPASS_MODE),
          .P_TOTAL_NUMBER_OF_CHANNELS (C_TOTAL_NUM_CHANNELS),
          .P_MASTER_CHANNEL_POINTER   (P_TX_MASTER_CH_PACKED_IDX)
        ) gtwiz_buffbypass_tx_inst (
          .gtwiz_buffbypass_tx_clk_in        (txusrclk2_int[P_TX_MASTER_CH_PACKED_IDX]),
          .gtwiz_buffbypass_tx_reset_in      (gtwiz_buffbypass_tx_reset_in),
          .gtwiz_buffbypass_tx_start_user_in (gtwiz_buffbypass_tx_start_user_in),
          .gtwiz_buffbypass_tx_resetdone_in  (gtwiz_buffbypass_tx_resetdone_int),
          .gtwiz_buffbypass_tx_done_out      (gtwiz_buffbypass_tx_done_out),
          .gtwiz_buffbypass_tx_error_out     (gtwiz_buffbypass_tx_error_out),
          .txphaligndone_in                  (txphaligndone_int),
          .txphinitdone_in                   (txphinitdone_int),
          .txdlysresetdone_in                (txdlysresetdone_int),
          .txsyncout_in                      (txsyncout_int),
          .txsyncdone_in                     (txsyncdone_int),
          .txphdlyreset_out                  (txphdlyreset_int),
          .txphalign_out                     (txphalign_int),
          .txphalignen_out                   (txphalignen_int),
          .txphdlypd_out                     (txphdlypd_int),
          .txphinit_out                      (txphinit_int),
          .txphovrden_out                    (txphovrden_int),
          .txdlysreset_out                   (txdlysreset_int),
          .txdlybypass_out                   (txdlybypass_int),
          .txdlyen_out                       (txdlyen_int),
          .txdlyovrden_out                   (txdlyovrden_int),
          .txphdlytstclk_out                 (txphdlytstclk_int),
          .txdlyhold_out                     (txdlyhold_int),
          .txdlyupdown_out                   (txdlyupdown_int),
          .txsyncmode_out                    (txsyncmode_int),
          .txsyncallin_out                   (txsyncallin_int),
          .txsyncin_out                      (txsyncin_int)
        );

      end

      // Generate one module instance per channel to implement single-lane transmit buffer bypass for each lane
      // independently, treating each lane as a master
      else if (C_TX_BUFFER_BYPASS_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_TX_BUFFER_BYPASS_INSTANCE_CTRL__PER_CHANNEL)
      begin : gen_per_channel_instance

        // Depending on the number of reset controller helper blocks, either use the single reset done indicator or the
        // logical combination of per-channel reset done indicators as the reset done indicator for use in this block
        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] gtwiz_buffbypass_tx_resetdone_int;

        if (C_RESET_CONTROLLER_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE)
        begin : gen_resetdone_single_instance
          assign gtwiz_buffbypass_tx_resetdone_int = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_tx_done_out}};
        end
        else begin : gen_resetdone_per_channel_instance
          assign gtwiz_buffbypass_tx_resetdone_int = gtwiz_reset_tx_done_out;
        end

        genvar gi_hb_txbb;
        for (gi_hb_txbb = 0; gi_hb_txbb < `ice_gth_t_gtwizard_gthe3_N_CH; gi_hb_txbb = gi_hb_txbb + 1) begin : gen_gtwiz_buffbypass_tx
          gtwizard_ultrascale_v1_6_2_gtwiz_buffbypass_tx #(
            .P_BUFFER_BYPASS_MODE       (C_TX_BUFFBYPASS_MODE),
            .P_TOTAL_NUMBER_OF_CHANNELS (1),
            .P_MASTER_CHANNEL_POINTER   (0)
          ) gtwiz_buffbypass_tx_inst (
            .gtwiz_buffbypass_tx_clk_in        (txusrclk2_int                     [gi_hb_txbb]),
            .gtwiz_buffbypass_tx_reset_in      (gtwiz_buffbypass_tx_reset_in      [gi_hb_txbb]),
            .gtwiz_buffbypass_tx_start_user_in (gtwiz_buffbypass_tx_start_user_in [gi_hb_txbb]),
            .gtwiz_buffbypass_tx_resetdone_in  (gtwiz_buffbypass_tx_resetdone_int [gi_hb_txbb]),
            .gtwiz_buffbypass_tx_done_out      (gtwiz_buffbypass_tx_done_out      [gi_hb_txbb]),
            .gtwiz_buffbypass_tx_error_out     (gtwiz_buffbypass_tx_error_out     [gi_hb_txbb]),
            .txphaligndone_in                  (txphaligndone_int                 [gi_hb_txbb]),
            .txphinitdone_in                   (txphinitdone_int                  [gi_hb_txbb]),
            .txdlysresetdone_in                (txdlysresetdone_int               [gi_hb_txbb]),
            .txsyncout_in                      (txsyncout_int                     [gi_hb_txbb]),
            .txsyncdone_in                     (txsyncdone_int                    [gi_hb_txbb]),
            .txphdlyreset_out                  (txphdlyreset_int                  [gi_hb_txbb]),
            .txphalign_out                     (txphalign_int                     [gi_hb_txbb]),
            .txphalignen_out                   (txphalignen_int                   [gi_hb_txbb]),
            .txphdlypd_out                     (txphdlypd_int                     [gi_hb_txbb]),
            .txphinit_out                      (txphinit_int                      [gi_hb_txbb]),
            .txphovrden_out                    (txphovrden_int                    [gi_hb_txbb]),
            .txdlysreset_out                   (txdlysreset_int                   [gi_hb_txbb]),
            .txdlybypass_out                   (txdlybypass_int                   [gi_hb_txbb]),
            .txdlyen_out                       (txdlyen_int                       [gi_hb_txbb]),
            .txdlyovrden_out                   (txdlyovrden_int                   [gi_hb_txbb]),
            .txphdlytstclk_out                 (txphdlytstclk_int                 [gi_hb_txbb]),
            .txdlyhold_out                     (txdlyhold_int                     [gi_hb_txbb]),
            .txdlyupdown_out                   (txdlyupdown_int                   [gi_hb_txbb]),
            .txsyncmode_out                    (txsyncmode_int                    [gi_hb_txbb]),
            .txsyncallin_out                   (txsyncallin_int                   [gi_hb_txbb]),
            .txsyncin_out                      (txsyncin_int                      [gi_hb_txbb])
          );
        end

      end

    end

    // Do not include the helper block within the core
    else begin : gen_no_or_external_tx_buffer_bypass

      if (C_TX_BUFFER_BYPASS_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_TX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance
        assign gtwiz_buffbypass_tx_done_out  = 1'b0;
        assign gtwiz_buffbypass_tx_error_out = 1'b0;
      end
      else begin : gen_per_channel_instance
        assign gtwiz_buffbypass_tx_done_out  = {(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
        assign gtwiz_buffbypass_tx_error_out = {(C_TX_BUFFER_BYPASS_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
      end

      assign txphdlyreset_int  = txphdlyreset_in;
      assign txphalign_int     = txphalign_in;
      assign txphalignen_int   = txphalignen_in;
      assign txphdlypd_int     = txphdlypd_in;
      assign txphinit_int      = txphinit_in;
      assign txphovrden_int    = txphovrden_in;
      assign txdlysreset_int   = txdlysreset_in;
      assign txdlybypass_int   = txdlybypass_in;
      assign txdlyen_int       = txdlyen_in;
      assign txdlyovrden_int   = txdlyovrden_in;
      assign txphdlytstclk_int = txphdlytstclk_in;
      assign txdlyhold_int     = txdlyhold_in;
      assign txdlyupdown_int   = txdlyupdown_in;
      assign txsyncmode_int    = txsyncmode_in;
      assign txsyncallin_int   = txsyncallin_in;
      assign txsyncin_int      = txsyncin_in;

    end


    // ----------------------------------------------------------------------------------------------------------------
    // Receiver buffer bypass controller helper block
    // ----------------------------------------------------------------------------------------------------------------

    // Include the helper block within the core
    if ((C_RX_ENABLE == `ice_gth_t_gtwizard_gthe3_RX_ENABLE__ENABLED) &&
        (C_RX_BUFFER_MODE == `ice_gth_t_gtwizard_gthe3_RX_BUFFER_MODE__BYPASS) &&
        (C_LOCATE_RX_BUFFER_BYPASS_CONTROLLER == `ice_gth_t_gtwizard_gthe3_LOCATE_RX_BUFFER_BYPASS_CONTROLLER__CORE))
        begin : gen_rx_buffer_bypass_internal

      // Generate a single module instance which uses the designated receiver master channel as the receive buffer
      // bypass master channel, and all other channels as receive buffer bypass slave channels
      if (C_RX_BUFFER_BYPASS_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance

        // Depending on the number of reset controller helper blocks, either use the single reset done indicator or the
        // logical combination of per-channel reset done indicators as the reset done indicator for use in this block
        wire gtwiz_buffbypass_rx_resetdone_int;

        if (C_RESET_CONTROLLER_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE)
        begin : gen_resetdone_single_instance
          assign gtwiz_buffbypass_rx_resetdone_int = gtwiz_reset_rx_done_out;
        end
        else begin : gen_resetdone_per_channel_instance
          assign gtwiz_buffbypass_rx_resetdone_int = &gtwiz_reset_rx_done_out;
        end

        gtwizard_ultrascale_v1_6_2_gtwiz_buffbypass_rx #(
          .P_BUFFER_BYPASS_MODE       (C_RX_BUFFBYPASS_MODE),
          .P_TOTAL_NUMBER_OF_CHANNELS (C_TOTAL_NUM_CHANNELS),
          .P_MASTER_CHANNEL_POINTER   (P_RX_MASTER_CH_PACKED_IDX)
        ) gtwiz_buffbypass_rx_inst (
          .gtwiz_buffbypass_rx_clk_in        (rxusrclk2_int[P_RX_MASTER_CH_PACKED_IDX]),
          .gtwiz_buffbypass_rx_reset_in      (gtwiz_buffbypass_rx_reset_in),
          .gtwiz_buffbypass_rx_start_user_in (gtwiz_buffbypass_rx_start_user_in),
          .gtwiz_buffbypass_rx_resetdone_in  (gtwiz_buffbypass_rx_resetdone_int),
          .gtwiz_buffbypass_rx_done_out      (gtwiz_buffbypass_rx_done_out),
          .gtwiz_buffbypass_rx_error_out     (gtwiz_buffbypass_rx_error_out),
          .rxphaligndone_in                  (rxphaligndone_int),
          .rxdlysresetdone_in                (rxdlysresetdone_int),
          .rxsyncout_in                      (rxsyncout_int),
          .rxsyncdone_in                     (rxsyncdone_int),
          .rxphdlyreset_out                  (rxphdlyreset_int),
          .rxphalign_out                     (rxphalign_int),
          .rxphalignen_out                   (rxphalignen_int),
          .rxphdlypd_out                     (rxphdlypd_int),
          .rxphovrden_out                    (rxphovrden_int),
          .rxdlysreset_out                   (rxdlysreset_int),
          .rxdlybypass_out                   (rxdlybypass_int),
          .rxdlyen_out                       (rxdlyen_int),
          .rxdlyovrden_out                   (rxdlyovrden_int),
          .rxsyncmode_out                    (rxsyncmode_int),
          .rxsyncallin_out                   (rxsyncallin_int),
          .rxsyncin_out                      (rxsyncin_int)
        );

      end

      // Generate one module instance per channel to implement single-lane receive buffer bypass for each lane
      // independently, treating each lane as a master
      else if (C_RX_BUFFER_BYPASS_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RX_BUFFER_BYPASS_INSTANCE_CTRL__PER_CHANNEL)
      begin : gen_per_channel_instance

        // Depending on the number of reset controller helper blocks, either use the single reset done indicator or the
        // logical combination of per-channel reset done indicators as the reset done indicator for use in this block
        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] gtwiz_buffbypass_rx_resetdone_int;

        if (C_RESET_CONTROLLER_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE)
        begin : gen_resetdone_single_instance
          assign gtwiz_buffbypass_rx_resetdone_int = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_rx_done_out}};
        end
        else begin : gen_resetdone_per_channel_instance
          assign gtwiz_buffbypass_rx_resetdone_int = gtwiz_reset_rx_done_out;
        end

        genvar gi_hb_rxbb;
        for (gi_hb_rxbb = 0; gi_hb_rxbb < `ice_gth_t_gtwizard_gthe3_N_CH; gi_hb_rxbb = gi_hb_rxbb + 1) begin : gen_gtwiz_buffbypass_rx
          gtwizard_ultrascale_v1_6_2_gtwiz_buffbypass_rx #(
            .P_BUFFER_BYPASS_MODE       (C_RX_BUFFBYPASS_MODE),
            .P_TOTAL_NUMBER_OF_CHANNELS (1),
            .P_MASTER_CHANNEL_POINTER   (0)
          ) gtwiz_buffbypass_rx_inst (
            .gtwiz_buffbypass_rx_clk_in        (rxusrclk2_int                     [gi_hb_rxbb]),
            .gtwiz_buffbypass_rx_reset_in      (gtwiz_buffbypass_rx_reset_in      [gi_hb_rxbb]),
            .gtwiz_buffbypass_rx_start_user_in (gtwiz_buffbypass_rx_start_user_in [gi_hb_rxbb]),
            .gtwiz_buffbypass_rx_resetdone_in  (gtwiz_buffbypass_rx_resetdone_int [gi_hb_rxbb]),
            .gtwiz_buffbypass_rx_done_out      (gtwiz_buffbypass_rx_done_out      [gi_hb_rxbb]),
            .gtwiz_buffbypass_rx_error_out     (gtwiz_buffbypass_rx_error_out     [gi_hb_rxbb]),
            .rxphaligndone_in                  (rxphaligndone_int                 [gi_hb_rxbb]),
            .rxdlysresetdone_in                (rxdlysresetdone_int               [gi_hb_rxbb]),
            .rxsyncout_in                      (rxsyncout_int                     [gi_hb_rxbb]),
            .rxsyncdone_in                     (rxsyncdone_int                    [gi_hb_rxbb]),
            .rxphdlyreset_out                  (rxphdlyreset_int                  [gi_hb_rxbb]),
            .rxphalign_out                     (rxphalign_int                     [gi_hb_rxbb]),
            .rxphalignen_out                   (rxphalignen_int                   [gi_hb_rxbb]),
            .rxphdlypd_out                     (rxphdlypd_int                     [gi_hb_rxbb]),
            .rxphovrden_out                    (rxphovrden_int                    [gi_hb_rxbb]),
            .rxdlysreset_out                   (rxdlysreset_int                   [gi_hb_rxbb]),
            .rxdlybypass_out                   (rxdlybypass_int                   [gi_hb_rxbb]),
            .rxdlyen_out                       (rxdlyen_int                       [gi_hb_rxbb]),
            .rxdlyovrden_out                   (rxdlyovrden_int                   [gi_hb_rxbb]),
            .rxsyncmode_out                    (rxsyncmode_int                    [gi_hb_rxbb]),
            .rxsyncallin_out                   (rxsyncallin_int                   [gi_hb_rxbb]),
            .rxsyncin_out                      (rxsyncin_int                      [gi_hb_rxbb])
          );
        end

      end

    end

    // Do not include the helper block within the core
    else begin : gen_no_or_external_rx_buffer_bypass

      if (C_RX_BUFFER_BYPASS_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RX_BUFFER_BYPASS_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance
        assign gtwiz_buffbypass_rx_done_out  = 1'b0;
        assign gtwiz_buffbypass_rx_error_out = 1'b0;
      end
      else begin : gen_per_channel_instance
        assign gtwiz_buffbypass_rx_done_out  = {(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
        assign gtwiz_buffbypass_rx_error_out = {(C_RX_BUFFER_BYPASS_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
      end

      assign rxphdlyreset_int = rxphdlyreset_in;
      assign rxphalign_int    = rxphalign_in;
      assign rxphalignen_int  = rxphalignen_in;
      assign rxphdlypd_int    = rxphdlypd_in;
      assign rxphovrden_int   = rxphovrden_in;
      assign rxdlysreset_int  = rxdlysreset_in;
      assign rxdlybypass_int  = rxdlybypass_in;
      assign rxdlyen_int      = rxdlyen_in;
      assign rxdlyovrden_int  = rxdlyovrden_in;
      assign rxsyncmode_int   = rxsyncmode_in;
      assign rxsyncallin_int  = rxsyncallin_in;
      assign rxsyncin_int     = rxsyncin_in;

    end


    // ----------------------------------------------------------------------------------------------------------------
    // Reset controller helper block
    // ----------------------------------------------------------------------------------------------------------------

    // Include the helper block within the core
    if (C_LOCATE_RESET_CONTROLLER == `ice_gth_t_gtwizard_gthe3_LOCATE_RESET_CONTROLLER__CORE) begin : gen_reset_controller_internal

      // Generate a single module instance which controls all PLLs and all channels within the core
      if (C_RESET_CONTROLLER_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance

        // Depending on the number of user clocking network helper blocks, either use the single user clock active
        // indicator or a logical combination of per-channel user clock active indicators as the user clock active
        // indicator for use in this block
        wire gtwiz_reset_userclk_tx_active_int;
        wire gtwiz_reset_userclk_rx_active_int;

        if (C_TX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE)
        begin : gen_txuserclkactive_single_instance
          assign gtwiz_reset_userclk_tx_active_int = gtwiz_userclk_tx_active_out;
        end
        else begin : gen_txuserclkactive_per_channel_instance
          assign gtwiz_reset_userclk_tx_active_int = &gtwiz_userclk_tx_active_out;
        end
        if (C_RX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE)
        begin : gen_rxuserclkactive_single_instance
          assign gtwiz_reset_userclk_rx_active_int = gtwiz_userclk_rx_active_out;
        end
        else begin : gen_rxuserclkactive_per_channel_instance

          // When each channel has an independent receiver user clock, synchronize each receiver user clock active
          // indicator independently before combining them
          wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] gtwiz_userclk_rx_active_sync;

          genvar gi_ch_rxclk;
          for (gi_ch_rxclk = 0; gi_ch_rxclk < `ice_gth_t_gtwizard_gthe3_N_CH; gi_ch_rxclk = gi_ch_rxclk + 1) begin : gen_ch_rxclk
            gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_gtwiz_reset_userclk_rx_active_inst (
              .clk_in (gtwiz_reset_clk_freerun_in),
              .i_in   (gtwiz_userclk_rx_active_out[gi_ch_rxclk]),
              .o_out  (gtwiz_userclk_rx_active_sync[gi_ch_rxclk])
            );
          end
          assign gtwiz_reset_userclk_rx_active_int = &gtwiz_userclk_rx_active_sync;
        end

        wire gtwiz_reset_gtpowergood_int;
        wire gtwiz_reset_plllock_tx_int;
        wire gtwiz_reset_txresetdone_int;
        wire gtwiz_reset_plllock_rx_int;
        wire gtwiz_reset_rxcdrlock_int;
        wire gtwiz_reset_rxresetdone_int;
        wire gtwiz_reset_pllreset_tx_int;
        wire gtwiz_reset_txprogdivreset_int;
        wire gtwiz_reset_gttxreset_int;
        wire gtwiz_reset_txuserrdy_int;
        wire gtwiz_reset_pllreset_rx_int;
        wire gtwiz_reset_rxprogdivreset_int;
        wire gtwiz_reset_gtrxreset_int;
        wire gtwiz_reset_rxuserrdy_int;

        // Combine the appropriate PLL lock signals such that the reset controller can sense when all PLLs which clock
        // each data direction are locked, regardless of what PLL source is used
        case (C_TX_PLL_TYPE)
          `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0: assign gtwiz_reset_plllock_tx_int = &qpll0lock_int;
          `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1: assign gtwiz_reset_plllock_tx_int = &qpll1lock_int;
          `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL:  assign gtwiz_reset_plllock_tx_int = &cplllock_int;
        endcase
        case (C_RX_PLL_TYPE)
          `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0: assign gtwiz_reset_plllock_rx_int = &qpll0lock_int;
          `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1: assign gtwiz_reset_plllock_rx_int = &qpll1lock_int;
          `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL:  assign gtwiz_reset_plllock_rx_int = &cplllock_int;
        endcase

        // Combine the power good, reset done, and CDR lock indicators across all channels, per data direction
        assign gtwiz_reset_gtpowergood_int = &gtpowergood_int;
        assign gtwiz_reset_rxcdrlock_int   = &rxcdrlock_int;

        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] txresetdone_sync;
        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] rxresetdone_sync;
        genvar gi_ch_xrd;
        for (gi_ch_xrd = 0; gi_ch_xrd < `ice_gth_t_gtwizard_gthe3_N_CH; gi_ch_xrd = gi_ch_xrd + 1) begin : gen_ch_xrd
          gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_txresetdone_inst (
            .clk_in (gtwiz_reset_clk_freerun_in),
            .i_in   (txresetdone_int[gi_ch_xrd]),
            .o_out  (txresetdone_sync[gi_ch_xrd])
          );
          gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_rxresetdone_inst (
            .clk_in (gtwiz_reset_clk_freerun_in),
            .i_in   (rxresetdone_int[gi_ch_xrd]),
            .o_out  (rxresetdone_sync[gi_ch_xrd])
          );
        end
        assign gtwiz_reset_txresetdone_int = &txresetdone_sync;
        assign gtwiz_reset_rxresetdone_int = &rxresetdone_sync;

        // Assign tie-off values for use in reset controller state machine traversal
        wire gtwiz_reset_tx_enabled_tie_int;
        wire gtwiz_reset_rx_enabled_tie_int;
        wire gtwiz_reset_shared_pll_tie_int;

        if (C_TX_ENABLE == `ice_gth_t_gtwizard_gthe3_TX_ENABLE__ENABLED)
        begin : gen_reset_tx_enabled_tie_high
          assign gtwiz_reset_tx_enabled_tie_int = 1'b1;
        end
        else begin : gen_reset_tx_enabled_tie_low
          assign gtwiz_reset_tx_enabled_tie_int = 1'b0;
        end

        if (C_RX_ENABLE == `ice_gth_t_gtwizard_gthe3_RX_ENABLE__ENABLED)
        begin : gen_reset_rx_enabled_tie_high
          assign gtwiz_reset_rx_enabled_tie_int = 1'b1;
        end
        else begin : gen_reset_rx_enabled_tie_low
          assign gtwiz_reset_rx_enabled_tie_int = 1'b0;
        end

        if (C_TX_PLL_TYPE == C_RX_PLL_TYPE)
        begin : gen_reset_shared_pll_tie_high
          assign gtwiz_reset_shared_pll_tie_int = 1'b1;
        end
        else begin : gen_reset_shared_pll_tie_low
          assign gtwiz_reset_shared_pll_tie_int = 1'b0;
        end

        // Instantiate the single reset controller
        gtwizard_ultrascale_v1_6_2_gtwiz_reset #(
          .P_FREERUN_FREQUENCY       (C_FREERUN_FREQUENCY),
          .P_USE_CPLL_CAL            (P_USE_CPLL_CAL),
          .P_TX_PLL_TYPE             (C_TX_PLL_TYPE),
          .P_RX_PLL_TYPE             (C_RX_PLL_TYPE),
          .P_RX_LINE_RATE            (C_RX_LINE_RATE),
          .P_CDR_TIMEOUT_FREERUN_CYC (P_CDR_TIMEOUT_FREERUN_CYC)
        ) gtwiz_reset_inst (
          .gtwiz_reset_clk_freerun_in         (gtwiz_reset_clk_freerun_in),
          .gtwiz_reset_all_in                 (gtwiz_reset_all_in),
          .gtwiz_reset_tx_pll_and_datapath_in (gtwiz_reset_tx_pll_and_datapath_in),
          .gtwiz_reset_tx_datapath_in         (gtwiz_reset_tx_datapath_in),
          .gtwiz_reset_rx_pll_and_datapath_in (gtwiz_reset_rx_pll_and_datapath_in),
          .gtwiz_reset_rx_datapath_in         (gtwiz_reset_rx_datapath_in),
          .gtwiz_reset_rx_cdr_stable_out      (gtwiz_reset_rx_cdr_stable_out),
          .gtwiz_reset_tx_done_out            (gtwiz_reset_tx_done_out),
          .gtwiz_reset_rx_done_out            (gtwiz_reset_rx_done_out),
          .gtwiz_reset_userclk_tx_active_in   (gtwiz_reset_userclk_tx_active_int),
          .gtwiz_reset_userclk_rx_active_in   (gtwiz_reset_userclk_rx_active_int),
          .gtpowergood_in                     (gtwiz_reset_gtpowergood_int),
          .txusrclk2_in                       (txusrclk2_int[P_TX_MASTER_CH_PACKED_IDX]),
          .plllock_tx_in                      (gtwiz_reset_plllock_tx_int),
          .txresetdone_in                     (gtwiz_reset_txresetdone_int),
          .rxusrclk2_in                       (rxusrclk2_int[P_RX_MASTER_CH_PACKED_IDX]),
          .plllock_rx_in                      (gtwiz_reset_plllock_rx_int),
          .rxcdrlock_in                       (gtwiz_reset_rxcdrlock_int),
          .rxresetdone_in                     (gtwiz_reset_rxresetdone_int),
          .pllreset_tx_out                    (gtwiz_reset_pllreset_tx_int),
          .txprogdivreset_out                 (gtwiz_reset_txprogdivreset_int),
          .gttxreset_out                      (gtwiz_reset_gttxreset_int),
          .txuserrdy_out                      (gtwiz_reset_txuserrdy_int),
          .pllreset_rx_out                    (gtwiz_reset_pllreset_rx_int),
          .rxprogdivreset_out                 (gtwiz_reset_rxprogdivreset_int),
          .gtrxreset_out                      (gtwiz_reset_gtrxreset_int),
          .rxuserrdy_out                      (gtwiz_reset_rxuserrdy_int),
          .tx_enabled_tie_in                  (gtwiz_reset_tx_enabled_tie_int),
          .rx_enabled_tie_in                  (gtwiz_reset_rx_enabled_tie_int),
          .shared_pll_tie_in                  (gtwiz_reset_shared_pll_tie_int)
        );

        // Drive the internal PLL reset inputs with the appropriate PLL reset signals produced by the reset controller.
        // The single reset controller instance generates independent transmit PLL reset and receive PLL reset outputs,
        // which are used across all such PLLs in the core.
        if ((C_GT_REV == 11) || (C_GT_REV == 12) || (C_GT_REV == 14) || (C_GT_REV == 16)) begin : gen_rst_assign_es
          case ({C_TX_PLL_TYPE, C_RX_PLL_TYPE})
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}: begin : gen_txqpll0_rxqpll0
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int || gtwiz_reset_pllreset_rx_int}};
              assign qpll1reset_int = qpll1reset_in;
              assign cpllreset_int  = cpllreset_in;
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}: begin : gen_txqpll0_rxqpll1
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int}};
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_rx_int}};
              assign cpllreset_int  = cpllreset_in;
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:  begin : gen_txqpll0_rxcpll
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int}};
              assign qpll1reset_int = qpll1reset_in;
              assign cpllreset_int  = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_rx_int}};
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}: begin : gen_txqpll1_rxqpll0
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_rx_int}};
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int}};
              assign cpllreset_int  = cpllreset_in;
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}: begin : gen_txqpll1_rxqpll1
              assign qpll0reset_int = qpll0reset_in;
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int || gtwiz_reset_pllreset_rx_int}};
              assign cpllreset_int  = cpllreset_in;
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:  begin : gen_txqpll1_rxcpll
              assign qpll0reset_int = qpll0reset_in;
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int}};
              assign cpllreset_int  = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_rx_int}};
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}: begin : gen_txcpll_rxqpll0
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_rx_int}};
              assign qpll1reset_int = qpll1reset_in;
              assign cpllreset_int  = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_tx_int}};
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}: begin : gen_txcpll_rxqpll1
              assign qpll0reset_int = qpll0reset_in;
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_rx_int}};
              assign cpllreset_int  = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_tx_int}};
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:  begin : gen_txcpll_rxcpll
              assign qpll0reset_int = qpll0reset_in;
              assign qpll1reset_int = qpll1reset_in;
              assign cpllreset_int  = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_tx_int || gtwiz_reset_pllreset_rx_int}};
            end
          endcase
          assign cpllpd_int  = cpllpd_in;
        end
        else begin : gen_rst_assign_prod
          case ({C_TX_PLL_TYPE, C_RX_PLL_TYPE})
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}: begin : gen_txqpll0_rxqpll0
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int || gtwiz_reset_pllreset_rx_int}};
              assign qpll1reset_int = qpll1reset_in;
              assign cpllpd_int     = cpllpd_in;
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}: begin : gen_txqpll0_rxqpll1
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int}};
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_rx_int}};
              assign cpllpd_int     = cpllpd_in;
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:  begin : gen_txqpll0_rxcpll
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int}};
              assign qpll1reset_int = qpll1reset_in;
              assign cpllpd_int     = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_rx_int}};
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}: begin : gen_txqpll1_rxqpll0
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_rx_int}};
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int}};
              assign cpllpd_int     = cpllpd_in;
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}: begin : gen_txqpll1_rxqpll1
              assign qpll0reset_int = qpll0reset_in;
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int || gtwiz_reset_pllreset_rx_int}};
              assign cpllpd_int     = cpllpd_in;
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:  begin : gen_txqpll1_rxcpll
              assign qpll0reset_int = qpll0reset_in;
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_tx_int}};
              assign cpllpd_int     = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_rx_int}};
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}: begin : gen_txcpll_rxqpll0
              assign qpll0reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_rx_int}};
              assign qpll1reset_int = qpll1reset_in;
              assign cpllpd_int     = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_tx_int}};
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}: begin : gen_txcpll_rxqpll1
              assign qpll0reset_int = qpll0reset_in;
              assign qpll1reset_int = {`ice_gth_t_gtwizard_gthe3_SF_CM{gtwiz_reset_pllreset_rx_int}};
              assign cpllpd_int     = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_tx_int}};
            end

            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:  begin : gen_txcpll_rxcpll
              assign qpll0reset_int = qpll0reset_in;
              assign qpll1reset_int = qpll1reset_in;
              assign cpllpd_int     = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_pllreset_tx_int || gtwiz_reset_pllreset_rx_int}};
            end
          endcase
          assign cpllreset_int  = cpllreset_in;
        end

        // Fan out appropriate reset controller outputs to all transceiver channels
        assign txprogdivreset_int  = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_txprogdivreset_int}};
        assign gttxreset_int       = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_gttxreset_int}};
        assign txuserrdy_int       = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_txuserrdy_int}};
        assign rxprogdivreset_int  = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_rxprogdivreset_int}};
        assign gtrxreset_int       = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_gtrxreset_int}};
        assign rxuserrdy_int       = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_reset_rxuserrdy_int}};

      end

      // Generate one module instance per channel, each of which is used to control only that channel and its PLL(s)
      else if (C_RESET_CONTROLLER_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__PER_CHANNEL)
      begin : gen_per_channel_instance

        // Depending on the number of user clocking network helper blocks, either use the single user clock active
        // indicator or per-channel user clock active indicators as the user clock active indicators for use in the
        // per-channel instances of this block
        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] gtwiz_reset_userclk_tx_active_int;
        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] gtwiz_reset_userclk_rx_active_int;

        if (C_TX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_TX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE)
        begin : gen_txuserclkactive_single_instance
          assign gtwiz_reset_userclk_tx_active_int = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_userclk_tx_active_out}};
        end
        else begin : gen_txuserclkactive_per_channel_instance
          assign gtwiz_reset_userclk_tx_active_int = gtwiz_userclk_tx_active_out;
        end
        if (C_RX_USER_CLOCKING_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RX_USER_CLOCKING_INSTANCE_CTRL__SINGLE_INSTANCE)
        begin : gen_rxuserclkactive_single_instance
          assign gtwiz_reset_userclk_rx_active_int = {`ice_gth_t_gtwizard_gthe3_N_CH{gtwiz_userclk_rx_active_out}};
        end
        else begin : gen_rxuserclkactive_per_channel_instance
          assign gtwiz_reset_userclk_rx_active_int = gtwiz_userclk_rx_active_out;
        end

        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] gtwiz_reset_plllock_tx_int;
        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] gtwiz_reset_plllock_rx_int;
        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] gtwiz_reset_pllreset_tx_int;
        wire [`ice_gth_t_gtwizard_gthe3_N_CH-1:0] gtwiz_reset_pllreset_rx_int;

        // Assign tie-off values for use in reset controller state machine traversal
        wire gtwiz_reset_tx_enabled_tie_int;
        wire gtwiz_reset_rx_enabled_tie_int;
        wire gtwiz_reset_shared_pll_tie_int;

        if (C_TX_ENABLE == `ice_gth_t_gtwizard_gthe3_TX_ENABLE__ENABLED)
        begin : gen_reset_tx_enabled_tie_high
          assign gtwiz_reset_tx_enabled_tie_int = 1'b1;
        end
        else begin : gen_reset_tx_enabled_tie_low
          assign gtwiz_reset_tx_enabled_tie_int = 1'b0;
        end

        if (C_RX_ENABLE == `ice_gth_t_gtwizard_gthe3_RX_ENABLE__ENABLED)
        begin : gen_reset_rx_enabled_tie_high
          assign gtwiz_reset_rx_enabled_tie_int = 1'b1;
        end
        else begin : gen_reset_rx_enabled_tie_low
          assign gtwiz_reset_rx_enabled_tie_int = 1'b0;
        end

        if (C_TX_PLL_TYPE == C_RX_PLL_TYPE)
        begin : gen_reset_shared_pll_tie_high
          assign gtwiz_reset_shared_pll_tie_int = 1'b1;
        end
        else begin : gen_reset_shared_pll_tie_low
          assign gtwiz_reset_shared_pll_tie_int = 1'b0;
        end

        genvar gi_hb_rst_ch;
        for (gi_hb_rst_ch = 0; gi_hb_rst_ch < `ice_gth_t_gtwizard_gthe3_N_CH; gi_hb_rst_ch = gi_hb_rst_ch + 1) begin : gen_channel_container

          // For each data direction of each channel, select the appropriate PLL lock signal such that the reset
          // controller can sense when that channel's PLL is locked (or PLLs are locked)
          case (C_TX_PLL_TYPE)
            `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0:
              assign gtwiz_reset_plllock_tx_int[gi_hb_rst_ch] = qpll0lock_int[f_idx_cm(gi_hb_rst_ch)];
            `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1:
              assign gtwiz_reset_plllock_tx_int[gi_hb_rst_ch] = qpll1lock_int[f_idx_cm(gi_hb_rst_ch)];
            `ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL:
              assign gtwiz_reset_plllock_tx_int[gi_hb_rst_ch] = cplllock_int[gi_hb_rst_ch];
          endcase
          case (C_RX_PLL_TYPE)
            `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0:
              assign gtwiz_reset_plllock_rx_int[gi_hb_rst_ch] = qpll0lock_int[f_idx_cm(gi_hb_rst_ch)];
            `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1:
              assign gtwiz_reset_plllock_rx_int[gi_hb_rst_ch] = qpll1lock_int[f_idx_cm(gi_hb_rst_ch)];
            `ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL:
              assign gtwiz_reset_plllock_rx_int[gi_hb_rst_ch] = cplllock_int[gi_hb_rst_ch];
          endcase

          // Instantiate a reset controller per channel
          gtwizard_ultrascale_v1_6_2_gtwiz_reset #(
            .P_FREERUN_FREQUENCY       (C_FREERUN_FREQUENCY),
            .P_USE_CPLL_CAL            (P_USE_CPLL_CAL),
            .P_TX_PLL_TYPE             (C_TX_PLL_TYPE),
            .P_RX_PLL_TYPE             (C_RX_PLL_TYPE),
            .P_RX_LINE_RATE            (C_RX_LINE_RATE),
            .P_CDR_TIMEOUT_FREERUN_CYC (P_CDR_TIMEOUT_FREERUN_CYC)
          ) gtwiz_reset_inst (
            .gtwiz_reset_clk_freerun_in         (gtwiz_reset_clk_freerun_in         [gi_hb_rst_ch]),
            .gtwiz_reset_all_in                 (gtwiz_reset_all_in                 [gi_hb_rst_ch]),
            .gtwiz_reset_tx_pll_and_datapath_in (gtwiz_reset_tx_pll_and_datapath_in [gi_hb_rst_ch]),
            .gtwiz_reset_tx_datapath_in         (gtwiz_reset_tx_datapath_in         [gi_hb_rst_ch]),
            .gtwiz_reset_rx_pll_and_datapath_in (gtwiz_reset_rx_pll_and_datapath_in [gi_hb_rst_ch]),
            .gtwiz_reset_rx_datapath_in         (gtwiz_reset_rx_datapath_in         [gi_hb_rst_ch]),
            .gtwiz_reset_rx_cdr_stable_out      (gtwiz_reset_rx_cdr_stable_out      [gi_hb_rst_ch]),
            .gtwiz_reset_tx_done_out            (gtwiz_reset_tx_done_out            [gi_hb_rst_ch]),
            .gtwiz_reset_rx_done_out            (gtwiz_reset_rx_done_out            [gi_hb_rst_ch]),
            .gtwiz_reset_userclk_tx_active_in   (gtwiz_reset_userclk_tx_active_int  [gi_hb_rst_ch]),
            .gtwiz_reset_userclk_rx_active_in   (gtwiz_reset_userclk_rx_active_int  [gi_hb_rst_ch]),
            .gtpowergood_in                     (gtpowergood_int                    [gi_hb_rst_ch]),
            .txusrclk2_in                       (txusrclk2_int                      [gi_hb_rst_ch]),
            .plllock_tx_in                      (gtwiz_reset_plllock_tx_int         [gi_hb_rst_ch]),
            .txresetdone_in                     (txresetdone_int                    [gi_hb_rst_ch]),
            .rxusrclk2_in                       (rxusrclk2_int                      [gi_hb_rst_ch]),
            .plllock_rx_in                      (gtwiz_reset_plllock_rx_int         [gi_hb_rst_ch]),
            .rxcdrlock_in                       (rxcdrlock_int                      [gi_hb_rst_ch]),
            .rxresetdone_in                     (rxresetdone_int                    [gi_hb_rst_ch]),
            .pllreset_tx_out                    (gtwiz_reset_pllreset_tx_int        [gi_hb_rst_ch]),
            .txprogdivreset_out                 (txprogdivreset_int                 [gi_hb_rst_ch]),
            .gttxreset_out                      (gttxreset_int                      [gi_hb_rst_ch]),
            .txuserrdy_out                      (txuserrdy_int                      [gi_hb_rst_ch]),
            .pllreset_rx_out                    (gtwiz_reset_pllreset_rx_int        [gi_hb_rst_ch]),
            .rxprogdivreset_out                 (rxprogdivreset_int                 [gi_hb_rst_ch]),
            .gtrxreset_out                      (gtrxreset_int                      [gi_hb_rst_ch]),
            .rxuserrdy_out                      (rxuserrdy_int                      [gi_hb_rst_ch]),
            .tx_enabled_tie_in                  (gtwiz_reset_tx_enabled_tie_int),
            .rx_enabled_tie_in                  (gtwiz_reset_rx_enabled_tie_int),
            .shared_pll_tie_in                  (gtwiz_reset_shared_pll_tie_int)
          );

          // If the core is configured to use the CPLL for either the transmit or the receive data direction, drive the
          // CPLL reset input with the appropriate PLL reset signals from by the reset controller, for each channel.
          if ((C_GT_REV == 11) || (C_GT_REV == 12) || (C_GT_REV == 14) || (C_GT_REV == 16)) begin : gen_cpllreset_assign_es
            case ({C_TX_PLL_TYPE, C_RX_PLL_TYPE})
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}:
                assign cpllreset_int[gi_hb_rst_ch] = cpllreset_in[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}:
                assign cpllreset_int[gi_hb_rst_ch] = cpllreset_in[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:
                assign cpllreset_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_rx_int[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}:
                assign cpllreset_int[gi_hb_rst_ch] = cpllreset_in[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}:
                assign cpllreset_int[gi_hb_rst_ch] = cpllreset_in[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:
                assign cpllreset_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_rx_int[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}:
                assign cpllreset_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_tx_int[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}:
                assign cpllreset_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_tx_int[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:
                assign cpllreset_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_tx_int[gi_hb_rst_ch] ||
                                                     gtwiz_reset_pllreset_rx_int[gi_hb_rst_ch];
            endcase
            assign cpllpd_int[gi_hb_rst_ch] = cpllpd_in[gi_hb_rst_ch];
          end
          else begin : gen_cpllreset_assign_prod
            case ({C_TX_PLL_TYPE, C_RX_PLL_TYPE})
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}:
                assign cpllpd_int[gi_hb_rst_ch] = cpllpd_in[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}:
                assign cpllpd_int[gi_hb_rst_ch] = cpllpd_in[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:
                assign cpllpd_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_rx_int[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}:
                assign cpllpd_int[gi_hb_rst_ch] = cpllpd_in[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}:
                assign cpllpd_int[gi_hb_rst_ch] = cpllpd_in[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:
                assign cpllpd_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_rx_int[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}:
                assign cpllpd_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_tx_int[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}:
                assign cpllpd_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_tx_int[gi_hb_rst_ch];
              {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:
                assign cpllpd_int[gi_hb_rst_ch] = gtwiz_reset_pllreset_tx_int[gi_hb_rst_ch] ||
                                                  gtwiz_reset_pllreset_rx_int[gi_hb_rst_ch];
            endcase
            assign cpllreset_int[gi_hb_rst_ch] = cpllreset_in[gi_hb_rst_ch];
          end
        end

        // If the core is configured to use a QPLL for either the transmit or the receive data direction, drive the
        // QPLL reset inputs with the appropriate PLL reset signals from by the reset controller, for each channel.
        // Because a single QPLL may be shared by multiple channels, and therefore multiple reset controllers in this
        // configuration, combine the relevant reset controllers' signals to drive each QPLL reset signal.
        genvar gi_hb_rst_cm;
        for (gi_hb_rst_cm = 0; gi_hb_rst_cm < `ice_gth_t_gtwizard_gthe3_SF_CM; gi_hb_rst_cm = gi_hb_rst_cm + 1) begin : gen_common_container

          case ({C_TX_PLL_TYPE, C_RX_PLL_TYPE})
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}: begin : gen_txqpll0_rxqpll0
              assign qpll0reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_tx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]) ||
                     (|gtwiz_reset_pllreset_rx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
              assign qpll1reset_int[gi_hb_rst_cm] = qpll1reset_in[gi_hb_rst_cm];
            end
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}: begin : gen_txqpll0_rxqpll1
              assign qpll0reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_tx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
              assign qpll1reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_rx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
            end
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL0, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:  begin : gen_txqpll0_rxcpll
              assign qpll0reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_tx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
              assign qpll1reset_int[gi_hb_rst_cm] = qpll1reset_in[gi_hb_rst_cm];
            end
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}: begin : gen_txqpll1_rxqpll0
              assign qpll0reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_rx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
              assign qpll1reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_tx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
            end
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}: begin : gen_txqpll1_rxqpll1
              assign qpll0reset_int[gi_hb_rst_cm] = qpll0reset_in[gi_hb_rst_cm];
              assign qpll1reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_tx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]) ||
                     (|gtwiz_reset_pllreset_rx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
            end
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__QPLL1, 32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:  begin : gen_txqpll1_rxcpll
              assign qpll0reset_int[gi_hb_rst_cm] = qpll0reset_in[gi_hb_rst_cm];
              assign qpll1reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_tx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
            end
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL0}: begin : gen_txcpll_rxqpll0
              assign qpll0reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_rx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
              assign qpll1reset_int[gi_hb_rst_cm] = qpll1reset_in[gi_hb_rst_cm];
            end
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__QPLL1}: begin : gen_txcpll_rxqpll1
              assign qpll0reset_int[gi_hb_rst_cm] = qpll0reset_in[gi_hb_rst_cm];
              assign qpll1reset_int[gi_hb_rst_cm] =
                     (|gtwiz_reset_pllreset_rx_int[f_idx_ch_ub(gi_hb_rst_cm):f_idx_ch_lb(gi_hb_rst_cm)]);
            end
            {32'd`ice_gth_t_gtwizard_gthe3_TX_PLL_TYPE__CPLL,  32'd`ice_gth_t_gtwizard_gthe3_RX_PLL_TYPE__CPLL}:  begin : gen_txcpll_rxcpll
              assign qpll0reset_int[gi_hb_rst_cm] = qpll0reset_in[gi_hb_rst_cm];
              assign qpll1reset_int[gi_hb_rst_cm] = qpll1reset_in[gi_hb_rst_cm];
            end
          endcase

        end

      end
    end

    // Do not include the helper block within the core
    else begin : gen_reset_controller_external

      if (C_RESET_CONTROLLER_INSTANCE_CTRL == `ice_gth_t_gtwizard_gthe3_RESET_CONTROLLER_INSTANCE_CTRL__SINGLE_INSTANCE)
      begin : gen_single_instance
        assign gtwiz_reset_rx_cdr_stable_out = 1'b0;
      end
      else begin : gen_per_channel_instance
        assign gtwiz_reset_rx_cdr_stable_out = {(C_RESET_CONTROLLER_INSTANCE_CTRL*`ice_gth_t_gtwizard_gthe3_N_CH){1'b0}};
      end

      assign gtwiz_reset_tx_done_out = gtwiz_reset_tx_done_in;
      assign gtwiz_reset_rx_done_out = gtwiz_reset_rx_done_in;
      assign qpll0reset_int          = qpll0reset_in;
      assign qpll1reset_int          = qpll1reset_in;
      assign cpllreset_int           = cpllreset_in;
      assign cpllpd_int              = cpllpd_in;
      assign txprogdivreset_int      = txprogdivreset_in;
      assign gttxreset_int           = gttxreset_in;
      assign txuserrdy_int           = txuserrdy_in;
      assign rxprogdivreset_int      = rxprogdivreset_in;
      assign gtrxreset_int           = gtrxreset_in;
      assign rxuserrdy_int           = rxuserrdy_in;

    end

    // Assign QPLL reset outputs to corresponding reset controller signals for use in the case where the common block
    // is not within the core
    assign gtwiz_reset_qpll0reset_out = qpll0reset_int;
    assign gtwiz_reset_qpll1reset_out = qpll1reset_int;


    // ----------------------------------------------------------------------------------------------------------------
    // Transmitter user data width sizing helper block
    // ----------------------------------------------------------------------------------------------------------------

    // Include the helper block within the core
    if ((C_TX_ENABLE == `ice_gth_t_gtwizard_gthe3_TX_ENABLE__ENABLED) &&
        (C_LOCATE_USER_DATA_WIDTH_SIZING == `ice_gth_t_gtwizard_gthe3_LOCATE_USER_DATA_WIDTH_SIZING__CORE))
        begin : gen_tx_userdata_internal

      // Declare vectors for the helper block to drive transceiver-facing TXDATA, TXCTRL0, and TXCTRL1 ports
      wire [(C_TOTAL_NUM_CHANNELS*128)-1:0] gtwiz_userdata_tx_txdata_int;
      wire [(C_TOTAL_NUM_CHANNELS* 16)-1:0] gtwiz_userdata_tx_txctrl0_int;
      wire [(C_TOTAL_NUM_CHANNELS* 16)-1:0] gtwiz_userdata_tx_txctrl1_int;

      gtwizard_ultrascale_v1_6_2_gtwiz_userdata_tx #(
        .P_TX_USER_DATA_WIDTH       (C_TX_USER_DATA_WIDTH),
        .P_TX_DATA_ENCODING         (C_TX_DATA_ENCODING),
        .P_TOTAL_NUMBER_OF_CHANNELS (C_TOTAL_NUM_CHANNELS)
      ) gtwiz_userdata_tx_inst (
        .gtwiz_userdata_tx_in (gtwiz_userdata_tx_in),
        .txdata_out           (gtwiz_userdata_tx_txdata_int),
        .txctrl0_out          (gtwiz_userdata_tx_txctrl0_int),
        .txctrl1_out          (gtwiz_userdata_tx_txctrl1_int)
      );

      // The txdata_int vector is always driven by the helper block when it is present
      assign txdata_int = gtwiz_userdata_tx_txdata_int;

      // The txctrl0_int and txctrl1_int vectors are driven by the helper block only when transmitter data decoding is
      // raw and user data width is modulus 10; otherwise, they are driven by corresponding inputs
      if ((C_TX_DATA_ENCODING == `ice_gth_t_gtwizard_gthe3_TX_DATA_ENCODING__RAW) &&
          (C_TX_USER_DATA_WIDTH % 10 == 0))
          begin : gen_tx_ctrl_internal
        assign txctrl0_int = gtwiz_userdata_tx_txctrl0_int;
        assign txctrl1_int = gtwiz_userdata_tx_txctrl1_int;
      end
      else begin : gen_tx_ctrl_external
        assign txctrl0_int = txctrl0_in;
        assign txctrl1_int = txctrl1_in;
      end

    end

    // Do not include the helper block within the core
    else begin : gen_no_tx_userdata_internal
      assign txdata_int  = txdata_in;
      assign txctrl0_int = txctrl0_in;
      assign txctrl1_int = txctrl1_in;
    end


    // ----------------------------------------------------------------------------------------------------------------
    // Receiver user data width sizing helper block
    // ----------------------------------------------------------------------------------------------------------------

    // Include the helper block within the core
    if ((C_RX_ENABLE == `ice_gth_t_gtwizard_gthe3_RX_ENABLE__ENABLED) &&
        (C_LOCATE_USER_DATA_WIDTH_SIZING == `ice_gth_t_gtwizard_gthe3_LOCATE_USER_DATA_WIDTH_SIZING__CORE))
        begin : gen_rx_userdata_internal

      gtwizard_ultrascale_v1_6_2_gtwiz_userdata_rx #(
        .P_RX_USER_DATA_WIDTH       (C_RX_USER_DATA_WIDTH),
        .P_RX_DATA_DECODING         (C_RX_DATA_DECODING),
        .P_TOTAL_NUMBER_OF_CHANNELS (C_TOTAL_NUM_CHANNELS)
      ) gtwiz_userdata_rx_inst (
        .rxdata_in             (rxdata_int),
        .rxctrl0_in            (rxctrl0_int),
        .rxctrl1_in            (rxctrl1_int),
        .gtwiz_userdata_rx_out (gtwiz_userdata_rx_out)
      );
    end

    // Do not include the helper block within the core
    else begin : gen_no_rx_userdata_internal
      assign gtwiz_userdata_rx_out = {C_TOTAL_NUM_CHANNELS*C_RX_USER_DATA_WIDTH{1'b0}};
    end

  end
  endgenerate


endmodule

module ice_gth_t_gthe3_channel_wrapper #(
  parameter MASTER_EN                                     = 0,
  parameter NUM_CHANNELS                                  = 1
)(
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CFGRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CLKRSVD0,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CLKRSVD1,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CPLLLOCKDETCLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CPLLLOCKEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CPLLPD,
  input                  [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_CPLLREFCLKSEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CPLLRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_DMONFIFORESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_DMONITORCLK,
  input                  [(NUM_CHANNELS*9)-1:0] GTHE3_CHANNEL_DRPADDR,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_DRPCLK,
  input                 [(NUM_CHANNELS*16)-1:0] GTHE3_CHANNEL_DRPDI,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_DRPEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_DRPWE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EVODDPHICALDONE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EVODDPHICALSTART,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EVODDPHIDRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EVODDPHIDWREN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EVODDPHIXRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EVODDPHIXWREN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EYESCANMODE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EYESCANRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EYESCANTRIGGER,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTGREFCLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTHRXN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTHRXP,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTNORTHREFCLK0,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTNORTHREFCLK1,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTREFCLK0,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTREFCLK1,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTRESETSEL,
  input                 [(NUM_CHANNELS*16)-1:0] GTHE3_CHANNEL_GTRSVD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTRXRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTSOUTHREFCLK0,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTSOUTHREFCLK1,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTTXRESET,
  input                  [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_LOOPBACK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_LPBKRXTXSEREN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_LPBKTXRXSEREN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIERSTIDLE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIERSTTXSYNCSTART,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIEUSERRATEDONE,
  input                 [(NUM_CHANNELS*16)-1:0] GTHE3_CHANNEL_PCSRSVDIN,
  input                  [(NUM_CHANNELS*5)-1:0] GTHE3_CHANNEL_PCSRSVDIN2,
  input                  [(NUM_CHANNELS*5)-1:0] GTHE3_CHANNEL_PMARSVDIN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_QPLL0CLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_QPLL0REFCLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_QPLL1CLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_QPLL1REFCLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RESETOVRD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RSTCLKENTX,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RX8B10BEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXBUFRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCDRFREQRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCDRHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCDROVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCDRRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCDRRESETRSV,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCHBONDEN,
  input                  [(NUM_CHANNELS*5)-1:0] GTHE3_CHANNEL_RXCHBONDI,
  input                  [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_RXCHBONDLEVEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCHBONDMASTER,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCHBONDSLAVE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCOMMADETEN,
  input                  [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXDFEAGCCTRL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFEAGCHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFEAGCOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFELFHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFELFOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFELPMRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP10HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP10OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP11HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP11OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP12HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP12OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP13HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP13OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP14HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP14OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP15HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP15OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP2HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP2OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP3HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP3OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP4HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP4OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP5HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP5OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP6HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP6OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP7HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP7OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP8HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP8OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP9HOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFETAP9OVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFEUTHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFEUTOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFEVPHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFEVPOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFEVSEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDFEXYDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDLYBYPASS,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDLYEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDLYOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDLYSRESET,
  input                  [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXELECIDLEMODE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXGEARBOXSLIP,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLATCLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLPMEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLPMGCHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLPMGCOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLPMHFHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLPMHFOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLPMLFHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLPMLFKLOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLPMOSHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXLPMOSOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXMCOMMAALIGNEN,
  input                  [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXMONITORSEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOOBRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSCALRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSHOLD,
  input                  [(NUM_CHANNELS*4)-1:0] GTHE3_CHANNEL_RXOSINTCFG,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSINTEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSINTHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSINTOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSINTSTROBE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSINTTESTOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSOVRDEN,
  input                  [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_RXOUTCLKSEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPCOMMAALIGNEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPCSRESET,
  input                  [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXPD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPHALIGN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPHALIGNEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPHDLYPD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPHDLYRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPHOVRDEN,
  input                  [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXPLLCLKSEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPMARESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPOLARITY,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPRBSCNTRESET,
  input                  [(NUM_CHANNELS*4)-1:0] GTHE3_CHANNEL_RXPRBSSEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPROGDIVRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXQPIEN,
  input                  [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_RXRATE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXRATEMODE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSLIDE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSLIPOUTCLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSLIPPMA,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSYNCALLIN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSYNCIN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSYNCMODE,
  input                  [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXSYSCLKSEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXUSERRDY,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXUSRCLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXUSRCLK2,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_SIGVALIDCLK,
  input                 [(NUM_CHANNELS*20)-1:0] GTHE3_CHANNEL_TSTIN,
  input                  [(NUM_CHANNELS*8)-1:0] GTHE3_CHANNEL_TX8B10BBYPASS,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TX8B10BEN,
  input                  [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_TXBUFDIFFCTRL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXCOMINIT,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXCOMSAS,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXCOMWAKE,
  input                 [(NUM_CHANNELS*16)-1:0] GTHE3_CHANNEL_TXCTRL0,
  input                 [(NUM_CHANNELS*16)-1:0] GTHE3_CHANNEL_TXCTRL1,
  input                  [(NUM_CHANNELS*8)-1:0] GTHE3_CHANNEL_TXCTRL2,
  input                [(NUM_CHANNELS*128)-1:0] GTHE3_CHANNEL_TXDATA,
  input                  [(NUM_CHANNELS*8)-1:0] GTHE3_CHANNEL_TXDATAEXTENDRSVD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDEEMPH,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDETECTRX,
  input                  [(NUM_CHANNELS*4)-1:0] GTHE3_CHANNEL_TXDIFFCTRL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDIFFPD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDLYBYPASS,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDLYEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDLYHOLD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDLYOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDLYSRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDLYUPDOWN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXELECIDLE,
  input                  [(NUM_CHANNELS*6)-1:0] GTHE3_CHANNEL_TXHEADER,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXINHIBIT,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXLATCLK,
  input                  [(NUM_CHANNELS*7)-1:0] GTHE3_CHANNEL_TXMAINCURSOR,
  input                  [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_TXMARGIN,
  input                  [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_TXOUTCLKSEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPCSRESET,
  input                  [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_TXPD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPDELECIDLEMODE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPHALIGN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPHALIGNEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPHDLYPD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPHDLYRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPHDLYTSTCLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPHINIT,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPHOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPIPPMEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPIPPMOVRDEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPIPPMPD,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPIPPMSEL,
  input                  [(NUM_CHANNELS*5)-1:0] GTHE3_CHANNEL_TXPIPPMSTEPSIZE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPISOPD,
  input                  [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_TXPLLCLKSEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPMARESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPOLARITY,
  input                  [(NUM_CHANNELS*5)-1:0] GTHE3_CHANNEL_TXPOSTCURSOR,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPOSTCURSORINV,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPRBSFORCEERR,
  input                  [(NUM_CHANNELS*4)-1:0] GTHE3_CHANNEL_TXPRBSSEL,
  input                  [(NUM_CHANNELS*5)-1:0] GTHE3_CHANNEL_TXPRECURSOR,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPRECURSORINV,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPROGDIVRESET,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXQPIBIASEN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXQPISTRONGPDOWN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXQPIWEAKPUP,
  input                  [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_TXRATE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXRATEMODE,
  input                  [(NUM_CHANNELS*7)-1:0] GTHE3_CHANNEL_TXSEQUENCE,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXSWING,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXSYNCALLIN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXSYNCIN,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXSYNCMODE,
  input                  [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_TXSYSCLKSEL,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXUSERRDY,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXUSRCLK,
  input                  [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXUSRCLK2,

  output                 [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_BUFGTCE,
  output                 [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_BUFGTCEMASK,
  output                 [(NUM_CHANNELS*9)-1:0] GTHE3_CHANNEL_BUFGTDIV,
  output                 [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_BUFGTRESET,
  output                 [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_BUFGTRSTMASK,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CPLLFBCLKLOST,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CPLLLOCK,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_CPLLREFCLKLOST,
  output                [(NUM_CHANNELS*17)-1:0] GTHE3_CHANNEL_DMONITOROUT,
  output                [(NUM_CHANNELS*16)-1:0] GTHE3_CHANNEL_DRPDO,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_DRPRDY,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_EYESCANDATAERROR,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTHTXN,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTHTXP,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTPOWERGOOD,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_GTREFCLKMONITOR,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIERATEGEN3,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIERATEIDLE,
  output                 [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_PCIERATEQPLLPD,
  output                 [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_PCIERATEQPLLRESET,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIESYNCTXSYNCDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIEUSERGEN3RDY,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIEUSERPHYSTATUSRST,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PCIEUSERRATESTART,
  output                [(NUM_CHANNELS*12)-1:0] GTHE3_CHANNEL_PCSRSVDOUT,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_PHYSTATUS,
  output                 [(NUM_CHANNELS*8)-1:0] GTHE3_CHANNEL_PINRSRVDAS,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RESETEXCEPTION,
  output                 [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_RXBUFSTATUS,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXBYTEISALIGNED,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXBYTEREALIGN,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCDRLOCK,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCDRPHDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCHANBONDSEQ,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCHANISALIGNED,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCHANREALIGN,
  output                 [(NUM_CHANNELS*5)-1:0] GTHE3_CHANNEL_RXCHBONDO,
  output                 [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXCLKCORCNT,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCOMINITDET,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCOMMADET,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCOMSASDET,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXCOMWAKEDET,
  output                [(NUM_CHANNELS*16)-1:0] GTHE3_CHANNEL_RXCTRL0,
  output                [(NUM_CHANNELS*16)-1:0] GTHE3_CHANNEL_RXCTRL1,
  output                 [(NUM_CHANNELS*8)-1:0] GTHE3_CHANNEL_RXCTRL2,
  output                 [(NUM_CHANNELS*8)-1:0] GTHE3_CHANNEL_RXCTRL3,
  output               [(NUM_CHANNELS*128)-1:0] GTHE3_CHANNEL_RXDATA,
  output                 [(NUM_CHANNELS*8)-1:0] GTHE3_CHANNEL_RXDATAEXTENDRSVD,
  output                 [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXDATAVALID,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXDLYSRESETDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXELECIDLE,
  output                 [(NUM_CHANNELS*6)-1:0] GTHE3_CHANNEL_RXHEADER,
  output                 [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXHEADERVALID,
  output                 [(NUM_CHANNELS*7)-1:0] GTHE3_CHANNEL_RXMONITOROUT,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSINTDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSINTSTARTED,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSINTSTROBEDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOSINTSTROBESTARTED,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOUTCLK,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOUTCLKFABRIC,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXOUTCLKPCS,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPHALIGNDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPHALIGNERR,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPMARESETDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPRBSERR,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPRBSLOCKED,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXPRGDIVRESETDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXQPISENN,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXQPISENP,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXRATEDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXRECCLKOUT,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXRESETDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSLIDERDY,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSLIPDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSLIPOUTCLKRDY,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSLIPPMARDY,
  output                 [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_RXSTARTOFSEQ,
  output                 [(NUM_CHANNELS*3)-1:0] GTHE3_CHANNEL_RXSTATUS,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSYNCDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXSYNCOUT,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_RXVALID,
  output                 [(NUM_CHANNELS*2)-1:0] GTHE3_CHANNEL_TXBUFSTATUS,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXCOMFINISH,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXDLYSRESETDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXOUTCLK,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXOUTCLKFABRIC,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXOUTCLKPCS,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPHALIGNDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPHINITDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPMARESETDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXPRGDIVRESETDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXQPISENN,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXQPISENP,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXRATEDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXRESETDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXSYNCDONE,
  output                 [(NUM_CHANNELS*1)-1:0] GTHE3_CHANNEL_TXSYNCOUT
);



gtwizard_ultrascale_v1_6_2_gthe3_channel #(
  .GTHE3_CHANNEL_ACJTAG_DEBUG_MODE              (1'b0),
  .GTHE3_CHANNEL_ACJTAG_MODE                    (1'b0),
  .GTHE3_CHANNEL_ACJTAG_RESET                   (1'b0),
  .GTHE3_CHANNEL_ADAPT_CFG0                     (16'b1111100000000000),
  .GTHE3_CHANNEL_ADAPT_CFG1                     (16'b0000000000000000),
  .GTHE3_CHANNEL_ALIGN_COMMA_DOUBLE             ("FALSE"),
  .GTHE3_CHANNEL_ALIGN_COMMA_ENABLE             (10'b0000000000),
  .GTHE3_CHANNEL_ALIGN_COMMA_WORD               (1),
  .GTHE3_CHANNEL_ALIGN_MCOMMA_DET               ("FALSE"),
  .GTHE3_CHANNEL_ALIGN_MCOMMA_VALUE             (10'b1010000011),
  .GTHE3_CHANNEL_ALIGN_PCOMMA_DET               ("FALSE"),
  .GTHE3_CHANNEL_ALIGN_PCOMMA_VALUE             (10'b0101111100),
  .GTHE3_CHANNEL_A_RXOSCALRESET                 (1'b0),
  .GTHE3_CHANNEL_A_RXPROGDIVRESET               (1'b0),
  .GTHE3_CHANNEL_A_TXPROGDIVRESET               (1'b0),
  .GTHE3_CHANNEL_CBCC_DATA_SOURCE_SEL           ("ENCODED"),
  .GTHE3_CHANNEL_CDR_SWAP_MODE_EN               (1'b0),
  .GTHE3_CHANNEL_CFGRESET_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_CFGRESET_VAL                   (1'b0),
  .GTHE3_CHANNEL_CHAN_BOND_KEEP_ALIGN           ("FALSE"),
  .GTHE3_CHANNEL_CHAN_BOND_MAX_SKEW             (1),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_1_1              (10'b0000000000),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_1_2              (10'b0000000000),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_1_3              (10'b0000000000),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_1_4              (10'b0000000000),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_1_ENABLE         (4'b1111),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_2_1              (10'b0000000000),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_2_2              (10'b0000000000),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_2_3              (10'b0000000000),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_2_4              (10'b0000000000),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_2_ENABLE         (4'b1111),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_2_USE            ("FALSE"),
  .GTHE3_CHANNEL_CHAN_BOND_SEQ_LEN              (1),
  .GTHE3_CHANNEL_CLKRSVD0_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_CLKRSVD0_VAL                   (1'b0),
  .GTHE3_CHANNEL_CLKRSVD1_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_CLKRSVD1_VAL                   (1'b0),
  .GTHE3_CHANNEL_CLK_CORRECT_USE                ("FALSE"),
  .GTHE3_CHANNEL_CLK_COR_KEEP_IDLE              ("FALSE"),
  .GTHE3_CHANNEL_CLK_COR_MAX_LAT                (20),
  .GTHE3_CHANNEL_CLK_COR_MIN_LAT                (18),
  .GTHE3_CHANNEL_CLK_COR_PRECEDENCE             ("TRUE"),
  .GTHE3_CHANNEL_CLK_COR_REPEAT_WAIT            (0),
  .GTHE3_CHANNEL_CLK_COR_SEQ_1_1                (10'b0000000000),
  .GTHE3_CHANNEL_CLK_COR_SEQ_1_2                (10'b0000000000),
  .GTHE3_CHANNEL_CLK_COR_SEQ_1_3                (10'b0000000000),
  .GTHE3_CHANNEL_CLK_COR_SEQ_1_4                (10'b0000000000),
  .GTHE3_CHANNEL_CLK_COR_SEQ_1_ENABLE           (4'b1111),
  .GTHE3_CHANNEL_CLK_COR_SEQ_2_1                (10'b0000000000),
  .GTHE3_CHANNEL_CLK_COR_SEQ_2_2                (10'b0000000000),
  .GTHE3_CHANNEL_CLK_COR_SEQ_2_3                (10'b0000000000),
  .GTHE3_CHANNEL_CLK_COR_SEQ_2_4                (10'b0000000000),
  .GTHE3_CHANNEL_CLK_COR_SEQ_2_ENABLE           (4'b1111),
  .GTHE3_CHANNEL_CLK_COR_SEQ_2_USE              ("FALSE"),
  .GTHE3_CHANNEL_CLK_COR_SEQ_LEN                (1),
  .GTHE3_CHANNEL_CPLLLOCKDETCLK_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_CPLLLOCKDETCLK_VAL             (1'b0),
  .GTHE3_CHANNEL_CPLLLOCKEN_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_CPLLLOCKEN_VAL                 (1'b0),
  .GTHE3_CHANNEL_CPLLPD_TIE_EN                  (1'b0),
  .GTHE3_CHANNEL_CPLLPD_VAL                     (1'b1),
  .GTHE3_CHANNEL_CPLLREFCLKSEL_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_CPLLREFCLKSEL_VAL              (3'b001),
  .GTHE3_CHANNEL_CPLLRESET_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_CPLLRESET_VAL                  (1'b1),
  .GTHE3_CHANNEL_CPLL_CFG0                      (16'b0110011111111000),
  .GTHE3_CHANNEL_CPLL_CFG1                      (16'b1010010010101100),
  .GTHE3_CHANNEL_CPLL_CFG2                      (16'b0101000000000111),
  .GTHE3_CHANNEL_CPLL_CFG3                      (6'b000000),
  .GTHE3_CHANNEL_CPLL_FBDIV                     (2),
  .GTHE3_CHANNEL_CPLL_FBDIV_45                  (5),
  .GTHE3_CHANNEL_CPLL_INIT_CFG0                 (16'b0000001010110010),
  .GTHE3_CHANNEL_CPLL_INIT_CFG1                 (8'b00000000),
  .GTHE3_CHANNEL_CPLL_LOCK_CFG                  (16'b0000000111101000),
  .GTHE3_CHANNEL_CPLL_REFCLK_DIV                (1),
  .GTHE3_CHANNEL_DDI_CTRL                       (2'b00),
  .GTHE3_CHANNEL_DDI_REALIGN_WAIT               (15),
  .GTHE3_CHANNEL_DEC_MCOMMA_DETECT              ("FALSE"),
  .GTHE3_CHANNEL_DEC_PCOMMA_DETECT              ("FALSE"),
  .GTHE3_CHANNEL_DEC_VALID_COMMA_ONLY           ("FALSE"),
  .GTHE3_CHANNEL_DFE_D_X_REL_POS                (1'b0),
  .GTHE3_CHANNEL_DFE_VCM_COMP_EN                (1'b0),
  .GTHE3_CHANNEL_DMONFIFORESET_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_DMONFIFORESET_VAL              (1'b0),
  .GTHE3_CHANNEL_DMONITORCLK_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_DMONITORCLK_VAL                (1'b0),
  .GTHE3_CHANNEL_DMONITOR_CFG0                  (10'b0000000000),
  .GTHE3_CHANNEL_DMONITOR_CFG1                  (8'b00000000),
  .GTHE3_CHANNEL_DRPADDR_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_DRPADDR_VAL                    (9'b000000000),
  .GTHE3_CHANNEL_DRPCLK_TIE_EN                  (1'b0),
  .GTHE3_CHANNEL_DRPCLK_VAL                     (1'b0),
  .GTHE3_CHANNEL_DRPDI_TIE_EN                   (1'b0),
  .GTHE3_CHANNEL_DRPDI_VAL                      (16'b0000000000000000),
  .GTHE3_CHANNEL_DRPEN_TIE_EN                   (1'b0),
  .GTHE3_CHANNEL_DRPEN_VAL                      (1'b0),
  .GTHE3_CHANNEL_DRPWE_TIE_EN                   (1'b0),
  .GTHE3_CHANNEL_DRPWE_VAL                      (1'b0),
  .GTHE3_CHANNEL_ES_CLK_PHASE_SEL               (1'b0),
  .GTHE3_CHANNEL_ES_CONTROL                     (6'b000000),
  .GTHE3_CHANNEL_ES_ERRDET_EN                   ("FALSE"),
  .GTHE3_CHANNEL_ES_EYE_SCAN_EN                 ("FALSE"),
  .GTHE3_CHANNEL_ES_HORZ_OFFSET                 (12'b000000000000),
  .GTHE3_CHANNEL_ES_PMA_CFG                     (10'b0000000000),
  .GTHE3_CHANNEL_ES_PRESCALE                    (5'b00000),
  .GTHE3_CHANNEL_ES_QUALIFIER0                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_QUALIFIER1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_QUALIFIER2                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_QUALIFIER3                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_QUALIFIER4                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_QUAL_MASK0                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_QUAL_MASK1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_QUAL_MASK2                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_QUAL_MASK3                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_QUAL_MASK4                  (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_SDATA_MASK0                 (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_SDATA_MASK1                 (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_SDATA_MASK2                 (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_SDATA_MASK3                 (16'b0000000000000000),
  .GTHE3_CHANNEL_ES_SDATA_MASK4                 (16'b0000000000000000),
  .GTHE3_CHANNEL_EVODDPHICALDONE_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_EVODDPHICALDONE_VAL            (1'b0),
  .GTHE3_CHANNEL_EVODDPHICALSTART_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_EVODDPHICALSTART_VAL           (1'b0),
  .GTHE3_CHANNEL_EVODDPHIDRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_EVODDPHIDRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_EVODDPHIDWREN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_EVODDPHIDWREN_VAL              (1'b0),
  .GTHE3_CHANNEL_EVODDPHIXRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_EVODDPHIXRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_EVODDPHIXWREN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_EVODDPHIXWREN_VAL              (1'b0),
  .GTHE3_CHANNEL_EVODD_PHI_CFG                  (11'b00000000000),
  .GTHE3_CHANNEL_EYESCANMODE_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_EYESCANMODE_VAL                (1'b0),
  .GTHE3_CHANNEL_EYESCANRESET_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_EYESCANRESET_VAL               (1'b0),
  .GTHE3_CHANNEL_EYESCANTRIGGER_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_EYESCANTRIGGER_VAL             (1'b0),
  .GTHE3_CHANNEL_EYE_SCAN_SWAP_EN               (1'b0),
  .GTHE3_CHANNEL_FTS_DESKEW_SEQ_ENABLE          (4'b1111),
  .GTHE3_CHANNEL_FTS_LANE_DESKEW_CFG            (4'b1111),
  .GTHE3_CHANNEL_FTS_LANE_DESKEW_EN             ("FALSE"),
  .GTHE3_CHANNEL_GEARBOX_MODE                   (5'b10001),
  .GTHE3_CHANNEL_GM_BIAS_SELECT                 (1'b0),
  .GTHE3_CHANNEL_GTGREFCLK_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_GTGREFCLK_VAL                  (1'b0),
  .GTHE3_CHANNEL_GTHRXN_TIE_EN                  (1'b0),
  .GTHE3_CHANNEL_GTHRXN_VAL                     (1'b0),
  .GTHE3_CHANNEL_GTHRXP_TIE_EN                  (1'b0),
  .GTHE3_CHANNEL_GTHRXP_VAL                     (1'b0),
  .GTHE3_CHANNEL_GTNORTHREFCLK0_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_GTNORTHREFCLK0_VAL             (1'b0),
  .GTHE3_CHANNEL_GTNORTHREFCLK1_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_GTNORTHREFCLK1_VAL             (1'b0),
  .GTHE3_CHANNEL_GTREFCLK0_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_GTREFCLK0_VAL                  (1'b0),
  .GTHE3_CHANNEL_GTREFCLK1_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_GTREFCLK1_VAL                  (1'b0),
  .GTHE3_CHANNEL_GTRESETSEL_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_GTRESETSEL_VAL                 (1'b0),
  .GTHE3_CHANNEL_GTRSVD_TIE_EN                  (1'b0),
  .GTHE3_CHANNEL_GTRSVD_VAL                     (16'b0000000000000000),
  .GTHE3_CHANNEL_GTRXRESET_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_GTRXRESET_VAL                  (1'b0),
  .GTHE3_CHANNEL_GTSOUTHREFCLK0_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_GTSOUTHREFCLK0_VAL             (1'b0),
  .GTHE3_CHANNEL_GTSOUTHREFCLK1_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_GTSOUTHREFCLK1_VAL             (1'b0),
  .GTHE3_CHANNEL_GTTXRESET_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_GTTXRESET_VAL                  (1'b0),
  .GTHE3_CHANNEL_LOCAL_MASTER                   (1'b1),
  .GTHE3_CHANNEL_LOOPBACK_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_LOOPBACK_VAL                   (3'b000),
  .GTHE3_CHANNEL_LPBKRXTXSEREN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_LPBKRXTXSEREN_VAL              (1'b0),
  .GTHE3_CHANNEL_LPBKTXRXSEREN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_LPBKTXRXSEREN_VAL              (1'b0),
  .GTHE3_CHANNEL_OOBDIVCTL                      (2'b00),
  .GTHE3_CHANNEL_OOB_PWRUP                      (1'b0),
  .GTHE3_CHANNEL_PCI3_AUTO_REALIGN              ("OVR_1K_BLK"),
  .GTHE3_CHANNEL_PCI3_PIPE_RX_ELECIDLE          (1'b0),
  .GTHE3_CHANNEL_PCI3_RX_ASYNC_EBUF_BYPASS      (2'b00),
  .GTHE3_CHANNEL_PCI3_RX_ELECIDLE_EI2_ENABLE    (1'b0),
  .GTHE3_CHANNEL_PCI3_RX_ELECIDLE_H2L_COUNT     (6'b000000),
  .GTHE3_CHANNEL_PCI3_RX_ELECIDLE_H2L_DISABLE   (3'b000),
  .GTHE3_CHANNEL_PCI3_RX_ELECIDLE_HI_COUNT      (6'b000000),
  .GTHE3_CHANNEL_PCI3_RX_ELECIDLE_LP4_DISABLE   (1'b0),
  .GTHE3_CHANNEL_PCI3_RX_FIFO_DISABLE           (1'b0),
  .GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_TIE_EN     (1'b0),
  .GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_VAL        (1'b0),
  .GTHE3_CHANNEL_PCIERSTIDLE_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_PCIERSTIDLE_VAL                (1'b0),
  .GTHE3_CHANNEL_PCIERSTTXSYNCSTART_TIE_EN      (1'b0),
  .GTHE3_CHANNEL_PCIERSTTXSYNCSTART_VAL         (1'b0),
  .GTHE3_CHANNEL_PCIEUSERRATEDONE_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_PCIEUSERRATEDONE_VAL           (1'b0),
  .GTHE3_CHANNEL_PCIE_BUFG_DIV_CTRL             (16'b0001000000000000),
  .GTHE3_CHANNEL_PCIE_RXPCS_CFG_GEN3            (16'b0000001010100100),
  .GTHE3_CHANNEL_PCIE_RXPMA_CFG                 (16'b0000000000001010),
  .GTHE3_CHANNEL_PCIE_TXPCS_CFG_GEN3            (16'b0010010010100100),
  .GTHE3_CHANNEL_PCIE_TXPMA_CFG                 (16'b0000000000001010),
  .GTHE3_CHANNEL_PCSRSVDIN2_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_PCSRSVDIN2_VAL                 (5'b00000),
  .GTHE3_CHANNEL_PCSRSVDIN_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_PCSRSVDIN_VAL                  (16'b0000000000000000),
  .GTHE3_CHANNEL_PCS_PCIE_EN                    ("FALSE"),
  .GTHE3_CHANNEL_PCS_RSVD0                      (16'b0000000000000000),
  .GTHE3_CHANNEL_PCS_RSVD1                      (3'b000),
  .GTHE3_CHANNEL_PD_TRANS_TIME_FROM_P2          (12'b000000111100),
  .GTHE3_CHANNEL_PD_TRANS_TIME_NONE_P2          (8'b00011001),
  .GTHE3_CHANNEL_PD_TRANS_TIME_TO_P2            (8'b01100100),
  .GTHE3_CHANNEL_PLL_SEL_MODE_GEN12             (2'b00),
  .GTHE3_CHANNEL_PLL_SEL_MODE_GEN3              (2'b11),
  .GTHE3_CHANNEL_PMARSVDIN_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_PMARSVDIN_VAL                  (5'b00000),
  .GTHE3_CHANNEL_PMA_RSV1                       (16'b1111000000000000),
  .GTHE3_CHANNEL_PROCESS_PAR                    (3'b010),
  .GTHE3_CHANNEL_QPLL0CLK_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_QPLL0CLK_VAL                   (1'b0),
  .GTHE3_CHANNEL_QPLL0REFCLK_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_QPLL0REFCLK_VAL                (1'b0),
  .GTHE3_CHANNEL_QPLL1CLK_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_QPLL1CLK_VAL                   (1'b0),
  .GTHE3_CHANNEL_QPLL1REFCLK_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_QPLL1REFCLK_VAL                (1'b0),
  .GTHE3_CHANNEL_RATE_SW_USE_DRP                (1'b1),
  .GTHE3_CHANNEL_RESETOVRD_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RESETOVRD_VAL                  (1'b0),
  .GTHE3_CHANNEL_RESET_POWERSAVE_DISABLE        (1'b0),
  .GTHE3_CHANNEL_RSTCLKENTX_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RSTCLKENTX_VAL                 (1'b0),
  .GTHE3_CHANNEL_RX8B10BEN_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RX8B10BEN_VAL                  (1'b0),
  .GTHE3_CHANNEL_RXBUFRESET_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXBUFRESET_TIME                (5'b00011),
  .GTHE3_CHANNEL_RXBUFRESET_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXBUF_ADDR_MODE                ("FAST"),
  .GTHE3_CHANNEL_RXBUF_EIDLE_HI_CNT             (4'b1000),
  .GTHE3_CHANNEL_RXBUF_EIDLE_LO_CNT             (4'b0000),
  .GTHE3_CHANNEL_RXBUF_EN                       ("FALSE"),
  .GTHE3_CHANNEL_RXBUF_RESET_ON_CB_CHANGE       ("TRUE"),
  .GTHE3_CHANNEL_RXBUF_RESET_ON_COMMAALIGN      ("FALSE"),
  .GTHE3_CHANNEL_RXBUF_RESET_ON_EIDLE           ("FALSE"),
  .GTHE3_CHANNEL_RXBUF_RESET_ON_RATE_CHANGE     ("TRUE"),
  .GTHE3_CHANNEL_RXBUF_THRESH_OVFLW             (0),
  .GTHE3_CHANNEL_RXBUF_THRESH_OVRD              ("FALSE"),
  .GTHE3_CHANNEL_RXBUF_THRESH_UNDFLW            (0),
  .GTHE3_CHANNEL_RXCDRFREQRESET_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXCDRFREQRESET_TIME            (5'b00001),
  .GTHE3_CHANNEL_RXCDRFREQRESET_VAL             (1'b0),
  .GTHE3_CHANNEL_RXCDRHOLD_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXCDRHOLD_VAL                  (1'b0),
  .GTHE3_CHANNEL_RXCDROVRDEN_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXCDROVRDEN_VAL                (1'b0),
  .GTHE3_CHANNEL_RXCDRPHRESET_TIME              (5'b00001),
  .GTHE3_CHANNEL_RXCDRRESETRSV_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXCDRRESETRSV_VAL              (1'b0),
  .GTHE3_CHANNEL_RXCDRRESET_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXCDRRESET_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXCDR_CFG0                     (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_CFG0_GEN3                (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_CFG1                     (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_CFG1_GEN3                (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_CFG2                     (16'b0000011111010110),
  .GTHE3_CHANNEL_RXCDR_CFG2_GEN3                (16'b0000011111100110),
  .GTHE3_CHANNEL_RXCDR_CFG3                     (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_CFG3_GEN3                (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_CFG4                     (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_CFG4_GEN3                (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_CFG5                     (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_CFG5_GEN3                (16'b0000000000000000),
  .GTHE3_CHANNEL_RXCDR_FR_RESET_ON_EIDLE        (1'b0),
  .GTHE3_CHANNEL_RXCDR_HOLD_DURING_EIDLE        (1'b0),
  .GTHE3_CHANNEL_RXCDR_LOCK_CFG0                (16'b0100010010000000),
  .GTHE3_CHANNEL_RXCDR_LOCK_CFG1                (16'b0101111111111111),
  .GTHE3_CHANNEL_RXCDR_LOCK_CFG2                (16'b0111011111000011),
  .GTHE3_CHANNEL_RXCDR_PH_RESET_ON_EIDLE        (1'b0),
  .GTHE3_CHANNEL_RXCFOK_CFG0                    (16'b0100000000000000),
  .GTHE3_CHANNEL_RXCFOK_CFG1                    (16'b0000000001100101),
  .GTHE3_CHANNEL_RXCFOK_CFG2                    (16'b0000000000101110),
  .GTHE3_CHANNEL_RXCHBONDEN_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXCHBONDEN_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXCHBONDI_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXCHBONDI_VAL                  (5'b00000),
  .GTHE3_CHANNEL_RXCHBONDLEVEL_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXCHBONDLEVEL_VAL              (3'b000),
  .GTHE3_CHANNEL_RXCHBONDMASTER_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXCHBONDMASTER_VAL             (1'b0),
  .GTHE3_CHANNEL_RXCHBONDSLAVE_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXCHBONDSLAVE_VAL              (1'b0),
  .GTHE3_CHANNEL_RXCOMMADETEN_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_RXCOMMADETEN_VAL               (1'b0),
  .GTHE3_CHANNEL_RXDFEAGCCTRL_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_RXDFEAGCCTRL_VAL               (2'b01),
  .GTHE3_CHANNEL_RXDFEAGCHOLD_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_RXDFEAGCHOLD_VAL               (1'b0),
  .GTHE3_CHANNEL_RXDFEAGCOVRDEN_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXDFEAGCOVRDEN_VAL             (1'b0),
  .GTHE3_CHANNEL_RXDFELFHOLD_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXDFELFHOLD_VAL                (1'b0),
  .GTHE3_CHANNEL_RXDFELFOVRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFELFOVRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFELPMRESET_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFELPMRESET_TIME             (7'b0001111),
  .GTHE3_CHANNEL_RXDFELPMRESET_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFELPM_KL_CFG0               (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFELPM_KL_CFG1               (16'b0000000000000010),
  .GTHE3_CHANNEL_RXDFELPM_KL_CFG2               (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFETAP10HOLD_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXDFETAP10HOLD_VAL             (1'b0),
  .GTHE3_CHANNEL_RXDFETAP10OVRDEN_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_RXDFETAP10OVRDEN_VAL           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP11HOLD_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXDFETAP11HOLD_VAL             (1'b0),
  .GTHE3_CHANNEL_RXDFETAP11OVRDEN_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_RXDFETAP11OVRDEN_VAL           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP12HOLD_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXDFETAP12HOLD_VAL             (1'b0),
  .GTHE3_CHANNEL_RXDFETAP12OVRDEN_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_RXDFETAP12OVRDEN_VAL           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP13HOLD_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXDFETAP13HOLD_VAL             (1'b0),
  .GTHE3_CHANNEL_RXDFETAP13OVRDEN_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_RXDFETAP13OVRDEN_VAL           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP14HOLD_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXDFETAP14HOLD_VAL             (1'b0),
  .GTHE3_CHANNEL_RXDFETAP14OVRDEN_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_RXDFETAP14OVRDEN_VAL           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP15HOLD_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXDFETAP15HOLD_VAL             (1'b0),
  .GTHE3_CHANNEL_RXDFETAP15OVRDEN_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_RXDFETAP15OVRDEN_VAL           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP2HOLD_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP2HOLD_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFETAP2OVRDEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXDFETAP2OVRDEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXDFETAP3HOLD_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP3HOLD_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFETAP3OVRDEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXDFETAP3OVRDEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXDFETAP4HOLD_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP4HOLD_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFETAP4OVRDEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXDFETAP4OVRDEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXDFETAP5HOLD_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP5HOLD_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFETAP5OVRDEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXDFETAP5OVRDEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXDFETAP6HOLD_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP6HOLD_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFETAP6OVRDEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXDFETAP6OVRDEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXDFETAP7HOLD_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP7HOLD_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFETAP7OVRDEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXDFETAP7OVRDEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXDFETAP8HOLD_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP8HOLD_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFETAP8OVRDEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXDFETAP8OVRDEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXDFETAP9HOLD_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFETAP9HOLD_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFETAP9OVRDEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXDFETAP9OVRDEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXDFEUTHOLD_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXDFEUTHOLD_VAL                (1'b0),
  .GTHE3_CHANNEL_RXDFEUTOVRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFEUTOVRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFEVPHOLD_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXDFEVPHOLD_VAL                (1'b0),
  .GTHE3_CHANNEL_RXDFEVPOVRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXDFEVPOVRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_RXDFEVSEN_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXDFEVSEN_VAL                  (1'b0),
  .GTHE3_CHANNEL_RXDFEXYDEN_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXDFEXYDEN_VAL                 (1'b1),
  .GTHE3_CHANNEL_RXDFE_CFG0                     (16'b0000101000000000),
  .GTHE3_CHANNEL_RXDFE_CFG1                     (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_GC_CFG0                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_GC_CFG1                  (16'b0111100001110000),
  .GTHE3_CHANNEL_RXDFE_GC_CFG2                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_H2_CFG0                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_H2_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_H3_CFG0                  (16'b0100000000000000),
  .GTHE3_CHANNEL_RXDFE_H3_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_H4_CFG0                  (16'b0010000000000000),
  .GTHE3_CHANNEL_RXDFE_H4_CFG1                  (16'b0000000000000011),
  .GTHE3_CHANNEL_RXDFE_H5_CFG0                  (16'b0010000000000000),
  .GTHE3_CHANNEL_RXDFE_H5_CFG1                  (16'b0000000000000011),
  .GTHE3_CHANNEL_RXDFE_H6_CFG0                  (16'b0010000000000000),
  .GTHE3_CHANNEL_RXDFE_H6_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_H7_CFG0                  (16'b0010000000000000),
  .GTHE3_CHANNEL_RXDFE_H7_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_H8_CFG0                  (16'b0010000000000000),
  .GTHE3_CHANNEL_RXDFE_H8_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_H9_CFG0                  (16'b0010000000000000),
  .GTHE3_CHANNEL_RXDFE_H9_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HA_CFG0                  (16'b0010000000000000),
  .GTHE3_CHANNEL_RXDFE_HA_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HB_CFG0                  (16'b0010000000000000),
  .GTHE3_CHANNEL_RXDFE_HB_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HC_CFG0                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HC_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HD_CFG0                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HD_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HE_CFG0                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HE_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HF_CFG0                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_HF_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_OS_CFG0                  (16'b1000000000000000),
  .GTHE3_CHANNEL_RXDFE_OS_CFG1                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXDFE_UT_CFG0                  (16'b1000000000000000),
  .GTHE3_CHANNEL_RXDFE_UT_CFG1                  (16'b0000000000000011),
  .GTHE3_CHANNEL_RXDFE_VP_CFG0                  (16'b1010101000000000),
  .GTHE3_CHANNEL_RXDFE_VP_CFG1                  (16'b0000000000110011),
  .GTHE3_CHANNEL_RXDLYBYPASS_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXDLYBYPASS_VAL                (1'b1),
  .GTHE3_CHANNEL_RXDLYEN_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_RXDLYEN_VAL                    (1'b0),
  .GTHE3_CHANNEL_RXDLYOVRDEN_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXDLYOVRDEN_VAL                (1'b0),
  .GTHE3_CHANNEL_RXDLYSRESET_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXDLYSRESET_VAL                (1'b0),
  .GTHE3_CHANNEL_RXDLY_CFG                      (16'b0000000000011111),
  .GTHE3_CHANNEL_RXDLY_LCFG                     (16'b0000000000110000),
  .GTHE3_CHANNEL_RXELECIDLEMODE_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXELECIDLEMODE_VAL             (2'b11),
  .GTHE3_CHANNEL_RXELECIDLE_CFG                 ("Sigcfg_4"),
  .GTHE3_CHANNEL_RXGBOX_FIFO_INIT_RD_ADDR       (4),
  .GTHE3_CHANNEL_RXGEARBOXSLIP_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXGEARBOXSLIP_VAL              (1'b0),
  .GTHE3_CHANNEL_RXGEARBOX_EN                   ("TRUE"),
  .GTHE3_CHANNEL_RXISCANRESET_TIME              (5'b00001),
  .GTHE3_CHANNEL_RXLATCLK_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_RXLATCLK_VAL                   (1'b0),
  .GTHE3_CHANNEL_RXLPMEN_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_RXLPMEN_VAL                    (1'b0),
  .GTHE3_CHANNEL_RXLPMGCHOLD_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXLPMGCHOLD_VAL                (1'b0),
  .GTHE3_CHANNEL_RXLPMGCOVRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXLPMGCOVRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_RXLPMHFHOLD_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXLPMHFHOLD_VAL                (1'b0),
  .GTHE3_CHANNEL_RXLPMHFOVRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXLPMHFOVRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_RXLPMLFHOLD_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXLPMLFHOLD_VAL                (1'b0),
  .GTHE3_CHANNEL_RXLPMLFKLOVRDEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXLPMLFKLOVRDEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXLPMOSHOLD_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXLPMOSHOLD_VAL                (1'b0),
  .GTHE3_CHANNEL_RXLPMOSOVRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXLPMOSOVRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_RXLPM_CFG                      (16'b0000000000000000),
  .GTHE3_CHANNEL_RXLPM_GC_CFG                   (16'b0001000000000000),
  .GTHE3_CHANNEL_RXLPM_KH_CFG0                  (16'b0000000000000000),
  .GTHE3_CHANNEL_RXLPM_KH_CFG1                  (16'b0000000000000010),
  .GTHE3_CHANNEL_RXLPM_OS_CFG0                  (16'b1000000000000000),
  .GTHE3_CHANNEL_RXLPM_OS_CFG1                  (16'b0000000000000010),
  .GTHE3_CHANNEL_RXMCOMMAALIGNEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXMCOMMAALIGNEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXMONITORSEL_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_RXMONITORSEL_VAL               (2'b00),
  .GTHE3_CHANNEL_RXOOBRESET_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXOOBRESET_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXOOB_CFG                      (9'b000000110),
  .GTHE3_CHANNEL_RXOOB_CLK_CFG                  ("PMA"),
  .GTHE3_CHANNEL_RXOSCALRESET_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_RXOSCALRESET_TIME              (5'b00011),
  .GTHE3_CHANNEL_RXOSCALRESET_VAL               (1'b0),
  .GTHE3_CHANNEL_RXOSHOLD_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_RXOSHOLD_VAL                   (1'b0),
  .GTHE3_CHANNEL_RXOSINTCFG_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXOSINTCFG_VAL                 (4'b1101),
  .GTHE3_CHANNEL_RXOSINTEN_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXOSINTEN_VAL                  (1'b1),
  .GTHE3_CHANNEL_RXOSINTHOLD_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXOSINTHOLD_VAL                (1'b0),
  .GTHE3_CHANNEL_RXOSINTOVRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXOSINTOVRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_RXOSINTSTROBE_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_RXOSINTSTROBE_VAL              (1'b0),
  .GTHE3_CHANNEL_RXOSINTTESTOVRDEN_TIE_EN       (1'b0),
  .GTHE3_CHANNEL_RXOSINTTESTOVRDEN_VAL          (1'b0),
  .GTHE3_CHANNEL_RXOSOVRDEN_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXOSOVRDEN_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXOUTCLKSEL_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXOUTCLKSEL_VAL                (3'b101),
  .GTHE3_CHANNEL_RXOUT_DIV                      (2),
  .GTHE3_CHANNEL_RXPCOMMAALIGNEN_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_RXPCOMMAALIGNEN_VAL            (1'b0),
  .GTHE3_CHANNEL_RXPCSRESET_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXPCSRESET_TIME                (5'b00011),
  .GTHE3_CHANNEL_RXPCSRESET_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXPD_TIE_EN                    (1'b0),
  .GTHE3_CHANNEL_RXPD_VAL                       (2'b00),
  .GTHE3_CHANNEL_RXPHALIGNEN_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXPHALIGNEN_VAL                (1'b0),
  .GTHE3_CHANNEL_RXPHALIGN_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXPHALIGN_VAL                  (1'b0),
  .GTHE3_CHANNEL_RXPHBEACON_CFG                 (16'b0000000000000000),
  .GTHE3_CHANNEL_RXPHDLYPD_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXPHDLYPD_VAL                  (1'b1),
  .GTHE3_CHANNEL_RXPHDLYRESET_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_RXPHDLYRESET_VAL               (1'b0),
  .GTHE3_CHANNEL_RXPHDLY_CFG                    (16'b0010000000100000),
  .GTHE3_CHANNEL_RXPHOVRDEN_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXPHOVRDEN_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXPHSAMP_CFG                   (16'b0010000100000000),
  .GTHE3_CHANNEL_RXPHSLIP_CFG                   (16'b0110011000100010),
  .GTHE3_CHANNEL_RXPH_MONITOR_SEL               (5'b00000),
  .GTHE3_CHANNEL_RXPI_CFG0                      (2'b00),
  .GTHE3_CHANNEL_RXPI_CFG1                      (2'b00),
  .GTHE3_CHANNEL_RXPI_CFG2                      (2'b00),
  .GTHE3_CHANNEL_RXPI_CFG3                      (2'b00),
  .GTHE3_CHANNEL_RXPI_CFG4                      (1'b0),
  .GTHE3_CHANNEL_RXPI_CFG5                      (1'b1),
  .GTHE3_CHANNEL_RXPI_CFG6                      (3'b000),
  .GTHE3_CHANNEL_RXPI_LPM                       (1'b0),
  .GTHE3_CHANNEL_RXPI_VREFSEL                   (1'b0),
  .GTHE3_CHANNEL_RXPLLCLKSEL_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXPLLCLKSEL_VAL                (2'b11),
  .GTHE3_CHANNEL_RXPMACLK_SEL                   ("DATA"),
  .GTHE3_CHANNEL_RXPMARESET_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXPMARESET_TIME                (5'b00011),
  .GTHE3_CHANNEL_RXPMARESET_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXPOLARITY_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXPOLARITY_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXPRBSCNTRESET_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXPRBSCNTRESET_VAL             (1'b0),
  .GTHE3_CHANNEL_RXPRBSSEL_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXPRBSSEL_VAL                  (4'b0000),
  .GTHE3_CHANNEL_RXPRBS_ERR_LOOPBACK            (1'b0),
  .GTHE3_CHANNEL_RXPRBS_LINKACQ_CNT             (15),
  .GTHE3_CHANNEL_RXPROGDIVRESET_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_RXPROGDIVRESET_VAL             (1'b0),
  .GTHE3_CHANNEL_RXQPIEN_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_RXQPIEN_VAL                    (1'b0),
  .GTHE3_CHANNEL_RXRATEMODE_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXRATEMODE_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXRATE_TIE_EN                  (1'b0),
  .GTHE3_CHANNEL_RXRATE_VAL                     (3'b000),
  .GTHE3_CHANNEL_RXSLIDE_AUTO_WAIT              (7),
  .GTHE3_CHANNEL_RXSLIDE_MODE                   ("OFF"),
  .GTHE3_CHANNEL_RXSLIDE_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_RXSLIDE_VAL                    (1'b0),
  .GTHE3_CHANNEL_RXSLIPOUTCLK_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_RXSLIPOUTCLK_VAL               (1'b0),
  .GTHE3_CHANNEL_RXSLIPPMA_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXSLIPPMA_VAL                  (1'b0),
  .GTHE3_CHANNEL_RXSYNCALLIN_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXSYNCALLIN_VAL                (1'b0),
  .GTHE3_CHANNEL_RXSYNCIN_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_RXSYNCIN_VAL                   (1'b0),
  .GTHE3_CHANNEL_RXSYNCMODE_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_RXSYNCMODE_VAL                 (1'b0),
  .GTHE3_CHANNEL_RXSYNC_MULTILANE               (1'b0),
  .GTHE3_CHANNEL_RXSYNC_OVRD                    (1'b0),
  .GTHE3_CHANNEL_RXSYNC_SKIP_DA                 (1'b0),
  .GTHE3_CHANNEL_RXSYSCLKSEL_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_RXSYSCLKSEL_VAL                (2'b10),
  .GTHE3_CHANNEL_RXUSERRDY_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXUSERRDY_VAL                  (1'b1),
  .GTHE3_CHANNEL_RXUSRCLK2_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_RXUSRCLK2_VAL                  (1'b0),
  .GTHE3_CHANNEL_RXUSRCLK_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_RXUSRCLK_VAL                   (1'b0),
  .GTHE3_CHANNEL_RX_AFE_CM_EN                   (1'b0),
  .GTHE3_CHANNEL_RX_BIAS_CFG0                   (16'b0000101010110100),
  .GTHE3_CHANNEL_RX_BUFFER_CFG                  (6'b000000),
  .GTHE3_CHANNEL_RX_CAPFF_SARC_ENB              (1'b0),
  .GTHE3_CHANNEL_RX_CLK25_DIV                   (7),
  .GTHE3_CHANNEL_RX_CLKMUX_EN                   (1'b1),
  .GTHE3_CHANNEL_RX_CLK_SLIP_OVRD               (5'b00000),
  .GTHE3_CHANNEL_RX_CM_BUF_CFG                  (4'b1010),
  .GTHE3_CHANNEL_RX_CM_BUF_PD                   (1'b0),
  .GTHE3_CHANNEL_RX_CM_SEL                      (2'b11),
  .GTHE3_CHANNEL_RX_CM_TRIM                     (4'b1010),
  .GTHE3_CHANNEL_RX_CTLE3_LPF                   (8'b00000001),
  .GTHE3_CHANNEL_RX_DATA_WIDTH                  (64),
  .GTHE3_CHANNEL_RX_DDI_SEL                     (6'b000000),
  .GTHE3_CHANNEL_RX_DEFER_RESET_BUF_EN          ("TRUE"),
  .GTHE3_CHANNEL_RX_DFELPM_CFG0                 (4'b0110),
  .GTHE3_CHANNEL_RX_DFELPM_CFG1                 (1'b1),
  .GTHE3_CHANNEL_RX_DFELPM_KLKH_AGC_STUP_EN     (1'b1),
  .GTHE3_CHANNEL_RX_DFE_AGC_CFG0                (2'b10),
  .GTHE3_CHANNEL_RX_DFE_AGC_CFG1                (3'b100),
  .GTHE3_CHANNEL_RX_DFE_KL_LPM_KH_CFG0          (2'b01),
  .GTHE3_CHANNEL_RX_DFE_KL_LPM_KH_CFG1          (3'b100),
  .GTHE3_CHANNEL_RX_DFE_KL_LPM_KL_CFG0          (2'b01),
  .GTHE3_CHANNEL_RX_DFE_KL_LPM_KL_CFG1          (3'b100),
  .GTHE3_CHANNEL_RX_DFE_LPM_HOLD_DURING_EIDLE   (1'b0),
  .GTHE3_CHANNEL_RX_DISPERR_SEQ_MATCH           ("TRUE"),
  .GTHE3_CHANNEL_RX_DIVRESET_TIME               (5'b00001),
  .GTHE3_CHANNEL_RX_EN_HI_LR                    (1'b0),
  .GTHE3_CHANNEL_RX_EYESCAN_VS_CODE             (7'b0000000),
  .GTHE3_CHANNEL_RX_EYESCAN_VS_NEG_DIR          (1'b0),
  .GTHE3_CHANNEL_RX_EYESCAN_VS_RANGE            (2'b00),
  .GTHE3_CHANNEL_RX_EYESCAN_VS_UT_SIGN          (1'b0),
  .GTHE3_CHANNEL_RX_FABINT_USRCLK_FLOP          (1'b0),
  .GTHE3_CHANNEL_RX_INT_DATAWIDTH               (1),
  .GTHE3_CHANNEL_RX_PMA_POWER_SAVE              (1'b0),
  .GTHE3_CHANNEL_RX_PROGDIV_CFG                 (33.0),
  .GTHE3_CHANNEL_RX_SAMPLE_PERIOD               (3'b111),
  .GTHE3_CHANNEL_RX_SIG_VALID_DLY               (11),
  .GTHE3_CHANNEL_RX_SUM_DFETAPREP_EN            (1'b0),
  .GTHE3_CHANNEL_RX_SUM_IREF_TUNE               (4'b0000),
  .GTHE3_CHANNEL_RX_SUM_RES_CTRL                (2'b00),
  .GTHE3_CHANNEL_RX_SUM_VCMTUNE                 (4'b0000),
  .GTHE3_CHANNEL_RX_SUM_VCM_OVWR                (1'b0),
  .GTHE3_CHANNEL_RX_SUM_VREF_TUNE               (3'b000),
  .GTHE3_CHANNEL_RX_TUNE_AFE_OS                 (2'b10),
  .GTHE3_CHANNEL_RX_WIDEMODE_CDR                (1'b0),
  .GTHE3_CHANNEL_RX_XCLK_SEL                    ("RXDES"),
  .GTHE3_CHANNEL_SAS_MAX_COM                    (64),
  .GTHE3_CHANNEL_SAS_MIN_COM                    (36),
  .GTHE3_CHANNEL_SATA_BURST_SEQ_LEN             (4'b1110),
  .GTHE3_CHANNEL_SATA_BURST_VAL                 (3'b100),
  .GTHE3_CHANNEL_SATA_CPLL_CFG                  ("VCO_3000MHZ"),
  .GTHE3_CHANNEL_SATA_EIDLE_VAL                 (3'b100),
  .GTHE3_CHANNEL_SATA_MAX_BURST                 (8),
  .GTHE3_CHANNEL_SATA_MAX_INIT                  (21),
  .GTHE3_CHANNEL_SATA_MAX_WAKE                  (7),
  .GTHE3_CHANNEL_SATA_MIN_BURST                 (4),
  .GTHE3_CHANNEL_SATA_MIN_INIT                  (12),
  .GTHE3_CHANNEL_SATA_MIN_WAKE                  (4),
  .GTHE3_CHANNEL_SHOW_REALIGN_COMMA             ("TRUE"),
  .GTHE3_CHANNEL_SIGVALIDCLK_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_SIGVALIDCLK_VAL                (1'b0),
  .GTHE3_CHANNEL_SIM_RECEIVER_DETECT_PASS       ("TRUE"),
  .GTHE3_CHANNEL_SIM_RESET_SPEEDUP              ("TRUE"),
  .GTHE3_CHANNEL_SIM_TX_EIDLE_DRIVE_LEVEL       (1'b0),
  .GTHE3_CHANNEL_SIM_VERSION                    (2),
  .GTHE3_CHANNEL_TAPDLY_SET_TX                  (2'b00),
  .GTHE3_CHANNEL_TEMPERATUR_PAR                 (4'b0010),
  .GTHE3_CHANNEL_TERM_RCAL_CFG                  (15'b100001000010000),
  .GTHE3_CHANNEL_TERM_RCAL_OVRD                 (3'b000),
  .GTHE3_CHANNEL_TRANS_TIME_RATE                (8'b00001110),
  .GTHE3_CHANNEL_TSTIN_TIE_EN                   (1'b1),
  .GTHE3_CHANNEL_TSTIN_VAL                      (20'b00000000000000000000),
  .GTHE3_CHANNEL_TST_RSV0                       (8'b00000000),
  .GTHE3_CHANNEL_TST_RSV1                       (8'b00000000),
  .GTHE3_CHANNEL_TX8B10BBYPASS_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_TX8B10BBYPASS_VAL              (8'b00000000),
  .GTHE3_CHANNEL_TX8B10BEN_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TX8B10BEN_VAL                  (1'b0),
  .GTHE3_CHANNEL_TXBUFDIFFCTRL_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_TXBUFDIFFCTRL_VAL              (3'b000),
  .GTHE3_CHANNEL_TXBUF_EN                       ("FALSE"),
  .GTHE3_CHANNEL_TXBUF_RESET_ON_RATE_CHANGE     ("TRUE"),
  .GTHE3_CHANNEL_TXCOMINIT_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXCOMINIT_VAL                  (1'b0),
  .GTHE3_CHANNEL_TXCOMSAS_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXCOMSAS_VAL                   (1'b0),
  .GTHE3_CHANNEL_TXCOMWAKE_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXCOMWAKE_VAL                  (1'b0),
  .GTHE3_CHANNEL_TXCTRL0_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_TXCTRL0_VAL                    (16'b0000000000000000),
  .GTHE3_CHANNEL_TXCTRL1_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_TXCTRL1_VAL                    (16'b0000000000000000),
  .GTHE3_CHANNEL_TXCTRL2_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_TXCTRL2_VAL                    (8'b00000000),
  .GTHE3_CHANNEL_TXDATAEXTENDRSVD_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_TXDATAEXTENDRSVD_VAL           (8'b00000000),
  .GTHE3_CHANNEL_TXDATA_TIE_EN                  (1'b0),
  .GTHE3_CHANNEL_TXDATA_VAL                     (1'b0),
  .GTHE3_CHANNEL_TXDEEMPH_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXDEEMPH_VAL                   (1'b0),
  .GTHE3_CHANNEL_TXDETECTRX_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXDETECTRX_VAL                 (1'b0),
  .GTHE3_CHANNEL_TXDIFFCTRL_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXDIFFCTRL_VAL                 (4'b1100),
  .GTHE3_CHANNEL_TXDIFFPD_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXDIFFPD_VAL                   (1'b0),
  .GTHE3_CHANNEL_TXDLYBYPASS_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXDLYBYPASS_VAL                (1'b1),
  .GTHE3_CHANNEL_TXDLYEN_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_TXDLYEN_VAL                    (1'b0),
  .GTHE3_CHANNEL_TXDLYHOLD_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXDLYHOLD_VAL                  (1'b0),
  .GTHE3_CHANNEL_TXDLYOVRDEN_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXDLYOVRDEN_VAL                (1'b0),
  .GTHE3_CHANNEL_TXDLYSRESET_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXDLYSRESET_VAL                (1'b0),
  .GTHE3_CHANNEL_TXDLYUPDOWN_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXDLYUPDOWN_VAL                (1'b0),
  .GTHE3_CHANNEL_TXDLY_CFG                      (16'b0000000000001001),
  .GTHE3_CHANNEL_TXDLY_LCFG                     (16'b0000000001010000),
  .GTHE3_CHANNEL_TXDRVBIAS_N                    (4'b1010),
  .GTHE3_CHANNEL_TXDRVBIAS_P                    (4'b1010),
  .GTHE3_CHANNEL_TXELECIDLE_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXELECIDLE_VAL                 (1'b0),
  .GTHE3_CHANNEL_TXFIFO_ADDR_CFG                ("LOW"),
  .GTHE3_CHANNEL_TXGBOX_FIFO_INIT_RD_ADDR       (4),
  .GTHE3_CHANNEL_TXGEARBOX_EN                   ("TRUE"),
  .GTHE3_CHANNEL_TXHEADER_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXHEADER_VAL                   (6'b000000),
  .GTHE3_CHANNEL_TXINHIBIT_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXINHIBIT_VAL                  (1'b0),
  .GTHE3_CHANNEL_TXLATCLK_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXLATCLK_VAL                   (1'b0),
  .GTHE3_CHANNEL_TXMAINCURSOR_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_TXMAINCURSOR_VAL               (7'b1000000),
  .GTHE3_CHANNEL_TXMARGIN_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXMARGIN_VAL                   (3'b000),
  .GTHE3_CHANNEL_TXOUTCLKSEL_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXOUTCLKSEL_VAL                (3'b101),
  .GTHE3_CHANNEL_TXOUT_DIV                      (2),
  .GTHE3_CHANNEL_TXPCSRESET_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXPCSRESET_TIME                (5'b00011),
  .GTHE3_CHANNEL_TXPCSRESET_VAL                 (1'b0),
  .GTHE3_CHANNEL_TXPDELECIDLEMODE_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_TXPDELECIDLEMODE_VAL           (1'b0),
  .GTHE3_CHANNEL_TXPD_TIE_EN                    (1'b0),
  .GTHE3_CHANNEL_TXPD_VAL                       (2'b00),
  .GTHE3_CHANNEL_TXPHALIGNEN_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXPHALIGNEN_VAL                (1'b0),
  .GTHE3_CHANNEL_TXPHALIGN_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXPHALIGN_VAL                  (1'b0),
  .GTHE3_CHANNEL_TXPHDLYPD_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXPHDLYPD_VAL                  (1'b1),
  .GTHE3_CHANNEL_TXPHDLYRESET_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_TXPHDLYRESET_VAL               (1'b0),
  .GTHE3_CHANNEL_TXPHDLYTSTCLK_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_TXPHDLYTSTCLK_VAL              (1'b0),
  .GTHE3_CHANNEL_TXPHDLY_CFG0                   (16'b0010000000100000),
  .GTHE3_CHANNEL_TXPHDLY_CFG1                   (16'b0000000001110101),
  .GTHE3_CHANNEL_TXPHINIT_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXPHINIT_VAL                   (1'b0),
  .GTHE3_CHANNEL_TXPHOVRDEN_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXPHOVRDEN_VAL                 (1'b0),
  .GTHE3_CHANNEL_TXPH_CFG                       (16'b0000100110000000),
  .GTHE3_CHANNEL_TXPH_MONITOR_SEL               (5'b00000),
  .GTHE3_CHANNEL_TXPIPPMEN_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXPIPPMEN_VAL                  (1'b0),
  .GTHE3_CHANNEL_TXPIPPMOVRDEN_TIE_EN           (1'b0),
  .GTHE3_CHANNEL_TXPIPPMOVRDEN_VAL              (1'b0),
  .GTHE3_CHANNEL_TXPIPPMPD_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXPIPPMPD_VAL                  (1'b0),
  .GTHE3_CHANNEL_TXPIPPMSEL_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXPIPPMSEL_VAL                 (1'b0),
  .GTHE3_CHANNEL_TXPIPPMSTEPSIZE_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_TXPIPPMSTEPSIZE_VAL            (5'b00000),
  .GTHE3_CHANNEL_TXPISOPD_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXPISOPD_VAL                   (1'b0),
  .GTHE3_CHANNEL_TXPI_CFG0                      (2'b00),
  .GTHE3_CHANNEL_TXPI_CFG1                      (2'b00),
  .GTHE3_CHANNEL_TXPI_CFG2                      (2'b00),
  .GTHE3_CHANNEL_TXPI_CFG3                      (1'b0),
  .GTHE3_CHANNEL_TXPI_CFG4                      (1'b1),
  .GTHE3_CHANNEL_TXPI_CFG5                      (3'b000),
  .GTHE3_CHANNEL_TXPI_GRAY_SEL                  (1'b0),
  .GTHE3_CHANNEL_TXPI_INVSTROBE_SEL             (1'b0),
  .GTHE3_CHANNEL_TXPI_LPM                       (1'b0),
  .GTHE3_CHANNEL_TXPI_PPMCLK_SEL                ("TXUSRCLK2"),
  .GTHE3_CHANNEL_TXPI_PPM_CFG                   (8'b00000000),
  .GTHE3_CHANNEL_TXPI_SYNFREQ_PPM               (3'b001),
  .GTHE3_CHANNEL_TXPI_VREFSEL                   (1'b0),
  .GTHE3_CHANNEL_TXPLLCLKSEL_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXPLLCLKSEL_VAL                (2'b11),
  .GTHE3_CHANNEL_TXPMARESET_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXPMARESET_TIME                (5'b00011),
  .GTHE3_CHANNEL_TXPMARESET_VAL                 (1'b0),
  .GTHE3_CHANNEL_TXPOLARITY_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXPOLARITY_VAL                 (1'b0),
  .GTHE3_CHANNEL_TXPOSTCURSORINV_TIE_EN         (1'b0),
  .GTHE3_CHANNEL_TXPOSTCURSORINV_VAL            (1'b0),
  .GTHE3_CHANNEL_TXPOSTCURSOR_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_TXPOSTCURSOR_VAL               (5'b00000),
  .GTHE3_CHANNEL_TXPRBSFORCEERR_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_TXPRBSFORCEERR_VAL             (1'b0),
  .GTHE3_CHANNEL_TXPRBSSEL_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXPRBSSEL_VAL                  (4'b0000),
  .GTHE3_CHANNEL_TXPRECURSORINV_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_TXPRECURSORINV_VAL             (1'b0),
  .GTHE3_CHANNEL_TXPRECURSOR_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXPRECURSOR_VAL                (5'b00000),
  .GTHE3_CHANNEL_TXPROGDIVRESET_TIE_EN          (1'b0),
  .GTHE3_CHANNEL_TXPROGDIVRESET_VAL             (1'b0),
  .GTHE3_CHANNEL_TXQPIBIASEN_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXQPIBIASEN_VAL                (1'b0),
  .GTHE3_CHANNEL_TXQPISTRONGPDOWN_TIE_EN        (1'b0),
  .GTHE3_CHANNEL_TXQPISTRONGPDOWN_VAL           (1'b0),
  .GTHE3_CHANNEL_TXQPIWEAKPUP_TIE_EN            (1'b0),
  .GTHE3_CHANNEL_TXQPIWEAKPUP_VAL               (1'b0),
  .GTHE3_CHANNEL_TXRATEMODE_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXRATEMODE_VAL                 (1'b0),
  .GTHE3_CHANNEL_TXRATE_TIE_EN                  (1'b0),
  .GTHE3_CHANNEL_TXRATE_VAL                     (3'b000),
  .GTHE3_CHANNEL_TXSEQUENCE_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXSEQUENCE_VAL                 (7'b0000000),
  .GTHE3_CHANNEL_TXSWING_TIE_EN                 (1'b0),
  .GTHE3_CHANNEL_TXSWING_VAL                    (1'b0),
  .GTHE3_CHANNEL_TXSYNCALLIN_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXSYNCALLIN_VAL                (1'b0),
  .GTHE3_CHANNEL_TXSYNCIN_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXSYNCIN_VAL                   (1'b0),
  .GTHE3_CHANNEL_TXSYNCMODE_TIE_EN              (1'b0),
  .GTHE3_CHANNEL_TXSYNCMODE_VAL                 (1'b0),
  .GTHE3_CHANNEL_TXSYNC_MULTILANE               (1'b0),
  .GTHE3_CHANNEL_TXSYNC_OVRD                    (1'b0),
  .GTHE3_CHANNEL_TXSYNC_SKIP_DA                 (1'b0),
  .GTHE3_CHANNEL_TXSYSCLKSEL_TIE_EN             (1'b0),
  .GTHE3_CHANNEL_TXSYSCLKSEL_VAL                (2'b10),
  .GTHE3_CHANNEL_TXUSERRDY_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXUSERRDY_VAL                  (1'b1),
  .GTHE3_CHANNEL_TXUSRCLK2_TIE_EN               (1'b0),
  .GTHE3_CHANNEL_TXUSRCLK2_VAL                  (1'b0),
  .GTHE3_CHANNEL_TXUSRCLK_TIE_EN                (1'b0),
  .GTHE3_CHANNEL_TXUSRCLK_VAL                   (1'b0),
  .GTHE3_CHANNEL_TX_CLK25_DIV                   (7),
  .GTHE3_CHANNEL_TX_CLKMUX_EN                   (1'b1),
  .GTHE3_CHANNEL_TX_DATA_WIDTH                  (64),
  .GTHE3_CHANNEL_TX_DCD_CFG                     (6'b000010),
  .GTHE3_CHANNEL_TX_DCD_EN                      (1'b0),
  .GTHE3_CHANNEL_TX_DEEMPH0                     (6'b000000),
  .GTHE3_CHANNEL_TX_DEEMPH1                     (6'b000000),
  .GTHE3_CHANNEL_TX_DIVRESET_TIME               (5'b00001),
  .GTHE3_CHANNEL_TX_DRIVE_MODE                  ("DIRECT"),
  .GTHE3_CHANNEL_TX_EIDLE_ASSERT_DELAY          (3'b100),
  .GTHE3_CHANNEL_TX_EIDLE_DEASSERT_DELAY        (3'b011),
  .GTHE3_CHANNEL_TX_EML_PHI_TUNE                (1'b0),
  .GTHE3_CHANNEL_TX_FABINT_USRCLK_FLOP          (1'b0),
  .GTHE3_CHANNEL_TX_IDLE_DATA_ZERO              (1'b0),
  .GTHE3_CHANNEL_TX_INT_DATAWIDTH               (1),
  .GTHE3_CHANNEL_TX_LOOPBACK_DRIVE_HIZ          ("FALSE"),
  .GTHE3_CHANNEL_TX_MAINCURSOR_SEL              (1'b0),
  .GTHE3_CHANNEL_TX_MARGIN_FULL_0               (7'b1001111),
  .GTHE3_CHANNEL_TX_MARGIN_FULL_1               (7'b1001110),
  .GTHE3_CHANNEL_TX_MARGIN_FULL_2               (7'b1001100),
  .GTHE3_CHANNEL_TX_MARGIN_FULL_3               (7'b1001010),
  .GTHE3_CHANNEL_TX_MARGIN_FULL_4               (7'b1001000),
  .GTHE3_CHANNEL_TX_MARGIN_LOW_0                (7'b1000110),
  .GTHE3_CHANNEL_TX_MARGIN_LOW_1                (7'b1000101),
  .GTHE3_CHANNEL_TX_MARGIN_LOW_2                (7'b1000011),
  .GTHE3_CHANNEL_TX_MARGIN_LOW_3                (7'b1000010),
  .GTHE3_CHANNEL_TX_MARGIN_LOW_4                (7'b1000000),
  .GTHE3_CHANNEL_TX_MODE_SEL                    (3'b000),
  .GTHE3_CHANNEL_TX_PMADATA_OPT                 (1'b0),
  .GTHE3_CHANNEL_TX_PMA_POWER_SAVE              (1'b0),
  .GTHE3_CHANNEL_TX_PROGCLK_SEL                 ("PREPI"),
  .GTHE3_CHANNEL_TX_PROGDIV_CFG                 (33.0),
  .GTHE3_CHANNEL_TX_QPI_STATUS_EN               (1'b0),
  .GTHE3_CHANNEL_TX_RXDETECT_CFG                (14'b00000000110010),
  .GTHE3_CHANNEL_TX_RXDETECT_REF                (3'b100),
  .GTHE3_CHANNEL_TX_SAMPLE_PERIOD               (3'b111),
  .GTHE3_CHANNEL_TX_SARC_LPBK_ENB               (1'b0),
  .GTHE3_CHANNEL_TX_XCLK_SEL                    ("TXOUT"),
  .GTHE3_CHANNEL_USE_PCS_CLK_PHASE_SEL          (1'b0),
  .GTHE3_CHANNEL_WB_MODE                        (2'b00),
  .NUM_CHANNELS                                 (NUM_CHANNELS)
) channel_inst (

  // inputs
  .GTHE3_CHANNEL_CFGRESET                       (GTHE3_CHANNEL_CFGRESET),
  .GTHE3_CHANNEL_CLKRSVD0                       (GTHE3_CHANNEL_CLKRSVD0),
  .GTHE3_CHANNEL_CLKRSVD1                       (GTHE3_CHANNEL_CLKRSVD1),
  .GTHE3_CHANNEL_CPLLLOCKDETCLK                 (GTHE3_CHANNEL_CPLLLOCKDETCLK),
  .GTHE3_CHANNEL_CPLLLOCKEN                     (GTHE3_CHANNEL_CPLLLOCKEN),
  .GTHE3_CHANNEL_CPLLPD                         (GTHE3_CHANNEL_CPLLPD),
  .GTHE3_CHANNEL_CPLLREFCLKSEL                  (GTHE3_CHANNEL_CPLLREFCLKSEL),
  .GTHE3_CHANNEL_CPLLRESET                      (GTHE3_CHANNEL_CPLLRESET),
  .GTHE3_CHANNEL_DMONFIFORESET                  (GTHE3_CHANNEL_DMONFIFORESET),
  .GTHE3_CHANNEL_DMONITORCLK                    (GTHE3_CHANNEL_DMONITORCLK),
  .GTHE3_CHANNEL_DRPADDR                        (GTHE3_CHANNEL_DRPADDR),
  .GTHE3_CHANNEL_DRPCLK                         (GTHE3_CHANNEL_DRPCLK),
  .GTHE3_CHANNEL_DRPDI                          (GTHE3_CHANNEL_DRPDI),
  .GTHE3_CHANNEL_DRPEN                          (GTHE3_CHANNEL_DRPEN),
  .GTHE3_CHANNEL_DRPWE                          (GTHE3_CHANNEL_DRPWE),
  .GTHE3_CHANNEL_EVODDPHICALDONE                (GTHE3_CHANNEL_EVODDPHICALDONE),
  .GTHE3_CHANNEL_EVODDPHICALSTART               (GTHE3_CHANNEL_EVODDPHICALSTART),
  .GTHE3_CHANNEL_EVODDPHIDRDEN                  (GTHE3_CHANNEL_EVODDPHIDRDEN),
  .GTHE3_CHANNEL_EVODDPHIDWREN                  (GTHE3_CHANNEL_EVODDPHIDWREN),
  .GTHE3_CHANNEL_EVODDPHIXRDEN                  (GTHE3_CHANNEL_EVODDPHIXRDEN),
  .GTHE3_CHANNEL_EVODDPHIXWREN                  (GTHE3_CHANNEL_EVODDPHIXWREN),
  .GTHE3_CHANNEL_EYESCANMODE                    (GTHE3_CHANNEL_EYESCANMODE),
  .GTHE3_CHANNEL_EYESCANRESET                   (GTHE3_CHANNEL_EYESCANRESET),
  .GTHE3_CHANNEL_EYESCANTRIGGER                 (GTHE3_CHANNEL_EYESCANTRIGGER),
  .GTHE3_CHANNEL_GTGREFCLK                      (GTHE3_CHANNEL_GTGREFCLK),
  .GTHE3_CHANNEL_GTHRXN                         (GTHE3_CHANNEL_GTHRXN),
  .GTHE3_CHANNEL_GTHRXP                         (GTHE3_CHANNEL_GTHRXP),
  .GTHE3_CHANNEL_GTNORTHREFCLK0                 (GTHE3_CHANNEL_GTNORTHREFCLK0),
  .GTHE3_CHANNEL_GTNORTHREFCLK1                 (GTHE3_CHANNEL_GTNORTHREFCLK1),
  .GTHE3_CHANNEL_GTREFCLK0                      (GTHE3_CHANNEL_GTREFCLK0),
  .GTHE3_CHANNEL_GTREFCLK1                      (GTHE3_CHANNEL_GTREFCLK1),
  .GTHE3_CHANNEL_GTRESETSEL                     (GTHE3_CHANNEL_GTRESETSEL),
  .GTHE3_CHANNEL_GTRSVD                         (GTHE3_CHANNEL_GTRSVD),
  .GTHE3_CHANNEL_GTRXRESET                      (GTHE3_CHANNEL_GTRXRESET),
  .GTHE3_CHANNEL_GTSOUTHREFCLK0                 (GTHE3_CHANNEL_GTSOUTHREFCLK0),
  .GTHE3_CHANNEL_GTSOUTHREFCLK1                 (GTHE3_CHANNEL_GTSOUTHREFCLK1),
  .GTHE3_CHANNEL_GTTXRESET                      (GTHE3_CHANNEL_GTTXRESET),
  .GTHE3_CHANNEL_LOOPBACK                       (GTHE3_CHANNEL_LOOPBACK),
  .GTHE3_CHANNEL_LPBKRXTXSEREN                  (GTHE3_CHANNEL_LPBKRXTXSEREN),
  .GTHE3_CHANNEL_LPBKTXRXSEREN                  (GTHE3_CHANNEL_LPBKTXRXSEREN),
  .GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE            (GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE),
  .GTHE3_CHANNEL_PCIERSTIDLE                    (GTHE3_CHANNEL_PCIERSTIDLE),
  .GTHE3_CHANNEL_PCIERSTTXSYNCSTART             (GTHE3_CHANNEL_PCIERSTTXSYNCSTART),
  .GTHE3_CHANNEL_PCIEUSERRATEDONE               (GTHE3_CHANNEL_PCIEUSERRATEDONE),
  .GTHE3_CHANNEL_PCSRSVDIN                      (GTHE3_CHANNEL_PCSRSVDIN),
  .GTHE3_CHANNEL_PCSRSVDIN2                     (GTHE3_CHANNEL_PCSRSVDIN2),
  .GTHE3_CHANNEL_PMARSVDIN                      (GTHE3_CHANNEL_PMARSVDIN),
  .GTHE3_CHANNEL_QPLL0CLK                       (GTHE3_CHANNEL_QPLL0CLK),
  .GTHE3_CHANNEL_QPLL0REFCLK                    (GTHE3_CHANNEL_QPLL0REFCLK),
  .GTHE3_CHANNEL_QPLL1CLK                       (GTHE3_CHANNEL_QPLL1CLK),
  .GTHE3_CHANNEL_QPLL1REFCLK                    (GTHE3_CHANNEL_QPLL1REFCLK),
  .GTHE3_CHANNEL_RESETOVRD                      (GTHE3_CHANNEL_RESETOVRD),
  .GTHE3_CHANNEL_RSTCLKENTX                     (GTHE3_CHANNEL_RSTCLKENTX),
  .GTHE3_CHANNEL_RX8B10BEN                      (GTHE3_CHANNEL_RX8B10BEN),
  .GTHE3_CHANNEL_RXBUFRESET                     (GTHE3_CHANNEL_RXBUFRESET),
  .GTHE3_CHANNEL_RXCDRFREQRESET                 (GTHE3_CHANNEL_RXCDRFREQRESET),
  .GTHE3_CHANNEL_RXCDRHOLD                      (GTHE3_CHANNEL_RXCDRHOLD),
  .GTHE3_CHANNEL_RXCDROVRDEN                    (GTHE3_CHANNEL_RXCDROVRDEN),
  .GTHE3_CHANNEL_RXCDRRESET                     (GTHE3_CHANNEL_RXCDRRESET),
  .GTHE3_CHANNEL_RXCDRRESETRSV                  (GTHE3_CHANNEL_RXCDRRESETRSV),
  .GTHE3_CHANNEL_RXCHBONDEN                     (GTHE3_CHANNEL_RXCHBONDEN),
  .GTHE3_CHANNEL_RXCHBONDI                      (GTHE3_CHANNEL_RXCHBONDI),
  .GTHE3_CHANNEL_RXCHBONDLEVEL                  (GTHE3_CHANNEL_RXCHBONDLEVEL),
  .GTHE3_CHANNEL_RXCHBONDMASTER                 (GTHE3_CHANNEL_RXCHBONDMASTER),
  .GTHE3_CHANNEL_RXCHBONDSLAVE                  (GTHE3_CHANNEL_RXCHBONDSLAVE),
  .GTHE3_CHANNEL_RXCOMMADETEN                   (GTHE3_CHANNEL_RXCOMMADETEN),
  .GTHE3_CHANNEL_RXDFEAGCCTRL                   (GTHE3_CHANNEL_RXDFEAGCCTRL),
  .GTHE3_CHANNEL_RXDFEAGCHOLD                   (GTHE3_CHANNEL_RXDFEAGCHOLD),
  .GTHE3_CHANNEL_RXDFEAGCOVRDEN                 (GTHE3_CHANNEL_RXDFEAGCOVRDEN),
  .GTHE3_CHANNEL_RXDFELFHOLD                    (GTHE3_CHANNEL_RXDFELFHOLD),
  .GTHE3_CHANNEL_RXDFELFOVRDEN                  (GTHE3_CHANNEL_RXDFELFOVRDEN),
  .GTHE3_CHANNEL_RXDFELPMRESET                  (GTHE3_CHANNEL_RXDFELPMRESET),
  .GTHE3_CHANNEL_RXDFETAP10HOLD                 (GTHE3_CHANNEL_RXDFETAP10HOLD),
  .GTHE3_CHANNEL_RXDFETAP10OVRDEN               (GTHE3_CHANNEL_RXDFETAP10OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP11HOLD                 (GTHE3_CHANNEL_RXDFETAP11HOLD),
  .GTHE3_CHANNEL_RXDFETAP11OVRDEN               (GTHE3_CHANNEL_RXDFETAP11OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP12HOLD                 (GTHE3_CHANNEL_RXDFETAP12HOLD),
  .GTHE3_CHANNEL_RXDFETAP12OVRDEN               (GTHE3_CHANNEL_RXDFETAP12OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP13HOLD                 (GTHE3_CHANNEL_RXDFETAP13HOLD),
  .GTHE3_CHANNEL_RXDFETAP13OVRDEN               (GTHE3_CHANNEL_RXDFETAP13OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP14HOLD                 (GTHE3_CHANNEL_RXDFETAP14HOLD),
  .GTHE3_CHANNEL_RXDFETAP14OVRDEN               (GTHE3_CHANNEL_RXDFETAP14OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP15HOLD                 (GTHE3_CHANNEL_RXDFETAP15HOLD),
  .GTHE3_CHANNEL_RXDFETAP15OVRDEN               (GTHE3_CHANNEL_RXDFETAP15OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP2HOLD                  (GTHE3_CHANNEL_RXDFETAP2HOLD),
  .GTHE3_CHANNEL_RXDFETAP2OVRDEN                (GTHE3_CHANNEL_RXDFETAP2OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP3HOLD                  (GTHE3_CHANNEL_RXDFETAP3HOLD),
  .GTHE3_CHANNEL_RXDFETAP3OVRDEN                (GTHE3_CHANNEL_RXDFETAP3OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP4HOLD                  (GTHE3_CHANNEL_RXDFETAP4HOLD),
  .GTHE3_CHANNEL_RXDFETAP4OVRDEN                (GTHE3_CHANNEL_RXDFETAP4OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP5HOLD                  (GTHE3_CHANNEL_RXDFETAP5HOLD),
  .GTHE3_CHANNEL_RXDFETAP5OVRDEN                (GTHE3_CHANNEL_RXDFETAP5OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP6HOLD                  (GTHE3_CHANNEL_RXDFETAP6HOLD),
  .GTHE3_CHANNEL_RXDFETAP6OVRDEN                (GTHE3_CHANNEL_RXDFETAP6OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP7HOLD                  (GTHE3_CHANNEL_RXDFETAP7HOLD),
  .GTHE3_CHANNEL_RXDFETAP7OVRDEN                (GTHE3_CHANNEL_RXDFETAP7OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP8HOLD                  (GTHE3_CHANNEL_RXDFETAP8HOLD),
  .GTHE3_CHANNEL_RXDFETAP8OVRDEN                (GTHE3_CHANNEL_RXDFETAP8OVRDEN),
  .GTHE3_CHANNEL_RXDFETAP9HOLD                  (GTHE3_CHANNEL_RXDFETAP9HOLD),
  .GTHE3_CHANNEL_RXDFETAP9OVRDEN                (GTHE3_CHANNEL_RXDFETAP9OVRDEN),
  .GTHE3_CHANNEL_RXDFEUTHOLD                    (GTHE3_CHANNEL_RXDFEUTHOLD),
  .GTHE3_CHANNEL_RXDFEUTOVRDEN                  (GTHE3_CHANNEL_RXDFEUTOVRDEN),
  .GTHE3_CHANNEL_RXDFEVPHOLD                    (GTHE3_CHANNEL_RXDFEVPHOLD),
  .GTHE3_CHANNEL_RXDFEVPOVRDEN                  (GTHE3_CHANNEL_RXDFEVPOVRDEN),
  .GTHE3_CHANNEL_RXDFEVSEN                      (GTHE3_CHANNEL_RXDFEVSEN),
  .GTHE3_CHANNEL_RXDFEXYDEN                     (GTHE3_CHANNEL_RXDFEXYDEN),
  .GTHE3_CHANNEL_RXDLYBYPASS                    (GTHE3_CHANNEL_RXDLYBYPASS),
  .GTHE3_CHANNEL_RXDLYEN                        (GTHE3_CHANNEL_RXDLYEN),
  .GTHE3_CHANNEL_RXDLYOVRDEN                    (GTHE3_CHANNEL_RXDLYOVRDEN),
  .GTHE3_CHANNEL_RXDLYSRESET                    (GTHE3_CHANNEL_RXDLYSRESET),
  .GTHE3_CHANNEL_RXELECIDLEMODE                 (GTHE3_CHANNEL_RXELECIDLEMODE),
  .GTHE3_CHANNEL_RXGEARBOXSLIP                  (GTHE3_CHANNEL_RXGEARBOXSLIP),
  .GTHE3_CHANNEL_RXLATCLK                       (GTHE3_CHANNEL_RXLATCLK),
  .GTHE3_CHANNEL_RXLPMEN                        (GTHE3_CHANNEL_RXLPMEN),
  .GTHE3_CHANNEL_RXLPMGCHOLD                    (GTHE3_CHANNEL_RXLPMGCHOLD),
  .GTHE3_CHANNEL_RXLPMGCOVRDEN                  (GTHE3_CHANNEL_RXLPMGCOVRDEN),
  .GTHE3_CHANNEL_RXLPMHFHOLD                    (GTHE3_CHANNEL_RXLPMHFHOLD),
  .GTHE3_CHANNEL_RXLPMHFOVRDEN                  (GTHE3_CHANNEL_RXLPMHFOVRDEN),
  .GTHE3_CHANNEL_RXLPMLFHOLD                    (GTHE3_CHANNEL_RXLPMLFHOLD),
  .GTHE3_CHANNEL_RXLPMLFKLOVRDEN                (GTHE3_CHANNEL_RXLPMLFKLOVRDEN),
  .GTHE3_CHANNEL_RXLPMOSHOLD                    (GTHE3_CHANNEL_RXLPMOSHOLD),
  .GTHE3_CHANNEL_RXLPMOSOVRDEN                  (GTHE3_CHANNEL_RXLPMOSOVRDEN),
  .GTHE3_CHANNEL_RXMCOMMAALIGNEN                (GTHE3_CHANNEL_RXMCOMMAALIGNEN),
  .GTHE3_CHANNEL_RXMONITORSEL                   (GTHE3_CHANNEL_RXMONITORSEL),
  .GTHE3_CHANNEL_RXOOBRESET                     (GTHE3_CHANNEL_RXOOBRESET),
  .GTHE3_CHANNEL_RXOSCALRESET                   (GTHE3_CHANNEL_RXOSCALRESET),
  .GTHE3_CHANNEL_RXOSHOLD                       (GTHE3_CHANNEL_RXOSHOLD),
  .GTHE3_CHANNEL_RXOSINTCFG                     (GTHE3_CHANNEL_RXOSINTCFG),
  .GTHE3_CHANNEL_RXOSINTEN                      (GTHE3_CHANNEL_RXOSINTEN),
  .GTHE3_CHANNEL_RXOSINTHOLD                    (GTHE3_CHANNEL_RXOSINTHOLD),
  .GTHE3_CHANNEL_RXOSINTOVRDEN                  (GTHE3_CHANNEL_RXOSINTOVRDEN),
  .GTHE3_CHANNEL_RXOSINTSTROBE                  (GTHE3_CHANNEL_RXOSINTSTROBE),
  .GTHE3_CHANNEL_RXOSINTTESTOVRDEN              (GTHE3_CHANNEL_RXOSINTTESTOVRDEN),
  .GTHE3_CHANNEL_RXOSOVRDEN                     (GTHE3_CHANNEL_RXOSOVRDEN),
  .GTHE3_CHANNEL_RXOUTCLKSEL                    (GTHE3_CHANNEL_RXOUTCLKSEL),
  .GTHE3_CHANNEL_RXPCOMMAALIGNEN                (GTHE3_CHANNEL_RXPCOMMAALIGNEN),
  .GTHE3_CHANNEL_RXPCSRESET                     (GTHE3_CHANNEL_RXPCSRESET),
  .GTHE3_CHANNEL_RXPD                           (GTHE3_CHANNEL_RXPD),
  .GTHE3_CHANNEL_RXPHALIGN                      (GTHE3_CHANNEL_RXPHALIGN),
  .GTHE3_CHANNEL_RXPHALIGNEN                    (GTHE3_CHANNEL_RXPHALIGNEN),
  .GTHE3_CHANNEL_RXPHDLYPD                      (GTHE3_CHANNEL_RXPHDLYPD),
  .GTHE3_CHANNEL_RXPHDLYRESET                   (GTHE3_CHANNEL_RXPHDLYRESET),
  .GTHE3_CHANNEL_RXPHOVRDEN                     (GTHE3_CHANNEL_RXPHOVRDEN),
  .GTHE3_CHANNEL_RXPLLCLKSEL                    (GTHE3_CHANNEL_RXPLLCLKSEL),
  .GTHE3_CHANNEL_RXPMARESET                     (GTHE3_CHANNEL_RXPMARESET),
  .GTHE3_CHANNEL_RXPOLARITY                     (GTHE3_CHANNEL_RXPOLARITY),
  .GTHE3_CHANNEL_RXPRBSCNTRESET                 (GTHE3_CHANNEL_RXPRBSCNTRESET),
  .GTHE3_CHANNEL_RXPRBSSEL                      (GTHE3_CHANNEL_RXPRBSSEL),
  .GTHE3_CHANNEL_RXPROGDIVRESET                 (GTHE3_CHANNEL_RXPROGDIVRESET),
  .GTHE3_CHANNEL_RXQPIEN                        (GTHE3_CHANNEL_RXQPIEN),
  .GTHE3_CHANNEL_RXRATE                         (GTHE3_CHANNEL_RXRATE),
  .GTHE3_CHANNEL_RXRATEMODE                     (GTHE3_CHANNEL_RXRATEMODE),
  .GTHE3_CHANNEL_RXSLIDE                        (GTHE3_CHANNEL_RXSLIDE),
  .GTHE3_CHANNEL_RXSLIPOUTCLK                   (GTHE3_CHANNEL_RXSLIPOUTCLK),
  .GTHE3_CHANNEL_RXSLIPPMA                      (GTHE3_CHANNEL_RXSLIPPMA),
  .GTHE3_CHANNEL_RXSYNCALLIN                    (GTHE3_CHANNEL_RXSYNCALLIN),
  .GTHE3_CHANNEL_RXSYNCIN                       (GTHE3_CHANNEL_RXSYNCIN),
  .GTHE3_CHANNEL_RXSYNCMODE                     (GTHE3_CHANNEL_RXSYNCMODE),
  .GTHE3_CHANNEL_RXSYSCLKSEL                    (GTHE3_CHANNEL_RXSYSCLKSEL),
  .GTHE3_CHANNEL_RXUSERRDY                      (GTHE3_CHANNEL_RXUSERRDY),
  .GTHE3_CHANNEL_RXUSRCLK                       (GTHE3_CHANNEL_RXUSRCLK),
  .GTHE3_CHANNEL_RXUSRCLK2                      (GTHE3_CHANNEL_RXUSRCLK2),
  .GTHE3_CHANNEL_SIGVALIDCLK                    (GTHE3_CHANNEL_SIGVALIDCLK),
  .GTHE3_CHANNEL_TSTIN                          (GTHE3_CHANNEL_TSTIN),
  .GTHE3_CHANNEL_TX8B10BBYPASS                  (GTHE3_CHANNEL_TX8B10BBYPASS),
  .GTHE3_CHANNEL_TX8B10BEN                      (GTHE3_CHANNEL_TX8B10BEN),
  .GTHE3_CHANNEL_TXBUFDIFFCTRL                  (GTHE3_CHANNEL_TXBUFDIFFCTRL),
  .GTHE3_CHANNEL_TXCOMINIT                      (GTHE3_CHANNEL_TXCOMINIT),
  .GTHE3_CHANNEL_TXCOMSAS                       (GTHE3_CHANNEL_TXCOMSAS),
  .GTHE3_CHANNEL_TXCOMWAKE                      (GTHE3_CHANNEL_TXCOMWAKE),
  .GTHE3_CHANNEL_TXCTRL0                        (GTHE3_CHANNEL_TXCTRL0),
  .GTHE3_CHANNEL_TXCTRL1                        (GTHE3_CHANNEL_TXCTRL1),
  .GTHE3_CHANNEL_TXCTRL2                        (GTHE3_CHANNEL_TXCTRL2),
  .GTHE3_CHANNEL_TXDATA                         (GTHE3_CHANNEL_TXDATA),
  .GTHE3_CHANNEL_TXDATAEXTENDRSVD               (GTHE3_CHANNEL_TXDATAEXTENDRSVD),
  .GTHE3_CHANNEL_TXDEEMPH                       (GTHE3_CHANNEL_TXDEEMPH),
  .GTHE3_CHANNEL_TXDETECTRX                     (GTHE3_CHANNEL_TXDETECTRX),
  .GTHE3_CHANNEL_TXDIFFCTRL                     (GTHE3_CHANNEL_TXDIFFCTRL),
  .GTHE3_CHANNEL_TXDIFFPD                       (GTHE3_CHANNEL_TXDIFFPD),
  .GTHE3_CHANNEL_TXDLYBYPASS                    (GTHE3_CHANNEL_TXDLYBYPASS),
  .GTHE3_CHANNEL_TXDLYEN                        (GTHE3_CHANNEL_TXDLYEN),
  .GTHE3_CHANNEL_TXDLYHOLD                      (GTHE3_CHANNEL_TXDLYHOLD),
  .GTHE3_CHANNEL_TXDLYOVRDEN                    (GTHE3_CHANNEL_TXDLYOVRDEN),
  .GTHE3_CHANNEL_TXDLYSRESET                    (GTHE3_CHANNEL_TXDLYSRESET),
  .GTHE3_CHANNEL_TXDLYUPDOWN                    (GTHE3_CHANNEL_TXDLYUPDOWN),
  .GTHE3_CHANNEL_TXELECIDLE                     (GTHE3_CHANNEL_TXELECIDLE),
  .GTHE3_CHANNEL_TXHEADER                       (GTHE3_CHANNEL_TXHEADER),
  .GTHE3_CHANNEL_TXINHIBIT                      (GTHE3_CHANNEL_TXINHIBIT),
  .GTHE3_CHANNEL_TXLATCLK                       (GTHE3_CHANNEL_TXLATCLK),
  .GTHE3_CHANNEL_TXMAINCURSOR                   (GTHE3_CHANNEL_TXMAINCURSOR),
  .GTHE3_CHANNEL_TXMARGIN                       (GTHE3_CHANNEL_TXMARGIN),
  .GTHE3_CHANNEL_TXOUTCLKSEL                    (GTHE3_CHANNEL_TXOUTCLKSEL),
  .GTHE3_CHANNEL_TXPCSRESET                     (GTHE3_CHANNEL_TXPCSRESET),
  .GTHE3_CHANNEL_TXPD                           (GTHE3_CHANNEL_TXPD),
  .GTHE3_CHANNEL_TXPDELECIDLEMODE               (GTHE3_CHANNEL_TXPDELECIDLEMODE),
  .GTHE3_CHANNEL_TXPHALIGN                      (GTHE3_CHANNEL_TXPHALIGN),
  .GTHE3_CHANNEL_TXPHALIGNEN                    (GTHE3_CHANNEL_TXPHALIGNEN),
  .GTHE3_CHANNEL_TXPHDLYPD                      (GTHE3_CHANNEL_TXPHDLYPD),
  .GTHE3_CHANNEL_TXPHDLYRESET                   (GTHE3_CHANNEL_TXPHDLYRESET),
  .GTHE3_CHANNEL_TXPHDLYTSTCLK                  (GTHE3_CHANNEL_TXPHDLYTSTCLK),
  .GTHE3_CHANNEL_TXPHINIT                       (GTHE3_CHANNEL_TXPHINIT),
  .GTHE3_CHANNEL_TXPHOVRDEN                     (GTHE3_CHANNEL_TXPHOVRDEN),
  .GTHE3_CHANNEL_TXPIPPMEN                      (GTHE3_CHANNEL_TXPIPPMEN),
  .GTHE3_CHANNEL_TXPIPPMOVRDEN                  (GTHE3_CHANNEL_TXPIPPMOVRDEN),
  .GTHE3_CHANNEL_TXPIPPMPD                      (GTHE3_CHANNEL_TXPIPPMPD),
  .GTHE3_CHANNEL_TXPIPPMSEL                     (GTHE3_CHANNEL_TXPIPPMSEL),
  .GTHE3_CHANNEL_TXPIPPMSTEPSIZE                (GTHE3_CHANNEL_TXPIPPMSTEPSIZE),
  .GTHE3_CHANNEL_TXPISOPD                       (GTHE3_CHANNEL_TXPISOPD),
  .GTHE3_CHANNEL_TXPLLCLKSEL                    (GTHE3_CHANNEL_TXPLLCLKSEL),
  .GTHE3_CHANNEL_TXPMARESET                     (GTHE3_CHANNEL_TXPMARESET),
  .GTHE3_CHANNEL_TXPOLARITY                     (GTHE3_CHANNEL_TXPOLARITY),
  .GTHE3_CHANNEL_TXPOSTCURSOR                   (GTHE3_CHANNEL_TXPOSTCURSOR),
  .GTHE3_CHANNEL_TXPOSTCURSORINV                (GTHE3_CHANNEL_TXPOSTCURSORINV),
  .GTHE3_CHANNEL_TXPRBSFORCEERR                 (GTHE3_CHANNEL_TXPRBSFORCEERR),
  .GTHE3_CHANNEL_TXPRBSSEL                      (GTHE3_CHANNEL_TXPRBSSEL),
  .GTHE3_CHANNEL_TXPRECURSOR                    (GTHE3_CHANNEL_TXPRECURSOR),
  .GTHE3_CHANNEL_TXPRECURSORINV                 (GTHE3_CHANNEL_TXPRECURSORINV),
  .GTHE3_CHANNEL_TXPROGDIVRESET                 (GTHE3_CHANNEL_TXPROGDIVRESET),
  .GTHE3_CHANNEL_TXQPIBIASEN                    (GTHE3_CHANNEL_TXQPIBIASEN),
  .GTHE3_CHANNEL_TXQPISTRONGPDOWN               (GTHE3_CHANNEL_TXQPISTRONGPDOWN),
  .GTHE3_CHANNEL_TXQPIWEAKPUP                   (GTHE3_CHANNEL_TXQPIWEAKPUP),
  .GTHE3_CHANNEL_TXRATE                         (GTHE3_CHANNEL_TXRATE),
  .GTHE3_CHANNEL_TXRATEMODE                     (GTHE3_CHANNEL_TXRATEMODE),
  .GTHE3_CHANNEL_TXSEQUENCE                     (GTHE3_CHANNEL_TXSEQUENCE),
  .GTHE3_CHANNEL_TXSWING                        (GTHE3_CHANNEL_TXSWING),
  .GTHE3_CHANNEL_TXSYNCALLIN                    (GTHE3_CHANNEL_TXSYNCALLIN),
  .GTHE3_CHANNEL_TXSYNCIN                       (GTHE3_CHANNEL_TXSYNCIN),
  .GTHE3_CHANNEL_TXSYNCMODE                     (GTHE3_CHANNEL_TXSYNCMODE),
  .GTHE3_CHANNEL_TXSYSCLKSEL                    (GTHE3_CHANNEL_TXSYSCLKSEL),
  .GTHE3_CHANNEL_TXUSERRDY                      (GTHE3_CHANNEL_TXUSERRDY),
  .GTHE3_CHANNEL_TXUSRCLK                       (GTHE3_CHANNEL_TXUSRCLK),
  .GTHE3_CHANNEL_TXUSRCLK2                      (GTHE3_CHANNEL_TXUSRCLK2),

  // outputs
  .GTHE3_CHANNEL_BUFGTCE                        (GTHE3_CHANNEL_BUFGTCE),
  .GTHE3_CHANNEL_BUFGTCEMASK                    (GTHE3_CHANNEL_BUFGTCEMASK),
  .GTHE3_CHANNEL_BUFGTDIV                       (GTHE3_CHANNEL_BUFGTDIV),
  .GTHE3_CHANNEL_BUFGTRESET                     (GTHE3_CHANNEL_BUFGTRESET),
  .GTHE3_CHANNEL_BUFGTRSTMASK                   (GTHE3_CHANNEL_BUFGTRSTMASK),
  .GTHE3_CHANNEL_CPLLFBCLKLOST                  (GTHE3_CHANNEL_CPLLFBCLKLOST),
  .GTHE3_CHANNEL_CPLLLOCK                       (GTHE3_CHANNEL_CPLLLOCK),
  .GTHE3_CHANNEL_CPLLREFCLKLOST                 (GTHE3_CHANNEL_CPLLREFCLKLOST),
  .GTHE3_CHANNEL_DMONITOROUT                    (GTHE3_CHANNEL_DMONITOROUT),
  .GTHE3_CHANNEL_DRPDO                          (GTHE3_CHANNEL_DRPDO),
  .GTHE3_CHANNEL_DRPRDY                         (GTHE3_CHANNEL_DRPRDY),
  .GTHE3_CHANNEL_EYESCANDATAERROR               (GTHE3_CHANNEL_EYESCANDATAERROR),
  .GTHE3_CHANNEL_GTHTXN                         (GTHE3_CHANNEL_GTHTXN),
  .GTHE3_CHANNEL_GTHTXP                         (GTHE3_CHANNEL_GTHTXP),
  .GTHE3_CHANNEL_GTPOWERGOOD                    (GTHE3_CHANNEL_GTPOWERGOOD),
  .GTHE3_CHANNEL_GTREFCLKMONITOR                (GTHE3_CHANNEL_GTREFCLKMONITOR),
  .GTHE3_CHANNEL_PCIERATEGEN3                   (GTHE3_CHANNEL_PCIERATEGEN3),
  .GTHE3_CHANNEL_PCIERATEIDLE                   (GTHE3_CHANNEL_PCIERATEIDLE),
  .GTHE3_CHANNEL_PCIERATEQPLLPD                 (GTHE3_CHANNEL_PCIERATEQPLLPD),
  .GTHE3_CHANNEL_PCIERATEQPLLRESET              (GTHE3_CHANNEL_PCIERATEQPLLRESET),
  .GTHE3_CHANNEL_PCIESYNCTXSYNCDONE             (GTHE3_CHANNEL_PCIESYNCTXSYNCDONE),
  .GTHE3_CHANNEL_PCIEUSERGEN3RDY                (GTHE3_CHANNEL_PCIEUSERGEN3RDY),
  .GTHE3_CHANNEL_PCIEUSERPHYSTATUSRST           (GTHE3_CHANNEL_PCIEUSERPHYSTATUSRST),
  .GTHE3_CHANNEL_PCIEUSERRATESTART              (GTHE3_CHANNEL_PCIEUSERRATESTART),
  .GTHE3_CHANNEL_PCSRSVDOUT                     (GTHE3_CHANNEL_PCSRSVDOUT),
  .GTHE3_CHANNEL_PHYSTATUS                      (GTHE3_CHANNEL_PHYSTATUS),
  .GTHE3_CHANNEL_PINRSRVDAS                     (GTHE3_CHANNEL_PINRSRVDAS),
  .GTHE3_CHANNEL_RESETEXCEPTION                 (GTHE3_CHANNEL_RESETEXCEPTION),
  .GTHE3_CHANNEL_RXBUFSTATUS                    (GTHE3_CHANNEL_RXBUFSTATUS),
  .GTHE3_CHANNEL_RXBYTEISALIGNED                (GTHE3_CHANNEL_RXBYTEISALIGNED),
  .GTHE3_CHANNEL_RXBYTEREALIGN                  (GTHE3_CHANNEL_RXBYTEREALIGN),
  .GTHE3_CHANNEL_RXCDRLOCK                      (GTHE3_CHANNEL_RXCDRLOCK),
  .GTHE3_CHANNEL_RXCDRPHDONE                    (GTHE3_CHANNEL_RXCDRPHDONE),
  .GTHE3_CHANNEL_RXCHANBONDSEQ                  (GTHE3_CHANNEL_RXCHANBONDSEQ),
  .GTHE3_CHANNEL_RXCHANISALIGNED                (GTHE3_CHANNEL_RXCHANISALIGNED),
  .GTHE3_CHANNEL_RXCHANREALIGN                  (GTHE3_CHANNEL_RXCHANREALIGN),
  .GTHE3_CHANNEL_RXCHBONDO                      (GTHE3_CHANNEL_RXCHBONDO),
  .GTHE3_CHANNEL_RXCLKCORCNT                    (GTHE3_CHANNEL_RXCLKCORCNT),
  .GTHE3_CHANNEL_RXCOMINITDET                   (GTHE3_CHANNEL_RXCOMINITDET),
  .GTHE3_CHANNEL_RXCOMMADET                     (GTHE3_CHANNEL_RXCOMMADET),
  .GTHE3_CHANNEL_RXCOMSASDET                    (GTHE3_CHANNEL_RXCOMSASDET),
  .GTHE3_CHANNEL_RXCOMWAKEDET                   (GTHE3_CHANNEL_RXCOMWAKEDET),
  .GTHE3_CHANNEL_RXCTRL0                        (GTHE3_CHANNEL_RXCTRL0),
  .GTHE3_CHANNEL_RXCTRL1                        (GTHE3_CHANNEL_RXCTRL1),
  .GTHE3_CHANNEL_RXCTRL2                        (GTHE3_CHANNEL_RXCTRL2),
  .GTHE3_CHANNEL_RXCTRL3                        (GTHE3_CHANNEL_RXCTRL3),
  .GTHE3_CHANNEL_RXDATA                         (GTHE3_CHANNEL_RXDATA),
  .GTHE3_CHANNEL_RXDATAEXTENDRSVD               (GTHE3_CHANNEL_RXDATAEXTENDRSVD),
  .GTHE3_CHANNEL_RXDATAVALID                    (GTHE3_CHANNEL_RXDATAVALID),
  .GTHE3_CHANNEL_RXDLYSRESETDONE                (GTHE3_CHANNEL_RXDLYSRESETDONE),
  .GTHE3_CHANNEL_RXELECIDLE                     (GTHE3_CHANNEL_RXELECIDLE),
  .GTHE3_CHANNEL_RXHEADER                       (GTHE3_CHANNEL_RXHEADER),
  .GTHE3_CHANNEL_RXHEADERVALID                  (GTHE3_CHANNEL_RXHEADERVALID),
  .GTHE3_CHANNEL_RXMONITOROUT                   (GTHE3_CHANNEL_RXMONITOROUT),
  .GTHE3_CHANNEL_RXOSINTDONE                    (GTHE3_CHANNEL_RXOSINTDONE),
  .GTHE3_CHANNEL_RXOSINTSTARTED                 (GTHE3_CHANNEL_RXOSINTSTARTED),
  .GTHE3_CHANNEL_RXOSINTSTROBEDONE              (GTHE3_CHANNEL_RXOSINTSTROBEDONE),
  .GTHE3_CHANNEL_RXOSINTSTROBESTARTED           (GTHE3_CHANNEL_RXOSINTSTROBESTARTED),
  .GTHE3_CHANNEL_RXOUTCLK                       (GTHE3_CHANNEL_RXOUTCLK),
  .GTHE3_CHANNEL_RXOUTCLKFABRIC                 (GTHE3_CHANNEL_RXOUTCLKFABRIC),
  .GTHE3_CHANNEL_RXOUTCLKPCS                    (GTHE3_CHANNEL_RXOUTCLKPCS),
  .GTHE3_CHANNEL_RXPHALIGNDONE                  (GTHE3_CHANNEL_RXPHALIGNDONE),
  .GTHE3_CHANNEL_RXPHALIGNERR                   (GTHE3_CHANNEL_RXPHALIGNERR),
  .GTHE3_CHANNEL_RXPMARESETDONE                 (GTHE3_CHANNEL_RXPMARESETDONE),
  .GTHE3_CHANNEL_RXPRBSERR                      (GTHE3_CHANNEL_RXPRBSERR),
  .GTHE3_CHANNEL_RXPRBSLOCKED                   (GTHE3_CHANNEL_RXPRBSLOCKED),
  .GTHE3_CHANNEL_RXPRGDIVRESETDONE              (GTHE3_CHANNEL_RXPRGDIVRESETDONE),
  .GTHE3_CHANNEL_RXQPISENN                      (GTHE3_CHANNEL_RXQPISENN),
  .GTHE3_CHANNEL_RXQPISENP                      (GTHE3_CHANNEL_RXQPISENP),
  .GTHE3_CHANNEL_RXRATEDONE                     (GTHE3_CHANNEL_RXRATEDONE),
  .GTHE3_CHANNEL_RXRECCLKOUT                    (GTHE3_CHANNEL_RXRECCLKOUT),
  .GTHE3_CHANNEL_RXRESETDONE                    (GTHE3_CHANNEL_RXRESETDONE),
  .GTHE3_CHANNEL_RXSLIDERDY                     (GTHE3_CHANNEL_RXSLIDERDY),
  .GTHE3_CHANNEL_RXSLIPDONE                     (GTHE3_CHANNEL_RXSLIPDONE),
  .GTHE3_CHANNEL_RXSLIPOUTCLKRDY                (GTHE3_CHANNEL_RXSLIPOUTCLKRDY),
  .GTHE3_CHANNEL_RXSLIPPMARDY                   (GTHE3_CHANNEL_RXSLIPPMARDY),
  .GTHE3_CHANNEL_RXSTARTOFSEQ                   (GTHE3_CHANNEL_RXSTARTOFSEQ),
  .GTHE3_CHANNEL_RXSTATUS                       (GTHE3_CHANNEL_RXSTATUS),
  .GTHE3_CHANNEL_RXSYNCDONE                     (GTHE3_CHANNEL_RXSYNCDONE),
  .GTHE3_CHANNEL_RXSYNCOUT                      (GTHE3_CHANNEL_RXSYNCOUT),
  .GTHE3_CHANNEL_RXVALID                        (GTHE3_CHANNEL_RXVALID),
  .GTHE3_CHANNEL_TXBUFSTATUS                    (GTHE3_CHANNEL_TXBUFSTATUS),
  .GTHE3_CHANNEL_TXCOMFINISH                    (GTHE3_CHANNEL_TXCOMFINISH),
  .GTHE3_CHANNEL_TXDLYSRESETDONE                (GTHE3_CHANNEL_TXDLYSRESETDONE),
  .GTHE3_CHANNEL_TXOUTCLK                       (GTHE3_CHANNEL_TXOUTCLK),
  .GTHE3_CHANNEL_TXOUTCLKFABRIC                 (GTHE3_CHANNEL_TXOUTCLKFABRIC),
  .GTHE3_CHANNEL_TXOUTCLKPCS                    (GTHE3_CHANNEL_TXOUTCLKPCS),
  .GTHE3_CHANNEL_TXPHALIGNDONE                  (GTHE3_CHANNEL_TXPHALIGNDONE),
  .GTHE3_CHANNEL_TXPHINITDONE                   (GTHE3_CHANNEL_TXPHINITDONE),
  .GTHE3_CHANNEL_TXPMARESETDONE                 (GTHE3_CHANNEL_TXPMARESETDONE),
  .GTHE3_CHANNEL_TXPRGDIVRESETDONE              (GTHE3_CHANNEL_TXPRGDIVRESETDONE),
  .GTHE3_CHANNEL_TXQPISENN                      (GTHE3_CHANNEL_TXQPISENN),
  .GTHE3_CHANNEL_TXQPISENP                      (GTHE3_CHANNEL_TXQPISENP),
  .GTHE3_CHANNEL_TXRATEDONE                     (GTHE3_CHANNEL_TXRATEDONE),
  .GTHE3_CHANNEL_TXRESETDONE                    (GTHE3_CHANNEL_TXRESETDONE),
  .GTHE3_CHANNEL_TXSYNCDONE                     (GTHE3_CHANNEL_TXSYNCDONE),
  .GTHE3_CHANNEL_TXSYNCOUT                      (GTHE3_CHANNEL_TXSYNCOUT)
);

endmodule

module gtwizard_ultrascale_v1_6_2_gthe3_channel #(


  // -------------------------------------------------------------------------------------------------------------------
  // Parameters controlling primitive wrapper HDL generation
  // -------------------------------------------------------------------------------------------------------------------
  parameter         NUM_CHANNELS = 4,


  // -------------------------------------------------------------------------------------------------------------------
  // Parameters relating to GTHE3_CHANNEL primitive
  // -------------------------------------------------------------------------------------------------------------------

  // primitive wrapper parameters which override corresponding GTHE3_CHANNEL primitive parameters
  parameter   [0:0] GTHE3_CHANNEL_ACJTAG_DEBUG_MODE = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_ACJTAG_MODE = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_ACJTAG_RESET = 1'b0,
  parameter  [15:0] GTHE3_CHANNEL_ADAPT_CFG0 = 16'hF800,
  parameter  [15:0] GTHE3_CHANNEL_ADAPT_CFG1 = 16'h0000,
  parameter         GTHE3_CHANNEL_ALIGN_COMMA_DOUBLE = "FALSE",
  parameter   [9:0] GTHE3_CHANNEL_ALIGN_COMMA_ENABLE = 10'b0001111111,
  parameter integer GTHE3_CHANNEL_ALIGN_COMMA_WORD = 1,
  parameter         GTHE3_CHANNEL_ALIGN_MCOMMA_DET = "TRUE",
  parameter   [9:0] GTHE3_CHANNEL_ALIGN_MCOMMA_VALUE = 10'b1010000011,
  parameter         GTHE3_CHANNEL_ALIGN_PCOMMA_DET = "TRUE",
  parameter   [9:0] GTHE3_CHANNEL_ALIGN_PCOMMA_VALUE = 10'b0101111100,
  parameter   [0:0] GTHE3_CHANNEL_A_RXOSCALRESET = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_A_RXPROGDIVRESET = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_A_TXPROGDIVRESET = 1'b0,
  parameter         GTHE3_CHANNEL_CBCC_DATA_SOURCE_SEL = "DECODED",
  parameter   [0:0] GTHE3_CHANNEL_CDR_SWAP_MODE_EN = 1'b0,
  parameter         GTHE3_CHANNEL_CHAN_BOND_KEEP_ALIGN = "FALSE",
  parameter integer GTHE3_CHANNEL_CHAN_BOND_MAX_SKEW = 7,
  parameter   [9:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_1_1 = 10'b0101111100,
  parameter   [9:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_1_2 = 10'b0000000000,
  parameter   [9:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_1_3 = 10'b0000000000,
  parameter   [9:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_1_4 = 10'b0000000000,
  parameter   [3:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_1_ENABLE = 4'b1111,
  parameter   [9:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_2_1 = 10'b0100000000,
  parameter   [9:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_2_2 = 10'b0100000000,
  parameter   [9:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_2_3 = 10'b0100000000,
  parameter   [9:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_2_4 = 10'b0100000000,
  parameter   [3:0] GTHE3_CHANNEL_CHAN_BOND_SEQ_2_ENABLE = 4'b1111,
  parameter         GTHE3_CHANNEL_CHAN_BOND_SEQ_2_USE = "FALSE",
  parameter integer GTHE3_CHANNEL_CHAN_BOND_SEQ_LEN = 2,
  parameter         GTHE3_CHANNEL_CLK_CORRECT_USE = "TRUE",
  parameter         GTHE3_CHANNEL_CLK_COR_KEEP_IDLE = "FALSE",
  parameter integer GTHE3_CHANNEL_CLK_COR_MAX_LAT = 20,
  parameter integer GTHE3_CHANNEL_CLK_COR_MIN_LAT = 18,
  parameter         GTHE3_CHANNEL_CLK_COR_PRECEDENCE = "TRUE",
  parameter integer GTHE3_CHANNEL_CLK_COR_REPEAT_WAIT = 0,
  parameter   [9:0] GTHE3_CHANNEL_CLK_COR_SEQ_1_1 = 10'b0100011100,
  parameter   [9:0] GTHE3_CHANNEL_CLK_COR_SEQ_1_2 = 10'b0000000000,
  parameter   [9:0] GTHE3_CHANNEL_CLK_COR_SEQ_1_3 = 10'b0000000000,
  parameter   [9:0] GTHE3_CHANNEL_CLK_COR_SEQ_1_4 = 10'b0000000000,
  parameter   [3:0] GTHE3_CHANNEL_CLK_COR_SEQ_1_ENABLE = 4'b1111,
  parameter   [9:0] GTHE3_CHANNEL_CLK_COR_SEQ_2_1 = 10'b0100000000,
  parameter   [9:0] GTHE3_CHANNEL_CLK_COR_SEQ_2_2 = 10'b0100000000,
  parameter   [9:0] GTHE3_CHANNEL_CLK_COR_SEQ_2_3 = 10'b0100000000,
  parameter   [9:0] GTHE3_CHANNEL_CLK_COR_SEQ_2_4 = 10'b0100000000,
  parameter   [3:0] GTHE3_CHANNEL_CLK_COR_SEQ_2_ENABLE = 4'b1111,
  parameter         GTHE3_CHANNEL_CLK_COR_SEQ_2_USE = "FALSE",
  parameter integer GTHE3_CHANNEL_CLK_COR_SEQ_LEN = 2,
  parameter  [15:0] GTHE3_CHANNEL_CPLL_CFG0 = 16'h20F8,
  parameter  [15:0] GTHE3_CHANNEL_CPLL_CFG1 = 16'hA494,
  parameter  [15:0] GTHE3_CHANNEL_CPLL_CFG2 = 16'hF001,
  parameter   [5:0] GTHE3_CHANNEL_CPLL_CFG3 = 6'h00,
  parameter integer GTHE3_CHANNEL_CPLL_FBDIV = 4,
  parameter integer GTHE3_CHANNEL_CPLL_FBDIV_45 = 4,
  parameter  [15:0] GTHE3_CHANNEL_CPLL_INIT_CFG0 = 16'h001E,
  parameter   [7:0] GTHE3_CHANNEL_CPLL_INIT_CFG1 = 8'h00,
  parameter  [15:0] GTHE3_CHANNEL_CPLL_LOCK_CFG = 16'h01E8,
  parameter integer GTHE3_CHANNEL_CPLL_REFCLK_DIV = 1,
  parameter   [1:0] GTHE3_CHANNEL_DDI_CTRL = 2'b00,
  parameter integer GTHE3_CHANNEL_DDI_REALIGN_WAIT = 15,
  parameter         GTHE3_CHANNEL_DEC_MCOMMA_DETECT = "TRUE",
  parameter         GTHE3_CHANNEL_DEC_PCOMMA_DETECT = "TRUE",
  parameter         GTHE3_CHANNEL_DEC_VALID_COMMA_ONLY = "TRUE",
  parameter   [0:0] GTHE3_CHANNEL_DFE_D_X_REL_POS = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DFE_VCM_COMP_EN = 1'b0,
  parameter   [9:0] GTHE3_CHANNEL_DMONITOR_CFG0 = 10'h000,
  parameter   [7:0] GTHE3_CHANNEL_DMONITOR_CFG1 = 8'h00,
  parameter   [0:0] GTHE3_CHANNEL_ES_CLK_PHASE_SEL = 1'b0,
  parameter   [5:0] GTHE3_CHANNEL_ES_CONTROL = 6'b000000,
  parameter         GTHE3_CHANNEL_ES_ERRDET_EN = "FALSE",
  parameter         GTHE3_CHANNEL_ES_EYE_SCAN_EN = "FALSE",
  parameter  [11:0] GTHE3_CHANNEL_ES_HORZ_OFFSET = 12'h000,
  parameter   [9:0] GTHE3_CHANNEL_ES_PMA_CFG = 10'b0000000000,
  parameter   [4:0] GTHE3_CHANNEL_ES_PRESCALE = 5'b00000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUALIFIER0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUALIFIER1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUALIFIER2 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUALIFIER3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUALIFIER4 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUAL_MASK0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUAL_MASK1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUAL_MASK2 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUAL_MASK3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_QUAL_MASK4 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_SDATA_MASK0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_SDATA_MASK1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_SDATA_MASK2 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_SDATA_MASK3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_ES_SDATA_MASK4 = 16'h0000,
  parameter  [10:0] GTHE3_CHANNEL_EVODD_PHI_CFG = 11'b00000000000,
  parameter   [0:0] GTHE3_CHANNEL_EYE_SCAN_SWAP_EN = 1'b0,
  parameter   [3:0] GTHE3_CHANNEL_FTS_DESKEW_SEQ_ENABLE = 4'b1111,
  parameter   [3:0] GTHE3_CHANNEL_FTS_LANE_DESKEW_CFG = 4'b1111,
  parameter         GTHE3_CHANNEL_FTS_LANE_DESKEW_EN = "FALSE",
  parameter   [4:0] GTHE3_CHANNEL_GEARBOX_MODE = 5'b00000,
  parameter   [0:0] GTHE3_CHANNEL_GM_BIAS_SELECT = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_LOCAL_MASTER = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_OOBDIVCTL = 2'b00,
  parameter   [0:0] GTHE3_CHANNEL_OOB_PWRUP = 1'b0,
  parameter         GTHE3_CHANNEL_PCI3_AUTO_REALIGN = "FRST_SMPL",
  parameter   [0:0] GTHE3_CHANNEL_PCI3_PIPE_RX_ELECIDLE = 1'b1,
  parameter   [1:0] GTHE3_CHANNEL_PCI3_RX_ASYNC_EBUF_BYPASS = 2'b00,
  parameter   [0:0] GTHE3_CHANNEL_PCI3_RX_ELECIDLE_EI2_ENABLE = 1'b0,
  parameter   [5:0] GTHE3_CHANNEL_PCI3_RX_ELECIDLE_H2L_COUNT = 6'b000000,
  parameter   [2:0] GTHE3_CHANNEL_PCI3_RX_ELECIDLE_H2L_DISABLE = 3'b000,
  parameter   [5:0] GTHE3_CHANNEL_PCI3_RX_ELECIDLE_HI_COUNT = 6'b000000,
  parameter   [0:0] GTHE3_CHANNEL_PCI3_RX_ELECIDLE_LP4_DISABLE = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCI3_RX_FIFO_DISABLE = 1'b0,
  parameter  [15:0] GTHE3_CHANNEL_PCIE_BUFG_DIV_CTRL = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_PCIE_RXPCS_CFG_GEN3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_PCIE_RXPMA_CFG = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_PCIE_TXPCS_CFG_GEN3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_PCIE_TXPMA_CFG = 16'h0000,
  parameter         GTHE3_CHANNEL_PCS_PCIE_EN = "FALSE",
  parameter  [15:0] GTHE3_CHANNEL_PCS_RSVD0 = 16'b0000000000000000,
  parameter   [2:0] GTHE3_CHANNEL_PCS_RSVD1 = 3'b000,
  parameter  [11:0] GTHE3_CHANNEL_PD_TRANS_TIME_FROM_P2 = 12'h03C,
  parameter   [7:0] GTHE3_CHANNEL_PD_TRANS_TIME_NONE_P2 = 8'h19,
  parameter   [7:0] GTHE3_CHANNEL_PD_TRANS_TIME_TO_P2 = 8'h64,
  parameter   [1:0] GTHE3_CHANNEL_PLL_SEL_MODE_GEN12 = 2'h0,
  parameter   [1:0] GTHE3_CHANNEL_PLL_SEL_MODE_GEN3 = 2'h0,
  parameter  [15:0] GTHE3_CHANNEL_PMA_RSV1 = 16'h0000,
  parameter   [2:0] GTHE3_CHANNEL_PROCESS_PAR = 3'b010,
  parameter   [0:0] GTHE3_CHANNEL_RATE_SW_USE_DRP = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RESET_POWERSAVE_DISABLE = 1'b0,
  parameter   [4:0] GTHE3_CHANNEL_RXBUFRESET_TIME = 5'b00001,
  parameter         GTHE3_CHANNEL_RXBUF_ADDR_MODE = "FULL",
  parameter   [3:0] GTHE3_CHANNEL_RXBUF_EIDLE_HI_CNT = 4'b1000,
  parameter   [3:0] GTHE3_CHANNEL_RXBUF_EIDLE_LO_CNT = 4'b0000,
  parameter         GTHE3_CHANNEL_RXBUF_EN = "TRUE",
  parameter         GTHE3_CHANNEL_RXBUF_RESET_ON_CB_CHANGE = "TRUE",
  parameter         GTHE3_CHANNEL_RXBUF_RESET_ON_COMMAALIGN = "FALSE",
  parameter         GTHE3_CHANNEL_RXBUF_RESET_ON_EIDLE = "FALSE",
  parameter         GTHE3_CHANNEL_RXBUF_RESET_ON_RATE_CHANGE = "TRUE",
  parameter integer GTHE3_CHANNEL_RXBUF_THRESH_OVFLW = 0,
  parameter         GTHE3_CHANNEL_RXBUF_THRESH_OVRD = "FALSE",
  parameter integer GTHE3_CHANNEL_RXBUF_THRESH_UNDFLW = 4,
  parameter   [4:0] GTHE3_CHANNEL_RXCDRFREQRESET_TIME = 5'b00001,
  parameter   [4:0] GTHE3_CHANNEL_RXCDRPHRESET_TIME = 5'b00001,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG0_GEN3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG1 = 16'h0080,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG1_GEN3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG2 = 16'h07E6,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG2_GEN3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG3_GEN3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG4 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG4_GEN3 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG5 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_CFG5_GEN3 = 16'h0000,
  parameter   [0:0] GTHE3_CHANNEL_RXCDR_FR_RESET_ON_EIDLE = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDR_HOLD_DURING_EIDLE = 1'b0,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_LOCK_CFG0 = 16'h5080,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_LOCK_CFG1 = 16'h07E0,
  parameter  [15:0] GTHE3_CHANNEL_RXCDR_LOCK_CFG2 = 16'h7C42,
  parameter   [0:0] GTHE3_CHANNEL_RXCDR_PH_RESET_ON_EIDLE = 1'b0,
  parameter  [15:0] GTHE3_CHANNEL_RXCFOK_CFG0 = 16'h4000,
  parameter  [15:0] GTHE3_CHANNEL_RXCFOK_CFG1 = 16'h0060,
  parameter  [15:0] GTHE3_CHANNEL_RXCFOK_CFG2 = 16'h000E,
  parameter   [6:0] GTHE3_CHANNEL_RXDFELPMRESET_TIME = 7'b0001111,
  parameter  [15:0] GTHE3_CHANNEL_RXDFELPM_KL_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFELPM_KL_CFG1 = 16'h0032,
  parameter  [15:0] GTHE3_CHANNEL_RXDFELPM_KL_CFG2 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_CFG0 = 16'h0A00,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_GC_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_GC_CFG1 = 16'h7840,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_GC_CFG2 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H2_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H2_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H3_CFG0 = 16'h4000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H3_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H4_CFG0 = 16'h2000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H4_CFG1 = 16'h0003,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H5_CFG0 = 16'h2000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H5_CFG1 = 16'h0003,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H6_CFG0 = 16'h2000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H6_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H7_CFG0 = 16'h2000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H7_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H8_CFG0 = 16'h2000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H8_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H9_CFG0 = 16'h2000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_H9_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HA_CFG0 = 16'h2000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HA_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HB_CFG0 = 16'h2000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HB_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HC_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HC_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HD_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HD_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HE_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HE_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HF_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_HF_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_OS_CFG0 = 16'h8000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_OS_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_UT_CFG0 = 16'h8000,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_UT_CFG1 = 16'h0003,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_VP_CFG0 = 16'hAA00,
  parameter  [15:0] GTHE3_CHANNEL_RXDFE_VP_CFG1 = 16'h0033,
  parameter  [15:0] GTHE3_CHANNEL_RXDLY_CFG = 16'h001F,
  parameter  [15:0] GTHE3_CHANNEL_RXDLY_LCFG = 16'h0030,
  parameter         GTHE3_CHANNEL_RXELECIDLE_CFG = "Sigcfg_4",
  parameter integer GTHE3_CHANNEL_RXGBOX_FIFO_INIT_RD_ADDR = 4,
  parameter         GTHE3_CHANNEL_RXGEARBOX_EN = "FALSE",
  parameter   [4:0] GTHE3_CHANNEL_RXISCANRESET_TIME = 5'b00001,
  parameter  [15:0] GTHE3_CHANNEL_RXLPM_CFG = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXLPM_GC_CFG = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXLPM_KH_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXLPM_KH_CFG1 = 16'h0002,
  parameter  [15:0] GTHE3_CHANNEL_RXLPM_OS_CFG0 = 16'h8000,
  parameter  [15:0] GTHE3_CHANNEL_RXLPM_OS_CFG1 = 16'h0002,
  parameter   [8:0] GTHE3_CHANNEL_RXOOB_CFG = 9'b000000110,
  parameter         GTHE3_CHANNEL_RXOOB_CLK_CFG = "PMA",
  parameter   [4:0] GTHE3_CHANNEL_RXOSCALRESET_TIME = 5'b00011,
  parameter integer GTHE3_CHANNEL_RXOUT_DIV = 4,
  parameter   [4:0] GTHE3_CHANNEL_RXPCSRESET_TIME = 5'b00001,
  parameter  [15:0] GTHE3_CHANNEL_RXPHBEACON_CFG = 16'h0000,
  parameter  [15:0] GTHE3_CHANNEL_RXPHDLY_CFG = 16'h2020,
  parameter  [15:0] GTHE3_CHANNEL_RXPHSAMP_CFG = 16'h2100,
  parameter  [15:0] GTHE3_CHANNEL_RXPHSLIP_CFG = 16'h6622,
  parameter   [4:0] GTHE3_CHANNEL_RXPH_MONITOR_SEL = 5'b00000,
  parameter   [1:0] GTHE3_CHANNEL_RXPI_CFG0 = 2'b00,
  parameter   [1:0] GTHE3_CHANNEL_RXPI_CFG1 = 2'b00,
  parameter   [1:0] GTHE3_CHANNEL_RXPI_CFG2 = 2'b00,
  parameter   [1:0] GTHE3_CHANNEL_RXPI_CFG3 = 2'b00,
  parameter   [0:0] GTHE3_CHANNEL_RXPI_CFG4 = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPI_CFG5 = 1'b1,
  parameter   [2:0] GTHE3_CHANNEL_RXPI_CFG6 = 3'b000,
  parameter   [0:0] GTHE3_CHANNEL_RXPI_LPM = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPI_VREFSEL = 1'b0,
  parameter         GTHE3_CHANNEL_RXPMACLK_SEL = "DATA",
  parameter   [4:0] GTHE3_CHANNEL_RXPMARESET_TIME = 5'b00001,
  parameter   [0:0] GTHE3_CHANNEL_RXPRBS_ERR_LOOPBACK = 1'b0,
  parameter integer GTHE3_CHANNEL_RXPRBS_LINKACQ_CNT = 15,
  parameter integer GTHE3_CHANNEL_RXSLIDE_AUTO_WAIT = 7,
  parameter         GTHE3_CHANNEL_RXSLIDE_MODE = "OFF",
  parameter   [0:0] GTHE3_CHANNEL_RXSYNC_MULTILANE = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSYNC_OVRD = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSYNC_SKIP_DA = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RX_AFE_CM_EN = 1'b0,
  parameter  [15:0] GTHE3_CHANNEL_RX_BIAS_CFG0 = 16'h0AD4,
  parameter   [5:0] GTHE3_CHANNEL_RX_BUFFER_CFG = 6'b000000,
  parameter   [0:0] GTHE3_CHANNEL_RX_CAPFF_SARC_ENB = 1'b0,
  parameter integer GTHE3_CHANNEL_RX_CLK25_DIV = 8,
  parameter   [0:0] GTHE3_CHANNEL_RX_CLKMUX_EN = 1'b1,
  parameter   [4:0] GTHE3_CHANNEL_RX_CLK_SLIP_OVRD = 5'b00000,
  parameter   [3:0] GTHE3_CHANNEL_RX_CM_BUF_CFG = 4'b1010,
  parameter   [0:0] GTHE3_CHANNEL_RX_CM_BUF_PD = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_RX_CM_SEL = 2'b11,
  parameter   [3:0] GTHE3_CHANNEL_RX_CM_TRIM = 4'b0100,
  parameter   [7:0] GTHE3_CHANNEL_RX_CTLE3_LPF = 8'b00000000,
  parameter integer GTHE3_CHANNEL_RX_DATA_WIDTH = 20,
  parameter   [5:0] GTHE3_CHANNEL_RX_DDI_SEL = 6'b000000,
  parameter         GTHE3_CHANNEL_RX_DEFER_RESET_BUF_EN = "TRUE",
  parameter   [3:0] GTHE3_CHANNEL_RX_DFELPM_CFG0 = 4'b0110,
  parameter   [0:0] GTHE3_CHANNEL_RX_DFELPM_CFG1 = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RX_DFELPM_KLKH_AGC_STUP_EN = 1'b1,
  parameter   [1:0] GTHE3_CHANNEL_RX_DFE_AGC_CFG0 = 2'b00,
  parameter   [2:0] GTHE3_CHANNEL_RX_DFE_AGC_CFG1 = 3'b100,
  parameter   [1:0] GTHE3_CHANNEL_RX_DFE_KL_LPM_KH_CFG0 = 2'b01,
  parameter   [2:0] GTHE3_CHANNEL_RX_DFE_KL_LPM_KH_CFG1 = 3'b010,
  parameter   [1:0] GTHE3_CHANNEL_RX_DFE_KL_LPM_KL_CFG0 = 2'b01,
  parameter   [2:0] GTHE3_CHANNEL_RX_DFE_KL_LPM_KL_CFG1 = 3'b010,
  parameter   [0:0] GTHE3_CHANNEL_RX_DFE_LPM_HOLD_DURING_EIDLE = 1'b0,
  parameter         GTHE3_CHANNEL_RX_DISPERR_SEQ_MATCH = "TRUE",
  parameter   [4:0] GTHE3_CHANNEL_RX_DIVRESET_TIME = 5'b00001,
  parameter   [0:0] GTHE3_CHANNEL_RX_EN_HI_LR = 1'b0,
  parameter   [6:0] GTHE3_CHANNEL_RX_EYESCAN_VS_CODE = 7'b0000000,
  parameter   [0:0] GTHE3_CHANNEL_RX_EYESCAN_VS_NEG_DIR = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_RX_EYESCAN_VS_RANGE = 2'b00,
  parameter   [0:0] GTHE3_CHANNEL_RX_EYESCAN_VS_UT_SIGN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RX_FABINT_USRCLK_FLOP = 1'b0,
  parameter integer GTHE3_CHANNEL_RX_INT_DATAWIDTH = 1,
  parameter   [0:0] GTHE3_CHANNEL_RX_PMA_POWER_SAVE = 1'b0,
  parameter    real GTHE3_CHANNEL_RX_PROGDIV_CFG = 0.0,
  parameter   [2:0] GTHE3_CHANNEL_RX_SAMPLE_PERIOD = 3'b101,
  parameter integer GTHE3_CHANNEL_RX_SIG_VALID_DLY = 11,
  parameter   [0:0] GTHE3_CHANNEL_RX_SUM_DFETAPREP_EN = 1'b0,
  parameter   [3:0] GTHE3_CHANNEL_RX_SUM_IREF_TUNE = 4'b0000,
  parameter   [1:0] GTHE3_CHANNEL_RX_SUM_RES_CTRL = 2'b00,
  parameter   [3:0] GTHE3_CHANNEL_RX_SUM_VCMTUNE = 4'b0000,
  parameter   [0:0] GTHE3_CHANNEL_RX_SUM_VCM_OVWR = 1'b0,
  parameter   [2:0] GTHE3_CHANNEL_RX_SUM_VREF_TUNE = 3'b000,
  parameter   [1:0] GTHE3_CHANNEL_RX_TUNE_AFE_OS = 2'b00,
  parameter   [0:0] GTHE3_CHANNEL_RX_WIDEMODE_CDR = 1'b0,
  parameter         GTHE3_CHANNEL_RX_XCLK_SEL = "RXDES",
  parameter integer GTHE3_CHANNEL_SAS_MAX_COM = 64,
  parameter integer GTHE3_CHANNEL_SAS_MIN_COM = 36,
  parameter   [3:0] GTHE3_CHANNEL_SATA_BURST_SEQ_LEN = 4'b1111,
  parameter   [2:0] GTHE3_CHANNEL_SATA_BURST_VAL = 3'b100,
  parameter         GTHE3_CHANNEL_SATA_CPLL_CFG = "VCO_3000MHZ",
  parameter   [2:0] GTHE3_CHANNEL_SATA_EIDLE_VAL = 3'b100,
  parameter integer GTHE3_CHANNEL_SATA_MAX_BURST = 8,
  parameter integer GTHE3_CHANNEL_SATA_MAX_INIT = 21,
  parameter integer GTHE3_CHANNEL_SATA_MAX_WAKE = 7,
  parameter integer GTHE3_CHANNEL_SATA_MIN_BURST = 4,
  parameter integer GTHE3_CHANNEL_SATA_MIN_INIT = 12,
  parameter integer GTHE3_CHANNEL_SATA_MIN_WAKE = 4,
  parameter         GTHE3_CHANNEL_SHOW_REALIGN_COMMA = "TRUE",
  parameter         GTHE3_CHANNEL_SIM_RECEIVER_DETECT_PASS = "TRUE",
  parameter         GTHE3_CHANNEL_SIM_RESET_SPEEDUP = "TRUE",
  parameter   [0:0] GTHE3_CHANNEL_SIM_TX_EIDLE_DRIVE_LEVEL = 1'b0,
  parameter integer GTHE3_CHANNEL_SIM_VERSION = 2,
  parameter   [1:0] GTHE3_CHANNEL_TAPDLY_SET_TX = 2'h0,
  parameter   [3:0] GTHE3_CHANNEL_TEMPERATUR_PAR = 4'b0010,
  parameter  [14:0] GTHE3_CHANNEL_TERM_RCAL_CFG = 15'b100001000010000,
  parameter   [2:0] GTHE3_CHANNEL_TERM_RCAL_OVRD = 3'b000,
  parameter   [7:0] GTHE3_CHANNEL_TRANS_TIME_RATE = 8'h0E,
  parameter   [7:0] GTHE3_CHANNEL_TST_RSV0 = 8'h00,
  parameter   [7:0] GTHE3_CHANNEL_TST_RSV1 = 8'h00,
  parameter         GTHE3_CHANNEL_TXBUF_EN = "TRUE",
  parameter         GTHE3_CHANNEL_TXBUF_RESET_ON_RATE_CHANGE = "FALSE",
  parameter  [15:0] GTHE3_CHANNEL_TXDLY_CFG = 16'h001F,
  parameter  [15:0] GTHE3_CHANNEL_TXDLY_LCFG = 16'h0030,
  parameter   [3:0] GTHE3_CHANNEL_TXDRVBIAS_N = 4'b1010,
  parameter   [3:0] GTHE3_CHANNEL_TXDRVBIAS_P = 4'b1100,
  parameter         GTHE3_CHANNEL_TXFIFO_ADDR_CFG = "LOW",
  parameter integer GTHE3_CHANNEL_TXGBOX_FIFO_INIT_RD_ADDR = 4,
  parameter         GTHE3_CHANNEL_TXGEARBOX_EN = "FALSE",
  parameter integer GTHE3_CHANNEL_TXOUT_DIV = 4,
  parameter   [4:0] GTHE3_CHANNEL_TXPCSRESET_TIME = 5'b00001,
  parameter  [15:0] GTHE3_CHANNEL_TXPHDLY_CFG0 = 16'h2020,
  parameter  [15:0] GTHE3_CHANNEL_TXPHDLY_CFG1 = 16'h0001,
  parameter  [15:0] GTHE3_CHANNEL_TXPH_CFG = 16'h0980,
  parameter   [4:0] GTHE3_CHANNEL_TXPH_MONITOR_SEL = 5'b00000,
  parameter   [1:0] GTHE3_CHANNEL_TXPI_CFG0 = 2'b00,
  parameter   [1:0] GTHE3_CHANNEL_TXPI_CFG1 = 2'b00,
  parameter   [1:0] GTHE3_CHANNEL_TXPI_CFG2 = 2'b00,
  parameter   [0:0] GTHE3_CHANNEL_TXPI_CFG3 = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPI_CFG4 = 1'b1,
  parameter   [2:0] GTHE3_CHANNEL_TXPI_CFG5 = 3'b000,
  parameter   [0:0] GTHE3_CHANNEL_TXPI_GRAY_SEL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPI_INVSTROBE_SEL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPI_LPM = 1'b0,
  parameter         GTHE3_CHANNEL_TXPI_PPMCLK_SEL = "TXUSRCLK2",
  parameter   [7:0] GTHE3_CHANNEL_TXPI_PPM_CFG = 8'b00000000,
  parameter   [2:0] GTHE3_CHANNEL_TXPI_SYNFREQ_PPM = 3'b000,
  parameter   [0:0] GTHE3_CHANNEL_TXPI_VREFSEL = 1'b0,
  parameter   [4:0] GTHE3_CHANNEL_TXPMARESET_TIME = 5'b00001,
  parameter   [0:0] GTHE3_CHANNEL_TXSYNC_MULTILANE = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSYNC_OVRD = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSYNC_SKIP_DA = 1'b0,
  parameter integer GTHE3_CHANNEL_TX_CLK25_DIV = 8,
  parameter   [0:0] GTHE3_CHANNEL_TX_CLKMUX_EN = 1'b1,
  parameter integer GTHE3_CHANNEL_TX_DATA_WIDTH = 20,
  parameter   [5:0] GTHE3_CHANNEL_TX_DCD_CFG = 6'b000010,
  parameter   [0:0] GTHE3_CHANNEL_TX_DCD_EN = 1'b0,
  parameter   [5:0] GTHE3_CHANNEL_TX_DEEMPH0 = 6'b000000,
  parameter   [5:0] GTHE3_CHANNEL_TX_DEEMPH1 = 6'b000000,
  parameter   [4:0] GTHE3_CHANNEL_TX_DIVRESET_TIME = 5'b00001,
  parameter         GTHE3_CHANNEL_TX_DRIVE_MODE = "DIRECT",
  parameter   [2:0] GTHE3_CHANNEL_TX_EIDLE_ASSERT_DELAY = 3'b110,
  parameter   [2:0] GTHE3_CHANNEL_TX_EIDLE_DEASSERT_DELAY = 3'b100,
  parameter   [0:0] GTHE3_CHANNEL_TX_EML_PHI_TUNE = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TX_FABINT_USRCLK_FLOP = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TX_IDLE_DATA_ZERO = 1'b0,
  parameter integer GTHE3_CHANNEL_TX_INT_DATAWIDTH = 1,
  parameter         GTHE3_CHANNEL_TX_LOOPBACK_DRIVE_HIZ = "FALSE",
  parameter   [0:0] GTHE3_CHANNEL_TX_MAINCURSOR_SEL = 1'b0,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_FULL_0 = 7'b1001110,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_FULL_1 = 7'b1001001,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_FULL_2 = 7'b1000101,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_FULL_3 = 7'b1000010,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_FULL_4 = 7'b1000000,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_LOW_0 = 7'b1000110,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_LOW_1 = 7'b1000100,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_LOW_2 = 7'b1000010,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_LOW_3 = 7'b1000000,
  parameter   [6:0] GTHE3_CHANNEL_TX_MARGIN_LOW_4 = 7'b1000000,
  parameter   [2:0] GTHE3_CHANNEL_TX_MODE_SEL = 3'b000,
  parameter   [0:0] GTHE3_CHANNEL_TX_PMADATA_OPT = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TX_PMA_POWER_SAVE = 1'b0,
  parameter         GTHE3_CHANNEL_TX_PROGCLK_SEL = "POSTPI",
  parameter    real GTHE3_CHANNEL_TX_PROGDIV_CFG = 0.0,
  parameter   [0:0] GTHE3_CHANNEL_TX_QPI_STATUS_EN = 1'b0,
  parameter  [13:0] GTHE3_CHANNEL_TX_RXDETECT_CFG = 14'h0032,
  parameter   [2:0] GTHE3_CHANNEL_TX_RXDETECT_REF = 3'b100,
  parameter   [2:0] GTHE3_CHANNEL_TX_SAMPLE_PERIOD = 3'b101,
  parameter   [0:0] GTHE3_CHANNEL_TX_SARC_LPBK_ENB = 1'b0,
  parameter         GTHE3_CHANNEL_TX_XCLK_SEL = "TXOUT",
  parameter   [0:0] GTHE3_CHANNEL_USE_PCS_CLK_PHASE_SEL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_WB_MODE = 2'b00,

  // primitive wrapper parameters which specify GTHE3_CHANNEL primitive input port default driver values
  parameter   [0:0] GTHE3_CHANNEL_CFGRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CLKRSVD0_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CLKRSVD1_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CPLLLOCKDETCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CPLLLOCKEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CPLLPD_VAL = 1'b0,
  parameter   [2:0] GTHE3_CHANNEL_CPLLREFCLKSEL_VAL = 3'b0,
  parameter   [0:0] GTHE3_CHANNEL_CPLLRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DMONFIFORESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DMONITORCLK_VAL = 1'b0,
  parameter   [8:0] GTHE3_CHANNEL_DRPADDR_VAL = 9'b0,
  parameter   [0:0] GTHE3_CHANNEL_DRPCLK_VAL = 1'b0,
  parameter  [15:0] GTHE3_CHANNEL_DRPDI_VAL = 16'b0,
  parameter   [0:0] GTHE3_CHANNEL_DRPEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DRPWE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHICALDONE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHICALSTART_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHIDRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHIDWREN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHIXRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHIXWREN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EYESCANMODE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EYESCANRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EYESCANTRIGGER_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTGREFCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTHRXN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTHRXP_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTNORTHREFCLK0_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTNORTHREFCLK1_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTREFCLK0_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTREFCLK1_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTRESETSEL_VAL = 1'b0,
  parameter  [15:0] GTHE3_CHANNEL_GTRSVD_VAL = 16'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTRXRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTSOUTHREFCLK0_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTSOUTHREFCLK1_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTTXRESET_VAL = 1'b0,
  parameter   [2:0] GTHE3_CHANNEL_LOOPBACK_VAL = 3'b0,
  parameter   [0:0] GTHE3_CHANNEL_LPBKRXTXSEREN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_LPBKTXRXSEREN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCIERSTIDLE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCIERSTTXSYNCSTART_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCIEUSERRATEDONE_VAL = 1'b0,
  parameter  [15:0] GTHE3_CHANNEL_PCSRSVDIN_VAL = 16'b0,
  parameter   [4:0] GTHE3_CHANNEL_PCSRSVDIN2_VAL = 5'b0,
  parameter   [4:0] GTHE3_CHANNEL_PMARSVDIN_VAL = 5'b0,
  parameter   [0:0] GTHE3_CHANNEL_QPLL0CLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_QPLL0REFCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_QPLL1CLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_QPLL1REFCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RESETOVRD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RSTCLKENTX_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RX8B10BEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXBUFRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDRFREQRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDRHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDROVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDRRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDRRESETRSV_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCHBONDEN_VAL = 1'b0,
  parameter   [4:0] GTHE3_CHANNEL_RXCHBONDI_VAL = 5'b0,
  parameter   [2:0] GTHE3_CHANNEL_RXCHBONDLEVEL_VAL = 3'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCHBONDMASTER_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCHBONDSLAVE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCOMMADETEN_VAL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_RXDFEAGCCTRL_VAL = 2'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEAGCHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEAGCOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFELFHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFELFOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFELPMRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP10HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP10OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP11HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP11OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP12HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP12OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP13HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP13OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP14HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP14OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP15HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP15OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP2HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP2OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP3HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP3OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP4HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP4OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP5HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP5OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP6HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP6OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP7HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP7OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP8HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP8OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP9HOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP9OVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEUTHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEUTOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEVPHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEVPOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEVSEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEXYDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDLYBYPASS_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDLYEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDLYOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDLYSRESET_VAL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_RXELECIDLEMODE_VAL = 2'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXGEARBOXSLIP_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLATCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMGCHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMGCOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMHFHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMHFOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMLFHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMLFKLOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMOSHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMOSOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXMCOMMAALIGNEN_VAL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_RXMONITORSEL_VAL = 2'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOOBRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSCALRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSHOLD_VAL = 1'b0,
  parameter   [3:0] GTHE3_CHANNEL_RXOSINTCFG_VAL = 4'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTSTROBE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTTESTOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSOVRDEN_VAL = 1'b0,
  parameter   [2:0] GTHE3_CHANNEL_RXOUTCLKSEL_VAL = 3'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPCOMMAALIGNEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPCSRESET_VAL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_RXPD_VAL = 2'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHALIGN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHALIGNEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHDLYPD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHDLYRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHOVRDEN_VAL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_RXPLLCLKSEL_VAL = 2'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPMARESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPOLARITY_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPRBSCNTRESET_VAL = 1'b0,
  parameter   [3:0] GTHE3_CHANNEL_RXPRBSSEL_VAL = 4'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPROGDIVRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXQPIEN_VAL = 1'b0,
  parameter   [2:0] GTHE3_CHANNEL_RXRATE_VAL = 3'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXRATEMODE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSLIDE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSLIPOUTCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSLIPPMA_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSYNCALLIN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSYNCIN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSYNCMODE_VAL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_RXSYSCLKSEL_VAL = 2'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXUSERRDY_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXUSRCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXUSRCLK2_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_SIGVALIDCLK_VAL = 1'b0,
  parameter  [19:0] GTHE3_CHANNEL_TSTIN_VAL = 20'b0,
  parameter   [7:0] GTHE3_CHANNEL_TX8B10BBYPASS_VAL = 8'b0,
  parameter   [0:0] GTHE3_CHANNEL_TX8B10BEN_VAL = 1'b0,
  parameter   [2:0] GTHE3_CHANNEL_TXBUFDIFFCTRL_VAL = 3'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXCOMINIT_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXCOMSAS_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXCOMWAKE_VAL = 1'b0,
  parameter  [15:0] GTHE3_CHANNEL_TXCTRL0_VAL = 16'b0,
  parameter  [15:0] GTHE3_CHANNEL_TXCTRL1_VAL = 16'b0,
  parameter   [7:0] GTHE3_CHANNEL_TXCTRL2_VAL = 8'b0,
  parameter [127:0] GTHE3_CHANNEL_TXDATA_VAL = 128'b0,
  parameter   [7:0] GTHE3_CHANNEL_TXDATAEXTENDRSVD_VAL = 8'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDEEMPH_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDETECTRX_VAL = 1'b0,
  parameter   [3:0] GTHE3_CHANNEL_TXDIFFCTRL_VAL = 4'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDIFFPD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYBYPASS_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYHOLD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYSRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYUPDOWN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXELECIDLE_VAL = 1'b0,
  parameter   [5:0] GTHE3_CHANNEL_TXHEADER_VAL = 6'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXINHIBIT_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXLATCLK_VAL = 1'b0,
  parameter   [6:0] GTHE3_CHANNEL_TXMAINCURSOR_VAL = 7'b0,
  parameter   [2:0] GTHE3_CHANNEL_TXMARGIN_VAL = 3'b0,
  parameter   [2:0] GTHE3_CHANNEL_TXOUTCLKSEL_VAL = 3'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPCSRESET_VAL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_TXPD_VAL = 2'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPDELECIDLEMODE_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHALIGN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHALIGNEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHDLYPD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHDLYRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHDLYTSTCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHINIT_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPIPPMEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPIPPMOVRDEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPIPPMPD_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPIPPMSEL_VAL = 1'b0,
  parameter   [4:0] GTHE3_CHANNEL_TXPIPPMSTEPSIZE_VAL = 5'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPISOPD_VAL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_TXPLLCLKSEL_VAL = 2'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPMARESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPOLARITY_VAL = 1'b0,
  parameter   [4:0] GTHE3_CHANNEL_TXPOSTCURSOR_VAL = 5'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPOSTCURSORINV_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPRBSFORCEERR_VAL = 1'b0,
  parameter   [3:0] GTHE3_CHANNEL_TXPRBSSEL_VAL = 4'b0,
  parameter   [4:0] GTHE3_CHANNEL_TXPRECURSOR_VAL = 5'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPRECURSORINV_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPROGDIVRESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXQPIBIASEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXQPISTRONGPDOWN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXQPIWEAKPUP_VAL = 1'b0,
  parameter   [2:0] GTHE3_CHANNEL_TXRATE_VAL = 3'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXRATEMODE_VAL = 1'b0,
  parameter   [6:0] GTHE3_CHANNEL_TXSEQUENCE_VAL = 7'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSWING_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSYNCALLIN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSYNCIN_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSYNCMODE_VAL = 1'b0,
  parameter   [1:0] GTHE3_CHANNEL_TXSYSCLKSEL_VAL = 2'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXUSERRDY_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXUSRCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXUSRCLK2_VAL = 1'b0,

  // primitive wrapper parameters which control GTHE3_CHANNEL primitive input port tie-off enablement
  parameter   [0:0] GTHE3_CHANNEL_CFGRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CLKRSVD0_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CLKRSVD1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CPLLLOCKDETCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CPLLLOCKEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CPLLPD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CPLLREFCLKSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_CPLLRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DMONFIFORESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DMONITORCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DRPADDR_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DRPCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DRPDI_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DRPEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_DRPWE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHICALDONE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHICALSTART_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHIDRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHIDWREN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHIXRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EVODDPHIXWREN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EYESCANMODE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EYESCANRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_EYESCANTRIGGER_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTGREFCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTHRXN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTHRXP_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTNORTHREFCLK0_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTNORTHREFCLK1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTREFCLK0_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTREFCLK1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTRESETSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTRSVD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTRXRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTSOUTHREFCLK0_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTSOUTHREFCLK1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_GTTXRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_LOOPBACK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_LPBKRXTXSEREN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_LPBKTXRXSEREN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCIERSTIDLE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCIERSTTXSYNCSTART_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCIEUSERRATEDONE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCSRSVDIN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PCSRSVDIN2_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_PMARSVDIN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_QPLL0CLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_QPLL0REFCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_QPLL1CLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_QPLL1REFCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RESETOVRD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RSTCLKENTX_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RX8B10BEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXBUFRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDRFREQRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDRHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDROVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDRRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCDRRESETRSV_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCHBONDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCHBONDI_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCHBONDLEVEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCHBONDMASTER_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCHBONDSLAVE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXCOMMADETEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEAGCCTRL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEAGCHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEAGCOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFELFHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFELFOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFELPMRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP10HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP10OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP11HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP11OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP12HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP12OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP13HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP13OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP14HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP14OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP15HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP15OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP2HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP2OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP3HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP3OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP4HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP4OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP5HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP5OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP6HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP6OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP7HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP7OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP8HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP8OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP9HOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFETAP9OVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEUTHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEUTOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEVPHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEVPOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEVSEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDFEXYDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDLYBYPASS_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDLYEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDLYOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXDLYSRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXELECIDLEMODE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXGEARBOXSLIP_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLATCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMGCHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMGCOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMHFHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMHFOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMLFHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMLFKLOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMOSHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXLPMOSOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXMCOMMAALIGNEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXMONITORSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOOBRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSCALRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTCFG_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTSTROBE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSINTTESTOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOSOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXOUTCLKSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPCOMMAALIGNEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPCSRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHALIGN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHALIGNEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHDLYPD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHDLYRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPHOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPLLCLKSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPMARESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPOLARITY_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPRBSCNTRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPRBSSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXPROGDIVRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXQPIEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXRATE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXRATEMODE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSLIDE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSLIPOUTCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSLIPPMA_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSYNCALLIN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSYNCIN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSYNCMODE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXSYSCLKSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXUSERRDY_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXUSRCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_RXUSRCLK2_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_SIGVALIDCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TSTIN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TX8B10BBYPASS_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TX8B10BEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXBUFDIFFCTRL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXCOMINIT_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXCOMSAS_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXCOMWAKE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXCTRL0_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXCTRL1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXCTRL2_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDATA_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDATAEXTENDRSVD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDEEMPH_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDETECTRX_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDIFFCTRL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDIFFPD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYBYPASS_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYHOLD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYSRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXDLYUPDOWN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXELECIDLE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXHEADER_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXINHIBIT_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXLATCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXMAINCURSOR_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXMARGIN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXOUTCLKSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPCSRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPDELECIDLEMODE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHALIGN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHALIGNEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHDLYPD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHDLYRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHDLYTSTCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHINIT_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPHOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPIPPMEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPIPPMOVRDEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPIPPMPD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPIPPMSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPIPPMSTEPSIZE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPISOPD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPLLCLKSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPMARESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPOLARITY_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPOSTCURSOR_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPOSTCURSORINV_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPRBSFORCEERR_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPRBSSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPRECURSOR_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPRECURSORINV_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXPROGDIVRESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXQPIBIASEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXQPISTRONGPDOWN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXQPIWEAKPUP_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXRATE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXRATEMODE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSEQUENCE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSWING_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSYNCALLIN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSYNCIN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSYNCMODE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXSYSCLKSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXUSERRDY_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXUSRCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_CHANNEL_TXUSRCLK2_TIE_EN = 1'b0

)(


  // -------------------------------------------------------------------------------------------------------------------
  // Ports relating to GTHE3_CHANNEL primitive
  // -------------------------------------------------------------------------------------------------------------------

  // primitive wrapper input ports which can drive corresponding GTHE3_CHANNEL primitive input ports
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CFGRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CLKRSVD0,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CLKRSVD1,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLLOCKDETCLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLLOCKEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLPD,
  input  wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_CPLLREFCLKSEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DMONFIFORESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DMONITORCLK,
  input  wire [(NUM_CHANNELS*  9)-1:0] GTHE3_CHANNEL_DRPADDR,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DRPCLK,
  input  wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_DRPDI,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DRPEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DRPWE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHICALDONE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHICALSTART,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHIDRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHIDWREN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHIXRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHIXWREN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EYESCANMODE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EYESCANRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EYESCANTRIGGER,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTGREFCLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTHRXN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTHRXP,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTNORTHREFCLK0,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTNORTHREFCLK1,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTREFCLK0,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTREFCLK1,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTRESETSEL,
  input  wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_GTRSVD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTRXRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTSOUTHREFCLK0,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTSOUTHREFCLK1,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTTXRESET,
  input  wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_LOOPBACK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_LPBKRXTXSEREN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_LPBKTXRXSEREN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIERSTIDLE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIERSTTXSYNCSTART,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIEUSERRATEDONE,
  input  wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_PCSRSVDIN,
  input  wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_PCSRSVDIN2,
  input  wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_PMARSVDIN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_QPLL0CLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_QPLL0REFCLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_QPLL1CLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_QPLL1REFCLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RESETOVRD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RSTCLKENTX,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RX8B10BEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXBUFRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRFREQRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDROVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRRESETRSV,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCHBONDEN,
  input  wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_RXCHBONDI,
  input  wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_RXCHBONDLEVEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCHBONDMASTER,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCHBONDSLAVE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCOMMADETEN,
  input  wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXDFEAGCCTRL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEAGCHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEAGCOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFELFHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFELFOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFELPMRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP10HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP10OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP11HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP11OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP12HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP12OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP13HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP13OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP14HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP14OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP15HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP15OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP2HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP2OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP3HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP3OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP4HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP4OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP5HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP5OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP6HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP6OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP7HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP7OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP8HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP8OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP9HOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP9OVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEUTHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEUTOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEVPHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEVPOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEVSEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEXYDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDLYBYPASS,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDLYEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDLYOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDLYSRESET,
  input  wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXELECIDLEMODE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXGEARBOXSLIP,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLATCLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMGCHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMGCOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMHFHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMHFOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMLFHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMLFKLOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMOSHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMOSOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXMCOMMAALIGNEN,
  input  wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXMONITORSEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOOBRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSCALRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSHOLD,
  input  wire [(NUM_CHANNELS*  4)-1:0] GTHE3_CHANNEL_RXOSINTCFG,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTSTROBE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTTESTOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSOVRDEN,
  input  wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_RXOUTCLKSEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPCOMMAALIGNEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPCSRESET,
  input  wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXPD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHALIGN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHALIGNEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHDLYPD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHDLYRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHOVRDEN,
  input  wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXPLLCLKSEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPMARESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPOLARITY,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPRBSCNTRESET,
  input  wire [(NUM_CHANNELS*  4)-1:0] GTHE3_CHANNEL_RXPRBSSEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPROGDIVRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXQPIEN,
  input  wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_RXRATE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXRATEMODE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIDE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIPOUTCLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIPPMA,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSYNCALLIN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSYNCIN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSYNCMODE,
  input  wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXSYSCLKSEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXUSERRDY,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXUSRCLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXUSRCLK2,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_SIGVALIDCLK,
  input  wire [(NUM_CHANNELS* 20)-1:0] GTHE3_CHANNEL_TSTIN,
  input  wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_TX8B10BBYPASS,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TX8B10BEN,
  input  wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_TXBUFDIFFCTRL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXCOMINIT,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXCOMSAS,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXCOMWAKE,
  input  wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_TXCTRL0,
  input  wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_TXCTRL1,
  input  wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_TXCTRL2,
  input  wire [(NUM_CHANNELS*128)-1:0] GTHE3_CHANNEL_TXDATA,
  input  wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_TXDATAEXTENDRSVD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDEEMPH,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDETECTRX,
  input  wire [(NUM_CHANNELS*  4)-1:0] GTHE3_CHANNEL_TXDIFFCTRL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDIFFPD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYBYPASS,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYHOLD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYSRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYUPDOWN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXELECIDLE,
  input  wire [(NUM_CHANNELS*  6)-1:0] GTHE3_CHANNEL_TXHEADER,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXINHIBIT,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXLATCLK,
  input  wire [(NUM_CHANNELS*  7)-1:0] GTHE3_CHANNEL_TXMAINCURSOR,
  input  wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_TXMARGIN,
  input  wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_TXOUTCLKSEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPCSRESET,
  input  wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_TXPD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPDELECIDLEMODE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHALIGN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHALIGNEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHDLYPD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHDLYRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHDLYTSTCLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHINIT,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPIPPMEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPIPPMOVRDEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPIPPMPD,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPIPPMSEL,
  input  wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_TXPIPPMSTEPSIZE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPISOPD,
  input  wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_TXPLLCLKSEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPMARESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPOLARITY,
  input  wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_TXPOSTCURSOR,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPOSTCURSORINV,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPRBSFORCEERR,
  input  wire [(NUM_CHANNELS*  4)-1:0] GTHE3_CHANNEL_TXPRBSSEL,
  input  wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_TXPRECURSOR,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPRECURSORINV,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPROGDIVRESET,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXQPIBIASEN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXQPISTRONGPDOWN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXQPIWEAKPUP,
  input  wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_TXRATE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXRATEMODE,
  input  wire [(NUM_CHANNELS*  7)-1:0] GTHE3_CHANNEL_TXSEQUENCE,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSWING,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSYNCALLIN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSYNCIN,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSYNCMODE,
  input  wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_TXSYSCLKSEL,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXUSERRDY,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXUSRCLK,
  input  wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXUSRCLK2,

  // primitive wrapper output ports which are driven by corresponding GTHE3_CHANNEL primitive output ports
  output wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_BUFGTCE,
  output wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_BUFGTCEMASK,
  output wire [(NUM_CHANNELS*  9)-1:0] GTHE3_CHANNEL_BUFGTDIV,
  output wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_BUFGTRESET,
  output wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_BUFGTRSTMASK,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLFBCLKLOST,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLLOCK,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLREFCLKLOST,
  output wire [(NUM_CHANNELS* 17)-1:0] GTHE3_CHANNEL_DMONITOROUT,
  output wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_DRPDO,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DRPRDY,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EYESCANDATAERROR,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTHTXN,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTHTXP,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTPOWERGOOD,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTREFCLKMONITOR,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIERATEGEN3,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIERATEIDLE,
  output wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_PCIERATEQPLLPD,
  output wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_PCIERATEQPLLRESET,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIESYNCTXSYNCDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIEUSERGEN3RDY,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIEUSERPHYSTATUSRST,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIEUSERRATESTART,
  output wire [(NUM_CHANNELS* 12)-1:0] GTHE3_CHANNEL_PCSRSVDOUT,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PHYSTATUS,
  output wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_PINRSRVDAS,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RESETEXCEPTION,
  output wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_RXBUFSTATUS,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXBYTEISALIGNED,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXBYTEREALIGN,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRLOCK,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRPHDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCHANBONDSEQ,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCHANISALIGNED,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCHANREALIGN,
  output wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_RXCHBONDO,
  output wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXCLKCORCNT,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCOMINITDET,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCOMMADET,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCOMSASDET,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCOMWAKEDET,
  output wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_RXCTRL0,
  output wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_RXCTRL1,
  output wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_RXCTRL2,
  output wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_RXCTRL3,
  output wire [(NUM_CHANNELS*128)-1:0] GTHE3_CHANNEL_RXDATA,
  output wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_RXDATAEXTENDRSVD,
  output wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXDATAVALID,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDLYSRESETDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXELECIDLE,
  output wire [(NUM_CHANNELS*  6)-1:0] GTHE3_CHANNEL_RXHEADER,
  output wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXHEADERVALID,
  output wire [(NUM_CHANNELS*  7)-1:0] GTHE3_CHANNEL_RXMONITOROUT,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTSTARTED,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTSTROBEDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTSTROBESTARTED,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOUTCLK,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOUTCLKFABRIC,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOUTCLKPCS,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHALIGNDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHALIGNERR,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPMARESETDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPRBSERR,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPRBSLOCKED,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPRGDIVRESETDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXQPISENN,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXQPISENP,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXRATEDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXRECCLKOUT,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXRESETDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIDERDY,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIPDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIPOUTCLKRDY,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIPPMARDY,
  output wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXSTARTOFSEQ,
  output wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_RXSTATUS,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSYNCDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSYNCOUT,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXVALID,
  output wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_TXBUFSTATUS,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXCOMFINISH,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYSRESETDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXOUTCLK,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXOUTCLKFABRIC,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXOUTCLKPCS,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHALIGNDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHINITDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPMARESETDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPRGDIVRESETDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXQPISENN,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXQPISENP,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXRATEDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXRESETDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSYNCDONE,
  output wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSYNCOUT

);


  // -------------------------------------------------------------------------------------------------------------------
  // HDL generation of wiring and instances relating to GTHE3_CHANNEL primitive
  // -------------------------------------------------------------------------------------------------------------------

  generate if (NUM_CHANNELS > 0) begin : gthe3_channel_gen

    // for each GTHE3_CHANNEL primitive input port, declare a vector scaled to drive all generated instances
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CFGRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CLKRSVD0_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CLKRSVD1_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLLOCKDETCLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLLOCKEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLPD_int;
    wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_CPLLREFCLKSEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_CPLLRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DMONFIFORESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DMONITORCLK_int;
    wire [(NUM_CHANNELS*  9)-1:0] GTHE3_CHANNEL_DRPADDR_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DRPCLK_int;
    wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_DRPDI_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DRPEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_DRPWE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHICALDONE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHICALSTART_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHIDRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHIDWREN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHIXRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EVODDPHIXWREN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EYESCANMODE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EYESCANRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_EYESCANTRIGGER_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTGREFCLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTHRXN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTHRXP_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTNORTHREFCLK0_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTNORTHREFCLK1_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTREFCLK0_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTREFCLK1_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTRESETSEL_int;
    wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_GTRSVD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTRXRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTSOUTHREFCLK0_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTSOUTHREFCLK1_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_GTTXRESET_int;
    wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_LOOPBACK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_LPBKRXTXSEREN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_LPBKTXRXSEREN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIERSTIDLE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIERSTTXSYNCSTART_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_PCIEUSERRATEDONE_int;
    wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_PCSRSVDIN_int;
    wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_PCSRSVDIN2_int;
    wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_PMARSVDIN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_QPLL0CLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_QPLL0REFCLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_QPLL1CLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_QPLL1REFCLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RESETOVRD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RSTCLKENTX_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RX8B10BEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXBUFRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRFREQRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDROVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCDRRESETRSV_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCHBONDEN_int;
    wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_RXCHBONDI_int;
    wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_RXCHBONDLEVEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCHBONDMASTER_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCHBONDSLAVE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXCOMMADETEN_int;
    wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXDFEAGCCTRL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEAGCHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEAGCOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFELFHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFELFOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFELPMRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP10HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP10OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP11HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP11OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP12HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP12OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP13HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP13OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP14HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP14OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP15HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP15OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP2HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP2OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP3HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP3OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP4HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP4OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP5HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP5OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP6HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP6OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP7HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP7OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP8HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP8OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP9HOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFETAP9OVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEUTHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEUTOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEVPHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEVPOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEVSEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDFEXYDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDLYBYPASS_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDLYEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDLYOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXDLYSRESET_int;
    wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXELECIDLEMODE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXGEARBOXSLIP_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLATCLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMGCHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMGCOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMHFHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMHFOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMLFHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMLFKLOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMOSHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXLPMOSOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXMCOMMAALIGNEN_int;
    wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXMONITORSEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOOBRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSCALRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSHOLD_int;
    wire [(NUM_CHANNELS*  4)-1:0] GTHE3_CHANNEL_RXOSINTCFG_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTSTROBE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSINTTESTOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXOSOVRDEN_int;
    wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_RXOUTCLKSEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPCOMMAALIGNEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPCSRESET_int;
    wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXPD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHALIGN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHALIGNEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHDLYPD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHDLYRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPHOVRDEN_int;
    wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXPLLCLKSEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPMARESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPOLARITY_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPRBSCNTRESET_int;
    wire [(NUM_CHANNELS*  4)-1:0] GTHE3_CHANNEL_RXPRBSSEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXPROGDIVRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXQPIEN_int;
    wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_RXRATE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXRATEMODE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIDE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIPOUTCLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSLIPPMA_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSYNCALLIN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSYNCIN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXSYNCMODE_int;
    wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_RXSYSCLKSEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXUSERRDY_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXUSRCLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_RXUSRCLK2_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_SIGVALIDCLK_int;
    wire [(NUM_CHANNELS* 20)-1:0] GTHE3_CHANNEL_TSTIN_int;
    wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_TX8B10BBYPASS_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TX8B10BEN_int;
    wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_TXBUFDIFFCTRL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXCOMINIT_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXCOMSAS_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXCOMWAKE_int;
    wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_TXCTRL0_int;
    wire [(NUM_CHANNELS* 16)-1:0] GTHE3_CHANNEL_TXCTRL1_int;
    wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_TXCTRL2_int;
    wire [(NUM_CHANNELS*128)-1:0] GTHE3_CHANNEL_TXDATA_int;
    wire [(NUM_CHANNELS*  8)-1:0] GTHE3_CHANNEL_TXDATAEXTENDRSVD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDEEMPH_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDETECTRX_int;
    wire [(NUM_CHANNELS*  4)-1:0] GTHE3_CHANNEL_TXDIFFCTRL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDIFFPD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYBYPASS_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYHOLD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYSRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXDLYUPDOWN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXELECIDLE_int;
    wire [(NUM_CHANNELS*  6)-1:0] GTHE3_CHANNEL_TXHEADER_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXINHIBIT_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXLATCLK_int;
    wire [(NUM_CHANNELS*  7)-1:0] GTHE3_CHANNEL_TXMAINCURSOR_int;
    wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_TXMARGIN_int;
    wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_TXOUTCLKSEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPCSRESET_int;
    wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_TXPD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPDELECIDLEMODE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHALIGN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHALIGNEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHDLYPD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHDLYRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHDLYTSTCLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHINIT_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPHOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPIPPMEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPIPPMOVRDEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPIPPMPD_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPIPPMSEL_int;
    wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_TXPIPPMSTEPSIZE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPISOPD_int;
    wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_TXPLLCLKSEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPMARESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPOLARITY_int;
    wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_TXPOSTCURSOR_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPOSTCURSORINV_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPRBSFORCEERR_int;
    wire [(NUM_CHANNELS*  4)-1:0] GTHE3_CHANNEL_TXPRBSSEL_int;
    wire [(NUM_CHANNELS*  5)-1:0] GTHE3_CHANNEL_TXPRECURSOR_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPRECURSORINV_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXPROGDIVRESET_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXQPIBIASEN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXQPISTRONGPDOWN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXQPIWEAKPUP_int;
    wire [(NUM_CHANNELS*  3)-1:0] GTHE3_CHANNEL_TXRATE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXRATEMODE_int;
    wire [(NUM_CHANNELS*  7)-1:0] GTHE3_CHANNEL_TXSEQUENCE_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSWING_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSYNCALLIN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSYNCIN_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXSYNCMODE_int;
    wire [(NUM_CHANNELS*  2)-1:0] GTHE3_CHANNEL_TXSYSCLKSEL_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXUSERRDY_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXUSRCLK_int;
    wire [(NUM_CHANNELS*  1)-1:0] GTHE3_CHANNEL_TXUSRCLK2_int;

    // assign each vector either the corresponding tie-off value or the corresponding input port, scaled appropriately
    if (GTHE3_CHANNEL_CFGRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_CFGRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_CFGRESET_VAL}};
    else
      assign GTHE3_CHANNEL_CFGRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_CFGRESET}};

    if (GTHE3_CHANNEL_CLKRSVD0_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_CLKRSVD0_int = {NUM_CHANNELS{GTHE3_CHANNEL_CLKRSVD0_VAL}};
    else
      assign GTHE3_CHANNEL_CLKRSVD0_int = {NUM_CHANNELS{GTHE3_CHANNEL_CLKRSVD0}};

    if (GTHE3_CHANNEL_CLKRSVD1_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_CLKRSVD1_int = {NUM_CHANNELS{GTHE3_CHANNEL_CLKRSVD1_VAL}};
    else
      assign GTHE3_CHANNEL_CLKRSVD1_int = {NUM_CHANNELS{GTHE3_CHANNEL_CLKRSVD1}};

    if (GTHE3_CHANNEL_CPLLLOCKDETCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_CPLLLOCKDETCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLLOCKDETCLK_VAL}};
    else
      assign GTHE3_CHANNEL_CPLLLOCKDETCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLLOCKDETCLK}};

    if (GTHE3_CHANNEL_CPLLLOCKEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_CPLLLOCKEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLLOCKEN_VAL}};
    else
      assign GTHE3_CHANNEL_CPLLLOCKEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLLOCKEN}};

    if (GTHE3_CHANNEL_CPLLPD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_CPLLPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLPD_VAL}};
    else
      assign GTHE3_CHANNEL_CPLLPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLPD}};

    if (GTHE3_CHANNEL_CPLLREFCLKSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_CPLLREFCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLREFCLKSEL_VAL}};
    else
      assign GTHE3_CHANNEL_CPLLREFCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLREFCLKSEL}};

    if (GTHE3_CHANNEL_CPLLRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_CPLLRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLRESET_VAL}};
    else
      assign GTHE3_CHANNEL_CPLLRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_CPLLRESET}};

    if (GTHE3_CHANNEL_DMONFIFORESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_DMONFIFORESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_DMONFIFORESET_VAL}};
    else
      assign GTHE3_CHANNEL_DMONFIFORESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_DMONFIFORESET}};

    if (GTHE3_CHANNEL_DMONITORCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_DMONITORCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_DMONITORCLK_VAL}};
    else
      assign GTHE3_CHANNEL_DMONITORCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_DMONITORCLK}};

    if (GTHE3_CHANNEL_DRPADDR_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_DRPADDR_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPADDR_VAL}};
    else
      assign GTHE3_CHANNEL_DRPADDR_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPADDR}};

    if (GTHE3_CHANNEL_DRPCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_DRPCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPCLK_VAL}};
    else
      assign GTHE3_CHANNEL_DRPCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPCLK}};

    if (GTHE3_CHANNEL_DRPDI_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_DRPDI_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPDI_VAL}};
    else
      assign GTHE3_CHANNEL_DRPDI_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPDI}};

    if (GTHE3_CHANNEL_DRPEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_DRPEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPEN_VAL}};
    else
      assign GTHE3_CHANNEL_DRPEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPEN}};

    if (GTHE3_CHANNEL_DRPWE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_DRPWE_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPWE_VAL}};
    else
      assign GTHE3_CHANNEL_DRPWE_int = {NUM_CHANNELS{GTHE3_CHANNEL_DRPWE}};

    if (GTHE3_CHANNEL_EVODDPHICALDONE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_EVODDPHICALDONE_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHICALDONE_VAL}};
    else
      assign GTHE3_CHANNEL_EVODDPHICALDONE_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHICALDONE}};

    if (GTHE3_CHANNEL_EVODDPHICALSTART_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_EVODDPHICALSTART_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHICALSTART_VAL}};
    else
      assign GTHE3_CHANNEL_EVODDPHICALSTART_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHICALSTART}};

    if (GTHE3_CHANNEL_EVODDPHIDRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_EVODDPHIDRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHIDRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_EVODDPHIDRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHIDRDEN}};

    if (GTHE3_CHANNEL_EVODDPHIDWREN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_EVODDPHIDWREN_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHIDWREN_VAL}};
    else
      assign GTHE3_CHANNEL_EVODDPHIDWREN_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHIDWREN}};

    if (GTHE3_CHANNEL_EVODDPHIXRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_EVODDPHIXRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHIXRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_EVODDPHIXRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHIXRDEN}};

    if (GTHE3_CHANNEL_EVODDPHIXWREN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_EVODDPHIXWREN_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHIXWREN_VAL}};
    else
      assign GTHE3_CHANNEL_EVODDPHIXWREN_int = {NUM_CHANNELS{GTHE3_CHANNEL_EVODDPHIXWREN}};

    if (GTHE3_CHANNEL_EYESCANMODE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_EYESCANMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_EYESCANMODE_VAL}};
    else
      assign GTHE3_CHANNEL_EYESCANMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_EYESCANMODE}};

    if (GTHE3_CHANNEL_EYESCANRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_EYESCANRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_EYESCANRESET_VAL}};
    else
      assign GTHE3_CHANNEL_EYESCANRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_EYESCANRESET}};

    if (GTHE3_CHANNEL_EYESCANTRIGGER_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_EYESCANTRIGGER_int = {NUM_CHANNELS{GTHE3_CHANNEL_EYESCANTRIGGER_VAL}};
    else
      assign GTHE3_CHANNEL_EYESCANTRIGGER_int = {NUM_CHANNELS{GTHE3_CHANNEL_EYESCANTRIGGER}};

    if (GTHE3_CHANNEL_GTGREFCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTGREFCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTGREFCLK_VAL}};
    else
      assign GTHE3_CHANNEL_GTGREFCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTGREFCLK}};

    if (GTHE3_CHANNEL_GTHRXN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTHRXN_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTHRXN_VAL}};
    else
      assign GTHE3_CHANNEL_GTHRXN_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTHRXN}};

    if (GTHE3_CHANNEL_GTHRXP_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTHRXP_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTHRXP_VAL}};
    else
      assign GTHE3_CHANNEL_GTHRXP_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTHRXP}};

    if (GTHE3_CHANNEL_GTNORTHREFCLK0_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTNORTHREFCLK0_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTNORTHREFCLK0_VAL}};
    else
      assign GTHE3_CHANNEL_GTNORTHREFCLK0_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTNORTHREFCLK0}};

    if (GTHE3_CHANNEL_GTNORTHREFCLK1_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTNORTHREFCLK1_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTNORTHREFCLK1_VAL}};
    else
      assign GTHE3_CHANNEL_GTNORTHREFCLK1_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTNORTHREFCLK1}};

    if (GTHE3_CHANNEL_GTREFCLK0_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTREFCLK0_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTREFCLK0_VAL}};
    else
      assign GTHE3_CHANNEL_GTREFCLK0_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTREFCLK0}};

    if (GTHE3_CHANNEL_GTREFCLK1_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTREFCLK1_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTREFCLK1_VAL}};
    else
      assign GTHE3_CHANNEL_GTREFCLK1_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTREFCLK1}};

    if (GTHE3_CHANNEL_GTRESETSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTRESETSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTRESETSEL_VAL}};
    else
      assign GTHE3_CHANNEL_GTRESETSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTRESETSEL}};

    if (GTHE3_CHANNEL_GTRSVD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTRSVD_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTRSVD_VAL}};
    else
      assign GTHE3_CHANNEL_GTRSVD_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTRSVD}};

    if (GTHE3_CHANNEL_GTRXRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTRXRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTRXRESET_VAL}};
    else
      assign GTHE3_CHANNEL_GTRXRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTRXRESET}};

    if (GTHE3_CHANNEL_GTSOUTHREFCLK0_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTSOUTHREFCLK0_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTSOUTHREFCLK0_VAL}};
    else
      assign GTHE3_CHANNEL_GTSOUTHREFCLK0_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTSOUTHREFCLK0}};

    if (GTHE3_CHANNEL_GTSOUTHREFCLK1_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTSOUTHREFCLK1_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTSOUTHREFCLK1_VAL}};
    else
      assign GTHE3_CHANNEL_GTSOUTHREFCLK1_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTSOUTHREFCLK1}};

    if (GTHE3_CHANNEL_GTTXRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_GTTXRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTTXRESET_VAL}};
    else
      assign GTHE3_CHANNEL_GTTXRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_GTTXRESET}};

    if (GTHE3_CHANNEL_LOOPBACK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_LOOPBACK_int = {NUM_CHANNELS{GTHE3_CHANNEL_LOOPBACK_VAL}};
    else
      assign GTHE3_CHANNEL_LOOPBACK_int = {NUM_CHANNELS{GTHE3_CHANNEL_LOOPBACK}};

    if (GTHE3_CHANNEL_LPBKRXTXSEREN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_LPBKRXTXSEREN_int = {NUM_CHANNELS{GTHE3_CHANNEL_LPBKRXTXSEREN_VAL}};
    else
      assign GTHE3_CHANNEL_LPBKRXTXSEREN_int = {NUM_CHANNELS{GTHE3_CHANNEL_LPBKRXTXSEREN}};

    if (GTHE3_CHANNEL_LPBKTXRXSEREN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_LPBKTXRXSEREN_int = {NUM_CHANNELS{GTHE3_CHANNEL_LPBKTXRXSEREN_VAL}};
    else
      assign GTHE3_CHANNEL_LPBKTXRXSEREN_int = {NUM_CHANNELS{GTHE3_CHANNEL_LPBKTXRXSEREN}};

    if (GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_VAL}};
    else
      assign GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE}};

    if (GTHE3_CHANNEL_PCIERSTIDLE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_PCIERSTIDLE_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCIERSTIDLE_VAL}};
    else
      assign GTHE3_CHANNEL_PCIERSTIDLE_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCIERSTIDLE}};

    if (GTHE3_CHANNEL_PCIERSTTXSYNCSTART_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_PCIERSTTXSYNCSTART_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCIERSTTXSYNCSTART_VAL}};
    else
      assign GTHE3_CHANNEL_PCIERSTTXSYNCSTART_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCIERSTTXSYNCSTART}};

    if (GTHE3_CHANNEL_PCIEUSERRATEDONE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_PCIEUSERRATEDONE_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCIEUSERRATEDONE_VAL}};
    else
      assign GTHE3_CHANNEL_PCIEUSERRATEDONE_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCIEUSERRATEDONE}};

    if (GTHE3_CHANNEL_PCSRSVDIN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_PCSRSVDIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCSRSVDIN_VAL}};
    else
      assign GTHE3_CHANNEL_PCSRSVDIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCSRSVDIN}};

    if (GTHE3_CHANNEL_PCSRSVDIN2_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_PCSRSVDIN2_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCSRSVDIN2_VAL}};
    else
      assign GTHE3_CHANNEL_PCSRSVDIN2_int = {NUM_CHANNELS{GTHE3_CHANNEL_PCSRSVDIN2}};

    if (GTHE3_CHANNEL_PMARSVDIN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_PMARSVDIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_PMARSVDIN_VAL}};
    else
      assign GTHE3_CHANNEL_PMARSVDIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_PMARSVDIN}};

    if (GTHE3_CHANNEL_QPLL0CLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_QPLL0CLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_QPLL0CLK_VAL}};
    else
      assign GTHE3_CHANNEL_QPLL0CLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_QPLL0CLK}};

    if (GTHE3_CHANNEL_QPLL0REFCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_QPLL0REFCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_QPLL0REFCLK_VAL}};
    else
      assign GTHE3_CHANNEL_QPLL0REFCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_QPLL0REFCLK}};

    if (GTHE3_CHANNEL_QPLL1CLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_QPLL1CLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_QPLL1CLK_VAL}};
    else
      assign GTHE3_CHANNEL_QPLL1CLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_QPLL1CLK}};

    if (GTHE3_CHANNEL_QPLL1REFCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_QPLL1REFCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_QPLL1REFCLK_VAL}};
    else
      assign GTHE3_CHANNEL_QPLL1REFCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_QPLL1REFCLK}};

    if (GTHE3_CHANNEL_RESETOVRD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RESETOVRD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RESETOVRD_VAL}};
    else
      assign GTHE3_CHANNEL_RESETOVRD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RESETOVRD}};

    if (GTHE3_CHANNEL_RSTCLKENTX_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RSTCLKENTX_int = {NUM_CHANNELS{GTHE3_CHANNEL_RSTCLKENTX_VAL}};
    else
      assign GTHE3_CHANNEL_RSTCLKENTX_int = {NUM_CHANNELS{GTHE3_CHANNEL_RSTCLKENTX}};

    if (GTHE3_CHANNEL_RX8B10BEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RX8B10BEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RX8B10BEN_VAL}};
    else
      assign GTHE3_CHANNEL_RX8B10BEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RX8B10BEN}};

    if (GTHE3_CHANNEL_RXBUFRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXBUFRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXBUFRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXBUFRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXBUFRESET}};

    if (GTHE3_CHANNEL_RXCDRFREQRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCDRFREQRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDRFREQRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXCDRFREQRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDRFREQRESET}};

    if (GTHE3_CHANNEL_RXCDRHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCDRHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDRHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXCDRHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDRHOLD}};

    if (GTHE3_CHANNEL_RXCDROVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCDROVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDROVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXCDROVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDROVRDEN}};

    if (GTHE3_CHANNEL_RXCDRRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCDRRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDRRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXCDRRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDRRESET}};

    if (GTHE3_CHANNEL_RXCDRRESETRSV_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCDRRESETRSV_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDRRESETRSV_VAL}};
    else
      assign GTHE3_CHANNEL_RXCDRRESETRSV_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCDRRESETRSV}};

    if (GTHE3_CHANNEL_RXCHBONDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCHBONDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXCHBONDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDEN}};

    if (GTHE3_CHANNEL_RXCHBONDI_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCHBONDI_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDI_VAL}};
    else
      assign GTHE3_CHANNEL_RXCHBONDI_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDI}};

    if (GTHE3_CHANNEL_RXCHBONDLEVEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCHBONDLEVEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDLEVEL_VAL}};
    else
      assign GTHE3_CHANNEL_RXCHBONDLEVEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDLEVEL}};

    if (GTHE3_CHANNEL_RXCHBONDMASTER_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCHBONDMASTER_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDMASTER_VAL}};
    else
      assign GTHE3_CHANNEL_RXCHBONDMASTER_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDMASTER}};

    if (GTHE3_CHANNEL_RXCHBONDSLAVE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCHBONDSLAVE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDSLAVE_VAL}};
    else
      assign GTHE3_CHANNEL_RXCHBONDSLAVE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCHBONDSLAVE}};

    if (GTHE3_CHANNEL_RXCOMMADETEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXCOMMADETEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCOMMADETEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXCOMMADETEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXCOMMADETEN}};

    if (GTHE3_CHANNEL_RXDFEAGCCTRL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFEAGCCTRL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEAGCCTRL_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFEAGCCTRL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEAGCCTRL}};

    if (GTHE3_CHANNEL_RXDFEAGCHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFEAGCHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEAGCHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFEAGCHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEAGCHOLD}};

    if (GTHE3_CHANNEL_RXDFEAGCOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFEAGCOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEAGCOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFEAGCOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEAGCOVRDEN}};

    if (GTHE3_CHANNEL_RXDFELFHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFELFHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFELFHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFELFHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFELFHOLD}};

    if (GTHE3_CHANNEL_RXDFELFOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFELFOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFELFOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFELFOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFELFOVRDEN}};

    if (GTHE3_CHANNEL_RXDFELPMRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFELPMRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFELPMRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFELPMRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFELPMRESET}};

    if (GTHE3_CHANNEL_RXDFETAP10HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP10HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP10HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP10HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP10HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP10OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP10OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP10OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP10OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP10OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP11HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP11HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP11HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP11HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP11HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP11OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP11OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP11OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP11OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP11OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP12HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP12HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP12HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP12HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP12HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP12OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP12OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP12OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP12OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP12OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP13HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP13HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP13HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP13HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP13HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP13OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP13OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP13OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP13OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP13OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP14HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP14HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP14HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP14HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP14HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP14OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP14OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP14OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP14OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP14OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP15HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP15HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP15HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP15HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP15HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP15OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP15OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP15OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP15OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP15OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP2HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP2HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP2HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP2HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP2HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP2OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP2OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP2OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP2OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP2OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP3HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP3HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP3HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP3HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP3HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP3OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP3OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP3OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP3OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP3OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP4HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP4HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP4HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP4HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP4HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP4OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP4OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP4OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP4OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP4OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP5HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP5HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP5HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP5HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP5HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP5OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP5OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP5OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP5OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP5OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP6HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP6HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP6HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP6HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP6HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP6OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP6OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP6OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP6OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP6OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP7HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP7HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP7HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP7HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP7HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP7OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP7OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP7OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP7OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP7OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP8HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP8HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP8HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP8HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP8HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP8OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP8OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP8OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP8OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP8OVRDEN}};

    if (GTHE3_CHANNEL_RXDFETAP9HOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP9HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP9HOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP9HOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP9HOLD}};

    if (GTHE3_CHANNEL_RXDFETAP9OVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFETAP9OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP9OVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFETAP9OVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFETAP9OVRDEN}};

    if (GTHE3_CHANNEL_RXDFEUTHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFEUTHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEUTHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFEUTHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEUTHOLD}};

    if (GTHE3_CHANNEL_RXDFEUTOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFEUTOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEUTOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFEUTOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEUTOVRDEN}};

    if (GTHE3_CHANNEL_RXDFEVPHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFEVPHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEVPHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFEVPHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEVPHOLD}};

    if (GTHE3_CHANNEL_RXDFEVPOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFEVPOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEVPOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFEVPOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEVPOVRDEN}};

    if (GTHE3_CHANNEL_RXDFEVSEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFEVSEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEVSEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFEVSEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEVSEN}};

    if (GTHE3_CHANNEL_RXDFEXYDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDFEXYDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEXYDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDFEXYDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDFEXYDEN}};

    if (GTHE3_CHANNEL_RXDLYBYPASS_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDLYBYPASS_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDLYBYPASS_VAL}};
    else
      assign GTHE3_CHANNEL_RXDLYBYPASS_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDLYBYPASS}};

    if (GTHE3_CHANNEL_RXDLYEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDLYEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDLYEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDLYEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDLYEN}};

    if (GTHE3_CHANNEL_RXDLYOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDLYOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDLYOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXDLYOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDLYOVRDEN}};

    if (GTHE3_CHANNEL_RXDLYSRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXDLYSRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDLYSRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXDLYSRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXDLYSRESET}};

    if (GTHE3_CHANNEL_RXELECIDLEMODE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXELECIDLEMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXELECIDLEMODE_VAL}};
    else
      assign GTHE3_CHANNEL_RXELECIDLEMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXELECIDLEMODE}};

    if (GTHE3_CHANNEL_RXGEARBOXSLIP_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXGEARBOXSLIP_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXGEARBOXSLIP_VAL}};
    else
      assign GTHE3_CHANNEL_RXGEARBOXSLIP_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXGEARBOXSLIP}};

    if (GTHE3_CHANNEL_RXLATCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLATCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLATCLK_VAL}};
    else
      assign GTHE3_CHANNEL_RXLATCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLATCLK}};

    if (GTHE3_CHANNEL_RXLPMEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLPMEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXLPMEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMEN}};

    if (GTHE3_CHANNEL_RXLPMGCHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLPMGCHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMGCHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXLPMGCHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMGCHOLD}};

    if (GTHE3_CHANNEL_RXLPMGCOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLPMGCOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMGCOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXLPMGCOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMGCOVRDEN}};

    if (GTHE3_CHANNEL_RXLPMHFHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLPMHFHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMHFHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXLPMHFHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMHFHOLD}};

    if (GTHE3_CHANNEL_RXLPMHFOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLPMHFOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMHFOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXLPMHFOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMHFOVRDEN}};

    if (GTHE3_CHANNEL_RXLPMLFHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLPMLFHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMLFHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXLPMLFHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMLFHOLD}};

    if (GTHE3_CHANNEL_RXLPMLFKLOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLPMLFKLOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMLFKLOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXLPMLFKLOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMLFKLOVRDEN}};

    if (GTHE3_CHANNEL_RXLPMOSHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLPMOSHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMOSHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXLPMOSHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMOSHOLD}};

    if (GTHE3_CHANNEL_RXLPMOSOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXLPMOSOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMOSOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXLPMOSOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXLPMOSOVRDEN}};

    if (GTHE3_CHANNEL_RXMCOMMAALIGNEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXMCOMMAALIGNEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXMCOMMAALIGNEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXMCOMMAALIGNEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXMCOMMAALIGNEN}};

    if (GTHE3_CHANNEL_RXMONITORSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXMONITORSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXMONITORSEL_VAL}};
    else
      assign GTHE3_CHANNEL_RXMONITORSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXMONITORSEL}};

    if (GTHE3_CHANNEL_RXOOBRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOOBRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOOBRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXOOBRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOOBRESET}};

    if (GTHE3_CHANNEL_RXOSCALRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOSCALRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSCALRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXOSCALRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSCALRESET}};

    if (GTHE3_CHANNEL_RXOSHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOSHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXOSHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSHOLD}};

    if (GTHE3_CHANNEL_RXOSINTCFG_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOSINTCFG_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTCFG_VAL}};
    else
      assign GTHE3_CHANNEL_RXOSINTCFG_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTCFG}};

    if (GTHE3_CHANNEL_RXOSINTEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOSINTEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXOSINTEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTEN}};

    if (GTHE3_CHANNEL_RXOSINTHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOSINTHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_RXOSINTHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTHOLD}};

    if (GTHE3_CHANNEL_RXOSINTOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOSINTOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXOSINTOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTOVRDEN}};

    if (GTHE3_CHANNEL_RXOSINTSTROBE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOSINTSTROBE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTSTROBE_VAL}};
    else
      assign GTHE3_CHANNEL_RXOSINTSTROBE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTSTROBE}};

    if (GTHE3_CHANNEL_RXOSINTTESTOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOSINTTESTOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTTESTOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXOSINTTESTOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSINTTESTOVRDEN}};

    if (GTHE3_CHANNEL_RXOSOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOSOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXOSOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOSOVRDEN}};

    if (GTHE3_CHANNEL_RXOUTCLKSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXOUTCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOUTCLKSEL_VAL}};
    else
      assign GTHE3_CHANNEL_RXOUTCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXOUTCLKSEL}};

    if (GTHE3_CHANNEL_RXPCOMMAALIGNEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPCOMMAALIGNEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPCOMMAALIGNEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXPCOMMAALIGNEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPCOMMAALIGNEN}};

    if (GTHE3_CHANNEL_RXPCSRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPCSRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPCSRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXPCSRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPCSRESET}};

    if (GTHE3_CHANNEL_RXPD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPD_VAL}};
    else
      assign GTHE3_CHANNEL_RXPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPD}};

    if (GTHE3_CHANNEL_RXPHALIGN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPHALIGN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHALIGN_VAL}};
    else
      assign GTHE3_CHANNEL_RXPHALIGN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHALIGN}};

    if (GTHE3_CHANNEL_RXPHALIGNEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPHALIGNEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHALIGNEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXPHALIGNEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHALIGNEN}};

    if (GTHE3_CHANNEL_RXPHDLYPD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPHDLYPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHDLYPD_VAL}};
    else
      assign GTHE3_CHANNEL_RXPHDLYPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHDLYPD}};

    if (GTHE3_CHANNEL_RXPHDLYRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPHDLYRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHDLYRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXPHDLYRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHDLYRESET}};

    if (GTHE3_CHANNEL_RXPHOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPHOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXPHOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPHOVRDEN}};

    if (GTHE3_CHANNEL_RXPLLCLKSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPLLCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPLLCLKSEL_VAL}};
    else
      assign GTHE3_CHANNEL_RXPLLCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPLLCLKSEL}};

    if (GTHE3_CHANNEL_RXPMARESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPMARESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPMARESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXPMARESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPMARESET}};

    if (GTHE3_CHANNEL_RXPOLARITY_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPOLARITY_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPOLARITY_VAL}};
    else
      assign GTHE3_CHANNEL_RXPOLARITY_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPOLARITY}};

    if (GTHE3_CHANNEL_RXPRBSCNTRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPRBSCNTRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPRBSCNTRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXPRBSCNTRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPRBSCNTRESET}};

    if (GTHE3_CHANNEL_RXPRBSSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPRBSSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPRBSSEL_VAL}};
    else
      assign GTHE3_CHANNEL_RXPRBSSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPRBSSEL}};

    if (GTHE3_CHANNEL_RXPROGDIVRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXPROGDIVRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPROGDIVRESET_VAL}};
    else
      assign GTHE3_CHANNEL_RXPROGDIVRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXPROGDIVRESET}};

    if (GTHE3_CHANNEL_RXQPIEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXQPIEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXQPIEN_VAL}};
    else
      assign GTHE3_CHANNEL_RXQPIEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXQPIEN}};

    if (GTHE3_CHANNEL_RXRATE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXRATE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXRATE_VAL}};
    else
      assign GTHE3_CHANNEL_RXRATE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXRATE}};

    if (GTHE3_CHANNEL_RXRATEMODE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXRATEMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXRATEMODE_VAL}};
    else
      assign GTHE3_CHANNEL_RXRATEMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXRATEMODE}};

    if (GTHE3_CHANNEL_RXSLIDE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXSLIDE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSLIDE_VAL}};
    else
      assign GTHE3_CHANNEL_RXSLIDE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSLIDE}};

    if (GTHE3_CHANNEL_RXSLIPOUTCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXSLIPOUTCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSLIPOUTCLK_VAL}};
    else
      assign GTHE3_CHANNEL_RXSLIPOUTCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSLIPOUTCLK}};

    if (GTHE3_CHANNEL_RXSLIPPMA_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXSLIPPMA_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSLIPPMA_VAL}};
    else
      assign GTHE3_CHANNEL_RXSLIPPMA_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSLIPPMA}};

    if (GTHE3_CHANNEL_RXSYNCALLIN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXSYNCALLIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSYNCALLIN_VAL}};
    else
      assign GTHE3_CHANNEL_RXSYNCALLIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSYNCALLIN}};

    if (GTHE3_CHANNEL_RXSYNCIN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXSYNCIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSYNCIN_VAL}};
    else
      assign GTHE3_CHANNEL_RXSYNCIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSYNCIN}};

    if (GTHE3_CHANNEL_RXSYNCMODE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXSYNCMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSYNCMODE_VAL}};
    else
      assign GTHE3_CHANNEL_RXSYNCMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSYNCMODE}};

    if (GTHE3_CHANNEL_RXSYSCLKSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXSYSCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSYSCLKSEL_VAL}};
    else
      assign GTHE3_CHANNEL_RXSYSCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXSYSCLKSEL}};

    if (GTHE3_CHANNEL_RXUSERRDY_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXUSERRDY_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXUSERRDY_VAL}};
    else
      assign GTHE3_CHANNEL_RXUSERRDY_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXUSERRDY}};

    if (GTHE3_CHANNEL_RXUSRCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXUSRCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXUSRCLK_VAL}};
    else
      assign GTHE3_CHANNEL_RXUSRCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXUSRCLK}};

    if (GTHE3_CHANNEL_RXUSRCLK2_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_RXUSRCLK2_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXUSRCLK2_VAL}};
    else
      assign GTHE3_CHANNEL_RXUSRCLK2_int = {NUM_CHANNELS{GTHE3_CHANNEL_RXUSRCLK2}};

    if (GTHE3_CHANNEL_SIGVALIDCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_SIGVALIDCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_SIGVALIDCLK_VAL}};
    else
      assign GTHE3_CHANNEL_SIGVALIDCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_SIGVALIDCLK}};

    if (GTHE3_CHANNEL_TSTIN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TSTIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TSTIN_VAL}};
    else
      assign GTHE3_CHANNEL_TSTIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TSTIN}};

    if (GTHE3_CHANNEL_TX8B10BBYPASS_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TX8B10BBYPASS_int = {NUM_CHANNELS{GTHE3_CHANNEL_TX8B10BBYPASS_VAL}};
    else
      assign GTHE3_CHANNEL_TX8B10BBYPASS_int = {NUM_CHANNELS{GTHE3_CHANNEL_TX8B10BBYPASS}};

    if (GTHE3_CHANNEL_TX8B10BEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TX8B10BEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TX8B10BEN_VAL}};
    else
      assign GTHE3_CHANNEL_TX8B10BEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TX8B10BEN}};

    if (GTHE3_CHANNEL_TXBUFDIFFCTRL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXBUFDIFFCTRL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXBUFDIFFCTRL_VAL}};
    else
      assign GTHE3_CHANNEL_TXBUFDIFFCTRL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXBUFDIFFCTRL}};

    if (GTHE3_CHANNEL_TXCOMINIT_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXCOMINIT_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCOMINIT_VAL}};
    else
      assign GTHE3_CHANNEL_TXCOMINIT_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCOMINIT}};

    if (GTHE3_CHANNEL_TXCOMSAS_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXCOMSAS_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCOMSAS_VAL}};
    else
      assign GTHE3_CHANNEL_TXCOMSAS_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCOMSAS}};

    if (GTHE3_CHANNEL_TXCOMWAKE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXCOMWAKE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCOMWAKE_VAL}};
    else
      assign GTHE3_CHANNEL_TXCOMWAKE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCOMWAKE}};

    if (GTHE3_CHANNEL_TXCTRL0_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXCTRL0_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCTRL0_VAL}};
    else
      assign GTHE3_CHANNEL_TXCTRL0_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCTRL0}};

    if (GTHE3_CHANNEL_TXCTRL1_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXCTRL1_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCTRL1_VAL}};
    else
      assign GTHE3_CHANNEL_TXCTRL1_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCTRL1}};

    if (GTHE3_CHANNEL_TXCTRL2_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXCTRL2_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCTRL2_VAL}};
    else
      assign GTHE3_CHANNEL_TXCTRL2_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXCTRL2}};

    if (GTHE3_CHANNEL_TXDATA_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDATA_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDATA_VAL}};
    else
      assign GTHE3_CHANNEL_TXDATA_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDATA}};

    if (GTHE3_CHANNEL_TXDATAEXTENDRSVD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDATAEXTENDRSVD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDATAEXTENDRSVD_VAL}};
    else
      assign GTHE3_CHANNEL_TXDATAEXTENDRSVD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDATAEXTENDRSVD}};

    if (GTHE3_CHANNEL_TXDEEMPH_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDEEMPH_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDEEMPH_VAL}};
    else
      assign GTHE3_CHANNEL_TXDEEMPH_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDEEMPH}};

    if (GTHE3_CHANNEL_TXDETECTRX_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDETECTRX_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDETECTRX_VAL}};
    else
      assign GTHE3_CHANNEL_TXDETECTRX_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDETECTRX}};

    if (GTHE3_CHANNEL_TXDIFFCTRL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDIFFCTRL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDIFFCTRL_VAL}};
    else
      assign GTHE3_CHANNEL_TXDIFFCTRL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDIFFCTRL}};

    if (GTHE3_CHANNEL_TXDIFFPD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDIFFPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDIFFPD_VAL}};
    else
      assign GTHE3_CHANNEL_TXDIFFPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDIFFPD}};

    if (GTHE3_CHANNEL_TXDLYBYPASS_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDLYBYPASS_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYBYPASS_VAL}};
    else
      assign GTHE3_CHANNEL_TXDLYBYPASS_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYBYPASS}};

    if (GTHE3_CHANNEL_TXDLYEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDLYEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYEN_VAL}};
    else
      assign GTHE3_CHANNEL_TXDLYEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYEN}};

    if (GTHE3_CHANNEL_TXDLYHOLD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDLYHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYHOLD_VAL}};
    else
      assign GTHE3_CHANNEL_TXDLYHOLD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYHOLD}};

    if (GTHE3_CHANNEL_TXDLYOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDLYOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_TXDLYOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYOVRDEN}};

    if (GTHE3_CHANNEL_TXDLYSRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDLYSRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYSRESET_VAL}};
    else
      assign GTHE3_CHANNEL_TXDLYSRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYSRESET}};

    if (GTHE3_CHANNEL_TXDLYUPDOWN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXDLYUPDOWN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYUPDOWN_VAL}};
    else
      assign GTHE3_CHANNEL_TXDLYUPDOWN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXDLYUPDOWN}};

    if (GTHE3_CHANNEL_TXELECIDLE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXELECIDLE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXELECIDLE_VAL}};
    else
      assign GTHE3_CHANNEL_TXELECIDLE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXELECIDLE}};

    if (GTHE3_CHANNEL_TXHEADER_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXHEADER_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXHEADER_VAL}};
    else
      assign GTHE3_CHANNEL_TXHEADER_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXHEADER}};

    if (GTHE3_CHANNEL_TXINHIBIT_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXINHIBIT_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXINHIBIT_VAL}};
    else
      assign GTHE3_CHANNEL_TXINHIBIT_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXINHIBIT}};

    if (GTHE3_CHANNEL_TXLATCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXLATCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXLATCLK_VAL}};
    else
      assign GTHE3_CHANNEL_TXLATCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXLATCLK}};

    if (GTHE3_CHANNEL_TXMAINCURSOR_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXMAINCURSOR_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXMAINCURSOR_VAL}};
    else
      assign GTHE3_CHANNEL_TXMAINCURSOR_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXMAINCURSOR}};

    if (GTHE3_CHANNEL_TXMARGIN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXMARGIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXMARGIN_VAL}};
    else
      assign GTHE3_CHANNEL_TXMARGIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXMARGIN}};

    if (GTHE3_CHANNEL_TXOUTCLKSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXOUTCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXOUTCLKSEL_VAL}};
    else
      assign GTHE3_CHANNEL_TXOUTCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXOUTCLKSEL}};

    if (GTHE3_CHANNEL_TXPCSRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPCSRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPCSRESET_VAL}};
    else
      assign GTHE3_CHANNEL_TXPCSRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPCSRESET}};

    if (GTHE3_CHANNEL_TXPD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPD_VAL}};
    else
      assign GTHE3_CHANNEL_TXPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPD}};

    if (GTHE3_CHANNEL_TXPDELECIDLEMODE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPDELECIDLEMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPDELECIDLEMODE_VAL}};
    else
      assign GTHE3_CHANNEL_TXPDELECIDLEMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPDELECIDLEMODE}};

    if (GTHE3_CHANNEL_TXPHALIGN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPHALIGN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHALIGN_VAL}};
    else
      assign GTHE3_CHANNEL_TXPHALIGN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHALIGN}};

    if (GTHE3_CHANNEL_TXPHALIGNEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPHALIGNEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHALIGNEN_VAL}};
    else
      assign GTHE3_CHANNEL_TXPHALIGNEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHALIGNEN}};

    if (GTHE3_CHANNEL_TXPHDLYPD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPHDLYPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHDLYPD_VAL}};
    else
      assign GTHE3_CHANNEL_TXPHDLYPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHDLYPD}};

    if (GTHE3_CHANNEL_TXPHDLYRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPHDLYRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHDLYRESET_VAL}};
    else
      assign GTHE3_CHANNEL_TXPHDLYRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHDLYRESET}};

    if (GTHE3_CHANNEL_TXPHDLYTSTCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPHDLYTSTCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHDLYTSTCLK_VAL}};
    else
      assign GTHE3_CHANNEL_TXPHDLYTSTCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHDLYTSTCLK}};

    if (GTHE3_CHANNEL_TXPHINIT_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPHINIT_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHINIT_VAL}};
    else
      assign GTHE3_CHANNEL_TXPHINIT_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHINIT}};

    if (GTHE3_CHANNEL_TXPHOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPHOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_TXPHOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPHOVRDEN}};

    if (GTHE3_CHANNEL_TXPIPPMEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPIPPMEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMEN_VAL}};
    else
      assign GTHE3_CHANNEL_TXPIPPMEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMEN}};

    if (GTHE3_CHANNEL_TXPIPPMOVRDEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPIPPMOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMOVRDEN_VAL}};
    else
      assign GTHE3_CHANNEL_TXPIPPMOVRDEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMOVRDEN}};

    if (GTHE3_CHANNEL_TXPIPPMPD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPIPPMPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMPD_VAL}};
    else
      assign GTHE3_CHANNEL_TXPIPPMPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMPD}};

    if (GTHE3_CHANNEL_TXPIPPMSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPIPPMSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMSEL_VAL}};
    else
      assign GTHE3_CHANNEL_TXPIPPMSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMSEL}};

    if (GTHE3_CHANNEL_TXPIPPMSTEPSIZE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPIPPMSTEPSIZE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMSTEPSIZE_VAL}};
    else
      assign GTHE3_CHANNEL_TXPIPPMSTEPSIZE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPIPPMSTEPSIZE}};

    if (GTHE3_CHANNEL_TXPISOPD_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPISOPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPISOPD_VAL}};
    else
      assign GTHE3_CHANNEL_TXPISOPD_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPISOPD}};

    if (GTHE3_CHANNEL_TXPLLCLKSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPLLCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPLLCLKSEL_VAL}};
    else
      assign GTHE3_CHANNEL_TXPLLCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPLLCLKSEL}};

    if (GTHE3_CHANNEL_TXPMARESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPMARESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPMARESET_VAL}};
    else
      assign GTHE3_CHANNEL_TXPMARESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPMARESET}};

    if (GTHE3_CHANNEL_TXPOLARITY_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPOLARITY_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPOLARITY_VAL}};
    else
      assign GTHE3_CHANNEL_TXPOLARITY_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPOLARITY}};

    if (GTHE3_CHANNEL_TXPOSTCURSOR_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPOSTCURSOR_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPOSTCURSOR_VAL}};
    else
      assign GTHE3_CHANNEL_TXPOSTCURSOR_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPOSTCURSOR}};

    if (GTHE3_CHANNEL_TXPOSTCURSORINV_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPOSTCURSORINV_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPOSTCURSORINV_VAL}};
    else
      assign GTHE3_CHANNEL_TXPOSTCURSORINV_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPOSTCURSORINV}};

    if (GTHE3_CHANNEL_TXPRBSFORCEERR_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPRBSFORCEERR_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPRBSFORCEERR_VAL}};
    else
      assign GTHE3_CHANNEL_TXPRBSFORCEERR_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPRBSFORCEERR}};

    if (GTHE3_CHANNEL_TXPRBSSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPRBSSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPRBSSEL_VAL}};
    else
      assign GTHE3_CHANNEL_TXPRBSSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPRBSSEL}};

    if (GTHE3_CHANNEL_TXPRECURSOR_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPRECURSOR_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPRECURSOR_VAL}};
    else
      assign GTHE3_CHANNEL_TXPRECURSOR_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPRECURSOR}};

    if (GTHE3_CHANNEL_TXPRECURSORINV_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPRECURSORINV_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPRECURSORINV_VAL}};
    else
      assign GTHE3_CHANNEL_TXPRECURSORINV_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPRECURSORINV}};

    if (GTHE3_CHANNEL_TXPROGDIVRESET_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXPROGDIVRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPROGDIVRESET_VAL}};
    else
      assign GTHE3_CHANNEL_TXPROGDIVRESET_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXPROGDIVRESET}};

    if (GTHE3_CHANNEL_TXQPIBIASEN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXQPIBIASEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXQPIBIASEN_VAL}};
    else
      assign GTHE3_CHANNEL_TXQPIBIASEN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXQPIBIASEN}};

    if (GTHE3_CHANNEL_TXQPISTRONGPDOWN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXQPISTRONGPDOWN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXQPISTRONGPDOWN_VAL}};
    else
      assign GTHE3_CHANNEL_TXQPISTRONGPDOWN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXQPISTRONGPDOWN}};

    if (GTHE3_CHANNEL_TXQPIWEAKPUP_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXQPIWEAKPUP_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXQPIWEAKPUP_VAL}};
    else
      assign GTHE3_CHANNEL_TXQPIWEAKPUP_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXQPIWEAKPUP}};

    if (GTHE3_CHANNEL_TXRATE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXRATE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXRATE_VAL}};
    else
      assign GTHE3_CHANNEL_TXRATE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXRATE}};

    if (GTHE3_CHANNEL_TXRATEMODE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXRATEMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXRATEMODE_VAL}};
    else
      assign GTHE3_CHANNEL_TXRATEMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXRATEMODE}};

    if (GTHE3_CHANNEL_TXSEQUENCE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXSEQUENCE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSEQUENCE_VAL}};
    else
      assign GTHE3_CHANNEL_TXSEQUENCE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSEQUENCE}};

    if (GTHE3_CHANNEL_TXSWING_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXSWING_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSWING_VAL}};
    else
      assign GTHE3_CHANNEL_TXSWING_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSWING}};

    if (GTHE3_CHANNEL_TXSYNCALLIN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXSYNCALLIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSYNCALLIN_VAL}};
    else
      assign GTHE3_CHANNEL_TXSYNCALLIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSYNCALLIN}};

    if (GTHE3_CHANNEL_TXSYNCIN_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXSYNCIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSYNCIN_VAL}};
    else
      assign GTHE3_CHANNEL_TXSYNCIN_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSYNCIN}};

    if (GTHE3_CHANNEL_TXSYNCMODE_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXSYNCMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSYNCMODE_VAL}};
    else
      assign GTHE3_CHANNEL_TXSYNCMODE_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSYNCMODE}};

    if (GTHE3_CHANNEL_TXSYSCLKSEL_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXSYSCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSYSCLKSEL_VAL}};
    else
      assign GTHE3_CHANNEL_TXSYSCLKSEL_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXSYSCLKSEL}};

    if (GTHE3_CHANNEL_TXUSERRDY_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXUSERRDY_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXUSERRDY_VAL}};
    else
      assign GTHE3_CHANNEL_TXUSERRDY_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXUSERRDY}};

    if (GTHE3_CHANNEL_TXUSRCLK_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXUSRCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXUSRCLK_VAL}};
    else
      assign GTHE3_CHANNEL_TXUSRCLK_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXUSRCLK}};

    if (GTHE3_CHANNEL_TXUSRCLK2_TIE_EN == 1'b1)
      assign GTHE3_CHANNEL_TXUSRCLK2_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXUSRCLK2_VAL}};
    else
      assign GTHE3_CHANNEL_TXUSRCLK2_int = {NUM_CHANNELS{GTHE3_CHANNEL_TXUSRCLK2}};

    // generate the appropriate number of GTHE3_CHANNEL primitive instances, mapping parameters and ports
    genvar ch;
    for (ch = 0; ch < NUM_CHANNELS; ch = ch + 1) begin : gen_gthe3_channel_inst

      GTHE3_CHANNEL #(
        .ACJTAG_DEBUG_MODE            (GTHE3_CHANNEL_ACJTAG_DEBUG_MODE           ),
        .ACJTAG_MODE                  (GTHE3_CHANNEL_ACJTAG_MODE                 ),
        .ACJTAG_RESET                 (GTHE3_CHANNEL_ACJTAG_RESET                ),
        .ADAPT_CFG0                   (GTHE3_CHANNEL_ADAPT_CFG0                  ),
        .ADAPT_CFG1                   (GTHE3_CHANNEL_ADAPT_CFG1                  ),
        .ALIGN_COMMA_DOUBLE           (GTHE3_CHANNEL_ALIGN_COMMA_DOUBLE          ),
        .ALIGN_COMMA_ENABLE           (GTHE3_CHANNEL_ALIGN_COMMA_ENABLE          ),
        .ALIGN_COMMA_WORD             (GTHE3_CHANNEL_ALIGN_COMMA_WORD            ),
        .ALIGN_MCOMMA_DET             (GTHE3_CHANNEL_ALIGN_MCOMMA_DET            ),
        .ALIGN_MCOMMA_VALUE           (GTHE3_CHANNEL_ALIGN_MCOMMA_VALUE          ),
        .ALIGN_PCOMMA_DET             (GTHE3_CHANNEL_ALIGN_PCOMMA_DET            ),
        .ALIGN_PCOMMA_VALUE           (GTHE3_CHANNEL_ALIGN_PCOMMA_VALUE          ),
        .A_RXOSCALRESET               (GTHE3_CHANNEL_A_RXOSCALRESET              ),
        .A_RXPROGDIVRESET             (GTHE3_CHANNEL_A_RXPROGDIVRESET            ),
        .A_TXPROGDIVRESET             (GTHE3_CHANNEL_A_TXPROGDIVRESET            ),
        .CBCC_DATA_SOURCE_SEL         (GTHE3_CHANNEL_CBCC_DATA_SOURCE_SEL        ),
        .CDR_SWAP_MODE_EN             (GTHE3_CHANNEL_CDR_SWAP_MODE_EN            ),
        .CHAN_BOND_KEEP_ALIGN         (GTHE3_CHANNEL_CHAN_BOND_KEEP_ALIGN        ),
        .CHAN_BOND_MAX_SKEW           (GTHE3_CHANNEL_CHAN_BOND_MAX_SKEW          ),
        .CHAN_BOND_SEQ_1_1            (GTHE3_CHANNEL_CHAN_BOND_SEQ_1_1           ),
        .CHAN_BOND_SEQ_1_2            (GTHE3_CHANNEL_CHAN_BOND_SEQ_1_2           ),
        .CHAN_BOND_SEQ_1_3            (GTHE3_CHANNEL_CHAN_BOND_SEQ_1_3           ),
        .CHAN_BOND_SEQ_1_4            (GTHE3_CHANNEL_CHAN_BOND_SEQ_1_4           ),
        .CHAN_BOND_SEQ_1_ENABLE       (GTHE3_CHANNEL_CHAN_BOND_SEQ_1_ENABLE      ),
        .CHAN_BOND_SEQ_2_1            (GTHE3_CHANNEL_CHAN_BOND_SEQ_2_1           ),
        .CHAN_BOND_SEQ_2_2            (GTHE3_CHANNEL_CHAN_BOND_SEQ_2_2           ),
        .CHAN_BOND_SEQ_2_3            (GTHE3_CHANNEL_CHAN_BOND_SEQ_2_3           ),
        .CHAN_BOND_SEQ_2_4            (GTHE3_CHANNEL_CHAN_BOND_SEQ_2_4           ),
        .CHAN_BOND_SEQ_2_ENABLE       (GTHE3_CHANNEL_CHAN_BOND_SEQ_2_ENABLE      ),
        .CHAN_BOND_SEQ_2_USE          (GTHE3_CHANNEL_CHAN_BOND_SEQ_2_USE         ),
        .CHAN_BOND_SEQ_LEN            (GTHE3_CHANNEL_CHAN_BOND_SEQ_LEN           ),
        .CLK_CORRECT_USE              (GTHE3_CHANNEL_CLK_CORRECT_USE             ),
        .CLK_COR_KEEP_IDLE            (GTHE3_CHANNEL_CLK_COR_KEEP_IDLE           ),
        .CLK_COR_MAX_LAT              (GTHE3_CHANNEL_CLK_COR_MAX_LAT             ),
        .CLK_COR_MIN_LAT              (GTHE3_CHANNEL_CLK_COR_MIN_LAT             ),
        .CLK_COR_PRECEDENCE           (GTHE3_CHANNEL_CLK_COR_PRECEDENCE          ),
        .CLK_COR_REPEAT_WAIT          (GTHE3_CHANNEL_CLK_COR_REPEAT_WAIT         ),
        .CLK_COR_SEQ_1_1              (GTHE3_CHANNEL_CLK_COR_SEQ_1_1             ),
        .CLK_COR_SEQ_1_2              (GTHE3_CHANNEL_CLK_COR_SEQ_1_2             ),
        .CLK_COR_SEQ_1_3              (GTHE3_CHANNEL_CLK_COR_SEQ_1_3             ),
        .CLK_COR_SEQ_1_4              (GTHE3_CHANNEL_CLK_COR_SEQ_1_4             ),
        .CLK_COR_SEQ_1_ENABLE         (GTHE3_CHANNEL_CLK_COR_SEQ_1_ENABLE        ),
        .CLK_COR_SEQ_2_1              (GTHE3_CHANNEL_CLK_COR_SEQ_2_1             ),
        .CLK_COR_SEQ_2_2              (GTHE3_CHANNEL_CLK_COR_SEQ_2_2             ),
        .CLK_COR_SEQ_2_3              (GTHE3_CHANNEL_CLK_COR_SEQ_2_3             ),
        .CLK_COR_SEQ_2_4              (GTHE3_CHANNEL_CLK_COR_SEQ_2_4             ),
        .CLK_COR_SEQ_2_ENABLE         (GTHE3_CHANNEL_CLK_COR_SEQ_2_ENABLE        ),
        .CLK_COR_SEQ_2_USE            (GTHE3_CHANNEL_CLK_COR_SEQ_2_USE           ),
        .CLK_COR_SEQ_LEN              (GTHE3_CHANNEL_CLK_COR_SEQ_LEN             ),
        .CPLL_CFG0                    (GTHE3_CHANNEL_CPLL_CFG0                   ),
        .CPLL_CFG1                    (GTHE3_CHANNEL_CPLL_CFG1                   ),
        .CPLL_CFG2                    (GTHE3_CHANNEL_CPLL_CFG2                   ),
        .CPLL_CFG3                    (GTHE3_CHANNEL_CPLL_CFG3                   ),
        .CPLL_FBDIV                   (GTHE3_CHANNEL_CPLL_FBDIV                  ),
        .CPLL_FBDIV_45                (GTHE3_CHANNEL_CPLL_FBDIV_45               ),
        .CPLL_INIT_CFG0               (GTHE3_CHANNEL_CPLL_INIT_CFG0              ),
        .CPLL_INIT_CFG1               (GTHE3_CHANNEL_CPLL_INIT_CFG1              ),
        .CPLL_LOCK_CFG                (GTHE3_CHANNEL_CPLL_LOCK_CFG               ),
        .CPLL_REFCLK_DIV              (GTHE3_CHANNEL_CPLL_REFCLK_DIV             ),
        .DDI_CTRL                     (GTHE3_CHANNEL_DDI_CTRL                    ),
        .DDI_REALIGN_WAIT             (GTHE3_CHANNEL_DDI_REALIGN_WAIT            ),
        .DEC_MCOMMA_DETECT            (GTHE3_CHANNEL_DEC_MCOMMA_DETECT           ),
        .DEC_PCOMMA_DETECT            (GTHE3_CHANNEL_DEC_PCOMMA_DETECT           ),
        .DEC_VALID_COMMA_ONLY         (GTHE3_CHANNEL_DEC_VALID_COMMA_ONLY        ),
        .DFE_D_X_REL_POS              (GTHE3_CHANNEL_DFE_D_X_REL_POS             ),
        .DFE_VCM_COMP_EN              (GTHE3_CHANNEL_DFE_VCM_COMP_EN             ),
        .DMONITOR_CFG0                (GTHE3_CHANNEL_DMONITOR_CFG0               ),
        .DMONITOR_CFG1                (GTHE3_CHANNEL_DMONITOR_CFG1               ),
        .ES_CLK_PHASE_SEL             (GTHE3_CHANNEL_ES_CLK_PHASE_SEL            ),
        .ES_CONTROL                   (GTHE3_CHANNEL_ES_CONTROL                  ),
        .ES_ERRDET_EN                 (GTHE3_CHANNEL_ES_ERRDET_EN                ),
        .ES_EYE_SCAN_EN               (GTHE3_CHANNEL_ES_EYE_SCAN_EN              ),
        .ES_HORZ_OFFSET               (GTHE3_CHANNEL_ES_HORZ_OFFSET              ),
        .ES_PMA_CFG                   (GTHE3_CHANNEL_ES_PMA_CFG                  ),
        .ES_PRESCALE                  (GTHE3_CHANNEL_ES_PRESCALE                 ),
        .ES_QUALIFIER0                (GTHE3_CHANNEL_ES_QUALIFIER0               ),
        .ES_QUALIFIER1                (GTHE3_CHANNEL_ES_QUALIFIER1               ),
        .ES_QUALIFIER2                (GTHE3_CHANNEL_ES_QUALIFIER2               ),
        .ES_QUALIFIER3                (GTHE3_CHANNEL_ES_QUALIFIER3               ),
        .ES_QUALIFIER4                (GTHE3_CHANNEL_ES_QUALIFIER4               ),
        .ES_QUAL_MASK0                (GTHE3_CHANNEL_ES_QUAL_MASK0               ),
        .ES_QUAL_MASK1                (GTHE3_CHANNEL_ES_QUAL_MASK1               ),
        .ES_QUAL_MASK2                (GTHE3_CHANNEL_ES_QUAL_MASK2               ),
        .ES_QUAL_MASK3                (GTHE3_CHANNEL_ES_QUAL_MASK3               ),
        .ES_QUAL_MASK4                (GTHE3_CHANNEL_ES_QUAL_MASK4               ),
        .ES_SDATA_MASK0               (GTHE3_CHANNEL_ES_SDATA_MASK0              ),
        .ES_SDATA_MASK1               (GTHE3_CHANNEL_ES_SDATA_MASK1              ),
        .ES_SDATA_MASK2               (GTHE3_CHANNEL_ES_SDATA_MASK2              ),
        .ES_SDATA_MASK3               (GTHE3_CHANNEL_ES_SDATA_MASK3              ),
        .ES_SDATA_MASK4               (GTHE3_CHANNEL_ES_SDATA_MASK4              ),
        .EVODD_PHI_CFG                (GTHE3_CHANNEL_EVODD_PHI_CFG               ),
        .EYE_SCAN_SWAP_EN             (GTHE3_CHANNEL_EYE_SCAN_SWAP_EN            ),
        .FTS_DESKEW_SEQ_ENABLE        (GTHE3_CHANNEL_FTS_DESKEW_SEQ_ENABLE       ),
        .FTS_LANE_DESKEW_CFG          (GTHE3_CHANNEL_FTS_LANE_DESKEW_CFG         ),
        .FTS_LANE_DESKEW_EN           (GTHE3_CHANNEL_FTS_LANE_DESKEW_EN          ),
        .GEARBOX_MODE                 (GTHE3_CHANNEL_GEARBOX_MODE                ),
        .GM_BIAS_SELECT               (GTHE3_CHANNEL_GM_BIAS_SELECT              ),
        .LOCAL_MASTER                 (GTHE3_CHANNEL_LOCAL_MASTER                ),
        .OOBDIVCTL                    (GTHE3_CHANNEL_OOBDIVCTL                   ),
        .OOB_PWRUP                    (GTHE3_CHANNEL_OOB_PWRUP                   ),
        .PCI3_AUTO_REALIGN            (GTHE3_CHANNEL_PCI3_AUTO_REALIGN           ),
        .PCI3_PIPE_RX_ELECIDLE        (GTHE3_CHANNEL_PCI3_PIPE_RX_ELECIDLE       ),
        .PCI3_RX_ASYNC_EBUF_BYPASS    (GTHE3_CHANNEL_PCI3_RX_ASYNC_EBUF_BYPASS   ),
        .PCI3_RX_ELECIDLE_EI2_ENABLE  (GTHE3_CHANNEL_PCI3_RX_ELECIDLE_EI2_ENABLE ),
        .PCI3_RX_ELECIDLE_H2L_COUNT   (GTHE3_CHANNEL_PCI3_RX_ELECIDLE_H2L_COUNT  ),
        .PCI3_RX_ELECIDLE_H2L_DISABLE (GTHE3_CHANNEL_PCI3_RX_ELECIDLE_H2L_DISABLE),
        .PCI3_RX_ELECIDLE_HI_COUNT    (GTHE3_CHANNEL_PCI3_RX_ELECIDLE_HI_COUNT   ),
        .PCI3_RX_ELECIDLE_LP4_DISABLE (GTHE3_CHANNEL_PCI3_RX_ELECIDLE_LP4_DISABLE),
        .PCI3_RX_FIFO_DISABLE         (GTHE3_CHANNEL_PCI3_RX_FIFO_DISABLE        ),
        .PCIE_BUFG_DIV_CTRL           (GTHE3_CHANNEL_PCIE_BUFG_DIV_CTRL          ),
        .PCIE_RXPCS_CFG_GEN3          (GTHE3_CHANNEL_PCIE_RXPCS_CFG_GEN3         ),
        .PCIE_RXPMA_CFG               (GTHE3_CHANNEL_PCIE_RXPMA_CFG              ),
        .PCIE_TXPCS_CFG_GEN3          (GTHE3_CHANNEL_PCIE_TXPCS_CFG_GEN3         ),
        .PCIE_TXPMA_CFG               (GTHE3_CHANNEL_PCIE_TXPMA_CFG              ),
        .PCS_PCIE_EN                  (GTHE3_CHANNEL_PCS_PCIE_EN                 ),
        .PCS_RSVD0                    (GTHE3_CHANNEL_PCS_RSVD0                   ),
        .PCS_RSVD1                    (GTHE3_CHANNEL_PCS_RSVD1                   ),
        .PD_TRANS_TIME_FROM_P2        (GTHE3_CHANNEL_PD_TRANS_TIME_FROM_P2       ),
        .PD_TRANS_TIME_NONE_P2        (GTHE3_CHANNEL_PD_TRANS_TIME_NONE_P2       ),
        .PD_TRANS_TIME_TO_P2          (GTHE3_CHANNEL_PD_TRANS_TIME_TO_P2         ),
        .PLL_SEL_MODE_GEN12           (GTHE3_CHANNEL_PLL_SEL_MODE_GEN12          ),
        .PLL_SEL_MODE_GEN3            (GTHE3_CHANNEL_PLL_SEL_MODE_GEN3           ),
        .PMA_RSV1                     (GTHE3_CHANNEL_PMA_RSV1                    ),
        .PROCESS_PAR                  (GTHE3_CHANNEL_PROCESS_PAR                 ),
        .RATE_SW_USE_DRP              (GTHE3_CHANNEL_RATE_SW_USE_DRP             ),
        .RESET_POWERSAVE_DISABLE      (GTHE3_CHANNEL_RESET_POWERSAVE_DISABLE     ),
        .RXBUFRESET_TIME              (GTHE3_CHANNEL_RXBUFRESET_TIME             ),
        .RXBUF_ADDR_MODE              (GTHE3_CHANNEL_RXBUF_ADDR_MODE             ),
        .RXBUF_EIDLE_HI_CNT           (GTHE3_CHANNEL_RXBUF_EIDLE_HI_CNT          ),
        .RXBUF_EIDLE_LO_CNT           (GTHE3_CHANNEL_RXBUF_EIDLE_LO_CNT          ),
        .RXBUF_EN                     (GTHE3_CHANNEL_RXBUF_EN                    ),
        .RXBUF_RESET_ON_CB_CHANGE     (GTHE3_CHANNEL_RXBUF_RESET_ON_CB_CHANGE    ),
        .RXBUF_RESET_ON_COMMAALIGN    (GTHE3_CHANNEL_RXBUF_RESET_ON_COMMAALIGN   ),
        .RXBUF_RESET_ON_EIDLE         (GTHE3_CHANNEL_RXBUF_RESET_ON_EIDLE        ),
        .RXBUF_RESET_ON_RATE_CHANGE   (GTHE3_CHANNEL_RXBUF_RESET_ON_RATE_CHANGE  ),
        .RXBUF_THRESH_OVFLW           (GTHE3_CHANNEL_RXBUF_THRESH_OVFLW          ),
        .RXBUF_THRESH_OVRD            (GTHE3_CHANNEL_RXBUF_THRESH_OVRD           ),
        .RXBUF_THRESH_UNDFLW          (GTHE3_CHANNEL_RXBUF_THRESH_UNDFLW         ),
        .RXCDRFREQRESET_TIME          (GTHE3_CHANNEL_RXCDRFREQRESET_TIME         ),
        .RXCDRPHRESET_TIME            (GTHE3_CHANNEL_RXCDRPHRESET_TIME           ),
        .RXCDR_CFG0                   (GTHE3_CHANNEL_RXCDR_CFG0                  ),
        .RXCDR_CFG0_GEN3              (GTHE3_CHANNEL_RXCDR_CFG0_GEN3             ),
        .RXCDR_CFG1                   (GTHE3_CHANNEL_RXCDR_CFG1                  ),
        .RXCDR_CFG1_GEN3              (GTHE3_CHANNEL_RXCDR_CFG1_GEN3             ),
        .RXCDR_CFG2                   (GTHE3_CHANNEL_RXCDR_CFG2                  ),
        .RXCDR_CFG2_GEN3              (GTHE3_CHANNEL_RXCDR_CFG2_GEN3             ),
        .RXCDR_CFG3                   (GTHE3_CHANNEL_RXCDR_CFG3                  ),
        .RXCDR_CFG3_GEN3              (GTHE3_CHANNEL_RXCDR_CFG3_GEN3             ),
        .RXCDR_CFG4                   (GTHE3_CHANNEL_RXCDR_CFG4                  ),
        .RXCDR_CFG4_GEN3              (GTHE3_CHANNEL_RXCDR_CFG4_GEN3             ),
        .RXCDR_CFG5                   (GTHE3_CHANNEL_RXCDR_CFG5                  ),
        .RXCDR_CFG5_GEN3              (GTHE3_CHANNEL_RXCDR_CFG5_GEN3             ),
        .RXCDR_FR_RESET_ON_EIDLE      (GTHE3_CHANNEL_RXCDR_FR_RESET_ON_EIDLE     ),
        .RXCDR_HOLD_DURING_EIDLE      (GTHE3_CHANNEL_RXCDR_HOLD_DURING_EIDLE     ),
        .RXCDR_LOCK_CFG0              (GTHE3_CHANNEL_RXCDR_LOCK_CFG0             ),
        .RXCDR_LOCK_CFG1              (GTHE3_CHANNEL_RXCDR_LOCK_CFG1             ),
        .RXCDR_LOCK_CFG2              (GTHE3_CHANNEL_RXCDR_LOCK_CFG2             ),
        .RXCDR_PH_RESET_ON_EIDLE      (GTHE3_CHANNEL_RXCDR_PH_RESET_ON_EIDLE     ),
        .RXCFOK_CFG0                  (GTHE3_CHANNEL_RXCFOK_CFG0                 ),
        .RXCFOK_CFG1                  (GTHE3_CHANNEL_RXCFOK_CFG1                 ),
        .RXCFOK_CFG2                  (GTHE3_CHANNEL_RXCFOK_CFG2                 ),
        .RXDFELPMRESET_TIME           (GTHE3_CHANNEL_RXDFELPMRESET_TIME          ),
        .RXDFELPM_KL_CFG0             (GTHE3_CHANNEL_RXDFELPM_KL_CFG0            ),
        .RXDFELPM_KL_CFG1             (GTHE3_CHANNEL_RXDFELPM_KL_CFG1            ),
        .RXDFELPM_KL_CFG2             (GTHE3_CHANNEL_RXDFELPM_KL_CFG2            ),
        .RXDFE_CFG0                   (GTHE3_CHANNEL_RXDFE_CFG0                  ),
        .RXDFE_CFG1                   (GTHE3_CHANNEL_RXDFE_CFG1                  ),
        .RXDFE_GC_CFG0                (GTHE3_CHANNEL_RXDFE_GC_CFG0               ),
        .RXDFE_GC_CFG1                (GTHE3_CHANNEL_RXDFE_GC_CFG1               ),
        .RXDFE_GC_CFG2                (GTHE3_CHANNEL_RXDFE_GC_CFG2               ),
        .RXDFE_H2_CFG0                (GTHE3_CHANNEL_RXDFE_H2_CFG0               ),
        .RXDFE_H2_CFG1                (GTHE3_CHANNEL_RXDFE_H2_CFG1               ),
        .RXDFE_H3_CFG0                (GTHE3_CHANNEL_RXDFE_H3_CFG0               ),
        .RXDFE_H3_CFG1                (GTHE3_CHANNEL_RXDFE_H3_CFG1               ),
        .RXDFE_H4_CFG0                (GTHE3_CHANNEL_RXDFE_H4_CFG0               ),
        .RXDFE_H4_CFG1                (GTHE3_CHANNEL_RXDFE_H4_CFG1               ),
        .RXDFE_H5_CFG0                (GTHE3_CHANNEL_RXDFE_H5_CFG0               ),
        .RXDFE_H5_CFG1                (GTHE3_CHANNEL_RXDFE_H5_CFG1               ),
        .RXDFE_H6_CFG0                (GTHE3_CHANNEL_RXDFE_H6_CFG0               ),
        .RXDFE_H6_CFG1                (GTHE3_CHANNEL_RXDFE_H6_CFG1               ),
        .RXDFE_H7_CFG0                (GTHE3_CHANNEL_RXDFE_H7_CFG0               ),
        .RXDFE_H7_CFG1                (GTHE3_CHANNEL_RXDFE_H7_CFG1               ),
        .RXDFE_H8_CFG0                (GTHE3_CHANNEL_RXDFE_H8_CFG0               ),
        .RXDFE_H8_CFG1                (GTHE3_CHANNEL_RXDFE_H8_CFG1               ),
        .RXDFE_H9_CFG0                (GTHE3_CHANNEL_RXDFE_H9_CFG0               ),
        .RXDFE_H9_CFG1                (GTHE3_CHANNEL_RXDFE_H9_CFG1               ),
        .RXDFE_HA_CFG0                (GTHE3_CHANNEL_RXDFE_HA_CFG0               ),
        .RXDFE_HA_CFG1                (GTHE3_CHANNEL_RXDFE_HA_CFG1               ),
        .RXDFE_HB_CFG0                (GTHE3_CHANNEL_RXDFE_HB_CFG0               ),
        .RXDFE_HB_CFG1                (GTHE3_CHANNEL_RXDFE_HB_CFG1               ),
        .RXDFE_HC_CFG0                (GTHE3_CHANNEL_RXDFE_HC_CFG0               ),
        .RXDFE_HC_CFG1                (GTHE3_CHANNEL_RXDFE_HC_CFG1               ),
        .RXDFE_HD_CFG0                (GTHE3_CHANNEL_RXDFE_HD_CFG0               ),
        .RXDFE_HD_CFG1                (GTHE3_CHANNEL_RXDFE_HD_CFG1               ),
        .RXDFE_HE_CFG0                (GTHE3_CHANNEL_RXDFE_HE_CFG0               ),
        .RXDFE_HE_CFG1                (GTHE3_CHANNEL_RXDFE_HE_CFG1               ),
        .RXDFE_HF_CFG0                (GTHE3_CHANNEL_RXDFE_HF_CFG0               ),
        .RXDFE_HF_CFG1                (GTHE3_CHANNEL_RXDFE_HF_CFG1               ),
        .RXDFE_OS_CFG0                (GTHE3_CHANNEL_RXDFE_OS_CFG0               ),
        .RXDFE_OS_CFG1                (GTHE3_CHANNEL_RXDFE_OS_CFG1               ),
        .RXDFE_UT_CFG0                (GTHE3_CHANNEL_RXDFE_UT_CFG0               ),
        .RXDFE_UT_CFG1                (GTHE3_CHANNEL_RXDFE_UT_CFG1               ),
        .RXDFE_VP_CFG0                (GTHE3_CHANNEL_RXDFE_VP_CFG0               ),
        .RXDFE_VP_CFG1                (GTHE3_CHANNEL_RXDFE_VP_CFG1               ),
        .RXDLY_CFG                    (GTHE3_CHANNEL_RXDLY_CFG                   ),
        .RXDLY_LCFG                   (GTHE3_CHANNEL_RXDLY_LCFG                  ),
        .RXELECIDLE_CFG               (GTHE3_CHANNEL_RXELECIDLE_CFG              ),
        .RXGBOX_FIFO_INIT_RD_ADDR     (GTHE3_CHANNEL_RXGBOX_FIFO_INIT_RD_ADDR    ),
        .RXGEARBOX_EN                 (GTHE3_CHANNEL_RXGEARBOX_EN                ),
        .RXISCANRESET_TIME            (GTHE3_CHANNEL_RXISCANRESET_TIME           ),
        .RXLPM_CFG                    (GTHE3_CHANNEL_RXLPM_CFG                   ),
        .RXLPM_GC_CFG                 (GTHE3_CHANNEL_RXLPM_GC_CFG                ),
        .RXLPM_KH_CFG0                (GTHE3_CHANNEL_RXLPM_KH_CFG0               ),
        .RXLPM_KH_CFG1                (GTHE3_CHANNEL_RXLPM_KH_CFG1               ),
        .RXLPM_OS_CFG0                (GTHE3_CHANNEL_RXLPM_OS_CFG0               ),
        .RXLPM_OS_CFG1                (GTHE3_CHANNEL_RXLPM_OS_CFG1               ),
        .RXOOB_CFG                    (GTHE3_CHANNEL_RXOOB_CFG                   ),
        .RXOOB_CLK_CFG                (GTHE3_CHANNEL_RXOOB_CLK_CFG               ),
        .RXOSCALRESET_TIME            (GTHE3_CHANNEL_RXOSCALRESET_TIME           ),
        .RXOUT_DIV                    (GTHE3_CHANNEL_RXOUT_DIV                   ),
        .RXPCSRESET_TIME              (GTHE3_CHANNEL_RXPCSRESET_TIME             ),
        .RXPHBEACON_CFG               (GTHE3_CHANNEL_RXPHBEACON_CFG              ),
        .RXPHDLY_CFG                  (GTHE3_CHANNEL_RXPHDLY_CFG                 ),
        .RXPHSAMP_CFG                 (GTHE3_CHANNEL_RXPHSAMP_CFG                ),
        .RXPHSLIP_CFG                 (GTHE3_CHANNEL_RXPHSLIP_CFG                ),
        .RXPH_MONITOR_SEL             (GTHE3_CHANNEL_RXPH_MONITOR_SEL            ),
        .RXPI_CFG0                    (GTHE3_CHANNEL_RXPI_CFG0                   ),
        .RXPI_CFG1                    (GTHE3_CHANNEL_RXPI_CFG1                   ),
        .RXPI_CFG2                    (GTHE3_CHANNEL_RXPI_CFG2                   ),
        .RXPI_CFG3                    (GTHE3_CHANNEL_RXPI_CFG3                   ),
        .RXPI_CFG4                    (GTHE3_CHANNEL_RXPI_CFG4                   ),
        .RXPI_CFG5                    (GTHE3_CHANNEL_RXPI_CFG5                   ),
        .RXPI_CFG6                    (GTHE3_CHANNEL_RXPI_CFG6                   ),
        .RXPI_LPM                     (GTHE3_CHANNEL_RXPI_LPM                    ),
        .RXPI_VREFSEL                 (GTHE3_CHANNEL_RXPI_VREFSEL                ),
        .RXPMACLK_SEL                 (GTHE3_CHANNEL_RXPMACLK_SEL                ),
        .RXPMARESET_TIME              (GTHE3_CHANNEL_RXPMARESET_TIME             ),
        .RXPRBS_ERR_LOOPBACK          (GTHE3_CHANNEL_RXPRBS_ERR_LOOPBACK         ),
        .RXPRBS_LINKACQ_CNT           (GTHE3_CHANNEL_RXPRBS_LINKACQ_CNT          ),
        .RXSLIDE_AUTO_WAIT            (GTHE3_CHANNEL_RXSLIDE_AUTO_WAIT           ),
        .RXSLIDE_MODE                 (GTHE3_CHANNEL_RXSLIDE_MODE                ),
        .RXSYNC_MULTILANE             (GTHE3_CHANNEL_RXSYNC_MULTILANE            ),
        .RXSYNC_OVRD                  (GTHE3_CHANNEL_RXSYNC_OVRD                 ),
        .RXSYNC_SKIP_DA               (GTHE3_CHANNEL_RXSYNC_SKIP_DA              ),
        .RX_AFE_CM_EN                 (GTHE3_CHANNEL_RX_AFE_CM_EN                ),
        .RX_BIAS_CFG0                 (GTHE3_CHANNEL_RX_BIAS_CFG0                ),
        .RX_BUFFER_CFG                (GTHE3_CHANNEL_RX_BUFFER_CFG               ),
        .RX_CAPFF_SARC_ENB            (GTHE3_CHANNEL_RX_CAPFF_SARC_ENB           ),
        .RX_CLK25_DIV                 (GTHE3_CHANNEL_RX_CLK25_DIV                ),
        .RX_CLKMUX_EN                 (GTHE3_CHANNEL_RX_CLKMUX_EN                ),
        .RX_CLK_SLIP_OVRD             (GTHE3_CHANNEL_RX_CLK_SLIP_OVRD            ),
        .RX_CM_BUF_CFG                (GTHE3_CHANNEL_RX_CM_BUF_CFG               ),
        .RX_CM_BUF_PD                 (GTHE3_CHANNEL_RX_CM_BUF_PD                ),
        .RX_CM_SEL                    (GTHE3_CHANNEL_RX_CM_SEL                   ),
        .RX_CM_TRIM                   (GTHE3_CHANNEL_RX_CM_TRIM                  ),
        .RX_CTLE3_LPF                 (GTHE3_CHANNEL_RX_CTLE3_LPF                ),
        .RX_DATA_WIDTH                (GTHE3_CHANNEL_RX_DATA_WIDTH               ),
        .RX_DDI_SEL                   (GTHE3_CHANNEL_RX_DDI_SEL                  ),
        .RX_DEFER_RESET_BUF_EN        (GTHE3_CHANNEL_RX_DEFER_RESET_BUF_EN       ),
        .RX_DFELPM_CFG0               (GTHE3_CHANNEL_RX_DFELPM_CFG0              ),
        .RX_DFELPM_CFG1               (GTHE3_CHANNEL_RX_DFELPM_CFG1              ),
        .RX_DFELPM_KLKH_AGC_STUP_EN   (GTHE3_CHANNEL_RX_DFELPM_KLKH_AGC_STUP_EN  ),
        .RX_DFE_AGC_CFG0              (GTHE3_CHANNEL_RX_DFE_AGC_CFG0             ),
        .RX_DFE_AGC_CFG1              (GTHE3_CHANNEL_RX_DFE_AGC_CFG1             ),
        .RX_DFE_KL_LPM_KH_CFG0        (GTHE3_CHANNEL_RX_DFE_KL_LPM_KH_CFG0       ),
        .RX_DFE_KL_LPM_KH_CFG1        (GTHE3_CHANNEL_RX_DFE_KL_LPM_KH_CFG1       ),
        .RX_DFE_KL_LPM_KL_CFG0        (GTHE3_CHANNEL_RX_DFE_KL_LPM_KL_CFG0       ),
        .RX_DFE_KL_LPM_KL_CFG1        (GTHE3_CHANNEL_RX_DFE_KL_LPM_KL_CFG1       ),
        .RX_DFE_LPM_HOLD_DURING_EIDLE (GTHE3_CHANNEL_RX_DFE_LPM_HOLD_DURING_EIDLE),
        .RX_DISPERR_SEQ_MATCH         (GTHE3_CHANNEL_RX_DISPERR_SEQ_MATCH        ),
        .RX_DIVRESET_TIME             (GTHE3_CHANNEL_RX_DIVRESET_TIME            ),
        .RX_EN_HI_LR                  (GTHE3_CHANNEL_RX_EN_HI_LR                 ),
        .RX_EYESCAN_VS_CODE           (GTHE3_CHANNEL_RX_EYESCAN_VS_CODE          ),
        .RX_EYESCAN_VS_NEG_DIR        (GTHE3_CHANNEL_RX_EYESCAN_VS_NEG_DIR       ),
        .RX_EYESCAN_VS_RANGE          (GTHE3_CHANNEL_RX_EYESCAN_VS_RANGE         ),
        .RX_EYESCAN_VS_UT_SIGN        (GTHE3_CHANNEL_RX_EYESCAN_VS_UT_SIGN       ),
        .RX_FABINT_USRCLK_FLOP        (GTHE3_CHANNEL_RX_FABINT_USRCLK_FLOP       ),
        .RX_INT_DATAWIDTH             (GTHE3_CHANNEL_RX_INT_DATAWIDTH            ),
        .RX_PMA_POWER_SAVE            (GTHE3_CHANNEL_RX_PMA_POWER_SAVE           ),
        .RX_PROGDIV_CFG               (GTHE3_CHANNEL_RX_PROGDIV_CFG              ),
        .RX_SAMPLE_PERIOD             (GTHE3_CHANNEL_RX_SAMPLE_PERIOD            ),
        .RX_SIG_VALID_DLY             (GTHE3_CHANNEL_RX_SIG_VALID_DLY            ),
        .RX_SUM_DFETAPREP_EN          (GTHE3_CHANNEL_RX_SUM_DFETAPREP_EN         ),
        .RX_SUM_IREF_TUNE             (GTHE3_CHANNEL_RX_SUM_IREF_TUNE            ),
        .RX_SUM_RES_CTRL              (GTHE3_CHANNEL_RX_SUM_RES_CTRL             ),
        .RX_SUM_VCMTUNE               (GTHE3_CHANNEL_RX_SUM_VCMTUNE              ),
        .RX_SUM_VCM_OVWR              (GTHE3_CHANNEL_RX_SUM_VCM_OVWR             ),
        .RX_SUM_VREF_TUNE             (GTHE3_CHANNEL_RX_SUM_VREF_TUNE            ),
        .RX_TUNE_AFE_OS               (GTHE3_CHANNEL_RX_TUNE_AFE_OS              ),
        .RX_WIDEMODE_CDR              (GTHE3_CHANNEL_RX_WIDEMODE_CDR             ),
        .RX_XCLK_SEL                  (GTHE3_CHANNEL_RX_XCLK_SEL                 ),
        .SAS_MAX_COM                  (GTHE3_CHANNEL_SAS_MAX_COM                 ),
        .SAS_MIN_COM                  (GTHE3_CHANNEL_SAS_MIN_COM                 ),
        .SATA_BURST_SEQ_LEN           (GTHE3_CHANNEL_SATA_BURST_SEQ_LEN          ),
        .SATA_BURST_VAL               (GTHE3_CHANNEL_SATA_BURST_VAL              ),
        .SATA_CPLL_CFG                (GTHE3_CHANNEL_SATA_CPLL_CFG               ),
        .SATA_EIDLE_VAL               (GTHE3_CHANNEL_SATA_EIDLE_VAL              ),
        .SATA_MAX_BURST               (GTHE3_CHANNEL_SATA_MAX_BURST              ),
        .SATA_MAX_INIT                (GTHE3_CHANNEL_SATA_MAX_INIT               ),
        .SATA_MAX_WAKE                (GTHE3_CHANNEL_SATA_MAX_WAKE               ),
        .SATA_MIN_BURST               (GTHE3_CHANNEL_SATA_MIN_BURST              ),
        .SATA_MIN_INIT                (GTHE3_CHANNEL_SATA_MIN_INIT               ),
        .SATA_MIN_WAKE                (GTHE3_CHANNEL_SATA_MIN_WAKE               ),
        .SHOW_REALIGN_COMMA           (GTHE3_CHANNEL_SHOW_REALIGN_COMMA          ),
        .SIM_RECEIVER_DETECT_PASS     (GTHE3_CHANNEL_SIM_RECEIVER_DETECT_PASS    ),
        .SIM_RESET_SPEEDUP            (GTHE3_CHANNEL_SIM_RESET_SPEEDUP           ),
        .SIM_TX_EIDLE_DRIVE_LEVEL     (GTHE3_CHANNEL_SIM_TX_EIDLE_DRIVE_LEVEL    ),
        .SIM_VERSION                  (GTHE3_CHANNEL_SIM_VERSION                 ),
        .TAPDLY_SET_TX                (GTHE3_CHANNEL_TAPDLY_SET_TX               ),
        .TEMPERATUR_PAR               (GTHE3_CHANNEL_TEMPERATUR_PAR              ),
        .TERM_RCAL_CFG                (GTHE3_CHANNEL_TERM_RCAL_CFG               ),
        .TERM_RCAL_OVRD               (GTHE3_CHANNEL_TERM_RCAL_OVRD              ),
        .TRANS_TIME_RATE              (GTHE3_CHANNEL_TRANS_TIME_RATE             ),
        .TST_RSV0                     (GTHE3_CHANNEL_TST_RSV0                    ),
        .TST_RSV1                     (GTHE3_CHANNEL_TST_RSV1                    ),
        .TXBUF_EN                     (GTHE3_CHANNEL_TXBUF_EN                    ),
        .TXBUF_RESET_ON_RATE_CHANGE   (GTHE3_CHANNEL_TXBUF_RESET_ON_RATE_CHANGE  ),
        .TXDLY_CFG                    (GTHE3_CHANNEL_TXDLY_CFG                   ),
        .TXDLY_LCFG                   (GTHE3_CHANNEL_TXDLY_LCFG                  ),
        .TXDRVBIAS_N                  (GTHE3_CHANNEL_TXDRVBIAS_N                 ),
        .TXDRVBIAS_P                  (GTHE3_CHANNEL_TXDRVBIAS_P                 ),
        .TXFIFO_ADDR_CFG              (GTHE3_CHANNEL_TXFIFO_ADDR_CFG             ),
        .TXGBOX_FIFO_INIT_RD_ADDR     (GTHE3_CHANNEL_TXGBOX_FIFO_INIT_RD_ADDR    ),
        .TXGEARBOX_EN                 (GTHE3_CHANNEL_TXGEARBOX_EN                ),
        .TXOUT_DIV                    (GTHE3_CHANNEL_TXOUT_DIV                   ),
        .TXPCSRESET_TIME              (GTHE3_CHANNEL_TXPCSRESET_TIME             ),
        .TXPHDLY_CFG0                 (GTHE3_CHANNEL_TXPHDLY_CFG0                ),
        .TXPHDLY_CFG1                 (GTHE3_CHANNEL_TXPHDLY_CFG1                ),
        .TXPH_CFG                     (GTHE3_CHANNEL_TXPH_CFG                    ),
        .TXPH_MONITOR_SEL             (GTHE3_CHANNEL_TXPH_MONITOR_SEL            ),
        .TXPI_CFG0                    (GTHE3_CHANNEL_TXPI_CFG0                   ),
        .TXPI_CFG1                    (GTHE3_CHANNEL_TXPI_CFG1                   ),
        .TXPI_CFG2                    (GTHE3_CHANNEL_TXPI_CFG2                   ),
        .TXPI_CFG3                    (GTHE3_CHANNEL_TXPI_CFG3                   ),
        .TXPI_CFG4                    (GTHE3_CHANNEL_TXPI_CFG4                   ),
        .TXPI_CFG5                    (GTHE3_CHANNEL_TXPI_CFG5                   ),
        .TXPI_GRAY_SEL                (GTHE3_CHANNEL_TXPI_GRAY_SEL               ),
        .TXPI_INVSTROBE_SEL           (GTHE3_CHANNEL_TXPI_INVSTROBE_SEL          ),
        .TXPI_LPM                     (GTHE3_CHANNEL_TXPI_LPM                    ),
        .TXPI_PPMCLK_SEL              (GTHE3_CHANNEL_TXPI_PPMCLK_SEL             ),
        .TXPI_PPM_CFG                 (GTHE3_CHANNEL_TXPI_PPM_CFG                ),
        .TXPI_SYNFREQ_PPM             (GTHE3_CHANNEL_TXPI_SYNFREQ_PPM            ),
        .TXPI_VREFSEL                 (GTHE3_CHANNEL_TXPI_VREFSEL                ),
        .TXPMARESET_TIME              (GTHE3_CHANNEL_TXPMARESET_TIME             ),
        .TXSYNC_MULTILANE             (GTHE3_CHANNEL_TXSYNC_MULTILANE            ),
        .TXSYNC_OVRD                  (GTHE3_CHANNEL_TXSYNC_OVRD                 ),
        .TXSYNC_SKIP_DA               (GTHE3_CHANNEL_TXSYNC_SKIP_DA              ),
        .TX_CLK25_DIV                 (GTHE3_CHANNEL_TX_CLK25_DIV                ),
        .TX_CLKMUX_EN                 (GTHE3_CHANNEL_TX_CLKMUX_EN                ),
        .TX_DATA_WIDTH                (GTHE3_CHANNEL_TX_DATA_WIDTH               ),
        .TX_DCD_CFG                   (GTHE3_CHANNEL_TX_DCD_CFG                  ),
        .TX_DCD_EN                    (GTHE3_CHANNEL_TX_DCD_EN                   ),
        .TX_DEEMPH0                   (GTHE3_CHANNEL_TX_DEEMPH0                  ),
        .TX_DEEMPH1                   (GTHE3_CHANNEL_TX_DEEMPH1                  ),
        .TX_DIVRESET_TIME             (GTHE3_CHANNEL_TX_DIVRESET_TIME            ),
        .TX_DRIVE_MODE                (GTHE3_CHANNEL_TX_DRIVE_MODE               ),
        .TX_EIDLE_ASSERT_DELAY        (GTHE3_CHANNEL_TX_EIDLE_ASSERT_DELAY       ),
        .TX_EIDLE_DEASSERT_DELAY      (GTHE3_CHANNEL_TX_EIDLE_DEASSERT_DELAY     ),
        .TX_EML_PHI_TUNE              (GTHE3_CHANNEL_TX_EML_PHI_TUNE             ),
        .TX_FABINT_USRCLK_FLOP        (GTHE3_CHANNEL_TX_FABINT_USRCLK_FLOP       ),
        .TX_IDLE_DATA_ZERO            (GTHE3_CHANNEL_TX_IDLE_DATA_ZERO           ),
        .TX_INT_DATAWIDTH             (GTHE3_CHANNEL_TX_INT_DATAWIDTH            ),
        .TX_LOOPBACK_DRIVE_HIZ        (GTHE3_CHANNEL_TX_LOOPBACK_DRIVE_HIZ       ),
        .TX_MAINCURSOR_SEL            (GTHE3_CHANNEL_TX_MAINCURSOR_SEL           ),
        .TX_MARGIN_FULL_0             (GTHE3_CHANNEL_TX_MARGIN_FULL_0            ),
        .TX_MARGIN_FULL_1             (GTHE3_CHANNEL_TX_MARGIN_FULL_1            ),
        .TX_MARGIN_FULL_2             (GTHE3_CHANNEL_TX_MARGIN_FULL_2            ),
        .TX_MARGIN_FULL_3             (GTHE3_CHANNEL_TX_MARGIN_FULL_3            ),
        .TX_MARGIN_FULL_4             (GTHE3_CHANNEL_TX_MARGIN_FULL_4            ),
        .TX_MARGIN_LOW_0              (GTHE3_CHANNEL_TX_MARGIN_LOW_0             ),
        .TX_MARGIN_LOW_1              (GTHE3_CHANNEL_TX_MARGIN_LOW_1             ),
        .TX_MARGIN_LOW_2              (GTHE3_CHANNEL_TX_MARGIN_LOW_2             ),
        .TX_MARGIN_LOW_3              (GTHE3_CHANNEL_TX_MARGIN_LOW_3             ),
        .TX_MARGIN_LOW_4              (GTHE3_CHANNEL_TX_MARGIN_LOW_4             ),
        .TX_MODE_SEL                  (GTHE3_CHANNEL_TX_MODE_SEL                 ),
        .TX_PMADATA_OPT               (GTHE3_CHANNEL_TX_PMADATA_OPT              ),
        .TX_PMA_POWER_SAVE            (GTHE3_CHANNEL_TX_PMA_POWER_SAVE           ),
        .TX_PROGCLK_SEL               (GTHE3_CHANNEL_TX_PROGCLK_SEL              ),
        .TX_PROGDIV_CFG               (GTHE3_CHANNEL_TX_PROGDIV_CFG              ),
        .TX_QPI_STATUS_EN             (GTHE3_CHANNEL_TX_QPI_STATUS_EN            ),
        .TX_RXDETECT_CFG              (GTHE3_CHANNEL_TX_RXDETECT_CFG             ),
        .TX_RXDETECT_REF              (GTHE3_CHANNEL_TX_RXDETECT_REF             ),
        .TX_SAMPLE_PERIOD             (GTHE3_CHANNEL_TX_SAMPLE_PERIOD            ),
        .TX_SARC_LPBK_ENB             (GTHE3_CHANNEL_TX_SARC_LPBK_ENB            ),
        .TX_XCLK_SEL                  (GTHE3_CHANNEL_TX_XCLK_SEL                 ),
        .USE_PCS_CLK_PHASE_SEL        (GTHE3_CHANNEL_USE_PCS_CLK_PHASE_SEL       ),
        .WB_MODE                      (GTHE3_CHANNEL_WB_MODE                     )
      ) GTHE3_CHANNEL_PRIM_INST (
        .CFGRESET                 (GTHE3_CHANNEL_CFGRESET_int            [((ch+1)*  1)-1:(ch*  1)]),
        .CLKRSVD0                 (GTHE3_CHANNEL_CLKRSVD0_int            [((ch+1)*  1)-1:(ch*  1)]),
        .CLKRSVD1                 (GTHE3_CHANNEL_CLKRSVD1_int            [((ch+1)*  1)-1:(ch*  1)]),
        .CPLLLOCKDETCLK           (GTHE3_CHANNEL_CPLLLOCKDETCLK_int      [((ch+1)*  1)-1:(ch*  1)]),
        .CPLLLOCKEN               (GTHE3_CHANNEL_CPLLLOCKEN_int          [((ch+1)*  1)-1:(ch*  1)]),
        .CPLLPD                   (GTHE3_CHANNEL_CPLLPD_int              [((ch+1)*  1)-1:(ch*  1)]),
        .CPLLREFCLKSEL            (GTHE3_CHANNEL_CPLLREFCLKSEL_int       [((ch+1)*  3)-1:(ch*  3)]),
        .CPLLRESET                (GTHE3_CHANNEL_CPLLRESET_int           [((ch+1)*  1)-1:(ch*  1)]),
        .DMONFIFORESET            (GTHE3_CHANNEL_DMONFIFORESET_int       [((ch+1)*  1)-1:(ch*  1)]),
        .DMONITORCLK              (GTHE3_CHANNEL_DMONITORCLK_int         [((ch+1)*  1)-1:(ch*  1)]),
        .DRPADDR                  (GTHE3_CHANNEL_DRPADDR_int             [((ch+1)*  9)-1:(ch*  9)]),
        .DRPCLK                   (GTHE3_CHANNEL_DRPCLK_int              [((ch+1)*  1)-1:(ch*  1)]),
        .DRPDI                    (GTHE3_CHANNEL_DRPDI_int               [((ch+1)* 16)-1:(ch* 16)]),
        .DRPEN                    (GTHE3_CHANNEL_DRPEN_int               [((ch+1)*  1)-1:(ch*  1)]),
        .DRPWE                    (GTHE3_CHANNEL_DRPWE_int               [((ch+1)*  1)-1:(ch*  1)]),
        .EVODDPHICALDONE          (GTHE3_CHANNEL_EVODDPHICALDONE_int     [((ch+1)*  1)-1:(ch*  1)]),
        .EVODDPHICALSTART         (GTHE3_CHANNEL_EVODDPHICALSTART_int    [((ch+1)*  1)-1:(ch*  1)]),
        .EVODDPHIDRDEN            (GTHE3_CHANNEL_EVODDPHIDRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .EVODDPHIDWREN            (GTHE3_CHANNEL_EVODDPHIDWREN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .EVODDPHIXRDEN            (GTHE3_CHANNEL_EVODDPHIXRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .EVODDPHIXWREN            (GTHE3_CHANNEL_EVODDPHIXWREN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .EYESCANMODE              (GTHE3_CHANNEL_EYESCANMODE_int         [((ch+1)*  1)-1:(ch*  1)]),
        .EYESCANRESET             (GTHE3_CHANNEL_EYESCANRESET_int        [((ch+1)*  1)-1:(ch*  1)]),
        .EYESCANTRIGGER           (GTHE3_CHANNEL_EYESCANTRIGGER_int      [((ch+1)*  1)-1:(ch*  1)]),
        .GTGREFCLK                (GTHE3_CHANNEL_GTGREFCLK_int           [((ch+1)*  1)-1:(ch*  1)]),
        .GTHRXN                   (GTHE3_CHANNEL_GTHRXN_int              [((ch+1)*  1)-1:(ch*  1)]),
        .GTHRXP                   (GTHE3_CHANNEL_GTHRXP_int              [((ch+1)*  1)-1:(ch*  1)]),
        .GTNORTHREFCLK0           (GTHE3_CHANNEL_GTNORTHREFCLK0_int      [((ch+1)*  1)-1:(ch*  1)]),
        .GTNORTHREFCLK1           (GTHE3_CHANNEL_GTNORTHREFCLK1_int      [((ch+1)*  1)-1:(ch*  1)]),
        .GTREFCLK0                (GTHE3_CHANNEL_GTREFCLK0_int           [((ch+1)*  1)-1:(ch*  1)]),
        .GTREFCLK1                (GTHE3_CHANNEL_GTREFCLK1_int           [((ch+1)*  1)-1:(ch*  1)]),
        .GTRESETSEL               (GTHE3_CHANNEL_GTRESETSEL_int          [((ch+1)*  1)-1:(ch*  1)]),
        .GTRSVD                   (GTHE3_CHANNEL_GTRSVD_int              [((ch+1)* 16)-1:(ch* 16)]),
        .GTRXRESET                (GTHE3_CHANNEL_GTRXRESET_int           [((ch+1)*  1)-1:(ch*  1)]),
        .GTSOUTHREFCLK0           (GTHE3_CHANNEL_GTSOUTHREFCLK0_int      [((ch+1)*  1)-1:(ch*  1)]),
        .GTSOUTHREFCLK1           (GTHE3_CHANNEL_GTSOUTHREFCLK1_int      [((ch+1)*  1)-1:(ch*  1)]),
        .GTTXRESET                (GTHE3_CHANNEL_GTTXRESET_int           [((ch+1)*  1)-1:(ch*  1)]),
        .LOOPBACK                 (GTHE3_CHANNEL_LOOPBACK_int            [((ch+1)*  3)-1:(ch*  3)]),
        .LPBKRXTXSEREN            (GTHE3_CHANNEL_LPBKRXTXSEREN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .LPBKTXRXSEREN            (GTHE3_CHANNEL_LPBKTXRXSEREN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .PCIEEQRXEQADAPTDONE      (GTHE3_CHANNEL_PCIEEQRXEQADAPTDONE_int [((ch+1)*  1)-1:(ch*  1)]),
        .PCIERSTIDLE              (GTHE3_CHANNEL_PCIERSTIDLE_int         [((ch+1)*  1)-1:(ch*  1)]),
        .PCIERSTTXSYNCSTART       (GTHE3_CHANNEL_PCIERSTTXSYNCSTART_int  [((ch+1)*  1)-1:(ch*  1)]),
        .PCIEUSERRATEDONE         (GTHE3_CHANNEL_PCIEUSERRATEDONE_int    [((ch+1)*  1)-1:(ch*  1)]),
        .PCSRSVDIN                (GTHE3_CHANNEL_PCSRSVDIN_int           [((ch+1)* 16)-1:(ch* 16)]),
        .PCSRSVDIN2               (GTHE3_CHANNEL_PCSRSVDIN2_int          [((ch+1)*  5)-1:(ch*  5)]),
        .PMARSVDIN                (GTHE3_CHANNEL_PMARSVDIN_int           [((ch+1)*  5)-1:(ch*  5)]),
        .QPLL0CLK                 (GTHE3_CHANNEL_QPLL0CLK_int            [((ch+1)*  1)-1:(ch*  1)]),
        .QPLL0REFCLK              (GTHE3_CHANNEL_QPLL0REFCLK_int         [((ch+1)*  1)-1:(ch*  1)]),
        .QPLL1CLK                 (GTHE3_CHANNEL_QPLL1CLK_int            [((ch+1)*  1)-1:(ch*  1)]),
        .QPLL1REFCLK              (GTHE3_CHANNEL_QPLL1REFCLK_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RESETOVRD                (GTHE3_CHANNEL_RESETOVRD_int           [((ch+1)*  1)-1:(ch*  1)]),
        .RSTCLKENTX               (GTHE3_CHANNEL_RSTCLKENTX_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RX8B10BEN                (GTHE3_CHANNEL_RX8B10BEN_int           [((ch+1)*  1)-1:(ch*  1)]),
        .RXBUFRESET               (GTHE3_CHANNEL_RXBUFRESET_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXCDRFREQRESET           (GTHE3_CHANNEL_RXCDRFREQRESET_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXCDRHOLD                (GTHE3_CHANNEL_RXCDRHOLD_int           [((ch+1)*  1)-1:(ch*  1)]),
        .RXCDROVRDEN              (GTHE3_CHANNEL_RXCDROVRDEN_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXCDRRESET               (GTHE3_CHANNEL_RXCDRRESET_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXCDRRESETRSV            (GTHE3_CHANNEL_RXCDRRESETRSV_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXCHBONDEN               (GTHE3_CHANNEL_RXCHBONDEN_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXCHBONDI                (GTHE3_CHANNEL_RXCHBONDI_int           [((ch+1)*  5)-1:(ch*  5)]),
        .RXCHBONDLEVEL            (GTHE3_CHANNEL_RXCHBONDLEVEL_int       [((ch+1)*  3)-1:(ch*  3)]),
        .RXCHBONDMASTER           (GTHE3_CHANNEL_RXCHBONDMASTER_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXCHBONDSLAVE            (GTHE3_CHANNEL_RXCHBONDSLAVE_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXCOMMADETEN             (GTHE3_CHANNEL_RXCOMMADETEN_int        [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFEAGCCTRL             (GTHE3_CHANNEL_RXDFEAGCCTRL_int        [((ch+1)*  2)-1:(ch*  2)]),
        .RXDFEAGCHOLD             (GTHE3_CHANNEL_RXDFEAGCHOLD_int        [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFEAGCOVRDEN           (GTHE3_CHANNEL_RXDFEAGCOVRDEN_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFELFHOLD              (GTHE3_CHANNEL_RXDFELFHOLD_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFELFOVRDEN            (GTHE3_CHANNEL_RXDFELFOVRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFELPMRESET            (GTHE3_CHANNEL_RXDFELPMRESET_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP10HOLD           (GTHE3_CHANNEL_RXDFETAP10HOLD_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP10OVRDEN         (GTHE3_CHANNEL_RXDFETAP10OVRDEN_int    [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP11HOLD           (GTHE3_CHANNEL_RXDFETAP11HOLD_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP11OVRDEN         (GTHE3_CHANNEL_RXDFETAP11OVRDEN_int    [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP12HOLD           (GTHE3_CHANNEL_RXDFETAP12HOLD_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP12OVRDEN         (GTHE3_CHANNEL_RXDFETAP12OVRDEN_int    [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP13HOLD           (GTHE3_CHANNEL_RXDFETAP13HOLD_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP13OVRDEN         (GTHE3_CHANNEL_RXDFETAP13OVRDEN_int    [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP14HOLD           (GTHE3_CHANNEL_RXDFETAP14HOLD_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP14OVRDEN         (GTHE3_CHANNEL_RXDFETAP14OVRDEN_int    [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP15HOLD           (GTHE3_CHANNEL_RXDFETAP15HOLD_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP15OVRDEN         (GTHE3_CHANNEL_RXDFETAP15OVRDEN_int    [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP2HOLD            (GTHE3_CHANNEL_RXDFETAP2HOLD_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP2OVRDEN          (GTHE3_CHANNEL_RXDFETAP2OVRDEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP3HOLD            (GTHE3_CHANNEL_RXDFETAP3HOLD_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP3OVRDEN          (GTHE3_CHANNEL_RXDFETAP3OVRDEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP4HOLD            (GTHE3_CHANNEL_RXDFETAP4HOLD_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP4OVRDEN          (GTHE3_CHANNEL_RXDFETAP4OVRDEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP5HOLD            (GTHE3_CHANNEL_RXDFETAP5HOLD_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP5OVRDEN          (GTHE3_CHANNEL_RXDFETAP5OVRDEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP6HOLD            (GTHE3_CHANNEL_RXDFETAP6HOLD_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP6OVRDEN          (GTHE3_CHANNEL_RXDFETAP6OVRDEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP7HOLD            (GTHE3_CHANNEL_RXDFETAP7HOLD_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP7OVRDEN          (GTHE3_CHANNEL_RXDFETAP7OVRDEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP8HOLD            (GTHE3_CHANNEL_RXDFETAP8HOLD_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP8OVRDEN          (GTHE3_CHANNEL_RXDFETAP8OVRDEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP9HOLD            (GTHE3_CHANNEL_RXDFETAP9HOLD_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFETAP9OVRDEN          (GTHE3_CHANNEL_RXDFETAP9OVRDEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFEUTHOLD              (GTHE3_CHANNEL_RXDFEUTHOLD_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFEUTOVRDEN            (GTHE3_CHANNEL_RXDFEUTOVRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFEVPHOLD              (GTHE3_CHANNEL_RXDFEVPHOLD_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFEVPOVRDEN            (GTHE3_CHANNEL_RXDFEVPOVRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFEVSEN                (GTHE3_CHANNEL_RXDFEVSEN_int           [((ch+1)*  1)-1:(ch*  1)]),
        .RXDFEXYDEN               (GTHE3_CHANNEL_RXDFEXYDEN_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXDLYBYPASS              (GTHE3_CHANNEL_RXDLYBYPASS_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXDLYEN                  (GTHE3_CHANNEL_RXDLYEN_int             [((ch+1)*  1)-1:(ch*  1)]),
        .RXDLYOVRDEN              (GTHE3_CHANNEL_RXDLYOVRDEN_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXDLYSRESET              (GTHE3_CHANNEL_RXDLYSRESET_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXELECIDLEMODE           (GTHE3_CHANNEL_RXELECIDLEMODE_int      [((ch+1)*  2)-1:(ch*  2)]),
        .RXGEARBOXSLIP            (GTHE3_CHANNEL_RXGEARBOXSLIP_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXLATCLK                 (GTHE3_CHANNEL_RXLATCLK_int            [((ch+1)*  1)-1:(ch*  1)]),
        .RXLPMEN                  (GTHE3_CHANNEL_RXLPMEN_int             [((ch+1)*  1)-1:(ch*  1)]),
        .RXLPMGCHOLD              (GTHE3_CHANNEL_RXLPMGCHOLD_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXLPMGCOVRDEN            (GTHE3_CHANNEL_RXLPMGCOVRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXLPMHFHOLD              (GTHE3_CHANNEL_RXLPMHFHOLD_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXLPMHFOVRDEN            (GTHE3_CHANNEL_RXLPMHFOVRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXLPMLFHOLD              (GTHE3_CHANNEL_RXLPMLFHOLD_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXLPMLFKLOVRDEN          (GTHE3_CHANNEL_RXLPMLFKLOVRDEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXLPMOSHOLD              (GTHE3_CHANNEL_RXLPMOSHOLD_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXLPMOSOVRDEN            (GTHE3_CHANNEL_RXLPMOSOVRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXMCOMMAALIGNEN          (GTHE3_CHANNEL_RXMCOMMAALIGNEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXMONITORSEL             (GTHE3_CHANNEL_RXMONITORSEL_int        [((ch+1)*  2)-1:(ch*  2)]),
        .RXOOBRESET               (GTHE3_CHANNEL_RXOOBRESET_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSCALRESET             (GTHE3_CHANNEL_RXOSCALRESET_int        [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSHOLD                 (GTHE3_CHANNEL_RXOSHOLD_int            [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSINTCFG               (GTHE3_CHANNEL_RXOSINTCFG_int          [((ch+1)*  4)-1:(ch*  4)]),
        .RXOSINTEN                (GTHE3_CHANNEL_RXOSINTEN_int           [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSINTHOLD              (GTHE3_CHANNEL_RXOSINTHOLD_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSINTOVRDEN            (GTHE3_CHANNEL_RXOSINTOVRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSINTSTROBE            (GTHE3_CHANNEL_RXOSINTSTROBE_int       [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSINTTESTOVRDEN        (GTHE3_CHANNEL_RXOSINTTESTOVRDEN_int   [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSOVRDEN               (GTHE3_CHANNEL_RXOSOVRDEN_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXOUTCLKSEL              (GTHE3_CHANNEL_RXOUTCLKSEL_int         [((ch+1)*  3)-1:(ch*  3)]),
        .RXPCOMMAALIGNEN          (GTHE3_CHANNEL_RXPCOMMAALIGNEN_int     [((ch+1)*  1)-1:(ch*  1)]),
        .RXPCSRESET               (GTHE3_CHANNEL_RXPCSRESET_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXPD                     (GTHE3_CHANNEL_RXPD_int                [((ch+1)*  2)-1:(ch*  2)]),
        .RXPHALIGN                (GTHE3_CHANNEL_RXPHALIGN_int           [((ch+1)*  1)-1:(ch*  1)]),
        .RXPHALIGNEN              (GTHE3_CHANNEL_RXPHALIGNEN_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXPHDLYPD                (GTHE3_CHANNEL_RXPHDLYPD_int           [((ch+1)*  1)-1:(ch*  1)]),
        .RXPHDLYRESET             (GTHE3_CHANNEL_RXPHDLYRESET_int        [((ch+1)*  1)-1:(ch*  1)]),
        .RXPHOVRDEN               (GTHE3_CHANNEL_RXPHOVRDEN_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXPLLCLKSEL              (GTHE3_CHANNEL_RXPLLCLKSEL_int         [((ch+1)*  2)-1:(ch*  2)]),
        .RXPMARESET               (GTHE3_CHANNEL_RXPMARESET_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXPOLARITY               (GTHE3_CHANNEL_RXPOLARITY_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXPRBSCNTRESET           (GTHE3_CHANNEL_RXPRBSCNTRESET_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXPRBSSEL                (GTHE3_CHANNEL_RXPRBSSEL_int           [((ch+1)*  4)-1:(ch*  4)]),
        .RXPROGDIVRESET           (GTHE3_CHANNEL_RXPROGDIVRESET_int      [((ch+1)*  1)-1:(ch*  1)]),
        .RXQPIEN                  (GTHE3_CHANNEL_RXQPIEN_int             [((ch+1)*  1)-1:(ch*  1)]),
        .RXRATE                   (GTHE3_CHANNEL_RXRATE_int              [((ch+1)*  3)-1:(ch*  3)]),
        .RXRATEMODE               (GTHE3_CHANNEL_RXRATEMODE_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXSLIDE                  (GTHE3_CHANNEL_RXSLIDE_int             [((ch+1)*  1)-1:(ch*  1)]),
        .RXSLIPOUTCLK             (GTHE3_CHANNEL_RXSLIPOUTCLK_int        [((ch+1)*  1)-1:(ch*  1)]),
        .RXSLIPPMA                (GTHE3_CHANNEL_RXSLIPPMA_int           [((ch+1)*  1)-1:(ch*  1)]),
        .RXSYNCALLIN              (GTHE3_CHANNEL_RXSYNCALLIN_int         [((ch+1)*  1)-1:(ch*  1)]),
        .RXSYNCIN                 (GTHE3_CHANNEL_RXSYNCIN_int            [((ch+1)*  1)-1:(ch*  1)]),
        .RXSYNCMODE               (GTHE3_CHANNEL_RXSYNCMODE_int          [((ch+1)*  1)-1:(ch*  1)]),
        .RXSYSCLKSEL              (GTHE3_CHANNEL_RXSYSCLKSEL_int         [((ch+1)*  2)-1:(ch*  2)]),
        .RXUSERRDY                (GTHE3_CHANNEL_RXUSERRDY_int           [((ch+1)*  1)-1:(ch*  1)]),
        .RXUSRCLK                 (GTHE3_CHANNEL_RXUSRCLK_int            [((ch+1)*  1)-1:(ch*  1)]),
        .RXUSRCLK2                (GTHE3_CHANNEL_RXUSRCLK2_int           [((ch+1)*  1)-1:(ch*  1)]),
        .SIGVALIDCLK              (GTHE3_CHANNEL_SIGVALIDCLK_int         [((ch+1)*  1)-1:(ch*  1)]),
        .TSTIN                    (GTHE3_CHANNEL_TSTIN_int               [((ch+1)* 20)-1:(ch* 20)]),
        .TX8B10BBYPASS            (GTHE3_CHANNEL_TX8B10BBYPASS_int       [((ch+1)*  8)-1:(ch*  8)]),
        .TX8B10BEN                (GTHE3_CHANNEL_TX8B10BEN_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXBUFDIFFCTRL            (GTHE3_CHANNEL_TXBUFDIFFCTRL_int       [((ch+1)*  3)-1:(ch*  3)]),
        .TXCOMINIT                (GTHE3_CHANNEL_TXCOMINIT_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXCOMSAS                 (GTHE3_CHANNEL_TXCOMSAS_int            [((ch+1)*  1)-1:(ch*  1)]),
        .TXCOMWAKE                (GTHE3_CHANNEL_TXCOMWAKE_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXCTRL0                  (GTHE3_CHANNEL_TXCTRL0_int             [((ch+1)* 16)-1:(ch* 16)]),
        .TXCTRL1                  (GTHE3_CHANNEL_TXCTRL1_int             [((ch+1)* 16)-1:(ch* 16)]),
        .TXCTRL2                  (GTHE3_CHANNEL_TXCTRL2_int             [((ch+1)*  8)-1:(ch*  8)]),
        .TXDATA                   (GTHE3_CHANNEL_TXDATA_int              [((ch+1)*128)-1:(ch*128)]),
        .TXDATAEXTENDRSVD         (GTHE3_CHANNEL_TXDATAEXTENDRSVD_int    [((ch+1)*  8)-1:(ch*  8)]),
        .TXDEEMPH                 (GTHE3_CHANNEL_TXDEEMPH_int            [((ch+1)*  1)-1:(ch*  1)]),
        .TXDETECTRX               (GTHE3_CHANNEL_TXDETECTRX_int          [((ch+1)*  1)-1:(ch*  1)]),
        .TXDIFFCTRL               (GTHE3_CHANNEL_TXDIFFCTRL_int          [((ch+1)*  4)-1:(ch*  4)]),
        .TXDIFFPD                 (GTHE3_CHANNEL_TXDIFFPD_int            [((ch+1)*  1)-1:(ch*  1)]),
        .TXDLYBYPASS              (GTHE3_CHANNEL_TXDLYBYPASS_int         [((ch+1)*  1)-1:(ch*  1)]),
        .TXDLYEN                  (GTHE3_CHANNEL_TXDLYEN_int             [((ch+1)*  1)-1:(ch*  1)]),
        .TXDLYHOLD                (GTHE3_CHANNEL_TXDLYHOLD_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXDLYOVRDEN              (GTHE3_CHANNEL_TXDLYOVRDEN_int         [((ch+1)*  1)-1:(ch*  1)]),
        .TXDLYSRESET              (GTHE3_CHANNEL_TXDLYSRESET_int         [((ch+1)*  1)-1:(ch*  1)]),
        .TXDLYUPDOWN              (GTHE3_CHANNEL_TXDLYUPDOWN_int         [((ch+1)*  1)-1:(ch*  1)]),
        .TXELECIDLE               (GTHE3_CHANNEL_TXELECIDLE_int          [((ch+1)*  1)-1:(ch*  1)]),
        .TXHEADER                 (GTHE3_CHANNEL_TXHEADER_int            [((ch+1)*  6)-1:(ch*  6)]),
        .TXINHIBIT                (GTHE3_CHANNEL_TXINHIBIT_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXLATCLK                 (GTHE3_CHANNEL_TXLATCLK_int            [((ch+1)*  1)-1:(ch*  1)]),
        .TXMAINCURSOR             (GTHE3_CHANNEL_TXMAINCURSOR_int        [((ch+1)*  7)-1:(ch*  7)]),
        .TXMARGIN                 (GTHE3_CHANNEL_TXMARGIN_int            [((ch+1)*  3)-1:(ch*  3)]),
        .TXOUTCLKSEL              (GTHE3_CHANNEL_TXOUTCLKSEL_int         [((ch+1)*  3)-1:(ch*  3)]),
        .TXPCSRESET               (GTHE3_CHANNEL_TXPCSRESET_int          [((ch+1)*  1)-1:(ch*  1)]),
        .TXPD                     (GTHE3_CHANNEL_TXPD_int                [((ch+1)*  2)-1:(ch*  2)]),
        .TXPDELECIDLEMODE         (GTHE3_CHANNEL_TXPDELECIDLEMODE_int    [((ch+1)*  1)-1:(ch*  1)]),
        .TXPHALIGN                (GTHE3_CHANNEL_TXPHALIGN_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXPHALIGNEN              (GTHE3_CHANNEL_TXPHALIGNEN_int         [((ch+1)*  1)-1:(ch*  1)]),
        .TXPHDLYPD                (GTHE3_CHANNEL_TXPHDLYPD_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXPHDLYRESET             (GTHE3_CHANNEL_TXPHDLYRESET_int        [((ch+1)*  1)-1:(ch*  1)]),
        .TXPHDLYTSTCLK            (GTHE3_CHANNEL_TXPHDLYTSTCLK_int       [((ch+1)*  1)-1:(ch*  1)]),
        .TXPHINIT                 (GTHE3_CHANNEL_TXPHINIT_int            [((ch+1)*  1)-1:(ch*  1)]),
        .TXPHOVRDEN               (GTHE3_CHANNEL_TXPHOVRDEN_int          [((ch+1)*  1)-1:(ch*  1)]),
        .TXPIPPMEN                (GTHE3_CHANNEL_TXPIPPMEN_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXPIPPMOVRDEN            (GTHE3_CHANNEL_TXPIPPMOVRDEN_int       [((ch+1)*  1)-1:(ch*  1)]),
        .TXPIPPMPD                (GTHE3_CHANNEL_TXPIPPMPD_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXPIPPMSEL               (GTHE3_CHANNEL_TXPIPPMSEL_int          [((ch+1)*  1)-1:(ch*  1)]),
        .TXPIPPMSTEPSIZE          (GTHE3_CHANNEL_TXPIPPMSTEPSIZE_int     [((ch+1)*  5)-1:(ch*  5)]),
        .TXPISOPD                 (GTHE3_CHANNEL_TXPISOPD_int            [((ch+1)*  1)-1:(ch*  1)]),
        .TXPLLCLKSEL              (GTHE3_CHANNEL_TXPLLCLKSEL_int         [((ch+1)*  2)-1:(ch*  2)]),
        .TXPMARESET               (GTHE3_CHANNEL_TXPMARESET_int          [((ch+1)*  1)-1:(ch*  1)]),
        .TXPOLARITY               (GTHE3_CHANNEL_TXPOLARITY_int          [((ch+1)*  1)-1:(ch*  1)]),
        .TXPOSTCURSOR             (GTHE3_CHANNEL_TXPOSTCURSOR_int        [((ch+1)*  5)-1:(ch*  5)]),
        .TXPOSTCURSORINV          (GTHE3_CHANNEL_TXPOSTCURSORINV_int     [((ch+1)*  1)-1:(ch*  1)]),
        .TXPRBSFORCEERR           (GTHE3_CHANNEL_TXPRBSFORCEERR_int      [((ch+1)*  1)-1:(ch*  1)]),
        .TXPRBSSEL                (GTHE3_CHANNEL_TXPRBSSEL_int           [((ch+1)*  4)-1:(ch*  4)]),
        .TXPRECURSOR              (GTHE3_CHANNEL_TXPRECURSOR_int         [((ch+1)*  5)-1:(ch*  5)]),
        .TXPRECURSORINV           (GTHE3_CHANNEL_TXPRECURSORINV_int      [((ch+1)*  1)-1:(ch*  1)]),
        .TXPROGDIVRESET           (GTHE3_CHANNEL_TXPROGDIVRESET_int      [((ch+1)*  1)-1:(ch*  1)]),
        .TXQPIBIASEN              (GTHE3_CHANNEL_TXQPIBIASEN_int         [((ch+1)*  1)-1:(ch*  1)]),
        .TXQPISTRONGPDOWN         (GTHE3_CHANNEL_TXQPISTRONGPDOWN_int    [((ch+1)*  1)-1:(ch*  1)]),
        .TXQPIWEAKPUP             (GTHE3_CHANNEL_TXQPIWEAKPUP_int        [((ch+1)*  1)-1:(ch*  1)]),
        .TXRATE                   (GTHE3_CHANNEL_TXRATE_int              [((ch+1)*  3)-1:(ch*  3)]),
        .TXRATEMODE               (GTHE3_CHANNEL_TXRATEMODE_int          [((ch+1)*  1)-1:(ch*  1)]),
        .TXSEQUENCE               (GTHE3_CHANNEL_TXSEQUENCE_int          [((ch+1)*  7)-1:(ch*  7)]),
        .TXSWING                  (GTHE3_CHANNEL_TXSWING_int             [((ch+1)*  1)-1:(ch*  1)]),
        .TXSYNCALLIN              (GTHE3_CHANNEL_TXSYNCALLIN_int         [((ch+1)*  1)-1:(ch*  1)]),
        .TXSYNCIN                 (GTHE3_CHANNEL_TXSYNCIN_int            [((ch+1)*  1)-1:(ch*  1)]),
        .TXSYNCMODE               (GTHE3_CHANNEL_TXSYNCMODE_int          [((ch+1)*  1)-1:(ch*  1)]),
        .TXSYSCLKSEL              (GTHE3_CHANNEL_TXSYSCLKSEL_int         [((ch+1)*  2)-1:(ch*  2)]),
        .TXUSERRDY                (GTHE3_CHANNEL_TXUSERRDY_int           [((ch+1)*  1)-1:(ch*  1)]),
        .TXUSRCLK                 (GTHE3_CHANNEL_TXUSRCLK_int            [((ch+1)*  1)-1:(ch*  1)]),
        .TXUSRCLK2                (GTHE3_CHANNEL_TXUSRCLK2_int           [((ch+1)*  1)-1:(ch*  1)]),

        .BUFGTCE                  (GTHE3_CHANNEL_BUFGTCE                 [((ch+1)*  3)-1:(ch*  3)]),
        .BUFGTCEMASK              (GTHE3_CHANNEL_BUFGTCEMASK             [((ch+1)*  3)-1:(ch*  3)]),
        .BUFGTDIV                 (GTHE3_CHANNEL_BUFGTDIV                [((ch+1)*  9)-1:(ch*  9)]),
        .BUFGTRESET               (GTHE3_CHANNEL_BUFGTRESET              [((ch+1)*  3)-1:(ch*  3)]),
        .BUFGTRSTMASK             (GTHE3_CHANNEL_BUFGTRSTMASK            [((ch+1)*  3)-1:(ch*  3)]),
        .CPLLFBCLKLOST            (GTHE3_CHANNEL_CPLLFBCLKLOST           [((ch+1)*  1)-1:(ch*  1)]),
        .CPLLLOCK                 (GTHE3_CHANNEL_CPLLLOCK                [((ch+1)*  1)-1:(ch*  1)]),
        .CPLLREFCLKLOST           (GTHE3_CHANNEL_CPLLREFCLKLOST          [((ch+1)*  1)-1:(ch*  1)]),
        .DMONITOROUT              (GTHE3_CHANNEL_DMONITOROUT             [((ch+1)* 17)-1:(ch* 17)]),
        .DRPDO                    (GTHE3_CHANNEL_DRPDO                   [((ch+1)* 16)-1:(ch* 16)]),
        .DRPRDY                   (GTHE3_CHANNEL_DRPRDY                  [((ch+1)*  1)-1:(ch*  1)]),
        .EYESCANDATAERROR         (GTHE3_CHANNEL_EYESCANDATAERROR        [((ch+1)*  1)-1:(ch*  1)]),
        .GTHTXN                   (GTHE3_CHANNEL_GTHTXN                  [((ch+1)*  1)-1:(ch*  1)]),
        .GTHTXP                   (GTHE3_CHANNEL_GTHTXP                  [((ch+1)*  1)-1:(ch*  1)]),
        .GTPOWERGOOD              (GTHE3_CHANNEL_GTPOWERGOOD             [((ch+1)*  1)-1:(ch*  1)]),
        .GTREFCLKMONITOR          (GTHE3_CHANNEL_GTREFCLKMONITOR         [((ch+1)*  1)-1:(ch*  1)]),
        .PCIERATEGEN3             (GTHE3_CHANNEL_PCIERATEGEN3            [((ch+1)*  1)-1:(ch*  1)]),
        .PCIERATEIDLE             (GTHE3_CHANNEL_PCIERATEIDLE            [((ch+1)*  1)-1:(ch*  1)]),
        .PCIERATEQPLLPD           (GTHE3_CHANNEL_PCIERATEQPLLPD          [((ch+1)*  2)-1:(ch*  2)]),
        .PCIERATEQPLLRESET        (GTHE3_CHANNEL_PCIERATEQPLLRESET       [((ch+1)*  2)-1:(ch*  2)]),
        .PCIESYNCTXSYNCDONE       (GTHE3_CHANNEL_PCIESYNCTXSYNCDONE      [((ch+1)*  1)-1:(ch*  1)]),
        .PCIEUSERGEN3RDY          (GTHE3_CHANNEL_PCIEUSERGEN3RDY         [((ch+1)*  1)-1:(ch*  1)]),
        .PCIEUSERPHYSTATUSRST     (GTHE3_CHANNEL_PCIEUSERPHYSTATUSRST    [((ch+1)*  1)-1:(ch*  1)]),
        .PCIEUSERRATESTART        (GTHE3_CHANNEL_PCIEUSERRATESTART       [((ch+1)*  1)-1:(ch*  1)]),
        .PCSRSVDOUT               (GTHE3_CHANNEL_PCSRSVDOUT              [((ch+1)* 12)-1:(ch* 12)]),
        .PHYSTATUS                (GTHE3_CHANNEL_PHYSTATUS               [((ch+1)*  1)-1:(ch*  1)]),
        .PINRSRVDAS               (GTHE3_CHANNEL_PINRSRVDAS              [((ch+1)*  8)-1:(ch*  8)]),
        .RESETEXCEPTION           (GTHE3_CHANNEL_RESETEXCEPTION          [((ch+1)*  1)-1:(ch*  1)]),
        .RXBUFSTATUS              (GTHE3_CHANNEL_RXBUFSTATUS             [((ch+1)*  3)-1:(ch*  3)]),
        .RXBYTEISALIGNED          (GTHE3_CHANNEL_RXBYTEISALIGNED         [((ch+1)*  1)-1:(ch*  1)]),
        .RXBYTEREALIGN            (GTHE3_CHANNEL_RXBYTEREALIGN           [((ch+1)*  1)-1:(ch*  1)]),
        .RXCDRLOCK                (GTHE3_CHANNEL_RXCDRLOCK               [((ch+1)*  1)-1:(ch*  1)]),
        .RXCDRPHDONE              (GTHE3_CHANNEL_RXCDRPHDONE             [((ch+1)*  1)-1:(ch*  1)]),
        .RXCHANBONDSEQ            (GTHE3_CHANNEL_RXCHANBONDSEQ           [((ch+1)*  1)-1:(ch*  1)]),
        .RXCHANISALIGNED          (GTHE3_CHANNEL_RXCHANISALIGNED         [((ch+1)*  1)-1:(ch*  1)]),
        .RXCHANREALIGN            (GTHE3_CHANNEL_RXCHANREALIGN           [((ch+1)*  1)-1:(ch*  1)]),
        .RXCHBONDO                (GTHE3_CHANNEL_RXCHBONDO               [((ch+1)*  5)-1:(ch*  5)]),
        .RXCLKCORCNT              (GTHE3_CHANNEL_RXCLKCORCNT             [((ch+1)*  2)-1:(ch*  2)]),
        .RXCOMINITDET             (GTHE3_CHANNEL_RXCOMINITDET            [((ch+1)*  1)-1:(ch*  1)]),
        .RXCOMMADET               (GTHE3_CHANNEL_RXCOMMADET              [((ch+1)*  1)-1:(ch*  1)]),
        .RXCOMSASDET              (GTHE3_CHANNEL_RXCOMSASDET             [((ch+1)*  1)-1:(ch*  1)]),
        .RXCOMWAKEDET             (GTHE3_CHANNEL_RXCOMWAKEDET            [((ch+1)*  1)-1:(ch*  1)]),
        .RXCTRL0                  (GTHE3_CHANNEL_RXCTRL0                 [((ch+1)* 16)-1:(ch* 16)]),
        .RXCTRL1                  (GTHE3_CHANNEL_RXCTRL1                 [((ch+1)* 16)-1:(ch* 16)]),
        .RXCTRL2                  (GTHE3_CHANNEL_RXCTRL2                 [((ch+1)*  8)-1:(ch*  8)]),
        .RXCTRL3                  (GTHE3_CHANNEL_RXCTRL3                 [((ch+1)*  8)-1:(ch*  8)]),
        .RXDATA                   (GTHE3_CHANNEL_RXDATA                  [((ch+1)*128)-1:(ch*128)]),
        .RXDATAEXTENDRSVD         (GTHE3_CHANNEL_RXDATAEXTENDRSVD        [((ch+1)*  8)-1:(ch*  8)]),
        .RXDATAVALID              (GTHE3_CHANNEL_RXDATAVALID             [((ch+1)*  2)-1:(ch*  2)]),
        .RXDLYSRESETDONE          (GTHE3_CHANNEL_RXDLYSRESETDONE         [((ch+1)*  1)-1:(ch*  1)]),
        .RXELECIDLE               (GTHE3_CHANNEL_RXELECIDLE              [((ch+1)*  1)-1:(ch*  1)]),
        .RXHEADER                 (GTHE3_CHANNEL_RXHEADER                [((ch+1)*  6)-1:(ch*  6)]),
        .RXHEADERVALID            (GTHE3_CHANNEL_RXHEADERVALID           [((ch+1)*  2)-1:(ch*  2)]),
        .RXMONITOROUT             (GTHE3_CHANNEL_RXMONITOROUT            [((ch+1)*  7)-1:(ch*  7)]),
        .RXOSINTDONE              (GTHE3_CHANNEL_RXOSINTDONE             [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSINTSTARTED           (GTHE3_CHANNEL_RXOSINTSTARTED          [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSINTSTROBEDONE        (GTHE3_CHANNEL_RXOSINTSTROBEDONE       [((ch+1)*  1)-1:(ch*  1)]),
        .RXOSINTSTROBESTARTED     (GTHE3_CHANNEL_RXOSINTSTROBESTARTED    [((ch+1)*  1)-1:(ch*  1)]),
        .RXOUTCLK                 (GTHE3_CHANNEL_RXOUTCLK                [((ch+1)*  1)-1:(ch*  1)]),
        .RXOUTCLKFABRIC           (GTHE3_CHANNEL_RXOUTCLKFABRIC          [((ch+1)*  1)-1:(ch*  1)]),
        .RXOUTCLKPCS              (GTHE3_CHANNEL_RXOUTCLKPCS             [((ch+1)*  1)-1:(ch*  1)]),
        .RXPHALIGNDONE            (GTHE3_CHANNEL_RXPHALIGNDONE           [((ch+1)*  1)-1:(ch*  1)]),
        .RXPHALIGNERR             (GTHE3_CHANNEL_RXPHALIGNERR            [((ch+1)*  1)-1:(ch*  1)]),
        .RXPMARESETDONE           (GTHE3_CHANNEL_RXPMARESETDONE          [((ch+1)*  1)-1:(ch*  1)]),
        .RXPRBSERR                (GTHE3_CHANNEL_RXPRBSERR               [((ch+1)*  1)-1:(ch*  1)]),
        .RXPRBSLOCKED             (GTHE3_CHANNEL_RXPRBSLOCKED            [((ch+1)*  1)-1:(ch*  1)]),
        .RXPRGDIVRESETDONE        (GTHE3_CHANNEL_RXPRGDIVRESETDONE       [((ch+1)*  1)-1:(ch*  1)]),
        .RXQPISENN                (GTHE3_CHANNEL_RXQPISENN               [((ch+1)*  1)-1:(ch*  1)]),
        .RXQPISENP                (GTHE3_CHANNEL_RXQPISENP               [((ch+1)*  1)-1:(ch*  1)]),
        .RXRATEDONE               (GTHE3_CHANNEL_RXRATEDONE              [((ch+1)*  1)-1:(ch*  1)]),
        .RXRECCLKOUT              (GTHE3_CHANNEL_RXRECCLKOUT             [((ch+1)*  1)-1:(ch*  1)]),
        .RXRESETDONE              (GTHE3_CHANNEL_RXRESETDONE             [((ch+1)*  1)-1:(ch*  1)]),
        .RXSLIDERDY               (GTHE3_CHANNEL_RXSLIDERDY              [((ch+1)*  1)-1:(ch*  1)]),
        .RXSLIPDONE               (GTHE3_CHANNEL_RXSLIPDONE              [((ch+1)*  1)-1:(ch*  1)]),
        .RXSLIPOUTCLKRDY          (GTHE3_CHANNEL_RXSLIPOUTCLKRDY         [((ch+1)*  1)-1:(ch*  1)]),
        .RXSLIPPMARDY             (GTHE3_CHANNEL_RXSLIPPMARDY            [((ch+1)*  1)-1:(ch*  1)]),
        .RXSTARTOFSEQ             (GTHE3_CHANNEL_RXSTARTOFSEQ            [((ch+1)*  2)-1:(ch*  2)]),
        .RXSTATUS                 (GTHE3_CHANNEL_RXSTATUS                [((ch+1)*  3)-1:(ch*  3)]),
        .RXSYNCDONE               (GTHE3_CHANNEL_RXSYNCDONE              [((ch+1)*  1)-1:(ch*  1)]),
        .RXSYNCOUT                (GTHE3_CHANNEL_RXSYNCOUT               [((ch+1)*  1)-1:(ch*  1)]),
        .RXVALID                  (GTHE3_CHANNEL_RXVALID                 [((ch+1)*  1)-1:(ch*  1)]),
        .TXBUFSTATUS              (GTHE3_CHANNEL_TXBUFSTATUS             [((ch+1)*  2)-1:(ch*  2)]),
        .TXCOMFINISH              (GTHE3_CHANNEL_TXCOMFINISH             [((ch+1)*  1)-1:(ch*  1)]),
        .TXDLYSRESETDONE          (GTHE3_CHANNEL_TXDLYSRESETDONE         [((ch+1)*  1)-1:(ch*  1)]),
        .TXOUTCLK                 (GTHE3_CHANNEL_TXOUTCLK                [((ch+1)*  1)-1:(ch*  1)]),
        .TXOUTCLKFABRIC           (GTHE3_CHANNEL_TXOUTCLKFABRIC          [((ch+1)*  1)-1:(ch*  1)]),
        .TXOUTCLKPCS              (GTHE3_CHANNEL_TXOUTCLKPCS             [((ch+1)*  1)-1:(ch*  1)]),
        .TXPHALIGNDONE            (GTHE3_CHANNEL_TXPHALIGNDONE           [((ch+1)*  1)-1:(ch*  1)]),
        .TXPHINITDONE             (GTHE3_CHANNEL_TXPHINITDONE            [((ch+1)*  1)-1:(ch*  1)]),
        .TXPMARESETDONE           (GTHE3_CHANNEL_TXPMARESETDONE          [((ch+1)*  1)-1:(ch*  1)]),
        .TXPRGDIVRESETDONE        (GTHE3_CHANNEL_TXPRGDIVRESETDONE       [((ch+1)*  1)-1:(ch*  1)]),
        .TXQPISENN                (GTHE3_CHANNEL_TXQPISENN               [((ch+1)*  1)-1:(ch*  1)]),
        .TXQPISENP                (GTHE3_CHANNEL_TXQPISENP               [((ch+1)*  1)-1:(ch*  1)]),
        .TXRATEDONE               (GTHE3_CHANNEL_TXRATEDONE              [((ch+1)*  1)-1:(ch*  1)]),
        .TXRESETDONE              (GTHE3_CHANNEL_TXRESETDONE             [((ch+1)*  1)-1:(ch*  1)]),
        .TXSYNCDONE               (GTHE3_CHANNEL_TXSYNCDONE              [((ch+1)*  1)-1:(ch*  1)]),
        .TXSYNCOUT                (GTHE3_CHANNEL_TXSYNCOUT               [((ch+1)*  1)-1:(ch*  1)])
      );
    end

  end
  endgenerate


endmodule

module gtwizard_ultrascale_v1_6_2_gthe3_cpll_cal # (
        parameter REVISION = 2
)(
        // control signals
        input wire  [17:0] TXOUTCLK_PERIOD_IN,
        input wire  [15:0] WAIT_DEASSERT_CPLLPD_IN,
        input wire  [17:0] CNT_TOL_IN,
        input wire  [15:0] FREQ_COUNT_WINDOW_IN,
        // User Interface
        input wire         RESET_IN,
        input wire         CLK_IN,
        input wire         USER_TXPROGDIVRESET_IN,
        output reg         USER_TXPRGDIVRESETDONE_OUT,
        input wire  [2:0]  USER_TXOUTCLKSEL_IN,
        input wire         USER_TXOUTCLK_BUFG_CE_IN,
        input wire         USER_TXOUTCLK_BUFG_CLR_IN,
        output reg         USER_CPLLLOCK_OUT,
        input wire  [8:0]  USER_CHANNEL_DRPADDR_IN,
        input wire  [15:0] USER_CHANNEL_DRPDI_IN,
        input wire         USER_CHANNEL_DRPEN_IN,
        input wire         USER_CHANNEL_DRPWE_IN,
        output reg         USER_CHANNEL_DRPRDY_OUT,
        output reg [15:0]  USER_CHANNEL_DRPDO_OUT,
        // Debug Interface
        output wire        CPLL_CAL_FAIL,
        output wire        CPLL_CAL_DONE,
        output wire [15:0] DEBUG_OUT,
        // GT Interface
        input wire         GTHE3_TXOUTCLK_IN,
        input wire         GTHE3_CPLLLOCK_IN,
        output wire        GTHE3_CPLLRESET_OUT,
        output wire        GTHE3_CPLLPD_OUT,
        output reg         GTHE3_TXPROGDIVRESET_OUT,
        output reg  [2:0]  GTHE3_TXOUTCLKSEL_OUT,
        input wire         GTHE3_TXPRGDIVRESETDONE_IN,
        output reg [8:0]   GTHE3_CHANNEL_DRPADDR_OUT,
        output reg [15:0]  GTHE3_CHANNEL_DRPDI_OUT,
        output reg         GTHE3_CHANNEL_DRPEN_OUT,
        output reg         GTHE3_CHANNEL_DRPWE_OUT,
        input wire         GTHE3_CHANNEL_DRPRDY_IN,
        input wire  [15:0] GTHE3_CHANNEL_DRPDO_IN
);


  //DRP FSM
  localparam DRP_WAIT      = 0;
  localparam DRP_READ      = 1;
  localparam DRP_READ_ACK  = 2;
  localparam DRP_MODIFY    = 3;
  localparam DRP_WRITE     = 4;
  localparam DRP_WRITE_ACK = 5;
  localparam DRP_DONE      = 6;

  //CPLL CAL FSM
  localparam RESET                = 0;
  localparam READ_PROGCLK_SEL     = 1;
  localparam MODIFY_PROGCLK_SEL   = 2;
  localparam READ_PROGDIV         = 3;
  localparam MODIFY_PROGDIV       = 4;
  localparam MODIFY_TXOUTCLK_SEL  = 5;
  localparam ASSERT_CPLLRESET1    = 6;
  localparam READ_FBOOST          = 7;
  localparam MODIFY_FBOOST        = 8;
  localparam ASSERT_CPLLRESET     = 9;
  localparam ASSERT_CPLLPD        = 10;
  localparam DEASSERT_CPLLPD      = 11;
  localparam DEASSERT_CPLLRESET   = 12;
  localparam RESTORE_FBOOST       = 13;
  localparam WAIT_GTCPLLLOCK      = 14;
  localparam ASSERT_PROGDIVRESET  = 15;
  localparam WAIT_PRGDIVRESETDONE = 16;
  localparam CHECK_FREQ           = 17;
  localparam RESTORE_PROGDIV      = 18;
  localparam RESTORE_PROGCLK_SEL  = 19;
  localparam ASSERT_CPLLRESET2    = 20;
  localparam WAIT_GTCPLLLOCK2     = 21;
  localparam ASSERT_PROGDIVRESET2 = 22;
  localparam WAIT_PRGDIVRESETDONE2= 23;
  localparam CAL_FAIL             = 24;
  localparam CAL_DONE             = 25;

  reg [25:0] cpll_cal_state = 26'd0;
  wire [4:0] cpll_cal_state_bin;
  reg [6:0] drp_state = 7'd1;
  wire drp_done;
  reg [8:0] daddr = 9'd0;
  //reg [8:0] daddr = 9'd0;
  reg [15:0] di = 16'd0;
  wire drdy;
  wire [15:0] dout;
  reg den = 1'b0;
  reg dwe = 1'b0;
  reg wr = 1'b0;
  reg rd = 1'b0;
  reg [15:0] di_msk;
  reg [15:0] mask;
  //reg [15:0] mask;
  reg [15:0] wait_ctr;
  reg [3:0] repeat_ctr;
  reg [1:0] progclk_sel_store = 2'b00;
  reg [15:0] progdiv_cfg_store = 16'd0;
  reg fboost_store = 1'b0;
  reg mask_user_in = 1'b0;
  reg cpllreset_int = 1'b0;
  reg cpllpd_int = 1'b0;
  reg txprogdivreset_int = 1'b0;
  reg [2:0] txoutclksel_int = 3'b000;
  reg cal_fail_store = 1'b0;

  localparam [4:0]  WAIT_WIDTH_PROGDIVRESET = 5'd31;
  localparam [4:0]  WAIT_ASSERT_CPLLRESET   = 5'd31;
  localparam [4:0]  WAIT_ASSERT_CPLLPD      = 5'd31;
  localparam [4:0]  WAIT_DEASSERT_CPLLRESET = 5'd31;
  localparam [15:0] WAIT_CPLLLOCK = 16'h4000;
  localparam [3:0]  REPEAT_RESET_LIMIT = 4'd15;

  localparam [1:0]  MOD_PROGCLK_SEL = 2'b10;
  localparam [15:0] MOD_PROGDIV_CFG = 16'hA1A2; //divider 20
  localparam [2:0]  MOD_TXOUTCLK_SEL = 3'b101;
  localparam        MOD_FBOOST = 1'b1;
  localparam [8:0]  ADDR_TX_PROGCLK_SEL = 9'h00C;
  localparam [8:0]  ADDR_TX_PROGDIV_CFG = 9'h03E;
  localparam [8:0]  ADDR_FBOOST = 9'h0CB;

  // Drive TXOUTCLK with BUFG_GT-buffered source clock, divider = 1
  wire txoutclkmon;
  BUFG_GT bufg_gt_txoutclkmon_inst (
    .CE      (USER_TXOUTCLK_BUFG_CE_IN),
    .CEMASK  (1'b1),
    .CLR     (USER_TXOUTCLK_BUFG_CLR_IN),
    .CLRMASK (1'b1),
    .DIV     (3'b000),
    .I       (GTHE3_TXOUTCLK_IN),
    .O       (txoutclkmon)
  );

  wire reset_in_sync;
  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_resetin_inst (
    .clk_in (CLK_IN),
    .rst_in   (RESET_IN),
    .rst_out  (reset_in_sync)
  );

  wire gthe3_cplllock_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_cplllock_inst (
    .clk_in (CLK_IN),
    .i_in   (GTHE3_CPLLLOCK_IN),
    .o_out  (gthe3_cplllock_sync)
  );

  wire user_txprogdivreset_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_txprogdivreset_inst (
    .clk_in (CLK_IN),
    .i_in   (USER_TXPROGDIVRESET_IN),
    .o_out  (user_txprogdivreset_sync)
  );

  wire gthe3_txprgdivresetdone_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_txprgdivresetdone_inst (
    .clk_in (CLK_IN),
    .i_in   (GTHE3_TXPRGDIVRESETDONE_IN),
    .o_out  (gthe3_txprgdivresetdone_sync)
  );

  wire [2:0] user_txoutclksel_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_txoutclksel_inst0 (
    .clk_in (CLK_IN),
    .i_in   (USER_TXOUTCLKSEL_IN[0]),
    .o_out  (user_txoutclksel_sync[0])
  );
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_txoutclksel_inst1 (
    .clk_in (CLK_IN),
    .i_in   (USER_TXOUTCLKSEL_IN[1]),
    .o_out  (user_txoutclksel_sync[1])
  );
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_txoutclksel_inst2 (
    .clk_in (CLK_IN),
    .i_in   (USER_TXOUTCLKSEL_IN[2]),
    .o_out  (user_txoutclksel_sync[2])
  );

  assign GTHE3_CPLLRESET_OUT = cpllreset_int;
  assign GTHE3_CPLLPD_OUT = cpllpd_int;

  always @(posedge CLK_IN) begin
    if (mask_user_in | cpll_cal_state[CAL_FAIL] | cpll_cal_state[RESET] | reset_in_sync)
      USER_CPLLLOCK_OUT <= 1'b0;
    else
      USER_CPLLLOCK_OUT <= gthe3_cplllock_sync;
  end

  always @(posedge CLK_IN) begin
    if (mask_user_in)
      GTHE3_TXPROGDIVRESET_OUT <= txprogdivreset_int;
    else
      GTHE3_TXPROGDIVRESET_OUT <= user_txprogdivreset_sync;
  end

  always @(posedge CLK_IN) begin
    if (mask_user_in)
      GTHE3_TXOUTCLKSEL_OUT <= txoutclksel_int;
    else
      GTHE3_TXOUTCLKSEL_OUT <= user_txoutclksel_sync;
  end

  always @(posedge CLK_IN) begin
    if (mask_user_in)
      USER_TXPRGDIVRESETDONE_OUT <= 1'b0;
    else
      USER_TXPRGDIVRESETDONE_OUT <= gthe3_txprgdivresetdone_sync;
  end

  // frequency counter for txoutclk
  wire [17:0] txoutclk_freq_cnt;
  reg freq_counter_rst = 1'b1;
  wire freq_cnt_done;
  gtwizard_ultrascale_v1_6_2_gthe3_cpll_cal_freq_counter U_TXOUTCLK_FREQ_COUNTER
    (
      .freq_cnt_o(txoutclk_freq_cnt),
      .done_o(freq_cnt_done),
      .rst_i(freq_counter_rst),
      .test_term_cnt_i(FREQ_COUNT_WINDOW_IN),
      .ref_clk_i(CLK_IN),
      .test_clk_i(txoutclkmon)
    );

  //Debug signals
  assign DEBUG_OUT = {cpllreset_int,cpllpd_int,gthe3_cplllock_sync,fboost_store,freq_cnt_done,freq_counter_rst,mask_user_in,cpll_cal_state_bin,repeat_ctr};
  assign CPLL_CAL_FAIL = cpll_cal_state[CAL_FAIL];
  assign CPLL_CAL_DONE = cpll_cal_state[CAL_DONE];

  //CPLL CAL FSM
  always @(posedge CLK_IN) begin
      if (reset_in_sync) begin
        cpll_cal_state <= 0;
        cpll_cal_state[RESET] <= 1'b1;
        cpllreset_int <= 1'b0;
        cpllpd_int <= 1'b0;
        txprogdivreset_int <= 1'b0;
        mask_user_in <= 1'b0;
        wr <= 1'b0;
        rd <= 1'b0;
      end
      else begin
         cpll_cal_state <= 0;
         case(1'b1) // synthesis parallel_case full_case
             cpll_cal_state[RESET]:
             begin
               wait_ctr <= 16'd0;
               repeat_ctr <= 4'd0;
               mask_user_in <= 1'b1;
               di_msk <= 16'b0000_0000_0000_0000;
               cpll_cal_state[READ_PROGCLK_SEL] <= 1'b1;
             end

             cpll_cal_state[READ_PROGCLK_SEL]:
             begin
               rd <= 1'b1;
               if (drp_done) begin
                 rd <= 1'b0;
                 progclk_sel_store <= dout[11:10];
                 cpll_cal_state[MODIFY_PROGCLK_SEL] <= 1'b1;
               end
               else begin
                 cpll_cal_state[READ_PROGCLK_SEL] <= 1'b1;
               end
             end

             cpll_cal_state[MODIFY_PROGCLK_SEL]:
             begin
               if (!drp_done) begin
                 wr <= 1'b1;
                 cpll_cal_state[MODIFY_PROGCLK_SEL] <= 1'b1;
               end
               else begin
                 wr <= 1'b0;
                 cpll_cal_state[READ_PROGDIV] <= 1'b1;
               end
               di_msk<= {4'd0,MOD_PROGCLK_SEL,10'd0};
             end

             cpll_cal_state[READ_PROGDIV]:
             begin
               rd <= 1'b1;
               if (drp_done) begin
                 rd <= 1'b0;
                 progdiv_cfg_store <= dout;
                 cpll_cal_state[MODIFY_PROGDIV] <= 1'b1;
               end
               else begin
                 cpll_cal_state[READ_PROGDIV] <= 1'b1;
               end
             end

             cpll_cal_state[MODIFY_PROGDIV]:
             begin
               if (!drp_done) begin
                 wr <= 1'b1;
                 cpll_cal_state[MODIFY_PROGDIV] <= 1'b1;
               end
               else begin
                 wr <= 1'b0;
                 cpll_cal_state[MODIFY_TXOUTCLK_SEL] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
               di_msk<= MOD_PROGDIV_CFG;
             end

             cpll_cal_state[MODIFY_TXOUTCLK_SEL]:
             begin
               cpll_cal_state[ASSERT_CPLLRESET1] <= 1'b1;
             end

             cpll_cal_state[ASSERT_CPLLRESET1]:
             begin
               if (wait_ctr < WAIT_ASSERT_CPLLRESET) begin
                 wait_ctr <= wait_ctr + 1'b1;
                 cpll_cal_state[ASSERT_CPLLRESET1] <= 1'b1;
                 cpllreset_int <= 1'b1;
               end
               else begin
                 cpllreset_int <= 1'b0;
                 freq_counter_rst <= 1'b1;
                 cpll_cal_state[WAIT_GTCPLLLOCK] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
               repeat_ctr <= 4'd0;
             end

             cpll_cal_state[READ_FBOOST]:
             begin
               rd <= 1'b1;
               if (drp_done) begin
                 rd <= 1'b0;
                 fboost_store <= dout[2];
                 cpll_cal_state[MODIFY_FBOOST] <= 1'b1;
               end
               else begin
                 cpll_cal_state[READ_FBOOST] <= 1'b1;
               end
             end

             cpll_cal_state[MODIFY_FBOOST]:
             begin
               if (!drp_done) begin
                 wr <= 1'b1;
                 cpll_cal_state[MODIFY_FBOOST] <= 1'b1;
               end
               else begin
                 wr <= 1'b0;
                 cpll_cal_state[ASSERT_CPLLRESET] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
               di_msk<= {13'd0,MOD_FBOOST,2'b00};
             end

             cpll_cal_state[ASSERT_CPLLRESET]:
             begin
               if (wait_ctr < WAIT_ASSERT_CPLLRESET) begin
                 wait_ctr <= wait_ctr + 1'b1;
                 cpll_cal_state[ASSERT_CPLLRESET] <= 1'b1;
               end
               else begin
                 cpllreset_int <= 1'b1;
                 freq_counter_rst <= 1'b1;
                 cpll_cal_state[ASSERT_CPLLPD] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
             end

             cpll_cal_state[ASSERT_CPLLPD]:
             begin
               if (wait_ctr < WAIT_ASSERT_CPLLPD) begin
                 wait_ctr <= wait_ctr + 1'b1;
                 cpll_cal_state[ASSERT_CPLLPD] <= 1'b1;
               end
               else begin
                 cpllpd_int <= 1'b1;
                 cpll_cal_state[DEASSERT_CPLLPD] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
             end

             cpll_cal_state[DEASSERT_CPLLPD]:
             begin
               if (wait_ctr < WAIT_DEASSERT_CPLLPD_IN) begin
                 wait_ctr <= wait_ctr + 1'b1;
                 cpll_cal_state[DEASSERT_CPLLPD] <= 1'b1;
               end
               else begin
                 cpllpd_int <= 1'b0;
                 cpll_cal_state[DEASSERT_CPLLRESET] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
             end

             cpll_cal_state[DEASSERT_CPLLRESET]:
             begin
               if (wait_ctr < WAIT_DEASSERT_CPLLRESET) begin
                 wait_ctr <= wait_ctr + 1'b1;
                 cpll_cal_state[DEASSERT_CPLLRESET] <= 1'b1;
               end
               else begin
                 cpllreset_int <= 1'b0;
                 cpll_cal_state[RESTORE_FBOOST] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
             end

             cpll_cal_state[RESTORE_FBOOST]:
             begin
               if (!drp_done) begin
                 wr <= 1'b1;
                 cpll_cal_state[RESTORE_FBOOST] <= 1'b1;
               end
               else begin
                 wr <= 1'b0;
                 cpll_cal_state[WAIT_GTCPLLLOCK] <= 1'b1;
               end
               di_msk<= {13'd0,fboost_store,2'b00};
             end

             cpll_cal_state[WAIT_GTCPLLLOCK]:
             begin
               if(wait_ctr < WAIT_CPLLLOCK) begin
                 cpll_cal_state[WAIT_GTCPLLLOCK] <= 1'b1;
                 wait_ctr <= wait_ctr + 1'b1;
               end
               else begin
                 cpll_cal_state[ASSERT_PROGDIVRESET] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
             end

             cpll_cal_state[ASSERT_PROGDIVRESET]:
             begin
               if (wait_ctr < WAIT_WIDTH_PROGDIVRESET) begin
                 wait_ctr <= wait_ctr + 1'b1;
                 txprogdivreset_int <= 1'b1;
                 cpll_cal_state[ASSERT_PROGDIVRESET] <= 1'b1;
               end
               else begin
                 txprogdivreset_int <= 1'b0;
                 cpll_cal_state[WAIT_PRGDIVRESETDONE] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
             end

             cpll_cal_state[WAIT_PRGDIVRESETDONE]:
             begin
               if (gthe3_txprgdivresetdone_sync) begin
                 cpll_cal_state[CHECK_FREQ] <= 1'b1;
                 freq_counter_rst <= 1'b0;
               end
               else begin
                 cpll_cal_state[WAIT_PRGDIVRESETDONE] <= 1'b1;
               end
             end

             cpll_cal_state[CHECK_FREQ]:
             begin
               if(freq_cnt_done) begin
                 if ((txoutclk_freq_cnt >= (TXOUTCLK_PERIOD_IN - CNT_TOL_IN)) & (txoutclk_freq_cnt <= (TXOUTCLK_PERIOD_IN + CNT_TOL_IN))) begin
                   cpll_cal_state[RESTORE_PROGDIV] <= 1'b1;
                   cal_fail_store <= 1'b0;
                 end
                 else begin
                   if (repeat_ctr < REPEAT_RESET_LIMIT) begin
                     cpll_cal_state[READ_FBOOST] <= 1'b1;
                     repeat_ctr <= repeat_ctr + 1'b1;
                   end
                   else begin
                     cpll_cal_state[RESTORE_PROGDIV] <= 1'b1;
                     cal_fail_store <= 1'b1;
                   end
                 end
               end
               else
                 cpll_cal_state[CHECK_FREQ] <= 1'b1;
             end

             cpll_cal_state[RESTORE_PROGDIV]:
             begin
               if (!drp_done) begin
                 wr <= 1'b1;
                 cpll_cal_state[RESTORE_PROGDIV] <= 1'b1;
               end
               else begin
                 wr <= 1'b0;
                 cpll_cal_state[RESTORE_PROGCLK_SEL] <= 1'b1;
               end
               di_msk<= progdiv_cfg_store;
             end

             cpll_cal_state[RESTORE_PROGCLK_SEL]:
             begin
               if (!drp_done) begin
                 wr <= 1'b1;
                 cpll_cal_state[RESTORE_PROGCLK_SEL] <= 1'b1;
               end
               else begin
                 wr <= 1'b0;
                 cpll_cal_state[ASSERT_CPLLRESET2] <= 1'b1;
               end
               di_msk<= {4'd0,progclk_sel_store,10'd0};
             end

             cpll_cal_state[ASSERT_CPLLRESET2]:
             begin
               if (wait_ctr < WAIT_ASSERT_CPLLRESET) begin
                 wait_ctr <= wait_ctr + 1'b1;
                 cpll_cal_state[ASSERT_CPLLRESET2] <= 1'b1;
                 cpllreset_int <= 1'b1;
               end
               else begin
                 cpllreset_int <= 1'b0;
                 cpll_cal_state[WAIT_GTCPLLLOCK2] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
             end

             cpll_cal_state[WAIT_GTCPLLLOCK2]:
             begin
               if(gthe3_cplllock_sync)
                 cpll_cal_state[ASSERT_PROGDIVRESET2] <= 1'b1;
               else
                 cpll_cal_state[WAIT_GTCPLLLOCK2] <= 1'b1;
             end

             cpll_cal_state[ASSERT_PROGDIVRESET2]:
             begin
               if (wait_ctr < WAIT_WIDTH_PROGDIVRESET) begin
                 wait_ctr <= wait_ctr + 1'b1;
                 txprogdivreset_int <= 1'b1;
                 cpll_cal_state[ASSERT_PROGDIVRESET2] <= 1'b1;
               end
               else begin
                 txprogdivreset_int <= 1'b0;
                 cpll_cal_state[WAIT_PRGDIVRESETDONE2] <= 1'b1;
                 wait_ctr <= 16'd0;
               end
             end

             cpll_cal_state[WAIT_PRGDIVRESETDONE2]:
             begin
               if (gthe3_txprgdivresetdone_sync) begin
                 if (cal_fail_store)
                   cpll_cal_state[CAL_FAIL] <= 1'b1;
                 else
                   cpll_cal_state[CAL_DONE] <= 1'b1;
               end
               else begin
                 cpll_cal_state[WAIT_PRGDIVRESETDONE2] <= 1'b1;
               end
             end

             cpll_cal_state[CAL_FAIL]:
             begin
               cpll_cal_state[CAL_FAIL] <= 1'b1;
               mask_user_in <= 1'b0;
             end

             cpll_cal_state[CAL_DONE]:
             begin
               cpll_cal_state[CAL_DONE] <= 1'b1;
               mask_user_in <= 1'b0;
             end
         endcase
       end
   end // always block

   always @(posedge CLK_IN) begin
     if (cpll_cal_state[RESET])
       txoutclksel_int <= 3'b0;
     else if (cpll_cal_state[MODIFY_TXOUTCLK_SEL])
       txoutclksel_int <= MOD_TXOUTCLK_SEL;
   end

   always @(posedge CLK_IN) begin
     if      (cpll_cal_state[RESET]) begin
       daddr <= 9'h000;
       mask  <= 16'b1111_1111_1111_1111;
     end
     else if (cpll_cal_state[READ_PROGCLK_SEL] | cpll_cal_state[MODIFY_PROGCLK_SEL] | cpll_cal_state[RESTORE_PROGCLK_SEL]) begin
       daddr <= ADDR_TX_PROGCLK_SEL;
       mask  <= 16'b1111_0011_1111_1111;
     end
     else if (cpll_cal_state[READ_PROGDIV] | cpll_cal_state[MODIFY_PROGDIV] | cpll_cal_state[RESTORE_PROGDIV]) begin
       daddr <= ADDR_TX_PROGDIV_CFG;
       mask  <= 16'b0000_0000_0000_0000;
     end
     else if (cpll_cal_state[READ_FBOOST] | cpll_cal_state[MODIFY_FBOOST] | cpll_cal_state[RESTORE_FBOOST]) begin
       daddr <= ADDR_FBOOST;
       mask  <= 16'b1111_1111_1111_1011;
     end
   end


  // DRP FSM
  always @(posedge CLK_IN) begin
    if (mask_user_in) begin
      GTHE3_CHANNEL_DRPADDR_OUT <= daddr;
      GTHE3_CHANNEL_DRPDI_OUT   <= di;
      GTHE3_CHANNEL_DRPEN_OUT   <= den;
      GTHE3_CHANNEL_DRPWE_OUT   <= dwe;
      USER_CHANNEL_DRPRDY_OUT   <= 1'b0;
      USER_CHANNEL_DRPDO_OUT    <= 16'd0;
    end
    else begin
      GTHE3_CHANNEL_DRPADDR_OUT <= USER_CHANNEL_DRPADDR_IN;
      GTHE3_CHANNEL_DRPDI_OUT   <= USER_CHANNEL_DRPDI_IN;
      GTHE3_CHANNEL_DRPEN_OUT   <= USER_CHANNEL_DRPEN_IN;
      GTHE3_CHANNEL_DRPWE_OUT   <= USER_CHANNEL_DRPWE_IN;
      USER_CHANNEL_DRPRDY_OUT   <= GTHE3_CHANNEL_DRPRDY_IN;
      USER_CHANNEL_DRPDO_OUT    <= GTHE3_CHANNEL_DRPDO_IN;
    end
  end
  assign drdy = GTHE3_CHANNEL_DRPRDY_IN;
  assign dout = GTHE3_CHANNEL_DRPDO_IN;

  always @(posedge CLK_IN or posedge reset_in_sync) begin
  if (reset_in_sync) begin
    den <= 1'b0;
    dwe <= 1'b0;
    di <= 16'h0000;
    drp_state <= 0;
    drp_state[DRP_WAIT] <= 1'b1;
  end
  else begin
    drp_state <= 0;
    case (1'b1) // synthesis parallel_case full_case
        drp_state[DRP_WAIT]:
        begin
          if (wr | rd) drp_state[DRP_READ] <= 1'b1;
          else         drp_state[DRP_WAIT] <= 1'b1;
        end
        drp_state[DRP_READ]:
        begin
          den <= 1'b1;
          drp_state[DRP_READ_ACK] <= 1'b1;
        end
        drp_state[DRP_READ_ACK]:
        begin
          den <= 1'b0;
          if (drdy == 1'b1) begin
            if (rd) drp_state[DRP_DONE] <= 1'b1;
            else    drp_state[DRP_MODIFY] <= 1'b1;
          end
          else      drp_state[DRP_READ_ACK] <= 1'b1;
        end
        drp_state[DRP_MODIFY]:
        begin
          di <= di_msk | (dout & mask);
          drp_state[DRP_WRITE] <= 1'b1;
        end
        drp_state[DRP_WRITE]:
        begin
          den <= 1'b1;
          dwe <= 1'b1;
          drp_state[DRP_WRITE_ACK] <= 1'b1;
        end
        drp_state[DRP_WRITE_ACK]:
        begin
          den <= 1'b0;
          dwe <= 1'b0;
          if (drdy == 1'b1) drp_state[DRP_DONE] <= 1'b1;
          else              drp_state[DRP_WRITE_ACK] <= 1'b1;
        end
        drp_state[DRP_DONE]:
        begin
          drp_state[DRP_WAIT] <= 1'b1;
        end
    endcase
  end
  end

  assign drp_done = drp_state[DRP_DONE];


  //debug logic - convert one hot state to binary
  genvar i,j;
  generate
    for (j=0; j<5; j=j+1)
    begin : jl
      wire [26-1:0] tmp_mask;
      for (i=0; i<26; i=i+1)
      begin : il
        assign tmp_mask[i] = i[j];
      end
      assign cpll_cal_state_bin[j] = |(tmp_mask & cpll_cal_state);
    end
  endgenerate

endmodule //CPLL_CAL

`ifdef BOGUS
module gtwizard_ultrascale_v1_6_2_reset_synchronizer # (

  parameter FREQUENCY = 512

)(

  input  wire clk_in,
  input  wire rst_in,
  output wire rst_out

);

  reg tmpreg;
  always @(posedge clk_in) begin
    tmpreg  <= rst_in;
  end

  assign rst_out = tmpreg;

endmodule
`endif

module gtwizard_ultrascale_v1_6_2_reset_synchronizer # (

  parameter FREQUENCY = 512

)(

  input  wire clk_in,
  input  wire rst_in,
  output wire rst_out

);

  // Use 5 flip-flops as a single synchronizer, and tag each declaration with the appropriate synthesis attribute to
  // enable clustering. Each flip-flop in the synchronizer is asynchronously reset so that the downstream logic is also
  // asynchronously reset but encounters no reset assertion latency. The removal of reset is synchronous, so that the
  // downstream logic is also removed from reset synchronously. This module is designed for active-high reset use.

  (* ASYNC_REG = "TRUE" *) reg rst_in_meta  = 1'b0;
  (* ASYNC_REG = "TRUE" *) reg rst_in_sync1 = 1'b0;
  (* ASYNC_REG = "TRUE" *) reg rst_in_sync2 = 1'b0;
  (* ASYNC_REG = "TRUE" *) reg rst_in_sync3 = 1'b0;
                           reg rst_in_out   = 1'b0;

  always @(posedge clk_in, posedge rst_in) begin
    if (rst_in) begin
      rst_in_meta  <= 1'b1;
      rst_in_sync1 <= 1'b1;
      rst_in_sync2 <= 1'b1;
      rst_in_sync3 <= 1'b1;
      rst_in_out   <= 1'b1;
    end
    else begin
      rst_in_meta  <= 1'b0;
      rst_in_sync1 <= rst_in_meta;
      rst_in_sync2 <= rst_in_sync1;
      rst_in_sync3 <= rst_in_sync2;
      rst_in_out   <= rst_in_sync3;
    end
  end

  assign rst_out = rst_in_out;


endmodule

`ifdef BOGUS
module gtwizard_ultrascale_v1_6_2_bit_synchronizer # (
  parameter INITIALIZE = 5'b00000,
  parameter FREQUENCY  = 512
)(
  input  clk_in,
  input  i_in,
  output o_out
);

  reg tmpreg;
  always @(posedge clk_in) begin
    tmpreg  <= i_in;
  end

  assign o_out = tmpreg;

endmodule
`endif

module gtwizard_ultrascale_v1_6_2_bit_synchronizer # (

  parameter INITIALIZE = 5'b00000,
  parameter FREQUENCY  = 512

)(

  input  wire clk_in,
  input  wire i_in,
  output wire o_out

);

  // Use 5 flip-flops as a single synchronizer, and tag each declaration with the appropriate synthesis attribute to
  // enable clustering. Their GSR default values are provided by the INITIALIZE parameter.

  (* ASYNC_REG = "TRUE" *) reg i_in_meta  = INITIALIZE[0];
  (* ASYNC_REG = "TRUE" *) reg i_in_sync1 = INITIALIZE[1];
  (* ASYNC_REG = "TRUE" *) reg i_in_sync2 = INITIALIZE[2];
  (* ASYNC_REG = "TRUE" *) reg i_in_sync3 = INITIALIZE[3];
                           reg i_in_out   = INITIALIZE[4];

  always @(posedge clk_in) begin
    i_in_meta  <= i_in;
    i_in_sync1 <= i_in_meta;
    i_in_sync2 <= i_in_sync1;
    i_in_sync3 <= i_in_sync2;
    i_in_out   <= i_in_sync3;
  end

  assign o_out = i_in_out;


endmodule

module gtwizard_ultrascale_v1_6_2_gthe3_cpll_cal_freq_counter # (
  parameter REVISION = 1
)(
  output reg  [17:0] freq_cnt_o = 18'd0,
  output reg  done_o,
  input  wire rst_i,
  input  wire [15:0] test_term_cnt_i,
  input  wire ref_clk_i,
  input  wire test_clk_i
);
  //****************************************************************************
  //  Local Parameters
  //****************************************************************************
  localparam RESET_STATE = 0;
  localparam MEASURE_STATE = 1;
  localparam HOLD_STATE = 2;
  localparam UPDATE_STATE = 3;
  localparam DONE_STATE = 4;

  //****************************************************************************
  //  Local Signals
  //****************************************************************************
  reg [17:0] testclk_cnt = 18'h00000;
  reg [15:0] refclk_cnt = 16'h0000;
  reg [3:0] testclk_div4 = 4'h1;
  wire testclk_rst;
  wire testclk_en;
  reg [5:0] hold_clk = 6'd0;
  reg [4:0] state = 5'd1;

  (* ASYNC_REG = "TRUE" *) reg tstclk_rst_dly1, tstclk_rst_dly2;
  (* ASYNC_REG = "TRUE" *) reg testclk_en_dly1, testclk_en_dly2;

  //
  // need to get testclk_rst into TESTCLK_I domain
  //
  always @(posedge test_clk_i)
  begin
    tstclk_rst_dly1 <= testclk_rst;
    tstclk_rst_dly2 <= tstclk_rst_dly1;
  end

  //
  // need to get testclk_en into TESTCLK_I domain
  //
  always @(posedge test_clk_i)
  begin
    testclk_en_dly1 <= testclk_en;
    testclk_en_dly2 <= testclk_en_dly1;
  end

  always @(posedge test_clk_i)
  begin
    if (tstclk_rst_dly2 == 1'b1)
    begin
      testclk_div4 <= 4'h1;
    end
    else
    begin
      testclk_div4 <= {testclk_div4[2:0], testclk_div4[3]};
    end
  end

  wire testclk_rst_sync;

  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_testclk_rst_inst (
    .clk_in (test_clk_i),
    .rst_in   (testclk_rst),
    .rst_out  (testclk_rst_sync)
  );

  always @(posedge test_clk_i or posedge testclk_rst_sync)
  begin
    if (testclk_rst_sync == 1'b1)
    begin
      testclk_cnt <= 0;
    end
    else if (testclk_en_dly2 == 1'b1 && testclk_div4 == 4'h8)
    begin
      testclk_cnt <= testclk_cnt + 1;
    end
  end

/*   always @(posedge test_clk_i or posedge testclk_rst)
  begin
    if (testclk_rst == 1'b1)
    begin
      testclk_cnt <= 0;
    end
    else if (testclk_en_dly2 == 1'b1 && testclk_div4 == 4'h8)
    begin
      testclk_cnt <= testclk_cnt + 1;
    end
  end */

  always @(posedge ref_clk_i or posedge rst_i)
  begin
    if (rst_i)
      done_o <= 1'b0;
    else
      done_o <= state[DONE_STATE];
  end

  always @(posedge ref_clk_i or posedge rst_i)
  begin
    if (rst_i) begin
      state <= 0;
      state[RESET_STATE] <= 1'b1;
    end
    else begin
      state <= 0;
      case (1'b1) // synthesis parallel_case full_case
        state[RESET_STATE]:
          begin
            if (hold_clk == 6'h3F)
              state[MEASURE_STATE] <= 1'b1;
            else
              state[RESET_STATE] <= 1'b1;
          end
        state[MEASURE_STATE]:
          begin
            if (refclk_cnt == test_term_cnt_i)
              state[HOLD_STATE] <= 1'b1;
            else
              state[MEASURE_STATE] <= 1'b1;
          end
        state[HOLD_STATE]:
          begin
            if (hold_clk == 6'hF)
              state[UPDATE_STATE] <= 1'b1;
            else
              state[HOLD_STATE] <= 1'b1;
          end
        state[UPDATE_STATE]:
          begin
            freq_cnt_o <= testclk_cnt;
            state[DONE_STATE] <= 1'b1;
          end
        state[DONE_STATE]:
          begin
            state[DONE_STATE] <= 1'b1;
          end
      endcase
    end
  end

  assign testclk_rst = state[RESET_STATE];
  assign testclk_en = state[MEASURE_STATE];


  always @(posedge ref_clk_i)
  begin
    if (state[RESET_STATE] == 1'b1 || state[HOLD_STATE] == 1'b1)
      hold_clk <= hold_clk + 1;
    else
      hold_clk <= 0;
  end

  always @(posedge ref_clk_i)
  begin
    if (state[MEASURE_STATE] == 1'b1)
      refclk_cnt <= refclk_cnt + 1;
    else
      refclk_cnt <= 0;
  end




endmodule

module gtwizard_ultrascale_v1_6_2_gtwiz_userclk_tx #(

  parameter integer P_CONTENTS                     = 0,
  parameter integer P_FREQ_RATIO_SOURCE_TO_USRCLK  = 1,
  parameter integer P_FREQ_RATIO_USRCLK_TO_USRCLK2 = 1

)(

  input  wire gtwiz_userclk_tx_srcclk_in,
  input  wire gtwiz_userclk_tx_reset_in,
  output wire gtwiz_userclk_tx_usrclk_out,
  output wire gtwiz_userclk_tx_usrclk2_out,
  output wire gtwiz_userclk_tx_active_out

);


  // -------------------------------------------------------------------------------------------------------------------
  // Local parameters
  // -------------------------------------------------------------------------------------------------------------------

  // Convert integer parameters with known, limited legal range to a 3-bit local parameter values
  localparam integer P_USRCLK_INT_DIV  = P_FREQ_RATIO_SOURCE_TO_USRCLK - 1;
  localparam   [2:0] P_USRCLK_DIV      = P_USRCLK_INT_DIV[2:0];
  localparam integer P_USRCLK2_INT_DIV = (P_FREQ_RATIO_SOURCE_TO_USRCLK * P_FREQ_RATIO_USRCLK_TO_USRCLK2) - 1;
  localparam   [2:0] P_USRCLK2_DIV     = P_USRCLK2_INT_DIV[2:0];


  // -------------------------------------------------------------------------------------------------------------------
  // Transmitter user clocking network conditional generation, based on parameter values in module instantiation
  // -------------------------------------------------------------------------------------------------------------------
  generate if (1) begin: gen_gtwiz_userclk_tx_main

    // Use BUFG_GT instance(s) to drive TXUSRCLK and TXUSRCLK2, inferred for integral source to TXUSRCLK frequency ratio
    if (P_CONTENTS == 0) begin

      // Drive TXUSRCLK with BUFG_GT-buffered source clock, dividing the input by the integral source clock to TXUSRCLK
      // frequency ratio
      BUFG_GT bufg_gt_usrclk_inst (
        .CE      (1'b1),
        .CEMASK  (1'b0),
        .CLR     (gtwiz_userclk_tx_reset_in),
        .CLRMASK (1'b0),
        .DIV     (P_USRCLK_DIV),
        .I       (gtwiz_userclk_tx_srcclk_in),
        .O       (gtwiz_userclk_tx_usrclk_out)
      );

      // If TXUSRCLK and TXUSRCLK2 frequencies are identical, drive both from the same BUFG_GT. Otherwise, drive
      // TXUSRCLK2 from a second BUFG_GT instance, dividing the source clock down to the TXUSRCLK2 frequency.
      if (P_FREQ_RATIO_USRCLK_TO_USRCLK2 == 1)
        assign gtwiz_userclk_tx_usrclk2_out = gtwiz_userclk_tx_usrclk_out;
      else begin
        BUFG_GT bufg_gt_usrclk2_inst (
          .CE      (1'b1),
          .CEMASK  (1'b0),
          .CLR     (gtwiz_userclk_tx_reset_in),
          .CLRMASK (1'b0),
          .DIV     (P_USRCLK2_DIV),
          .I       (gtwiz_userclk_tx_srcclk_in),
          .O       (gtwiz_userclk_tx_usrclk2_out)
        );
      end

      // Indicate active helper block functionality when the BUFG_GT divider is not held in reset
      (* ASYNC_REG = "TRUE" *) reg gtwiz_userclk_tx_active_meta = 1'b0;
      (* ASYNC_REG = "TRUE" *) reg gtwiz_userclk_tx_active_sync = 1'b0;
      always @(posedge gtwiz_userclk_tx_usrclk2_out, posedge gtwiz_userclk_tx_reset_in) begin
        if (gtwiz_userclk_tx_reset_in) begin
          gtwiz_userclk_tx_active_meta <= 1'b0;
          gtwiz_userclk_tx_active_sync <= 1'b0;
        end
        else begin
          gtwiz_userclk_tx_active_meta <= 1'b1;
          gtwiz_userclk_tx_active_sync <= gtwiz_userclk_tx_active_meta;
        end
      end
      assign gtwiz_userclk_tx_active_out = gtwiz_userclk_tx_active_sync;

    end

  end
  endgenerate


endmodule

module gtwizard_ultrascale_v1_6_2_gtwiz_userclk_rx #(

  parameter integer P_CONTENTS                     = 0,
  parameter integer P_FREQ_RATIO_SOURCE_TO_USRCLK  = 1,
  parameter integer P_FREQ_RATIO_USRCLK_TO_USRCLK2 = 1

)(

  input  wire gtwiz_userclk_rx_srcclk_in,
  input  wire gtwiz_userclk_rx_reset_in,
  output wire gtwiz_userclk_rx_usrclk_out,
  output wire gtwiz_userclk_rx_usrclk2_out,
  output wire gtwiz_userclk_rx_active_out

);


  // -------------------------------------------------------------------------------------------------------------------
  // Local parameters
  // -------------------------------------------------------------------------------------------------------------------

  // Convert integer parameters with known, limited legal range to a 3-bit local parameter values
  localparam integer P_USRCLK_INT_DIV  = P_FREQ_RATIO_SOURCE_TO_USRCLK - 1;
  localparam   [2:0] P_USRCLK_DIV      = P_USRCLK_INT_DIV[2:0];
  localparam integer P_USRCLK2_INT_DIV = (P_FREQ_RATIO_SOURCE_TO_USRCLK * P_FREQ_RATIO_USRCLK_TO_USRCLK2) - 1;
  localparam   [2:0] P_USRCLK2_DIV     = P_USRCLK2_INT_DIV[2:0];


  // -------------------------------------------------------------------------------------------------------------------
  // Receiver user clocking network conditional generation, based on parameter values in module instantiation
  // -------------------------------------------------------------------------------------------------------------------
  generate if (1) begin: gen_gtwiz_userclk_rx_main

    // Use BUFG_GT instance(s) to drive RXUSRCLK and RXUSRCLK2, inferred for integral source to RXUSRCLK frequency ratio
    if (P_CONTENTS == 0) begin

      // Drive RXUSRCLK with BUFG_GT-buffered source clock, dividing the input by the integral source clock to RXUSRCLK
      // frequency ratio
      BUFG_GT bufg_gt_usrclk_inst (
        .CE      (1'b1),
        .CEMASK  (1'b0),
        .CLR     (gtwiz_userclk_rx_reset_in),
        .CLRMASK (1'b0),
        .DIV     (P_USRCLK_DIV),
        .I       (gtwiz_userclk_rx_srcclk_in),
        .O       (gtwiz_userclk_rx_usrclk_out)
      );

      // If RXUSRCLK and RXUSRCLK2 frequencies are identical, drive both from the same BUFG_GT. Otherwise, drive
      // RXUSRCLK2 from a second BUFG_GT instance, dividing the source clock down to the RXUSRCLK2 frequency.
      if (P_FREQ_RATIO_USRCLK_TO_USRCLK2 == 1)
        assign gtwiz_userclk_rx_usrclk2_out = gtwiz_userclk_rx_usrclk_out;
      else begin
        BUFG_GT bufg_gt_usrclk2_inst (
          .CE      (1'b1),
          .CEMASK  (1'b0),
          .CLR     (gtwiz_userclk_rx_reset_in),
          .CLRMASK (1'b0),
          .DIV     (P_USRCLK2_DIV),
          .I       (gtwiz_userclk_rx_srcclk_in),
          .O       (gtwiz_userclk_rx_usrclk2_out)
        );
      end

      // Indicate active helper block functionality when the BUFG_GT divider is not held in reset
      (* ASYNC_REG = "TRUE" *) reg gtwiz_userclk_rx_active_meta = 1'b0;
      (* ASYNC_REG = "TRUE" *) reg gtwiz_userclk_rx_active_sync = 1'b0;
/*
      always @(posedge gtwiz_userclk_rx_usrclk2_out, posedge gtwiz_userclk_rx_reset_in) begin
        if (gtwiz_userclk_rx_reset_in) begin
          gtwiz_userclk_rx_active_meta <= 1'b0;
          gtwiz_userclk_rx_active_sync <= 1'b0;
        end
        else begin
          gtwiz_userclk_rx_active_meta <= 1'b1;
          gtwiz_userclk_rx_active_sync <= gtwiz_userclk_rx_active_meta;
        end
      end
      assign gtwiz_userclk_rx_active_out = gtwiz_userclk_rx_active_sync;
*/
      always @(posedge gtwiz_userclk_rx_usrclk2_out) begin
        if (gtwiz_userclk_rx_reset_in) begin
          gtwiz_userclk_rx_active_meta <= 1'b0;
          gtwiz_userclk_rx_active_sync <= 1'b0;
        end
        else begin
          gtwiz_userclk_rx_active_meta <= 1'b1;
          gtwiz_userclk_rx_active_sync <= gtwiz_userclk_rx_active_meta;
        end
      end
      assign gtwiz_userclk_rx_active_out = gtwiz_userclk_rx_active_sync & ~gtwiz_userclk_rx_reset_in;

    end

  end
  endgenerate


endmodule

module gtwizard_ultrascale_v1_6_2_gtwiz_buffbypass_tx #(

  parameter integer P_BUFFER_BYPASS_MODE       = 0,
  parameter integer P_TOTAL_NUMBER_OF_CHANNELS = 1,
  parameter integer P_MASTER_CHANNEL_POINTER   = 0

)(

  // User interface ports
  input  wire gtwiz_buffbypass_tx_clk_in,
  input  wire gtwiz_buffbypass_tx_reset_in,
  input  wire gtwiz_buffbypass_tx_start_user_in,
  input  wire gtwiz_buffbypass_tx_resetdone_in,
  output reg  gtwiz_buffbypass_tx_done_out  = 1'b0,
  output reg  gtwiz_buffbypass_tx_error_out = 1'b0,

  // Transceiver interface ports
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txphaligndone_in,
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txphinitdone_in,
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txdlysresetdone_in,
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txsyncout_in,
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txsyncdone_in,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txphdlyreset_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txphalign_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txphalignen_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txphdlypd_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txphinit_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txphovrden_out,
  output reg  [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txdlysreset_out = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}},
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txdlybypass_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txdlyen_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txdlyovrden_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txphdlytstclk_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txdlyhold_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txdlyupdown_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txsyncmode_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txsyncallin_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] txsyncin_out

);


  // -------------------------------------------------------------------------------------------------------------------
  // Transmitter buffer bypass conditional generation, based on parameter values in module instantiation
  // -------------------------------------------------------------------------------------------------------------------
  localparam [1:0] ST_BUFFBYPASS_TX_IDLE                 = 2'd0;
  localparam [1:0] ST_BUFFBYPASS_TX_DEASSERT_TXDLYSRESET = 2'd1;
  localparam [1:0] ST_BUFFBYPASS_TX_WAIT_TXSYNCDONE      = 2'd2;
  localparam [1:0] ST_BUFFBYPASS_TX_DONE                 = 2'd3;

  generate if (1) begin: gen_gtwiz_buffbypass_tx_main

    // Use auto mode buffer bypass
    if (P_BUFFER_BYPASS_MODE == 0) begin : gen_auto_mode

      // For single-lane auto mode buffer bypass, perform specified input port tie-offs
      if (P_TOTAL_NUMBER_OF_CHANNELS == 1) begin : gen_assign_one_chan
        assign txphdlyreset_out  = 1'b0;
        assign txphalign_out     = 1'b0;
        assign txphalignen_out   = 1'b0;
        assign txphdlypd_out     = 1'b0;
        assign txphinit_out      = 1'b0;
        assign txphovrden_out    = 1'b0;
        assign txdlybypass_out   = 1'b0;
        assign txdlyen_out       = 1'b0;
        assign txdlyovrden_out   = 1'b0;
        assign txphdlytstclk_out = 1'b0;
        assign txdlyhold_out     = 1'b0;
        assign txdlyupdown_out   = 1'b0;
        assign txsyncmode_out    = 1'b1;
        assign txsyncallin_out   = txphaligndone_in;
        assign txsyncin_out      = 1'b0;
      end

      // For multi-lane auto mode buffer bypass, perform specified master and slave lane input port tie-offs
      else begin : gen_assign_multi_chan
        assign txphdlyreset_out  = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txphalign_out     = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txphalignen_out   = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txphdlypd_out     = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txphinit_out      = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txphovrden_out    = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txdlybypass_out   = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txdlyen_out       = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txdlyovrden_out   = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txphdlytstclk_out = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txdlyhold_out     = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign txdlyupdown_out   = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};

        genvar gi;
        for (gi = 0; gi < P_TOTAL_NUMBER_OF_CHANNELS; gi = gi + 1) begin : gen_assign_txsyncmode
          if (gi == P_MASTER_CHANNEL_POINTER)
            assign txsyncmode_out[gi] = 1'b1;
          else
            assign txsyncmode_out[gi] = 1'b0;
        end

        assign txsyncallin_out = {P_TOTAL_NUMBER_OF_CHANNELS{&txphaligndone_in}};
        assign txsyncin_out    = {P_TOTAL_NUMBER_OF_CHANNELS{txsyncout_in[P_MASTER_CHANNEL_POINTER]}};
      end

      // Detect the rising edge of the transmitter reset done re-synchronized input. Assign an internal buffer bypass
      // start signal to the OR of this reset done indicator, and the synchronous buffer bypass procedure user request.
      wire gtwiz_buffbypass_tx_resetdone_sync_int;

      gtwizard_ultrascale_v1_6_2_reset_inv_synchronizer reset_synchronizer_resetdone_inst (
        .clk_in  (gtwiz_buffbypass_tx_clk_in),
        .rst_in  (gtwiz_buffbypass_tx_resetdone_in),
        .rst_out (gtwiz_buffbypass_tx_resetdone_sync_int)
      );

      reg  gtwiz_buffbypass_tx_resetdone_reg = 1'b0;
      wire gtwiz_buffbypass_tx_start_int;

      always @(posedge gtwiz_buffbypass_tx_clk_in) begin
        if (gtwiz_buffbypass_tx_reset_in)
          gtwiz_buffbypass_tx_resetdone_reg <= 1'b0;
        else
          gtwiz_buffbypass_tx_resetdone_reg <= gtwiz_buffbypass_tx_resetdone_sync_int;
      end

      assign gtwiz_buffbypass_tx_start_int = (gtwiz_buffbypass_tx_resetdone_sync_int &&
                                             ~gtwiz_buffbypass_tx_resetdone_reg) || gtwiz_buffbypass_tx_start_user_in;

      // Synchronize the master channel's buffer bypass completion output (TXSYNCDONE) into the local clock domain
      // and detect its rising edge for purposes of safe state machine transitions
      reg  gtwiz_buffbypass_tx_master_syncdone_sync_reg = 1'b0;
      wire gtwiz_buffbypass_tx_master_syncdone_sync_int;
      wire gtwiz_buffbypass_tx_master_syncdone_sync_re;

      gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_master_syncdone_inst (
        .clk_in (gtwiz_buffbypass_tx_clk_in),
        .i_in   (txsyncdone_in[P_MASTER_CHANNEL_POINTER]),
        .o_out  (gtwiz_buffbypass_tx_master_syncdone_sync_int)
      );

      always @(posedge gtwiz_buffbypass_tx_clk_in)
        gtwiz_buffbypass_tx_master_syncdone_sync_reg <= gtwiz_buffbypass_tx_master_syncdone_sync_int;

      assign gtwiz_buffbypass_tx_master_syncdone_sync_re = gtwiz_buffbypass_tx_master_syncdone_sync_int &&
                                                          ~gtwiz_buffbypass_tx_master_syncdone_sync_reg;

      // Synchronize the master channel's phase alignment completion output (TXPHALIGNDONE) into the local clock domain
      wire gtwiz_buffbypass_tx_master_phaligndone_sync_int;

      gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_master_phaligndone_inst (
        .clk_in (gtwiz_buffbypass_tx_clk_in),
        .i_in   (txphaligndone_in[P_MASTER_CHANNEL_POINTER]),
        .o_out  (gtwiz_buffbypass_tx_master_phaligndone_sync_int)
      );

      // Implement a simple state machine to perform the transmitter auto mode buffer bypass procedure
      reg [1:0] sm_buffbypass_tx = ST_BUFFBYPASS_TX_IDLE;

      always @(posedge gtwiz_buffbypass_tx_clk_in) begin
        if (gtwiz_buffbypass_tx_reset_in) begin
          gtwiz_buffbypass_tx_done_out  <= 1'b0;
          gtwiz_buffbypass_tx_error_out <= 1'b0;
          txdlysreset_out               <= {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
          sm_buffbypass_tx              <= ST_BUFFBYPASS_TX_IDLE;
        end
        else begin
          case (sm_buffbypass_tx)

            // Upon assertion of the internal buffer bypass start signal, assert TXDLYSRESET output(s)
            default: begin
              if (gtwiz_buffbypass_tx_start_int) begin
                gtwiz_buffbypass_tx_done_out  <= 1'b0;
                gtwiz_buffbypass_tx_error_out <= 1'b0;
                txdlysreset_out               <= {P_TOTAL_NUMBER_OF_CHANNELS{1'b1}};
                sm_buffbypass_tx              <= ST_BUFFBYPASS_TX_DEASSERT_TXDLYSRESET;
              end
            end

            // De-assert the TXDLYSRESET output(s)
            ST_BUFFBYPASS_TX_DEASSERT_TXDLYSRESET: begin
              txdlysreset_out  <= {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
              sm_buffbypass_tx <= ST_BUFFBYPASS_TX_WAIT_TXSYNCDONE;
            end

            // Upon assertion of the synchronized TXSYNCDONE indicator, transition to the final state
            ST_BUFFBYPASS_TX_WAIT_TXSYNCDONE: begin
              if (gtwiz_buffbypass_tx_master_syncdone_sync_re)
                sm_buffbypass_tx <= ST_BUFFBYPASS_TX_DONE;
            end

            // Assert the buffer bypass procedure done user indicator, and set the procedure error flag if the
            // synchronized TXPHALIGNDONE indicator is not high
            ST_BUFFBYPASS_TX_DONE: begin
              gtwiz_buffbypass_tx_done_out  <= 1'b1;
              gtwiz_buffbypass_tx_error_out <= ~gtwiz_buffbypass_tx_master_phaligndone_sync_int;
              sm_buffbypass_tx              <= ST_BUFFBYPASS_TX_IDLE;
            end

          endcase
        end
      end

    end
  end
  endgenerate


endmodule

module gtwizard_ultrascale_v1_6_2_reset_inv_synchronizer # (

  parameter FREQUENCY = 512

)(

  input  wire clk_in,
  input  wire rst_in,
  output wire rst_out

);

  // Use 5 flip-flops as a single synchronizer, and tag each declaration with the appropriate synthesis attribute to
  // enable clustering. Each flip-flop in the synchronizer is asynchronously reset so that the downstream logic is also
  // asynchronously reset but encounters no reset assertion latency. The removal of reset is synchronous, so that the
  // downstream logic is also removed from reset synchronously. This module is designed for active-low reset use.

  (* ASYNC_REG = "TRUE" *) reg rst_in_meta  = 1'b0;
  (* ASYNC_REG = "TRUE" *) reg rst_in_sync1 = 1'b0;
  (* ASYNC_REG = "TRUE" *) reg rst_in_sync2 = 1'b0;
  (* ASYNC_REG = "TRUE" *) reg rst_in_sync3 = 1'b0;
                           reg rst_in_out   = 1'b0;

  always @(posedge clk_in, negedge rst_in) begin
    if (!rst_in) begin
      rst_in_meta  <= 1'b0;
      rst_in_sync1 <= 1'b0;
      rst_in_sync2 <= 1'b0;
      rst_in_sync3 <= 1'b0;
      rst_in_out   <= 1'b0;
    end
    else begin
      rst_in_meta  <= 1'b1;
      rst_in_sync1 <= rst_in_meta;
      rst_in_sync2 <= rst_in_sync1;
      rst_in_sync3 <= rst_in_sync2;
      rst_in_out   <= rst_in_sync3;
    end
  end

  assign rst_out = rst_in_out;


endmodule

module gtwizard_ultrascale_v1_6_2_gtwiz_buffbypass_rx #(

  parameter integer P_BUFFER_BYPASS_MODE       = 0,
  parameter integer P_TOTAL_NUMBER_OF_CHANNELS = 1,
  parameter integer P_MASTER_CHANNEL_POINTER   = 0

)(

  // User interface ports
  input  wire gtwiz_buffbypass_rx_clk_in,
  input  wire gtwiz_buffbypass_rx_reset_in,
  input  wire gtwiz_buffbypass_rx_start_user_in,
  input  wire gtwiz_buffbypass_rx_resetdone_in,
  output reg  gtwiz_buffbypass_rx_done_out  = 1'b0,
  output reg  gtwiz_buffbypass_rx_error_out = 1'b0,

  // Transceiver interface ports
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxphaligndone_in,
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxdlysresetdone_in,
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxsyncout_in,
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxsyncdone_in,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxphdlyreset_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxphalign_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxphalignen_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxphdlypd_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxphovrden_out,
  output reg  [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxdlysreset_out = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}},
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxdlybypass_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxdlyen_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxdlyovrden_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxsyncmode_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxsyncallin_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS-1):0] rxsyncin_out

);


  // -------------------------------------------------------------------------------------------------------------------
  // Receiver buffer bypass conditional generation, based on parameter values in module instantiation
  // -------------------------------------------------------------------------------------------------------------------
  localparam [1:0] ST_BUFFBYPASS_RX_IDLE                 = 2'd0;
  localparam [1:0] ST_BUFFBYPASS_RX_DEASSERT_RXDLYSRESET = 2'd1;
  localparam [1:0] ST_BUFFBYPASS_RX_WAIT_RXSYNCDONE      = 2'd2;
  localparam [1:0] ST_BUFFBYPASS_RX_DONE                 = 2'd3;

  generate if (1) begin: gen_gtwiz_buffbypass_rx_main

    // Use auto mode buffer bypass
    if (P_BUFFER_BYPASS_MODE == 0) begin : gen_auto_mode

      // For single-lane auto mode buffer bypass, perform specified input port tie-offs
      if (P_TOTAL_NUMBER_OF_CHANNELS == 1) begin : gen_assign_one_chan
        assign rxphdlyreset_out = 1'b0;
        assign rxphalign_out    = 1'b0;
        assign rxphalignen_out  = 1'b0;
        assign rxphdlypd_out    = 1'b0;
        assign rxphovrden_out   = 1'b0;
        assign rxdlybypass_out  = 1'b0;
        assign rxdlyen_out      = 1'b0;
        assign rxdlyovrden_out  = 1'b0;
        assign rxsyncmode_out   = 1'b1;
        assign rxsyncallin_out  = rxphaligndone_in;
        assign rxsyncin_out     = 1'b0;
      end

      // For multi-lane auto mode buffer bypass, perform specified master and slave lane input port tie-offs
      else begin : gen_assign_multi_chan
        assign rxphdlyreset_out = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign rxphalign_out    = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign rxphalignen_out  = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign rxphdlypd_out    = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign rxphovrden_out   = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign rxdlybypass_out  = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign rxdlyen_out      = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
        assign rxdlyovrden_out  = {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};

        genvar gi;
        for (gi = 0; gi < P_TOTAL_NUMBER_OF_CHANNELS; gi = gi + 1) begin : gen_assign_rxsyncmode
          if (gi == P_MASTER_CHANNEL_POINTER)
            assign rxsyncmode_out[gi] = 1'b1;
          else
            assign rxsyncmode_out[gi] = 1'b0;
        end

        assign rxsyncallin_out = {P_TOTAL_NUMBER_OF_CHANNELS{&rxphaligndone_in}};
        assign rxsyncin_out    = {P_TOTAL_NUMBER_OF_CHANNELS{rxsyncout_in[P_MASTER_CHANNEL_POINTER]}};
      end

      // Detect the rising edge of the receiver reset done re-synchronized input. Assign an internal buffer bypass
      // start signal to the OR of this reset done indicator, and the synchronous buffer bypass procedure user request.
      wire gtwiz_buffbypass_rx_resetdone_sync_int;

      gtwizard_ultrascale_v1_6_2_reset_inv_synchronizer reset_synchronizer_resetdone_inst (
        .clk_in  (gtwiz_buffbypass_rx_clk_in),
        .rst_in  (gtwiz_buffbypass_rx_resetdone_in),
        .rst_out (gtwiz_buffbypass_rx_resetdone_sync_int)
      );

      reg  gtwiz_buffbypass_rx_resetdone_reg = 1'b0;
      wire gtwiz_buffbypass_rx_start_int;

      always @(posedge gtwiz_buffbypass_rx_clk_in) begin
        if (gtwiz_buffbypass_rx_reset_in)
          gtwiz_buffbypass_rx_resetdone_reg <= 1'b0;
        else
          gtwiz_buffbypass_rx_resetdone_reg <= gtwiz_buffbypass_rx_resetdone_sync_int;
      end

      assign gtwiz_buffbypass_rx_start_int = (gtwiz_buffbypass_rx_resetdone_sync_int &&
                                             ~gtwiz_buffbypass_rx_resetdone_reg) || gtwiz_buffbypass_rx_start_user_in;

      // Synchronize the master channel's buffer bypass completion output (RXSYNCDONE) into the local clock domain
      // and detect its rising edge for purposes of safe state machine transitions
      reg  gtwiz_buffbypass_rx_master_syncdone_sync_reg = 1'b0;
      wire gtwiz_buffbypass_rx_master_syncdone_sync_int;
      wire gtwiz_buffbypass_rx_master_syncdone_sync_re;

      gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_mastersyncdone_inst (
        .clk_in (gtwiz_buffbypass_rx_clk_in),
        .i_in   (rxsyncdone_in[P_MASTER_CHANNEL_POINTER]),
        .o_out  (gtwiz_buffbypass_rx_master_syncdone_sync_int)
      );

      always @(posedge gtwiz_buffbypass_rx_clk_in)
        gtwiz_buffbypass_rx_master_syncdone_sync_reg <= gtwiz_buffbypass_rx_master_syncdone_sync_int;

      assign gtwiz_buffbypass_rx_master_syncdone_sync_re = gtwiz_buffbypass_rx_master_syncdone_sync_int &&
                                                          ~gtwiz_buffbypass_rx_master_syncdone_sync_reg;

      // Synchronize the master channel's phase alignment completion output (RXPHALIGNDONE) into the local clock domain
      wire gtwiz_buffbypass_rx_master_phaligndone_sync_int;

      gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_masterphaligndone_inst (
        .clk_in (gtwiz_buffbypass_rx_clk_in),
        .i_in   (rxphaligndone_in[P_MASTER_CHANNEL_POINTER]),
        .o_out  (gtwiz_buffbypass_rx_master_phaligndone_sync_int)
      );

      // Implement a simple state machine to perform the receiver auto mode buffer bypass procedure
      reg [1:0] sm_buffbypass_rx = ST_BUFFBYPASS_RX_IDLE;

      always @(posedge gtwiz_buffbypass_rx_clk_in) begin
        if (gtwiz_buffbypass_rx_reset_in) begin
          gtwiz_buffbypass_rx_done_out  <= 1'b0;
          gtwiz_buffbypass_rx_error_out <= 1'b0;
          rxdlysreset_out               <= {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
          sm_buffbypass_rx              <= ST_BUFFBYPASS_RX_IDLE;
        end
        else begin
          case (sm_buffbypass_rx)

            // Upon assertion of the internal buffer bypass start signal, assert RXDLYSRESET output(s)
            default: begin
              if (gtwiz_buffbypass_rx_start_int) begin
                gtwiz_buffbypass_rx_done_out  <= 1'b0;
                gtwiz_buffbypass_rx_error_out <= 1'b0;
                rxdlysreset_out               <= {P_TOTAL_NUMBER_OF_CHANNELS{1'b1}};
                sm_buffbypass_rx              <= ST_BUFFBYPASS_RX_DEASSERT_RXDLYSRESET;
              end
            end

            // De-assert the RXDLYSRESET output(s)
            ST_BUFFBYPASS_RX_DEASSERT_RXDLYSRESET: begin
              rxdlysreset_out  <= {P_TOTAL_NUMBER_OF_CHANNELS{1'b0}};
              sm_buffbypass_rx <= ST_BUFFBYPASS_RX_WAIT_RXSYNCDONE;
            end

            // Upon assertion of the synchronized RXSYNCDONE indicator, transition to the final state
            ST_BUFFBYPASS_RX_WAIT_RXSYNCDONE: begin
              if (gtwiz_buffbypass_rx_master_syncdone_sync_re)
                sm_buffbypass_rx <= ST_BUFFBYPASS_RX_DONE;
            end

            // Assert the buffer bypass procedure done user indicator, and set the procedure error flag if the
            // synchronized RXPHALIGNDONE indicator is not high
            ST_BUFFBYPASS_RX_DONE: begin
              gtwiz_buffbypass_rx_done_out  <= 1'b1;
              gtwiz_buffbypass_rx_error_out <= ~gtwiz_buffbypass_rx_master_phaligndone_sync_int;
              sm_buffbypass_rx              <= ST_BUFFBYPASS_RX_IDLE;
            end

          endcase
        end
      end

    end
  end
  endgenerate


endmodule

module gtwizard_ultrascale_v1_6_2_gtwiz_reset # (

  parameter real    P_FREERUN_FREQUENCY       = 200,
  parameter integer P_USE_CPLL_CAL            = 0,
  parameter integer P_TX_PLL_TYPE             = 0,
  parameter integer P_RX_PLL_TYPE             = 0,
  parameter real    P_RX_LINE_RATE            = 10.3125,
  parameter [25:0]  P_CDR_TIMEOUT_FREERUN_CYC = (37000 * P_FREERUN_FREQUENCY) / P_RX_LINE_RATE

)(

  // User interface ports
  input  wire gtwiz_reset_clk_freerun_in,
  input  wire gtwiz_reset_all_in,
  input  wire gtwiz_reset_tx_pll_and_datapath_in,
  input  wire gtwiz_reset_tx_datapath_in,
  input  wire gtwiz_reset_rx_pll_and_datapath_in,
  input  wire gtwiz_reset_rx_datapath_in,
  output wire gtwiz_reset_rx_cdr_stable_out,
  output wire gtwiz_reset_tx_done_out,
  output wire gtwiz_reset_rx_done_out,
  input  wire gtwiz_reset_userclk_tx_active_in,
  input  wire gtwiz_reset_userclk_rx_active_in,

  // Transceiver interface ports
  input  wire gtpowergood_in,
  input  wire txusrclk2_in,
  input  wire plllock_tx_in,
  input  wire txresetdone_in,
  input  wire rxusrclk2_in,
  input  wire plllock_rx_in,
  input  wire rxcdrlock_in,
  input  wire rxresetdone_in,
  output reg  pllreset_tx_out    = 1'b1,
  output wire txprogdivreset_out,
  output reg  gttxreset_out      = 1'b1,
  output reg  txuserrdy_out      = 1'b0,
  output reg  pllreset_rx_out,
  output reg  rxprogdivreset_out = 1'b1,
  output reg  gtrxreset_out      = 1'b1,
  output reg  rxuserrdy_out      = 1'b0,

  // Tie-offs based on core configuration
  input  wire tx_enabled_tie_in,
  input  wire rx_enabled_tie_in,
  input  wire shared_pll_tie_in

);


  // -------------------------------------------------------------------------------------------------------------------
  // "Reset all" state machine
  // -------------------------------------------------------------------------------------------------------------------

  // The "reset all" state machine responds to the synchronized gtwiz_reset_all_in input by resetting the enabled PLLs
  // and data paths of those transceiver resources to which the reset helper block is connected. It does so by guiding
  // the independent transmitter and receiver reset state machines, which are also user-accessible. The path through the
  // "reset all" state machine is a function of module input tie-offs, which depend on the core configuration.

  // Synchronize the "reset all" input signal into the free-running clock domain
  wire gtwiz_reset_all_sync;
  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_gtwiz_reset_all_inst (
    .clk_in  (gtwiz_reset_clk_freerun_in),
    .rst_in  (gtwiz_reset_all_in),
    .rst_out (gtwiz_reset_all_sync)
  );

  // Synchronize the transceiver power good indicator
  wire gtpowergood_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_gtpowergood_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (gtpowergood_in),
    .o_out  (gtpowergood_sync)
  );

  // Declare the "reset all" state machine reset timer registers
  reg       sm_reset_all_timer_clr = 1'b1;
  reg [2:0] sm_reset_all_timer_ctr = 3'd0;
  reg       sm_reset_all_timer_sat = 1'b0;

  // Declare local parameters used to represent both static and variable state machine state values
  localparam [2:0] ST_RESET_ALL_INIT        = 3'd0;
  localparam [2:0] ST_RESET_ALL_BRANCH      = 3'd1;
  localparam [2:0] ST_RESET_ALL_TX_PLL      = 3'd2;
  localparam [2:0] ST_RESET_ALL_TX_PLL_WAIT = 3'd3;
  localparam [2:0] ST_RESET_ALL_RX_DP       = 3'd4;
  localparam [2:0] ST_RESET_ALL_RX_PLL      = 3'd5;
  localparam [2:0] ST_RESET_ALL_RX_WAIT     = 3'd6;
  localparam [2:0] ST_RESET_ALL_DONE        = 3'd7;
  reg        [2:0] sm_reset_all             = ST_RESET_ALL_INIT;

  // Declare relevant internal control and status registers of this and other state machines
  reg gtwiz_reset_tx_pll_and_datapath_int = 1'b0;
  reg gtwiz_reset_tx_done_int             = 1'b0;
  reg gtwiz_reset_rx_pll_and_datapath_int = 1'b0;
  reg gtwiz_reset_rx_datapath_int         = 1'b0;
  reg gtwiz_reset_rx_done_int             = 1'b0;

  // Implement the "reset all" state machine control and its outputs as a single sequential process. The state machine
  // is reset by the synchronized gtwiz_reset_all_sync input.
  always @(posedge gtwiz_reset_clk_freerun_in) begin
    if (gtwiz_reset_all_sync) begin
      gtwiz_reset_tx_pll_and_datapath_int <= 1'b0;
      gtwiz_reset_rx_pll_and_datapath_int <= 1'b0;
      gtwiz_reset_rx_datapath_int         <= 1'b0;
      sm_reset_all_timer_clr              <= 1'b1;
      sm_reset_all                        <= ST_RESET_ALL_BRANCH;
    end
    else begin
      case (sm_reset_all)

        // Upon initial configuration, check or wait for the transceiver power good indicator to be asserted before
        // proceeding with the sequence automatically
        ST_RESET_ALL_INIT: begin
          if (gtpowergood_sync)
            sm_reset_all <= ST_RESET_ALL_BRANCH;
        end

        // If the transmitter is enabled, begin by resetting the TX PLL. If the transmitter is disabled, begin by
        // resetting the RX PLL.
        ST_RESET_ALL_BRANCH: begin
          if (tx_enabled_tie_in)
            sm_reset_all <= ST_RESET_ALL_TX_PLL;
          else
            sm_reset_all <= ST_RESET_ALL_RX_PLL;
          sm_reset_all_timer_clr <= 1'b1;
        end

        // Force the transmitter reset state machine to reset the TX PLL and data path
        ST_RESET_ALL_TX_PLL: begin
          gtwiz_reset_tx_pll_and_datapath_int <= 1'b1;
          sm_reset_all                        <= ST_RESET_ALL_TX_PLL_WAIT;
        end

        // Await completion of the TX PLL and data path reset sequence. Then, if the receiver is enabled, continue by
        // either resetting just the RX data path (if the receiver and transmitter share a PLL) or the RX PLL (if the
        // receiver and transmitter PLLs are indepdendent). If the receiver is disabled, complete the sequence.
        ST_RESET_ALL_TX_PLL_WAIT: begin
          gtwiz_reset_tx_pll_and_datapath_int <= 1'b0;
          sm_reset_all_timer_clr              <= 1'b0;
          if (gtwiz_reset_tx_done_int && (~sm_reset_all_timer_clr) && sm_reset_all_timer_sat) begin
            if (rx_enabled_tie_in) begin
              if (shared_pll_tie_in)
                sm_reset_all <= ST_RESET_ALL_RX_DP;
              else
                sm_reset_all <= ST_RESET_ALL_RX_PLL;
            end
            else
              sm_reset_all <= ST_RESET_ALL_DONE;
            sm_reset_all_timer_clr <= 1'b1;
          end
        end

        // Force the receiver reset state machine to reset the RX data path
        ST_RESET_ALL_RX_DP: begin
          gtwiz_reset_rx_datapath_int <= 1'b1;
          sm_reset_all                <= ST_RESET_ALL_RX_WAIT;
        end

        // Force the receiver reset state machine to reset the RX PLL and data path
        ST_RESET_ALL_RX_PLL: begin
          gtwiz_reset_rx_pll_and_datapath_int <= 1'b1;
          sm_reset_all                        <= ST_RESET_ALL_RX_WAIT;
        end

        // Await completion of whichever RX reset sequence was performed
        ST_RESET_ALL_RX_WAIT: begin
          gtwiz_reset_rx_datapath_int         <= 1'b0;
          sm_reset_all_timer_clr              <= 1'b0;
          gtwiz_reset_rx_pll_and_datapath_int <= 1'b0;
          if (gtwiz_reset_rx_done_int && (~sm_reset_all_timer_clr) && sm_reset_all_timer_sat) begin
            sm_reset_all           <= ST_RESET_ALL_DONE;
            sm_reset_all_timer_clr <= 1'b1;
          end
        end

      endcase
    end
  end

  // Generate a small "reset all" state machine reset timer, used to stall certain states to guarantee that their
  // synchronized input values are being used at the appropriate time
  always @(posedge gtwiz_reset_clk_freerun_in) begin
    if (sm_reset_all_timer_clr) begin
      sm_reset_all_timer_ctr <= 3'd0;
      sm_reset_all_timer_sat <= 1'b0;
    end
    else begin
      if (sm_reset_all_timer_ctr != 3'd7)
        sm_reset_all_timer_ctr <= sm_reset_all_timer_ctr + 3'd1;
      else
        sm_reset_all_timer_sat <= 1'b1;
    end
  end


  // -------------------------------------------------------------------------------------------------------------------
  // Transmitter reset state machine
  // -------------------------------------------------------------------------------------------------------------------

  // The transmitter reset state machine responds to various synchronized inputs by resetting enabled transmitter-
  // related transceiver resources to which the reset helper block is connected. Various entry points to the sequential
  // reset sequence are available.

  // Synchronize the OR of all user input and internal TX reset signals for use in resetting the TX reset state machine
  wire gtwiz_reset_tx_any;
  wire gtwiz_reset_tx_any_sync;
  assign gtwiz_reset_tx_any = gtwiz_reset_tx_pll_and_datapath_in  ||
                              gtwiz_reset_tx_pll_and_datapath_int ||
                              gtwiz_reset_tx_datapath_in;
  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_gtwiz_reset_tx_any_inst (
    .clk_in  (gtwiz_reset_clk_freerun_in),
    .rst_in  (gtwiz_reset_tx_any),
    .rst_out (gtwiz_reset_tx_any_sync)
  );

  // Synchronize the OR of the user input and internal TX PLL and data path reset signals
  wire gtwiz_reset_tx_pll_and_datapath_sync;
  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_gtwiz_reset_tx_pll_and_datapath_inst (
    .clk_in  (gtwiz_reset_clk_freerun_in),
    .rst_in  (gtwiz_reset_tx_pll_and_datapath_in || gtwiz_reset_tx_pll_and_datapath_int),
    .rst_out (gtwiz_reset_tx_pll_and_datapath_sync)
  );

  // Use another synchronizer to delay the above signal for purposes of its detection following reset
  wire gtwiz_reset_tx_pll_and_datapath_dly;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_gtwiz_reset_tx_pll_and_datapath_dly_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (gtwiz_reset_tx_pll_and_datapath_sync),
    .o_out  (gtwiz_reset_tx_pll_and_datapath_dly)
  );

  // Synchronize the TX data path reset user input
  wire gtwiz_reset_tx_datapath_sync;
  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_gtwiz_reset_tx_datapath_inst (
    .clk_in  (gtwiz_reset_clk_freerun_in),
    .rst_in  (gtwiz_reset_tx_datapath_in),
    .rst_out (gtwiz_reset_tx_datapath_sync)
  );

  // Use another synchronizer to delay the above signal for purposes of its detection following reset
  wire gtwiz_reset_tx_datapath_dly;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_gtwiz_reset_tx_datapath_dly_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (gtwiz_reset_tx_datapath_sync),
    .o_out  (gtwiz_reset_tx_datapath_dly)
  );

  // Synchronize the TX user clock active indicator
  wire gtwiz_reset_userclk_tx_active_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_gtwiz_reset_userclk_tx_active_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (gtwiz_reset_userclk_tx_active_in),
    .o_out  (gtwiz_reset_userclk_tx_active_sync)
  );

  // Synchronize the TX PLL lock indicator
  wire plllock_tx_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_plllock_tx_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (plllock_tx_in),
    .o_out  (plllock_tx_sync)
  );

  // Declare the TX state machine reset timer registers
  reg       sm_reset_tx_timer_clr = 1'b1;
  reg [2:0] sm_reset_tx_timer_ctr = 3'd0;
  reg       sm_reset_tx_timer_sat = 1'b0;

  // Declare the TX state machine PLL reset timer registers
  localparam [9:0] P_TX_PLL_RESET_FREERUN_CYC = (P_TX_PLL_TYPE == 2 && P_USE_CPLL_CAL == 0) ?
                                                (2 * P_FREERUN_FREQUENCY) + 2 : 7;
  reg        sm_reset_tx_pll_timer_clr = 1'b1;
  reg  [9:0] sm_reset_tx_pll_timer_ctr = 10'd0;
  reg        sm_reset_tx_pll_timer_sat = 1'b0;
  wire [9:0] p_tx_pll_reset_freerun_cyc_int = P_TX_PLL_RESET_FREERUN_CYC;

  // Declare local parameters for TX reset state machine state values
  localparam [2:0] ST_RESET_TX_BRANCH         = 3'd0;
  localparam [2:0] ST_RESET_TX_PLL            = 3'd1;
  localparam [2:0] ST_RESET_TX_DATAPATH       = 3'd2;
  localparam [2:0] ST_RESET_TX_WAIT_LOCK      = 3'd3;
  localparam [2:0] ST_RESET_TX_WAIT_USERRDY   = 3'd4;
  localparam [2:0] ST_RESET_TX_WAIT_RESETDONE = 3'd5;
  localparam [2:0] ST_RESET_TX_IDLE           = 3'd6;
  reg        [2:0] sm_reset_tx                = ST_RESET_TX_BRANCH;

  // Implementation of transmitter reset state machine synchronous process
  always @(posedge gtwiz_reset_clk_freerun_in) begin

    // The state machine is synchronously reset by the synchronized OR of all user input and internal TX reset signals
    if (gtwiz_reset_tx_any_sync) begin
      gtwiz_reset_tx_done_int   <= 1'b0;
      sm_reset_tx_timer_clr     <= 1'b1;
      sm_reset_tx_pll_timer_clr <= 1'b1;
      sm_reset_tx               <= ST_RESET_TX_BRANCH;
    end
    else begin
      case (sm_reset_tx)

        // Once released from reset, branch to the reset control state indicated by the highest-priority synchronized
        // signal (which remains asserted due to its long synchronizer chain)
        ST_RESET_TX_BRANCH: begin
          if (gtwiz_reset_tx_pll_and_datapath_dly)
            sm_reset_tx <= ST_RESET_TX_PLL;
          else if (gtwiz_reset_tx_datapath_dly)
            sm_reset_tx <= ST_RESET_TX_DATAPATH;
          sm_reset_tx_timer_clr     <= 1'b1;
          sm_reset_tx_pll_timer_clr <= 1'b1;
        end

        // Assert the TX PLL and TX data path reset outputs
        ST_RESET_TX_PLL: begin
          pllreset_tx_out           <= 1'b1;
          gttxreset_out             <= 1'b1;
          txuserrdy_out             <= 1'b0;
          sm_reset_tx_pll_timer_clr <= 1'b0;
          if ((~sm_reset_tx_pll_timer_clr) && sm_reset_tx_pll_timer_sat) begin
            sm_reset_tx_pll_timer_clr <= 1'b1;
            sm_reset_tx               <= ST_RESET_TX_WAIT_LOCK;
          end
        end

        // Assert the TX data path reset output
        ST_RESET_TX_DATAPATH: begin
          gttxreset_out         <= 1'b1;
          txuserrdy_out         <= 1'b0;
          sm_reset_tx_timer_clr <= 1'b0;
          if ((~sm_reset_tx_timer_clr) && sm_reset_tx_timer_sat) begin
            sm_reset_tx_timer_clr <= 1'b1;
            sm_reset_tx           <= ST_RESET_TX_WAIT_LOCK;
          end
        end

        // De-assert the TX PLL reset output, and await the TX PLL lock indicator before de-asserting the TX data path
        // reset output
        ST_RESET_TX_WAIT_LOCK: begin
          pllreset_tx_out       <= 1'b0;
          sm_reset_tx_timer_clr <= 1'b0;
          if (plllock_tx_sync && (~sm_reset_tx_timer_clr) && sm_reset_tx_timer_sat) begin
            gttxreset_out         <= 1'b0;
            sm_reset_tx_timer_clr <= 1'b1;
            sm_reset_tx           <= ST_RESET_TX_WAIT_USERRDY;
          end
        end

        // Await the TX user clock active indicator from the TX user clocking helper block before asserting the TX user
        // ready output
        ST_RESET_TX_WAIT_USERRDY: begin
          sm_reset_tx_timer_clr <= 1'b0;
          if (gtwiz_reset_userclk_tx_active_sync && (~sm_reset_tx_timer_clr) && sm_reset_tx_timer_sat) begin
            txuserrdy_out         <= 1'b1;
            sm_reset_tx_timer_clr <= 1'b1;
            sm_reset_tx           <= ST_RESET_TX_WAIT_RESETDONE;
          end
        end

        // Await the TX reset done indicator before asserting the reset helper block TX reset done user output
        ST_RESET_TX_WAIT_RESETDONE: begin
          sm_reset_tx_timer_clr <= 1'b0;
          if (txresetdone_in && (~sm_reset_tx_timer_clr) && sm_reset_tx_timer_sat) begin
            gtwiz_reset_tx_done_int <= 1'b1;
            sm_reset_tx_timer_clr   <= 1'b1;
            sm_reset_tx             <= ST_RESET_TX_IDLE;
          end
        end

        // While idle, de-assert the reset helper block TX reset done user output if PLL lock is lost, signaling the
        // need for user intervention
        ST_RESET_TX_IDLE: begin
          if (!plllock_tx_sync)
            gtwiz_reset_tx_done_int <= 1'b0;
        end

        // Encountering the default case indicates a state register error, so de-assert the reset helper block TX
        // reset done user output, signaling the need for user intervention
        default: begin
          gtwiz_reset_tx_done_int <= 1'b0;
        end

      endcase
    end
  end

  // Generate a small TX state machine reset timer, used to stall certain states to guarantee that their synchronized
  // input values are being used at the appropriate time
  always @(posedge gtwiz_reset_clk_freerun_in) begin
    if (sm_reset_tx_timer_clr) begin
      sm_reset_tx_timer_ctr <= 3'd0;
      sm_reset_tx_timer_sat <= 1'b0;
    end
    else begin
      if (sm_reset_tx_timer_ctr != 3'd7)
        sm_reset_tx_timer_ctr <= sm_reset_tx_timer_ctr + 3'd1;
      else
        sm_reset_tx_timer_sat <= 1'b1;
    end
  end

  // Generate an TX PLL reset timer, used to indicate when the specified minimum TX PLL reset duration has expired. This
  // is used by the TX state machine to proceed beyond the ST_RESET_TX_PLL wait state.
  always @(posedge gtwiz_reset_clk_freerun_in) begin
    if (sm_reset_tx_pll_timer_clr) begin
      sm_reset_tx_pll_timer_ctr <= 10'd0;
      sm_reset_tx_pll_timer_sat <= 1'b0;
    end
    else begin
      if (sm_reset_tx_pll_timer_ctr != p_tx_pll_reset_freerun_cyc_int)
        sm_reset_tx_pll_timer_ctr <= sm_reset_tx_pll_timer_ctr + 10'd1;
      else
        sm_reset_tx_pll_timer_sat <= 1'b1;
    end
  end

  // Hold the TX programmable divider in reset until the TX PLL has locked
  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_txprogdivreset_inst (
    .clk_in  (gtwiz_reset_clk_freerun_in),
    .rst_in  (~plllock_tx_in),
    .rst_out (txprogdivreset_out)
  );

  // Synchronize the reset helper block TX reset done user output into the TXUSRCLK2 domain for user consumption
  gtwizard_ultrascale_v1_6_2_reset_inv_synchronizer reset_synchronizer_tx_done_inst (
    .clk_in  (txusrclk2_in),
    .rst_in  (gtwiz_reset_tx_done_int),
    .rst_out (gtwiz_reset_tx_done_out)
  );


  // -------------------------------------------------------------------------------------------------------------------
  // Receiver reset state machine
  // -------------------------------------------------------------------------------------------------------------------

  // The receiver reset state machine responds to various synchronized inputs by resetting enabled receiver-
  // related transceiver resources to which the reset helper block is connected. Various entry points to the sequential
  // reset sequence are available.

  // Initialize (for both synthesis and simulation) the RX PLL reset output flip-flop to 0 if the TX and RX PLLs are
  // shared upon device configuration, so as to not block TX PLL reset; or to 1 if the PLLs are independent, for
  // consistency with TX PLL initialization
  initial begin
    if (P_TX_PLL_TYPE == P_RX_PLL_TYPE)
      pllreset_rx_out = 1'b0;
    else
      pllreset_rx_out = 1'b1;
  end

  // Synchronize the OR of all user input and internal RX reset signals for use in resetting the RX reset state machine
  wire gtwiz_reset_rx_any;
  wire gtwiz_reset_rx_any_sync;
  assign gtwiz_reset_rx_any = gtwiz_reset_rx_pll_and_datapath_in  ||
                              gtwiz_reset_rx_pll_and_datapath_int ||
                              gtwiz_reset_rx_datapath_in          ||
                              gtwiz_reset_rx_datapath_int;
  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_gtwiz_reset_rx_any_inst (
    .clk_in  (gtwiz_reset_clk_freerun_in),
    .rst_in  (gtwiz_reset_rx_any),
    .rst_out (gtwiz_reset_rx_any_sync)
  );

  // Synchronize the OR of the user input and internal RX PLL and data path reset signals
  wire gtwiz_reset_rx_pll_and_datapath_sync;
  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_gtwiz_reset_rx_pll_and_datapath_inst (
    .clk_in  (gtwiz_reset_clk_freerun_in),
    .rst_in  (gtwiz_reset_rx_pll_and_datapath_in || gtwiz_reset_rx_pll_and_datapath_int),
    .rst_out (gtwiz_reset_rx_pll_and_datapath_sync)
  );

  // Use another synchronizer to delay the above signal for purposes of its detection following reset
  wire gtwiz_reset_rx_pll_and_datapath_dly;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_gtwiz_reset_rx_pll_and_datapath_dly_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (gtwiz_reset_rx_pll_and_datapath_sync),
    .o_out  (gtwiz_reset_rx_pll_and_datapath_dly)
  );

  // Synchronize the RX data path reset user input
  wire gtwiz_reset_rx_datapath_sync;
  gtwizard_ultrascale_v1_6_2_reset_synchronizer reset_synchronizer_gtwiz_reset_rx_datapath_inst (
    .clk_in  (gtwiz_reset_clk_freerun_in),
    .rst_in  (gtwiz_reset_rx_datapath_in || gtwiz_reset_rx_datapath_int),
    .rst_out (gtwiz_reset_rx_datapath_sync)
  );

  // Use another synchronizer to delay the above signal for purposes of its detection following reset
  wire gtwiz_reset_rx_datapath_dly;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_gtwiz_reset_rx_datapath_dly_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (gtwiz_reset_rx_datapath_sync),
    .o_out  (gtwiz_reset_rx_datapath_dly)
  );

  // Synchronize the RX user clock active indicator
  wire gtwiz_reset_userclk_rx_active_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_gtwiz_reset_userclk_rx_active_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (gtwiz_reset_userclk_rx_active_in),
    .o_out  (gtwiz_reset_userclk_rx_active_sync)
  );

  // Synchronize the RX PLL lock indicator
  wire plllock_rx_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_plllock_rx_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (plllock_rx_in),
    .o_out  (plllock_rx_sync)
  );

  // Synchronize the RX CDR lock indicator
  wire rxcdrlock_sync;
  gtwizard_ultrascale_v1_6_2_bit_synchronizer bit_synchronizer_rxcdrlock_inst (
    .clk_in (gtwiz_reset_clk_freerun_in),
    .i_in   (rxcdrlock_in),
    .o_out  (rxcdrlock_sync)
  );

  // Declare the RX state machine reset timer registers
  reg       sm_reset_rx_timer_clr = 1'b1;
  reg [2:0] sm_reset_rx_timer_ctr = 3'd0;
  reg       sm_reset_rx_timer_sat = 1'b0;

  // Declare the RX state machine PLL reset timer registers
  localparam [9:0] P_RX_PLL_RESET_FREERUN_CYC = (P_RX_PLL_TYPE == 2 && P_USE_CPLL_CAL == 0) ?
                                                (2 * P_FREERUN_FREQUENCY) + 2 : 7;
  reg        sm_reset_rx_pll_timer_clr = 1'b1;
  reg  [9:0] sm_reset_rx_pll_timer_ctr = 10'd0;
  reg        sm_reset_rx_pll_timer_sat = 1'b0;
  wire [9:0] p_rx_pll_reset_freerun_cyc_int = P_RX_PLL_RESET_FREERUN_CYC;

  // Declare the RX state machine CDR lock timeout counter
  reg         sm_reset_rx_cdr_to_clr = 1'b1;
  reg  [25:0] sm_reset_rx_cdr_to_ctr = 26'd0;
  reg         sm_reset_rx_cdr_to_sat = 1'b0;
  wire [25:0] p_cdr_timeout_freerun_cyc_int = P_CDR_TIMEOUT_FREERUN_CYC;

  // Declare local parameters for RX reset state machine state values
  localparam [2:0] ST_RESET_RX_BRANCH         = 3'd0;
  localparam [2:0] ST_RESET_RX_PLL            = 3'd1;
  localparam [2:0] ST_RESET_RX_DATAPATH       = 3'd2;
  localparam [2:0] ST_RESET_RX_WAIT_LOCK      = 3'd3;
  localparam [2:0] ST_RESET_RX_WAIT_CDR       = 3'd4;
  localparam [2:0] ST_RESET_RX_WAIT_USERRDY   = 3'd5;
  localparam [2:0] ST_RESET_RX_WAIT_RESETDONE = 3'd6;
  localparam [2:0] ST_RESET_RX_IDLE           = 3'd7;
  reg        [2:0] sm_reset_rx                = ST_RESET_RX_BRANCH;

  // Implementation of receiver reset state machine synchronous process
  always @(posedge gtwiz_reset_clk_freerun_in) begin

    // The state machine is synchronously reset by the synchronized OR of all user input and internal RX reset signals
    if (gtwiz_reset_rx_any_sync) begin
      gtwiz_reset_rx_done_int   <= 1'b0;
      sm_reset_rx_timer_clr     <= 1'b1;
      sm_reset_rx_pll_timer_clr <= 1'b1;
      sm_reset_rx_cdr_to_clr    <= 1'b1;
      sm_reset_rx               <= ST_RESET_RX_BRANCH;
    end
    else begin
      case (sm_reset_rx)

        // Once released from reset, branch to the reset control state indicated by the highest-priority synchronized
        // signal (which remains asserted due to its long synchronizer chain)
        ST_RESET_RX_BRANCH: begin
          if (gtwiz_reset_rx_pll_and_datapath_dly)
            sm_reset_rx <= ST_RESET_RX_PLL;
          else if (gtwiz_reset_rx_datapath_dly)
            sm_reset_rx <= ST_RESET_RX_DATAPATH;
          sm_reset_rx_timer_clr     <= 1'b1;
          sm_reset_rx_pll_timer_clr <= 1'b1;
          sm_reset_rx_cdr_to_clr    <= 1'b1;
        end

        // Assert the RX PLL, RX programmable divider, and RX data path reset outputs
        ST_RESET_RX_PLL: begin
          pllreset_rx_out           <= 1'b1;
          rxprogdivreset_out        <= 1'b1;
          gtrxreset_out             <= 1'b1;
          rxuserrdy_out             <= 1'b0;
          sm_reset_rx_pll_timer_clr <= 1'b0;
          if ((~sm_reset_rx_pll_timer_clr) && sm_reset_rx_pll_timer_sat) begin
            sm_reset_rx_pll_timer_clr <= 1'b1;
            sm_reset_rx               <= ST_RESET_RX_WAIT_LOCK;
          end
        end

        // Assert the RX data path and RX programmable divider reset outputs
        ST_RESET_RX_DATAPATH: begin
          rxprogdivreset_out    <= 1'b1;
          gtrxreset_out         <= 1'b1;
          rxuserrdy_out         <= 1'b0;
          sm_reset_rx_timer_clr <= 1'b0;
          if ((~sm_reset_rx_timer_clr) && sm_reset_rx_timer_sat) begin
            sm_reset_rx_timer_clr <= 1'b1;
            sm_reset_rx           <= ST_RESET_RX_WAIT_LOCK;
          end
        end

        // De-assert the RX PLL reset output, and await the RX PLL lock indicator before de-asserting the RX data path
        // reset output
        ST_RESET_RX_WAIT_LOCK: begin
          pllreset_rx_out       <= 1'b0;
          sm_reset_rx_timer_clr <= 1'b0;
          if (plllock_rx_sync && (~sm_reset_rx_timer_clr) && sm_reset_rx_timer_sat) begin
            gtrxreset_out          <= 1'b0;
            sm_reset_rx_timer_clr  <= 1'b1;
            sm_reset_rx_cdr_to_clr <= 1'b0;
            sm_reset_rx            <= ST_RESET_RX_WAIT_CDR;
          end
        end

        // Await an indication of CDR stability (either the direct transceiver RXCDRLOCK output, or expiration of the
        // specified maximum CDR locking time, whichever occurs first) before removing the RX programmable divider reset
        // and proceeding
        ST_RESET_RX_WAIT_CDR: begin
          if (rxcdrlock_sync || sm_reset_rx_cdr_to_sat) begin
            rxprogdivreset_out     <= 1'b0;
            sm_reset_rx_cdr_to_clr <= 1'b1;
            sm_reset_rx            <= ST_RESET_RX_WAIT_USERRDY;
          end
        end

        // Await the RX user clock active indicator from the RX user clocking helper block before asserting the RX user
        // ready output
        ST_RESET_RX_WAIT_USERRDY: begin
          sm_reset_rx_timer_clr <= 1'b0;
          if (gtwiz_reset_userclk_rx_active_sync && (~sm_reset_rx_timer_clr) && sm_reset_rx_timer_sat) begin
            rxuserrdy_out         <= 1'b1;
            sm_reset_rx_timer_clr <= 1'b1;
            sm_reset_rx           <= ST_RESET_RX_WAIT_RESETDONE;
          end
        end

        // Await the RX reset done indicator before asserting the reset helper block RX reset done user output
        ST_RESET_RX_WAIT_RESETDONE: begin
          sm_reset_rx_timer_clr <= 1'b0;
          if (rxresetdone_in && (~sm_reset_rx_timer_clr) && sm_reset_rx_timer_sat)
          begin
            gtwiz_reset_rx_done_int <= 1'b1;
            sm_reset_rx_timer_clr   <= 1'b1;
            sm_reset_rx             <= ST_RESET_RX_IDLE;
          end
        end

        // While idle, de-assert the reset helper block RX reset done user output if PLL lock is lost, signaling the
        // need for user intervention
        ST_RESET_RX_IDLE: begin
          if (!plllock_rx_sync)
            gtwiz_reset_rx_done_int <= 1'b0;
        end

      endcase
    end
  end

  // Generate a small RX state machine reset timer, used to stall certain states to guarantee that their synchronized
  // input values are being used at the appropriate time
  always @(posedge gtwiz_reset_clk_freerun_in) begin
    if (sm_reset_rx_timer_clr) begin
      sm_reset_rx_timer_ctr <= 3'd0;
      sm_reset_rx_timer_sat <= 1'b0;
    end
    else begin
      if (sm_reset_rx_timer_ctr != 3'd7)
        sm_reset_rx_timer_ctr <= sm_reset_rx_timer_ctr + 3'd1;
      else
        sm_reset_rx_timer_sat <= 1'b1;
    end
  end

  // Generate an RX PLL reset timer, used to indicate when the specified minimum RX PLL reset duration has expired. This
  // is used by the RX state machine to proceed beyond the ST_RESET_RX_PLL wait state.
  always @(posedge gtwiz_reset_clk_freerun_in) begin
    if (sm_reset_rx_pll_timer_clr) begin
      sm_reset_rx_pll_timer_ctr <= 10'd0;
      sm_reset_rx_pll_timer_sat <= 1'b0;
    end
    else begin
      if (sm_reset_rx_pll_timer_ctr != p_rx_pll_reset_freerun_cyc_int)
        sm_reset_rx_pll_timer_ctr <= sm_reset_rx_pll_timer_ctr + 10'd1;
      else
        sm_reset_rx_pll_timer_sat <= 1'b1;
    end
  end

  // Generate a CDR lock timeout timer, used to indicate when the specified maximum CDR locking time has expired. This
  // is used by the RX state machine to proceed beyond the ST_RESET_RX_WAIT_CDR wait state in the event that the
  // transceiver RXCDRLOCK output does not assert within that time period.
  always @(posedge gtwiz_reset_clk_freerun_in) begin
    if (sm_reset_rx_cdr_to_clr) begin
      sm_reset_rx_cdr_to_ctr <= 26'd0;
      sm_reset_rx_cdr_to_sat <= 1'b0;
    end
    else begin
      if (sm_reset_rx_cdr_to_ctr != p_cdr_timeout_freerun_cyc_int)
        sm_reset_rx_cdr_to_ctr <= sm_reset_rx_cdr_to_ctr + 26'd1;
      else
        sm_reset_rx_cdr_to_sat <= 1'b1;
    end
  end

  // Assign the RX CDR stable user indicator to the transceiver RXCDRLOCK output
  assign gtwiz_reset_rx_cdr_stable_out = rxcdrlock_sync;

  // Synchronize the reset helper block RX reset done user output into the RXUSRCLK2 domain for user consumption
  gtwizard_ultrascale_v1_6_2_reset_inv_synchronizer reset_synchronizer_rx_done_inst (
    .clk_in  (rxusrclk2_in),
    .rst_in  (gtwiz_reset_rx_done_int),
    .rst_out (gtwiz_reset_rx_done_out)
  );


endmodule

module gtwizard_ultrascale_v1_6_2_gtwiz_userdata_tx #(

  parameter integer P_TX_USER_DATA_WIDTH       = 32,
  parameter integer P_TX_DATA_ENCODING         = 0,
  parameter integer P_TOTAL_NUMBER_OF_CHANNELS = 1

)(

  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS*P_TX_USER_DATA_WIDTH)-1:0] gtwiz_userdata_tx_in,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS*128)-1:0]                  txdata_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS* 16)-1:0]                  txctrl0_out,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS* 16)-1:0]                  txctrl1_out

);


  // -------------------------------------------------------------------------------------------------------------------
  // Transmitter user data width sizing conditional generation, based on parameter values in module instantiation
  // -------------------------------------------------------------------------------------------------------------------
  generate if (1) begin : gen_gtwiz_userdata_tx_main
    genvar ch_idx;

    // If transmitter data encoding is either not raw mode, or is raw mode but user data width is not modulus 10, then
    // simply assign the transceiver-facing TXDATA vector with data bits from the user-facing data vector, plus padding.
    if ((P_TX_DATA_ENCODING != 0) ||
       ((P_TX_DATA_ENCODING == 0) && (P_TX_USER_DATA_WIDTH % 10 != 0))) begin : gen_txdata

      for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_assign
        if (P_TX_USER_DATA_WIDTH < 128) begin : gen_less_128
          assign txdata_out[(128*ch_idx)+127:(128*ch_idx)] =
            {{128-P_TX_USER_DATA_WIDTH{1'b0}},
             gtwiz_userdata_tx_in[(P_TX_USER_DATA_WIDTH*ch_idx)+P_TX_USER_DATA_WIDTH-1:
                                  (P_TX_USER_DATA_WIDTH*ch_idx)]};
        end
        else begin : gen_128
          assign txdata_out[(128*ch_idx)+127:(128*ch_idx)] =
             gtwiz_userdata_tx_in[(P_TX_USER_DATA_WIDTH*ch_idx)+P_TX_USER_DATA_WIDTH-1:
                                  (P_TX_USER_DATA_WIDTH*ch_idx)];
        end
      end

      // Transceiver-facing TXCTRL0 and TXCTRL1 vectors are not driven by this helper block in this configuration
      assign txctrl0_out = {P_TOTAL_NUMBER_OF_CHANNELS*16{1'b0}};
      assign txctrl1_out = {P_TOTAL_NUMBER_OF_CHANNELS*16{1'b0}};
    end

    // If transmitter data encoding is raw mode and user data width is modulus 10, then assign the specified combination
    // of transceiver-facing TXDATA, TXCTRL0, and TXCTRL1 vectors with the user-facing data vector. The interleaving of
    // these vectors is always in a repeating 8-1-1 bit pattern, scaling with the user data width size.
    else begin : gen_txdata_txctrl

      case (P_TX_USER_DATA_WIDTH)
        20:
          for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_width20
            assign txctrl1_out[(16*ch_idx)+15:(16*ch_idx)] =
              {14'b0,
               gtwiz_userdata_tx_in[(ch_idx*20)+19],
               gtwiz_userdata_tx_in[(ch_idx*20)+9]};
            assign txctrl0_out[(16*ch_idx)+15:(16*ch_idx)] =
              {14'b0,
               gtwiz_userdata_tx_in[(ch_idx*20)+18],
               gtwiz_userdata_tx_in[(ch_idx*20)+8]};
            assign txdata_out[(128*ch_idx)+127:(128*ch_idx)] =
              {112'b0,
               gtwiz_userdata_tx_in[(ch_idx*20)+17:(ch_idx*20)+10],
               gtwiz_userdata_tx_in[(ch_idx*20)+7:ch_idx*20]};
          end

        40:
          for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_width40
            assign txctrl1_out[(16*ch_idx)+15:(16*ch_idx)] =
              {12'b0,
               gtwiz_userdata_tx_in[(ch_idx*40)+39],
               gtwiz_userdata_tx_in[(ch_idx*40)+29],
               gtwiz_userdata_tx_in[(ch_idx*40)+19],
               gtwiz_userdata_tx_in[(ch_idx*40)+9]};
            assign txctrl0_out[(16*ch_idx)+15:(16*ch_idx)] =
              {12'b0,
               gtwiz_userdata_tx_in[(ch_idx*40)+38],
               gtwiz_userdata_tx_in[(ch_idx*40)+28],
               gtwiz_userdata_tx_in[(ch_idx*40)+18],
               gtwiz_userdata_tx_in[(ch_idx*40)+8]};
            assign txdata_out[(128*ch_idx)+127:(128*ch_idx)] =
              {96'b0,
               gtwiz_userdata_tx_in[(ch_idx*40)+37:(ch_idx*40)+30],
               gtwiz_userdata_tx_in[(ch_idx*40)+27:(ch_idx*40)+20],
               gtwiz_userdata_tx_in[(ch_idx*40)+17:(ch_idx*40)+10],
               gtwiz_userdata_tx_in[(ch_idx*40)+7:ch_idx*40]};
          end

        80:
          for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_width80
            assign txctrl1_out[(16*ch_idx)+15:(16*ch_idx)] =
              {8'b0,
               gtwiz_userdata_tx_in[(ch_idx*80)+79],
               gtwiz_userdata_tx_in[(ch_idx*80)+69],
               gtwiz_userdata_tx_in[(ch_idx*80)+59],
               gtwiz_userdata_tx_in[(ch_idx*80)+49],
               gtwiz_userdata_tx_in[(ch_idx*80)+39],
               gtwiz_userdata_tx_in[(ch_idx*80)+29],
               gtwiz_userdata_tx_in[(ch_idx*80)+19],
               gtwiz_userdata_tx_in[(ch_idx*80)+9]};
            assign txctrl0_out[(16*ch_idx)+15:(16*ch_idx)] =
              {8'b0,
               gtwiz_userdata_tx_in[(ch_idx*80)+78],
               gtwiz_userdata_tx_in[(ch_idx*80)+68],
               gtwiz_userdata_tx_in[(ch_idx*80)+58],
               gtwiz_userdata_tx_in[(ch_idx*80)+48],
               gtwiz_userdata_tx_in[(ch_idx*80)+38],
               gtwiz_userdata_tx_in[(ch_idx*80)+28],
               gtwiz_userdata_tx_in[(ch_idx*80)+18],
               gtwiz_userdata_tx_in[(ch_idx*80)+8]};
            assign txdata_out[(128*ch_idx)+127:(128*ch_idx)] =
              {64'b0,
               gtwiz_userdata_tx_in[(ch_idx*80)+77:(ch_idx*80)+70],
               gtwiz_userdata_tx_in[(ch_idx*80)+67:(ch_idx*80)+60],
               gtwiz_userdata_tx_in[(ch_idx*80)+57:(ch_idx*80)+50],
               gtwiz_userdata_tx_in[(ch_idx*80)+47:(ch_idx*80)+40],
               gtwiz_userdata_tx_in[(ch_idx*80)+37:(ch_idx*80)+30],
               gtwiz_userdata_tx_in[(ch_idx*80)+27:(ch_idx*80)+20],
               gtwiz_userdata_tx_in[(ch_idx*80)+17:(ch_idx*80)+10],
               gtwiz_userdata_tx_in[(ch_idx*80)+7:ch_idx*80]};
          end

        160:
          for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_width160
            assign txctrl1_out[(16*ch_idx)+15:(16*ch_idx)] =
              {gtwiz_userdata_tx_in[(ch_idx*160)+159],
               gtwiz_userdata_tx_in[(ch_idx*160)+149],
               gtwiz_userdata_tx_in[(ch_idx*160)+139],
               gtwiz_userdata_tx_in[(ch_idx*160)+129],
               gtwiz_userdata_tx_in[(ch_idx*160)+119],
               gtwiz_userdata_tx_in[(ch_idx*160)+109],
               gtwiz_userdata_tx_in[(ch_idx*160)+99],
               gtwiz_userdata_tx_in[(ch_idx*160)+89],
               gtwiz_userdata_tx_in[(ch_idx*160)+79],
               gtwiz_userdata_tx_in[(ch_idx*160)+69],
               gtwiz_userdata_tx_in[(ch_idx*160)+59],
               gtwiz_userdata_tx_in[(ch_idx*160)+49],
               gtwiz_userdata_tx_in[(ch_idx*160)+39],
               gtwiz_userdata_tx_in[(ch_idx*160)+29],
               gtwiz_userdata_tx_in[(ch_idx*160)+19],
               gtwiz_userdata_tx_in[(ch_idx*160)+9]};
            assign txctrl0_out[(16*ch_idx)+15:(16*ch_idx)] =
              {gtwiz_userdata_tx_in[(ch_idx*160)+158],
               gtwiz_userdata_tx_in[(ch_idx*160)+148],
               gtwiz_userdata_tx_in[(ch_idx*160)+138],
               gtwiz_userdata_tx_in[(ch_idx*160)+128],
               gtwiz_userdata_tx_in[(ch_idx*160)+118],
               gtwiz_userdata_tx_in[(ch_idx*160)+108],
               gtwiz_userdata_tx_in[(ch_idx*160)+98],
               gtwiz_userdata_tx_in[(ch_idx*160)+88],
               gtwiz_userdata_tx_in[(ch_idx*160)+78],
               gtwiz_userdata_tx_in[(ch_idx*160)+68],
               gtwiz_userdata_tx_in[(ch_idx*160)+58],
               gtwiz_userdata_tx_in[(ch_idx*160)+48],
               gtwiz_userdata_tx_in[(ch_idx*160)+38],
               gtwiz_userdata_tx_in[(ch_idx*160)+28],
               gtwiz_userdata_tx_in[(ch_idx*160)+18],
               gtwiz_userdata_tx_in[(ch_idx*160)+8]};
            assign txdata_out[(128*ch_idx)+127:(128*ch_idx)] =
              {gtwiz_userdata_tx_in[(ch_idx*160)+157:(ch_idx*160)+150],
               gtwiz_userdata_tx_in[(ch_idx*160)+147:(ch_idx*160)+140],
               gtwiz_userdata_tx_in[(ch_idx*160)+137:(ch_idx*160)+130],
               gtwiz_userdata_tx_in[(ch_idx*160)+127:(ch_idx*160)+120],
               gtwiz_userdata_tx_in[(ch_idx*160)+117:(ch_idx*160)+110],
               gtwiz_userdata_tx_in[(ch_idx*160)+107:(ch_idx*160)+100],
               gtwiz_userdata_tx_in[(ch_idx*160)+97:(ch_idx*160)+90],
               gtwiz_userdata_tx_in[(ch_idx*160)+87:(ch_idx*160)+80],
               gtwiz_userdata_tx_in[(ch_idx*160)+77:(ch_idx*160)+70],
               gtwiz_userdata_tx_in[(ch_idx*160)+67:(ch_idx*160)+60],
               gtwiz_userdata_tx_in[(ch_idx*160)+57:(ch_idx*160)+50],
               gtwiz_userdata_tx_in[(ch_idx*160)+47:(ch_idx*160)+40],
               gtwiz_userdata_tx_in[(ch_idx*160)+37:(ch_idx*160)+30],
               gtwiz_userdata_tx_in[(ch_idx*160)+27:(ch_idx*160)+20],
               gtwiz_userdata_tx_in[(ch_idx*160)+17:(ch_idx*160)+10],
               gtwiz_userdata_tx_in[(ch_idx*160)+7:ch_idx*160]};
          end

        default:
          begin : gen_default
            // The default case is a configuration error scenario should never be exercised
            assign txdata_out  = {P_TOTAL_NUMBER_OF_CHANNELS*128{1'b1}};
            assign txctrl0_out = {P_TOTAL_NUMBER_OF_CHANNELS*16{1'b1}};
            assign txctrl1_out = {P_TOTAL_NUMBER_OF_CHANNELS*16{1'b1}};
          end
      endcase

    end

  end
  endgenerate


endmodule

module gtwizard_ultrascale_v1_6_2_gtwiz_userdata_rx #(

  parameter integer P_RX_USER_DATA_WIDTH       = 32,
  parameter integer P_RX_DATA_DECODING         = 0,
  parameter integer P_TOTAL_NUMBER_OF_CHANNELS = 1

)(

  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS*128)-1:0]                  rxdata_in,
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS* 16)-1:0]                  rxctrl0_in,
  input  wire [(P_TOTAL_NUMBER_OF_CHANNELS* 16)-1:0]                  rxctrl1_in,
  output wire [(P_TOTAL_NUMBER_OF_CHANNELS*P_RX_USER_DATA_WIDTH)-1:0] gtwiz_userdata_rx_out

);


  // -------------------------------------------------------------------------------------------------------------------
  // Receiver user data width sizing conditional generation, based on parameter values in module instantiation
  // -------------------------------------------------------------------------------------------------------------------
  generate if (1) begin : gen_gtwiz_userdata_rx_main
    genvar ch_idx;

    // If receiver data decoding is either not raw mode, or is raw mode but user data width is not modulus 10, then
    // simply assign the user-facing data vector with the active data bits from the transceiver-facing RXDATA vector.
    if ((P_RX_DATA_DECODING != 0) ||
       ((P_RX_DATA_DECODING == 0) && (P_RX_USER_DATA_WIDTH % 10 != 0))) begin : gen_rxdata

      for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_assign
        assign gtwiz_userdata_rx_out[(P_RX_USER_DATA_WIDTH*ch_idx)+P_RX_USER_DATA_WIDTH-1:
                                     (P_RX_USER_DATA_WIDTH*ch_idx)] =
          rxdata_in[(128*ch_idx)+P_RX_USER_DATA_WIDTH-1:(128*ch_idx)];
      end

    end

    // If receiver data decoding is raw mode and user data width is modulus 10, then assign the user-facing data vector
    // with the specified combination of transceiver-facing RXDATA, RXCTRL0, and RXCTRL1 vectors. The interleaving of
    // these vectors is always in a repeating 8-1-1 bit pattern, scaling with the user data width size.
    else begin : gen_rxdata_rxctrl

      case (P_RX_USER_DATA_WIDTH)
        20:
          for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_width20
            assign gtwiz_userdata_rx_out [(ch_idx*20)+19:(ch_idx*20)] =
              {rxctrl1_in[(ch_idx* 16)+1],
               rxctrl0_in[(ch_idx* 16)+1],
               rxdata_in [(ch_idx*128)+15:(ch_idx*128)+8],
               rxctrl1_in[(ch_idx* 16)],
               rxctrl0_in[(ch_idx* 16)],
               rxdata_in [(ch_idx*128)+7:ch_idx*128]};
          end

        40:
          for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_width40
            assign gtwiz_userdata_rx_out [(ch_idx*40)+39:(ch_idx*40)] =
              {rxctrl1_in[(ch_idx* 16)+3],
               rxctrl0_in[(ch_idx* 16)+3],
               rxdata_in [(ch_idx*128)+31:(ch_idx*128)+24],
               rxctrl1_in[(ch_idx* 16)+2],
               rxctrl0_in[(ch_idx* 16)+2],
               rxdata_in [(ch_idx*128)+23:(ch_idx*128)+16],
               rxctrl1_in[(ch_idx* 16)+1],
               rxctrl0_in[(ch_idx* 16)+1],
               rxdata_in [(ch_idx*128)+15:(ch_idx*128)+8],
               rxctrl1_in[(ch_idx* 16)],
               rxctrl0_in[(ch_idx* 16)],
               rxdata_in [(ch_idx*128)+7:ch_idx*128]};
          end

        80:
          for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_width80
            assign gtwiz_userdata_rx_out [(ch_idx*80)+79:(ch_idx*80)] =
              {rxctrl1_in[(ch_idx* 16)+7],
               rxctrl0_in[(ch_idx* 16)+7],
               rxdata_in [(ch_idx*128)+63:(ch_idx*128)+56],
               rxctrl1_in[(ch_idx* 16)+6],
               rxctrl0_in[(ch_idx* 16)+6],
               rxdata_in [(ch_idx*128)+55:(ch_idx*128)+48],
               rxctrl1_in[(ch_idx* 16)+5],
               rxctrl0_in[(ch_idx* 16)+5],
               rxdata_in [(ch_idx*128)+47:(ch_idx*128)+40],
               rxctrl1_in[(ch_idx* 16)+4],
               rxctrl0_in[(ch_idx* 16)+4],
               rxdata_in [(ch_idx*128)+39:(ch_idx*128)+32],
               rxctrl1_in[(ch_idx* 16)+3],
               rxctrl0_in[(ch_idx* 16)+3],
               rxdata_in [(ch_idx*128)+31:(ch_idx*128)+24],
               rxctrl1_in[(ch_idx* 16)+2],
               rxctrl0_in[(ch_idx* 16)+2],
               rxdata_in [(ch_idx*128)+23:(ch_idx*128)+16],
               rxctrl1_in[(ch_idx* 16)+1],
               rxctrl0_in[(ch_idx* 16)+1],
               rxdata_in [(ch_idx*128)+15:(ch_idx*128)+8],
               rxctrl1_in[(ch_idx* 16)],
               rxctrl0_in[(ch_idx* 16)],
               rxdata_in [(ch_idx*128)+7:ch_idx*128]};
          end

        160:
          for (ch_idx = 0; ch_idx < P_TOTAL_NUMBER_OF_CHANNELS; ch_idx = ch_idx + 1) begin : gen_width160
            assign gtwiz_userdata_rx_out [(ch_idx*160)+159:(ch_idx*160)] =
              {rxctrl1_in[(ch_idx* 16)+15],
               rxctrl0_in[(ch_idx* 16)+15],
               rxdata_in [(ch_idx*128)+127:(ch_idx*128)+120],
               rxctrl1_in[(ch_idx* 16)+14],
               rxctrl0_in[(ch_idx* 16)+14],
               rxdata_in [(ch_idx*128)+119:(ch_idx*128)+112],
               rxctrl1_in[(ch_idx* 16)+13],
               rxctrl0_in[(ch_idx* 16)+13],
               rxdata_in [(ch_idx*128)+111:(ch_idx*128)+104],
               rxctrl1_in[(ch_idx* 16)+12],
               rxctrl0_in[(ch_idx* 16)+12],
               rxdata_in [(ch_idx*128)+103:(ch_idx*128)+96],
               rxctrl1_in[(ch_idx* 16)+11],
               rxctrl0_in[(ch_idx* 16)+11],
               rxdata_in [(ch_idx*128)+95:(ch_idx*128)+88],
               rxctrl1_in[(ch_idx* 16)+10],
               rxctrl0_in[(ch_idx* 16)+10],
               rxdata_in [(ch_idx*128)+87:(ch_idx*128)+80],
               rxctrl1_in[(ch_idx* 16)+9],
               rxctrl0_in[(ch_idx* 16)+9],
               rxdata_in [(ch_idx*128)+79:(ch_idx*128)+72],
               rxctrl1_in[(ch_idx* 16)+8],
               rxctrl0_in[(ch_idx* 16)+8],
               rxdata_in [(ch_idx*128)+71:(ch_idx*128)+64],
               rxctrl1_in[(ch_idx* 16)+7],
               rxctrl0_in[(ch_idx* 16)+7],
               rxdata_in [(ch_idx*128)+63:(ch_idx*128)+56],
               rxctrl1_in[(ch_idx* 16)+6],
               rxctrl0_in[(ch_idx* 16)+6],
               rxdata_in [(ch_idx*128)+55:(ch_idx*128)+48],
               rxctrl1_in[(ch_idx* 16)+5],
               rxctrl0_in[(ch_idx* 16)+5],
               rxdata_in [(ch_idx*128)+47:(ch_idx*128)+40],
               rxctrl1_in[(ch_idx* 16)+4],
               rxctrl0_in[(ch_idx* 16)+4],
               rxdata_in [(ch_idx*128)+39:(ch_idx*128)+32],
               rxctrl1_in[(ch_idx* 16)+3],
               rxctrl0_in[(ch_idx* 16)+3],
               rxdata_in [(ch_idx*128)+31:(ch_idx*128)+24],
               rxctrl1_in[(ch_idx* 16)+2],
               rxctrl0_in[(ch_idx* 16)+2],
               rxdata_in [(ch_idx*128)+23:(ch_idx*128)+16],
               rxctrl1_in[(ch_idx* 16)+1],
               rxctrl0_in[(ch_idx* 16)+1],
               rxdata_in [(ch_idx*128)+15:(ch_idx*128)+8],
               rxctrl1_in[(ch_idx* 16)],
               rxctrl0_in[(ch_idx* 16)],
               rxdata_in [(ch_idx*128)+7:ch_idx*128]};
          end

        default:
          begin : gen_default
            // The default case is a configuration error scenario should never be exercised
            assign gtwiz_userdata_rx_out = {P_TOTAL_NUMBER_OF_CHANNELS*P_RX_USER_DATA_WIDTH{1'b1}};
          end
      endcase

    end

  end
  endgenerate


endmodule

module gtwizard_ultrascale_v1_6_2_gthe3_common #(


  // -------------------------------------------------------------------------------------------------------------------
  // Parameters relating to GTHE3_COMMON primitive
  // -------------------------------------------------------------------------------------------------------------------

  // primitive wrapper parameters which override corresponding GTHE3_COMMON primitive parameters
  parameter  [15:0] GTHE3_COMMON_BIAS_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_BIAS_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_BIAS_CFG2 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_BIAS_CFG3 = 16'h0040,
  parameter  [15:0] GTHE3_COMMON_BIAS_CFG4 = 16'h0000,
  parameter   [9:0] GTHE3_COMMON_BIAS_CFG_RSVD = 10'b0000000000,
  parameter  [15:0] GTHE3_COMMON_COMMON_CFG0 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_COMMON_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_POR_CFG = 16'h0004,
  parameter  [15:0] GTHE3_COMMON_QPLL0_CFG0 = 16'h3018,
  parameter  [15:0] GTHE3_COMMON_QPLL0_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_QPLL0_CFG1_G3 = 16'h0020,
  parameter  [15:0] GTHE3_COMMON_QPLL0_CFG2 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_QPLL0_CFG2_G3 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_QPLL0_CFG3 = 16'h0120,
  parameter  [15:0] GTHE3_COMMON_QPLL0_CFG4 = 16'h0009,
  parameter   [9:0] GTHE3_COMMON_QPLL0_CP = 10'b0000011111,
  parameter   [9:0] GTHE3_COMMON_QPLL0_CP_G3 = 10'b0000011111,
  parameter integer GTHE3_COMMON_QPLL0_FBDIV = 66,
  parameter integer GTHE3_COMMON_QPLL0_FBDIV_G3 = 80,
  parameter  [15:0] GTHE3_COMMON_QPLL0_INIT_CFG0 = 16'h0000,
  parameter   [7:0] GTHE3_COMMON_QPLL0_INIT_CFG1 = 8'h00,
  parameter  [15:0] GTHE3_COMMON_QPLL0_LOCK_CFG = 16'h01E8,
  parameter  [15:0] GTHE3_COMMON_QPLL0_LOCK_CFG_G3 = 16'h01E8,
  parameter   [9:0] GTHE3_COMMON_QPLL0_LPF = 10'b1111111111,
  parameter   [9:0] GTHE3_COMMON_QPLL0_LPF_G3 = 10'b1111111111,
  parameter integer GTHE3_COMMON_QPLL0_REFCLK_DIV = 2,
  parameter  [15:0] GTHE3_COMMON_QPLL0_SDM_CFG0 = 16'b0000000001000000,
  parameter  [15:0] GTHE3_COMMON_QPLL0_SDM_CFG1 = 16'b0000000000000000,
  parameter  [15:0] GTHE3_COMMON_QPLL0_SDM_CFG2 = 16'b0000000000000000,
  parameter  [15:0] GTHE3_COMMON_QPLL1_CFG0 = 16'h3018,
  parameter  [15:0] GTHE3_COMMON_QPLL1_CFG1 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_QPLL1_CFG1_G3 = 16'h0020,
  parameter  [15:0] GTHE3_COMMON_QPLL1_CFG2 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_QPLL1_CFG2_G3 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_QPLL1_CFG3 = 16'h0120,
  parameter  [15:0] GTHE3_COMMON_QPLL1_CFG4 = 16'h0009,
  parameter   [9:0] GTHE3_COMMON_QPLL1_CP = 10'b0000011111,
  parameter   [9:0] GTHE3_COMMON_QPLL1_CP_G3 = 10'b0000011111,
  parameter integer GTHE3_COMMON_QPLL1_FBDIV = 66,
  parameter integer GTHE3_COMMON_QPLL1_FBDIV_G3 = 80,
  parameter  [15:0] GTHE3_COMMON_QPLL1_INIT_CFG0 = 16'h0000,
  parameter   [7:0] GTHE3_COMMON_QPLL1_INIT_CFG1 = 8'h00,
  parameter  [15:0] GTHE3_COMMON_QPLL1_LOCK_CFG = 16'h01E8,
  parameter  [15:0] GTHE3_COMMON_QPLL1_LOCK_CFG_G3 = 16'h21E8,
  parameter   [9:0] GTHE3_COMMON_QPLL1_LPF = 10'b1111111111,
  parameter   [9:0] GTHE3_COMMON_QPLL1_LPF_G3 = 10'b1111111111,
  parameter integer GTHE3_COMMON_QPLL1_REFCLK_DIV = 2,
  parameter  [15:0] GTHE3_COMMON_QPLL1_SDM_CFG0 = 16'b0000000001000000,
  parameter  [15:0] GTHE3_COMMON_QPLL1_SDM_CFG1 = 16'b0000000000000000,
  parameter  [15:0] GTHE3_COMMON_QPLL1_SDM_CFG2 = 16'b0000000000000000,
  parameter  [15:0] GTHE3_COMMON_RSVD_ATTR0 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_RSVD_ATTR1 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_RSVD_ATTR2 = 16'h0000,
  parameter  [15:0] GTHE3_COMMON_RSVD_ATTR3 = 16'h0000,
  parameter   [1:0] GTHE3_COMMON_RXRECCLKOUT0_SEL = 2'b00,
  parameter   [1:0] GTHE3_COMMON_RXRECCLKOUT1_SEL = 2'b00,
  parameter   [0:0] GTHE3_COMMON_SARC_EN = 1'b1,
  parameter   [0:0] GTHE3_COMMON_SARC_SEL = 1'b0,
  parameter  [15:0] GTHE3_COMMON_SDM0DATA1_0 = 16'b0000000000000000,
  parameter   [8:0] GTHE3_COMMON_SDM0DATA1_1 = 9'b000000000,
  parameter  [15:0] GTHE3_COMMON_SDM0INITSEED0_0 = 16'b0000000000000000,
  parameter   [8:0] GTHE3_COMMON_SDM0INITSEED0_1 = 9'b000000000,
  parameter   [0:0] GTHE3_COMMON_SDM0_DATA_PIN_SEL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_SDM0_WIDTH_PIN_SEL = 1'b0,
  parameter  [15:0] GTHE3_COMMON_SDM1DATA1_0 = 16'b0000000000000000,
  parameter   [8:0] GTHE3_COMMON_SDM1DATA1_1 = 9'b000000000,
  parameter  [15:0] GTHE3_COMMON_SDM1INITSEED0_0 = 16'b0000000000000000,
  parameter   [8:0] GTHE3_COMMON_SDM1INITSEED0_1 = 9'b000000000,
  parameter   [0:0] GTHE3_COMMON_SDM1_DATA_PIN_SEL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_SDM1_WIDTH_PIN_SEL = 1'b0,
  parameter         GTHE3_COMMON_SIM_RESET_SPEEDUP = "TRUE",
  parameter integer GTHE3_COMMON_SIM_VERSION = 2,

  // primitive wrapper parameters which specify GTHE3_COMMON primitive input port default driver values
  parameter   [0:0] GTHE3_COMMON_BGBYPASSB_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_BGMONITORENB_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_BGPDB_VAL = 1'b0,
  parameter   [4:0] GTHE3_COMMON_BGRCALOVRD_VAL = 5'b0,
  parameter   [0:0] GTHE3_COMMON_BGRCALOVRDENB_VAL = 1'b0,
  parameter   [8:0] GTHE3_COMMON_DRPADDR_VAL = 9'b0,
  parameter   [0:0] GTHE3_COMMON_DRPCLK_VAL = 1'b0,
  parameter  [15:0] GTHE3_COMMON_DRPDI_VAL = 16'b0,
  parameter   [0:0] GTHE3_COMMON_DRPEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_DRPWE_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTGREFCLK0_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTGREFCLK1_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTNORTHREFCLK00_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTNORTHREFCLK01_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTNORTHREFCLK10_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTNORTHREFCLK11_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTREFCLK00_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTREFCLK01_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTREFCLK10_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTREFCLK11_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTSOUTHREFCLK00_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTSOUTHREFCLK01_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTSOUTHREFCLK10_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTSOUTHREFCLK11_VAL = 1'b0,
  parameter   [7:0] GTHE3_COMMON_PMARSVD0_VAL = 8'b0,
  parameter   [7:0] GTHE3_COMMON_PMARSVD1_VAL = 8'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0CLKRSVD0_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0CLKRSVD1_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0LOCKDETCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0LOCKEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0PD_VAL = 1'b0,
  parameter   [2:0] GTHE3_COMMON_QPLL0REFCLKSEL_VAL = 3'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0RESET_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1CLKRSVD0_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1CLKRSVD1_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1LOCKDETCLK_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1LOCKEN_VAL = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1PD_VAL = 1'b0,
  parameter   [2:0] GTHE3_COMMON_QPLL1REFCLKSEL_VAL = 3'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1RESET_VAL = 1'b0,
  parameter   [7:0] GTHE3_COMMON_QPLLRSVD1_VAL = 8'b0,
  parameter   [4:0] GTHE3_COMMON_QPLLRSVD2_VAL = 5'b0,
  parameter   [4:0] GTHE3_COMMON_QPLLRSVD3_VAL = 5'b0,
  parameter   [7:0] GTHE3_COMMON_QPLLRSVD4_VAL = 8'b0,
  parameter   [0:0] GTHE3_COMMON_RCALENB_VAL = 1'b0,

  // primitive wrapper parameters which control GTHE3_COMMON primitive input port tie-off enablement
  parameter   [0:0] GTHE3_COMMON_BGBYPASSB_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_BGMONITORENB_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_BGPDB_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_BGRCALOVRD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_BGRCALOVRDENB_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_DRPADDR_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_DRPCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_DRPDI_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_DRPEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_DRPWE_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTGREFCLK0_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTGREFCLK1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTNORTHREFCLK00_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTNORTHREFCLK01_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTNORTHREFCLK10_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTNORTHREFCLK11_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTREFCLK00_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTREFCLK01_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTREFCLK10_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTREFCLK11_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTSOUTHREFCLK00_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTSOUTHREFCLK01_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTSOUTHREFCLK10_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_GTSOUTHREFCLK11_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_PMARSVD0_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_PMARSVD1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0CLKRSVD0_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0CLKRSVD1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0LOCKDETCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0LOCKEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0PD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0REFCLKSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL0RESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1CLKRSVD0_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1CLKRSVD1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1LOCKDETCLK_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1LOCKEN_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1PD_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1REFCLKSEL_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLL1RESET_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLLRSVD1_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLLRSVD2_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLLRSVD3_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_QPLLRSVD4_TIE_EN = 1'b0,
  parameter   [0:0] GTHE3_COMMON_RCALENB_TIE_EN = 1'b0

)(


  // -------------------------------------------------------------------------------------------------------------------
  // Ports relating to GTHE3_COMMON primitive
  // -------------------------------------------------------------------------------------------------------------------

  // primitive wrapper input ports which can drive corresponding GTHE3_COMMON primitive input ports
  input  wire [ 0:0] GTHE3_COMMON_BGBYPASSB,
  input  wire [ 0:0] GTHE3_COMMON_BGMONITORENB,
  input  wire [ 0:0] GTHE3_COMMON_BGPDB,
  input  wire [ 4:0] GTHE3_COMMON_BGRCALOVRD,
  input  wire [ 0:0] GTHE3_COMMON_BGRCALOVRDENB,
  input  wire [ 8:0] GTHE3_COMMON_DRPADDR,
  input  wire [ 0:0] GTHE3_COMMON_DRPCLK,
  input  wire [15:0] GTHE3_COMMON_DRPDI,
  input  wire [ 0:0] GTHE3_COMMON_DRPEN,
  input  wire [ 0:0] GTHE3_COMMON_DRPWE,
  input  wire [ 0:0] GTHE3_COMMON_GTGREFCLK0,
  input  wire [ 0:0] GTHE3_COMMON_GTGREFCLK1,
  input  wire [ 0:0] GTHE3_COMMON_GTNORTHREFCLK00,
  input  wire [ 0:0] GTHE3_COMMON_GTNORTHREFCLK01,
  input  wire [ 0:0] GTHE3_COMMON_GTNORTHREFCLK10,
  input  wire [ 0:0] GTHE3_COMMON_GTNORTHREFCLK11,
  input  wire [ 0:0] GTHE3_COMMON_GTREFCLK00,
  input  wire [ 0:0] GTHE3_COMMON_GTREFCLK01,
  input  wire [ 0:0] GTHE3_COMMON_GTREFCLK10,
  input  wire [ 0:0] GTHE3_COMMON_GTREFCLK11,
  input  wire [ 0:0] GTHE3_COMMON_GTSOUTHREFCLK00,
  input  wire [ 0:0] GTHE3_COMMON_GTSOUTHREFCLK01,
  input  wire [ 0:0] GTHE3_COMMON_GTSOUTHREFCLK10,
  input  wire [ 0:0] GTHE3_COMMON_GTSOUTHREFCLK11,
  input  wire [ 7:0] GTHE3_COMMON_PMARSVD0,
  input  wire [ 7:0] GTHE3_COMMON_PMARSVD1,
  input  wire [ 0:0] GTHE3_COMMON_QPLL0CLKRSVD0,
  input  wire [ 0:0] GTHE3_COMMON_QPLL0CLKRSVD1,
  input  wire [ 0:0] GTHE3_COMMON_QPLL0LOCKDETCLK,
  input  wire [ 0:0] GTHE3_COMMON_QPLL0LOCKEN,
  input  wire [ 0:0] GTHE3_COMMON_QPLL0PD,
  input  wire [ 2:0] GTHE3_COMMON_QPLL0REFCLKSEL,
  input  wire [ 0:0] GTHE3_COMMON_QPLL0RESET,
  input  wire [ 0:0] GTHE3_COMMON_QPLL1CLKRSVD0,
  input  wire [ 0:0] GTHE3_COMMON_QPLL1CLKRSVD1,
  input  wire [ 0:0] GTHE3_COMMON_QPLL1LOCKDETCLK,
  input  wire [ 0:0] GTHE3_COMMON_QPLL1LOCKEN,
  input  wire [ 0:0] GTHE3_COMMON_QPLL1PD,
  input  wire [ 2:0] GTHE3_COMMON_QPLL1REFCLKSEL,
  input  wire [ 0:0] GTHE3_COMMON_QPLL1RESET,
  input  wire [ 7:0] GTHE3_COMMON_QPLLRSVD1,
  input  wire [ 4:0] GTHE3_COMMON_QPLLRSVD2,
  input  wire [ 4:0] GTHE3_COMMON_QPLLRSVD3,
  input  wire [ 7:0] GTHE3_COMMON_QPLLRSVD4,
  input  wire [ 0:0] GTHE3_COMMON_RCALENB,

  // primitive wrapper output ports which are driven by corresponding GTHE3_COMMON primitive output ports
  output wire [15:0] GTHE3_COMMON_DRPDO,
  output wire [ 0:0] GTHE3_COMMON_DRPRDY,
  output wire [ 7:0] GTHE3_COMMON_PMARSVDOUT0,
  output wire [ 7:0] GTHE3_COMMON_PMARSVDOUT1,
  output wire [ 0:0] GTHE3_COMMON_QPLL0FBCLKLOST,
  output wire [ 0:0] GTHE3_COMMON_QPLL0LOCK,
  output wire [ 0:0] GTHE3_COMMON_QPLL0OUTCLK,
  output wire [ 0:0] GTHE3_COMMON_QPLL0OUTREFCLK,
  output wire [ 0:0] GTHE3_COMMON_QPLL0REFCLKLOST,
  output wire [ 0:0] GTHE3_COMMON_QPLL1FBCLKLOST,
  output wire [ 0:0] GTHE3_COMMON_QPLL1LOCK,
  output wire [ 0:0] GTHE3_COMMON_QPLL1OUTCLK,
  output wire [ 0:0] GTHE3_COMMON_QPLL1OUTREFCLK,
  output wire [ 0:0] GTHE3_COMMON_QPLL1REFCLKLOST,
  output wire [ 7:0] GTHE3_COMMON_QPLLDMONITOR0,
  output wire [ 7:0] GTHE3_COMMON_QPLLDMONITOR1,
  output wire [ 0:0] GTHE3_COMMON_REFCLKOUTMONITOR0,
  output wire [ 0:0] GTHE3_COMMON_REFCLKOUTMONITOR1,
  output wire [ 1:0] GTHE3_COMMON_RXRECCLK0_SEL,
  output wire [ 1:0] GTHE3_COMMON_RXRECCLK1_SEL

);


  // -------------------------------------------------------------------------------------------------------------------
  // HDL generation of wiring and instances relating to GTHE3_COMMON primitive
  // -------------------------------------------------------------------------------------------------------------------

  generate if (1) begin : gthe3_common_gen

    // for each GTHE3_COMMON primitive input port, declare a properly-sized vector
    wire [ 0:0] GTHE3_COMMON_BGBYPASSB_int;
    wire [ 0:0] GTHE3_COMMON_BGMONITORENB_int;
    wire [ 0:0] GTHE3_COMMON_BGPDB_int;
    wire [ 4:0] GTHE3_COMMON_BGRCALOVRD_int;
    wire [ 0:0] GTHE3_COMMON_BGRCALOVRDENB_int;
    wire [ 8:0] GTHE3_COMMON_DRPADDR_int;
    wire [ 0:0] GTHE3_COMMON_DRPCLK_int;
    wire [15:0] GTHE3_COMMON_DRPDI_int;
    wire [ 0:0] GTHE3_COMMON_DRPEN_int;
    wire [ 0:0] GTHE3_COMMON_DRPWE_int;
    wire [ 0:0] GTHE3_COMMON_GTGREFCLK0_int;
    wire [ 0:0] GTHE3_COMMON_GTGREFCLK1_int;
    wire [ 0:0] GTHE3_COMMON_GTNORTHREFCLK00_int;
    wire [ 0:0] GTHE3_COMMON_GTNORTHREFCLK01_int;
    wire [ 0:0] GTHE3_COMMON_GTNORTHREFCLK10_int;
    wire [ 0:0] GTHE3_COMMON_GTNORTHREFCLK11_int;
    wire [ 0:0] GTHE3_COMMON_GTREFCLK00_int;
    wire [ 0:0] GTHE3_COMMON_GTREFCLK01_int;
    wire [ 0:0] GTHE3_COMMON_GTREFCLK10_int;
    wire [ 0:0] GTHE3_COMMON_GTREFCLK11_int;
    wire [ 0:0] GTHE3_COMMON_GTSOUTHREFCLK00_int;
    wire [ 0:0] GTHE3_COMMON_GTSOUTHREFCLK01_int;
    wire [ 0:0] GTHE3_COMMON_GTSOUTHREFCLK10_int;
    wire [ 0:0] GTHE3_COMMON_GTSOUTHREFCLK11_int;
    wire [ 7:0] GTHE3_COMMON_PMARSVD0_int;
    wire [ 7:0] GTHE3_COMMON_PMARSVD1_int;
    wire [ 0:0] GTHE3_COMMON_QPLL0CLKRSVD0_int;
    wire [ 0:0] GTHE3_COMMON_QPLL0CLKRSVD1_int;
    wire [ 0:0] GTHE3_COMMON_QPLL0LOCKDETCLK_int;
    wire [ 0:0] GTHE3_COMMON_QPLL0LOCKEN_int;
    wire [ 0:0] GTHE3_COMMON_QPLL0PD_int;
    wire [ 2:0] GTHE3_COMMON_QPLL0REFCLKSEL_int;
    wire [ 0:0] GTHE3_COMMON_QPLL0RESET_int;
    wire [ 0:0] GTHE3_COMMON_QPLL1CLKRSVD0_int;
    wire [ 0:0] GTHE3_COMMON_QPLL1CLKRSVD1_int;
    wire [ 0:0] GTHE3_COMMON_QPLL1LOCKDETCLK_int;
    wire [ 0:0] GTHE3_COMMON_QPLL1LOCKEN_int;
    wire [ 0:0] GTHE3_COMMON_QPLL1PD_int;
    wire [ 2:0] GTHE3_COMMON_QPLL1REFCLKSEL_int;
    wire [ 0:0] GTHE3_COMMON_QPLL1RESET_int;
    wire [ 7:0] GTHE3_COMMON_QPLLRSVD1_int;
    wire [ 4:0] GTHE3_COMMON_QPLLRSVD2_int;
    wire [ 4:0] GTHE3_COMMON_QPLLRSVD3_int;
    wire [ 7:0] GTHE3_COMMON_QPLLRSVD4_int;
    wire [ 0:0] GTHE3_COMMON_RCALENB_int;

    // assign each vector either the corresponding tie-off value or the corresponding input port
    if (GTHE3_COMMON_BGBYPASSB_TIE_EN == 1'b1)
      assign GTHE3_COMMON_BGBYPASSB_int = GTHE3_COMMON_BGBYPASSB_VAL;
    else
      assign GTHE3_COMMON_BGBYPASSB_int = GTHE3_COMMON_BGBYPASSB;

    if (GTHE3_COMMON_BGMONITORENB_TIE_EN == 1'b1)
      assign GTHE3_COMMON_BGMONITORENB_int = GTHE3_COMMON_BGMONITORENB_VAL;
    else
      assign GTHE3_COMMON_BGMONITORENB_int = GTHE3_COMMON_BGMONITORENB;

    if (GTHE3_COMMON_BGPDB_TIE_EN == 1'b1)
      assign GTHE3_COMMON_BGPDB_int = GTHE3_COMMON_BGPDB_VAL;
    else
      assign GTHE3_COMMON_BGPDB_int = GTHE3_COMMON_BGPDB;

    if (GTHE3_COMMON_BGRCALOVRD_TIE_EN == 1'b1)
      assign GTHE3_COMMON_BGRCALOVRD_int = GTHE3_COMMON_BGRCALOVRD_VAL;
    else
      assign GTHE3_COMMON_BGRCALOVRD_int = GTHE3_COMMON_BGRCALOVRD;

    if (GTHE3_COMMON_BGRCALOVRDENB_TIE_EN == 1'b1)
      assign GTHE3_COMMON_BGRCALOVRDENB_int = GTHE3_COMMON_BGRCALOVRDENB_VAL;
    else
      assign GTHE3_COMMON_BGRCALOVRDENB_int = GTHE3_COMMON_BGRCALOVRDENB;

    if (GTHE3_COMMON_DRPADDR_TIE_EN == 1'b1)
      assign GTHE3_COMMON_DRPADDR_int = GTHE3_COMMON_DRPADDR_VAL;
    else
      assign GTHE3_COMMON_DRPADDR_int = GTHE3_COMMON_DRPADDR;

    if (GTHE3_COMMON_DRPCLK_TIE_EN == 1'b1)
      assign GTHE3_COMMON_DRPCLK_int = GTHE3_COMMON_DRPCLK_VAL;
    else
      assign GTHE3_COMMON_DRPCLK_int = GTHE3_COMMON_DRPCLK;

    if (GTHE3_COMMON_DRPDI_TIE_EN == 1'b1)
      assign GTHE3_COMMON_DRPDI_int = GTHE3_COMMON_DRPDI_VAL;
    else
      assign GTHE3_COMMON_DRPDI_int = GTHE3_COMMON_DRPDI;

    if (GTHE3_COMMON_DRPEN_TIE_EN == 1'b1)
      assign GTHE3_COMMON_DRPEN_int = GTHE3_COMMON_DRPEN_VAL;
    else
      assign GTHE3_COMMON_DRPEN_int = GTHE3_COMMON_DRPEN;

    if (GTHE3_COMMON_DRPWE_TIE_EN == 1'b1)
      assign GTHE3_COMMON_DRPWE_int = GTHE3_COMMON_DRPWE_VAL;
    else
      assign GTHE3_COMMON_DRPWE_int = GTHE3_COMMON_DRPWE;

    if (GTHE3_COMMON_GTGREFCLK0_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTGREFCLK0_int = GTHE3_COMMON_GTGREFCLK0_VAL;
    else
      assign GTHE3_COMMON_GTGREFCLK0_int = GTHE3_COMMON_GTGREFCLK0;

    if (GTHE3_COMMON_GTGREFCLK1_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTGREFCLK1_int = GTHE3_COMMON_GTGREFCLK1_VAL;
    else
      assign GTHE3_COMMON_GTGREFCLK1_int = GTHE3_COMMON_GTGREFCLK1;

    if (GTHE3_COMMON_GTNORTHREFCLK00_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTNORTHREFCLK00_int = GTHE3_COMMON_GTNORTHREFCLK00_VAL;
    else
      assign GTHE3_COMMON_GTNORTHREFCLK00_int = GTHE3_COMMON_GTNORTHREFCLK00;

    if (GTHE3_COMMON_GTNORTHREFCLK01_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTNORTHREFCLK01_int = GTHE3_COMMON_GTNORTHREFCLK01_VAL;
    else
      assign GTHE3_COMMON_GTNORTHREFCLK01_int = GTHE3_COMMON_GTNORTHREFCLK01;

    if (GTHE3_COMMON_GTNORTHREFCLK10_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTNORTHREFCLK10_int = GTHE3_COMMON_GTNORTHREFCLK10_VAL;
    else
      assign GTHE3_COMMON_GTNORTHREFCLK10_int = GTHE3_COMMON_GTNORTHREFCLK10;

    if (GTHE3_COMMON_GTNORTHREFCLK11_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTNORTHREFCLK11_int = GTHE3_COMMON_GTNORTHREFCLK11_VAL;
    else
      assign GTHE3_COMMON_GTNORTHREFCLK11_int = GTHE3_COMMON_GTNORTHREFCLK11;

    if (GTHE3_COMMON_GTREFCLK00_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTREFCLK00_int = GTHE3_COMMON_GTREFCLK00_VAL;
    else
      assign GTHE3_COMMON_GTREFCLK00_int = GTHE3_COMMON_GTREFCLK00;

    if (GTHE3_COMMON_GTREFCLK01_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTREFCLK01_int = GTHE3_COMMON_GTREFCLK01_VAL;
    else
      assign GTHE3_COMMON_GTREFCLK01_int = GTHE3_COMMON_GTREFCLK01;

    if (GTHE3_COMMON_GTREFCLK10_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTREFCLK10_int = GTHE3_COMMON_GTREFCLK10_VAL;
    else
      assign GTHE3_COMMON_GTREFCLK10_int = GTHE3_COMMON_GTREFCLK10;

    if (GTHE3_COMMON_GTREFCLK11_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTREFCLK11_int = GTHE3_COMMON_GTREFCLK11_VAL;
    else
      assign GTHE3_COMMON_GTREFCLK11_int = GTHE3_COMMON_GTREFCLK11;

    if (GTHE3_COMMON_GTSOUTHREFCLK00_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTSOUTHREFCLK00_int = GTHE3_COMMON_GTSOUTHREFCLK00_VAL;
    else
      assign GTHE3_COMMON_GTSOUTHREFCLK00_int = GTHE3_COMMON_GTSOUTHREFCLK00;

    if (GTHE3_COMMON_GTSOUTHREFCLK01_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTSOUTHREFCLK01_int = GTHE3_COMMON_GTSOUTHREFCLK01_VAL;
    else
      assign GTHE3_COMMON_GTSOUTHREFCLK01_int = GTHE3_COMMON_GTSOUTHREFCLK01;

    if (GTHE3_COMMON_GTSOUTHREFCLK10_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTSOUTHREFCLK10_int = GTHE3_COMMON_GTSOUTHREFCLK10_VAL;
    else
      assign GTHE3_COMMON_GTSOUTHREFCLK10_int = GTHE3_COMMON_GTSOUTHREFCLK10;

    if (GTHE3_COMMON_GTSOUTHREFCLK11_TIE_EN == 1'b1)
      assign GTHE3_COMMON_GTSOUTHREFCLK11_int = GTHE3_COMMON_GTSOUTHREFCLK11_VAL;
    else
      assign GTHE3_COMMON_GTSOUTHREFCLK11_int = GTHE3_COMMON_GTSOUTHREFCLK11;

    if (GTHE3_COMMON_PMARSVD0_TIE_EN == 1'b1)
      assign GTHE3_COMMON_PMARSVD0_int = GTHE3_COMMON_PMARSVD0_VAL;
    else
      assign GTHE3_COMMON_PMARSVD0_int = GTHE3_COMMON_PMARSVD0;

    if (GTHE3_COMMON_PMARSVD1_TIE_EN == 1'b1)
      assign GTHE3_COMMON_PMARSVD1_int = GTHE3_COMMON_PMARSVD1_VAL;
    else
      assign GTHE3_COMMON_PMARSVD1_int = GTHE3_COMMON_PMARSVD1;

    if (GTHE3_COMMON_QPLL0CLKRSVD0_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL0CLKRSVD0_int = GTHE3_COMMON_QPLL0CLKRSVD0_VAL;
    else
      assign GTHE3_COMMON_QPLL0CLKRSVD0_int = GTHE3_COMMON_QPLL0CLKRSVD0;

    if (GTHE3_COMMON_QPLL0CLKRSVD1_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL0CLKRSVD1_int = GTHE3_COMMON_QPLL0CLKRSVD1_VAL;
    else
      assign GTHE3_COMMON_QPLL0CLKRSVD1_int = GTHE3_COMMON_QPLL0CLKRSVD1;

    if (GTHE3_COMMON_QPLL0LOCKDETCLK_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL0LOCKDETCLK_int = GTHE3_COMMON_QPLL0LOCKDETCLK_VAL;
    else
      assign GTHE3_COMMON_QPLL0LOCKDETCLK_int = GTHE3_COMMON_QPLL0LOCKDETCLK;

    if (GTHE3_COMMON_QPLL0LOCKEN_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL0LOCKEN_int = GTHE3_COMMON_QPLL0LOCKEN_VAL;
    else
      assign GTHE3_COMMON_QPLL0LOCKEN_int = GTHE3_COMMON_QPLL0LOCKEN;

    if (GTHE3_COMMON_QPLL0PD_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL0PD_int = GTHE3_COMMON_QPLL0PD_VAL;
    else
      assign GTHE3_COMMON_QPLL0PD_int = GTHE3_COMMON_QPLL0PD;

    if (GTHE3_COMMON_QPLL0REFCLKSEL_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL0REFCLKSEL_int = GTHE3_COMMON_QPLL0REFCLKSEL_VAL;
    else
      assign GTHE3_COMMON_QPLL0REFCLKSEL_int = GTHE3_COMMON_QPLL0REFCLKSEL;

    if (GTHE3_COMMON_QPLL0RESET_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL0RESET_int = GTHE3_COMMON_QPLL0RESET_VAL;
    else
      assign GTHE3_COMMON_QPLL0RESET_int = GTHE3_COMMON_QPLL0RESET;

    if (GTHE3_COMMON_QPLL1CLKRSVD0_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL1CLKRSVD0_int = GTHE3_COMMON_QPLL1CLKRSVD0_VAL;
    else
      assign GTHE3_COMMON_QPLL1CLKRSVD0_int = GTHE3_COMMON_QPLL1CLKRSVD0;

    if (GTHE3_COMMON_QPLL1CLKRSVD1_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL1CLKRSVD1_int = GTHE3_COMMON_QPLL1CLKRSVD1_VAL;
    else
      assign GTHE3_COMMON_QPLL1CLKRSVD1_int = GTHE3_COMMON_QPLL1CLKRSVD1;

    if (GTHE3_COMMON_QPLL1LOCKDETCLK_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL1LOCKDETCLK_int = GTHE3_COMMON_QPLL1LOCKDETCLK_VAL;
    else
      assign GTHE3_COMMON_QPLL1LOCKDETCLK_int = GTHE3_COMMON_QPLL1LOCKDETCLK;

    if (GTHE3_COMMON_QPLL1LOCKEN_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL1LOCKEN_int = GTHE3_COMMON_QPLL1LOCKEN_VAL;
    else
      assign GTHE3_COMMON_QPLL1LOCKEN_int = GTHE3_COMMON_QPLL1LOCKEN;

    if (GTHE3_COMMON_QPLL1PD_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL1PD_int = GTHE3_COMMON_QPLL1PD_VAL;
    else
      assign GTHE3_COMMON_QPLL1PD_int = GTHE3_COMMON_QPLL1PD;

    if (GTHE3_COMMON_QPLL1REFCLKSEL_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL1REFCLKSEL_int = GTHE3_COMMON_QPLL1REFCLKSEL_VAL;
    else
      assign GTHE3_COMMON_QPLL1REFCLKSEL_int = GTHE3_COMMON_QPLL1REFCLKSEL;

    if (GTHE3_COMMON_QPLL1RESET_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLL1RESET_int = GTHE3_COMMON_QPLL1RESET_VAL;
    else
      assign GTHE3_COMMON_QPLL1RESET_int = GTHE3_COMMON_QPLL1RESET;

    if (GTHE3_COMMON_QPLLRSVD1_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLLRSVD1_int = GTHE3_COMMON_QPLLRSVD1_VAL;
    else
      assign GTHE3_COMMON_QPLLRSVD1_int = GTHE3_COMMON_QPLLRSVD1;

    if (GTHE3_COMMON_QPLLRSVD2_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLLRSVD2_int = GTHE3_COMMON_QPLLRSVD2_VAL;
    else
      assign GTHE3_COMMON_QPLLRSVD2_int = GTHE3_COMMON_QPLLRSVD2;

    if (GTHE3_COMMON_QPLLRSVD3_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLLRSVD3_int = GTHE3_COMMON_QPLLRSVD3_VAL;
    else
      assign GTHE3_COMMON_QPLLRSVD3_int = GTHE3_COMMON_QPLLRSVD3;

    if (GTHE3_COMMON_QPLLRSVD4_TIE_EN == 1'b1)
      assign GTHE3_COMMON_QPLLRSVD4_int = GTHE3_COMMON_QPLLRSVD4_VAL;
    else
      assign GTHE3_COMMON_QPLLRSVD4_int = GTHE3_COMMON_QPLLRSVD4;

    if (GTHE3_COMMON_RCALENB_TIE_EN == 1'b1)
      assign GTHE3_COMMON_RCALENB_int = GTHE3_COMMON_RCALENB_VAL;
    else
      assign GTHE3_COMMON_RCALENB_int = GTHE3_COMMON_RCALENB;

    // generate the GTHE3_COMMON primitive instance, mapping parameters and ports
    GTHE3_COMMON #(
      .BIAS_CFG0           (GTHE3_COMMON_BIAS_CFG0          ),
      .BIAS_CFG1           (GTHE3_COMMON_BIAS_CFG1          ),
      .BIAS_CFG2           (GTHE3_COMMON_BIAS_CFG2          ),
      .BIAS_CFG3           (GTHE3_COMMON_BIAS_CFG3          ),
      .BIAS_CFG4           (GTHE3_COMMON_BIAS_CFG4          ),
      .BIAS_CFG_RSVD       (GTHE3_COMMON_BIAS_CFG_RSVD      ),
      .COMMON_CFG0         (GTHE3_COMMON_COMMON_CFG0        ),
      .COMMON_CFG1         (GTHE3_COMMON_COMMON_CFG1        ),
      .POR_CFG             (GTHE3_COMMON_POR_CFG            ),
      .QPLL0_CFG0          (GTHE3_COMMON_QPLL0_CFG0         ),
      .QPLL0_CFG1          (GTHE3_COMMON_QPLL0_CFG1         ),
      .QPLL0_CFG1_G3       (GTHE3_COMMON_QPLL0_CFG1_G3      ),
      .QPLL0_CFG2          (GTHE3_COMMON_QPLL0_CFG2         ),
      .QPLL0_CFG2_G3       (GTHE3_COMMON_QPLL0_CFG2_G3      ),
      .QPLL0_CFG3          (GTHE3_COMMON_QPLL0_CFG3         ),
      .QPLL0_CFG4          (GTHE3_COMMON_QPLL0_CFG4         ),
      .QPLL0_CP            (GTHE3_COMMON_QPLL0_CP           ),
      .QPLL0_CP_G3         (GTHE3_COMMON_QPLL0_CP_G3        ),
      .QPLL0_FBDIV         (GTHE3_COMMON_QPLL0_FBDIV        ),
      .QPLL0_FBDIV_G3      (GTHE3_COMMON_QPLL0_FBDIV_G3     ),
      .QPLL0_INIT_CFG0     (GTHE3_COMMON_QPLL0_INIT_CFG0    ),
      .QPLL0_INIT_CFG1     (GTHE3_COMMON_QPLL0_INIT_CFG1    ),
      .QPLL0_LOCK_CFG      (GTHE3_COMMON_QPLL0_LOCK_CFG     ),
      .QPLL0_LOCK_CFG_G3   (GTHE3_COMMON_QPLL0_LOCK_CFG_G3  ),
      .QPLL0_LPF           (GTHE3_COMMON_QPLL0_LPF          ),
      .QPLL0_LPF_G3        (GTHE3_COMMON_QPLL0_LPF_G3       ),
      .QPLL0_REFCLK_DIV    (GTHE3_COMMON_QPLL0_REFCLK_DIV   ),
      .QPLL0_SDM_CFG0      (GTHE3_COMMON_QPLL0_SDM_CFG0     ),
      .QPLL0_SDM_CFG1      (GTHE3_COMMON_QPLL0_SDM_CFG1     ),
      .QPLL0_SDM_CFG2      (GTHE3_COMMON_QPLL0_SDM_CFG2     ),
      .QPLL1_CFG0          (GTHE3_COMMON_QPLL1_CFG0         ),
      .QPLL1_CFG1          (GTHE3_COMMON_QPLL1_CFG1         ),
      .QPLL1_CFG1_G3       (GTHE3_COMMON_QPLL1_CFG1_G3      ),
      .QPLL1_CFG2          (GTHE3_COMMON_QPLL1_CFG2         ),
      .QPLL1_CFG2_G3       (GTHE3_COMMON_QPLL1_CFG2_G3      ),
      .QPLL1_CFG3          (GTHE3_COMMON_QPLL1_CFG3         ),
      .QPLL1_CFG4          (GTHE3_COMMON_QPLL1_CFG4         ),
      .QPLL1_CP            (GTHE3_COMMON_QPLL1_CP           ),
      .QPLL1_CP_G3         (GTHE3_COMMON_QPLL1_CP_G3        ),
      .QPLL1_FBDIV         (GTHE3_COMMON_QPLL1_FBDIV        ),
      .QPLL1_FBDIV_G3      (GTHE3_COMMON_QPLL1_FBDIV_G3     ),
      .QPLL1_INIT_CFG0     (GTHE3_COMMON_QPLL1_INIT_CFG0    ),
      .QPLL1_INIT_CFG1     (GTHE3_COMMON_QPLL1_INIT_CFG1    ),
      .QPLL1_LOCK_CFG      (GTHE3_COMMON_QPLL1_LOCK_CFG     ),
      .QPLL1_LOCK_CFG_G3   (GTHE3_COMMON_QPLL1_LOCK_CFG_G3  ),
      .QPLL1_LPF           (GTHE3_COMMON_QPLL1_LPF          ),
      .QPLL1_LPF_G3        (GTHE3_COMMON_QPLL1_LPF_G3       ),
      .QPLL1_REFCLK_DIV    (GTHE3_COMMON_QPLL1_REFCLK_DIV   ),
      .QPLL1_SDM_CFG0      (GTHE3_COMMON_QPLL1_SDM_CFG0     ),
      .QPLL1_SDM_CFG1      (GTHE3_COMMON_QPLL1_SDM_CFG1     ),
      .QPLL1_SDM_CFG2      (GTHE3_COMMON_QPLL1_SDM_CFG2     ),
      .RSVD_ATTR0          (GTHE3_COMMON_RSVD_ATTR0         ),
      .RSVD_ATTR1          (GTHE3_COMMON_RSVD_ATTR1         ),
      .RSVD_ATTR2          (GTHE3_COMMON_RSVD_ATTR2         ),
      .RSVD_ATTR3          (GTHE3_COMMON_RSVD_ATTR3         ),
      .RXRECCLKOUT0_SEL    (GTHE3_COMMON_RXRECCLKOUT0_SEL   ),
      .RXRECCLKOUT1_SEL    (GTHE3_COMMON_RXRECCLKOUT1_SEL   ),
      .SARC_EN             (GTHE3_COMMON_SARC_EN            ),
      .SARC_SEL            (GTHE3_COMMON_SARC_SEL           ),
      .SDM0DATA1_0         (GTHE3_COMMON_SDM0DATA1_0        ),
      .SDM0DATA1_1         (GTHE3_COMMON_SDM0DATA1_1        ),
      .SDM0INITSEED0_0     (GTHE3_COMMON_SDM0INITSEED0_0    ),
      .SDM0INITSEED0_1     (GTHE3_COMMON_SDM0INITSEED0_1    ),
      .SDM0_DATA_PIN_SEL   (GTHE3_COMMON_SDM0_DATA_PIN_SEL  ),
      .SDM0_WIDTH_PIN_SEL  (GTHE3_COMMON_SDM0_WIDTH_PIN_SEL ),
      .SDM1DATA1_0         (GTHE3_COMMON_SDM1DATA1_0        ),
      .SDM1DATA1_1         (GTHE3_COMMON_SDM1DATA1_1        ),
      .SDM1INITSEED0_0     (GTHE3_COMMON_SDM1INITSEED0_0    ),
      .SDM1INITSEED0_1     (GTHE3_COMMON_SDM1INITSEED0_1    ),
      .SDM1_DATA_PIN_SEL   (GTHE3_COMMON_SDM1_DATA_PIN_SEL  ),
      .SDM1_WIDTH_PIN_SEL  (GTHE3_COMMON_SDM1_WIDTH_PIN_SEL ),
      .SIM_RESET_SPEEDUP   (GTHE3_COMMON_SIM_RESET_SPEEDUP  ),
      .SIM_VERSION         (GTHE3_COMMON_SIM_VERSION        )
    ) GTHE3_COMMON_PRIM_INST (
      .BGBYPASSB         (GTHE3_COMMON_BGBYPASSB_int         [ 0:0]),
      .BGMONITORENB      (GTHE3_COMMON_BGMONITORENB_int      [ 0:0]),
      .BGPDB             (GTHE3_COMMON_BGPDB_int             [ 0:0]),
      .BGRCALOVRD        (GTHE3_COMMON_BGRCALOVRD_int        [ 4:0]),
      .BGRCALOVRDENB     (GTHE3_COMMON_BGRCALOVRDENB_int     [ 0:0]),
      .DRPADDR           (GTHE3_COMMON_DRPADDR_int           [ 8:0]),
      .DRPCLK            (GTHE3_COMMON_DRPCLK_int            [ 0:0]),
      .DRPDI             (GTHE3_COMMON_DRPDI_int             [15:0]),
      .DRPEN             (GTHE3_COMMON_DRPEN_int             [ 0:0]),
      .DRPWE             (GTHE3_COMMON_DRPWE_int             [ 0:0]),
      .GTGREFCLK0        (GTHE3_COMMON_GTGREFCLK0_int        [ 0:0]),
      .GTGREFCLK1        (GTHE3_COMMON_GTGREFCLK1_int        [ 0:0]),
      .GTNORTHREFCLK00   (GTHE3_COMMON_GTNORTHREFCLK00_int   [ 0:0]),
      .GTNORTHREFCLK01   (GTHE3_COMMON_GTNORTHREFCLK01_int   [ 0:0]),
      .GTNORTHREFCLK10   (GTHE3_COMMON_GTNORTHREFCLK10_int   [ 0:0]),
      .GTNORTHREFCLK11   (GTHE3_COMMON_GTNORTHREFCLK11_int   [ 0:0]),
      .GTREFCLK00        (GTHE3_COMMON_GTREFCLK00_int        [ 0:0]),
      .GTREFCLK01        (GTHE3_COMMON_GTREFCLK01_int        [ 0:0]),
      .GTREFCLK10        (GTHE3_COMMON_GTREFCLK10_int        [ 0:0]),
      .GTREFCLK11        (GTHE3_COMMON_GTREFCLK11_int        [ 0:0]),
      .GTSOUTHREFCLK00   (GTHE3_COMMON_GTSOUTHREFCLK00_int   [ 0:0]),
      .GTSOUTHREFCLK01   (GTHE3_COMMON_GTSOUTHREFCLK01_int   [ 0:0]),
      .GTSOUTHREFCLK10   (GTHE3_COMMON_GTSOUTHREFCLK10_int   [ 0:0]),
      .GTSOUTHREFCLK11   (GTHE3_COMMON_GTSOUTHREFCLK11_int   [ 0:0]),
      .PMARSVD0          (GTHE3_COMMON_PMARSVD0_int          [ 7:0]),
      .PMARSVD1          (GTHE3_COMMON_PMARSVD1_int          [ 7:0]),
      .QPLL0CLKRSVD0     (GTHE3_COMMON_QPLL0CLKRSVD0_int     [ 0:0]),
      .QPLL0CLKRSVD1     (GTHE3_COMMON_QPLL0CLKRSVD1_int     [ 0:0]),
      .QPLL0LOCKDETCLK   (GTHE3_COMMON_QPLL0LOCKDETCLK_int   [ 0:0]),
      .QPLL0LOCKEN       (GTHE3_COMMON_QPLL0LOCKEN_int       [ 0:0]),
      .QPLL0PD           (GTHE3_COMMON_QPLL0PD_int           [ 0:0]),
      .QPLL0REFCLKSEL    (GTHE3_COMMON_QPLL0REFCLKSEL_int    [ 2:0]),
      .QPLL0RESET        (GTHE3_COMMON_QPLL0RESET_int        [ 0:0]),
      .QPLL1CLKRSVD0     (GTHE3_COMMON_QPLL1CLKRSVD0_int     [ 0:0]),
      .QPLL1CLKRSVD1     (GTHE3_COMMON_QPLL1CLKRSVD1_int     [ 0:0]),
      .QPLL1LOCKDETCLK   (GTHE3_COMMON_QPLL1LOCKDETCLK_int   [ 0:0]),
      .QPLL1LOCKEN       (GTHE3_COMMON_QPLL1LOCKEN_int       [ 0:0]),
      .QPLL1PD           (GTHE3_COMMON_QPLL1PD_int           [ 0:0]),
      .QPLL1REFCLKSEL    (GTHE3_COMMON_QPLL1REFCLKSEL_int    [ 2:0]),
      .QPLL1RESET        (GTHE3_COMMON_QPLL1RESET_int        [ 0:0]),
      .QPLLRSVD1         (GTHE3_COMMON_QPLLRSVD1_int         [ 7:0]),
      .QPLLRSVD2         (GTHE3_COMMON_QPLLRSVD2_int         [ 4:0]),
      .QPLLRSVD3         (GTHE3_COMMON_QPLLRSVD3_int         [ 4:0]),
      .QPLLRSVD4         (GTHE3_COMMON_QPLLRSVD4_int         [ 7:0]),
      .RCALENB           (GTHE3_COMMON_RCALENB_int           [ 0:0]),

      .DRPDO             (GTHE3_COMMON_DRPDO                 [15:0]),
      .DRPRDY            (GTHE3_COMMON_DRPRDY                [ 0:0]),
      .PMARSVDOUT0       (GTHE3_COMMON_PMARSVDOUT0           [ 7:0]),
      .PMARSVDOUT1       (GTHE3_COMMON_PMARSVDOUT1           [ 7:0]),
      .QPLL0FBCLKLOST    (GTHE3_COMMON_QPLL0FBCLKLOST        [ 0:0]),
      .QPLL0LOCK         (GTHE3_COMMON_QPLL0LOCK             [ 0:0]),
      .QPLL0OUTCLK       (GTHE3_COMMON_QPLL0OUTCLK           [ 0:0]),
      .QPLL0OUTREFCLK    (GTHE3_COMMON_QPLL0OUTREFCLK        [ 0:0]),
      .QPLL0REFCLKLOST   (GTHE3_COMMON_QPLL0REFCLKLOST       [ 0:0]),
      .QPLL1FBCLKLOST    (GTHE3_COMMON_QPLL1FBCLKLOST        [ 0:0]),
      .QPLL1LOCK         (GTHE3_COMMON_QPLL1LOCK             [ 0:0]),
      .QPLL1OUTCLK       (GTHE3_COMMON_QPLL1OUTCLK           [ 0:0]),
      .QPLL1OUTREFCLK    (GTHE3_COMMON_QPLL1OUTREFCLK        [ 0:0]),
      .QPLL1REFCLKLOST   (GTHE3_COMMON_QPLL1REFCLKLOST       [ 0:0]),
      .QPLLDMONITOR0     (GTHE3_COMMON_QPLLDMONITOR0         [ 7:0]),
      .QPLLDMONITOR1     (GTHE3_COMMON_QPLLDMONITOR1         [ 7:0]),
      .REFCLKOUTMONITOR0 (GTHE3_COMMON_REFCLKOUTMONITOR0     [ 0:0]),
      .REFCLKOUTMONITOR1 (GTHE3_COMMON_REFCLKOUTMONITOR1     [ 0:0]),
      .RXRECCLK0_SEL     (GTHE3_COMMON_RXRECCLK0_SEL         [ 1:0]),
      .RXRECCLK1_SEL     (GTHE3_COMMON_RXRECCLK1_SEL         [ 1:0])

    );

  end
  endgenerate


endmodule

module fec_gen(clk,reset,clr,din,fec);
  parameter DW = 64;

  input clk,reset,clr;
  input [DW-1:0] din;
  output [31:0] fec;

  function [31:0] crc32x64;
    input [63:0] Data;
    input [31:0] crc;
    reg [63:0] d;
    reg [31:0] c;
    reg [31:0] newcrc;
  begin
    d = Data;
    c = crc;

    newcrc[0] = d[62] ^ d[60] ^ d[58] ^ d[57] ^ d[55] ^ d[52] ^ d[51] ^ d[50] ^ d[48] ^ d[47] ^ 
                d[45] ^ d[44] ^ d[43] ^ d[42] ^ d[39] ^ d[36] ^ d[33] ^ d[32] ^ d[31] ^ d[30] ^ 
                d[29] ^ d[27] ^ d[22] ^ d[21] ^ d[18] ^ d[11] ^ d[9] ^ d[0] ^ 
                c[0] ^ c[1] ^ c[4] ^ c[7] ^ c[10] ^ c[11] ^ c[12] ^ c[13] ^ c[15] ^ c[16] ^ c[18] ^ c[19] ^ c[20] ^ c[23] ^ c[25] ^ c[26] ^ c[28] ^ c[30];
    newcrc[1] = d[63] ^ d[61] ^ d[59] ^ d[58] ^ d[56] ^ d[53] ^ d[52] ^ d[51] ^ d[49] ^ d[48] ^ 
                d[46] ^ d[45] ^ d[44] ^ d[43] ^ d[40] ^ d[37] ^ d[34] ^ d[33] ^ d[32] ^ d[31] ^ 
                d[30] ^ d[28] ^ d[23] ^ d[22] ^ d[19] ^ d[12] ^ d[10] ^ d[1] ^ 
                c[0] ^ c[1] ^ c[2] ^ c[5] ^ c[8] ^ c[11] ^ c[12] ^ c[13] ^ c[14] ^ c[16] ^ c[17] ^ c[19] ^ c[20] ^ c[21] ^ c[24] ^ c[26] ^ c[27] ^ c[29] ^ c[31];
    newcrc[2] = d[59] ^ d[58] ^ d[55] ^ d[54] ^ d[53] ^ d[51] ^ d[49] ^ d[48] ^ d[46] ^ d[43] ^ 
                d[42] ^ d[41] ^ d[39] ^ d[38] ^ d[36] ^ d[35] ^ d[34] ^ d[30] ^ d[27] ^ d[24] ^ 
                d[23] ^ d[22] ^ d[21] ^ d[20] ^ d[18] ^ d[13] ^ d[9] ^ d[2] ^ d[0] ^ 
                c[2] ^ c[3] ^ c[4] ^ c[6] ^ c[7] ^ c[9] ^ c[10] ^ c[11] ^ c[14] ^ c[16] ^ c[17] ^ c[19] ^ c[21] ^ c[22] ^ c[23] ^ c[26] ^ c[27];
    newcrc[3] = d[60] ^ d[59] ^ d[56] ^ d[55] ^ d[54] ^ d[52] ^ d[50] ^ d[49] ^ d[47] ^ d[44] ^
                d[43] ^ d[42] ^ d[40] ^ d[39] ^ d[37] ^ d[36] ^ d[35] ^ d[31] ^ d[28] ^ d[25] ^
                d[24] ^ d[23] ^ d[22] ^ d[21] ^ d[19] ^ d[14] ^ d[10] ^ d[3] ^ d[1] ^
                c[3] ^ c[4] ^ c[5] ^ c[7] ^ c[8] ^ c[10] ^ c[11] ^ c[12] ^ c[15] ^ c[17] ^ c[18] ^ c[20] ^ c[22] ^ c[23] ^ c[24] ^ c[27] ^ c[28];
    newcrc[4] = d[61] ^ d[60] ^ d[57] ^ d[56] ^ d[55] ^ d[53] ^ d[51] ^ d[50] ^ d[48] ^ d[45] ^
                d[44] ^ d[43] ^ d[41] ^ d[40] ^ d[38] ^ d[37] ^ d[36] ^ d[32] ^ d[29] ^ d[26] ^
                d[25] ^ d[24] ^ d[23] ^ d[22] ^ d[20] ^ d[15] ^ d[11] ^ d[4] ^ d[2] ^
                c[0] ^ c[4] ^ c[5] ^ c[6] ^ c[8] ^ c[9] ^ c[11] ^ c[12] ^ c[13] ^ c[16] ^ c[18] ^ c[19] ^ c[21] ^ c[23] ^ c[24] ^ c[25] ^ c[28] ^ c[29];
    newcrc[5] = d[62] ^ d[61] ^ d[58] ^ d[57] ^ d[56] ^ d[54] ^ d[52] ^ d[51] ^ d[49] ^ d[46] ^
                d[45] ^ d[44] ^ d[42] ^ d[41] ^ d[39] ^ d[38] ^ d[37] ^ d[33] ^ d[30] ^ d[27] ^
                d[26] ^ d[25] ^ d[24] ^ d[23] ^ d[21] ^ d[16] ^ d[12] ^ d[5] ^ d[3] ^
                c[1] ^ c[5] ^ c[6] ^ c[7] ^ c[9] ^ c[10] ^ c[12] ^ c[13] ^ c[14] ^ c[17] ^ c[19] ^ c[20] ^ c[22] ^ c[24] ^ c[25] ^ c[26] ^ c[29] ^ c[30];
    newcrc[6] = d[63] ^ d[62] ^ d[59] ^ d[58] ^ d[57] ^ d[55] ^ d[53] ^ d[52] ^ d[50] ^ d[47] ^
                d[46] ^ d[45] ^ d[43] ^ d[42] ^ d[40] ^ d[39] ^ d[38] ^ d[34] ^ d[31] ^ d[28] ^
                d[27] ^ d[26] ^ d[25] ^ d[24] ^ d[22] ^ d[17] ^ d[13] ^ d[6] ^ d[4] ^
                c[2] ^ c[6] ^ c[7] ^ c[8] ^ c[10] ^ c[11] ^ c[13] ^ c[14] ^ c[15] ^ c[18] ^ c[20] ^ c[21] ^ c[23] ^ c[25] ^ c[26] ^ c[27] ^ c[30] ^ c[31];
    newcrc[7] = d[63] ^ d[60] ^ d[59] ^ d[58] ^ d[56] ^ d[54] ^ d[53] ^ d[51] ^ d[48] ^ d[47] ^
                d[46] ^ d[44] ^ d[43] ^ d[41] ^ d[40] ^ d[39] ^ d[35] ^ d[32] ^ d[29] ^ d[28] ^
                d[27] ^ d[26] ^ d[25] ^ d[23] ^ d[18] ^ d[14] ^ d[7] ^ d[5] ^
                c[0] ^ c[3] ^ c[7] ^ c[8] ^ c[9] ^ c[11] ^ c[12] ^ c[14] ^ c[15] ^ c[16] ^ c[19] ^ c[21] ^ c[22] ^ c[24] ^ c[26] ^ c[27] ^ c[28] ^ c[31];
    newcrc[8] = d[61] ^ d[60] ^ d[59] ^ d[57] ^ d[55] ^ d[54] ^ d[52] ^ d[49] ^ d[48] ^ d[47] ^
                d[45] ^ d[44] ^ d[42] ^ d[41] ^ d[40] ^ d[36] ^ d[33] ^ d[30] ^ d[29] ^ d[28] ^
                d[27] ^ d[26] ^ d[24] ^ d[19] ^ d[15] ^ d[8] ^ d[6] ^
                c[1] ^ c[4] ^ c[8] ^ c[9] ^ c[10] ^ c[12] ^ c[13] ^ c[15] ^ c[16] ^ c[17] ^ c[20] ^ c[22] ^ c[23] ^ c[25] ^ c[27] ^ c[28] ^ c[29];
    newcrc[9] = d[62] ^ d[61] ^ d[60] ^ d[58] ^ d[56] ^ d[55] ^ d[53] ^ d[50] ^ d[49] ^ d[48] ^
                d[46] ^ d[45] ^ d[43] ^ d[42] ^ d[41] ^ d[37] ^ d[34] ^ d[31] ^ d[30] ^ d[29] ^
                d[28] ^ d[27] ^ d[25] ^ d[20] ^ d[16] ^ d[9] ^ d[7] ^
                c[2] ^ c[5] ^ c[9] ^ c[10] ^ c[11] ^ c[13] ^ c[14] ^ c[16] ^ c[17] ^ c[18] ^ c[21] ^ c[23] ^ c[24] ^ c[26] ^ c[28] ^ c[29] ^ c[30];
    newcrc[10] = d[63] ^ d[62] ^ d[61] ^ d[59] ^ d[57] ^ d[56] ^ d[54] ^ d[51] ^ d[50] ^ d[49] ^
                 d[47] ^ d[46] ^ d[44] ^ d[43] ^ d[42] ^ d[38] ^ d[35] ^ d[32] ^ d[31] ^ d[30] ^
                 d[29] ^ d[28] ^ d[26] ^ d[21] ^ d[17] ^ d[10] ^ d[8] ^
                 c[0] ^ c[3] ^ c[6] ^ c[10] ^ c[11] ^ c[12] ^ c[14] ^ c[15] ^ c[17] ^ c[18] ^ c[19] ^ c[22] ^ c[24] ^ c[25] ^ c[27] ^ c[29] ^ c[30] ^ c[31];
    newcrc[11] = d[63] ^ d[42] ^ d[21] ^ d[0] ^ c[10] ^ c[31];
    newcrc[12] = d[43] ^ d[22] ^ d[1] ^ c[11];
    newcrc[13] = d[44] ^ d[23] ^ d[2] ^ c[12];
    newcrc[14] = d[45] ^ d[24] ^ d[3] ^ c[13];
    newcrc[15] = d[46] ^ d[25] ^ d[4] ^ c[14];
    newcrc[16] = d[47] ^ d[26] ^ d[5] ^ c[15];
    newcrc[17] = d[48] ^ d[27] ^ d[6] ^ c[16];
    newcrc[18] = d[49] ^ d[28] ^ d[7] ^ c[17];
    newcrc[19] = d[50] ^ d[29] ^ d[8] ^ c[18];
    newcrc[20] = d[51] ^ d[30] ^ d[9] ^ c[19];
    newcrc[21] = d[62] ^ d[60] ^ d[58] ^ d[57] ^ d[55] ^ d[51] ^ d[50] ^ d[48] ^ d[47] ^ d[45] ^
                 d[44] ^ d[43] ^ d[42] ^ d[39] ^ d[36] ^ d[33] ^ d[32] ^ d[30] ^ d[29] ^ d[27] ^
                 d[22] ^ d[21] ^ d[18] ^ d[11] ^ d[10] ^ d[9] ^ d[0] ^
                 c[0] ^ c[1] ^ c[4] ^ c[7] ^ c[10] ^ c[11] ^ c[12] ^ c[13] ^ c[15] ^ c[16] ^ c[18] ^ c[19] ^ c[23] ^ c[25] ^ c[26] ^ c[28] ^ c[30];
    newcrc[22] = d[63] ^ d[61] ^ d[59] ^ d[58] ^ d[56] ^ d[52] ^ d[51] ^ d[49] ^ d[48] ^ d[46] ^
                 d[45] ^ d[44] ^ d[43] ^ d[40] ^ d[37] ^ d[34] ^ d[33] ^ d[31] ^ d[30] ^ d[28] ^ 
                 d[23] ^ d[22] ^ d[19] ^ d[12] ^ d[11] ^ d[10] ^ d[1] ^
                 c[1] ^ c[2] ^ c[5] ^ c[8] ^ c[11] ^ c[12] ^ c[13] ^ c[14] ^ c[16] ^ c[17] ^ c[19] ^ c[20] ^ c[24] ^ c[26] ^ c[27] ^ c[29] ^ c[31];
    newcrc[23] = d[59] ^ d[58] ^ d[55] ^ d[53] ^ d[51] ^ d[49] ^ d[48] ^ d[46] ^ d[43] ^ d[42] ^
                 d[41] ^ d[39] ^ d[38] ^ d[36] ^ d[35] ^ d[34] ^ d[33] ^ d[30] ^ d[27] ^ d[24] ^
                 d[23] ^ d[22] ^ d[21] ^ d[20] ^ d[18] ^ d[13] ^ d[12] ^ d[9] ^ d[2] ^ d[0] ^
                 c[1] ^ c[2] ^ c[3] ^ c[4] ^ c[6] ^ c[7] ^ c[9] ^ c[10] ^ c[11] ^ c[14] ^ c[16] ^ c[17] ^ c[19] ^ c[21] ^ c[23] ^ c[26] ^ c[27];
    newcrc[24] = d[60] ^ d[59] ^ d[56] ^ d[54] ^ d[52] ^ d[50] ^ d[49] ^ d[47] ^ d[44] ^ d[43] ^
                 d[42] ^ d[40] ^ d[39] ^ d[37] ^ d[36] ^ d[35] ^ d[34] ^ d[31] ^ d[28] ^ d[25] ^
                 d[24] ^ d[23] ^ d[22] ^ d[21] ^ d[19] ^ d[14] ^ d[13] ^ d[10] ^ d[3] ^ d[1] ^ 
                 c[2] ^ c[3] ^ c[4] ^ c[5] ^ c[7] ^ c[8] ^ c[10] ^ c[11] ^ c[12] ^ c[15] ^ c[17] ^ c[18] ^ c[20] ^ c[22] ^ c[24] ^ c[27] ^ c[28];
    newcrc[25] = d[61] ^ d[60] ^ d[57] ^ d[55] ^ d[53] ^ d[51] ^ d[50] ^ d[48] ^ d[45] ^ d[44] ^
                 d[43] ^ d[41] ^ d[40] ^ d[38] ^ d[37] ^ d[36] ^ d[35] ^ d[32] ^ d[29] ^ d[26] ^
                 d[25] ^ d[24] ^ d[23] ^ d[22] ^ d[20] ^ d[15] ^ d[14] ^ d[11] ^ d[4] ^ d[2] ^
                 c[0] ^ c[3] ^ c[4] ^ c[5] ^ c[6] ^ c[8] ^ c[9] ^ c[11] ^ c[12] ^ c[13] ^ c[16] ^ c[18] ^ c[19] ^ c[21] ^ c[23] ^ c[25] ^ c[28] ^ c[29];
    newcrc[26] = d[62] ^ d[61] ^ d[58] ^ d[56] ^ d[54] ^ d[52] ^ d[51] ^ d[49] ^ d[46] ^ d[45] ^ 
                 d[44] ^ d[42] ^ d[41] ^ d[39] ^ d[38] ^ d[37] ^ d[36] ^ d[33] ^ d[30] ^ d[27] ^ 
                 d[26] ^ d[25] ^ d[24] ^ d[23] ^ d[21] ^ d[16] ^ d[15] ^ d[12] ^ d[5] ^ d[3] ^ 
                 c[1] ^ c[4] ^ c[5] ^ c[6] ^ c[7] ^ c[9] ^ c[10] ^ c[12] ^ c[13] ^ c[14] ^ c[17] ^ c[19] ^ c[20] ^ c[22] ^ c[24] ^ c[26] ^ c[29] ^ c[30];
    newcrc[27] = d[63] ^ d[62] ^ d[59] ^ d[57] ^ d[55] ^ d[53] ^ d[52] ^ d[50] ^ d[47] ^ d[46] ^ 
                 d[45] ^ d[43] ^ d[42] ^ d[40] ^ d[39] ^ d[38] ^ d[37] ^ d[34] ^ d[31] ^ d[28] ^ 
                 d[27] ^ d[26] ^ d[25] ^ d[24] ^ d[22] ^ d[17] ^ d[16] ^ d[13] ^ d[6] ^ d[4] ^ 
                 c[2] ^ c[5] ^ c[6] ^ c[7] ^ c[8] ^ c[10] ^ c[11] ^ c[13] ^ c[14] ^ c[15] ^ c[18] ^ c[20] ^ c[21] ^ c[23] ^ c[25] ^ c[27] ^ c[30] ^ c[31];
    newcrc[28] = d[63] ^ d[60] ^ d[58] ^ d[56] ^ d[54] ^ d[53] ^ d[51] ^ d[48] ^ d[47] ^ d[46] ^ 
                 d[44] ^ d[43] ^ d[41] ^ d[40] ^ d[39] ^ d[38] ^ d[35] ^ d[32] ^ d[29] ^ d[28] ^ 
                 d[27] ^ d[26] ^ d[25] ^ d[23] ^ d[18] ^ d[17] ^ d[14] ^ d[7] ^ d[5] ^
                 c[0] ^ c[3] ^ c[6] ^ c[7] ^ c[8] ^ c[9] ^ c[11] ^ c[12] ^ c[14] ^ c[15] ^ c[16] ^ c[19] ^ c[21] ^ c[22] ^ c[24] ^ c[26] ^ c[28] ^ c[31];
    newcrc[29] = d[61] ^ d[59] ^ d[57] ^ d[55] ^ d[54] ^ d[52] ^ d[49] ^ d[48] ^ d[47] ^ d[45] ^ 
                 d[44] ^ d[42] ^ d[41] ^ d[40] ^ d[39] ^ d[36] ^ d[33] ^ d[30] ^ d[29] ^ d[28] ^ 
                 d[27] ^ d[26] ^ d[24] ^ d[19] ^ d[18] ^ d[15] ^ d[8] ^ d[6] ^ 
                 c[1] ^ c[4] ^ c[7] ^ c[8] ^ c[9] ^ c[10] ^ c[12] ^ c[13] ^ c[15] ^ c[16] ^ c[17] ^ c[20] ^ c[22] ^ c[23] ^ c[25] ^ c[27] ^ c[29];
    newcrc[30] = d[62] ^ d[60] ^ d[58] ^ d[56] ^ d[55] ^ d[53] ^ d[50] ^ d[49] ^ d[48] ^ d[46] ^ 
                 d[45] ^ d[43] ^ d[42] ^ d[41] ^ d[40] ^ d[37] ^ d[34] ^ d[31] ^ d[30] ^ d[29] ^ 
                 d[28] ^ d[27] ^ d[25] ^ d[20] ^ d[19] ^ d[16] ^ d[9] ^ d[7] ^ 
                 c[2] ^ c[5] ^ c[8] ^ c[9] ^ c[10] ^ c[11] ^ c[13] ^ c[14] ^ c[16] ^ c[17] ^ c[18] ^ c[21] ^ c[23] ^ c[24] ^ c[26] ^ c[28] ^ c[30];
    newcrc[31] = d[63] ^ d[61] ^ d[59] ^ d[57] ^ d[56] ^ d[54] ^ d[51] ^ d[50] ^ d[49] ^ d[47] ^ 
                 d[46] ^ d[44] ^ d[43] ^ d[42] ^ d[41] ^ d[38] ^ d[35] ^ d[32] ^ d[31] ^ d[30] ^ 
                 d[29] ^ d[28] ^ d[26] ^ d[21] ^ d[20] ^ d[17] ^ d[10] ^ d[8] ^ 
                 c[0] ^ c[3] ^ c[6] ^ c[9] ^ c[10] ^ c[11] ^ c[12] ^ c[14] ^ c[15] ^ c[17] ^ c[18] ^ c[19] ^ c[22] ^ c[24] ^ c[25] ^ c[27] ^ c[29] ^ c[31];
    crc32x64 = newcrc;
  end
  endfunction

  wire [31:0] crc_i;
  reg [DW-1:0] dind;
  reg [31:0] crcd,crcp,crcpp,crcppp;
  reg clrd;
  always@(posedge clk) begin
    if(reset) begin
      dind <= 0;
      crcd <= 0;
    end
    else begin
      clrd <= clr;
      dind <= din;
      crcd <= (clr)? 0 : crc_i;
      crcp <= crc_i; 
      crcpp <= crcp;
      crcppp <= crcpp;
    end
  end

  assign crc_i = crc32x64(dind,crcd);

  assign fec = crcppp;

endmodule

module fec_decode(clk,rst, vin,din, vout,dout,auxout, error_rst,total_error,corr_error,total_packets,fecdrst,fecdbg,fecdtrig);
  input clk,rst;
  input vin;
  input [63:0] din;
  output vout;
  output [63:0] dout;
  output [31:0] auxout;
  input error_rst;
  output [31:0] total_error;
  output [31:0] corr_error;
  output [31:0] total_packets;
  input fecdrst;
  output [8*64-1:0] fecdbg;
  output fecdtrig;

  wire output_enable;
  reg  [63:0] out_data;
  reg  [63:0] correction,correction_reg,correctiond;
  wire [31:0] parity;
  wire [31:0] syndrome_back2112;
  wire [31:0] syndrome_forward32;
  reg  [31:0] frame_syndrome;
  reg  [63:0] corrections_0;
  wire [63:0] correction_;
  reg  [9:0] carryover;
  wire [10:0] corrections [63:0];
  reg fecdtrig_i;

  /* our received data cache */
  wire [63:0] rx_data;
  reg [63:0] rx_data_reg;

  genvar i;
  reg vind;
  wire vinr;
  wire fec_clr;
  wire fecvalid;
  reg [31:0] tpkts;
  reg [31:0] terror;
  reg [31:0] cerror;
  wire sof,carry_en; 

  reg [5:0] vin_cnt;
  always@(posedge clk) begin
    if(rst | (vin_cnt==6'd32) | ~vin) vin_cnt <= 0;
    else vin_cnt <= vin_cnt+1;

    if(rst) tpkts <= 0;
    else if(vin_cnt == 6'd32) tpkts <= tpkts+1;
  end
  assign fec_clr = (vin_cnt==0);

  always@(posedge clk) vind <= vin;
  assign vinr = vin & ~vind;

  reg vaux; 
  reg [31:0] daux;
  always@(posedge clk) begin
    vaux <= (vin_cnt==31); // valid on cnt==32
    if (vaux) daux <= din[63:32];
  end
  assign auxout = daux;

  delayreg #(1,4)   dlypipe0 (clk,rst,(vin_cnt==32),fecvalid);
  delayreg #(1,2)   dlypipe1 (clk,rst,fecvalid,sof);
  delayreg #(1,2)   dlypipe2 (clk,rst,sof,carry_en);
  delayreg #(64,50) dlypipe3 (clk,rst,din,rx_data);
  delayreg #(1,51)  dlypipe4 (clk,rst,(vin & !(vin_cnt==32)),output_enable);
  delayreg #(64,8)  dlypipe5 (clk,rst,correction,correction_);

  always@(posedge clk) begin
    if(rst|error_rst) begin
      terror <= 0;
      cerror <= 0;
      correctiond <= 0;
      fecdtrig_i <= 0;
    end
    else begin
      correctiond <= correction;
      if(fecvalid & (parity!=0)) terror <= terror+1;
      if((correction != 0) && (correctiond == 0)) cerror <= cerror+1;

      //fecdtrig_i <= (fecdrst)? 0 : (correction != 0) & (correctiond == 0) | fecdtrig_i;
      fecdtrig_i <= (fecdrst)? 0 : (terror != cerror) | fecdtrig_i;
    end
  end  

  assign vout = output_enable;
  assign dout = out_data;
  assign total_error = terror;
  assign corr_error = cerror;
  assign total_packets = tpkts;

  /* calculate the CRC */
  fec_gen fec_inst (clk,rst,fec_clr,din,parity);
  wire [31:0] parityx = (fecvalid)? parity : 0;

  /* rotate backwards 2112 bits */
  backwards2112 b2112(clk,rst,parityx,syndrome_back2112); 

  /* rotate forward by a codeword */
  forward64 f64(clk,rst,frame_syndrome,syndrome_forward32);

  always@(posedge clk) begin
    if(rst) frame_syndrome <= 0;
    else frame_syndrome <= (sof)? syndrome_back2112 : syndrome_forward32;
  end

  /* serialized correction calls */
  assign corrections[0] = corrections_0;
  generate
    for(i=0; i<63; i=i+1) begin
      correctionx #(i) c0x(clk,rst, frame_syndrome, corrections[i+1]);
    end
  endgenerate

  always @(posedge clk) begin
    /* set the first correction (which is the incoming syndrome_back2112 syndrome */
    corrections_0 <= (frame_syndrome[20:0]==21'b0) ? frame_syndrome[31:21] : 11'b0;
     
    /* build the correction register */
    correction <= {corrections[0], 53'b0} |
      { 1'b0, corrections[ 1], 52'b0} | { 2'b0, corrections[ 2], 51'b0} | { 3'b0, corrections[ 3], 50'b0} | { 4'b0, corrections[ 4], 49'b0} |
      { 5'b0, corrections[ 5], 48'b0} | { 6'b0, corrections[ 6], 47'b0} | { 7'b0, corrections[ 7], 46'b0} | { 8'b0, corrections[ 8], 45'b0} |
      { 9'b0, corrections[ 9], 44'b0} | {10'b0, corrections[10], 43'b0} | {11'b0, corrections[11], 42'b0} | {12'b0, corrections[12], 41'b0} |
      {13'b0, corrections[13], 40'b0} | {14'b0, corrections[14], 39'b0} | {15'b0, corrections[15], 38'b0} | {16'b0, corrections[16], 37'b0} |
      {17'b0, corrections[17], 36'b0} | {18'b0, corrections[18], 35'b0} | {19'b0, corrections[19], 34'b0} | {20'b0, corrections[20], 33'b0} |
      {21'b0, corrections[21], 32'b0} | {22'b0, corrections[22], 31'b0} | {23'b0, corrections[23], 30'b0} | {24'b0, corrections[24], 29'b0} |
      {25'b0, corrections[25], 28'b0} | {26'b0, corrections[26], 27'b0} | {27'b0, corrections[27], 26'b0} | {28'b0, corrections[28], 25'b0} |
      {29'b0, corrections[29], 24'b0} | {30'b0, corrections[30], 23'b0} | {31'b0, corrections[31], 22'b0} | {32'b0, corrections[32], 21'b0} |
      {33'b0, corrections[33], 20'b0} | {34'b0, corrections[34], 19'b0} | {35'b0, corrections[35], 18'b0} | {36'b0, corrections[36], 17'b0} |
      {37'b0, corrections[37], 16'b0} | {38'b0, corrections[38], 15'b0} | {39'b0, corrections[39], 14'b0} | {40'b0, corrections[40], 13'b0} |
      {41'b0, corrections[41], 12'b0} | {42'b0, corrections[42], 11'b0} | {43'b0, corrections[43], 10'b0} | {44'b0, corrections[44],  9'b0} |
      {45'b0, corrections[45],  8'b0} | {46'b0, corrections[46],  7'b0} | {47'b0, corrections[47],  6'b0} | {48'b0, corrections[48],  5'b0} |
      {49'b0, corrections[49],  4'b0} | {50'b0, corrections[50],  3'b0} | {51'b0, corrections[51],  2'b0} | {52'b0, corrections[52],  1'b0} |
      {53'b0, corrections[53]} | {54'b0, corrections[54][10:1]} | {55'b0, corrections[55][10:2]} | {56'b0, corrections[56][10:3]} |
      {57'b0, corrections[57][10:4]} | {58'b0, corrections[58][10:5]} | {59'b0, corrections[59][10:6]} | {60'b0, corrections[60][10:7]} |
      {61'b0, corrections[61][10:8]} | {62'b0, corrections[62][10:9]} | {63'b0, corrections[63][10]} | {carryover, 54'b0};

    /* handle carryover */
    carryover <= {corrections[54][0], 9'b0} | {corrections[55][1:0], 8'b0} | {corrections[56][2:0], 7'b0} | {corrections[57][3:0], 6'b0} |
      {corrections[58][4:0], 5'b0} | {corrections[59][5:0], 4'b0} | {corrections[60][6:0], 3'b0} | {corrections[61][7:0], 2'b0} |
      {corrections[62][8:0], 1'b0} | {corrections[63][9:0]};

    out_data <= rx_data ^ correction_;
    rx_data_reg <= rx_data;
    correction_reg <= correction_;
  end

  assign fecdtrig = fecdtrig_i; //(correction != 0) & (correctiond == 0);
  assign fecdbg = {out_data,
                   rx_data_reg,
                   correction_reg,
                   din,
                   frame_syndrome,parity,
                   syndrome_back2112,syndrome_forward32,
                   cerror,terror,
                   rst,6'd0,vin,7'd0,fec_clr,7'd0,fecvalid,7'd0,sof,7'd0,carry_en,7'd0,output_enable,6'd0,carryover};

endmodule

module backwards2112(clk,rst,din,dout);

  input clk,rst;
  input [31:0] din;
  output [31:0] dout;

  reg [31:0] next;

  reg [31:0] data;
  reg [31:0] parity_i;

  assign dout = parity_i;

  always @(posedge clk) begin
    if(rst) begin 
      data <= 0;
      parity_i <= 0;
    end
    else begin
      data <= din;
      parity_i <= next;
    end
  end

  always @(*) begin
    next[0] = data[1] ^ data[5] ^ data[6] ^ data[8] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28];
    next[1] = data[0] ^ data[2] ^ data[6] ^ data[7] ^ data[9] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[2] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[12] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30];
    next[3] = data[0] ^ data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[13] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[4] = data[1] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[5] = data[0] ^ data[2] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[27] ^ data[28] ^ data[30];
    next[6] = data[0] ^ data[1] ^ data[3] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[28] ^ data[29] ^ data[31];
    next[7] = data[1] ^ data[2] ^ data[4] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[29] ^ data[30];
    next[8] = data[2] ^ data[3] ^ data[5] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[30] ^ data[31];
    next[9] = data[0] ^ data[3] ^ data[4] ^ data[6] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[31];
    next[10] = data[0] ^ data[1] ^ data[4] ^ data[5] ^ data[7] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[27];
    next[11] = data[2] ^ data[23];
    next[12] = data[3] ^ data[24];
    next[13] = data[4] ^ data[25];
    next[14] = data[5] ^ data[26];
    next[15] = data[6] ^ data[27];
    next[16] = data[7] ^ data[28];
    next[17] = data[8] ^ data[29];
    next[18] = data[9] ^ data[30];
    next[19] = data[10] ^ data[31];
    next[20] = data[11];
    next[21] = data[1] ^ data[5] ^ data[6] ^ data[8] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28];
    next[22] = data[0] ^ data[2] ^ data[6] ^ data[7] ^ data[9] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[23] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30];
    next[24] = data[0] ^ data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[25] = data[1] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[26] = data[0] ^ data[2] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[27] ^ data[28] ^ data[30];
    next[27] = data[0] ^ data[1] ^ data[3] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[28] ^ data[29] ^ data[31];
    next[28] = data[1] ^ data[2] ^ data[4] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[29] ^ data[30];
    next[29] = data[2] ^ data[3] ^ data[5] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[30] ^ data[31];
    next[30] = data[3] ^ data[4] ^ data[6] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[31];
    next[31] = data[0] ^ data[4] ^ data[5] ^ data[7] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27];
  end

endmodule

module forward64 (clk,rst, din,dout);

  input clk,rst;
  input [31:0] din;
  output [31:0] dout;


  reg [31:0] next;
  wire [31:0] data;
  wire [31:0] out_i;

  assign dout = out_i;
  assign data = din;
  assign out_i = next;

  always @(*) begin
    next[0] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[1] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[2] = data[2] ^ data[3] ^ data[4] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[27];
    next[3] = data[3] ^ data[4] ^ data[5] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[28];
    next[4] = data[0] ^ data[4] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[29];
    next[5] = data[1] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[30];
    next[6] = data[2] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30] ^ data[31];
    next[7] = data[0] ^ data[3] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[8] = data[1] ^ data[4] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[9] = data[2] ^ data[5] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[10] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[11] = data[10] ^ data[31];
    next[12] = data[11];
    next[13] = data[12];
    next[14] = data[13];
    next[15] = data[14];
    next[16] = data[15];
    next[17] = data[16];
    next[18] = data[17];
    next[19] = data[18];
    next[20] = data[19];
    next[21] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[22] = data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[23] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[21] ^ data[23] ^ data[26] ^ data[27];
    next[24] = data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[22] ^ data[24] ^ data[27] ^ data[28];
    next[25] = data[0] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[23] ^ data[25] ^ data[28] ^ data[29];
    next[26] = data[1] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[26] ^ data[29] ^ data[30];
    next[27] = data[2] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[27] ^ data[30] ^ data[31];
    next[28] = data[0] ^ data[3] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[28] ^ data[31];
    next[29] = data[1] ^ data[4] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[29];
    next[30] = data[2] ^ data[5] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[30];
    next[31] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
  end

endmodule

module correctionx(clk,rst,din,correction);

  parameter CN = 0;

  input clk,rst;
  input [31:0] din;
  output [10:0] correction;


  reg [31:0] next;
  reg [31:0] data;
  reg [10:0] correction_i;

  assign correction = correction_i;

  always @(posedge clk) begin
    if (rst) begin
      data <= 0;
      correction_i <= 0;
    end
    else begin
      data <= din;
      correction_i <= (next[20:0]==0) ? next[31:21] : 11'b0;
    end
  end

generate
  if(CN == 0) begin
    always @(*) begin
      next[0] = data[31];
      next[1] = data[0];
      next[2] = data[1] ^ data[31];
      next[3] = data[2];
      next[4] = data[3];
      next[5] = data[4];
      next[6] = data[5];
      next[7] = data[6];
      next[8] = data[7];
      next[9] = data[8];
      next[10] = data[9];
      next[11] = data[10] ^ data[31];
      next[12] = data[11];
      next[13] = data[12];
      next[14] = data[13];
      next[15] = data[14];
      next[16] = data[15];
      next[17] = data[16];
      next[18] = data[17];
      next[19] = data[18];
      next[20] = data[19];
      next[21] = data[20] ^ data[31];
      next[22] = data[21];
      next[23] = data[22] ^ data[31];
      next[24] = data[23];
      next[25] = data[24];
      next[26] = data[25];
      next[27] = data[26];
      next[28] = data[27];
      next[29] = data[28];
      next[30] = data[29];
      next[31] = data[30];
    end
  end
  if(CN == 1) begin
    always @(*) begin
      next[0] = data[30];
      next[1] = data[31];
      next[2] = data[0] ^ data[30];
      next[3] = data[1] ^ data[31];
      next[4] = data[2];
      next[5] = data[3];
      next[6] = data[4];
      next[7] = data[5];
      next[8] = data[6];
      next[9] = data[7];
      next[10] = data[8];
      next[11] = data[9] ^ data[30];
      next[12] = data[10] ^ data[31];
      next[13] = data[11];
      next[14] = data[12];
      next[15] = data[13];
      next[16] = data[14];
      next[17] = data[15];
      next[18] = data[16];
      next[19] = data[17];
      next[20] = data[18];
      next[21] = data[19] ^ data[30];
      next[22] = data[20] ^ data[31];
      next[23] = data[21] ^ data[30];
      next[24] = data[22] ^ data[31];
      next[25] = data[23];
      next[26] = data[24];
      next[27] = data[25];
      next[28] = data[26];
      next[29] = data[27];
      next[30] = data[28];
      next[31] = data[29];
    end
  end
  if(CN == 2) begin
    always @(*) begin
      next[0] = data[29];
      next[1] = data[30];
      next[2] = data[29] ^ data[31];
      next[3] = data[0] ^ data[30];
      next[4] = data[1] ^ data[31];
      next[5] = data[2];
      next[6] = data[3];
      next[7] = data[4];
      next[8] = data[5];
      next[9] = data[6];
      next[10] = data[7];
      next[11] = data[8] ^ data[29];
      next[12] = data[9] ^ data[30];
      next[13] = data[10] ^ data[31];
      next[14] = data[11];
      next[15] = data[12];
      next[16] = data[13];
      next[17] = data[14];
      next[18] = data[15];
      next[19] = data[16];
      next[20] = data[17];
      next[21] = data[18] ^ data[29];
      next[22] = data[19] ^ data[30];
      next[23] = data[20] ^ data[29] ^ data[31];
      next[24] = data[21] ^ data[30];
      next[25] = data[22] ^ data[31];
      next[26] = data[23];
      next[27] = data[24];
      next[28] = data[25];
      next[29] = data[26];
      next[30] = data[27];
      next[31] = data[28];
    end
  end
  if(CN == 3) begin
    always @(*) begin
      next[0] = data[28];
      next[1] = data[29];
      next[2] = data[28] ^ data[30];
      next[3] = data[29] ^ data[31];
      next[4] = data[0] ^ data[30];
      next[5] = data[1] ^ data[31];
      next[6] = data[2];
      next[7] = data[3];
      next[8] = data[4];
      next[9] = data[5];
      next[10] = data[6];
      next[11] = data[7] ^ data[28];
      next[12] = data[8] ^ data[29];
      next[13] = data[9] ^ data[30];
      next[14] = data[10] ^ data[31];
      next[15] = data[11];
      next[16] = data[12];
      next[17] = data[13];
      next[18] = data[14];
      next[19] = data[15];
      next[20] = data[16];
      next[21] = data[17] ^ data[28];
      next[22] = data[18] ^ data[29];
      next[23] = data[19] ^ data[28] ^ data[30];
      next[24] = data[20] ^ data[29] ^ data[31];
      next[25] = data[21] ^ data[30];
      next[26] = data[22] ^ data[31];
      next[27] = data[23];
      next[28] = data[24];
      next[29] = data[25];
      next[30] = data[26];
      next[31] = data[27];
    end
  end
  if(CN == 4) begin
    always @(*) begin
      next[0] = data[27];
      next[1] = data[28];
      next[2] = data[27] ^ data[29];
      next[3] = data[28] ^ data[30];
      next[4] = data[29] ^ data[31];
      next[5] = data[0] ^ data[30];
      next[6] = data[1] ^ data[31];
      next[7] = data[2];
      next[8] = data[3];
      next[9] = data[4];
      next[10] = data[5];
      next[11] = data[6] ^ data[27];
      next[12] = data[7] ^ data[28];
      next[13] = data[8] ^ data[29];
      next[14] = data[9] ^ data[30];
      next[15] = data[10] ^ data[31];
      next[16] = data[11];
      next[17] = data[12];
      next[18] = data[13];
      next[19] = data[14];
      next[20] = data[15];
      next[21] = data[16] ^ data[27];
      next[22] = data[17] ^ data[28];
      next[23] = data[18] ^ data[27] ^ data[29];
      next[24] = data[19] ^ data[28] ^ data[30];
      next[25] = data[20] ^ data[29] ^ data[31];
      next[26] = data[21] ^ data[30];
      next[27] = data[22] ^ data[31];
      next[28] = data[23];
      next[29] = data[24];
      next[30] = data[25];
      next[31] = data[26];
    end
  end
  if(CN == 5) begin
    always @(*) begin
      next[0] = data[26];
      next[1] = data[27];
      next[2] = data[26] ^ data[28];
      next[3] = data[27] ^ data[29];
      next[4] = data[28] ^ data[30];
      next[5] = data[29] ^ data[31];
      next[6] = data[0] ^ data[30];
      next[7] = data[1] ^ data[31];
      next[8] = data[2];
      next[9] = data[3];
      next[10] = data[4];
      next[11] = data[5] ^ data[26];
      next[12] = data[6] ^ data[27];
      next[13] = data[7] ^ data[28];
      next[14] = data[8] ^ data[29];
      next[15] = data[9] ^ data[30];
      next[16] = data[10] ^ data[31];
      next[17] = data[11];
      next[18] = data[12];
      next[19] = data[13];
      next[20] = data[14];
      next[21] = data[15] ^ data[26];
      next[22] = data[16] ^ data[27];
      next[23] = data[17] ^ data[26] ^ data[28];
      next[24] = data[18] ^ data[27] ^ data[29];
      next[25] = data[19] ^ data[28] ^ data[30];
      next[26] = data[20] ^ data[29] ^ data[31];
      next[27] = data[21] ^ data[30];
      next[28] = data[22] ^ data[31];
      next[29] = data[23];
      next[30] = data[24];
      next[31] = data[25];
    end
  end
  if(CN == 6) begin
    always @(*) begin
      next[0] = data[25];
      next[1] = data[26];
      next[2] = data[25] ^ data[27];
      next[3] = data[26] ^ data[28];
      next[4] = data[27] ^ data[29];
      next[5] = data[28] ^ data[30];
      next[6] = data[29] ^ data[31];
      next[7] = data[0] ^ data[30];
      next[8] = data[1] ^ data[31];
      next[9] = data[2];
      next[10] = data[3];
      next[11] = data[4] ^ data[25];
      next[12] = data[5] ^ data[26];
      next[13] = data[6] ^ data[27];
      next[14] = data[7] ^ data[28];
      next[15] = data[8] ^ data[29];
      next[16] = data[9] ^ data[30];
      next[17] = data[10] ^ data[31];
      next[18] = data[11];
      next[19] = data[12];
      next[20] = data[13];
      next[21] = data[14] ^ data[25];
      next[22] = data[15] ^ data[26];
      next[23] = data[16] ^ data[25] ^ data[27];
      next[24] = data[17] ^ data[26] ^ data[28];
      next[25] = data[18] ^ data[27] ^ data[29];
      next[26] = data[19] ^ data[28] ^ data[30];
      next[27] = data[20] ^ data[29] ^ data[31];
      next[28] = data[21] ^ data[30];
      next[29] = data[22] ^ data[31];
      next[30] = data[23];
      next[31] = data[24];
    end
  end
  if(CN == 7) begin
    always @(*) begin
      next[0] = data[24];
      next[1] = data[25];
      next[2] = data[24] ^ data[26];
      next[3] = data[25] ^ data[27];
      next[4] = data[26] ^ data[28];
      next[5] = data[27] ^ data[29];
      next[6] = data[28] ^ data[30];
      next[7] = data[29] ^ data[31];
      next[8] = data[0] ^ data[30];
      next[9] = data[1] ^ data[31];
      next[10] = data[2];
      next[11] = data[3] ^ data[24];
      next[12] = data[4] ^ data[25];
      next[13] = data[5] ^ data[26];
      next[14] = data[6] ^ data[27];
      next[15] = data[7] ^ data[28];
      next[16] = data[8] ^ data[29];
      next[17] = data[9] ^ data[30];
      next[18] = data[10] ^ data[31];
      next[19] = data[11];
      next[20] = data[12];
      next[21] = data[13] ^ data[24];
      next[22] = data[14] ^ data[25];
      next[23] = data[15] ^ data[24] ^ data[26];
      next[24] = data[16] ^ data[25] ^ data[27];
      next[25] = data[17] ^ data[26] ^ data[28];
      next[26] = data[18] ^ data[27] ^ data[29];
      next[27] = data[19] ^ data[28] ^ data[30];
      next[28] = data[20] ^ data[29] ^ data[31];
      next[29] = data[21] ^ data[30];
      next[30] = data[22] ^ data[31];
      next[31] = data[23];
    end
  end
  if(CN == 8) begin
    always @(*) begin
      next[0] = data[23];
      next[1] = data[24];
      next[2] = data[23] ^ data[25];
      next[3] = data[24] ^ data[26];
      next[4] = data[25] ^ data[27];
      next[5] = data[26] ^ data[28];
      next[6] = data[27] ^ data[29];
      next[7] = data[28] ^ data[30];
      next[8] = data[29] ^ data[31];
      next[9] = data[0] ^ data[30];
      next[10] = data[1] ^ data[31];
      next[11] = data[2] ^ data[23];
      next[12] = data[3] ^ data[24];
      next[13] = data[4] ^ data[25];
      next[14] = data[5] ^ data[26];
      next[15] = data[6] ^ data[27];
      next[16] = data[7] ^ data[28];
      next[17] = data[8] ^ data[29];
      next[18] = data[9] ^ data[30];
      next[19] = data[10] ^ data[31];
      next[20] = data[11];
      next[21] = data[12] ^ data[23];
      next[22] = data[13] ^ data[24];
      next[23] = data[14] ^ data[23] ^ data[25];
      next[24] = data[15] ^ data[24] ^ data[26];
      next[25] = data[16] ^ data[25] ^ data[27];
      next[26] = data[17] ^ data[26] ^ data[28];
      next[27] = data[18] ^ data[27] ^ data[29];
      next[28] = data[19] ^ data[28] ^ data[30];
      next[29] = data[20] ^ data[29] ^ data[31];
      next[30] = data[21] ^ data[30];
      next[31] = data[22] ^ data[31];
    end
  end
  if(CN == 9) begin
    always @(*) begin
      next[0] = data[22] ^ data[31];
      next[1] = data[23];
      next[2] = data[22] ^ data[24] ^ data[31];
      next[3] = data[23] ^ data[25];
      next[4] = data[24] ^ data[26];
      next[5] = data[25] ^ data[27];
      next[6] = data[26] ^ data[28];
      next[7] = data[27] ^ data[29];
      next[8] = data[28] ^ data[30];
      next[9] = data[29] ^ data[31];
      next[10] = data[0] ^ data[30];
      next[11] = data[1] ^ data[22];
      next[12] = data[2] ^ data[23];
      next[13] = data[3] ^ data[24];
      next[14] = data[4] ^ data[25];
      next[15] = data[5] ^ data[26];
      next[16] = data[6] ^ data[27];
      next[17] = data[7] ^ data[28];
      next[18] = data[8] ^ data[29];
      next[19] = data[9] ^ data[30];
      next[20] = data[10] ^ data[31];
      next[21] = data[11] ^ data[22] ^ data[31];
      next[22] = data[12] ^ data[23];
      next[23] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[24] = data[14] ^ data[23] ^ data[25];
      next[25] = data[15] ^ data[24] ^ data[26];
      next[26] = data[16] ^ data[25] ^ data[27];
      next[27] = data[17] ^ data[26] ^ data[28];
      next[28] = data[18] ^ data[27] ^ data[29];
      next[29] = data[19] ^ data[28] ^ data[30];
      next[30] = data[20] ^ data[29] ^ data[31];
      next[31] = data[21] ^ data[30];
    end
  end
  if(CN == 10) begin
    always @(*) begin
      next[0] = data[21] ^ data[30];
      next[1] = data[22] ^ data[31];
      next[2] = data[21] ^ data[23] ^ data[30];
      next[3] = data[22] ^ data[24] ^ data[31];
      next[4] = data[23] ^ data[25];
      next[5] = data[24] ^ data[26];
      next[6] = data[25] ^ data[27];
      next[7] = data[26] ^ data[28];
      next[8] = data[27] ^ data[29];
      next[9] = data[28] ^ data[30];
      next[10] = data[29] ^ data[31];
      next[11] = data[0] ^ data[21];
      next[12] = data[1] ^ data[22];
      next[13] = data[2] ^ data[23];
      next[14] = data[3] ^ data[24];
      next[15] = data[4] ^ data[25];
      next[16] = data[5] ^ data[26];
      next[17] = data[6] ^ data[27];
      next[18] = data[7] ^ data[28];
      next[19] = data[8] ^ data[29];
      next[20] = data[9] ^ data[30];
      next[21] = data[10] ^ data[21] ^ data[30] ^ data[31];
      next[22] = data[11] ^ data[22] ^ data[31];
      next[23] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[24] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[25] = data[14] ^ data[23] ^ data[25];
      next[26] = data[15] ^ data[24] ^ data[26];
      next[27] = data[16] ^ data[25] ^ data[27];
      next[28] = data[17] ^ data[26] ^ data[28];
      next[29] = data[18] ^ data[27] ^ data[29];
      next[30] = data[19] ^ data[28] ^ data[30];
      next[31] = data[20] ^ data[29] ^ data[31];
    end
  end
  if(CN == 11) begin
    always @(*) begin
      next[0] = data[20] ^ data[29] ^ data[31];
      next[1] = data[21] ^ data[30];
      next[2] = data[20] ^ data[22] ^ data[29];
      next[3] = data[21] ^ data[23] ^ data[30];
      next[4] = data[22] ^ data[24] ^ data[31];
      next[5] = data[23] ^ data[25];
      next[6] = data[24] ^ data[26];
      next[7] = data[25] ^ data[27];
      next[8] = data[26] ^ data[28];
      next[9] = data[27] ^ data[29];
      next[10] = data[28] ^ data[30];
      next[11] = data[20];
      next[12] = data[0] ^ data[21];
      next[13] = data[1] ^ data[22];
      next[14] = data[2] ^ data[23];
      next[15] = data[3] ^ data[24];
      next[16] = data[4] ^ data[25];
      next[17] = data[5] ^ data[26];
      next[18] = data[6] ^ data[27];
      next[19] = data[7] ^ data[28];
      next[20] = data[8] ^ data[29];
      next[21] = data[9] ^ data[20] ^ data[29] ^ data[30] ^ data[31];
      next[22] = data[10] ^ data[21] ^ data[30] ^ data[31];
      next[23] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[24] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[25] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[26] = data[14] ^ data[23] ^ data[25];
      next[27] = data[15] ^ data[24] ^ data[26];
      next[28] = data[16] ^ data[25] ^ data[27];
      next[29] = data[17] ^ data[26] ^ data[28];
      next[30] = data[18] ^ data[27] ^ data[29];
      next[31] = data[19] ^ data[28] ^ data[30];
    end
  end
  if(CN == 12) begin
    always @(*) begin
      next[0] = data[19] ^ data[28] ^ data[30];
      next[1] = data[20] ^ data[29] ^ data[31];
      next[2] = data[19] ^ data[21] ^ data[28];
      next[3] = data[20] ^ data[22] ^ data[29];
      next[4] = data[21] ^ data[23] ^ data[30];
      next[5] = data[22] ^ data[24] ^ data[31];
      next[6] = data[23] ^ data[25];
      next[7] = data[24] ^ data[26];
      next[8] = data[25] ^ data[27];
      next[9] = data[26] ^ data[28];
      next[10] = data[27] ^ data[29];
      next[11] = data[19];
      next[12] = data[20];
      next[13] = data[0] ^ data[21];
      next[14] = data[1] ^ data[22];
      next[15] = data[2] ^ data[23];
      next[16] = data[3] ^ data[24];
      next[17] = data[4] ^ data[25];
      next[18] = data[5] ^ data[26];
      next[19] = data[6] ^ data[27];
      next[20] = data[7] ^ data[28];
      next[21] = data[8] ^ data[19] ^ data[28] ^ data[29] ^ data[30];
      next[22] = data[9] ^ data[20] ^ data[29] ^ data[30] ^ data[31];
      next[23] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[24] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[25] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[26] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[27] = data[14] ^ data[23] ^ data[25];
      next[28] = data[15] ^ data[24] ^ data[26];
      next[29] = data[16] ^ data[25] ^ data[27];
      next[30] = data[17] ^ data[26] ^ data[28];
      next[31] = data[18] ^ data[27] ^ data[29];
    end
  end
  if(CN == 13) begin
    always @(*) begin
      next[0] = data[18] ^ data[27] ^ data[29];
      next[1] = data[19] ^ data[28] ^ data[30];
      next[2] = data[18] ^ data[20] ^ data[27] ^ data[31];
      next[3] = data[19] ^ data[21] ^ data[28];
      next[4] = data[20] ^ data[22] ^ data[29];
      next[5] = data[21] ^ data[23] ^ data[30];
      next[6] = data[22] ^ data[24] ^ data[31];
      next[7] = data[23] ^ data[25];
      next[8] = data[24] ^ data[26];
      next[9] = data[25] ^ data[27];
      next[10] = data[26] ^ data[28];
      next[11] = data[18];
      next[12] = data[19];
      next[13] = data[20];
      next[14] = data[0] ^ data[21];
      next[15] = data[1] ^ data[22];
      next[16] = data[2] ^ data[23];
      next[17] = data[3] ^ data[24];
      next[18] = data[4] ^ data[25];
      next[19] = data[5] ^ data[26];
      next[20] = data[6] ^ data[27];
      next[21] = data[7] ^ data[18] ^ data[27] ^ data[28] ^ data[29];
      next[22] = data[8] ^ data[19] ^ data[28] ^ data[29] ^ data[30];
      next[23] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[24] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[25] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[26] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[27] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[28] = data[14] ^ data[23] ^ data[25];
      next[29] = data[15] ^ data[24] ^ data[26];
      next[30] = data[16] ^ data[25] ^ data[27];
      next[31] = data[17] ^ data[26] ^ data[28];
    end
  end
  if(CN == 14) begin
    always @(*) begin
      next[0] = data[17] ^ data[26] ^ data[28];
      next[1] = data[18] ^ data[27] ^ data[29];
      next[2] = data[17] ^ data[19] ^ data[26] ^ data[30];
      next[3] = data[18] ^ data[20] ^ data[27] ^ data[31];
      next[4] = data[19] ^ data[21] ^ data[28];
      next[5] = data[20] ^ data[22] ^ data[29];
      next[6] = data[21] ^ data[23] ^ data[30];
      next[7] = data[22] ^ data[24] ^ data[31];
      next[8] = data[23] ^ data[25];
      next[9] = data[24] ^ data[26];
      next[10] = data[25] ^ data[27];
      next[11] = data[17];
      next[12] = data[18];
      next[13] = data[19];
      next[14] = data[20];
      next[15] = data[0] ^ data[21];
      next[16] = data[1] ^ data[22];
      next[17] = data[2] ^ data[23];
      next[18] = data[3] ^ data[24];
      next[19] = data[4] ^ data[25];
      next[20] = data[5] ^ data[26];
      next[21] = data[6] ^ data[17] ^ data[26] ^ data[27] ^ data[28];
      next[22] = data[7] ^ data[18] ^ data[27] ^ data[28] ^ data[29];
      next[23] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[24] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[25] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[26] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[27] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[28] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[29] = data[14] ^ data[23] ^ data[25];
      next[30] = data[15] ^ data[24] ^ data[26];
      next[31] = data[16] ^ data[25] ^ data[27];
    end
  end
  if(CN == 15) begin
    always @(*) begin
      next[0] = data[16] ^ data[25] ^ data[27];
      next[1] = data[17] ^ data[26] ^ data[28];
      next[2] = data[16] ^ data[18] ^ data[25] ^ data[29];
      next[3] = data[17] ^ data[19] ^ data[26] ^ data[30];
      next[4] = data[18] ^ data[20] ^ data[27] ^ data[31];
      next[5] = data[19] ^ data[21] ^ data[28];
      next[6] = data[20] ^ data[22] ^ data[29];
      next[7] = data[21] ^ data[23] ^ data[30];
      next[8] = data[22] ^ data[24] ^ data[31];
      next[9] = data[23] ^ data[25];
      next[10] = data[24] ^ data[26];
      next[11] = data[16];
      next[12] = data[17];
      next[13] = data[18];
      next[14] = data[19];
      next[15] = data[20];
      next[16] = data[0] ^ data[21];
      next[17] = data[1] ^ data[22];
      next[18] = data[2] ^ data[23];
      next[19] = data[3] ^ data[24];
      next[20] = data[4] ^ data[25];
      next[21] = data[5] ^ data[16] ^ data[25] ^ data[26] ^ data[27];
      next[22] = data[6] ^ data[17] ^ data[26] ^ data[27] ^ data[28];
      next[23] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[24] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[25] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[26] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[27] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[28] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[29] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[30] = data[14] ^ data[23] ^ data[25];
      next[31] = data[15] ^ data[24] ^ data[26];
    end
  end
  if(CN == 16) begin
    always @(*) begin
      next[0] = data[15] ^ data[24] ^ data[26];
      next[1] = data[16] ^ data[25] ^ data[27];
      next[2] = data[15] ^ data[17] ^ data[24] ^ data[28];
      next[3] = data[16] ^ data[18] ^ data[25] ^ data[29];
      next[4] = data[17] ^ data[19] ^ data[26] ^ data[30];
      next[5] = data[18] ^ data[20] ^ data[27] ^ data[31];
      next[6] = data[19] ^ data[21] ^ data[28];
      next[7] = data[20] ^ data[22] ^ data[29];
      next[8] = data[21] ^ data[23] ^ data[30];
      next[9] = data[22] ^ data[24] ^ data[31];
      next[10] = data[23] ^ data[25];
      next[11] = data[15];
      next[12] = data[16];
      next[13] = data[17];
      next[14] = data[18];
      next[15] = data[19];
      next[16] = data[20];
      next[17] = data[0] ^ data[21];
      next[18] = data[1] ^ data[22];
      next[19] = data[2] ^ data[23];
      next[20] = data[3] ^ data[24];
      next[21] = data[4] ^ data[15] ^ data[24] ^ data[25] ^ data[26];
      next[22] = data[5] ^ data[16] ^ data[25] ^ data[26] ^ data[27];
      next[23] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[24] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[25] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[26] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[27] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[28] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[29] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[30] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[31] = data[14] ^ data[23] ^ data[25];
    end
  end
  if(CN == 17) begin
    always @(*) begin
      next[0] = data[14] ^ data[23] ^ data[25];
      next[1] = data[15] ^ data[24] ^ data[26];
      next[2] = data[14] ^ data[16] ^ data[23] ^ data[27];
      next[3] = data[15] ^ data[17] ^ data[24] ^ data[28];
      next[4] = data[16] ^ data[18] ^ data[25] ^ data[29];
      next[5] = data[17] ^ data[19] ^ data[26] ^ data[30];
      next[6] = data[18] ^ data[20] ^ data[27] ^ data[31];
      next[7] = data[19] ^ data[21] ^ data[28];
      next[8] = data[20] ^ data[22] ^ data[29];
      next[9] = data[21] ^ data[23] ^ data[30];
      next[10] = data[22] ^ data[24] ^ data[31];
      next[11] = data[14];
      next[12] = data[15];
      next[13] = data[16];
      next[14] = data[17];
      next[15] = data[18];
      next[16] = data[19];
      next[17] = data[20];
      next[18] = data[0] ^ data[21];
      next[19] = data[1] ^ data[22];
      next[20] = data[2] ^ data[23];
      next[21] = data[3] ^ data[14] ^ data[23] ^ data[24] ^ data[25];
      next[22] = data[4] ^ data[15] ^ data[24] ^ data[25] ^ data[26];
      next[23] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[24] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[25] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[26] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[27] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[28] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[29] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[30] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[31] = data[13] ^ data[22] ^ data[24] ^ data[31];
    end
  end
  if(CN == 18) begin
    always @(*) begin
      next[0] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[1] = data[14] ^ data[23] ^ data[25];
      next[2] = data[13] ^ data[15] ^ data[22] ^ data[26] ^ data[31];
      next[3] = data[14] ^ data[16] ^ data[23] ^ data[27];
      next[4] = data[15] ^ data[17] ^ data[24] ^ data[28];
      next[5] = data[16] ^ data[18] ^ data[25] ^ data[29];
      next[6] = data[17] ^ data[19] ^ data[26] ^ data[30];
      next[7] = data[18] ^ data[20] ^ data[27] ^ data[31];
      next[8] = data[19] ^ data[21] ^ data[28];
      next[9] = data[20] ^ data[22] ^ data[29];
      next[10] = data[21] ^ data[23] ^ data[30];
      next[11] = data[13];
      next[12] = data[14];
      next[13] = data[15];
      next[14] = data[16];
      next[15] = data[17];
      next[16] = data[18];
      next[17] = data[19];
      next[18] = data[20];
      next[19] = data[0] ^ data[21];
      next[20] = data[1] ^ data[22];
      next[21] = data[2] ^ data[13] ^ data[22] ^ data[23] ^ data[24] ^ data[31];
      next[22] = data[3] ^ data[14] ^ data[23] ^ data[24] ^ data[25];
      next[23] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[24] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[25] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[26] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[27] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[28] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[29] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[30] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[31] = data[12] ^ data[21] ^ data[23] ^ data[30];
    end
  end
  if(CN == 19) begin
    always @(*) begin
      next[0] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[1] = data[13] ^ data[22] ^ data[24] ^ data[31];
      next[2] = data[12] ^ data[14] ^ data[21] ^ data[25] ^ data[30];
      next[3] = data[13] ^ data[15] ^ data[22] ^ data[26] ^ data[31];
      next[4] = data[14] ^ data[16] ^ data[23] ^ data[27];
      next[5] = data[15] ^ data[17] ^ data[24] ^ data[28];
      next[6] = data[16] ^ data[18] ^ data[25] ^ data[29];
      next[7] = data[17] ^ data[19] ^ data[26] ^ data[30];
      next[8] = data[18] ^ data[20] ^ data[27] ^ data[31];
      next[9] = data[19] ^ data[21] ^ data[28];
      next[10] = data[20] ^ data[22] ^ data[29];
      next[11] = data[12];
      next[12] = data[13];
      next[13] = data[14];
      next[14] = data[15];
      next[15] = data[16];
      next[16] = data[17];
      next[17] = data[18];
      next[18] = data[19];
      next[19] = data[20];
      next[20] = data[0] ^ data[21];
      next[21] = data[1] ^ data[12] ^ data[21] ^ data[22] ^ data[23] ^ data[30];
      next[22] = data[2] ^ data[13] ^ data[22] ^ data[23] ^ data[24] ^ data[31];
      next[23] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[24] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[25] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[26] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[27] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[28] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[29] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[30] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[31] = data[11] ^ data[20] ^ data[22] ^ data[29];
    end
  end
  if(CN == 20) begin
    always @(*) begin
      next[0] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[1] = data[12] ^ data[21] ^ data[23] ^ data[30];
      next[2] = data[11] ^ data[13] ^ data[20] ^ data[24] ^ data[29] ^ data[31];
      next[3] = data[12] ^ data[14] ^ data[21] ^ data[25] ^ data[30];
      next[4] = data[13] ^ data[15] ^ data[22] ^ data[26] ^ data[31];
      next[5] = data[14] ^ data[16] ^ data[23] ^ data[27];
      next[6] = data[15] ^ data[17] ^ data[24] ^ data[28];
      next[7] = data[16] ^ data[18] ^ data[25] ^ data[29];
      next[8] = data[17] ^ data[19] ^ data[26] ^ data[30];
      next[9] = data[18] ^ data[20] ^ data[27] ^ data[31];
      next[10] = data[19] ^ data[21] ^ data[28];
      next[11] = data[11];
      next[12] = data[12];
      next[13] = data[13];
      next[14] = data[14];
      next[15] = data[15];
      next[16] = data[16];
      next[17] = data[17];
      next[18] = data[18];
      next[19] = data[19];
      next[20] = data[20];
      next[21] = data[0] ^ data[11] ^ data[20] ^ data[21] ^ data[22] ^ data[29];
      next[22] = data[1] ^ data[12] ^ data[21] ^ data[22] ^ data[23] ^ data[30];
      next[23] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[24] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[25] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[26] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[27] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[28] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[29] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[30] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[31] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
    end
  end
  if(CN == 21) begin
    always @(*) begin
      next[0] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[1] = data[11] ^ data[20] ^ data[22] ^ data[29];
      next[2] = data[10] ^ data[12] ^ data[19] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[3] = data[11] ^ data[13] ^ data[20] ^ data[24] ^ data[29] ^ data[31];
      next[4] = data[12] ^ data[14] ^ data[21] ^ data[25] ^ data[30];
      next[5] = data[13] ^ data[15] ^ data[22] ^ data[26] ^ data[31];
      next[6] = data[14] ^ data[16] ^ data[23] ^ data[27];
      next[7] = data[15] ^ data[17] ^ data[24] ^ data[28];
      next[8] = data[16] ^ data[18] ^ data[25] ^ data[29];
      next[9] = data[17] ^ data[19] ^ data[26] ^ data[30];
      next[10] = data[18] ^ data[20] ^ data[27] ^ data[31];
      next[11] = data[10] ^ data[31];
      next[12] = data[11];
      next[13] = data[12];
      next[14] = data[13];
      next[15] = data[14];
      next[16] = data[15];
      next[17] = data[16];
      next[18] = data[17];
      next[19] = data[18];
      next[20] = data[19];
      next[21] = data[10] ^ data[19] ^ data[20] ^ data[21] ^ data[28] ^ data[31];
      next[22] = data[0] ^ data[11] ^ data[20] ^ data[21] ^ data[22] ^ data[29];
      next[23] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[24] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[25] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[26] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[27] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[28] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[29] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[30] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[31] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
    end
  end
  if(CN == 22) begin
    always @(*) begin
      next[0] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[1] = data[10] ^ data[19] ^ data[21] ^ data[28] ^ data[31];
      next[2] = data[9] ^ data[11] ^ data[18] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[3] = data[10] ^ data[12] ^ data[19] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[4] = data[11] ^ data[13] ^ data[20] ^ data[24] ^ data[29] ^ data[31];
      next[5] = data[12] ^ data[14] ^ data[21] ^ data[25] ^ data[30];
      next[6] = data[13] ^ data[15] ^ data[22] ^ data[26] ^ data[31];
      next[7] = data[14] ^ data[16] ^ data[23] ^ data[27];
      next[8] = data[15] ^ data[17] ^ data[24] ^ data[28];
      next[9] = data[16] ^ data[18] ^ data[25] ^ data[29];
      next[10] = data[17] ^ data[19] ^ data[26] ^ data[30];
      next[11] = data[9] ^ data[30];
      next[12] = data[10] ^ data[31];
      next[13] = data[11];
      next[14] = data[12];
      next[15] = data[13];
      next[16] = data[14];
      next[17] = data[15];
      next[18] = data[16];
      next[19] = data[17];
      next[20] = data[18];
      next[21] = data[9] ^ data[18] ^ data[19] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[22] = data[10] ^ data[19] ^ data[20] ^ data[21] ^ data[28] ^ data[31];
      next[23] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[24] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[25] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[26] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[27] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[28] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[29] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[30] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[31] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
    end
  end
  if(CN == 23) begin
    always @(*) begin
      next[0] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[1] = data[9] ^ data[18] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[2] = data[8] ^ data[10] ^ data[17] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[3] = data[9] ^ data[11] ^ data[18] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[4] = data[10] ^ data[12] ^ data[19] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[5] = data[11] ^ data[13] ^ data[20] ^ data[24] ^ data[29] ^ data[31];
      next[6] = data[12] ^ data[14] ^ data[21] ^ data[25] ^ data[30];
      next[7] = data[13] ^ data[15] ^ data[22] ^ data[26] ^ data[31];
      next[8] = data[14] ^ data[16] ^ data[23] ^ data[27];
      next[9] = data[15] ^ data[17] ^ data[24] ^ data[28];
      next[10] = data[16] ^ data[18] ^ data[25] ^ data[29];
      next[11] = data[8] ^ data[29];
      next[12] = data[9] ^ data[30];
      next[13] = data[10] ^ data[31];
      next[14] = data[11];
      next[15] = data[12];
      next[16] = data[13];
      next[17] = data[14];
      next[18] = data[15];
      next[19] = data[16];
      next[20] = data[17];
      next[21] = data[8] ^ data[17] ^ data[18] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[22] = data[9] ^ data[18] ^ data[19] ^ data[20] ^ data[27] ^ data[30] ^ data[31];
      next[23] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[24] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[25] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[26] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[27] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[28] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[29] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[30] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[31] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
    end
  end
  if(CN == 24) begin
    always @(*) begin
      next[0] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[1] = data[8] ^ data[17] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[2] = data[7] ^ data[9] ^ data[16] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[3] = data[8] ^ data[10] ^ data[17] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[4] = data[9] ^ data[11] ^ data[18] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[5] = data[10] ^ data[12] ^ data[19] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[6] = data[11] ^ data[13] ^ data[20] ^ data[24] ^ data[29] ^ data[31];
      next[7] = data[12] ^ data[14] ^ data[21] ^ data[25] ^ data[30];
      next[8] = data[13] ^ data[15] ^ data[22] ^ data[26] ^ data[31];
      next[9] = data[14] ^ data[16] ^ data[23] ^ data[27];
      next[10] = data[15] ^ data[17] ^ data[24] ^ data[28];
      next[11] = data[7] ^ data[28];
      next[12] = data[8] ^ data[29];
      next[13] = data[9] ^ data[30];
      next[14] = data[10] ^ data[31];
      next[15] = data[11];
      next[16] = data[12];
      next[17] = data[13];
      next[18] = data[14];
      next[19] = data[15];
      next[20] = data[16];
      next[21] = data[7] ^ data[16] ^ data[17] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[22] = data[8] ^ data[17] ^ data[18] ^ data[19] ^ data[26] ^ data[29] ^ data[30];
      next[23] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[24] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[25] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[26] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[27] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[28] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[29] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[30] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[31] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
    end
  end
  if(CN == 25) begin
    always @(*) begin
      next[0] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[1] = data[7] ^ data[16] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[2] = data[6] ^ data[8] ^ data[15] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[3] = data[7] ^ data[9] ^ data[16] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[4] = data[8] ^ data[10] ^ data[17] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[5] = data[9] ^ data[11] ^ data[18] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[6] = data[10] ^ data[12] ^ data[19] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[7] = data[11] ^ data[13] ^ data[20] ^ data[24] ^ data[29] ^ data[31];
      next[8] = data[12] ^ data[14] ^ data[21] ^ data[25] ^ data[30];
      next[9] = data[13] ^ data[15] ^ data[22] ^ data[26] ^ data[31];
      next[10] = data[14] ^ data[16] ^ data[23] ^ data[27];
      next[11] = data[6] ^ data[27];
      next[12] = data[7] ^ data[28];
      next[13] = data[8] ^ data[29];
      next[14] = data[9] ^ data[30];
      next[15] = data[10] ^ data[31];
      next[16] = data[11];
      next[17] = data[12];
      next[18] = data[13];
      next[19] = data[14];
      next[20] = data[15];
      next[21] = data[6] ^ data[15] ^ data[16] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[22] = data[7] ^ data[16] ^ data[17] ^ data[18] ^ data[25] ^ data[28] ^ data[29];
      next[23] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[24] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[25] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[26] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[27] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[28] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[29] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[30] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[31] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
    end
  end
  if(CN == 26) begin
    always @(*) begin
      next[0] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[1] = data[6] ^ data[15] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[2] = data[5] ^ data[7] ^ data[14] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[3] = data[6] ^ data[8] ^ data[15] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[4] = data[7] ^ data[9] ^ data[16] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[5] = data[8] ^ data[10] ^ data[17] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[6] = data[9] ^ data[11] ^ data[18] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[7] = data[10] ^ data[12] ^ data[19] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[8] = data[11] ^ data[13] ^ data[20] ^ data[24] ^ data[29] ^ data[31];
      next[9] = data[12] ^ data[14] ^ data[21] ^ data[25] ^ data[30];
      next[10] = data[13] ^ data[15] ^ data[22] ^ data[26] ^ data[31];
      next[11] = data[5] ^ data[26];
      next[12] = data[6] ^ data[27];
      next[13] = data[7] ^ data[28];
      next[14] = data[8] ^ data[29];
      next[15] = data[9] ^ data[30];
      next[16] = data[10] ^ data[31];
      next[17] = data[11];
      next[18] = data[12];
      next[19] = data[13];
      next[20] = data[14];
      next[21] = data[5] ^ data[14] ^ data[15] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[22] = data[6] ^ data[15] ^ data[16] ^ data[17] ^ data[24] ^ data[27] ^ data[28];
      next[23] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[24] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[25] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[26] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[27] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[28] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[29] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[30] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[31] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
    end
  end
  if(CN == 27) begin
    always @(*) begin
      next[0] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[1] = data[5] ^ data[14] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[2] = data[4] ^ data[6] ^ data[13] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[3] = data[5] ^ data[7] ^ data[14] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[4] = data[6] ^ data[8] ^ data[15] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[5] = data[7] ^ data[9] ^ data[16] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[6] = data[8] ^ data[10] ^ data[17] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[7] = data[9] ^ data[11] ^ data[18] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[8] = data[10] ^ data[12] ^ data[19] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[9] = data[11] ^ data[13] ^ data[20] ^ data[24] ^ data[29] ^ data[31];
      next[10] = data[12] ^ data[14] ^ data[21] ^ data[25] ^ data[30];
      next[11] = data[4] ^ data[25];
      next[12] = data[5] ^ data[26];
      next[13] = data[6] ^ data[27];
      next[14] = data[7] ^ data[28];
      next[15] = data[8] ^ data[29];
      next[16] = data[9] ^ data[30];
      next[17] = data[10] ^ data[31];
      next[18] = data[11];
      next[19] = data[12];
      next[20] = data[13];
      next[21] = data[4] ^ data[13] ^ data[14] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[22] = data[5] ^ data[14] ^ data[15] ^ data[16] ^ data[23] ^ data[26] ^ data[27];
      next[23] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[24] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[25] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[26] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[27] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[28] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[29] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[30] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[31] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
    end
  end
  if(CN == 28) begin
    always @(*) begin
      next[0] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[1] = data[4] ^ data[13] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[2] = data[3] ^ data[5] ^ data[12] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[3] = data[4] ^ data[6] ^ data[13] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[4] = data[5] ^ data[7] ^ data[14] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[5] = data[6] ^ data[8] ^ data[15] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[6] = data[7] ^ data[9] ^ data[16] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[7] = data[8] ^ data[10] ^ data[17] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[8] = data[9] ^ data[11] ^ data[18] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[9] = data[10] ^ data[12] ^ data[19] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[10] = data[11] ^ data[13] ^ data[20] ^ data[24] ^ data[29] ^ data[31];
      next[11] = data[3] ^ data[24];
      next[12] = data[4] ^ data[25];
      next[13] = data[5] ^ data[26];
      next[14] = data[6] ^ data[27];
      next[15] = data[7] ^ data[28];
      next[16] = data[8] ^ data[29];
      next[17] = data[9] ^ data[30];
      next[18] = data[10] ^ data[31];
      next[19] = data[11];
      next[20] = data[12];
      next[21] = data[3] ^ data[12] ^ data[13] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[22] = data[4] ^ data[13] ^ data[14] ^ data[15] ^ data[22] ^ data[25] ^ data[26] ^ data[31];
      next[23] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[24] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[25] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[26] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[27] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[28] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[29] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[30] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[31] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
    end
  end
  if(CN == 29) begin
    always @(*) begin
      next[0] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[1] = data[3] ^ data[12] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[2] = data[2] ^ data[4] ^ data[11] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
      next[3] = data[3] ^ data[5] ^ data[12] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[4] = data[4] ^ data[6] ^ data[13] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[5] = data[5] ^ data[7] ^ data[14] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[6] = data[6] ^ data[8] ^ data[15] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[7] = data[7] ^ data[9] ^ data[16] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[8] = data[8] ^ data[10] ^ data[17] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[9] = data[9] ^ data[11] ^ data[18] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[10] = data[10] ^ data[12] ^ data[19] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[11] = data[2] ^ data[23];
      next[12] = data[3] ^ data[24];
      next[13] = data[4] ^ data[25];
      next[14] = data[5] ^ data[26];
      next[15] = data[6] ^ data[27];
      next[16] = data[7] ^ data[28];
      next[17] = data[8] ^ data[29];
      next[18] = data[9] ^ data[30];
      next[19] = data[10] ^ data[31];
      next[20] = data[11];
      next[21] = data[2] ^ data[11] ^ data[12] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[22] = data[3] ^ data[12] ^ data[13] ^ data[14] ^ data[21] ^ data[24] ^ data[25] ^ data[30];
      next[23] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
      next[24] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[25] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[26] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[27] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[28] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[29] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[30] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[31] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
    end
  end
  if(CN == 30) begin
    always @(*) begin
      next[0] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[1] = data[2] ^ data[11] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[2] = data[1] ^ data[3] ^ data[10] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
      next[3] = data[2] ^ data[4] ^ data[11] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
      next[4] = data[3] ^ data[5] ^ data[12] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[5] = data[4] ^ data[6] ^ data[13] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[6] = data[5] ^ data[7] ^ data[14] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[7] = data[6] ^ data[8] ^ data[15] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[8] = data[7] ^ data[9] ^ data[16] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[9] = data[8] ^ data[10] ^ data[17] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[10] = data[9] ^ data[11] ^ data[18] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[11] = data[1] ^ data[22];
      next[12] = data[2] ^ data[23];
      next[13] = data[3] ^ data[24];
      next[14] = data[4] ^ data[25];
      next[15] = data[5] ^ data[26];
      next[16] = data[6] ^ data[27];
      next[17] = data[7] ^ data[28];
      next[18] = data[8] ^ data[29];
      next[19] = data[9] ^ data[30];
      next[20] = data[10] ^ data[31];
      next[21] = data[1] ^ data[10] ^ data[11] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[22] = data[2] ^ data[11] ^ data[12] ^ data[13] ^ data[20] ^ data[23] ^ data[24] ^ data[29] ^ data[31];
      next[23] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
      next[24] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
      next[25] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[26] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[27] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[28] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[29] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[30] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[31] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    end
  end
  if(CN == 31) begin
    always @(*) begin
      next[0] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[1] = data[1] ^ data[10] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[2] = data[0] ^ data[2] ^ data[9] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
      next[3] = data[1] ^ data[3] ^ data[10] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
      next[4] = data[2] ^ data[4] ^ data[11] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
      next[5] = data[3] ^ data[5] ^ data[12] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[6] = data[4] ^ data[6] ^ data[13] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[7] = data[5] ^ data[7] ^ data[14] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[8] = data[6] ^ data[8] ^ data[15] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[9] = data[7] ^ data[9] ^ data[16] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[10] = data[8] ^ data[10] ^ data[17] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[11] = data[0] ^ data[21];
      next[12] = data[1] ^ data[22];
      next[13] = data[2] ^ data[23];
      next[14] = data[3] ^ data[24];
      next[15] = data[4] ^ data[25];
      next[16] = data[5] ^ data[26];
      next[17] = data[6] ^ data[27];
      next[18] = data[7] ^ data[28];
      next[19] = data[8] ^ data[29];
      next[20] = data[9] ^ data[30];
      next[21] = data[0] ^ data[9] ^ data[10] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30];
      next[22] = data[1] ^ data[10] ^ data[11] ^ data[12] ^ data[19] ^ data[22] ^ data[23] ^ data[28] ^ data[30] ^ data[31];
      next[23] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
      next[24] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
      next[25] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
      next[26] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[27] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[28] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[29] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[30] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[31] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    end
  end
  if(CN == 32) begin
    always @(*) begin
      next[0] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[1] = data[0] ^ data[9] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
      next[2] = data[1] ^ data[8] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
      next[3] = data[0] ^ data[2] ^ data[9] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
      next[4] = data[1] ^ data[3] ^ data[10] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
      next[5] = data[2] ^ data[4] ^ data[11] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
      next[6] = data[3] ^ data[5] ^ data[12] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[7] = data[4] ^ data[6] ^ data[13] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[8] = data[5] ^ data[7] ^ data[14] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[9] = data[6] ^ data[8] ^ data[15] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[10] = data[7] ^ data[9] ^ data[16] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
      next[11] = data[20];
      next[12] = data[0] ^ data[21];
      next[13] = data[1] ^ data[22];
      next[14] = data[2] ^ data[23];
      next[15] = data[3] ^ data[24];
      next[16] = data[4] ^ data[25];
      next[17] = data[5] ^ data[26];
      next[18] = data[6] ^ data[27];
      next[19] = data[7] ^ data[28];
      next[20] = data[8] ^ data[29];
      next[21] = data[8] ^ data[9] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
      next[22] = data[0] ^ data[9] ^ data[10] ^ data[11] ^ data[18] ^ data[21] ^ data[22] ^ data[27] ^ data[29] ^ data[30];
      next[23] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
      next[24] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
      next[25] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
      next[26] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
      next[27] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
      next[28] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
      next[29] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
      next[30] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
      next[31] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    end
  end
  if(CN == 33) begin
   always @(*) begin
    next[0] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[1] = data[8] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[2] = data[0] ^ data[7] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28];
    next[3] = data[1] ^ data[8] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[4] = data[0] ^ data[2] ^ data[9] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[5] = data[1] ^ data[3] ^ data[10] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[6] = data[2] ^ data[4] ^ data[11] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[7] = data[3] ^ data[5] ^ data[12] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[8] = data[4] ^ data[6] ^ data[13] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[9] = data[5] ^ data[7] ^ data[14] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
    next[10] = data[6] ^ data[8] ^ data[15] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[11] = data[19];
    next[12] = data[20];
    next[13] = data[0] ^ data[21];
    next[14] = data[1] ^ data[22];
    next[15] = data[2] ^ data[23];
    next[16] = data[3] ^ data[24];
    next[17] = data[4] ^ data[25];
    next[18] = data[5] ^ data[26];
    next[19] = data[6] ^ data[27];
    next[20] = data[7] ^ data[28];
    next[21] = data[7] ^ data[8] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[22] = data[8] ^ data[9] ^ data[10] ^ data[17] ^ data[20] ^ data[21] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[23] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[24] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[25] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[26] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[27] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[28] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[29] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[30] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
    next[31] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
   end
  end
  if(CN == 34) begin
   always @(*) begin
    next[0] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[1] = data[7] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[2] = data[6] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[31];
    next[3] = data[0] ^ data[7] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28];
    next[4] = data[1] ^ data[8] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[5] = data[0] ^ data[2] ^ data[9] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[6] = data[1] ^ data[3] ^ data[10] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[7] = data[2] ^ data[4] ^ data[11] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[8] = data[3] ^ data[5] ^ data[12] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[9] = data[4] ^ data[6] ^ data[13] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[10] = data[5] ^ data[7] ^ data[14] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
    next[11] = data[18];
    next[12] = data[19];
    next[13] = data[20];
    next[14] = data[0] ^ data[21];
    next[15] = data[1] ^ data[22];
    next[16] = data[2] ^ data[23];
    next[17] = data[3] ^ data[24];
    next[18] = data[4] ^ data[25];
    next[19] = data[5] ^ data[26];
    next[20] = data[6] ^ data[27];
    next[21] = data[6] ^ data[7] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[22] = data[7] ^ data[8] ^ data[9] ^ data[16] ^ data[19] ^ data[20] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[23] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[24] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[25] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[26] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[27] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[28] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[29] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[30] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[31] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
   end
  end

  if(CN == 35) begin
   always @(*) begin
    next[0] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
    next[1] = data[6] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[2] = data[5] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[30] ^ data[31];
    next[3] = data[6] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[31];
    next[4] = data[0] ^ data[7] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28];
    next[5] = data[1] ^ data[8] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[6] = data[0] ^ data[2] ^ data[9] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[7] = data[1] ^ data[3] ^ data[10] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[8] = data[2] ^ data[4] ^ data[11] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[9] = data[3] ^ data[5] ^ data[12] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[10] = data[4] ^ data[6] ^ data[13] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[11] = data[17];
    next[12] = data[18];
    next[13] = data[19];
    next[14] = data[20];
    next[15] = data[0] ^ data[21];
    next[16] = data[1] ^ data[22];
    next[17] = data[2] ^ data[23];
    next[18] = data[3] ^ data[24];
    next[19] = data[4] ^ data[25];
    next[20] = data[5] ^ data[26];
    next[21] = data[5] ^ data[6] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29];
    next[22] = data[6] ^ data[7] ^ data[8] ^ data[15] ^ data[18] ^ data[19] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[23] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[24] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[25] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[26] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[27] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[28] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[29] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[30] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[31] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
   end
  end

  if(CN == 36) begin
   always @(*) begin
    next[0] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[1] = data[5] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[29];
    next[2] = data[4] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[29] ^ data[30] ^ data[31];
    next[3] = data[5] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[30] ^ data[31];
    next[4] = data[6] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[31];
    next[5] = data[0] ^ data[7] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28];
    next[6] = data[1] ^ data[8] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[7] = data[0] ^ data[2] ^ data[9] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[8] = data[1] ^ data[3] ^ data[10] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[9] = data[2] ^ data[4] ^ data[11] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[10] = data[3] ^ data[5] ^ data[12] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[11] = data[16];
    next[12] = data[17];
    next[13] = data[18];
    next[14] = data[19];
    next[15] = data[20];
    next[16] = data[0] ^ data[21];
    next[17] = data[1] ^ data[22];
    next[18] = data[2] ^ data[23];
    next[19] = data[3] ^ data[24];
    next[20] = data[4] ^ data[25];
    next[21] = data[4] ^ data[5] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[31];
    next[22] = data[5] ^ data[6] ^ data[7] ^ data[14] ^ data[17] ^ data[18] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29];
    next[23] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[24] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[25] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[26] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[27] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[28] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[29] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[30] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[31] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
   end
  end

  if(CN == 37) begin
   always @(*) begin
    next[0] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[1] = data[4] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[2] = data[3] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[28] ^ data[29] ^ data[30];
    next[3] = data[4] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[29] ^ data[30] ^ data[31];
    next[4] = data[5] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[30] ^ data[31];
    next[5] = data[6] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[31];
    next[6] = data[0] ^ data[7] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28];
    next[7] = data[1] ^ data[8] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[8] = data[0] ^ data[2] ^ data[9] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[9] = data[1] ^ data[3] ^ data[10] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[10] = data[2] ^ data[4] ^ data[11] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[11] = data[15];
    next[12] = data[16];
    next[13] = data[17];
    next[14] = data[18];
    next[15] = data[19];
    next[16] = data[20];
    next[17] = data[0] ^ data[21];
    next[18] = data[1] ^ data[22];
    next[19] = data[2] ^ data[23];
    next[20] = data[3] ^ data[24];
    next[21] = data[3] ^ data[4] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[30];
    next[22] = data[4] ^ data[5] ^ data[6] ^ data[13] ^ data[16] ^ data[17] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[31];
    next[23] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[24] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[25] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[26] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[27] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[28] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[29] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[30] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[31] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
   end
  end

  if(CN == 38) begin
   always @(*) begin
    next[0] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[1] = data[3] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[2] = data[2] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[3] = data[3] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[28] ^ data[29] ^ data[30];
    next[4] = data[4] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[29] ^ data[30] ^ data[31];
    next[5] = data[5] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[30] ^ data[31];
    next[6] = data[6] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[31];
    next[7] = data[0] ^ data[7] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28];
    next[8] = data[1] ^ data[8] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[9] = data[0] ^ data[2] ^ data[9] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[10] = data[1] ^ data[3] ^ data[10] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[11] = data[14];
    next[12] = data[15];
    next[13] = data[16];
    next[14] = data[17];
    next[15] = data[18];
    next[16] = data[19];
    next[17] = data[20];
    next[18] = data[0] ^ data[21];
    next[19] = data[1] ^ data[22];
    next[20] = data[2] ^ data[23];
    next[21] = data[2] ^ data[3] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[29];
    next[22] = data[3] ^ data[4] ^ data[5] ^ data[12] ^ data[15] ^ data[16] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[30];
    next[23] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[24] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[25] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[26] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[27] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[28] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[29] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[30] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[31] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
   end
  end

  if(CN == 39) begin
   always @(*) begin
    next[0] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[1] = data[2] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[29];
    next[2] = data[1] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[3] = data[2] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[4] = data[3] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[28] ^ data[29] ^ data[30];
    next[5] = data[4] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[29] ^ data[30] ^ data[31];
    next[6] = data[5] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[30] ^ data[31];
    next[7] = data[6] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[31];
    next[8] = data[0] ^ data[7] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28];
    next[9] = data[1] ^ data[8] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[10] = data[0] ^ data[2] ^ data[9] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[11] = data[13];
    next[12] = data[14];
    next[13] = data[15];
    next[14] = data[16];
    next[15] = data[17];
    next[16] = data[18];
    next[17] = data[19];
    next[18] = data[20];
    next[19] = data[0] ^ data[21];
    next[20] = data[1] ^ data[22];
    next[21] = data[1] ^ data[2] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[22] = data[2] ^ data[3] ^ data[4] ^ data[11] ^ data[14] ^ data[15] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[29];
    next[23] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[24] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[25] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[26] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[27] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[28] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[29] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[30] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[31] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
   end
  end

  if(CN == 40) begin
   always @(*) begin
    next[0] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[1] = data[1] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[2] = data[0] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[3] = data[1] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[4] = data[2] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[5] = data[3] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[28] ^ data[29] ^ data[30];
    next[6] = data[4] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[29] ^ data[30] ^ data[31];
    next[7] = data[5] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[30] ^ data[31];
    next[8] = data[6] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[31];
    next[9] = data[0] ^ data[7] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28];
    next[10] = data[1] ^ data[8] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[11] = data[12];
    next[12] = data[13];
    next[13] = data[14];
    next[14] = data[15];
    next[15] = data[16];
    next[16] = data[17];
    next[17] = data[18];
    next[18] = data[19];
    next[19] = data[20];
    next[20] = data[0] ^ data[21];
    next[21] = data[0] ^ data[1] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[22] = data[1] ^ data[2] ^ data[3] ^ data[10] ^ data[13] ^ data[14] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[28] ^ data[31];
    next[23] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[24] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[25] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[26] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[27] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[28] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[29] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[30] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[31] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
   end
  end

  if(CN == 41) begin
   always @(*) begin
    next[0] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[1] = data[0] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[2] = data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[3] = data[0] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[4] = data[1] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[5] = data[2] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[6] = data[3] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[28] ^ data[29] ^ data[30];
    next[7] = data[4] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[29] ^ data[30] ^ data[31];
    next[8] = data[5] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[30] ^ data[31];
    next[9] = data[6] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[31];
    next[10] = data[0] ^ data[7] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28];
    next[11] = data[11];
    next[12] = data[12];
    next[13] = data[13];
    next[14] = data[14];
    next[15] = data[15];
    next[16] = data[16];
    next[17] = data[17];
    next[18] = data[18];
    next[19] = data[19];
    next[20] = data[20];
    next[21] = data[0] ^ data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[22] = data[0] ^ data[1] ^ data[2] ^ data[9] ^ data[12] ^ data[13] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[27] ^ data[30];
    next[23] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[24] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[25] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[26] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[27] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[28] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[29] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[30] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[31] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
   end
  end

  if(CN == 42) begin
   always @(*) begin
    next[0] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[1] = data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[2] = data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[3] = data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[4] = data[0] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[5] = data[1] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[6] = data[2] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[7] = data[3] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[28] ^ data[29] ^ data[30];
    next[8] = data[4] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[29] ^ data[30] ^ data[31];
    next[9] = data[5] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[30] ^ data[31];
    next[10] = data[6] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[31];
    next[11] = data[10] ^ data[31];
    next[12] = data[11];
    next[13] = data[12];
    next[14] = data[13];
    next[15] = data[14];
    next[16] = data[15];
    next[17] = data[16];
    next[18] = data[17];
    next[19] = data[18];
    next[20] = data[19];
    next[21] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[22] = data[0] ^ data[1] ^ data[8] ^ data[11] ^ data[12] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[26] ^ data[29];
    next[23] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[24] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[25] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[26] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[27] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[28] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[29] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[30] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[31] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
   end
  end

  if(CN == 43) begin
   always @(*) begin
    next[0] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[1] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[2] = data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[3] = data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[4] = data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[5] = data[0] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[6] = data[1] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[7] = data[2] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[8] = data[3] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[28] ^ data[29] ^ data[30];
    next[9] = data[4] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[29] ^ data[30] ^ data[31];
    next[10] = data[5] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[30] ^ data[31];
    next[11] = data[9] ^ data[30];
    next[12] = data[10] ^ data[31];
    next[13] = data[11];
    next[14] = data[12];
    next[15] = data[13];
    next[16] = data[14];
    next[17] = data[15];
    next[18] = data[16];
    next[19] = data[17];
    next[20] = data[18];
    next[21] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[22] = data[0] ^ data[7] ^ data[10] ^ data[11] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[25] ^ data[28] ^ data[31];
    next[23] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[24] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[25] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[26] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[27] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[28] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[29] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[30] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[31] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
   end
  end

  if(CN == 44) begin
   always @(*) begin
    next[0] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[1] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[2] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[3] = data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[4] = data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[5] = data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[6] = data[0] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[7] = data[1] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[8] = data[2] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[9] = data[3] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[28] ^ data[29] ^ data[30];
    next[10] = data[4] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[29] ^ data[30] ^ data[31];
    next[11] = data[8] ^ data[29];
    next[12] = data[9] ^ data[30];
    next[13] = data[10] ^ data[31];
    next[14] = data[11];
    next[15] = data[12];
    next[16] = data[13];
    next[17] = data[14];
    next[18] = data[15];
    next[19] = data[16];
    next[20] = data[17];
    next[21] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[22] = data[6] ^ data[9] ^ data[10] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[24] ^ data[27] ^ data[30] ^ data[31];
    next[23] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[24] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[25] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[26] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[27] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[28] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[29] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[30] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[31] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
   end
  end

  if(CN == 45) begin
   always @(*) begin
    next[0] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[1] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[2] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[3] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[4] = data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[5] = data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[6] = data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[7] = data[0] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[8] = data[1] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[9] = data[2] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[10] = data[3] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[28] ^ data[29] ^ data[30];
    next[11] = data[7] ^ data[28];
    next[12] = data[8] ^ data[29];
    next[13] = data[9] ^ data[30];
    next[14] = data[10] ^ data[31];
    next[15] = data[11];
    next[16] = data[12];
    next[17] = data[13];
    next[18] = data[14];
    next[19] = data[15];
    next[20] = data[16];
    next[21] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[22] = data[5] ^ data[8] ^ data[9] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[23] ^ data[26] ^ data[29] ^ data[30] ^ data[31];
    next[23] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[24] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[25] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[26] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[27] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[28] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[29] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[30] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[31] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
   end
  end

  if(CN == 46) begin
   always @(*) begin
    next[0] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[1] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[2] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[3] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[4] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[5] = data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[6] = data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[7] = data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[8] = data[0] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[9] = data[1] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[10] = data[2] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[11] = data[6] ^ data[27];
    next[12] = data[7] ^ data[28];
    next[13] = data[8] ^ data[29];
    next[14] = data[9] ^ data[30];
    next[15] = data[10] ^ data[31];
    next[16] = data[11];
    next[17] = data[12];
    next[18] = data[13];
    next[19] = data[14];
    next[20] = data[15];
    next[21] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[22] = data[4] ^ data[7] ^ data[8] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[22] ^ data[25] ^ data[28] ^ data[29] ^ data[30] ^ data[31];
    next[23] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[24] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[25] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[26] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[27] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[28] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[29] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[30] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[31] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
   end
  end

  if(CN == 47) begin
   always @(*) begin
    next[0] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[1] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[2] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[3] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[4] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[5] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[6] = data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[7] = data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[8] = data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[9] = data[0] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[10] = data[1] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[11] = data[5] ^ data[26];
    next[12] = data[6] ^ data[27];
    next[13] = data[7] ^ data[28];
    next[14] = data[8] ^ data[29];
    next[15] = data[9] ^ data[30];
    next[16] = data[10] ^ data[31];
    next[17] = data[11];
    next[18] = data[12];
    next[19] = data[13];
    next[20] = data[14];
    next[21] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[22] = data[3] ^ data[6] ^ data[7] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[21] ^ data[24] ^ data[27] ^ data[28] ^ data[29] ^ data[30];
    next[23] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[24] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[25] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[26] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[27] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[28] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[29] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[30] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[31] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
   end
  end

  if(CN == 48) begin
   always @(*) begin
    next[0] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[1] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[2] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[3] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[4] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[5] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[6] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[7] = data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[8] = data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[9] = data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[10] = data[0] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[11] = data[4] ^ data[25];
    next[12] = data[5] ^ data[26];
    next[13] = data[6] ^ data[27];
    next[14] = data[7] ^ data[28];
    next[15] = data[8] ^ data[29];
    next[16] = data[9] ^ data[30];
    next[17] = data[10] ^ data[31];
    next[18] = data[11];
    next[19] = data[12];
    next[20] = data[13];
    next[21] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[22] = data[2] ^ data[5] ^ data[6] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[20] ^ data[23] ^ data[26] ^ data[27] ^ data[28] ^ data[29] ^ data[31];
    next[23] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[24] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[25] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[26] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[27] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[28] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[29] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[30] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[31] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
   end
  end

  if(CN == 49) begin
   always @(*) begin
    next[0] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[1] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[2] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[3] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[4] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[5] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[6] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[7] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[8] = data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[9] = data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[10] = data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[11] = data[3] ^ data[24];
    next[12] = data[4] ^ data[25];
    next[13] = data[5] ^ data[26];
    next[14] = data[6] ^ data[27];
    next[15] = data[7] ^ data[28];
    next[16] = data[8] ^ data[29];
    next[17] = data[9] ^ data[30];
    next[18] = data[10] ^ data[31];
    next[19] = data[11];
    next[20] = data[12];
    next[21] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[22] = data[1] ^ data[4] ^ data[5] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[19] ^ data[22] ^ data[25] ^ data[26] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[23] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[24] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[25] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[26] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[27] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[28] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[29] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[30] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[31] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
   end
  end

  if(CN == 50) begin
   always @(*) begin
    next[0] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[1] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[2] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[3] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[4] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[5] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[6] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[7] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[8] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[9] = data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[10] = data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[11] = data[2] ^ data[23];
    next[12] = data[3] ^ data[24];
    next[13] = data[4] ^ data[25];
    next[14] = data[5] ^ data[26];
    next[15] = data[6] ^ data[27];
    next[16] = data[7] ^ data[28];
    next[17] = data[8] ^ data[29];
    next[18] = data[9] ^ data[30];
    next[19] = data[10] ^ data[31];
    next[20] = data[11];
    next[21] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[22] = data[0] ^ data[3] ^ data[4] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[18] ^ data[21] ^ data[24] ^ data[25] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[23] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[24] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[25] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[26] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[27] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[28] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[29] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[30] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[31] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
   end
  end

  if(CN == 51) begin
   always @(*) begin
    next[0] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[1] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[2] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[3] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[4] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[5] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[6] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[7] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[8] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[9] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[10] = data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[11] = data[1] ^ data[22];
    next[12] = data[2] ^ data[23];
    next[13] = data[3] ^ data[24];
    next[14] = data[4] ^ data[25];
    next[15] = data[5] ^ data[26];
    next[16] = data[6] ^ data[27];
    next[17] = data[7] ^ data[28];
    next[18] = data[8] ^ data[29];
    next[19] = data[9] ^ data[30];
    next[20] = data[10] ^ data[31];
    next[21] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[22] = data[2] ^ data[3] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[17] ^ data[20] ^ data[23] ^ data[24] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[23] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[24] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[25] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[26] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[27] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[28] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[29] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[30] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[31] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
   end
  end

  if(CN == 52) begin
   always @(*) begin
    next[0] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[1] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[2] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[3] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[4] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[5] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[6] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[7] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[8] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[9] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[10] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[11] = data[0] ^ data[21];
    next[12] = data[1] ^ data[22];
    next[13] = data[2] ^ data[23];
    next[14] = data[3] ^ data[24];
    next[15] = data[4] ^ data[25];
    next[16] = data[5] ^ data[26];
    next[17] = data[6] ^ data[27];
    next[18] = data[7] ^ data[28];
    next[19] = data[8] ^ data[29];
    next[20] = data[9] ^ data[30];
    next[21] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[22] = data[1] ^ data[2] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[16] ^ data[19] ^ data[22] ^ data[23] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[30] ^ data[31];
    next[23] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[24] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[25] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[26] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[27] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[28] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[29] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[30] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[31] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
   end
  end

  if(CN == 53) begin
   always @(*) begin
    next[0] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[1] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[2] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[3] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[4] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[5] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[6] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[7] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[8] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[9] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[10] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[11] = data[20];
    next[12] = data[0] ^ data[21];
    next[13] = data[1] ^ data[22];
    next[14] = data[2] ^ data[23];
    next[15] = data[3] ^ data[24];
    next[16] = data[4] ^ data[25];
    next[17] = data[5] ^ data[26];
    next[18] = data[6] ^ data[27];
    next[19] = data[7] ^ data[28];
    next[20] = data[8] ^ data[29];
    next[21] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29];
    next[22] = data[0] ^ data[1] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[15] ^ data[18] ^ data[21] ^ data[22] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[30];
    next[23] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[24] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[25] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[26] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[27] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[28] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[29] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[30] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[31] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
   end
  end

  if(CN == 54) begin
   always @(*) begin
    next[0] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[1] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[2] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[3] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[4] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[5] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[6] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[7] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[8] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[9] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[10] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[11] = data[19];
    next[12] = data[20];
    next[13] = data[0] ^ data[21];
    next[14] = data[1] ^ data[22];
    next[15] = data[2] ^ data[23];
    next[16] = data[3] ^ data[24];
    next[17] = data[4] ^ data[25];
    next[18] = data[5] ^ data[26];
    next[19] = data[6] ^ data[27];
    next[20] = data[7] ^ data[28];
    next[21] = data[4] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28];
    next[22] = data[0] ^ data[5] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[14] ^ data[17] ^ data[20] ^ data[21] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[29];
    next[23] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[24] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[25] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[26] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[27] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[28] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[29] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[30] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[31] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
   end
  end

  if(CN == 55) begin
   always @(*) begin
    next[0] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[1] = data[4] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[2] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[3] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[4] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[5] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[6] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[7] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[8] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[9] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[10] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[11] = data[18];
    next[12] = data[19];
    next[13] = data[20];
    next[14] = data[0] ^ data[21];
    next[15] = data[1] ^ data[22];
    next[16] = data[2] ^ data[23];
    next[17] = data[3] ^ data[24];
    next[18] = data[4] ^ data[25];
    next[19] = data[5] ^ data[26];
    next[20] = data[6] ^ data[27];
    next[21] = data[3] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[31];
    next[22] = data[4] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[13] ^ data[16] ^ data[19] ^ data[20] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[28];
    next[23] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
    next[24] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[25] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[26] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[27] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[28] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[29] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[30] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[31] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
   end
  end

  if(CN == 56) begin
   always @(*) begin
    next[0] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[1] = data[3] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[2] = data[2] ^ data[5] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[3] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[4] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[5] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[6] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[7] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[8] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[9] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[10] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[11] = data[17];
    next[12] = data[18];
    next[13] = data[19];
    next[14] = data[20];
    next[15] = data[0] ^ data[21];
    next[16] = data[1] ^ data[22];
    next[17] = data[2] ^ data[23];
    next[18] = data[3] ^ data[24];
    next[19] = data[4] ^ data[25];
    next[20] = data[5] ^ data[26];
    next[21] = data[2] ^ data[4] ^ data[5] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[30];
    next[22] = data[3] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[12] ^ data[15] ^ data[18] ^ data[19] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[27] ^ data[31];
    next[23] = data[2] ^ data[5] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[30];
    next[24] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
    next[25] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[26] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[27] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[28] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[29] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[30] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[31] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
   end
  end

  if(CN == 57) begin
   always @(*) begin
    next[0] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[1] = data[2] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30];
    next[2] = data[1] ^ data[4] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[3] = data[2] ^ data[5] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[4] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[5] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[6] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[7] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[8] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[9] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[10] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[11] = data[16];
    next[12] = data[17];
    next[13] = data[18];
    next[14] = data[19];
    next[15] = data[20];
    next[16] = data[0] ^ data[21];
    next[17] = data[1] ^ data[22];
    next[18] = data[2] ^ data[23];
    next[19] = data[3] ^ data[24];
    next[20] = data[4] ^ data[25];
    next[21] = data[1] ^ data[3] ^ data[4] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[29] ^ data[31];
    next[22] = data[2] ^ data[4] ^ data[5] ^ data[7] ^ data[8] ^ data[11] ^ data[14] ^ data[17] ^ data[18] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[26] ^ data[30];
    next[23] = data[1] ^ data[4] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[29];
    next[24] = data[2] ^ data[5] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[30];
    next[25] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
    next[26] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[27] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[28] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[29] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[30] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[31] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
   end
  end

  if(CN == 58) begin
   always @(*) begin
    next[0] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[1] = data[1] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[31];
    next[2] = data[0] ^ data[3] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[3] = data[1] ^ data[4] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[4] = data[2] ^ data[5] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[5] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[6] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[7] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[8] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[9] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[10] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[11] = data[15];
    next[12] = data[16];
    next[13] = data[17];
    next[14] = data[18];
    next[15] = data[19];
    next[16] = data[20];
    next[17] = data[0] ^ data[21];
    next[18] = data[1] ^ data[22];
    next[19] = data[2] ^ data[23];
    next[20] = data[3] ^ data[24];
    next[21] = data[0] ^ data[2] ^ data[3] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[28] ^ data[30] ^ data[31];
    next[22] = data[1] ^ data[3] ^ data[4] ^ data[6] ^ data[7] ^ data[10] ^ data[13] ^ data[16] ^ data[17] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[25] ^ data[29] ^ data[31];
    next[23] = data[0] ^ data[3] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[28] ^ data[31];
    next[24] = data[1] ^ data[4] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[29];
    next[25] = data[2] ^ data[5] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[30];
    next[26] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
    next[27] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[28] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[29] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[30] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[31] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
   end
  end

  if(CN == 59) begin
   always @(*) begin
    next[0] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[1] = data[0] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[30] ^ data[31];
    next[2] = data[2] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30] ^ data[31];
    next[3] = data[0] ^ data[3] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[4] = data[1] ^ data[4] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[5] = data[2] ^ data[5] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[6] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[7] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[8] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[9] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[10] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[11] = data[14];
    next[12] = data[15];
    next[13] = data[16];
    next[14] = data[17];
    next[15] = data[18];
    next[16] = data[19];
    next[17] = data[20];
    next[18] = data[0] ^ data[21];
    next[19] = data[1] ^ data[22];
    next[20] = data[2] ^ data[23];
    next[21] = data[1] ^ data[2] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[27] ^ data[29] ^ data[30];
    next[22] = data[0] ^ data[2] ^ data[3] ^ data[5] ^ data[6] ^ data[9] ^ data[12] ^ data[15] ^ data[16] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[24] ^ data[28] ^ data[30] ^ data[31];
    next[23] = data[2] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[27] ^ data[30] ^ data[31];
    next[24] = data[0] ^ data[3] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[28] ^ data[31];
    next[25] = data[1] ^ data[4] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[29];
    next[26] = data[2] ^ data[5] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[30];
    next[27] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
    next[28] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[29] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[30] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[31] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
   end
  end

  if(CN == 60) begin
   always @(*) begin
    next[0] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[1] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[29] ^ data[30];
    next[2] = data[1] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[30];
    next[3] = data[2] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30] ^ data[31];
    next[4] = data[0] ^ data[3] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[5] = data[1] ^ data[4] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[6] = data[2] ^ data[5] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[7] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[8] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[9] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[10] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[11] = data[13];
    next[12] = data[14];
    next[13] = data[15];
    next[14] = data[16];
    next[15] = data[17];
    next[16] = data[18];
    next[17] = data[19];
    next[18] = data[20];
    next[19] = data[0] ^ data[21];
    next[20] = data[1] ^ data[22];
    next[21] = data[0] ^ data[1] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[22] = data[1] ^ data[2] ^ data[4] ^ data[5] ^ data[8] ^ data[11] ^ data[14] ^ data[15] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[23] ^ data[27] ^ data[29] ^ data[30];
    next[23] = data[1] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[26] ^ data[29] ^ data[30];
    next[24] = data[2] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[27] ^ data[30] ^ data[31];
    next[25] = data[0] ^ data[3] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[28] ^ data[31];
    next[26] = data[1] ^ data[4] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[29];
    next[27] = data[2] ^ data[5] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[30];
    next[28] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
    next[29] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[30] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[31] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
   end
  end

  if(CN == 61) begin
   always @(*) begin
    next[0] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[1] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[2] = data[0] ^ data[4] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[29];
    next[3] = data[1] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[30];
    next[4] = data[2] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30] ^ data[31];
    next[5] = data[0] ^ data[3] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[6] = data[1] ^ data[4] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[7] = data[2] ^ data[5] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[8] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[9] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[10] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[11] = data[12];
    next[12] = data[13];
    next[13] = data[14];
    next[14] = data[15];
    next[15] = data[16];
    next[16] = data[17];
    next[17] = data[18];
    next[18] = data[19];
    next[19] = data[20];
    next[20] = data[0] ^ data[21];
    next[21] = data[0] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[22] = data[0] ^ data[1] ^ data[3] ^ data[4] ^ data[7] ^ data[10] ^ data[13] ^ data[14] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[22] ^ data[26] ^ data[28] ^ data[29] ^ data[31];
    next[23] = data[0] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[23] ^ data[25] ^ data[28] ^ data[29];
    next[24] = data[1] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[26] ^ data[29] ^ data[30];
    next[25] = data[2] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[27] ^ data[30] ^ data[31];
    next[26] = data[0] ^ data[3] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[28] ^ data[31];
    next[27] = data[1] ^ data[4] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[29];
    next[28] = data[2] ^ data[5] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[30];
    next[29] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
    next[30] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[31] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
   end
  end

  if(CN == 62) begin
   always @(*) begin
    next[0] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[1] = data[0] ^ data[1] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[22] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[2] = data[3] ^ data[4] ^ data[5] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[28];
    next[3] = data[0] ^ data[4] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[29];
    next[4] = data[1] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[30];
    next[5] = data[2] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30] ^ data[31];
    next[6] = data[0] ^ data[3] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[7] = data[1] ^ data[4] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[8] = data[2] ^ data[5] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[9] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[10] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30] ^ data[31];
    next[11] = data[11];
    next[12] = data[12];
    next[13] = data[13];
    next[14] = data[14];
    next[15] = data[15];
    next[16] = data[16];
    next[17] = data[17];
    next[18] = data[18];
    next[19] = data[19];
    next[20] = data[20];
    next[21] = data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[22] = data[0] ^ data[2] ^ data[3] ^ data[6] ^ data[9] ^ data[12] ^ data[13] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[21] ^ data[25] ^ data[27] ^ data[28] ^ data[30];
    next[23] = data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[22] ^ data[24] ^ data[27] ^ data[28];
    next[24] = data[0] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[23] ^ data[25] ^ data[28] ^ data[29];
    next[25] = data[1] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[26] ^ data[29] ^ data[30];
    next[26] = data[2] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[27] ^ data[30] ^ data[31];
    next[27] = data[0] ^ data[3] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[28] ^ data[31];
    next[28] = data[1] ^ data[4] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[29];
    next[29] = data[2] ^ data[5] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[30];
    next[30] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
    next[31] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
   end
  end

  if(CN == 63) begin
   always @(*) begin
    next[0] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[20] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[1] = data[0] ^ data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[21] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[2] = data[2] ^ data[3] ^ data[4] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[21] ^ data[22] ^ data[23] ^ data[26] ^ data[27];
    next[3] = data[3] ^ data[4] ^ data[5] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[22] ^ data[23] ^ data[24] ^ data[27] ^ data[28];
    next[4] = data[0] ^ data[4] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[23] ^ data[24] ^ data[25] ^ data[28] ^ data[29];
    next[5] = data[1] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[25] ^ data[26] ^ data[29] ^ data[30];
    next[6] = data[2] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[26] ^ data[27] ^ data[30] ^ data[31];
    next[7] = data[0] ^ data[3] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[27] ^ data[28] ^ data[31];
    next[8] = data[1] ^ data[4] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[28] ^ data[29];
    next[9] = data[2] ^ data[5] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[29] ^ data[30];
    next[10] = data[0] ^ data[3] ^ data[6] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[30] ^ data[31];
    next[11] = data[10] ^ data[31];
    next[12] = data[11];
    next[13] = data[12];
    next[14] = data[13];
    next[15] = data[14];
    next[16] = data[15];
    next[17] = data[16];
    next[18] = data[17];
    next[19] = data[18];
    next[20] = data[19];
    next[21] = data[0] ^ data[1] ^ data[4] ^ data[7] ^ data[10] ^ data[11] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[18] ^ data[19] ^ data[23] ^ data[25] ^ data[26] ^ data[28] ^ data[30];
    next[22] = data[1] ^ data[2] ^ data[5] ^ data[8] ^ data[11] ^ data[12] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[20] ^ data[24] ^ data[26] ^ data[27] ^ data[29] ^ data[31];
    next[23] = data[1] ^ data[2] ^ data[3] ^ data[4] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[11] ^ data[14] ^ data[16] ^ data[17] ^ data[19] ^ data[21] ^ data[23] ^ data[26] ^ data[27];
    next[24] = data[2] ^ data[3] ^ data[4] ^ data[5] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[12] ^ data[15] ^ data[17] ^ data[18] ^ data[20] ^ data[22] ^ data[24] ^ data[27] ^ data[28];
    next[25] = data[0] ^ data[3] ^ data[4] ^ data[5] ^ data[6] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[13] ^ data[16] ^ data[18] ^ data[19] ^ data[21] ^ data[23] ^ data[25] ^ data[28] ^ data[29];
    next[26] = data[1] ^ data[4] ^ data[5] ^ data[6] ^ data[7] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[14] ^ data[17] ^ data[19] ^ data[20] ^ data[22] ^ data[24] ^ data[26] ^ data[29] ^ data[30];
    next[27] = data[2] ^ data[5] ^ data[6] ^ data[7] ^ data[8] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[15] ^ data[18] ^ data[20] ^ data[21] ^ data[23] ^ data[25] ^ data[27] ^ data[30] ^ data[31];
    next[28] = data[0] ^ data[3] ^ data[6] ^ data[7] ^ data[8] ^ data[9] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[16] ^ data[19] ^ data[21] ^ data[22] ^ data[24] ^ data[26] ^ data[28] ^ data[31];
    next[29] = data[1] ^ data[4] ^ data[7] ^ data[8] ^ data[9] ^ data[10] ^ data[12] ^ data[13] ^ data[15] ^ data[16] ^ data[17] ^ data[20] ^ data[22] ^ data[23] ^ data[25] ^ data[27] ^ data[29];
    next[30] = data[2] ^ data[5] ^ data[8] ^ data[9] ^ data[10] ^ data[11] ^ data[13] ^ data[14] ^ data[16] ^ data[17] ^ data[18] ^ data[21] ^ data[23] ^ data[24] ^ data[26] ^ data[28] ^ data[30];
    next[31] = data[0] ^ data[3] ^ data[6] ^ data[9] ^ data[10] ^ data[11] ^ data[12] ^ data[14] ^ data[15] ^ data[17] ^ data[18] ^ data[19] ^ data[22] ^ data[24] ^ data[25] ^ data[27] ^ data[29] ^ data[31];
   end
  end
endgenerate
endmodule

